/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  D-91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include "libpepsy.h"
#include "d_debug.h"
#include "p2xg.h"

/* globale daten vereinbaren */
int		dbg;					/* debug-modus einstellung */

DATEI		*data;					/* daten aus datei */
char		*vt1,*vt2,*vt3,*vt4;			/* vergleichstexte aus laenderdatei */
int		set_zahl = 0;				/* anzahl, wieviel sets schon erzeugt */
int		anz_mod = 0;				/* anzahl der angegebenen modelle */
int		anz_met = 0;				/* anzahl der angegebenen methoden */
char		**modelle = (char **) NULL;		/* namen der angegebenen modelle */
char		**methoden = (char **) NULL;		/* codierung der angegebenen methoden */
char		**ur_meth = (char **) NULL;		/* angegebene methoden-namen */

int		anz_knot = 0;				/* anzahl der knoten je modell */
int		anz_klas = 0;				/* anzahl der klassen je modell */
int		anz_par_var = 0;			/* anzahl der parametervariationen */

int		problemfall = 0;			/* welcher fall ergibt sich */

BOOL		only_sets = FALSE;			/* wenn TRUE, werden nur Setnamen angezeigt */
char		*verzeichnis = (char *) NULL;		/* verzeichnis fuer dateien */

long		w_spez = ALL;				/* werte, die ausgegeben werden sollen */
long		kn_sp  = ALL;				/* knoten, die ausgegeben werden sollen */
long		kl_sp  = ALL;				/* klassen, die ausgegeben werden sollen */

/****************************************************************************
* entfernt leerzeichen in einem string (z.B. bei knotennamen) 				*
****************************************************************************/
char *unspace(was)
char *was;
{
    static char		help[256];
    int			i,j,len;

    dbginfo(5, "unspace(%s)\n",was);


    len = strlen(was);

    for (i = 0,j = 0;i < len;i++)
	if (was[i] != ' ')
	    help[j++] = was[i];
	
    help[j] = '\0';

    dbginfo(10, "unspace() fertig. return=%s\n",help);

    return (help);
}

/****************************************************************************
* initialisiert die daten fuer eine klasse                    				*
****************************************************************************/
void
KLAS_init(was,nr)
	KLAS	*was;
	int	nr;
{
    dbginfo(5, "KLAS_init(%d)\n",nr);

    was->kl_nr 		= nr;
    was->knot_anz 	= 0;
    was->knoten 	= (KNOT *) 0;
    was->ch_ng.lambda 	= 0.0;
    was->ch_ng.mvz 	= 0.0;
    was->ch_ng.maa 	= 0.0;


    dbgtext(10, "KLAS_init() fertig.\n");
}

/****************************************************************************
* initialisiert die daten fuer einen knoten                   				*
****************************************************************************/
void KNOT_init(was)
KNOT *was;
{
    dbgtext(5, "KNOT_init()\n");

    was->kn_name 	= (char *) NULL;
    was->lambda 	= 0.0;
    was->e 		= 0.0;
    was->mue_inv 	= 0.0;
    was->rho 		= 0.0;
    was->mvz 		= 0.0;
    was->maa 		= 0.0;
    was->mwz 		= 0.0;
    was->mwsl 		= 0.0;

    dbgtext(10, "KNOT_init() fertig.\n");
}

/****************************************************************************
* initialisiert die daten fuer gesamte leistungsgroessen      				*
****************************************************************************/
void GELG_init(was)
GELG *was;
{
    dbgtext(5, "GELG_init()\n");

    was->lambda	= 0.0;
    was->rho 	= 0.0;
    was->maa 	= 0.0;

    dbgtext(10, "GELG_init() fertig.\n");
}

/****************************************************************************
* initialisiert die kompletten daten einer eingabedatei       				*
****************************************************************************/
void data_init(nr)
int nr;
{
    dbginfo(5, "data_init(%d)\n",nr);

    data[nr].verfahren_name	= (char *) NULL;
    data[nr].eingabe_datei 	= (char *) NULL;
    data[nr].ausgabe_datei 	= (char *) NULL;
    data[nr].klas_anz 		= 0;
    data[nr].klassen 		= (KLAS *) NULL;
    data[nr].gesamt_lg 		= (GELG *) NULL;
    data[nr].param_var		= FALSE;
    data[nr].param_wert		= 0.0;
    data[nr].param_string	= (char *) NULL;

    dbgtext(10, "data_init() fertig.\n");
}
		
/****************************************************************************
* initialisiert die daten fuer einen neuen bereich bei parametervariation   *
****************************************************************************/
void par_var_init(a_kl,a_kn,nr,gl,cn,buf)
int  *a_kl, *a_kn, *nr, *gl, *cn;
char *buf;
{
    char	*t1,*t2,*t3;

    /* !!! parametervariation: werte vermerken */
    anz_par_var++;
    *nr = anz_par_var - 1;

    if (anz_par_var > MAX) {
	data = (DATEI *) realloc(data,anz_par_var * sizeof(DATEI));
	data_init(*nr);
    }
    if (anz_par_var > 1) {
	data[*nr].verfahren_name = data[0].verfahren_name;
	data[*nr].eingabe_datei = data[0].eingabe_datei;
	data[*nr].ausgabe_datei = data[0].ausgabe_datei;
    }

    t1 = strchr(buf,'"');
    t2 = strrchr(buf,'"');
	
    data[*nr].param_var = TRUE;
    data[*nr].param_wert = atof(t2 + 1);
    data[*nr].param_string = (char *) malloc(t2-t1);
    strncpy(data[*nr].param_string,t1 + 1,t2 - t1 - 1);
    data[*nr].param_string[t2 - t1] = '\0';

    *a_kn = *a_kl = *gl = *cn = 0;

    dbginfo2(10, "par_var_init(): Parametervariation erkannt: %d\t%s",anz_par_var, buf);
}

/****************************************************************************
* liest daten aus der datei dat_nam in datenstruktur mit nr (z.Zt. nur nr=0)*
****************************************************************************/
void read_data_in_struct(dat_nam,nr)
char *dat_nam;
int  nr;
{
    FILE	*fp;
    char	buffer[BUFSIZE];
    char	hb[BUFSIZE];
    char	lo_erg[512];	/* gesamt-dateiname fuer ausgabedatei */
    int		mode = 0;	/* 0 = netz 1 = verfahren 2 = daten */
    int		submode = -1;	/* 0 = klasse 1 = charak 2 = gesamt */
    int		count = 0;	/* wieviele datensaetze schon gelesen */
    int		akt_kl = 0;	/* aktuelle auftragsklasse */
    int		akt_kn = 0;	/* aktueller knoten */
    int		gl_merk = 0;	/* merker fuer gesamtleistungsgroessen */
    int		cn_merk = 0;	/* merker fuer char. netzgroessen */
    char	*h1,*h2;
    char	*hp;
    int		i;

    dbginfo2(5, "read_data_in_struct(%s, %d)\n", dat_nam, nr);

    data_init(nr);

    if (dat_nam) {
	/* zuerst nach datei mit A_* suchen, dann mit a_* */
	if (problemfall == 1)
	    dat_nam[0]='A';
	if (verzeichnis) {
	    strcpy(lo_erg,verzeichnis);
	    strcat(lo_erg,"/");
	    strcat(lo_erg,dat_nam);
	} else
	    strcpy(lo_erg,dat_nam);
	if ((fp = fopen(lo_erg,"r")) == NULL) {
	    dat_nam[0]='a';
	    if (verzeichnis) {
		strcpy(lo_erg,verzeichnis);
		strcat(lo_erg,"/");
		strcat(lo_erg,dat_nam);
	    } else
		strcpy(lo_erg,dat_nam);
	    if ((fp = fopen(lo_erg,"r")) == NULL) {
		fprintf(stderr,"Cannot open file %s\n",lo_erg);
		exit(1);
	    }
	}
	/* dateiname vermerken */
	data[nr].ausgabe_datei = (char *) malloc(strlen(dat_nam)+1);
	strcpy(data[nr].ausgabe_datei,dat_nam);
    } else 
	fp = stdin;

    while (! feof(fp)) {
	fgets(buffer,BUFSIZE,fp);

	/* leerzeilen ueberlesen */
	if (*buffer == '\n')
	    continue;

	switch (mode) {
	case 0:	
	    if (strstr(buffer,"PARAMETER")) {
		par_var_init(&akt_kl,&akt_kn,&nr,&gl_merk,&cn_merk,
			     buffer);
		continue;
	    }
	    h1 = strchr(buffer,'\'');
	    if (h1) {
		h1++;
		mode = 1;
		h2 = strrchr(buffer,'\'');
		data[nr].eingabe_datei = (char *)
		    malloc(h2-h1+1);
		strncpy(data[nr].eingabe_datei,h1,h2-h1);
		data[nr].eingabe_datei[h2-h1]='\0';
	    }
	    break;
	case 1:	
	    h1 = strchr(buffer,'\'');
	    if (h1) {
		h1++;
		mode = 2;
		h2 = strrchr(buffer,'\'');
		data[nr].verfahren_name = (char *)
		    malloc(h2-h1+1);
		strncpy(data[nr].verfahren_name,h1,h2-h1);
		data[nr].verfahren_name[h2-h1]='\0';
	    }
	    break;
	case 2: 
	    /* 7 faelle:
	       neue auftragsklasse
	       klassen-daten
	       charakteristische lg
	       ch_lg-daten
	       gesamtleistungsgroessen
	       gelg-daten
	       oder parametervariation			
	       */
	    if (h1 = strstr(buffer,"PARAMETER")) {
		par_var_init(&akt_kl,&akt_kn,&nr,&gl_merk,&cn_merk,
			     buffer);
		mode = 0;
		submode = -1;
		count = 0;
		continue;
	    }
	    if (h1 = strstr(buffer,vt1)) {
		submode = 0;
		count = -2;	/* 2 zeilen ueberlesen */
		/* platz fuer neue klasse anfordern */
		akt_kn = 0;
		akt_kl++;
		data[nr].klas_anz = akt_kl;
		data[nr].klassen = (KLAS *) realloc(data[nr].klassen,
						    akt_kl * sizeof(KLAS));
		KLAS_init(&data[nr].klassen[akt_kl - 1],akt_kl);
		continue;
	    }
				
	    if (h1 = strstr(buffer,vt2)) {
		submode = 1;
		count = -2;	/* 2 zeilen ueberlesen */
		continue;
	    }
				
	    if (h1 = strstr(buffer,vt3)) {
		submode = 2;
		count = -2;	/* 2 zeilen ueberlesen */
		data[nr].gesamt_lg = (GELG *) malloc(akt_kn * sizeof(GELG));
		for (i = 0;i < akt_kn;i++)
		    GELG_init(&data[nr].gesamt_lg[i]);
		continue;
	    }
				
	    if (h1 = strstr(buffer,vt4)) {
		submode = 3;
		continue;
	    }
				
	    switch (submode) {
	    case 0:	if (count < 0)
		count++;
	    else {
		/* daten fuer neuen knoten vermerken */
		akt_kn++;
		data[nr].klassen[akt_kl - 1].knot_anz = akt_kn;
		data[nr].klassen[akt_kl - 1].knoten = (KNOT *)
		    realloc(data[nr].klassen[akt_kl - 1].knoten,
			    akt_kn * sizeof(KNOT));
		KNOT_init(&data[nr].klassen[akt_kl - 1].knoten[akt_kn - 1]);
		data[nr].klassen[akt_kl-1].knoten[akt_kn-1].kn_name = (char *) malloc(12 * sizeof(char));
		strncpy(data[nr].klassen[akt_kl-1].knoten[akt_kn-1].kn_name,buffer,11);
		data[nr].klassen[akt_kl-1].knoten[akt_kn-1].kn_name[11]='\0';
		/* trennzeichen suchen */
		hp = strchr(buffer,TRENNER);
		sscanf(hp + 1,"%lf %lf %lf %lf %lf %lf %lf %lf",
		       &data[nr].klassen[akt_kl-1].knoten[akt_kn-1].lambda,
		       &data[nr].klassen[akt_kl-1].knoten[akt_kn-1].e,
		       &data[nr].klassen[akt_kl-1].knoten[akt_kn-1].mue_inv,
		       &data[nr].klassen[akt_kl-1].knoten[akt_kn-1].rho,
		       &data[nr].klassen[akt_kl-1].knoten[akt_kn-1].mvz,
		       &data[nr].klassen[akt_kl-1].knoten[akt_kn-1].maa,
		       &data[nr].klassen[akt_kl-1].knoten[akt_kn-1].mwz,
		       &data[nr].klassen[akt_kl-1].knoten[akt_kn-1].mwsl);
	    }
		break;
	    case 1: if (count < 0)
		count++;
	    else {
		hp = strchr(buffer,TRENNER);	
		sscanf(hp + 1,"%lf %lf %lf",
		       &data[nr].klassen[cn_merk].ch_ng.lambda,
		       &data[nr].klassen[cn_merk].ch_ng.mvz,
		       &data[nr].klassen[cn_merk].ch_ng.maa);
		cn_merk++;
	    }
		break;
	    case 2: if (count < 0)
		count++;
	    else {
		hp = strchr(buffer,TRENNER);
		sscanf(hp + 1,"%lf %lf %lf",
		       &data[nr].gesamt_lg[gl_merk].lambda,
		       &data[nr].gesamt_lg[gl_merk].rho,
		       &data[nr].gesamt_lg[gl_merk].maa);
		gl_merk++;
	    }
		break;
	    case 3:		/* ueberlesen der legende */
		break;
	    }
	    break;
	}
    }

    if (dat_nam)
	fclose(fp);

    if (anz_klas == 0)
	anz_klas = data[nr].klas_anz;

    if (anz_knot == 0)
	anz_knot = data[nr].klassen[0].knot_anz;

    if (submode == -1)
	fprintf(stderr,"Datei %s entspricht nicht der Sprachkonvention!\n",
		dat_nam);

    dbgtext(10, "read_data_in_struct() fertig.\n");
}

/****************************************************************************
* entfernt die dynamisch allokierten speicherplaetze von data				*
****************************************************************************/
void rm_struct_data(nr)
int	nr;
{
    int	i,j;

    dbginfo(5, "rm_struct_data(%d)\n",nr);

    if (nr > 0) {
	if (data[nr].verfahren_name)
	    free(data[nr].verfahren_name);
	if (data[nr].eingabe_datei)
	    free(data[nr].eingabe_datei);
	if (data[nr].ausgabe_datei)
	    free(data[nr].ausgabe_datei);
    }
    if (data[nr].gesamt_lg)
	free(data[nr].gesamt_lg);
    if (data[nr].klassen) {
	for (i = 0;i < data[nr].klas_anz;i++) {
	    for (j = 0;j < data[nr].klassen[i].knot_anz;j++)
		free(data[nr].klassen[i].knoten[j].kn_name);
	    free(data[nr].klassen[i].knoten);
	}
	free(data[nr].klassen);
    }

    dbgtext(10, "rm_struct_data() fertig.\n");
}

/****************************************************************************
* gibt die daten aus der struktur mit nr aus (hilfe-ausgabe zum test)       *
****************************************************************************/
void show_data(nr)
int nr;	
{
    int	i,j;

    fprintf(stderr,"Name des Verfahrens: %s\n",data[nr].verfahren_name);
    fprintf(stderr,"Name der Eingabedatei: %s\n",data[nr].eingabe_datei);
    fprintf(stderr,"Name der Ausgabedatei: %s\n",data[nr].ausgabe_datei);
    if (data[nr].param_var) {
	fprintf(stderr,"Param.Var.Wert: %8.4lf\n",data[nr].param_wert);
	fprintf(stderr,"Param.Var.String: %s\n",data[nr].param_string);
    }
    fprintf(stderr,"Anzahl der Klassen: %d\n",data[nr].klas_anz);
    for (i = 0;i < data[nr].klas_anz;i++) {	
	fprintf(stderr,"\tKlasse %d:\n",data[nr].klassen[i].kl_nr);
	for (j = 0;j < data[nr].klassen[i].knot_anz;j++) {
	    fprintf(stderr,"\t\t%10s %5.3lf %5.3lf %5.3lf %5.3lf",
		    data[nr].klassen[i].knoten[j].kn_name,
		    data[nr].klassen[i].knoten[j].lambda,
		    data[nr].klassen[i].knoten[j].e,
		    data[nr].klassen[i].knoten[j].mue_inv,
		    data[nr].klassen[i].knoten[j].rho);
	    fprintf(stderr," %5.3lf %5.3lf %5.3lf %5.3lf\n",
		    data[nr].klassen[i].knoten[j].mvz,
		    data[nr].klassen[i].knoten[j].maa,
		    data[nr].klassen[i].knoten[j].mwz,
		    data[nr].klassen[i].knoten[j].mwsl);
	}
	fprintf(stderr,"\t\tc. NG. lambda: %5.3lf\n",
		data[nr].klassen[i].ch_ng.lambda);
	fprintf(stderr,"\t\tc. NG. mvz   : %5.3lf\n",
		data[nr].klassen[i].ch_ng.mvz);
	fprintf(stderr,"\t\tc. NG. maa   : %5.3lf\n",
		data[nr].klassen[i].ch_ng.maa);
    }
    if (data[nr].gesamt_lg) {
	fprintf(stderr,"\nGesamtleistungsgroessen:\n");
	for (i = 0;i < data[nr].klassen[0].knot_anz;i++)
	    fprintf(stderr,"\t%-10s: l: %5.3lf r:%5.3lf m:%5.3lf\n",
		    data[nr].klassen[0].knoten[i].kn_name,
		    data[nr].gesamt_lg[i].lambda,
		    data[nr].gesamt_lg[i].rho,
		    data[nr].gesamt_lg[i].maa);
    }
}

/****************************************************************************
* gibt konkrete werte auf datei im xgadd-format aus                         *
****************************************************************************/
void writeout_werte(fp,wovon,gr,zusatz)
FILE	*fp;	
KNOT	wovon;
long	gr;
BOOL	zusatz;
{
    dbgtext(5, "writeout_werte()\n");

    if (1 & gr && zusatz)
	fprintf(fp,"lambda ");
    if (1 & gr)
	fprintf(fp," %lf\n",wovon.lambda);
    if (2 & gr && zusatz)
	fprintf(fp,"e      ");
    if (2 & gr)
	fprintf(fp," %lf\n",wovon.e);
    if (4 & gr && zusatz)
	fprintf(fp,"1/mue  ");
    if (4 & gr)
	fprintf(fp," %lf\n",wovon.mue_inv);
    if (8 & gr && zusatz)
	fprintf(fp,"rho    ");
    if (8 & gr)
	fprintf(fp," %lf\n",wovon.rho);
    if (16 & gr && zusatz)
	fprintf(fp,"mvz    ");
    if (16 & gr)
	fprintf(fp," %lf\n",wovon.mvz);
    if (32 & gr && zusatz)
	fprintf(fp,"maa    ");
    if (32 & gr)
	fprintf(fp," %lf\n",wovon.maa);
    if (64 & gr && zusatz)
	fprintf(fp,"mwz    ");
    if (64 & gr)
	fprintf(fp," %lf\n",wovon.mwz);
    if (128 & gr && zusatz)
	fprintf(fp,"mwsl   ");
    if (128 & gr)
	fprintf(fp," %lf\n",wovon.mwsl);
}

/****************************************************************************
* erzeugt beschriftung der y-achse mit leistungsgroesse                     *
****************************************************************************/
void name_y_achse(fp,yax)
FILE	*fp;
long	yax;
{
    if (yax != ALL) {
	fprintf(fp,"@yaxis ");
	switch (yax) {
	case LAMBDA :	fprintf(fp,"lambda\n"); break;
	case E      :	fprintf(fp,"e\n"); break;
	case MUE_INV:	fprintf(fp,"1/mue\n"); break;
	case RHO    :	fprintf(fp,"rho\n"); break;
	case MVZ    :	fprintf(fp,"mvz\n"); break;
	case MAA    :	fprintf(fp,"maa\n"); break;
	case MWZ    :	fprintf(fp,"mwz\n"); break;
	case MWSL   :	fprintf(fp,"mwsl\n"); break;
	}
    }
}

/****************************************************************************
* erzeugt dateiheader fuer ersten fall                                      *
****************************************************************************/
void header_1(fp,d_nr,art,breite,offset)
FILE	*fp;
long	d_nr;
int	breite, offset;
BOOL	art;
{
    dbginfo2(5, "header_1(%d, %d)\n",anz_klas,anz_knot);

    /* keine klassen oder keine knoten, dann auch keinen header */
    if (anz_klas == 0 || anz_knot == 0)
	return;

    fprintf(fp,"@titletext Verfahren: %s\n",data[d_nr].verfahren_name);
    if (art) {
	fprintf(fp,"@barwidth  %d\n",breite);
	fprintf(fp,"@baroffset %d\n",offset);
	fprintf(fp,"@bargraph\n");
    }
    fprintf(fp,"\n");
}

/****************************************************************************
* erzeugt dateiheader fuer zweiten fall                                     *
****************************************************************************/
void header_2(fp,d_nr,art,breite,offset,xax,yax)
FILE	*fp;
long	d_nr;
int	breite,offset;
BOOL	art;
char	*xax;
long	yax;
{
    int i;

    dbginfo2(5, "header_2:(%d, %d)\n",anz_klas,anz_knot);

    /* keine klassen oder keine knoten, dann auch keinen header */
    if (anz_klas == 0 || anz_knot == 0)
	return;

    fprintf(fp,"@titletext Modell: %s\n",modelle[0]);
    if (xax)
	fprintf(fp,"@xaxis %s\n",xax);
    else {
	for (i = 0;((1 << i) & kl_sp) == 0;i++)
	    ;
	fprintf(fp,"@xaxis Klasse %d\n",i + 1);
    }

    name_y_achse(fp,yax);

    if (art) {
	fprintf(fp,"@barwidth  %d\n",breite);
	fprintf(fp,"@baroffset %d\n",offset);
	fprintf(fp,"@bargraph\n");
    }

    fprintf(fp,"\n");
}

/****************************************************************************
* erzeugt dateiheader fuer dritten fall (ein verfahren)                     *
****************************************************************************/
void header_3a(fp,d_nr,art,breite,offset,xax,yax)
FILE	*fp;
long	d_nr;
int	breite,offset;
BOOL	art;
char	*xax;
long	yax;
{
    int i;

    dbginfo2(5, "header_3a:(%d, %d)\n",anz_klas,anz_knot);

    /* keine klassen oder keine knoten, dann auch keinen header */
    if (anz_klas == 0 || anz_knot == 0)
	return;

    fprintf(fp,"@titletext Verfahren: %s\n",ur_meth[0]);
    if (xax)
	fprintf(fp,"@xaxis %s\n",xax);
    else {
	for (i = 0;((1 << i) & kl_sp) == 0;i++)
	    ;
	fprintf(fp,"@xaxis Klasse %d\n",i + 1);
    }

    name_y_achse(fp,yax);

    if (art) {
	fprintf(fp,"@barwidth  %d\n",breite);
	fprintf(fp,"@baroffset %d\n",offset);
	fprintf(fp,"@bargraph\n");
    }

    fprintf(fp,"\n");
}

/****************************************************************************
* erzeugt dateiheader fuer dritten fall (zwei verfahren)                    *
****************************************************************************/
void header_3b(fp,d_nr,art,breite,offset,xax,yax)
FILE	*fp;
long	d_nr;
int	breite,offset;
BOOL	art;
char	*xax;
long	yax;
{
    int	i;

    dbginfo2(5, "header_3b(%d, %d)\n",anz_klas,anz_knot);

    /* keine klassen oder keine knoten, dann auch keinen header */
    if (anz_klas == 0 || anz_knot == 0)
	return;

    fprintf(fp,"@titletext Verfahren/Modell-Vergleiche\n");
    if (xax)
	fprintf(fp,"@xaxis %s\n",xax);
    else {
	for (i = 0;((1 << i) & kl_sp) == 0;i++)
	    ;
	fprintf(fp,"@xaxis Klasse %d\n",i + 1);
    }

    name_y_achse(fp,yax);

    if (art) {
	fprintf(fp,"@barwidth  %d\n",breite);
	fprintf(fp,"@baroffset %d\n",offset);
	fprintf(fp,"@bargraph\n");
    }

    fprintf(fp,"\n");
}

/****************************************************************************
* erzeugt daten fuer ersten fall                                            *
****************************************************************************/
void fall_1_pv(fp,kl_nr,kn_nr,gr)
FILE	*fp;
long	kl_nr,kn_nr,gr;	/* klasse,knoten,groesse ~0=all */
{
    int	i,j,k,l;

    dbgtext(5, "fall_1_pv:()\n");

    for (i = 0;i < data[0].klas_anz;i++) {
	/* diese klasse beruecksichtigen ? */
	if (!((1 << i) & kl_nr))
	    continue; 
	for (j = 0;j < data[0].klassen[i].knot_anz 
	     && set_zahl < MAXD;j++) {
	    /* diesen knoten beruecksichtigen ? */
	    if (!((1 << j) & kn_nr))
		continue; 
	    fprintf(fp,"\"");
	    /* mehr als eine klasse, dann klasse mit in setnamen */
	    if (data[0].klas_anz != 1)
		fprintf(fp,"Klasse %d-",i+1);
	    fprintf(fp,"%s\n",data[0].klassen[i].knoten[j].kn_name);

	    /* nur setnamen sollen ausgegeben werden */
	    if (only_sets)
		continue;

	    for (l = 0;l < anz_par_var;l++) {
		fprintf(fp,"%lf ",data[l].param_wert);
		writeout_werte(fp,data[l].klassen[i].knoten[j],gr,FALSE);
	    }
		
	    fprintf(fp,"\n");
	    set_zahl++;
	}
    }
}

/****************************************************************************
* erzeugt daten fuer ersten fall                                            *
****************************************************************************/
void fall_1(fp,d_nr,kl_nr,kn_nr,gr)
FILE	*fp;
long	d_nr,kl_nr,kn_nr,gr;	/* data,klasse,knoten,groesse ~0=all */
{
    int	i,j,k;

    dbgtext(5, "fall_1()\n");

    if (d_nr < 0 || data[d_nr].klas_anz == 0)
	return;

    for (i = 0;i < data[d_nr].klas_anz;i++) {
	/* diese klasse beruecksichtigen ? */
	if (!((1 << i) & kl_nr))
	    continue; 
	for (j = 0;j < data[d_nr].klassen[i].knot_anz && set_zahl < MAXD;j++) {
	    /* diesen knoten beruecksichtigen ? */
	    if (!((1 << j) & kn_nr))
		continue; 

	    fprintf(fp,"\"");
	    /* mehr als eine klasse, dann klasse mit in setnamen */
	    if (data[d_nr].klas_anz != 1)
		fprintf(fp,"Klasse %d-",i+1);
	    fprintf(fp,"%s\n",data[d_nr].klassen[i].knoten[j].kn_name);

	    /* nur setnamen sollen ausgegeben werden */
	    if (! only_sets) {
		writeout_werte(fp,data[d_nr].klassen[i].knoten[j],gr,TRUE);
			
		fprintf(fp,"\n");
		set_zahl++;
	    }
	}
    }
}

/****************************************************************************
* erzeugt daten fuer zweiten und dritten fall                               *
****************************************************************************/
void fall_2(fp,d_nr,kl_nr,kn_nr,gr)
FILE	*fp;
long	d_nr,kl_nr,kn_nr,gr;	/* data,klasse,knoten,groesse ~0=all */
{
    int	i,j,k;

    dbginfo3(5, "fall_2:(%ld, %ld, %ld)\n",kl_nr,kn_nr,gr);

    if (d_nr < 0 || data[d_nr].klas_anz == 0 || set_zahl >= MAXD)
	return;

    /* momentan noch fuer alle knoten alle daten rausschreiben */
    for (i = 0;i < data[d_nr].klas_anz;i++) {
	/* diese klasse beruecksichtigen ? */
	if (!((1 << i) & kl_nr))
	    continue; 
	for (j = 0;j < data[d_nr].klassen[i].knot_anz;j++) {
	    /* diese klasse beruecksichtigen ? */
	    if (!((1 << j) & kn_nr))
		continue; 
	    fprintf(fp,"%s",unspace(data[d_nr].klassen[i].knoten[j].kn_name));
	    writeout_werte(fp,data[d_nr].klassen[i].knoten[j],gr,FALSE);
	}
    }
    set_zahl++;
    fprintf(fp,"\n");

    dbgtext(10, "fall_2() fertig\n");
}

/****************************************************************************
* ueberprueft, ob met ein verfahren ist, und liefert evtl. kodierung        *
****************************************************************************/
char *method_ok(met)
char *met;
{
    static char	ergebnis[16];
    pdb_entry_t	*help;

    dbginfo(5, "method_ok(%s)\n", met);

    if (!CountPepsyDB())
	GetPepsyDB();

    help = GetDBEntry(met);

    if (help)
    {
	strcpy(ergebnis,"a_");
	strcat(ergebnis,help->codierung);
	strcat(ergebnis,"_");

	return(ergebnis);
    }
	
    return((char *) NULL);
}

/****************************************************************************
* vermerkt name als modell                                                  *
****************************************************************************/
void store_modell(name)
char	*name;
{
    dbginfo2(5, "store_modell(Modell=%s) anzmod=%d\n",name,anz_mod + 1);

    anz_mod++;
    modelle = (char **) realloc(modelle,anz_mod * sizeof(char *));
    modelle[anz_mod - 1] = (char *) malloc(strlen(name) + 1);
    strcpy(modelle[anz_mod - 1],name);
}

/****************************************************************************
* vermerkt name als verfahren mit kodierung kuerzel                         *
****************************************************************************/
void store_methode(name,kuerzel)
char	*name,*kuerzel;
{
    dbginfo3(5, "store_methode Verfahren: %s %s %d\n", name,kuerzel,anz_met + 1);

    anz_met++;
    methoden = (char **) realloc(methoden,anz_met * sizeof(char *));
    methoden[anz_met - 1] = (char *) malloc(strlen(kuerzel) + 1);
    strcpy(methoden[anz_met - 1],kuerzel);
    ur_meth = (char **) realloc(ur_meth,anz_met * sizeof(char *));
    ur_meth[anz_met - 1] = (char *) malloc(strlen(name) + 1);
    strcpy(ur_meth[anz_met - 1],name);
}

/****************************************************************************
* reduziert die angegebenen klassen auf genau eine klasse                   *
****************************************************************************/
void klas_reduce()
{
    /* stellt genau eine klasse ein */
    int x,i;

    dbginfo(5, "klas_reduce(): kl_sp=%d\n",kl_sp);

    if (kl_sp == 0) {
	kl_sp = 1;
	dbginfo(10, "klas_reduce(): kl_sp=%ld\n",kl_sp);
	return;
    }

    x = i = 0;
    while (! x) {
	x = kl_sp & (1 << i);
	i++;
    }

    kl_sp = x;

    dbginfo(10, "klas_reduce(): kl_sp=%ld\n",kl_sp);

    return;
}

/****************************************************************************
* setzt klassen aus angegebenem string von kommandozeile                    *
****************************************************************************/
void a_s_set_klas(was)
char	*was;
{
    char	*zeig;
    int		nr;

    dbginfo(5, "a_s_set_klas(%s)\n",was);

    kl_sp = 0L;

    zeig = was + strlen(KEY1);

    /* wert einlesen, dann nach solange nach komma suchen, */
    /* bis nichts mehr gefunden wird. zwischendurch alle werte merken */

    sscanf(zeig,"%d",&nr);
    kl_sp |= (1 << (nr - 1));
    while (zeig = strchr(zeig,SPEZ_TREN)) {
	zeig++;
	sscanf(zeig,"%d",&nr);
	kl_sp |= (1 << (nr - 1));
    }

    dbginfo(10, "a_s_set_klas() fertig. kl_sp=%ld\n",kl_sp);
}

/****************************************************************************
* versucht aus zeig nr des angegebenen knotens zu bestimmen                 *
****************************************************************************/
int get_nr_from_name(was)
char	*was;
{
    static char	knn[14];
    int		nr = 0;
    int		i;
    char	*zeig;

    dbginfo(5, "get_nr_from_name(%s)\n",was);

    /* zeichenkette heraussuchen und nach entsprechender nr suchen */
    if (zeig = strchr(was,SPEZ_TREN)) {
	strncpy(knn,was,zeig-was);
	knn[zeig-was] = '\0';
    } else
	strcpy(knn,was);

    for (i = 0;i < data[0].klassen[0].knot_anz;i++) {
	if (! strncmp(knn,data[0].klassen[0].knoten[i].kn_name,strlen(knn))) {
	    nr = i + 1;
	    break;
	}
    }

    dbginfo(10, "get_nr_from_name() fertig. returns=%d\n",nr);

    return (nr);
}

/****************************************************************************
* setzt knoten aus angegebenem string von kommandozeile                     *
****************************************************************************/
void a_s_set_knot(was)
char	*was;
{
    char	*zeig,*help;
    int		nr = 0;

    dbginfo(5, "a_s_set_knot(%s)\n",was);

    kn_sp = 0L;

    zeig = was + strlen(KEY2) + 1;

    /* wert einlesen, dann noch solange nach komma suchen, */
    /* bis nichts mehr gefunden wird. zwischendurch alle werte merken */

    sscanf(zeig,"%d",&nr);
    if (! nr)
	nr = get_nr_from_name(zeig);
    if (nr)
	kn_sp |= (1 << (nr - 1));
    while (zeig = strchr(zeig,SPEZ_TREN)) {
	zeig++;
	nr = 0;
	sscanf(zeig,"%d",&nr);
	if (! nr)
	    nr = get_nr_from_name(zeig);
	if (nr)
	    kn_sp |= (1 << (nr - 1));
    }

    dbginfo(10, "a_s_set_knot() fertig. returns=%ld\n",kn_sp);
}

/****************************************************************************
* setzt leistungsgroessen aus angegebenem string von kommandozeile          *
****************************************************************************/
void a_s_set_lg(was)
char	*was;
{
    dbginfo(5, "a_s_set_lg(%s)\n",was);

    w_spez = 0L;

    if (strstr(was,KEY3))
	w_spez |= LAMBDA;
    if (strstr(was,KEY4))
	w_spez |= E;
    if (strstr(was,KEY5))
	w_spez |= MUE_INV;
    if (strstr(was,KEY6))
	w_spez |= RHO;
    if (strstr(was,KEY7))
	w_spez |= MVZ;
    if (strstr(was,KEY8))
	w_spez |= MAA;
    if (strstr(was,KEY9))
	w_spez |= MWZ;
    if (strstr(was,KEY10))
	w_spez |= MWSL;

    dbginfo(10, "a_s_set_lg() fertig. returns=%ld\n",w_spez);
}

/****************************************************************************
* analysiert string von kommandozeile zur datenspezifikation                *
****************************************************************************/
void analyse_string(was)
char	*was;
{
    char	st1[128],st2[128],st3[128];
    int		i;
    int		anz = 0;
    char	*zeig,*alt;

    
    if (!was)
	return;
    
    dbginfo(5, "analyse_string(%s)\n",was);

    /* zerlegen in einzelteile, durch suchen nach trennzeichen */
    zeig = strchr(was,STR_TREN);
    if (zeig) {
	anz ++;
	if (zeig - was) {
	    strncpy(st1,was,zeig - was);
	    st1[zeig-was]='\0';
	} else
	    strcpy(st1,"");
	alt = zeig + 1;

	zeig = strchr(alt,STR_TREN);
	if (zeig) {
	    anz ++;
	    if (zeig - alt) {
		strncpy(st2,alt,zeig - alt);
		st2[zeig-alt]='\0';
	    } else
		strcpy(st2,"");
	    alt = zeig + 1;

	    if (strlen(alt)) {
		strcpy(st3,alt);
	    } else {
		strcpy(st3,"");
	    }
	    anz ++;
	} else {
	    anz ++;
	    strcpy(st2,alt);
	    strcpy(st3,"");
	}
    } else {
	anz = 1;
	strcpy(st1,was);
	strcpy(st2,"");
	strcpy(st3,"");
    }

    switch (anz) {
    case 1:			/* klasse oder knoten oder leistungsgroesse angegeben */
	/* feststellen, was angegeben ist und  */
	/* setzen der entsprechenden variablen */
	if (! strncmp(st1,KEY1,strlen(KEY1)))
	    a_s_set_klas(st1);
	else
	    if (! strncmp(st1,KEY2,strlen(KEY2)))
		a_s_set_knot(st1);
	    else
		a_s_set_lg(st1);
	break;
    case 2:			/* klasse und knoten oder klasse und lg oder knoten und lg */
	/* feststellen, was angegeben ist und  */
	/* setzen der entsprechenden variablen */
	if (! strncmp(st1,KEY1,strlen(KEY1))) {
	    a_s_set_klas(st1);
	    if (! strncmp(st2,KEY2,strlen(KEY2)))
		a_s_set_knot(st2);
	    else
		a_s_set_lg(st2);
	}
	if (! strncmp(st1,KEY2,strlen(KEY2))) {
	    a_s_set_knot(st1);
	    a_s_set_lg(st2);
	}
	/* auch leeren string noch beruecksichtigen */
	if (strlen(st1) == 0) {
	    if (! strncmp(st2,KEY1,strlen(KEY1)))
		a_s_set_klas(st2);
	    else
		if (! strncmp(st2,KEY2,strlen(KEY2)))
		    a_s_set_knot(st2);
		else
		    a_s_set_lg(st2);
	}
	break;
    case 3:			/* klasse, knoten und leistungsgroesse angegeben */
	/* setzen der entsprechenden variablen */
	a_s_set_klas(st1);
	a_s_set_knot(st2);
	a_s_set_lg(st3);
	break;
    default: 
	/* dieser fall kann eigentlich nicht eintreten */
	fprintf(stderr,"Fehler bei der Spezifikationsangabe\n");
	break;
    }

    dbginfo4(10, "Anz: %d S1: %s S2: %s S3: %s\n",anz,st1,st2,st3);
}

/****************************************************************************
* ueberprueft vor alle methoden, ob datei vorhanden und vermerkt diese      *
****************************************************************************/
void test_and_set_methods(modell)
char	*modell;
{
    static char	ergebnis[128];	/* enthaelt temporaeren dateinamen */
    static char	code[10];
    	   char	lo_erg[512];	/* dateiname mit evtl. pfad */
    pdb_entry_t	*help;
    FILE	*fp;
    int		len;

    dbgtext(5, "test_and_set_methods()\n");

    ResetDBEntry();

    while (help=NextDBEntry()) {
	strcpy(ergebnis,"A_");
	strcat(ergebnis,help->codierung);
	strcat(ergebnis,"_");
	strcat(ergebnis,modell);
	if (verzeichnis) {
	    strcpy(lo_erg,verzeichnis);
	    strcat(lo_erg,"/");
	    strcat(lo_erg,ergebnis);
	} else
	    strcpy(lo_erg,ergebnis);
	if (fp = fopen(lo_erg,"r")) {
	    len = strlen(help->codierung) + 3;
	    strncpy(code,ergebnis,len);
	    code[len] = '\0';
	    store_methode(help->verfahren,code);
	    fclose(fp);
	    /* !!! da parametervariationsdatei gefunden, schleife eigentlich
	       beenden, weils sonst eh nicht unterstuetzt wird */
	} else {
	    ergebnis[0] = 'a';
	    if (verzeichnis) {
		strcpy(lo_erg,verzeichnis);
		strcat(lo_erg,"/");
		strcat(lo_erg,ergebnis);
	    } else
		strcpy(lo_erg,ergebnis);
	    if (fp = fopen(lo_erg,"r")) {
		len = strlen(help->codierung) + 3;
		strncpy(code,ergebnis,len);
		code[len] = '\0';
		store_methode(help->verfahren,code);
		fclose(fp);
	    }
	}
    }

    dbgtext(10, "test_and_set_methods() fertig.\n");
}

int
main(argc,argv)
	int	argc;
	char	**argv;
{
    char	*input = (char *) NULL;
    char	inp_nam[128];
    char	*help;
    int		i,j;
    char	*spez_str = (char *) NULL;	/* ausgabespezifikationsstring */
    FILE	*out_file = stdout;		/* ausgabe der xgadd daten auf */
    pid_t	pid;
    int		pipefd[2];
    char	**new_argv;
    int		new_argc = 2;
    int		res;

    new_argv = (char **) malloc(new_argc * sizeof(char *));
    new_argv[0] = (char *) malloc(6);
    new_argv[1] = (char *) NULL;
    strcpy(new_argv[0],"xgadd");

    /* grundmenge an platz fuer eingabedaten belegen */
    data = (DATEI *) malloc(MAX * sizeof(DATEI));

    /* parameter checken ob option, methode oder modell */
    for (i = 1;i < argc;i++) {
	if (argv[i][0] == '-') {
	    /* option gefunden */
	    if (dbg)
		fprintf(stderr,"Option: %s\n",argv[i]);

	    switch (*(argv[i] + 1)) {
	    case 'D':
		if (i+1 < argc) {
		    j = atoi(argv[i + 1]);
		    if (j) {
			i++;
			dbg = j;
		    } else
			dbg = 1;
		} else
		    dbg = 1;
		break;

	    case 'M':
		/* naechsten Parameter als Modell interpretieren */
		if (i + 1 < argc)
		    store_modell(argv[i + 1]);
		i++;
		break;

	    case 'S':
		/* spezifikationsstring zur Auswahl der Anzeigedaten */
		if (i + 1 < argc)
		    spez_str = argv[i + 1];
		i++;
		break;
	    case 'v':
		only_sets = TRUE;
		fprintf(stderr,"Folgende Sets koennten erzeugt werden:\n");
		break;
	    case 'p':
		if (i + 1 < argc) {
		    verzeichnis = (char *) malloc(strlen(argv[i + 1]) + 1);
		    strcpy(verzeichnis,argv[i + 1]);
		    i++;
		}				
		break;
	    case 'X':
		/* !!! argumentsvektor aufbauen und an xgadd geben */
		if (i + 1 < argc) {
		    char *help = argv[i+1];
		    char *tmp[128];

		    /*  angegebenen parameter zerlegen */
		    while (help) {
			sscanf(help,"%s",tmp);
			new_argc++;
			new_argv = (char **) realloc(new_argv,
						     new_argc * sizeof(char *));
			new_argv[new_argc - 2] = (char *) 
			    malloc(strlen(tmp) + 1);
			new_argv[new_argc - 1] = (char *) NULL;
			strcpy(new_argv[new_argc - 2],tmp);

			help = strchr(help,' ');
			if (help)
			    help++;
		    }
		    i++;
		}
		break;
	    default:
		fprintf(stderr,"Ungueltige Option: %s\n",argv[i]);
		break;
	    }
	} else 
	    /* methoden und modelle in feld merken */
	    if (help = method_ok(argv[i])) {
		/* methode gefunden */
		store_methode(argv[i],help);
	    } else {
		/* modell gefunden */;
		store_modell(argv[i]);
	    }
    }

    /* pepsy-datenbank der gueltigen verfahren laden */
    if (!CountPepsyDB())
	GetPepsyDB();


    /* wenn kein verfahren und ein modell angegeben, selbst verfahren suchen */
    if (anz_mod == 1 && anz_met == 0) 
	test_and_set_methods(modelle[0]);

    /* wenn eine methode angegeben wurde, mindestens auch ein modell noetig */
    if (anz_mod == 0 && anz_met > 0 || anz_mod > 0 && anz_met == 0) {
	fprintf(stderr,"Mindestens ein Modell und ein Verfahren angeben!\n");
	exit(1);
    }

    pipe(pipefd);

    if ((pid = fork()) == 0) {
	dbginfo(0, "Kindprozess %d\n",pid);
	close(0);
	dup(pipefd[0]);
	close(pipefd[1]);
	res = execvp("xgadd",new_argv);
	if (dbg || res) 
	    fprintf(stderr,"Result von execvp: %d\n",res);
	exit(1);
    } else
	if (pid > 0) {
	    dbginfo(0, "Vaterprozess %d\n",pid);
	    close(1);
	    dup(pipefd[1]);
	    close(pipefd[0]);
	}

    /* laenderabhaengigkeit benutzen */
    p_lang_init("pepout");
    vt1 = spr("pepo31");	/* Auftragsklasse */
    vt2 = spr("pepo37");	/* charakteristische Netzgroessen */
    vt3 = spr("pepo34");	/* Gesamtleistungsgroessen: */
    vt4 = spr("legen0");	/* Legende */

    /* gueltigen fall aussuchen und entsprechende daten ausgeben */
    if (anz_mod <= 1 && anz_met <=1)
	problemfall = 1;
    else
	if (anz_mod <= 1)
	    problemfall = 2;
	else
	    if (anz_met <= 1) 
		problemfall = 3; /* eigentlich fall 3a */
	    else
		problemfall = 4; /* eigentlich fall 3b */

    switch (problemfall) {
    case 1:
	dbgtext(20, "Fall 1\n");

	if (anz_mod == 1 && anz_met == 1) {
	    strcpy(inp_nam,methoden[0]);
	    strcat(inp_nam,modelle[0]);
	    input = inp_nam;
	}

	/* daten einlesen */
	read_data_in_struct(input,0);
	/* spezifikationsstring abarbeiten */
	analyse_string(spez_str);
	if (data[0].param_var) {
	    /* parametervariation */
	    if (w_spez == ALL)
		w_spez = RHO;
	    if (! only_sets) {
		fprintf(out_file,"@titletext Parametervar. Modell: %s\n",
			modelle[0]);	
		fprintf(out_file,"@xaxis %s\n",data[0].param_string);
		name_y_achse(out_file,w_spez);
		fprintf(out_file,"\n");
	    }
	    fall_1_pv(out_file,kl_sp,kn_sp,w_spez);
	} else {
	    /* header erzeugen */
	    if (! only_sets)
		header_1(out_file,0,TRUE,BARW,BARO);
	    /* daten fuer fall 1 ausgeben */
	    fall_1(out_file,0,kl_sp,kn_sp,w_spez);
	}
	break;

    case 2:
	dbgtext(20, "Fall 2\n");

	/* platz fuer alle einzulesenden dateien anfordern */
	data = (DATEI *) realloc(data,anz_met * sizeof(DATEI));
	/* daten einlesen */
	for (i = 0;i < anz_met;i++) {
	    strcpy(inp_nam,methoden[i]);
	    strcat(inp_nam,modelle[0]);
	    input = inp_nam;
	    read_data_in_struct(input,i);
	}
	/* spezifikationsstring abarbeiten */
	analyse_string(spez_str);

	if (! only_sets) {
	    if (w_spez == ALL)
		w_spez = RHO;
	    klas_reduce();
	    header_2(out_file,0,TRUE,BARW,BARO,(char *) 0,w_spez);
	}
	if (anz_knot && anz_klas)
	    for (i = 0;i < anz_met && set_zahl < MAXD;i++) {
		fprintf(out_file,"\"%s\n",ur_meth[i]);
		if (! only_sets)
		    fall_2(out_file,i,kl_sp,kn_sp,w_spez);
	    }
	break;

    case 3:
	dbgtext(20, "Fall 3a\n");

	/* platz fuer alle einzulesenden dateien anfordern */
	data = (DATEI *) realloc(data,anz_mod * sizeof(DATEI));
	/* daten einlesen */
	for (i = 0;i < anz_mod;i++) {
	    strcpy(inp_nam,methoden[0]);
	    strcat(inp_nam,modelle[i]);
	    input = inp_nam;
	    read_data_in_struct(input,i);
	}
	/* spezifikationsstring abarbeiten */
	analyse_string(spez_str);

	if (! only_sets) {
	    if (w_spez == ALL)
		w_spez = RHO;
	    klas_reduce();
	    header_3a(out_file,0,TRUE,BARW,BARO,(char *) 0,w_spez);
	}
	if (anz_knot && anz_klas)
	    for (i = 0;i < anz_mod && set_zahl < MAXD;i++) {
		fprintf(out_file,"\"%s\n",modelle[i]);
		if (! only_sets)
		    fall_2(out_file,i,kl_sp,kn_sp,w_spez);
	    }
	break;

    case 4:
	dbgtext(20, "Fall 3b\n");
				
	/* platz fuer alle einzulesenden dateien anfordern */
	data = (DATEI *) realloc(data,anz_mod*anz_met * sizeof(DATEI));
	/* daten einlesen */
	for (j = 0;j < anz_met;j++)
	    for (i = 0;i < anz_mod;i++) {
		strcpy(inp_nam,methoden[j]);
		strcat(inp_nam,modelle[i]);
		input = inp_nam;
		read_data_in_struct(input,j * anz_mod + i);
	    }
	/* spezifikationsstring abarbeiten */
	analyse_string(spez_str);

	if (! only_sets) {
	    if (w_spez == ALL)
		w_spez = RHO;
	    klas_reduce();
	    header_3b(out_file,0,TRUE,BARW,BARO,(char *) 0,w_spez); 
	}
	if (anz_knot && anz_klas)
	    for (j = 0;j < anz_met;j++)
		for (i = 0;i < anz_mod && set_zahl < MAXD;i++) {
		    fprintf(out_file,"\"%s-%s\n",ur_meth[j],modelle[i]);
		    if (! only_sets)
			fall_2(out_file,j * anz_mod + i,kl_sp,kn_sp,w_spez);
		}
	break;
    }

    dbginfo4(0, "# Modelle: %d  Methoden: %d Klassen: %d Knoten: %d\n",
		anz_mod,anz_met,anz_klas,anz_knot);
	
    /* anzeige der gespeicherten daten in parametervariation */
    if (dbg) 
	for (i = 0;i < anz_par_var;i++)
	    show_data(i);

    exit(0);
}
