/*

                                         P E P S Y
                         
        Performance and Evaluation Prediction System

        Copyright 1987, 1988

                 Universitaet Erlangen-Nuernberg,
                 Lehrstuhl fuer Betriebssysteme
                 Martensstrasse 1

             8520 Erlangen

        Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
        PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
        gefuehrt werden.
*/

#define LO_ALIASLIST_C
static char *SccsId="@(#)PEPSY:\taliaslist.c\t1.1\t93/07/07 10:51:44";

#include <stdio.h>
#include <string.h>
#include "libohs.h"
#include "d_debug.h"

/*	jetzt aus libohs.h 
extern int ohs_error_val;
extern char *ohs_error_text;
*/
extern char	*malloc();
extern void	free();
extern int dbg;


/************************************************************************
*                                                                       *
*       Funktion: aliaslist()                                           *
*                                                                       *
*       Aufgabe: verwaltet die Aliases von Schluesselwoertern bei /S()  *
*                                                                       *
*       Eingabe: Alias, Urspr. Wort, Modus (Abfrage, Einfuegen, Init)   *
*                                                                       *
************************************************************************/

int aliaslist(alias, word, mode)
char *alias;
char *word;
int mode;
{
    static int	max;
    char	errmsg[100];
    int 	i, retval, return_after_for;

    static	struct aliastype *first;  /* we need an anchor for our list */
    		struct aliastype *work1, *work2, *remember_work1;

    dbgtext(50, "aliaslist()\n");

    switch(mode)
    {
    case OH_request:
	if(alias == NULL || word == NULL)
	{
	    ohs_error_val = -15;
	    ohs_error_text = strdup(": NULL as argument in aliaslist");
	    return(OH_error);
	}
	dbginfo(52, "aliaslist(): alias='%s' mode=OH_request\n", alias);

	work1=first;
	for(i=0;i<max;i++)
	{
				/* check if alias exists */

	    if(strcmp(alias,work1->alias)==0){	/* yes: return word */
		strcpy(word,work1->word);

		dbginfo(50, "aliaslist(): fertig. alias='%s' -> OK\n", word);
		return(OH_ok);
	    }
	    else
	    {
		work1=work1->next; /* set pointer to next element */
	    }
	}
				/* no: return error */
	ohs_error_text=strdup(alias);
	ohs_error_val = -1;
	strcpy(word,"");

	retval=OH_error;
	break;

    case OH_add:	
	if(alias == NULL || word == NULL)
	{
	    ohs_error_val = -15;
	    ohs_error_text = strdup(": NULL as argument in aliaslist");
	    return(OH_error);
	}
	dbginfo(52, "aliaslist(): alias='%s' ", alias);
	dbginfo(52, "word='%s' mode=OH_add\n", word);

	work1=first;
	return_after_for = 0;
	for(i=0;i<max;i++)
	{	/* check whether alias is new or not */

	    if(strcmp(alias,work1->alias)==0)
	    {
		if (strcmp(word,work1->word)!=0)
		{
		    dbgtext(53, "aliaslist(): there exists a different alias, already\n");

		    sprintf(errmsg,"%s --> %s",alias,work1->word);
		    ohs_error_text=strdup(errmsg);
		    ohs_error_val = -2; /* There's already an alias */
		    i=max;
		    remember_work1=work1;
		    return_after_for=1;
		    retval=OH_error;
		}
		else
		{
		    dbgtext(53, "aliaslist(): this alias exists already!\n");

		    i=max;
		    remember_work1=work1;
		    return_after_for=1;
		    retval=OH_ok; /* same keyword & alias exists */
		}
	    }

	    work1=work1->next; /* set pointer to next element */
	}

	if (return_after_for)
	{
	    work1=remember_work1;
	}
	else
	{
	    dbgtext(53, "aliaslist(): this alias is new to me.\n");

				/* copy word and alias */
	    work1->alias=strdup(alias);
	    work1->word=strdup(word);
				/* generate next element */
	    work2=(struct aliastype *)malloc(sizeof(struct aliastype));
				/* and link it into list */
	    work1->next=work2;
	    work2->next=NULL;
				/* inc number of aliases */
	    max++;

	    retval=OH_ok;
	}
	break;

    case OH_init:
	dbgtext(52, "aliaslist(): mode=OH_init\n");

	work1=first;
	for(i=0;i<max;i++){	/* kill all aliases */
	    work2=work1->next; /* store adress of next element */
	    free(work1);	   /* and kill old one */
	    work1=work2;	   /* then goto (saved) adress of next element */
	}
	max=0;

				/* at last, reinit the anchor: */
	first=(struct aliastype *)malloc(sizeof(struct aliastype));

	retval=OH_ok;
	break;

    default:		
	dbgtext(52, "aliaslist(): ERROR unknown mode\n");

	ohs_error_text=strdup("aliaslist");
	ohs_error_val = -4; /* invalid mode */

	retval=OH_error;
	break;
    }

    dbginfo(50, "aliaslist() fertig. -> %s\n", (retval==OH_error)?"OH_error":"OH_ok");

    return(retval);
}
