/*

                                         P E P S Y
                         
        Performance and Evaluation Prediction System

        Copyright 1987, 1988

                 Universitaet Erlangen-Nuernberg,
                 Lehrstuhl fuer Betriebssysteme
                 Martensstrasse 1

             8520 Erlangen

        Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
        PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
        gefuehrt werden.
*/
#define LO_CONVERT_C
static char *SccsId ="@(#)PEPSY:\tconvert.c\t1.7\t94/02/10 17:14:33";

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <sys/stat.h>
#include "d_debug.h"
#include "libpepsy.h"
#include "libohs.h"

extern int dbg;
extern	char *malloc();
extern	char	*getenv();
extern	int	atoi();
extern	void	free();

static char logfilename[1024]; 


/************************************************************************
*                                                                       *
*       Funktion: convert()                                             *
*                                                                       *
*       Aufgabe: Erzeugt aus (evtl.) mehreren Sourcetexten den Hilfetext*
*                                                                       *
*       Eingabe: Liste der Sourcetexte, Name der Hilfedatei(ohne Endung)*
*                                                                       *
*       Ausgabe: OH_error bei Fehler, OH_ok bei Erfolg                  *
*                                                                       *
************************************************************************/

int convert(filelist, helpfile)
char **filelist;
char *helpfile;
{
	FILE *fp_orig, *fp_new;
	FILE *fp_hl, *fp_keyw;
	FILE *fp_keyw_unique;
	FILE *fp_composed;
	FILE *fp_log;
	int hlcount=0, keywcount=0, keyw_per_hl=0, maxkeyw=0;
	int maxlen_keyw=0, maxlen_hl=0;
	int i=0,j=0,k=0,l=0;
	struct OH_control seq;
	char line[1024];
	char c;
	char **keyw;
	int *act;
	int foundflag=0;
	struct stat *timestamp;
	long *hllist, *hloffset;
	char filename[512];
	char comp_name[512];
	int linecount=0;
	int log_ok=1;

/* compose final filename from base and environment */
	if(getenv(LANGUAGEENV)==0){
		ohs_error_val = -5;
		ohs_error_text=strdup( LANGUAGEENV);
		return(OH_error);
	}

	if(getenv(HOMEPATH)==0){
		ohs_error_val = -5;
		ohs_error_text=strdup( HOMEPATH);
		return(OH_error);
	}

	timestamp=(struct stat *)malloc(sizeof(struct stat));
	if(timestamp==NULL){
		ohs_error_val = -3;
		ohs_error_text=strdup("convert");
		return(OH_error);
	}

	sprintf(logfilename, "%s.log", helpfile);

	/* first a syntax check for all helpfiles */

	for(i=0;filelist[i]!=NULL;i++){
		if((fp_orig=fopen(filelist[i],"rb"))==NULL){
			ohs_error_val = -6;
			ohs_error_text=strdup(filelist[i]);
			return(OH_error);
		}
		fprintf(stderr,"Checking syntax of file %s\n",filelist[i]);

		j=0;			/* to count control-sequences */
		k=0;

		while(k!=-1) {		/* until EOF */
			k=fgetc(fp_orig);

			switch(k){
				case '/':	  j++;
						seq=get_c_seq(fp_orig);

						if(seq.c_seq=='E'){ /* Syntax Error */
							rewind(fp_orig);
							l=0;
							linecount=1;	/* get linenumber of j'th control sequence */
							while(l<j) {	/* in original subfile */
								c=fgetc(fp_orig);
								if(c=='/'){				/* found contr. seq. */
									l++;			/* increase control-counter */
									c=fgetc(fp_orig);		/* get contr.seq.-letter */
								}
								if(c=='\n')	linecount++;	/* found carr/ret */
							}
							sprintf(line,"Line #%d in file %s",linecount,filelist[i]);
							ohs_error_val = -11;
							ohs_error_text=strdup(line);
							fclose(fp_orig);
							return(OH_error);
						}
						break;

				default:	break;
			}
		} /* while */
		fclose(fp_orig);
	}
	fprintf(stderr,"Syntax of all helpfiles is ok.\n");
	/* compose all helpfiles to one */

	tmpnam(comp_name);	/* make a unique tempfilename */

	if((fp_composed=fopen(comp_name,"wb"))==NULL) {
		ohs_error_val = -6;
		ohs_error_text=strdup("for composed helpfiles");
		return(OH_error);
	}

	if((fp_log=fopen(logfilename,"wb"))==NULL) {
		log_ok = 0;
	}

	for(i=0;filelist[i]!=NULL;i++){
		if((fp_orig=fopen(filelist[i],"rb"))==NULL){
			ohs_error_val = -6;
			ohs_error_text=strdup(filelist[i]);
			return(OH_error);
		}

		j=0;			/* to count columns */
		k=0;
		while(k!=-1) {		/* until EOF */
			k=fgetc(fp_orig);
			if(k=='\r' || k=='\n') j=0;
			else	j++;
			switch(k){
				case '\r':
				case '\n':
				case ' ':
				case '\t':	while(isspace(k=fgetc(fp_orig)));

							fprintf(fp_composed," ");
							fseek(fp_orig,-1,1);
							break;

				case -1:	break; /* eof */

				case '/':	if((k=fgetc(fp_orig))=='#')
							get_until(fp_orig,line,")");
							else{
								fprintf(fp_composed,"/%c",k);
							}
							break;
				case '#':	if(j==1)   /* '#' in 1st column */
							{
								get_until(fp_orig,line,"\n");
								j=0;
							}
							else
							{	 /* '#' somewhere else */
								fputc(k,fp_composed);
							}
							break;
				default:	fputc(k,fp_composed);
			}
		} /* while */
		fclose(fp_orig);
	}

	fflush(fp_composed);	/* write it ! (just to be sure) */
	fclose(fp_composed);
	
	fprintf(stderr,"Created composed helpfile\n");

	/* All files in "filelist" are now merged to fp_composed */

	if((fp_composed=fopen(comp_name,"rb"))==NULL){ /* re-open for reading */
		ohs_error_val = -6;
		ohs_error_text=strdup("composed file for reading");
		return(OH_error);
	}

		sprintf(filename,"%s.hlp.%.2s",helpfile,getenv(LANGUAGEENV));

	if((fp_new=fopen(filename,"wb"))==NULL){
		perror(filename);
		ohs_error_val = -6;
		ohs_error_text=strdup(filename);
		return(OH_error);
	}

	if((fp_hl=tmpfile())==NULL) {
		ohs_error_val = -6;
		ohs_error_text=strdup("tempfile for headlines");
		return(OH_error);
	}

	if((fp_keyw=tmpfile())==NULL) {
		ohs_error_val = -6;
		ohs_error_text=strdup("tempfile for keywords");
		return(OH_error);
	}

	if((fp_keyw_unique=tmpfile())==NULL) {
		ohs_error_val = -6;
		ohs_error_text=strdup("tempfile for unique keywords");
		return(OH_error);
	}

	if(log_ok==1)
	{
		fprintf(fp_log,"\nLogfile for creation of file %s.hlp.%.2s\n\n",helpfile,getenv(LANGUAGEENV));
		fprintf(fp_log,"================================================================================\n");
		fprintf(fp_log,"\nSourcefiles included:\n\n");
	}

	for(i=0;filelist[i]!=NULL;i++){
		stat(filelist[i],timestamp);
		fprintf(fp_new,"%s: Size %ld, last modify:%s",filelist[i],timestamp->st_size,ctime(&(timestamp->st_mtime)));
		/* I don't append a '/n' as it is produced by ctime() !! */
		
		if(log_ok==1)
			fprintf(fp_log,"%s: Size %ld, last modify:%s",filelist[i],timestamp->st_size,ctime(&(timestamp->st_mtime)));
	}

	aliaslist("","",OH_init);
	
	k=0;

	/* scan file for headlines, keywords and # of keywords per headline */
	while(feof(fp_composed)==0){
		get_until(fp_composed,line,"/");

		if(feof(fp_composed)!=0) break;
		seq=get_c_seq(fp_composed);

		if(seq.c_seq!='E') {
			switch(seq.c_seq) {
/* Headline */		case 'H':	   /* count maximum number of keywords
						   in a section */
						if(keyw_per_hl>maxkeyw)
							maxkeyw = keyw_per_hl;

						keyw_per_hl=0;
						hlcount++;
						fprintf(fp_hl,"/H(%s#%s)\n",seq.argument1,seq.argument2);
						if((int)strlen(line)>maxlen_hl)
							maxlen_hl=strlen(line);
						break;

/* Keyword */		case 'K':
/* Non-Visible Keyword */	case 'N':	   keywcount++;
						keyw_per_hl++;
						fprintf(fp_keyw,"/K(%s)\n",seq.argument1);
						if((int)strlen(line)>maxlen_keyw)
							maxlen_keyw=strlen(line);
						break;

/* Substitute */	case 'S':	keywcount++;
						keyw_per_hl++;
						fprintf(fp_keyw,"/K(%s)\n",seq.argument2);

						if(aliaslist(seq.argument1, seq.argument2, OH_add)==OH_error)
						{
							char conflict[512];

							aliaslist(seq.argument1,conflict,OH_request);

							fprintf(stderr,"\nOHS Warning: Possible conflict within the aliases detected\n");
							fprintf(stderr,"             '%s' references to '%s' and '%s' !\n\n",seq.argument1, seq.argument2, conflict);
							if(log_ok==1)
							{
								if(k==0)
								{
									fprintf(fp_log,"================================================================================\n\n");
									k=1;  /* k is used here only to check wether we've already drawn a line in the logfile */
								}
									
								fprintf(fp_log,"Warning: Two or more keywords have the same alias: %s !\n",seq.argument1);
							}
						}

						if((int)strlen(line)>maxlen_keyw)
							maxlen_keyw=strlen(line);
						break;

/* Rest */			case 'B':	/* Bold */
					case 'P':	/* Paragraph */
					case '#':	/* Comment */
					case '/':	  /* Character '/' */
					case 'I':	/* Indent */
					case '_':	/* Spaces */
						break;

/* Error */			default:		sprintf(seq.argument1,"%c",seq.c_seq);
						ohs_error_text=strdup(seq.argument1);
						ohs_error_val = -10;
						break;
			} /* switch */
		}
		else {
		/* last section might have the most keywords in it, too !! */
			if(keyw_per_hl>maxkeyw)
				maxkeyw=keyw_per_hl;
		}
	} /* while */

	sprintf(line,"Helpfile contains %d headlines and %d keywords\n",hlcount,keywcount);
	fprintf(stderr,"%s",line);

	/* Now hlcount contains the number of headlines, keywcount the number
	   of keywords and maxkeyw the maximum number of keywords in a section.
	   Maxlen_hl holds the length of the longest headline, as maxlen_keyw
	   holds the length of the longest keyword.
	   All headlines are stored in tempfile1 and all keywords in tempfile2.
	   Let's allocate mem  */

	maxlen_keyw ++; /* Need space for \0 */
	maxlen_hl ++;   /* dto. */

	keyw	= (char **)malloc((keywcount+2)*sizeof(char *));

	if(keyw==NULL){
		ohs_error_val = -3;
		ohs_error_text=strdup("convert");
		fclose(fp_keyw);	fclose(fp_hl);
		fclose(fp_composed);
		return(OH_error);
	}

	i=j=0;

	rewind(fp_keyw);

	/* Read through the  Keywordfile and store only unique keywords */
	while(!feof(fp_keyw)){
		get_until(fp_keyw,line,"/");
		if(feof(fp_keyw)) break;
		seq=get_c_seq(fp_keyw);
		foundflag=0;
		for(i=0;i<j;i++) {
			if(strcmp(seq.argument1,keyw[i])==0)
				foundflag=1;
		}

		if(foundflag==0){  /*  Keyword wasn't read yet  */
			keyw[j]=strdup(seq.argument1);
			fprintf(fp_keyw_unique,"/K(%s)\n",keyw[j]);
			j++;
		}
	}

	keywcount=j;	/* now we have keywcount unique keywords */

	sprintf(line,"There are %d unique keywords\n",j);
	fprintf(stderr,"%s",line);

	hllist = (long *)malloc(maxkeyw*(keywcount+1)*sizeof(long));
	act = (int *)malloc((keywcount+1)*sizeof(int));
	hloffset = (long *)malloc((hlcount+1)*sizeof(long));

	for(i=0;i<keywcount;i++)
		for(j=0;j<maxkeyw;j++)
			hllist[i*maxkeyw+j]=-1L;
	for(i=0;i<keywcount;i++) act[i]=0;

	/* Ok. Now we got a file with headlines and one with every keyword
	   only once. Let's create the reference-list for the keywords */

	i=j=0;
	rewind(fp_composed);

	while(!feof(fp_composed)) {
		get_until(fp_composed,line,"/");
		if(feof(fp_composed)) break;

		seq=get_c_seq(fp_composed);

		switch(seq.c_seq) {
			case 'H':	   hloffset[j]=ftell(fp_composed)-strlen(seq.argument1)-strlen(seq.argument2)-5;
					/* cause we read headline, hldepth and /H(#)  */
					j++;
					break;
			case 'K':
			case 'N':	for(i=0;i<keywcount;i++){
						if((strcmp(seq.argument1,keyw[i])==0) &&
/* Don't get a reference twice */  		(hllist[i*maxkeyw+act[i]-1]!=hloffset[j-1])) {
							hllist[i*maxkeyw+act[i]]=hloffset[j-1];
							act[i]++;
						}
					}
					break;
			case 'S':	for(i=0;i<keywcount;i++){
						if((strcmp(seq.argument2,keyw[i])==0) &&
/* Don't get a reference twice */  		(hllist[i*maxkeyw+act[i]-1]!=hloffset[j-1])) {
							hllist[i*maxkeyw+act[i]]=hloffset[j-1];
							act[i]++;
						}
					}
					break;
			default:	break;
		} /* switch */
	} /* while */

	fflush(fp_hl);		fflush(fp_keyw_unique);
	fflush(fp_orig);	fflush(fp_composed);

	rewind(fp_hl);		rewind(fp_keyw_unique);
	rewind(fp_orig);	rewind(fp_composed);

/* Now let's build the new helpfile. First the intern data and headlines */

	for(i=0;filelist[i]!=NULL;i++);	/* count files in list */
	fprintf(fp_new,"%d\n",i);	/* write number of files */
	
	for(i=0;filelist[i]!=NULL;i++){
		stat(filelist[i],timestamp);
		fprintf(fp_new,"%s: Size %ld, last modify:%s",filelist[i],timestamp->st_size,ctime(&(timestamp->st_mtime)));
		/* I don't append a '/n' as it is produced by ctime() !! */
	}

	if(log_ok==1)
	{
		fprintf(fp_log,"\n================================================================================\n");
		fprintf(fp_log,"\nHeadlines: %d\nKeywords: %d\nMaximum number of keywords per headline: %d\n",hlcount,keywcount,maxkeyw);
	}

	fprintf(fp_new,"%d\n",hlcount);
	fprintf(fp_new,"%d\n",keywcount);
	fprintf(fp_new,"%d\n\n",maxkeyw);

	if(log_ok==1)
	{
		fprintf(fp_log,"\n================================================================================\n");
		fprintf(fp_log,"\nHeadlines in helpfile:\n\n");
	}

	for(i=0;i<hlcount;i++){
		get_until(fp_hl,line,"/");
		seq=get_c_seq(fp_hl);
		fprintf(fp_new,"/H(%s#%s) ",seq.argument1,seq.argument2);
		fprintf(fp_new,"%ld\n",hloffset[i]);

		if(log_ok==1)
		{
			for(j=0;j<atoi(seq.argument2)*INDENTMULT;j++)
				fprintf(fp_log," ");

			fprintf(fp_log,"%s\n",seq.argument1);
		}
	}

	fprintf(fp_new,"\n");

/* now come the keywords, each one followed by its reference-list. */
	if(log_ok==1)
	{
		fprintf(fp_log,"\n================================================================================\n");
		fprintf(fp_log,"\nKeywords in helpfile:\n\n");
	}

	for(i=0;i<keywcount;i++){
		int count_refs=0;

		get_until(fp_keyw_unique,line,"/");
		seq = get_c_seq(fp_keyw_unique);
		fprintf(fp_new,"/K(%s) ",seq.argument1);

		if(log_ok==1)
			fprintf(fp_log,"%s",seq.argument1);


		for(j=0;j<maxkeyw && hllist[i*maxkeyw+j]>-1L;j++){
			count_refs++;
			fprintf(fp_new,"%ld ",hllist[i*maxkeyw+j]);
		}
		fprintf(fp_new,"-1\n");

		if(log_ok==1)
		{
			for(j=0;j<(5-((int)strlen(seq.argument1)>>3));j++)
				fprintf(fp_log,"\t");
			fprintf(fp_log,"(referenced %d times)\n",count_refs);
		}
	}
	fprintf(fp_new,"\n");

	fprintf(fp_new,"%c",255);	/* to signal start of helptext */

	rewind(fp_composed);

	while(!feof(fp_composed)){
		c=fgetc(fp_composed);
		fputc(c,fp_new);
	}

	if(log_ok==1)
	{
		fprintf(fp_log,"\n================================================================================\n");
		fprintf(fp_log,"\nHelpfile was created properly.\n\n");
	}

	fclose(fp_orig);		fclose(fp_new);
	fclose(fp_hl);			fclose(fp_keyw);
	fclose(fp_keyw_unique); 	fclose(fp_composed);
	
	if(log_ok==1)
		fclose(fp_log);

	free(hllist);			free(keyw);
	free(act);
	free(timestamp);

	unlink(comp_name);	/* don't forget to delete tempfile */

	return(OH_ok);
}

