/*

                                         P E P S Y
                         
        Performance and Evaluation Prediction System

        Copyright 1987, 1988

                 Universitaet Erlangen-Nuernberg,
                 Lehrstuhl fuer Betriebssysteme
                 Martensstrasse 1

             8520 Erlangen

        Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
        PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
        gefuehrt werden.
*/
#define LO_OPENHELP_C
static char *SccsId="@(#)PEPSY:\topenhelp.c\t1.4\t93/12/21 14:33:25";

#include <stdio.h>
#include <string.h>
#include "libpepsy.h"
#include "libohs.h"
#include "d_debug.h"

extern char *getenv();
extern int dbg;



/************************************************************************
*                                                                       *
*       Funktion: openhelp()                                            *
*                                                                       *
*       Aufgabe: Oeffnet eine Hilfedatei                                *
*                                                                       *
*       Eingabe: Helpfilename (ohne Endungen o.ae.)                     *
*                                                                       *
*       Ausgabe: geoeffnetes Hilfefile, bei Fehler ist help.fp == NULL  *
*                                                                       *
************************************************************************/

struct OH_file openhelp(helpfile)
char *helpfile;
{
	FILE *fp_help;
	char filename[1024];
	char line[1024];
	int i=0,j=0;
	struct OH_file help;

	dbginfo(50, "openhelp('%s')\n", helpfile);

	line[0] = '\0';

	/* build final filename from base and environment */

	if(getenv(LANGUAGEENV)==NULL){
		dbgtext(20, "openhelp schlug fehl: environment LANGUAGEENV nicht gesetzt\n");
		ohs_error_val = -5;	/* environment not set */
		ohs_error_text=strdup(LANGUAGEENV);
		help.fp = NULL;
		return(help);
	}
	
	if(getenv(HOMEPATH)==NULL){
		dbgtext(20, "openhelp schlug fehl: environment HOMEPATH nicht gesetzt\n");
		ohs_error_val = -5;	/* environment not set */
		ohs_error_text=strdup(HOMEPATH);
		help.fp = NULL;
		return(help);
	}
	
	sprintf(filename,"%s/infos/%s.hlp.%.2s",getenv(HOMEPATH),helpfile,getenv(LANGUAGEENV));

	if((fp_help=fopen(filename,"rb"))==NULL){
		strcpy(line,getenv(LANGUAGEENV));
		switch(line[0]){
			case 'd':	strcpy(filename,"German"); break;
			case 'e':	strcpy(filename,"English"); break;
			case 'i':	strcpy(filename,"Italian"); break;
			case 'f':	strcpy(filename,"French"); break;
			case 's':	strcpy(filename,"Spanish"); break;
			default: 	sprintf(filename,"Unknown: .%s", getenv(LANGUAGEENV)); break;
		}
		/* couldn't open file */
		ohs_error_text=strdup(filename);
		ohs_error_val = -13;
		help.fp = NULL;
	}
	else
	{
	/* build the OH_file-struct */

	help.fp = fp_help;

	/* skip past list of included files */

	rewind(fp_help);
	fscanf(fp_help,"%d",&j);
	for(i=0;i<=j;i++){
		get_until(fp_help,line,"\n"); /* read one line */
	}

	fscanf(fp_help,"%d",&help.hl_count);
	fscanf(fp_help,"%d",&help.keyw_count);
	fscanf(fp_help,"%d",&help.max_keyw);

	/* read 'til headlines */

	get_until(fp_help,line,"/");

	help.start_of_hl=ftell(fp_help);
	for(i=0;i<help.hl_count;i++){
		get_until(fp_help,line,"/");
	}

	/* read 'til keywords */

	help.start_of_keyw=ftell(fp_help);
	for(i=0;i<help.keyw_count;i++){
		get_until(fp_help,line,"-1");
	}

	/* read 'til helptext */

	while((i=fgetc(fp_help)) != 255 && i>=0);

	if(i<0){
		fclose(help.fp);
		ohs_error_val = -7;
		ohs_error_text=strdup("openhelp");
		help.fp=NULL;
		return(help);
	}

	help.start_of_text=ftell(fp_help);

	rewind(fp_help);
	}

	
	dbgtext(50, "openhelp() fertig\n");

	return(help);
}
