/*

                                         P E P S Y
                         
        Performance and Evaluation Prediction System

        Copyright 1987, 1988

                 Universitaet Erlangen-Nuernberg,
                 Lehrstuhl fuer Betriebssysteme
                 Martensstrasse 1

             8520 Erlangen

        Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
        PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
        gefuehrt werden.
*/

#define LO_READHELP_C
static char *SccsId="@(#)PEPSY:\treadhelp.c\t1.7\t94/02/10 17:14:41";

#include <stdio.h>
#include <string.h>
#include "libpepsy.h"
#include "libohs.h"
#include "d_debug.h"

extern int ohs_error_val;
extern char *ohs_error_text;
extern int dbg;
extern int atoi();

static int reference;
static long last_offset;
static int times_found;



/************************************************************************
*                                                                       *
*       Funktion: readhelp_new()                                        *
*                                                                       *
*       Aufgabe: Positioniert Lesezeiger fuer neue Hilfe                *
*                                                                       *
*       Eingabe: Hilfe zu?, Modus(hl/keyw), Verweis#, Helpfile          *
*                                                                       *
*       Ausgabe: Tiefe des geoeffneten Absatzes, OH_error bei Fehler    *
*                                                                       *
************************************************************************/

static int readhelp_new(help, mode, number, helpfile)
char *help;
int mode;
int number;
struct OH_file helpfile;
{
	char				line[1024];
	int					i=0;
	struct OH_control	c_seq;
	char 				c;
	int					returnval=OH_ok;
	long				l=0L,k=0L,m=0L;

	dbginfo(50, "readhelp_new('%s', .)\n", help);

	rewind(helpfile.fp);		/* set position for first read */
	reference = 0;
	times_found = 0;
						
	aliaslist("","",OH_init);	/* kill aliaslist */
						
	switch(mode)
	{
		case OH_headline:

			if(searchheadline(help,helpfile)==FALSE)
			{	
				dbginfo(80, "readhelp_new(): not a headline:%s\n",help);
				ohs_error_val	= -8;	/* not a headline */
				ohs_error_text	= strdup(help);
				returnval		= OH_error;
			}
			else
			{	/* search correct headline in file */

				fseek(helpfile.fp,helpfile.start_of_hl,0);

				c_seq=get_c_seq(helpfile.fp);

				while(strcmp(c_seq.argument1,help)!=0)
				{
					get_until(helpfile.fp,line,"/");
					c_seq=get_c_seq(helpfile.fp);
				}
			
				dbginfo(0,"Found Headline. Has Depth %s\n",c_seq.argument2);
				fscanf(helpfile.fp,"%ld",&l); /* get offset of headline */
				fseek(helpfile.fp,l+helpfile.start_of_text-1,0);
				returnval = OH_ok;
			}

			break;

		case OH_keyword:
		case OH_skip:

			if(searchkeyword(help,helpfile)==FALSE)
			{
				dbginfo(80,"readhelp_new(): not a keyword:%s\n",help);
				ohs_error_val	= -9;	/* not a keyword */
				ohs_error_text	= strdup(help);
				returnval		= OH_error;
			}
			else
			{
				if(mode == OH_skip 
				   && (number>getreferences(help,helpfile)
				  	   || number <= 0))
				{
					i = getreferences(help,helpfile);
					dbginfo(80,"readhelp_new(): not enough references (%s",help);
					dbginfo(80," has only %d",i);
					dbginfo(80," of %d)\n",number);

					ohs_error_val	= -15; /* invalid parameter */
					ohs_error_text	= strdup("referencenumber too big !");
					returnval		= OH_error;
					break;
				}
				
				/* search correct keyword in file */

				fseek(helpfile.fp,helpfile.start_of_keyw,0);

				c_seq=get_c_seq(helpfile.fp);

				while(strcmp(c_seq.argument1,help)!=0)
				{
					get_until(helpfile.fp,line,"/");
					c_seq=get_c_seq(helpfile.fp);
				}

				fscanf(helpfile.fp, "%ld", &l);
				dbginfo(80,"readhelp_new(): first offset for keyword: %ld\n",l)
				m = l; /* save 1st offset in case we gotta go back */

				for(i=0; i<((mode == OH_keyword) ? 0 : number-1); i++)
				{
					dbginfo(90,"readhelp_new(): read referenceoffset # %d\n",i);
					fscanf(helpfile.fp,"%ld",&l); /* get offset #'number' for keyw */
					dbginfo(80,"readhelp_new(): next offset for keyword: %ld\n",l)
				}

				reference = i;

				if(l == last_offset) /* we don't want same text twice */
				{
					fscanf(helpfile.fp,"%ld",&k);

					if(k!=-1) /* next reference found */
					{
						l = k;
						reference ++;
					}
					else	/* no further reference, so let's take first one */
					{
						l = m;
						reference = 1;
					}
				}

				fseek(helpfile.fp,l+helpfile.start_of_text-1,0);

				dbgtext(30, "readhelp(): new keword text offset\n");

				returnval = OH_ok;
			}
			break;

		default:
			ohs_error_val = -4;
			ohs_error_text=strdup( "readhelp");
			returnval = OH_error;
	} /* switch */

	last_offset = l;

	/* get the outermost hldepth */
	l = ftell(helpfile.fp);

	c = fgetc(helpfile.fp); /* read '/' at beginning of helptext */	
	c_seq = get_c_seq(helpfile.fp);
	if(returnval != OH_error && c_seq.c_seq != 'E')
	{
		returnval = atoi(c_seq.argument2);
	}
	else
	{
		returnval = OH_error;
	}
	
	fseek(helpfile.fp, l, 0);

	dbgtext(50, "readhelp_new fertig\n");

	return(returnval);
}

/************************************************************************
*                                                                       *
*       Funktion: readhelp_cont()                                       *
*                                                                       *
*       Aufgabe: Faehrt fort, Hilfe zu lesen                            *
*                                                                       *
*       Eingabe: Hilfe zu?, Returnpuffer, Puffergroesse, Modus(hl/keyw),*
*                Helpfile, Klammertiefe                                 *
*       Ausgabe: Struktur mit gelesenem Hilfetext u.a. Infos            *
*                                                                       *
************************************************************************/

static struct OH_status readhelp_cont(help, buffer, buflen, mode, helpfile, outer_depth)
char *help;
char *buffer;
int buflen;
int mode;
struct OH_file helpfile;
int outer_depth;
{
	struct OH_status	status;
	static int			indent;
	int					i=0;
	struct OH_control	c_seq;
	char 				c;
	char				line[1024];
	long				l;

	dbginfo(50, "readhelp_cont('%s', .)\n", help);

	status.eohelp	= OH_more_text;
	status.textlen	= 0;
	status.textmode	= OH_normal;
	status.hldepth	= 0;
	status.indent	= 0; 
	status.reference= 0;
	
	strcpy(buffer,"");

	if(mode == OH_skip) /* just skip, don't read already ! */
	{
		return(status);
	}

	/* readhelp() was called with same keyword and same mode earlier! */

	dbgtext(55, "readhelp(): getting more text...\n");

	c=fgetc(helpfile.fp);

	if(c=='/')
	{	/* next thing to return is a control sequence */

		c_seq=get_c_seq(helpfile.fp);

		dbginfo(60, "readhelp_cont(): found control-sequence '%c'\n", c_seq.c_seq);

		switch(c_seq.c_seq)
		{
			case 'K':
			case 'B':
			case 'S':
				if((int)strlen(c_seq.argument1)<buflen)
				{
					strcpy(buffer,c_seq.argument1);

					status.eohelp	= OH_more_text;
					status.textlen	= strlen(c_seq.argument1);
					status.hldepth	= -1;
					status.indent	= indent;
					status.reference= reference;

					switch(c_seq.c_seq)
					{
						case 'K': status.textmode	= OH_keyword;
								  aliaslist(c_seq.argument1,c_seq.argument1,OH_add);
								  break;
						case 'B': status.textmode	= OH_bold;
								  break;
						case 'S': status.textmode	= OH_keyword;
								  aliaslist(c_seq.argument1,c_seq.argument2,OH_add);
								  break;
					}
				} else {
					/* Control sequence is too long for buffer */
					ohs_error_val = -14;
					ohs_error_text = strdup(c_seq.argument1);
					status.textmode = OH_error;
				}

				break;

			case 'H':
				if(atoi(c_seq.argument2) <= outer_depth || mode == OH_keyword)
				{
					times_found++;
				}

				if(times_found>=2)
				{
					buffer[0]='\0';
					status.textmode = OH_normal;
					status.textlen  = 0;
					status.indent   = indent;

					if(mode==OH_headline) /* a headline can only have one reference ! */
					{
						status.eohelp=OH_eohelp;
					}
					else
					{               /* look if keyword has further references */
						reference ++;

						fseek(helpfile.fp,helpfile.start_of_keyw,0);

						c_seq=get_c_seq(helpfile.fp);
						while(strcmp(c_seq.argument1,help)!=0){
							get_until(helpfile.fp,line,"/");
							c_seq=get_c_seq(helpfile.fp);
						}

						for(i=0;i<=reference;i++){
							fscanf(helpfile.fp,"%ld",&l); /* get next offset for keyw */
						}

						if(l==-1){
							status.eohelp=OH_eohelp;
						}else{
							fseek(helpfile.fp,l+helpfile.start_of_text-1,0);
							status.eohelp=OH_more_topic;
						}
					}
					times_found = 0;
				}
				else
				{
					if((int)strlen(c_seq.argument1)<buflen)
					{
						strcpy(buffer,c_seq.argument1);
	
						status.eohelp   = OH_more_text;
						status.textlen  = strlen(c_seq.argument1);
						status.hldepth  = -1;
						status.indent   = indent;
						status.reference= reference;
						status.textmode = OH_headline;
						status.hldepth  = atoi(c_seq.argument2);
					} else {
						/* Control sequence is too long for buffer */
						ohs_error_val = -14;
						ohs_error_text = strdup(c_seq.argument1);
						status.textmode = OH_error;
					}
				}

                break;

					
			case 'P':
				if(atoi(c_seq.argument1) < buflen)
				{
					for(i=0;i<atoi(c_seq.argument1);i++){
						buffer[i]='\n';
					}
					buffer[i]='\0';
					status.textmode	= OH_normal;
					status.textlen	= strlen(buffer);
					status.eohelp	= OH_more_text;
					status.indent	= indent;
					status.reference= reference;
				} else {
					ohs_error_val = -14;
					ohs_error_text = strdup("too many newlines");
					status.textmode = OH_error;
				}
				break;
			
			case '_':
				if(atoi(c_seq.argument1) < buflen)
				{
					for(i=0;i<atoi(c_seq.argument1);i++){
						buffer[i]=' ';
					}
					buffer[i]='\0';
					status.textmode	= OH_normal;
					status.textlen	= strlen(buffer);
					status.eohelp	= OH_more_text;
					status.indent	= indent;
					status.reference= reference;
				} else {
					ohs_error_val = -14;
					ohs_error_text = strdup("too many spaces");
					status.textmode = OH_error;
				}
				break;
			
			case 'I':
				buffer[0]='\0';
				status.textmode	= OH_normal;
				status.eohelp	= OH_more_text;
				status.textlen	= 0;
				status.indent	= indent = atoi(c_seq.argument1);
				status.reference= reference;
				break;
	
			case 'N':
				aliaslist(c_seq.argument1,c_seq.argument1,OH_add);

				buffer[0]='\0';
				status.textmode	= OH_normal;
				status.textlen	= 0;
				status.eohelp	= OH_more_text;
				status.indent	= indent;
				status.reference= reference;
				break;
	
			case '#':
				buffer[0]='\0';
				status.textmode	= OH_normal;
				status.textlen	= 0;
				status.eohelp	= OH_more_text;
				status.indent	= indent;
				status.reference= reference;
				break;
	
			case '/':
				buffer[0]='/';
				buffer[1]='\0';
				status.textmode	= OH_normal;
				status.textlen	= 1;
				status.eohelp	= OH_more_text;
				status.indent	= indent;
				status.reference= reference;
				break;
	
			default:
				buffer[0]='\0';
				status.textmode	= OH_error;
				status.textlen	= 0;
				status.eohelp	= OH_eohelp;
				status.indent	= indent;
				status.reference= reference;
				
				sprintf(c_seq.argument1,"%c",c_seq.c_seq);
				ohs_error_text	= strdup( c_seq.argument1);
				ohs_error_val	= -10;
				break;
		} /* switch */
	} /* if */
	else
	{
		i = 0;
		dbgtext(60, "readhelp(): returning plain text\n");

		while(c!='/' && c!=EOF && i<buflen)
		{
			buffer[i++] = (c=='\n')?' ':c;

			c=fgetc(helpfile.fp);
		}

		buffer[i]='\0';		/* finish buffer */

		if(c=='/'){		/* oops, now we've already started the next controlsequence */
			fseek(helpfile.fp,-1,1);	/* hush, get back one char ! */
		}

		status.textmode		= OH_normal;
		status.textlen		= strlen(buffer);
		status.eohelp		= (c!=EOF) ? OH_more_text : OH_eohelp;
		status.indent		= indent;
		status.reference	= reference;
	}

	dbgtext(50, "readhelp_cont() fertig\n");
	return(status);
}

/************************************************************************
*                                                                       *
*       Funktion: readhelp()                                            *
*                                                                       *
*       Aufgabe: Liest jeweils ein Stueck Hilfetext ein                 *
*                                                                       *
*       Eingabe: Hilfe zu?, Returnpuffer, Puffergroesse, Modus(hl/keyw),*
*                Helpfile                                               *
*       Ausgabe: Struktur mit gelesenem Hilfetext u.a. Infos            *
*                                                                       *
************************************************************************/

struct OH_status readhelp(help, buffer, buflen, mode, helpfile)
char *help;
char *buffer;
int buflen;
int mode;
struct OH_file helpfile;
{
	struct OH_status	status;
	static int			helpmode;
	static char			helpword[1024];
	static int			indent;
	static int			outer_depth;

	dbginfo(50, "readhelp('%s', ", help);
	dbginfo(50, "mode %d, ", mode);
	dbginfo(50, "buflen %d )\n", buflen );

	if(mode==OH_init){		/* got to initialize everything */
		dbgtext(55, "readhelp(): mode=OH_init\n");

		helpword[0]		= '\0';
		helpmode		= -2;
		indent			= 0;
		last_offset		= 0L;
		reference		= 0;
		times_found		= 0;
		status.eohelp	= OH_more_text;
		status.textlen 	= 0;
		status.textmode	= OH_normal;
		status.hldepth	= -1;
		status.indent = 0;
	}
	else
	{

		strcpy(buffer,"");

		if(strcmp(help,helpword)!=0 /* help for new helpword wanted or */
		   || helpmode!=mode 		/* switch between keyw and hl */
		   || mode == OH_skip)		/* want help for specific reference */
		{
			dbgtext(55, "readhelp(): new keyword/mode or skip. Initialize readhelp.\n");
			strcpy(helpword, help);
			helpmode = (mode==OH_skip) ? OH_keyword : mode;
			indent = 0;

			outer_depth = readhelp_new(help, mode, buflen, helpfile);

			if(outer_depth==OH_error)
			{
				ohs_error_val = (mode == OH_keyword) ? -9 : -8;
				ohs_error_text = strdup(help);
				status.textmode = OH_error;
				dbgtext(10, "readhelp: couldn't position fileoffset for new read\n");
				return(status);
			}
			else
			{
				status = readhelp_cont(help, buffer, buflen, mode, helpfile,outer_depth);
			}
		}
		else /* return more helptext */
		{	
			dbgtext(55, "readhelp(): getting more text...\n");
			status = readhelp_cont(help, buffer, buflen, mode, helpfile, outer_depth);

			if(status.eohelp == OH_eohelp)
			{
				helpmode = 1-helpmode;	/* assure that we start anew next time */
			}
		}
	}
	
	status.reference = reference;

	dbginfo(52, "readhelp() buffer = '%s'\n", buffer);
	dbginfo(52, "readhelp() status.textmode = %d\n", status.textmode);
	dbginfo(52, "readhelp() status.textlen  = %d\n", status.textlen);

	dbgtext(50, "readhelp() fertig\n");
	return(status);
}
