/*

                                         P E P S Y
                         
        Performance and Evaluation Prediction System

        Copyright 1987, 1988

                 Universitaet Erlangen-Nuernberg,
                 Lehrstuhl fuer Betriebssysteme
                 Martensstrasse 1

             8520 Erlangen

        Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
        PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
        gefuehrt werden.
*/
#define LO_TOC_C
static char *SccsId="@(#)PEPSY:\ttoc.c\t1.4\t94/02/10 17:14:43";

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "libohs.h"
#include "d_debug.h"

extern int atoi();
extern char* malloc();
extern int dbg;


/************************************************************************
*                                                                       *
*       Funktion: sorthl()                                              *
*                                                                       *
*       Aufgabe: Sortiert die Ueberschriftenliste mit Hierarchieebenen  *
*                                                                       *
*       Eingabe: Ueberschriftenliste, Hierarchieliste, Anz. d. Elemente *
*                                                                       *
*       Ausgabe: keinen                                                 *
*                                                                       *
************************************************************************/

void sorthl(value, ivalue, number)
char **value;
int *ivalue;
int number;
{
	int gap,i,j;
	char *a;
	int b;

	dbgtext(50, "sorthl()\n");

	for(i=0; i<number; i++)
	{
		while(isspace(value[i][0]))
			value[i]++;
	}

	for(gap=number/2 ; gap>0 ; gap/=2)
		for(i=gap ; i<number ; i++)
			for(j=i-gap ; j>=0 ; j-=gap) {
				if(strcasecmp(value[j],value[j+gap])<=0)
					break;
				a=value[j+gap];
				value[j+gap]=value[j];
				value[j]=a;

				b=ivalue[j+gap];
				ivalue[j+gap]=ivalue[j];
				ivalue[j]=b;
			}
	dbgtext(50, "sorthl() fertig.\n");
}

/************************************************************************
*                                                                       *
*       Funktion: gettoc()                                              *
*                                                                       *
*       Aufgabe: Liefert das Inhaltsverzeichnis                         *
*                                                                       *
*       Eingabe: Filtermaske, Returnpuffer, Hierarchiereturnpuffer,     *
*                Helpfile, sortieren?                                   *
*       Ausgabe: Anzahl der zurueckgegebenen Eintraege                  *
*                                                                       *
************************************************************************/

int gettoc(mask, buffer, depth, file, sort)
char *mask;
char **buffer;
int *depth;
struct OH_file file;
int sort;
{
    char		line[1024];
    long		l;
    long		len;
    int			i=0;
    int			j=0;
    int			k=0;
    int			m=0;
    struct OH_control	seq;
    int			maskmode;
    char 		*mask_up, *string_up;

    dbgtext(50, "gettoc()\n");

    mask_up = strdup(mask);

    for(i=0;i<(int)strlen(mask_up);i++)
    {
	if(isascii(mask_up[i]))
	    mask_up[i]=dtoupper(mask_up[i]);
    }

    if(isdigit(mask_up[0])==FALSE)
	maskmode=0;				/* mask alphabetical */
    else
	maskmode=1;				/* mask hldepth */

    l=ftell(file.fp);				/* save fileposition */
    fseek(file.fp,file.start_of_hl,0);		/* jump to Headlines */

    for(i=0;i<file.hl_count;i++){		/* fill buffer with hl*/
	seq=get_c_seq(file.fp);			/* and corresponding */
						/* headline-depth */
	for(k=0;k<(int)strlen(mask);k++){

	    string_up = strdup(seq.argument1);
	    for(m=0;m<(int)strlen(string_up);m++)
	    {
		if(isascii(string_up[m]))
		    string_up[m]=dtoupper(string_up[m]);
	    }

	    if( (maskmode==0 && strstr(string_up, mask_up)!=NULL)	/* string contained */ 
		|| (maskmode==1 && mask[k]==seq.argument2[0])		/* hldepth found */
		|| (strcmp(mask,"*")==0) )				/* show all */
	    {
		depth[j]=atoi(seq.argument2);

		len = strlen(seq.argument1) + INDENTMULT*depth[j] + 1;
		buffer[j]=(char *)malloc(len*sizeof(char));
		if (buffer[j])
		{
		    len=depth[j]*INDENTMULT;
		    buffer[j][len]='\0';
		    while(len) buffer[j][--len]=' ';
		}
		strcat(buffer[j], seq.argument1);


		dbginfo(55, "gettoc(): HL='%s'\n", buffer[j]);

		j++;
		break;
	    }
	    free(string_up);
	}

	buffer[j]=NULL;
	depth[j]=-1;

	get_until(file.fp,line,"/");
    }

    free(mask_up);

    fseek(file.fp,l,0);				/* restore filepos. */

    /* let's sort buffer if they want us to */
    if(sort==TRUE) sorthl(buffer,depth,j);

    dbgtext(50, "gettoc() fertig\n");

    return(j);
}
