/*
	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.


	database.c

	Modul mit Funktionen zum Lesen der PEPSY-QNS Datenbasis

	Matthias Kirschnick	03.06.92
*/
#define DATABASE_C

static char *SccsId = "@(#)PEPSY:\tdatabase.c\t1.12\t95/05/22 10:48:51";

#include <stdio.h>
#include <string.h>

#include "d_debug.h"
#include "database.h"
#include "libpepsy.h"

#define BUFFERLEN		5120
#define FATAL			-1
#define OK				0

		
extern	char *getenv();

extern	int	dbg;

static	pdb_entry_t	*akt_entry;
static	pdb_entry_t	*Database;
static	char		*PEPSYHOME;
static	int		DatabaseCount=0;



void	ResetDBEntry()
{
	akt_entry = (pdb_entry_t *)NULL;
}


pdb_entry_t *NextDBEntry()
{
    if (akt_entry == (pdb_entry_t *)NULL)
    {
	akt_entry   = Database;
    }
    else
	akt_entry   = akt_entry->next;


    dbginfo(55, "NextDBEntry(): Found '%s'\n", (akt_entry)?akt_entry->verfahren:"Kein Verfahren");

    return(akt_entry);
}


pdb_entry_t *GetDBEntry(verfahren)
char	*verfahren;
{
    static	pdb_entry_t	*search_entry;
		pdb_entry_t	*found_entry;
		int		len;


    found_entry		= (pdb_entry_t *)NULL;
    search_entry	= Database;
    len			= strlen(verfahren);

    while (search_entry != (pdb_entry_t *)NULL)
    {
	if (!strncmp(search_entry->verfahren, verfahren, len))
	{
	    found_entry		= search_entry;
	    search_entry	= (pdb_entry_t *)NULL;
	}
	else
	    search_entry	= search_entry->next;
    }

    if (search_entry)
    {
	dbgtext(50, "GetDBEntry()\n");
	dbginfo(60, "DataBaseEntry Dump: Verfahren %s\n", found_entry->verfahren);
	dbginfo(60, "                    Codierung %s\n", found_entry->codierung);
	dbginfo(60, "                    verfFlags %s\n", found_entry->verfflags);
	dbginfo(60, "                    Prefilter %s\n", found_entry->prefilter);
	dbginfo(60, "                    PrefFlags %s\n", found_entry->prefflags);
	dbginfo(60, "                    Postfiler %s\n", found_entry->postfilter);
	dbginfo(60, "                    Postfflgs %s\n", found_entry->postfflags);
	dbginfo(60, "                    Netztyp   %o\n", found_entry->netztyp);
	dbginfo(60, "                    MaxKnoten %d\n", found_entry->maxknoten);
	dbginfo(60, "                    MaxKlassn %d\n", found_entry->maxklassen);
	dbginfo(60, "                    Knotentyp 0%08o\n", found_entry->knotentypen);
	dbginfo(60, "                    UEW       %d\n", found_entry->uew);
	dbginfo(60, "                    Besh      %d\n", found_entry->besh);
	dbginfo(60, "                    ZusBlock  %s\n", found_entry->zusatzblock);
	dbginfo(60, "                    HomBedRa  %d\n", found_entry->homogene_bedraten);
	dbginfo(60, "                    minCOV    %f\n", found_entry->minCOV);
	dbginfo(60, "                    maxCOV    %f\n", found_entry->maxCOV);
    }

    return(found_entry);
}

int CountPepsyDB()
{
    return(DatabaseCount);
}

static int pdb_entry_cmp(a, b)
pdb_entry_t **a, **b;
{
    return(strcmp((*a)->verfahren, (*b)->verfahren));
}



void	 GetPepsyDB()
{
    /*	
	    Momentan werden nur die Eintraege in code.txt ausgewertet. Fuer eine vereinfachte
	    Verwaltung sollen in Zukunft die Verwaltungsdateien code.txt und grenzen[all].txt
	    zusammengefasst werden. Dann koennen durch eine Erweiterung des struct pdb_entry
	    alle Informationen gehalten werden.
    */

    char	CodeDatei[BUFFERLEN];
    char	GrenzenDatei[BUFFERLEN];
    char	buffer[BUFFERLEN];
    pdb_entry_t	*pdb_entry;
    pdb_entry_t	*last_entry;
    FILE	*fp;
    int		netztyp,
		knoten,
		klassen,
		knotentypen,
		parBE;
    double	minCOV,
		maxCOV;
    char	verfahren[64];
    char	uew[64];
    char	besh[64];
    char	zusatzblock[64];
    char	bedraten[64];


    if((PEPSYHOME = (char *)getenv("PEPSYHOME")) == (char *)NULL)
    {
	fprintf(stderr, "GetPepsyDB(): Environment variable PEPSYHOME undefined!\n");
	exit(FATAL);
    }

    if ((Database) && (dbg))
    {
	fprintf(stderr, "GetPepsyDB(): WARNING - Database read twice!\n");
    }
    else
    {
	strcpy(CodeDatei,PEPSYHOME);
	strcat(CodeDatei,"/infos/code.txt");

	if ((fp=(FILE*)fopen(CodeDatei, "r")) != (FILE *)NULL)
	{
	    dbginfo(55, "Building database (using %s)\n", CodeDatei);

	    last_entry	= 
	    Database	= (pdb_entry_t *)NULL;
	    
	    while(nextline(fp, buffer, BUFFERLEN) != EOF)
	    {
		if ((pdb_entry = (pdb_entry_t*)malloc(sizeof(pdb_entry_t))) != (pdb_entry_t*)NULL)
		{
		    if (last_entry)
			last_entry->next	= pdb_entry;
		    else
			Database		= pdb_entry;

		    DatabaseCount++;

		    pdb_entry->next			= (pdb_entry_t *)NULL;

		    pdb_entry->verfahren	= strdup(strtok(buffer,       "\t "));
		    pdb_entry->codierung	= strdup(strtok((char *)NULL, "\t "));
		    pdb_entry->verfflags	= strdup(strtok((char *)NULL, "\t "));
		    pdb_entry->prefilter	= strdup(strtok((char *)NULL, "\t "));
		    pdb_entry->prefflags	= strdup(strtok((char *)NULL, "\t "));
		    pdb_entry->postfilter	= strdup(strtok((char *)NULL, "\t "));
		    pdb_entry->postfflags	= strdup(strtok((char *)NULL, "\t "));

		    last_entry = pdb_entry;

		    dbginfo(55, "Found entry %s\n", pdb_entry->verfahren);
		}
		else
		{
		    fprintf(stderr, "GetPepsyDB(): Out of memory!\n");
		    exit(FATAL);
		}
	    }
	}
	else
	{
	    fprintf(stderr, "GetPepsyDB(): Cannot open Database (%s)!\n", CodeDatei);
	    exit(FATAL);
	}

	strcpy(GrenzenDatei,PEPSYHOME);
	strcat(GrenzenDatei,"/infos/grenzenall.txt");

	if ((fp=(FILE*)fopen(GrenzenDatei, "r")) != (FILE *)NULL)
	{
	    while(nextline(fp, buffer, BUFFERLEN) != EOF)
	    {
		sscanf(buffer, "%s %o %d %d %o %1s %1s %s %s %lf,%lf %d",
				verfahren,
				&netztyp,
				&knoten,
				&klassen,
				&knotentypen,
				uew,
				besh,
				zusatzblock,
				bedraten,
				&minCOV,
				&maxCOV,
				&parBE
				);

		pdb_entry = Database;
		
		while(pdb_entry != (pdb_entry_t*) NULL)
		{
		    if(!strcmp(pdb_entry->verfahren, verfahren))
		    {
			pdb_entry->netztyp 			= netztyp;
			pdb_entry->maxknoten			= knoten;
			pdb_entry->maxklassen			= klassen;
			pdb_entry->knotentypen			= knotentypen;
			pdb_entry->uew				= (uew[0] == '+')?1:0;
			pdb_entry->besh				= (besh[0] == '+')?1:0;
			pdb_entry->zusatzblock			= strdup(zusatzblock);
			pdb_entry->minCOV			= minCOV;
			pdb_entry->maxCOV			= maxCOV;
			pdb_entry->homogene_bedraten	= (strncmp(bedraten, "MMm-->gleich", 12)
												?0:1);
			pdb_entry->parBE			= parBE;

			pdb_entry = (pdb_entry_t*) NULL;
		    }
		    else
		    {
			pdb_entry = pdb_entry->next;
		    }
		}
	    }
	}
	else
	{
		fprintf(stderr, "GetPepsyDB(): Cannot open Database (%s)!\n", CodeDatei);
		exit(FATAL);
	}


	strcpy(GrenzenDatei,PEPSYHOME);
	strcat(GrenzenDatei,"/infos/grenzen.txt");

	if ((fp=(FILE*)fopen(GrenzenDatei, "r")) != (FILE *)NULL)
	{
	    while(nextline(fp, buffer, BUFFERLEN) != EOF)
	    {
		sscanf(buffer, "%s ", verfahren);

		pdb_entry = Database;
		
		while(pdb_entry != (pdb_entry_t*) NULL)
		{
		    if(!strcmp(pdb_entry->verfahren, verfahren))
		    {
			pdb_entry->internal_flags	= PDBF_STD;
			pdb_entry			= (pdb_entry_t*) NULL;
		    }
		    else
		    {
			pdb_entry			= pdb_entry->next;
		    }
		}
	    }
	}
	else
	{
	    fprintf(stderr, "GetPepsyDB(): Cannot open Database (%s)!\n", CodeDatei);
	    exit(FATAL);
	}

	pdb_entry = Database;

	while(pdb_entry != (pdb_entry_t*) NULL)
	{
	    dbginfo(60, "DataBaseEntry Dump: Verfahren %s\n", pdb_entry->verfahren);
	    dbginfo(60, "                    Codierung %s\n", pdb_entry->codierung);
	    dbginfo(60, "                    verfFlags %s\n", pdb_entry->verfflags);
	    dbginfo(60, "                    Prefilter %s\n", pdb_entry->prefilter);
	    dbginfo(60, "                    PrefFlags %s\n", pdb_entry->prefflags);
	    dbginfo(60, "                    Postfiler %s\n", pdb_entry->postfilter);
	    dbginfo(60, "                    Postfflgs %s\n", pdb_entry->postfflags);
	    dbginfo(60, "                    Netztyp   %d\n", pdb_entry->netztyp);
	    dbginfo(60, "                    MaxKnoten %d\n", pdb_entry->maxknoten);
	    dbginfo(60, "                    MaxKlassn %d\n", pdb_entry->maxklassen);
	    dbginfo(60, "                    Knotentyp 0%08o\n", pdb_entry->knotentypen);
	    dbginfo(60, "                    UEW       %d\n", pdb_entry->uew);
	    dbginfo(60, "                    Besh      %d\n", pdb_entry->besh);
	    dbginfo(60, "                    ZusBlock  %s\n", pdb_entry->zusatzblock);
	    dbginfo(60, "                    HomBedRa  %d\n", pdb_entry->homogene_bedraten);
	    dbginfo(60, "                    minCOV    %f\n", pdb_entry->minCOV);
	    dbginfo(60, "                    maxCOV    %f\n", pdb_entry->maxCOV);

	    pdb_entry = pdb_entry->next;
	}

	dbgtext(50, "Database build\n");
    }

    return;
}

void	 GetPepsyDBSorted()
{
    int		count;
    int		n;
    pdb_entry_t	**array;
    pdb_entry_t	*pdb_entry;


    GetPepsyDB();

    dbgtext(50, "GetPepsyDBSorted()\n");

    count = CountPepsyDB();

    dbginfo(75, "GetPepsyDBSorted(): Count=%d\n", count);

    array	= (pdb_entry_t **)calloc(count, sizeof(pdb_entry_t *));
    n		= 0;
    pdb_entry	= Database;

    while(pdb_entry && n<count)
    {
	array[n]	= pdb_entry;
	pdb_entry	= pdb_entry->next;
	n++;
    }
    dbgtext(55, "GetPepsyDBSorted(): DatabaseArray build!\n");

    qsort(array, CountPepsyDB(), sizeof(pdb_entry_t *), pdb_entry_cmp);

    dbgtext(55, "GetPepsyDBSorted(): DatabaseArray sorted!\n");

    Database	= array[0];
    pdb_entry	= Database;
    n		= 1;
    count	= CountPepsyDB();

    dbginfo(75, "GetPepsyDBSorted(): Count=%d\n", count);

    while(n < count)
    {
	pdb_entry->next = array[n];
	pdb_entry	= pdb_entry->next;
	n++;
    }

    pdb_entry->next = (pdb_entry_t *)NULL;

    dbgtext(55, "GetPepsyDBSorted(): Database rebuild\n");

    free(array);

    dbgtext(51, "GetPepsyDBSorted() fertig\n");
}
