/*
	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  D-91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/
#define GET_STANDARD_C

static char *SccsId = "%Z%PEPSY:\t%M%\t%R%.%L%\t%D% %T%";

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#include "d_debug.h"
#include "d_speicher.h"
#include "d_netz.h"
#define	EXTERN_KNOTENTYPEN
#include "d_konst.h"
#include "libpepsy.h"

#define	LPT_FATAL_OUT_OF_MEM_1		spr("ldmem1")
#define	LPT_FATAL_OUT_OF_MEM_2		spr("ldmem2")
#define	LPT_FATAL_NUM_NODES		spr("ldrdnn")
#define	LPT_FATAL_NUM_CLASSES		spr("ldrdjc")
#define	LPT_ERROR_NUMBER_SERVERS	spr("ldrdnm")
#define	LPT_ERROR_EXPECT_DEL_NODETYPE	spr("ldetyp")
#define	LPT_ERROR_EXPECT_DEL_NAME	spr("ldenum")
#define	LPT_ERROR_EXPECT_DEL_NODENAME	spr("ldenam")
#define	LPT_ERROR_EXPECT_2_PARAM	spr("ldepar")
#define	LPT_ERROR_JOBS_ARRRATE		spr("ldnjra")
#define	LPT_ERROR_READING_JOBS_ARRRATE	spr("ldnjar")
#define	LPT_ERROR_EXPECT_RATE_SCOV_E	spr("ldscve")
#define	LPT_ERROR_EXPECT_RATE_SCOV	spr("ldsrcv")

extern	netzstruktur	netz;
extern	int		dbg;

static	size_t		get_standard_message_mem = 0;
static	int		get_standard_p_lang_init = 0;

#define	NEXTLINE(_a, _b, _c)	if (Nextline(_a, _b, _c) == EOF) {\
								      sprintf(_b, "Line %d : FATAL: Unexpected End of File!", get_standard_line); \
								      cat_message(_b);\
								      return 1;\
							      }

static int Nextline(fp, buffer, size)
FILE *fp;
char *buffer;
int size;
{
    int r;

    do 
    {
	r = getline(fp, buffer, size);
	get_standard_line++;
    } while((*buffer == '\n' || *buffer == '#') && r!=EOF);

    return(r);
}
    
static void cat_message(s)
char *s;
{
    size_t	n;

    dbginfo(99, "cat_message('%s')\n", s);

    if (s)
    {
	n = (size_t)(1+strlen(s)+get_standard_message_mem+((get_standard_message_mem)?0:1));
	get_standard_message = (char *)realloc(get_standard_message, n);

	if (get_standard_message)
	{
	    if (!get_standard_message_mem)
		*get_standard_message = '\0';
	    get_standard_message_mem	= n;
	    
	    strcat(get_standard_message, s);
	    strcat(get_standard_message, "\n");
	}
/*	dbginfo(100, "cat_message(): get_standard_message='%s'\n", get_standard_message); */
    }
}

static void clear_message()
{
    free(get_standard_message);
    get_standard_message_mem	= 0;
    get_standard_message	= NULL;
}

int get_standard(fpin)
FILE *fpin;
{
    register netzstruktur *np = &netz;
    char	buffer[5120];
    char	*c, *cp;
    char	string[50], string1[50];
    int		i,j,k;
    int		retval = 0;

    dbgtext(50, "get_standard()\n");

    get_standard_line = 0;
    clear_message();

    if(!get_standard_p_lang_init)
    {
	if(p_lang_init("ld_ein") != 0)
	{
	    fprintf(stderr, "get_standard(): Cannot perform p_lang_init in PEPSY-QNS:/libpepsy.a:get_standard()!\n");
	    exit(-1);
	}
	
	get_standard_p_lang_init = 1;
    }

    /*	Die Anzahl der Knoten bzw. Klassen ist jeweils die erste Zahl nach dem Doppelpunkt
	
	1. Anzahl Knoten einlesen
	*/

    NEXTLINE(fpin, buffer, 5120);
    cp = strchr(buffer, ':');

    if (cp)
	np->knot = atoi(++cp);
    else
    {
	sprintf(buffer, LPT_FATAL_NUM_NODES, get_standard_line); 
	cat_message(buffer);
	dbgtext(51, "get_standard(): done. Returns FATAL\n");
	return(-1);
    }


    /*	2. Anzahl Klassen einlesen
     */

    NEXTLINE(fpin,buffer,5120);
    cp = strchr(buffer, ':');

    if (cp)
	np->klass = atoi(++cp);
    else
    {
	sprintf(buffer, LPT_FATAL_NUM_CLASSES, get_standard_line); 
	cat_message(buffer);
	dbgtext(51, "get_standard(): done. Returns FATAL\n");
	return(-1);
    }

    dbginfo2(55, "get_standard(): Knoten: %d Klassen: %d\n", np->knot, np->klass);



    if (np->knot == 0 || np->klass == 0)
	return (-1);


    /* Speicherplatz reservieren */

    if (sp_alloc() == 0)	/* aus libpepsy.a:speicher.c */
/*	||	(np->teilnr = i_1dim_alloc(np->knot)) == 0 ) ??? XXX */
    {
	cat_message(LPT_FATAL_OUT_OF_MEM_1);
	dbgtext(51, "get_standard(): done. Returns FATAL\n");
	return (-1);
    }

    /* Knotenbeschreibung lesen */

    NEXTLINE(fpin,buffer,5120);		/* ueberlesen von "KNOTENBESCHREIBUNG..."	*/
    NEXTLINE(fpin,buffer,5120);		/* ueberlesen von "  Knoten |   Name ..."	*/
    NEXTLINE(fpin,buffer,5120);		/* ueberlesen von "------------------..."	*/

    for (i=0;i<np->knot;i++)
    {
	NEXTLINE(fpin,buffer,5120);

	/*	a.	der name des Knotens
	 */

	cp = strchr(buffer, '|');

	if (cp)
	{
	    cp++;		/* skip '|'	`*/

	    dbginfo(75, "get_standard(): begin of nodename+space - '%s'\n", cp);

	    while(*cp && isspace(*cp))
		cp++;

	    dbginfo(75, "get_standard(): begin of nodename/space - '%s'\n", cp);

	    c	= np->knotname[i];
	    j	= 0;
	    while(*cp && *cp!='|' && j<NAM_LAENGE-1)
	    {
		j++;
		*c++ = *cp++;
	    }
	    *c-- = '\0';

	    dbginfo(75, "get_standard(): nodename+space='%s'\n", np->knotname[i]);
	
	    while (c>=np->knotname[i] && isspace(*c))
		*c-- = '\0';

	    dbginfo(75, "get_standard(): nodename/space='%s'\n", np->knotname[i]);


	    dbgtext(55, "get_standard(): calling get_knottyp\n");
	    
	    cp = strchr(cp, '|');
	    
	    if (cp)
	    {
		cp++;
		
		while(*cp && isspace(*cp))
		    cp++;

		np->knotentyp[i] = get_knottyp(cp);

		/* Voraussetzung: alle Knotentypnamen sind von der Form ././m-....
		 */
		if (np->knotentyp[i] != 0)
		{
		    if (sscanf(cp+4,"%d",&np->bede[i]) != 1)
		    {
			sprintf(buffer, LPT_ERROR_NUMBER_SERVERS, get_standard_line, np->knotname[i]);
			cat_message(buffer);
			retval++;
		    }
		}
		else
		{
		    np->bede[i]=0;
		}

		dbginfo4(52,
			 "get_standard(): %d, %s, bede:%d typ: %d\n ", 
			 i+1, np->knotname[i], np->bede[i], np->knotentyp[i]);
	    }
	    else
	    {
		sprintf(buffer,	LPT_ERROR_EXPECT_DEL_NODETYPE, get_standard_line);
		cat_message(buffer);
		retval++;
	    }
	}
	else
	{
		sprintf(buffer, LPT_ERROR_EXPECT_DEL_NAME, get_standard_line);
		cat_message(buffer);
		retval++;
	}
    }



    /* Netztyp bestimmen und Anzahl Auftraege einlesen    */

    np->typ = 0;

    NEXTLINE(fpin,buffer,5120);	/* ueberlesen von "KLASSENBESCHREIBUNG"	*/
    NEXTLINE(fpin,buffer,5120);	/* ueberlesen von "  Klasse  | Anku..." */
    NEXTLINE(fpin,buffer,5120); /* ueberlesen von "----------------..."	*/

    for (i=0;i<np->klass;i++)
    {
	NEXTLINE(fpin,buffer,5120);

	/*	skip to next |
	 */
	cp = strchr(buffer, '|');

	if (cp)
	{
	    cp++;

	    if (sscanf(cp, "%s %s", string, string1) != 2)
	    {
		sprintf(buffer, LPT_ERROR_EXPECT_2_PARAM, get_standard_line);
		cat_message(buffer);
		retval++;
	    }
	    else
	    {
		dbginfo2(60, "get_standard(): scanned string='%s' string1='%s'\n", string, string1);

		if(string[0] == '-') 			/* ankr is not given				*/
		{
		    np->typ		|= GESCHLOSSEN;	/* this job class is closed			*/
		    np->auftr[i]	=  atoi(string1); /* the number of jobs is the 2nd parameter	*/

		    dbginfo2(55, "get_standard(): auftr[class%d]: %d\n", i+1, np->auftr[i]);
		}
		else if(string1[0] == '-' )		/* the 2nd parameter is not given		*/
		{
		    np->typ	|= OFFEN;		/* this class is open				*/
		    np->ankr[i]	=  atof(string);	/* the arrival rate is given in the 1st param	*/

		    dbginfo2(55, "get_standard(): ankr[class%d]: %g\n", i+1, np->ankr[i]);
		}
		else
		{
		    sprintf(buffer, 
			    LPT_ERROR_JOBS_ARRRATE,
			    get_standard_line);
		    cat_message(buffer);
		    retval++;
		}
	    }
	}
	else
	{
	    sprintf(buffer, LPT_ERROR_READING_JOBS_ARRRATE,
		    get_standard_line, i+1);
	    cat_message(buffer);
	    retval++;
	}
    }


    if (((np->typ & OFFEN) == OFFEN) && ((np->typ & GESCHLOSSEN) == GESCHLOSSEN))
    {
	np->typ &= ~OFFEN;
	np->typ &= ~GESCHLOSSEN;
	np->typ |= GEMISCHT;
    }

    dbginfo(55, "get_standard():	erkannter Netztyp: %o\n", np->typ);




    /* Klassenspezifische Knotengroessen einlesen */

    NEXTLINE(fpin,buffer,5120);		/* ueberlesen von "KLASSENSPEZIFISCHE KNOTENGR..."	*/

    for (i=0;i<np->klass;i++)
    {
	dbginfo(55, "get_standard(): Klasse %d\n",i+1);

	NEXTLINE(fpin,buffer,5120);	/* ueberlesen von "KLASSE xxx				*/
	NEXTLINE(fpin,buffer,5120);	/*      lesen von "   Knoten     | Bedi..."		*/	

	if (i == 0)
	{
	    /* Testen, ob Besuchshaeufigkeiten angegeben worden sind anhande der Zahl der Worte	
	     */
	    if ((j=sscanf(buffer, " %*s %*1s %*s %*s %s", string)) == 1)
		np->haeuf = FALSE;
	    else
		np->haeuf = TRUE;

	    dbginfo2(55, "get_standard(): sscanf() returns %d, uewflag=%s\n", j,
		    (np->haeuf == TRUE)?"TRUE":"FALSE");
	}

	NEXTLINE(fpin,buffer,5120);	/* ueberlesen von "-----------------------"		*/

	for (j=0;j<np->knot;j++)
	{
	    NEXTLINE(fpin,buffer,5120);
	   
	    cp=strchr(buffer, '|');
	    
	    if (cp)
	    {
		cp++;

		if ((k=sscanf(cp,(np->haeuf==TRUE)?"%lf %lf":"%lf %lf %lf",
		       &np->bedr[j][i],
		       &np->quadvar[j][i],
		       &np->besh[j][i])) == ((np->haeuf==TRUE)?2:3))
		{
		    if (np->quadvar[j][i] == -1.0 )
			np->quadvar[j][i] = 1.0;

		    dbginfo2(55, "get_standard(): Klasse %d, Knoten %d: ", i+1, j+1);

		    if (np->haeuf==TRUE)
		    {
			dbginfo2(55, "bedr=%f, quadvar=%f\n",
				 np->bedr[j][i], np->quadvar[j][i]);
		    }
		    else
		    {
			dbginfo3(55, "bedr=%f, quadvar=%f, besh=%f\n",
				 np->bedr[j][i], np->quadvar[j][i], np->besh[j][i]);
		    }
		}
		else
		{
		    if (np->haeuf==FALSE)
			sprintf(buffer,
				LPT_ERROR_EXPECT_RATE_SCOV_E,
				get_standard_line);
		    else
			sprintf(buffer,
				LPT_ERROR_EXPECT_RATE_SCOV,
				get_standard_line);

		    cat_message(buffer);
		    retval++;
		}
	    }
	    else
	    {
		sprintf(buffer, LPT_ERROR_EXPECT_DEL_NODENAME,
			get_standard_line);
		cat_message(buffer);
		retval++;
	    }
	}

	if (np->haeuf == TRUE)
	{
	    /* Speicherplatz reservieren */
	    if (i == 0)
	    {
		if((np->uew = f_3dim_alloc(np->knot+1,np->knot+1,np->klass)) == (float ***)0 )
		{
		    cat_message(LPT_FATAL_OUT_OF_MEM_2);
		    return 1;
		}
	    }

	    /* Uebergangswahrscheinlichkeiten lesen */

	    NEXTLINE(fpin,buffer,5120);	/* ueberlesen von "UEBERGANGSWAHR..."	*/
	    NEXTLINE(fpin,buffer,5120);	/* ueberlesen von "von/zu   | aussen"	*/
	    NEXTLINE(fpin,buffer,5120);	/* ueberlesen von "-----------------"	*/

	    for (j=0;j<=np->knot;j++)
	    {
		NEXTLINE(fpin,buffer,5120);

		cp = strchr(buffer, '|');

		if (cp)
		{
		    cp++;

		    for(k=0;k<=np->knot;k++)
		    {
			/* blanks und tabs ueberlesen */
			while( *cp != '\0' && (*cp == ' ' || *cp == '\t'))
			    cp++;

			/* float einlesen */
			if(*cp == '-')
			{
			    np->uew[j][k][i] = 0.0;
			    cp++;
			}
			else
			    np->uew[j][k][i] = atof(cp);

			/* float ueberlesen */
			while( *cp != '\0' && ( isdigit(*cp) || *cp == '.' ))
			    cp++;

			dbginfo4(55, "get_standard(): uew[%d][%d][%d]=%f\n", j, k, i, np->uew[j][k][i]);
		    }
		}
		else
		{
		    sprintf(buffer, LPT_ERROR_EXPECT_DEL_NODENAME, get_standard_line);
		    cat_message(buffer);
		    retval++;
		}
	    }
	}
    }

    dbgtext(50, "get_standard() done\n");

    return(retval);
}

int knotentyp(string)
char *string;
{
	if( strncmp("FCFS",string,4) == 0 )
		return(1);
	if( strncmp("PS",string,2) == 0 )
		return(2);
	if( strncmp("NQ",string,2) == 0 )
		return(3);
	if( strncmp("LCFS-PRE",string,8) == 0 )
		return(4);
	return(0);
}
