/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/
#define PARSE_C


static char *SccsId = "@(#)PEPSY:\tparse.c\t1.8\t94/02/04 14:28:06";



#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <signal.h>
#include "d_debug.h"
#include "libpepsy.h"

/* extern double pow(); ist schon in math.h	*/

extern	int dbg;

static int level = 0;
static int i;
static int error;
static char *cp;


#define next()	while(*cp == ' ' || *cp == '\t' ) \
					cp++;

#define P(string)  if (dbg>50) { \
					for(i=0;i<level;i++) \
						fprintf(stderr," "); \
					fprintf(stderr,string); \
					}

/*
	
	Parser zur Erkennung einfacher arithmetischer Ausdruecke,
	nuetzlich vor allem bei Umrechnung Bedienzeiten in Bedienraten

	Es wird erkannt:

		a * b, a + b, a - b, a / b, a ^ b (Potenz)

		mit a,b Zahl oder geklammerter Ausdruck, z.B. (c + d)

	Rueckgabewert:  0    alles ok
				   -1    Fehler beim Auswerten der Eingabe
*/

void floatexcep()
{
	void floatexcep();

	signal(SIGFPE,floatexcep);
	error = 1;
}

int parse(string,result)
char *string;
double *result;
{
	void floatexcep();
	double expression();
	error = 0;
	cp = string;

	signal(SIGFPE,floatexcep);
	P("parse\n");
	next();
	*result = expression();

	P("parse\n");
	if(error != 0)
		return(-1);
	return(0);
}

double expression()
{
	double term();
	int ready = 0;
	double tempres = 0.0;

	level++;
	P("expression ");
	dbginfo(55, "%s", cp);
	tempres = term();

	while( !error && !ready )
	{
		switch(*cp)
		{

			case '+': cp++;
					  next();
					  P("+\n");
					  tempres += term();
					  break;
			case '-': cp++;
					  next();
					  P("-\n");
					  tempres -= term();
					  break;
			default:  next();
					  ready = 1;
					  break;
		}
	}

	P("expression ");
	dbginfo(55, "%g\n", tempres);
	level--;

	return(tempres);
}

double term()
{
	double factor();
	int ready = 0;
	double tempres = 0.0;

	level++;
	P("term  ");
	dbginfo(55, "%s",cp);

	tempres = factor();

	while( !error && !ready )
	{
		switch(*cp)
		{
			case '*' : cp++;
					   next();
					   P("*\n");
					   tempres *= factor();
					   break;
			case '/' : cp++;
					   next();
					   P("/\n");
					   tempres /= factor();
					   break;
			default  : next();
					   ready = 1;
					   break;
		}
	}

	P("term ");
	dbginfo(55, "%g\n",tempres);
	level--;
	return(tempres);
}

double factor()
{
	double primary(), number();
	int ready = 0;
	double tempres = 0.0;

	level++;
	P("factor  ");
	dbginfo(55, "%s",cp);

	tempres = primary();

	while( !error && !ready )
	{
		if(*cp != '^' )
		{
			ready = 1;
		}
		else
		{
			cp++;
			next();
			P("^\n");
			tempres = pow(tempres,primary());
		}
	}

	P("factor ");
	dbginfo(55, "%g\n",tempres);
	level--;
	return(tempres);
}

double number()
{
	double intpart = 0.0;
	double fractpart = 0.0;
	double expo = 0.0;
	double res;
	int factor = 10;
	int isign = 1;
	int esign = 1;

	level++;
	P("number  ");
	level++;
	dbginfo(55, "%s",cp);

	if(*cp == '+' || *cp == '-')
	{
		P("isign  ");
		dbginfo(55, "%s",cp);
		if(*cp == '-')
			isign = -1;
		cp++;
		P("isign  ");
		dbginfo(55, "%d\n",isign);
	}
	P("intpart  ");
	dbginfo(55, "%s",cp);
	while(isdigit(*cp))
	{
		intpart *= 10.0 ;
		intpart += (double) (*cp - '0');
		cp++;
	}
	P("intpart  ");
	dbginfo(55, "%g\n",intpart);
	if(*cp == '.')
	{
		cp++;
		P("fractpart  ");
		dbginfo(55, "%s",cp);
		while(isdigit(*cp))
		{
			fractpart += (double )( *cp - '0' ) / factor;
			factor *= 10;
			cp++;
		}
		P("fractpart ");
		dbginfo(55, "%g\n",fractpart);
	}
	res = isign * (intpart + fractpart );
	if(*cp == 'e' || *cp == 'E')
	{
		cp++;
		P("exponent  ");
		dbginfo(55, "%s",cp);
		if(*cp == '-' || *cp == '+' )
		{
			P("esign  ");
			dbginfo(55, "%g\n",fractpart);
			if(*cp == '-')
				esign = -1;
			cp++;
			P("esign  ");
			dbginfo(55, "%d\n",esign);
		}
		while(isdigit(*cp))
		{
			expo *= 10;
			expo += *cp - '0';
			cp++;
		}
		if(expo > 100)
		{
			error = 1;
			return(1.0);
		}
		expo = pow((double) 10,(double) expo);
		P("exponent  ");
		dbginfo(55, "%g\n",expo);
		if(esign > 0 )
			res *= expo;
		else
			res /= expo;
			
	}
	next();

	level--;
	P("number ");
	dbginfo(55, "%g\n",res);
	level--;
	return(res);
}

double primary()
{
	double tempres = 0.0;

	level++;
	P("primary  ");
	dbginfo(55, "%s",cp);
	if(*cp == '(' )
	{
		cp++;
		next();
		P("(\n");
		tempres = expression();
		P(")\n");
		if(*cp != ')' )
		{
			error = 1;
		}
		cp++;
		next();
	}
	else
		tempres = number();
	
	
	P("primary ");
	dbginfo(55, "%g\n",tempres);
	level--;
	return(tempres);
}
