/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988, 1994
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/


#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>


#ifdef sequent
#   define WEXITSTATUS(x) (x>>8)
#else
#   include <sys/wait.h>
#endif

#include "d_debug.h"
#include "d_netz.h"
#include "d_konst.h"
#include "database.h"
#include "libpepsy.h"

#define USAGE()	{ \
	fprintf(stderr,"%s\n%s\n%s\n%s\n",spr("usage1"),spr("usage2"),spr("usage3"),spr("usage4")); exit(1);}

/*
								
 Regelt fuer das im Aufruf angegebene Verfahren die Analyse
 Aus einer Eingabedatei wird durch die Analyse ein Ausgabedatei 
 mit Rohwerten und aus dieser durch den Postfilter eine global
 vergleichbare Ausgabedatei

 Zeitlich Abhaengigkeiten werden beruecksichtigt und somit
 die Antwortzeit minimiert
								
 Aufrufoptionen:						
	 verfahren :        Verfahren fuer das Zusatzeingabe erfolgen soll	
	 dateiname :        Einabedateiname				
	 analyseflags:      Aufrufoptionen fuer das Analyseprogramm
	 verfahrenoptionen: Aufrufoptionen fuer die eigentliche Analyse
	 verfahrenparamter: Paramter fuer die eigentliche Analyse, z.B.
						Grenzangaben

								
	Version 1.0

	Version 1.1	06.02.92	kirschni	Kosmetische Korrekturen
							Verbesserung der Anwendbarkeitsueberpruefung
							durch neue option -qv von auswahl(1)
	Version 1.2	06.02.92	kirschni	Mehrere Eingabedateien sind moeglich,
							ebenso die Angabe des Prefixes 'e_'
	Version 1.3	07.02.92	kirschni	Mehrere Verfahren koennen angegeben werden,
							die auf alle angegebenen Modelldateien ange-
							wendet werden. (Option -m)
	Sccs		03.03.92	kirschni	Sccs-Verwaltung eingefuehrt
*/

static char *SccsId = "@(#)PEPSY:\tanalyse.c\t1.17\t95/05/22 10:50:56";

static char *PEPSYHOME;

extern	char		*getenv();
extern	void		unlink();
extern	int		system();
extern	int		atoi();
extern	void		sleep();

int			dbg = 0;       	/* debug - niveau			*/

int scanline(fp, buf, maxlen)
FILE *fp;
char *buf;
int  maxlen;
{
    /* Read a line from the file until \n or EOF or buffer full
       return number of characters read (excluding the \n\0 sequence!) or EOF
       buffer contains \n\0 at end
       */

    int c;
    char *bp;
    int  len;


    dbginfo(55, "scanline(fp, buf, maxlen=%d)\n", maxlen);


    bp	= buf;
    len	= 0;

    c=getc(fp);

    while (c!='\n' && c!=EOF && len<=maxlen-2)
    {
	*bp=c;
	bp++;
	len++;

	c=getc(fp);
    }

    if (len<=maxlen-2)
    {
	*bp='\n';
	bp++;
	*bp='\0';
	bp++;
    }

    dbginfo(55, "scanline(): len=%d ", (c==EOF)?EOF:len);
    dbginfo(55, "line=%s", buf);

    return((c==EOF)?EOF:len);
}

void concat_logfile(logfilename, header)
char *logfilename;
char *header;
{
    FILE *lfp=NULL;
    FILE *fp=NULL;
    
    char line[1024];

    
    if (logfilename != (char *)0)
	if ((lfp=fopen(logfilename, "a+")) == (FILE *)NULL)
	{
	    fprintf(stderr, "analyse - ERROR. Cannot open logfile '%s'.\n", logfilename);
	}

    if (lfp)
	fprintf(lfp, "%s\n", header);

    /* append prefilterfile
     */
    if((fp=fopen("prefilter.err", "r")) != (FILE *)NULL)
    {
	while (scanline(fp, line, 1024) != EOF)
	{
	    fprintf(stderr, "prefilter - %s", line);

	    if (lfp)
		fprintf(lfp, "prefilter - %s", line);
	}
	    
	fclose(fp);
	unlink("prefilter.err");
    }

    /* append method error output
     */
    if((fp=fopen("method.err", "r")) != (FILE *)NULL)
    {
	while (scanline(fp, line, 1024) != EOF)
	{
	    fprintf(stderr, "method - %s", line);


	    if (lfp)
		fprintf(lfp, "method - %s", line);
	}

	fclose(fp);
	unlink("method.err");
    }

    /* append postfilter error output
     */
    if((fp=fopen("postfilter.err", "r")) != (FILE *)NULL)
    {
	while (scanline(fp, line, 1024) != EOF)
	{
	    fprintf(stderr, "postfilter - %s", line);


	    if (lfp)
		fprintf(lfp, "postfilter - %s", line);
	}

	fclose(fp);
	unlink("postfilter.err");
    }


    if (lfp)
	fclose(lfp);

}

int main(argc,argv)
int		argc;
char   *argv[];
{
    /*		Sccs support from SccsLogging.c	
     */	
    extern	int SccsRelease;
    extern	int SccsLevel;

    /*		fuer getopt(3C)			
     */
    extern	char	*optarg;
    extern	int	optind;

    FILE		*fpp       = NULL; /* Ausgabepipe */
    FILE		*fp        = NULL; /* a_datei die auf der pipe ausgegeben wird. */
    FILE		*pepout_fp = NULL;
    FILE		*out_fp    = NULL;
    static	char	num_parameter[255];
    static	char	opt_parameter[255];
    static	char	pref_parameter[255];
    static	char	postf_parameter[255];
    int			verbose		= FALSE;
    int			batch		= FALSE;
    int			force		= FALSE;
    int			watch_time	= FALSE;
    int			createlogfile	= FALSE;
    int			multimethod	= 1; /* Zahl der angegebenen Verfahren (option -m) */
    int			analyse_retvalue=0;
    int			index_der_ersten_e_datei;
    int			index_der_letzten_e_datei;
    int			index_des_ersten_verfahrens;
    int			index_des_letzten_verfahrens;
    int			e;	/* Laufzaehler fuer die Eingabedateien */
    int			v;	/* Laufzaehler fuer die Verfahren */
    int			c;
    int			retvalue;
    int			exitvalue;
    int			scanchars;
    struct stat 	e_buf;
    struct stat		a_buf;
    struct stat		v_buf;
    pdb_entry_t		*pdb_entry;
    pdb_entry_t		*pdbe;
    char		*verfahren;
    char	   	*datei;
    char		verf[256];
    char		buffer[5120];
    char		command[5120];
    char		e_datei[100];
    char		a_datei[100];
    char		l_datei[100];



    *num_parameter		= '\0'; 
    *opt_parameter		= '\0';
    *pref_parameter		= '\0';
    *postf_parameter		= '\0';

    if (p_lang_init(argv[0]) != 0)
	exit (-2);

    while(( c= getopt(argc,argv,"lm:Vfvbtd:N:O:P:p:")) != EOF)
    {
	switch(c)
	{
	case 'l':
	    createlogfile = TRUE;
	    break;
	case 'd':
	    if((dbg = atoi(optarg)) <= 0)
		dbg = 1;
	    break;
	case 't':
	    watch_time = TRUE;
	    break;
	case 'V':
	    fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
	    exit(0);
	    break;
	case 'v':
	    verbose = TRUE;
	    break;
	case 'f':
	    force = TRUE;
	    break;
	case 'b':
	    batch = TRUE;
	    break;
	case 'm':
	    multimethod = atoi(optarg);
	    break;
	case 'N':
	    (void) strcat(num_parameter, optarg);
	    (void) strcat(num_parameter, " ");
	    break;
	case 'O':
	    (void) strcat(opt_parameter, optarg);
	    (void) strcat(opt_parameter, " ");
	    break;
	case 'p':
	    (void) strcat(pref_parameter, optarg);
	    (void) strcat(pref_parameter, " ");
	    break;
	case 'P':
	    (void) strcat(postf_parameter, optarg);
	    (void) strcat(postf_parameter, " ");
	    break;
	default:
	    fprintf(stderr,"\n%s -%c !!!\n",spr("opterr"), c);
	    USAGE();
	    break;
	}
    }

    if((PEPSYHOME = (char *)getenv("PEPSYHOME")) == NULL)
    {
	fputs(spr("enverr"), stderr); fputc('\n', stderr);
	exit(1);
    }

    dbginfo(0, "analyse - Debugniveau %d\n", dbg);
    dbginfo(0, "analyse - Verfahrensoptionen  %s\n", opt_parameter);
    dbginfo(0, "analyse - Verfahrensparameter %s\n", num_parameter);

    GetPepsyDB();

    dbginfo(5, "analyse - (argc - optind) = %d\n", argc-optind);

    index_des_ersten_verfahrens		= optind;
    index_des_letzten_verfahrens	= optind+multimethod-1;
    index_der_ersten_e_datei		= optind+multimethod;
    index_der_letzten_e_datei		= argc-1;

    if ( (index_der_letzten_e_datei <= index_des_letzten_verfahrens)
	|| (multimethod <= 0))
    {
	fprintf(stderr, "%s\n", spr("moperr"));
	USAGE();
    }

    if (	index_der_letzten_e_datei - index_der_ersten_e_datei > 0
	||	index_des_letzten_verfahrens - index_des_ersten_verfahrens > 0)
    {
	/* Auto-batchmode falls mehrere Verfahren oder Eingabedateien angegeben werden */
	batch=TRUE; 
    }

    if (!batch)
    {
	if ((fpp = popen("more", "w")) == (FILE *)NULL)
	{
	    dbginfo(0, "analyse - %s\n", spr("piperr"));
	}
    }

    for ( v = index_des_ersten_verfahrens;
	 v <=index_des_letzten_verfahrens;
	 v++)
    {
	verfahren = argv[v];

	for ( e = index_der_ersten_e_datei;
	     e <=index_der_letzten_e_datei;
	     e++)
	{
	    datei = argv[e];

	    if (*datei == 'e' && *(datei+1) == '_')
		datei += 2;


	    dbginfo(0, "analyse - ----------\nanalyse - Verfahren   %s\n", verfahren);
	    dbginfo(0, "analyse - Datei       %s\n", datei);

	    pdb_entry = GetDBEntry(verfahren);

	    /* Dateiname basteln und 'stat' versuchen */

	    sprintf(e_datei, "e_%s", datei);
	    sprintf(a_datei, "a_%s_%s", pdb_entry->codierung, datei);
	    sprintf(l_datei, "l_%s", datei);

	    if(stat(e_datei,&e_buf) < 0)
	    {
		perror(e_datei);
		exit(1);
	    }

	    dbginfo(0, "analyse - Modification-Time %s ", e_datei);
	    dbginfo(0, "%d\n", (int)e_buf.st_mtime);

	    if(stat(a_datei,&a_buf) < 0)
		a_buf.st_mtime = 0;

	    dbginfo(0, "analyse - Modification-Time %s ", a_datei);
	    dbginfo(0, "%d\n", (int)a_buf.st_mtime);

	    if(strncmp(pdb_entry->verfflags, "none", 4) == 0)
		pdb_entry->verfflags[0] = '\0';
	    if(strncmp(pdb_entry->prefflags, "none", 4) == 0)
		pdb_entry->prefflags[0] = '\0';
	    if(strncmp(pdb_entry->postfflags,"none", 4) == 0)
		pdb_entry->postfflags[0] = '\0';

	    if(force == FALSE)
	    {
		/*
		   ueberpruefe Anwendbarkeit des Verfahrens durch Aufruf von
		   'auswahl -qvVerfahren datei' 
		   option -q (quiet) verhindert Textausgaben 
		   -m Testet nur ein bestimmtes Verfahren
		   */
					
		sprintf(command,"%s/bin/auswahl -aqm%s %s >/dev/null",
			PEPSYHOME, verfahren, datei);

		dbginfo(0, "analyse - command = '%s'\n", command);

		if((retvalue = system(command)) == -1)
		{
		    fprintf(stderr,"%s\n",spr("anal23"));
		    analyse_retvalue++;
		    continue;
		}

		if (WEXITSTATUS(retvalue) != 0)
		{
		    fprintf(stderr,"%s %s %s %s\n",verfahren,spr("anal24"),datei,spr("anal25"));
		    fprintf(stderr,"%s %s%s\n",spr("anal26"),datei,spr("anal27"));
		    analyse_retvalue++;
		    continue;
		}
	    }

	    /*
	       wenn Eingabe neuer als Ausgabedatei oder
	       Analyseverfahren zwischenzeitlich
	       modifiziert wurde, analysiere nochmals
	       */

	    sprintf(buffer,"%s/bin2/verfahren/%s", PEPSYHOME, verfahren);

	    if(stat(buffer,&v_buf) < 0)
	    {
		fprintf(stderr,"%s %s %s\n",spr("method"),buffer,spr("notfnd"));
		analyse_retvalue++;
		continue;
	    }

	    dbginfo(0, "analyse - Modification-Time %s ", buffer);
	    dbginfo(0, "%d\n", (int)v_buf.st_mtime);

	    if (   v_buf.st_mtime > a_buf.st_mtime 
		|| e_buf.st_mtime > a_buf.st_mtime 
		|| a_buf.st_size == (off_t) 0
		|| force == TRUE)
	    {
		if (dbg || verbose)
		{
		    if (e_buf.st_mtime > a_buf.st_mtime)
			fprintf(stderr,"analyse - %s%d%s%d)\n",
				spr("anal31"), (int)e_buf.st_mtime, spr("anal32"), (int)a_buf.st_mtime);
		    if (v_buf.st_mtime > a_buf.st_mtime)
			fprintf(stderr,"analyse - %s%d) > (%d)\n",
				spr("anal33"), (int)v_buf.st_mtime, (int)a_buf.st_mtime);

		    if (force)
			fprintf(stderr,"analyse - %s\n",spr("anal34"));
		}

		/*
		   nicht interaktive Verfahren ausfiltern
		   */
		if ( ((strncmp(pdb_entry->verfahren, "simulation", 10)) == 0)
		    ||((strncmp(pdb_entry->verfahren, "sim2",4)) == 0)		 
		    )
		{
		    sprintf(command,
			    "nohup /bin/sh %s/bin2/verfahren/%s %s %s%s%s %s%s%s %s%s%s %s %s 1>sim_nohup 2>&1 &\n",
			    PEPSYHOME, verfahren,
			    (verbose)?"-v":"",
			    (*opt_parameter   != '\0')?"-O '":"", opt_parameter,
			    (*opt_parameter   != '\0')?"'":"",
			    (*pref_parameter  != '\0')?"-p '":"", pref_parameter,
			    (*pref_parameter  != '\0')?"'":"",
			    (*postf_parameter != '\0')?"-P '":"", postf_parameter,
			    (*postf_parameter != '\0')?"'":"",
			    datei,a_datei);

		    if(dbg || verbose)
			fprintf(stderr, "analyse - %s '%s'\n", spr("aufruf"), command);

		    fprintf(stderr,"%s\n", spr("anal37"));
		    fprintf(stderr,"%s\n\n", spr("anal38"));

		    retvalue = system(command);

		    (void)sleep(2);
		    (void)system("cat sim_nohup");
		}
		else
		{
		    sprintf(command,
			    "%s/bin2/prefilter/%s %s %s %s 2>prefilter.err | %s %s/bin2/verfahren/%s %s %s %s 2>method.err | %s/bin2/postfilter/%s %s %s %s 2>postfilter.err",
			    PEPSYHOME, pdb_entry->prefilter, pref_parameter,
			    pdb_entry->prefflags, datei,
			    watch_time == TRUE ? "time" : "",
			    PEPSYHOME, pdb_entry->verfahren, pdb_entry->verfflags,
			    opt_parameter, num_parameter,
			    PEPSYHOME, pdb_entry->postfilter, pdb_entry->postfflags, postf_parameter,
			    datei);

		    if(dbg || verbose)
			fprintf(stderr,"analyse - %s '%s'\n", spr("aufruf"), command);

		    pepout_fp = popen(command, "r");

		    if (scanline(pepout_fp, buffer, 5120) != EOF)
		    {
			do
			{
			    if (strncmp(buffer, "#!analyse", 9) == 0)
			    {

				dbgtext(30, "analyse(): #!analyse found!\n");

				sscanf(buffer, "#!analyse %s", verf);

				dbginfo(30, "analyse(): Verfahren='%s'.\n", verf);

				pdbe = GetDBEntry(verf);

				if (pdbe)
				{
				    sprintf(a_datei, "a_%s_%s", pdbe->codierung, datei);
				
				    out_fp = fopen(a_datei, "w");
				}
			    }
			    else
			    {
				dbgtext(30, "analyse(): no #!analyse found! Opening a_datei.\n");

				sprintf(a_datei, "a_%s_%s", pdb_entry->codierung, datei);
			    
				out_fp = fopen(a_datei, "w");

				if (out_fp)
				    fputs(buffer, out_fp);
			    }

			    if (out_fp)
			    {

				dbgtext(30, "analyse(): Anhaengen der Daten....\n");

				while((scanchars = scanline(pepout_fp, buffer, 5120)) != EOF)
				{
				    if ((strncmp(buffer, "#!analyse", 9)) == 0)
					break;
				    else
					fputs(buffer, out_fp);
				}

				fclose(out_fp);
				out_fp = NULL;

				if(!batch)
				{
				    if ((fp=fopen(a_datei, "r")) == (FILE *)NULL)
				    {
					dbginfo(0, "analyse - %s\n", spr("opnerr"));
				    }
				    else
				    {
					while((c=getc(fp)) != EOF)
					    putc(c, fpp);
			
					fclose(fp);
					
					/*
					fputs("\n\n\n", fp);
					*/
				    }
				}
			    }
			}
			while ((scanchars != EOF) && (strncmp(buffer, "#!analyse", 9) == 0));
		    }

		    exitvalue = pclose(pepout_fp);

		    concat_logfile((createlogfile == TRUE)?l_datei:(char *)NULL, command);


		}

		if (exitvalue)
		{
		    fprintf(stderr, "%s - %s %d)\n", pdb_entry->verfahren, spr("proble"), exitvalue);
		    analyse_retvalue++;
		    continue;
		}
	    }
	    else
	    {
		if(!batch && a_buf.st_size > (off_t) 0)
		{
		    if ((fp=fopen(a_datei, "r")) == (FILE *)NULL)
		    {
			dbginfo(0, "analyse - %s\n", spr("opnerr"));
		    }
		    else
		    {
			while((c=getc(fp)) != EOF)
			    putc(c, fpp);
	
			fclose(fp);
			
			fputs("\n\n\n", fp);
		    }
		}

	    }

	    if (batch)
		fprintf(stderr, "analyse - %s %s done.\n", pdb_entry->verfahren, datei);

	} /* for (e) alle Eingabedateien abarbeiten */

    } /* for (v) alle Verfahren abarbeiten */

    if (fpp)
	pclose(fpp);

    exit(analyse_retvalue);
}
