/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988, 1994
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.




	Version 1.0

	Version 1.1	05.02.92	kirschni	Dateinamen koennen auch mit dem Prefix
							'e_' angegeben werden
	Version 1.2	05.02.92	kirschni	Mehrere Dateinamen koennen gleichzeitig
							angegeben werden
	Version 1.3	06.02.92	kirschni	Ausgabe bei '-f' Option uebersichtlicher
							gestaltet.
							Kosmetische Korrekturen.
	Version	1.4	06.02.92	kirschni	Option -v eingefuehrt, mit der die Anwend-
							barkeit eines bestimmten Verfahrens getestet
							werden kann.
	Version	1.5	06.02.92	kirschni	Option -q eingefuehrt, zur Beschleunigung der
							Anwendbarkeitsueberpruefung in analyse(1). Bei
							-q (quiet) wird auf jegliche Ausgabe verzichtet
								und der Exitcode entsprechend gesetzt.
								0 -> Verfahren anwendbar
								1 -> Verfahren nicht anwendbar
	Version 1.7	06.02.92	kirschni	die Option -v wird ersetzt durch -m (method) um
							Verwechselungen mit verbose zu vermeiden.
			06.03.92	kirschni	Einfuehrung der SCCS-Verwaltung

			02.09.92	kirschni	Neue Database-Struktur genutzt. Eigenes oeffnen
							der Grenzen-Dateien ist ueberfluessig
			02.09.92	kirschni	Erweiterte Angaben bei Knotentypen

			11.02.93	kirschni	Erweiterung um die -r-Option.
*/

static	char	*SccsId	= "%Z%PEPSY:\t%M%\t%R%.%L%\t%D% %T%";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "d_netz.h"
#include "d_konst.h"
#include "d_debug.h"
#include "d_file.h"
#include "database.h"
#include "libpepsy.h"

#define USAGE()	{ \
	fprintf(stderr,"Korrekter Aufruf: %s [-rRfa] [-mVerfahrensname] [-q] [-dnum] dateinname ...\n", argv[0]); \
	fprintf(stderr,"                  %s -l [-Rra] dateiname\n", argv[0]); \
	fprintf(stderr,"                  %s [-V]\n", argv[0]); \
	exit(1); }

/*
   Ermittelt fuer eine gegebene Eingabedatei die darauf anwendbaren	
   Leistungsbewertungsverfahren						
   Wenn schon Ergebnisse fuer ein Verfahren vorliegen, so wird dies in der Ausgabe
   durch ein vorgestelltes '+' angezeigt.
   								
   Aufrufoptionen:						
   -V:		Version ausgeben
   -f:		Es soll ausgegeben werden, warum ein Verfahren nicht anwendbar ist
   -a:		Alle Verfahren untersuchen
   -l:		Alle Verfahren werden einzeilig hintereinander ausgegeben. Verfahren, die noch einen
		Zusatzblock benoetigen werden nicht ausgegeben, also nur unmittelbar anwendbare Verfahren.
   -r:		Es werden ausschliesslich die Verfahren angezeigt, fuer die schon Ergebnisse vorliegen
   -R:		Eine Ausgabe des '+' unterbleibt, auch wenn schon Ergebnisse vorliegen sollten.
   -q:		Quiet-option. Im Zusammenhang mit -v gibt der Rueckgabewert das Auswahlergebnis
			bekannt. Bei mehreren Modellbeschreibungen wird der Rueckgabewert geodert, d.h.
			sobald ein Verfahren _nicht_ anwendbar ist wird 1 zurueckgeliefert.
   -mVN:	Es wird getestet ob das angegebene Verfahren VN anwendbar ist
   -dnum : 	Debug-Niveau		
   -Eingabedateiname			
   							
*/

netzstruktur netz;
int 	dbg = 0;       	/* debug - niveau			*/

/*
*	FehlerAusgabe verhindert ein wiederholtes ausgeben des immer gleichen
*	Fehlertextes "Verfahren bounds nicht anwendbar ..." etc.
*	Statt dessen werden Leerzeichen ausgegeben, so dass alle Gruende fuer
*	die Nichtanwendbarkeit des Verfahrens untereinander erscheinen.
*	Gesteuert wird das ueber die 'boolsche' Variable offset, die wenn TRUE
*	statt der Meldung die Leerzeichen bewirkt.
*
*	06.02.92 kirschni
*/
void FehlerAusgabe(fp, verfahren, grund, offset)
FILE *fp;
char *verfahren;
char *grund;
int	  offset;
{
    static int l;
    int i;

    if (!offset)
    {
	/*	ACHTUNG!
		Das Leerzeichen nach dem Verfahren ist unbedingt notwendig,
		da die Ausgabe z.B. von XPEPSY weiterverwendet wird.
		*/
	l=fprintf(fp,"%s : %s %s ",verfahren, spr("ausw10"), spr("ausw11"));
    }
    else
    {
	for(i=l; i; i--)
	    fputc(' ', fp);
    }
    
    fprintf(fp,"%s\n", grund);
}

int main(argc,argv)
int	argc;
char	*argv[];
{
    /*		Sccs support from SccsLogging.c			*/	
    extern int SccsRelease;
    extern int SccsLevel;

    extern int optind;
    extern char *optarg;

    extern char *getenv();
    extern char *strrchr();

    netzstruktur *np = &netz;
    char	 datei[100];	
    char	 buffer[5120];	
    char	 nicht_anwendbare_knotentypen[5120];
    char	 *cp;
    int		 c;
    int 	 i,
    		 j;	
    int		 results;
    int		 resulttest=FALSE;
    int		 noplus=FALSE;
    time_t     	 modelmtime;
    int		 anwendbar;
    int		 offset=0;
    int		 fehlerausgabe=FALSE;
    int		 line=FALSE;
    int		 quiet=FALSE;
    int		 vtest=FALSE;
    int		 alle_zeigen=FALSE;
    int		 notwendige_knotentypen;
    FILE       	*fpin,
  		*fpout = stdout;
    char	 testverfahren[256];
    char	 gtestverfahren[256];
    struct stat	 statbuf[1];

    pdb_entry_t	*pdb_entry;
    int		 retvalue=0;


    if (p_lang_init(argv[0]) != 0)
	exit (-2);

    while(( c= getopt(argc,argv,"Valfd:m:qrR")) != EOF)
    {
	switch(c)
	{
	case 'a':
	    alle_zeigen = 1;
	    dbgtext(0, "option a recognized!\n");
	    break;
	case 'd':
	    if((dbg = atoi(optarg)) <= 0)
		dbg = 1;
	    break;
	case 'f':
	    fehlerausgabe	= TRUE;
	    quiet		= FALSE;
	    line		= FALSE;
	    break;
	case 'l':
	    line		= TRUE;
	    quiet		= FALSE;
	    fehlerausgabe	= FALSE;
	    break;
	case 'q':
	    quiet = TRUE;
	    line = FALSE;
	    fehlerausgabe = FALSE;
	    break;
	case 'r':
	    resulttest = TRUE;
	    break;
	case 'R':
	    noplus = TRUE;
	    break;
	case 'm':
	    alle_zeigen = TRUE;
	    vtest	= TRUE;
	    strncpy(gtestverfahren, optarg, 19);
	    break;
	case 'V':
	    fprintf(stdout ,"%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
	    exit(0);
	    break;
	default:
	    fprintf(stderr, "\n%s -%c !!!\n",spr("ausw01"), c);
	    USAGE();
	    break;
	}
    }

    GetPepsyDBSorted();

    dbginfo(5, "argc - optind = %d\n", argc-optind);


    if (((argc - optind) == 0) ||
	(((argc - optind) > 1) && line))
    {
	USAGE();
    }


    while (optind < argc)
    {
	strcpy(datei, argv[optind]);

	if (!vtest && !line && !quiet)
	    fprintf(fpout, "\n\n%s:\n", datei);
	else
	    strcpy(testverfahren, gtestverfahren);

	dbginfo(5, "auswahl: debug-niveau %d\n", dbg);
	dbginfo(5, "auswahl: %s\n", (alle_zeigen)?"grenzen.alle":"nur grenzen");

	if ((fpin = Pepsy_fopen(PFT_MODEL, NULL, datei, "r")) == NULL)
	{
	    fprintf(stderr,"auswahl: %s '%s%s\n", spr("ausw07"), datei,spr("ausw04"));
	    optind++;
	    continue;
	}

	strcpy(buffer, "e_");
	strcat(buffer, datei);
	if (stat((char *)(buffer+((strncmp(datei, "e_",2))?0:2)), statbuf) == 0)
	{
	    modelmtime = statbuf->st_mtime;
	}
	else
	    modelmtime = (time_t)0;

	dbginfo(9, "auswahl: modelmtime=%ld\n", (long)modelmtime);

	get_standard(fpin);

	/* Datei schliessen */

	fclose(fpin);

	if(!fehlerausgabe && !vtest && !quiet && !line)
	{
	    fprintf(fpout,"%s\n",spr("ausw06"));
	    fprintf(fpout,"---------------------------------------------------------\n");
	}

	/*
	   Datensatz aus grenzen[all].txt lesen und anschliessend 	
	   Daten Eingabedaten auf Grenzen und Schranken ueberpruefen	
	   */

	notwendige_knotentypen = 0;

	for(i=0;i<np->knot;i++)
	{
	    dbginfo(30, "auswahl: Knotentyp 0%08o im Netz enthalten!\n", np->knotentyp[i]);

	    notwendige_knotentypen |= np->knotentyp[i];
	}

	dbginfo(25, "auswahl: Notwendige Knotentypen=0%08o\n", notwendige_knotentypen);


	ResetDBEntry();

	while ((pdb_entry = (vtest)?GetDBEntry(testverfahren):NextDBEntry()))
	{
	    strcpy(testverfahren, "unbekanntes Verfahren");

	    if (!alle_zeigen && ((pdb_entry->internal_flags&PDBF_STD) != PDBF_STD))
		continue;

	    anwendbar	= V_JA;
	    offset	= 0;

	    dbginfo(0, "auswahl: Teste %s ....\n", pdb_entry->verfahren);

	    /* Mit Eingabedaten vergleichen */

	    dbgtext(10, "auswahl:   ... Netztyp\n");

	    if (np->typ & ~(pdb_entry->netztyp)) /* Netztyp ueberpruefen */
	    {
		anwendbar = V_NEIN;

		if (fehlerausgabe)
		{	
		    FehlerAusgabe(fpout, pdb_entry->verfahren, spr("ausw12"), offset);
		    offset=1;
		}
	    }

	    dbginfo(15, "auswahl:       %s\n", (anwendbar==V_JA)?"OK":"Failed");

	    dbgtext(10, "auswahl:   ... Anzahl der Knoten\n");

	    if ((pdb_entry->maxknoten != 0)	/* Anzahl Knoten ueberpruefen */
		&& (np->knot > pdb_entry->maxknoten))
	    {
		anwendbar = V_NEIN;

		if (fehlerausgabe)
		{	
		    FehlerAusgabe(fpout, pdb_entry->verfahren, spr("ausw13"), offset);
		    offset=1;
		}
	    }

	    dbginfo(15, "auswahl:       %s\n", (anwendbar==V_JA)?"OK":"Failed");

	    dbgtext(10, "auswahl:   ... Anzahl der Klassen\n");

	    if ((pdb_entry->maxklassen != 0 )  /* Anzahl Klassen ueberpruefen */
		&& (np->klass > pdb_entry->maxklassen))
	    {
		anwendbar = V_NEIN;

		if (fehlerausgabe)
		{	
		    FehlerAusgabe(fpout, pdb_entry->verfahren, spr("ausw14"), offset);
		    offset=1;
		}
	    }

	    dbginfo(15, "auswahl:       %s\n", (anwendbar==V_JA)?"OK":"Failed");

	    dbgtext(10, "auswahl:   ... Knotentypen\n");

	    dbginfo(50, "auswahl:       erlaubt: 0%08o\n", pdb_entry->knotentypen);

	    if (notwendige_knotentypen & ~(pdb_entry->knotentypen)) /* Knotentyp ueberpruefen */
	    {
		anwendbar = V_NEIN;

		if (fehlerausgabe)
		{	
		    j = notwendige_knotentypen & ~(pdb_entry->knotentypen);
		    nicht_anwendbare_knotentypen[0]='\0';
		    i = 1;

		    while(j)
		    {
			if (j & 1)
			{
			    strcat(nicht_anwendbare_knotentypen, knotentypen[i]);
			    strcat(nicht_anwendbare_knotentypen, ", ");
			}
			i++;
			j = j>>1;
		    }
		    cp = strrchr(nicht_anwendbare_knotentypen, ',');
		    if (cp)
			*cp = '\0';

		    sprintf(buffer, "%s: (%s)", spr("ausw15"), nicht_anwendbare_knotentypen);
				
		    FehlerAusgabe(fpout, pdb_entry->verfahren, buffer, offset);
		    offset=1;
		}
	    }

	    dbginfo(15, "auswahl:       %s\n", (anwendbar==V_JA)?"OK":"Failed");

	    dbgtext(10, "auswahl:   ... Uew oder Besh\n");

	    if (np->haeuf == FALSE && pdb_entry->besh == 0) /* Uew und Besh ueberpruefen */
	    {
		anwendbar = V_NEIN;

		if (fehlerausgabe)
		{	
		    FehlerAusgabe(fpout, pdb_entry->verfahren, spr("ausw16"), offset);
		    offset=1;
		}
	    }

	    if (np->haeuf == TRUE && pdb_entry->uew == 0)
	    {
		anwendbar = V_NEIN;

		if (fehlerausgabe)
		{	
		    FehlerAusgabe(fpout, pdb_entry->verfahren, spr("ausw17"), offset);
		    offset=1;
		}
	    }


	    /*
	       Wenn der Zusatzblock noch eingegeben werden muss, das Verfahren bisher
	       aber anwendbar war, dann ist das Verfahren nur mit Zusatzinformation
	       anwendbar (anwendbar = V_VIELLEICHT)
	       */
	    if (zusatzblock_notwendig(datei, pdb_entry->verfahren) && anwendbar == TRUE)
		anwendbar = V_VIELLEICHT;

	    dbginfo(15, "auswahl:       %s\n", (anwendbar==V_JA)?"OK":"Failed");


	    /*	Bedienraten verschiedener Klassen in einer Bedieneinheit auf
		Gleichheit ueberpruefen */

	    dbgtext(10, "auswahl:   ... Bedienraten von multiple-Server-Knoten auf gleiche Bedienraten in allen Klassen\n");

	    if (pdb_entry->homogene_bedraten)
	    {
		for(i=0;i<np->knot;i++)
		{
		    if (np->knotentyp[i] == TYP1 || np->knotentyp[i] == TYP2)
		    {
			for(j=1;j<np->klass;j++)
			{
			    dbginfo4(4, "Ueberpruefe Bedienrate Knoten %d Klasse %d (%f,%f)\n",
				     i, j, np->bedr[i][j], np->bedr[i][0]);

			    if(np->bedr[i][j] != np->bedr[i][0])
			    {
				anwendbar = V_NEIN;
				j = np->klass;

				if (fehlerausgabe)
				{
				    sprintf(buffer, "%s %d %s", spr("ausw22"),i+1,spr("ausw23"));
				    FehlerAusgabe(fpout, pdb_entry->verfahren, buffer, offset);
				    offset=1;
				}
			    }
			}
		    }
		}
	    }

	    dbginfo(15, "auswahl:       %s\n", (anwendbar==V_JA)?"OK":"Failed");

	    /* Maximale Anzahl parallele Bedieneinheiten bei M/M/m Knoten */

	    dbgtext(10, "auswahl:   ... Anzahl paralleler Bedieneinheiten\n");

	    if(pdb_entry->parBE > 0)
	    {
		for(i=0;i<np->knot;i++)
		{
		    if (np->knotentyp[i] == TYP2 )
		    {
			if(np->bede[i] > pdb_entry->parBE)
			{
			    anwendbar = V_NEIN;

			    if (fehlerausgabe)
			    {
				sprintf(buffer, "%d  %s%d%s %d %s",
					i,
					spr("ausw25"),
					np->bede[i],
					spr("ausw26"),
					pdb_entry->parBE,
					spr("ausw27"));
				FehlerAusgabe(fpout, pdb_entry->verfahren, buffer, offset);
				offset=1;
			    }
			}
		    }
		}
	    }

	    dbginfo(15, "auswahl:       %s\n", (anwendbar==V_JA)?"OK":"Failed");


	    /* Quadvar ueberpruefen */

	    dbgtext(10, "auswahl:   ... QCOV\n");

	    for(i=0;i<np->knot;i++)
	    {
		if( np->knotentyp[i] != TYP3
		   && np->knotentyp[i] != TYP4
		   && np->knotentyp[i] != TYP9 )
		{
		    if (pdb_entry->maxCOV == -1)
		    {
			for(j=0;j<np->klass;j++)
			{
			    if(np->quadvar[i][j] < pdb_entry->minCOV)
			    {
				anwendbar = V_NEIN;
				i = np->knot;
				j = np->klass;

				if (fehlerausgabe)
				{
				    sprintf(buffer, "%s %f %s",
					    spr("ausw28"),
					    pdb_entry->minCOV,
					    spr("ausw27"));
				    FehlerAusgabe(fpout, pdb_entry->verfahren, buffer, offset);
				    offset=1;
				}

			    }
			}
		    }
		    else
		    {
			for(j=0;j<np->klass;j++)
			{
			    if	(	np->quadvar[i][j] < pdb_entry->minCOV
				 ||	np->quadvar[i][j] > pdb_entry->maxCOV)
			    {
				anwendbar = V_NEIN;
				i = np->knot;
				j = np->klass;

				if (fehlerausgabe)
				{
				    sprintf(buffer, "%s%f,%f%s",
					    spr("ausw29"),
					    pdb_entry->minCOV,
					    pdb_entry->maxCOV,
					    spr("ausw30"));
				    FehlerAusgabe(fpout, pdb_entry->verfahren, buffer, offset);
				    offset=1;
				}
			    }
			}
		    }
		}
	    }

	    dbginfo(15, "auswahl:       %s\n", (anwendbar==V_JA)?"OK":"Failed");

	    /*	
	       liegt schon eine datei mit Ergebnissen vor ?
	       */

	    dbgtext(10, "auswahl:   ... Ergebnisdatei\n");

	    results = 0;
	    sprintf(buffer, "a_%s_%s\0", pdb_entry->codierung, (strncmp(datei, "e_", 2))?datei:(char *)(datei+2));

	    dbginfo(15, "auswahl:            check for existing '%s'.\n", buffer);

	    if (stat(buffer, statbuf) == 0)
	    {
		dbginfo(20, "auswahl:            resultsmtime=%ld\n", (long)statbuf->st_mtime);

		if (statbuf->st_mtime>modelmtime)
		    results = 1;
		dbgtext(20, "auswahl:            file found!\n");
	    }
	    else
	    {
		dbgtext(20, "auswahl:            file not found!\n");
	    }

	    if (resulttest && !results)
	    {
		anwendbar = V_NEIN;

		if (fehlerausgabe)
		{
		    FehlerAusgabe(fpout, pdb_entry->verfahren, spr("norslt"), offset);
		    offset=1;
		}
	    }

	    dbginfo(15, "auswahl:       %s\n", (anwendbar==V_JA)?"OK":"Failed");

	    /* Ergebnis ausgeben */

	    if(!fehlerausgabe && !quiet )
	    {
		switch(anwendbar)
		{
		case V_JA:	
		    if (vtest)
			fprintf(fpout, "%s %s %s\n", pdb_entry->verfahren, spr("anwdbr"), datei);
		    else
			if (line)
			    fprintf(fpout, "%c%s ", (results && !noplus)?'+':' ', pdb_entry->verfahren);
			else
			    fprintf(fpout,"   %c%-20s  |\n", (results && !noplus)?'+':' ', pdb_entry->verfahren);
		    break;
		case V_VIELLEICHT:
		    if (!vtest && !line)
			fprintf(fpout,"                          | %c%-20s\n", (results && !noplus)?'+':' ', pdb_entry->verfahren);
		    break;
		case V_NEIN:
		    break;
		default:
		    fprintf(fpout,"??? %x\n",anwendbar);
		    break;
		}
	    }
	} /* while(...) */

	dbgtext(0, "auswahl: Ende\n");

	optind++;
	retvalue |= (anwendbar==V_JA)?0:1;
	
	if (line)
	    fprintf(fpout, "\n");

    } /* while noch eingabedateien vorhanden */

    fclose(fpout);

    return((vtest)?retvalue:0);
}
