/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988, 1994
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tpepout.c\t1.7\t95/05/22 10:55:22";

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <signal.h>
#include "d_konst.h"
#include "d_speicher.h"
#include "d_debug.h"
#include "libpepsy.h"
#include "database.h"

#define USAGE()	{ \
	fprintf(stderr,"%s\n",spr("pepo01")); exit(1);}

/*
  Der pepout-Ausgabefilter erzeugt aus standartisierten		
  Ausgaben von Berechnungsverfahren globale PEPSY-Standart-		
  Ausgabedateien. Die Bezeichnungen der Knoten werden in der		
  entsprechenden Eingabedatei hinterfragt und ausgegeben. Fehlt	
  diese Eingabedatei werden Knoten normal nummeriert. Bei An-		
  gabe einer falschen Eingabedatei koennen die Knotenbezeich-		
  nungen falsch sein !						
									
 Aufrufoptionen:							
 - dbg		: Debug_Niveau						
 - datei 	: Eingabedateiname (ohne "e_")				
									
*/

extern	int	atoi();

/* Netzgroessen */
int	knoten;			/* Anzahl der Knoten			*/
int	klassen;		/* Anzahl der Klassen			*/
int	netztyp;		/* Netztyp				*/
char	**knotname;		/* Namen der Knoten			*/
double *durchsatz;		/* Durchsatz durch den Bezugsknoten	*/
double *verweilzeit;		/* Zeit, bis ein Auftrag wieder am
					   Bezugsknoten erscheint	*/
double *fuellung;		/* Zahl der Auftraege im Netz */

/* Einzelleistungsgroessen */
double	**lambda;		/* Durchsatz				*/
double	**rho;			/* Auslastung				*/
double	**maa;			/* Mittlere Anzahl Auftraege		*/
double  **ee; 		        /* Besuchhauefigkeiten */

/* Gesamtleistungsgroessen */
double	*ges_lambda;		/* Gesamt-Durchsatz pro Knoten		*/
double	*ges_rho;		/* Gesamt-Auslastung pro Knoten		*/
double	*ges_maa;		/* Gesamte Mittlere Anzahl Auftraege  	*/
				/* pro Knoten				*/

int	dbg=0;			/* Debugniveu 				*/
int	debug=0;		/* Debugniveu 				*/

void best_knotnamen();
void pep_end();

int main(argc,argv)
int	argc;
char	*argv[];
{
    /*		Sccs support from SccsLogging.c			*/	
    extern int SccsRelease;
    extern int SccsLevel;

    /*		fuer getopt(3C)					*/
    extern	char	*optarg;
    extern	int	optind;
    int		c;

    FILE 	*fp_in;		/* Filedeskriptor fuer die Eingabedatei	*/
    FILE	*fp_out;	/* 	  "	    "	"  Ausgabedatei	*/
    char	edatei[NAM_LAENGE]; /* Name fuer Eingabedatei		*/
    int		eingabedatei;	/* an, ob eine Eingabedatei angege-	
				   ben worden ist			*/

    char	buffer[512];	/* Puffer fuer Eingabezeilen		*/
    char	verfahren[NAM_LAENGE]; /* Name des Analyseverfahrens	*/

    char  string[NAM_LAENGE];	/* Hilfsstring				*/
    char	*s;		/* Hilfsstring fuer Aufrufoptionen	*/
    int		i,r,durch;	/* Hilfsvariable			*/
    double	help2;		/*	 "				*/
    double	help3;		/*	 "				*/
    double	help4;		/*	 "				*/
    double	help5;		/*	 "				*/
    int		legend = TRUE;	/* boolsche Variable fuer Legendenausg.	*/


    signal(SIGFPE,pep_end);
    signal(SIGBUS,pep_end);
    signal(SIGINT,pep_end);
    signal(SIGHUP,pep_end);
    signal(SIGILL,pep_end);


    if (p_lang_init(argv[0]) != 0)
	exit(-2);

    while((c=getopt(argc, argv, "Vd:l")) != EOF)
	switch (c) 
	{
	case 'V':
	    fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
	    exit(0);
	    break;
	case 'd':
	    if ((dbg=atoi(optarg)) <0)
		dbg=1;
	    break;
	case 'l':
	    legend = FALSE;
	    break;
	default:
	    fprintf(stderr,"\n%s -%c !!!\n",spr("pepo03"), *s);
	    USAGE();
	    break;
	}

    if ((argc-optind) == 1)
    {
	eingabedatei = TRUE;
	strcpy(edatei,"e_");
	strcat(edatei,argv[optind]);
	if ((fp_in = fopen(edatei,"r")) == NULL)
	{
	    fprintf(stderr,"%s %s %s\n",spr("pepo04"),edatei,spr("pepo05"));
	    exit(2);
	}
    }
    else
    {
	if((argc-optind) == 0)	/* Keine Eingabedatei angegeben */
	{
	    eingabedatei = FALSE;
	}
	else
	{
	    fprintf(stderr,"%s\n",spr("pepo06"));
	    USAGE();
	}
    }

    fp_out = stdout;

    /* Erste Eingabezeile lesen
     */
    nextline(stdin,buffer,512);
   
    do
    {
	if (strncmp(buffer, "multioutput", 11) == 0)
	{
	    nextline(stdin, buffer, 512);
	    sscanf(buffer,"%s",verfahren);

	    fprintf(fp_out, "#!analyse %s\n", verfahren);
	}
	else
	{
	    sscanf(buffer,"%s",verfahren);
	}

	dbginfo(0, "pepout: Verfahren: '%s'\n", verfahren);

	
	/* Netztyp, Anzahl Knoten und Klassen  einlesen
	 */
	nextline(stdin,buffer,512);
	sscanf(buffer,"%d %d %d",&netztyp,&knoten,&klassen);

	dbginfo(0, "pepout: Netztyp: %d\n", netztyp);
	dbginfo(0, "pepout: Knoten: %d\n", knoten);
	dbginfo(0, "pepout: Klassen: %d\n", klassen);

	if (knoten <= 0 || klassen <= 0)
	{
	    dbgtext(0, "pepout: FATAL ERROR - (Knoten <=0 || Klassen <=0)\n");
	    exit(1);
	}

	/* Speicherplatz belegen */

	/* Knotennamen nur beim ersten Durchlauf ermitteln
	   ebenso Speicher auch nur einmal reservieren
	 */
	if (knotname == (char **)NULL && eingabedatei == TRUE && knoten > 0 && klassen > 0)
	{
	    knotname = c_2dim_alloc(knoten,NAM_LAENGE);
	    if(knotname == 0 )
	    {
		fprintf(stderr,"%s\n",spr("pepo18"));
		eingabedatei = FALSE;
	    }
	    else
	    {
		/* Knotennamen bestimmen */
		best_knotnamen(fp_in);
	    }

	    /* Einzelleistungsgroessen
	     */
	    lambda	= d_2dim_alloc(knoten,klassen);
	    rho	= d_2dim_alloc(knoten,klassen);
	    maa	= d_2dim_alloc(knoten,klassen);
	    ee 	= d_2dim_alloc(knoten,klassen);

	    /* Gesamt-Leistungsgroessen
	     */
	    if (klassen > 1 && knoten > 0)
	    {
		ges_lambda	= d_1dim_alloc(knoten);
		ges_rho		= d_1dim_alloc(knoten);
		ges_maa		= d_1dim_alloc(knoten);
	    }

	    if (klassen > 0)
	    {
		durchsatz	= d_1dim_alloc(klassen);
		verweilzeit	= d_1dim_alloc(klassen);
		fuellung	= d_1dim_alloc(klassen);
	    }

	    dbgtext(0, "pepout: Knotennamen ermittelt und Speicherplatz reserviert!\n");
	}

	/* Kopf ausgeben
	 */
	if (eingabedatei == TRUE)
	{
	    fprintf(fp_out,"%s  %s\n\n", spr("pepo20"), argv[optind]);
	    fprintf(fp_out,"%s '%s' %s\n\n", spr("pepo21"), edatei, spr("pepo22"));
	}
	else
	{
	    fprintf(fp_out,"%s\n",spr("pepo23"));
	    fprintf(fp_out,"%sn\n",spr("pepo24"));
	}

	switch(netztyp)
	{
	case GESCHLOSSEN:
	    strcpy(string,spr("pepo25"));
	    break;
	case OFFEN: 
	    strcpy(string,spr("pepo26"));
	    break;
	case GEMISCHT: 
	    strcpy(string,spr("pepo27"));
	    break;
	default:		
	    strcpy(string,"");
	    break;
	}

	fprintf(fp_out,"%s %s %s",spr("pepo28"), string,spr("pepo29"));
	fprintf(fp_out," '%s'  %s.\n", verfahren,spr("pepo30"));

	/* Einlesen und Ausgabe der Einzelleistungsgroessen  */

	for (r=0;r<klassen;r++)
	{
	    fprintf(fp_out,"\n\n%s %d \n\n",spr("pepo31"),r+1);
	    fprintf(fp_out,"%-10.10s |  %s \n",verfahren,spr("pepo32"));
	    fprintf(fp_out,
		    "-----------+----------------------------------------------------------------\n");

	    for (i=0; i<knoten; i++)
	    {
		if (eingabedatei == TRUE)
		    fprintf (fp_out,"%-10.10s |",knotname[i]);
		else
		    fprintf (fp_out,"%s %2d  |",spr("pepo33"),i+1);

		nextline(stdin,buffer,512);

		dbginfo(1, "pepout: %s\n",buffer);

		sscanf(buffer,"%lg %lg %lg %lg %lg %lg %lg %lg ",
		       &lambda[i][r],
		       &ee[i][r],
		       &help2,
		       &rho[i][r],
		       &help3,
		       &maa[i][r],
		       &help4,
		       &help5);
		fprintf(fp_out," %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f \n",
			lambda[i][r],
			ee[i][r],
			help2,
			rho[i][r],
			help3,
			maa[i][r],
			help4,
			help5);
	    }
	}

	/* Berechnung der Gesamtleistungsgroessen  */

	if (klassen > 1)
	{
	    for (i=0;i<knoten;i++)
	    {
	        ges_lambda[i]	= 0.0;
		ges_rho[i]	= 0.0;
		ges_maa[i]	= 0.0;

		for (r=0;r<klassen;r++)
		{
		    ges_lambda[i]	+= lambda[i][r];
		    ges_rho[i]		+= rho[i][r];
		    ges_maa[i]		+= maa[i][r];
		}
	    }
	}

	/* Gesamtleistungsgroessen ausgeben */

	if (klassen > 1)
	{
	    fprintf(fp_out,"\n\n%s\n\n",spr("pepo34"));
	    fprintf(fp_out,"%-10.10s |  %s  \n",verfahren,spr("pepo35"));
	    fprintf(fp_out,"-----------+--------------------------\n");

	    for (i=0;i<knoten;i++)
	    {
		if (eingabedatei == TRUE)
		    fprintf (fp_out,"%-10.10s |",knotname[i]);
		else
		    fprintf (fp_out,"%s %2d  |",spr("pepo36"),i+1);

		fprintf(fp_out," %7.3f %7.3f %7.3f \n",ges_lambda[i],ges_rho[i],ges_maa[i]);
	    }
	}

	fprintf (fp_out,"\n\n");

	/* Berechnug der Netzgroessen */

	for(r=0;r<klassen;r++)
	{
	    durch = FALSE;

	    for(i=0;i<knoten;i++)
	    {
		fuellung[r] +=  maa[i][r];

		if( !durch && ee[i][r] > 0.0)
		{
		    durchsatz[r]	= lambda[i][r] / ee[i][r];
		    durch		= TRUE;
		}
	    }

	    if(durchsatz[r] > 0.0)
		verweilzeit[r] = fuellung[r] / durchsatz[r];
	    else
		verweilzeit[r] = 0.0;
	}

	fprintf(fp_out,"\n\n%s\n\n",spr("pepo37"));
	fprintf(fp_out,"%-10.10s |  %s  \n",verfahren,spr("pepo38"));
	fprintf(fp_out,"-----------+--------------------------\n");

	if(klassen > 1)
	{
	    for(r=0;r<klassen;r++)
	    {
		fprintf(fp_out,"%s %2d  |",spr("pepo39"),r+1);
		fprintf(fp_out," %7.3f %7.3f %7.3f \n",durchsatz[r],verweilzeit[r],fuellung[r]);
	    }
	}
	else
	{
	    fprintf(fp_out,"           |");
	    fprintf(fp_out," %7.3f %7.3f %7.3f \n",durchsatz[0],verweilzeit[0],fuellung[0]);
	}

	fprintf(fp_out,"\n\n\n");

	dbgtext(0, "pepout: At end of Std-Description\n");
	
	while((c=nextline(stdin, buffer, 512)) != EOF)
	{
	    if (strncmp(buffer, "multioutput", 11) == 0)
		break;

	    fprintf(fp_out,"%s", buffer);
	}
 

	if (legend)
	{
	    fprintf(fp_out,"\n\n");
	    fprintf(fp_out,"%s\n",spr("legen0"));
	    fprintf(fp_out,"\n");
	    fprintf(fp_out,"%s\n",spr("legen1"));
	    fprintf(fp_out,"%s\n",spr("legen2"));
	    fprintf(fp_out,"%s\n",spr("legen3"));
	    fprintf(fp_out,"%s\n",spr("legen4"));
	    fprintf(fp_out,"%s\n",spr("legen5"));
	    fprintf(fp_out,"%s\n",spr("legen6"));
	}
	
	dbginfo(0, "pepout: (c!=EOF)=%s\n", (c!=EOF)?"true":"false");
    }
    while (c != EOF);

    /* Speicherplatz wieder freigeben */

    if (eingabedatei == TRUE && knoten > 0)
	c_2dim_free(knotname,knoten,11);

    if (knoten > 0 && klassen > 0)
    {
	d_2dim_free(lambda,knoten,klassen);
	d_2dim_free(rho,knoten,klassen);
	d_2dim_free(maa,knoten,klassen);
	d_2dim_free(ee,knoten,klassen);
    }
    if (klassen > 1 && knoten > 0)
    {
	d_1dim_free(ges_lambda,knoten);
	d_1dim_free(ges_rho,knoten);
	d_1dim_free(ges_maa,knoten);
    }
    if (klassen > 0)
    {
	d_1dim_free(durchsatz,klassen);
	d_1dim_free(verweilzeit,klassen);
	d_1dim_free(fuellung,klassen);
    }

    /* Dateien schliessen */

    if (eingabedatei == TRUE)
    {
	fclose(fp_in);
    }

    exit(0);

} /* pepout(...) */

/*
    Funktion best_knotnamen():					
								
    Aufrufparameter:			
    - fp : Filepointer einer Eingabedatei
								
    Bestimmt aus der als Parameter uebergebenen Eingabedatei 	
    die Namen der Knoten						
*/

void best_knotnamen(fp)
FILE	*fp;
{
    int 	i,j;		/* Hilfsvariable		*/
    char	*cp;		/* Hilfszeiger auf char	  	*/
    char	buffer[512];		/* Puffer fuer Eingabezeilen	*/

    nextline(fp,buffer,512);
    nextline(fp,buffer,512);
    nextline(fp,buffer,512);
    nextline(fp,buffer,512);
    nextline(fp,buffer,512);

    for (i=0;i<knoten;i++)
    {
	nextline(fp,buffer,512);

	cp = buffer;

	/* bis '|' lesen */
	while(*cp != '\0' && *cp++ != '|')
	;

	/* blanks ueberlesen */
	while( *cp != '\0' && (*cp == ' ' || *cp == '\t'))
	    cp++;

	for(j=0;j<=10 && *cp != '\0' && *cp != '|';j++ ,cp++)
	    knotname[i][j] = *cp;

	knotname[i][j] = '\0';
    }

} /* best_knotname(...) */

void pep_end()
{
    fprintf(stderr,"%s\n",spr("pepo40"));
    exit(1);
}




