/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\te2s.c\t1.8\t94/01/27 12:22:13";

#include <stdio.h>
#include <sys/types.h>
#include <d_netz.h>
#include <d_asym.h>
#include <modell.h>
#define	EXTERN_KNOTENTYPEN
#include <d_konst.h>

/************************* BS-Funktionen **********************************************/
extern void *calloc(),
				 exit();
extern char	*strncat();
extern int	 fclose();
extern FILE	*fopen();

/**************************** Funktionen aus anderen Modulen **************************/
extern char *stripblanks();
extern void	 fprintPRE(),
				 InitFirstXX();
extern void  get_asym();

/***************************** externe Variablen **************************************/
extern netzstruktur		netz;
extern asym_data_type	asym_data;

/******************************** globale Variablen ***********************************/
int		*firstWs;
int		*firstBe;
int		 AnzBe,
			 AnzWs,
			 AnzKl;
double	*uew;

extern	int	dbg;
extern int	random_selection;	/* Steuert die Auswahl der Listentypen global */

void Usage(sProgName)
char *sProgName;
{
		fprintf(stderr, "USAGE: %s [-d][-r] sim2 Dateiname_ohne_prefix\n", sProgName);
		exit(-1);
}

main(argc, argv)
	int argc;
	char *argv[];
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

	extern char *optarg;
	extern int   opterr, optind;

	int		 c;
	char		 InFileName[1024];
	FILE		*InFile,
				*OutFile;
	double	 par1, par2;
	int		 i, j, k, l, v;

	dbg=0;

	while((c=getopt(argc, argv, "Vdr")) != EOF)
	{
		switch(c)
		{
			case 'V' :	fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
							exit(0);
							break;
			case 'd'	: 	dbg = 5;
							break;
			case 'r'	:	random_selection = 1;
							break;
			case '?'	:
			default	:	Usage(argv[0]);
		}
	}

	if ((strcmp("sim2", argv[optind])) || (optind > argc-2))
	{
		Usage(argv[0]);
	}

	(void)strncpy(InFileName, "e_", (size_t)1024);
	(void)strncat(InFileName, argv[argc-1], (size_t)1020);
	InFile = fopen(InFileName, "r");
	if (InFile == (FILE *)NULL)
	{
		fprintf(stderr, "%s: FEHLER! Kann '%s' nicht oeffnen!\n", argv[0], InFileName);
		exit(-1);
	}

	OutFile		= stdout;

	get_standard(InFile);
	get_sim(InFile);
	get_asym(InFile);

	InitFirstXX();		/* AnzXX und firstXX[] werden initialisiert */

	uew  = (double *)calloc((unsigned)AnzWs+1, sizeof(double));

	for (i=0; i<netz.knot; i++)
	{
		netz.knotname[i] = stripblanks(netz.knotname[i]);
	}

	fprintf(OutFile, "%s\n\n", &InFileName[2]);

	fprintf(OutFile, "%d\n%d\n%d\n\n", AnzWs, AnzBe, AnzKl);

	for (i=0; i<=netz.knot; i++)
	{
		for (k=0; k<GetAnzWs(i); k++)
		{
			fprintf(OutFile, "%s-Q-%d %d %d ", GetName(i), k+1, GetListTyp(i), GetAnzBe(i));

			for (j=0; j<GetAnzBe(i); j++)
				fprintf(OutFile, "%d ", firstBe[i]+j)
			;
			fprintf(OutFile, "\n");
		}
	}

	fprintf(OutFile, "\n");

	for (i=0; i<=netz.knot; i++)
	{
		for (k=0; k<GetAnzBe(i); k++)
		{
			/*
			**	Fuer jede BE den Namen und den Typ ausgeben
			*/
			fprintf(OutFile, "%s-B-%d %d", GetName(i), k+1, GetBE_Typ(i));

			switch (GetBE_Typ(i))
			{
				case PS_BT:
					fprintf(OutFile, " %d\n", GetAnzSE(i));
					break;
				default:
					fprintf(OutFile, "\n");
					break;
			}

			for (l=0; l<AnzKl; l++)
			{
				v = GetBeVtlg(i, k, l, &par1, &par2);

				/*
				**	... die Verteilung und deren Parameter ...
				*/
				fprintf(OutFile, "%d %f %f ", v, par1, par2);

				/*
				**	... die WS in die der Job nach evtl. Unterbrechung zurueckgeschickt werden
				**		 soll ...
				*/
				fprintf(OutFile, "%d ", GetWs(i, l));

				/*
				**	... in PEPSY findet kein KW statt, deshalb NOKLWECHSEL (= -1) ...
				*/
				fprintf(OutFile, "-1 ");

				/*
				**	... in PEPSY sind alle KW mit BE-Wechsel verbunden ...
				*/
				for (j=0; j<AnzKl; j++)
					fprintf(OutFile, "0 ");

				/*
				**	... KWW, die einen Auftrag nach JobSuspend betreffen, in PEPSY kein SuspendKW,
				**		 deshalb Job behaelt die Klasse bei ...
				*/
				for (j=0; j<AnzKl; j++)
					if (j!=l)
						fprintf(OutFile, "0 ");
					else
						fprintf(OutFile, "1000 ");

				/*
				**	... Preemptive-Strategie, bei normalen Knoten nur Nullen, bei PRE-Knoten
				**		 unterbrechen Klassen hoeherer Prioritaet die niedrigeren Auftrags-
				**		 Klassen ...
				*/
				fprintPRE(OutFile, i, l);

				/*
				**	... und schliesslich noch die UEW. Umgeformt in Integerwerte 0...1000 ...
				*/
				for (j=0; j<=AnzWs; j++)
				{
					uew[j]  = netz.uew[i][GetKnoten(j)][l];
					uew[j] *= (j==GetWs(GetKnoten(j), l));
				}

				for (j=0; j<=AnzWs; j++)
				{
					fprintf(OutFile, "%d ", (int)(uew[j]*1000.0+0.5));
					uew[j]= 0;
				}
				fprintf(OutFile, "\n\n");
		  	}
		}
	}

	/*
	**	Nun noch die Klassenbeschreibungen ausgeben ...
	*/
	for (k=0; k<AnzKl; k++)
	{
		fprintf(OutFile, "Klasse%d %d %d 0\n",
								k,
								(netz.ankr[k]!=0.0)?OFFEN_KL:GESCHLOSSEN_KL,
								netz.auftr[k]);
	}

	/*
	**	... das wars.
	*/
	fprintf(OutFile, "\n");

	fclose(InFile);

	(void)exit(0);
}
