/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\ts2p.c\t1.4\t94/04/26 13:27:44";

#include <stdio.h>
#include <sys/types.h>
#include <d_netz.h>
#define	EXTERN_KNOTENTYPEN
#include <d_konst.h>

typedef int boolean;

extern FILE *fopen();
extern int	 fclose();
extern char	*strncpy(),
				*strncat();
extern void *calloc(),
				 exit();

extern netzstruktur netz;

extern void InitFirstXX();
extern boolean TestPRE();
extern boolean TestNONPRE();
extern int	GetAnzBe(),
				GetAnzWs();

int	 AnzKl, AnzBe, AnzWs;
int	*firstWs,
		*firstBe;

main(argc, argv)
int argc;
char *argv[];
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

	FILE		*RFile,						/* ResultsFile wurde vom Simulator erzeugt */
				*EFile,						/* PEPSY-Netzbeschreibungsdatei */
				*FFile;						/* FriedhofFile wird fuer den Postfilter erzeugt */
	double	*rho, *mwsl, *lambdaBe, *lambdaWs, *pjBe, *pjWs;
	double	 Rho;
	double	 r, l, w, t, e;
	int		 n, nk, m, i, k;
	char		 RFileName[1024],
				 EFileName[1024];

	if (argc!=2)  	/* keine Argumente, oder zuviele */
	{
		fprintf(stderr, "USAGE: %s dateiname_ohne_prefix\n", argv[0]);
		fprintf(stderr, "oder:  %s -V\n", argv[0]);
		exit(-1);
	}
	if (*argv[1]=='-' && *(argv[1]+1) == 'V')
	{
		fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
		exit(0);
	}
		

	(void)strncpy(RFileName, "r_", (size_t)1023); (void)strncat(RFileName, argv[1], (size_t)1020);
	(void)strncpy(EFileName, "e_", (size_t)1023); (void)strncat(EFileName, argv[1], (size_t)1020);

	FFile = stdout;

	if ((RFile = fopen(RFileName, "r")) == (FILE *)NULL)
	{
		fprintf(stderr, "%s: Kann '%s' nicht oeffnen!\n", argv[0], RFileName);
	}

	if ((EFile = fopen(EFileName, "r")) == (FILE *)NULL)
	{
		fprintf(stderr, "%s: Kann '%s' nicht oeffnen!\n", argv[0], EFileName);
	}

	get_standard(EFile);

	InitFirstXX();

	rho		= (double *)calloc((unsigned)AnzBe+1, sizeof(double));
	mwsl		= (double *)calloc((unsigned)AnzWs+1, sizeof(double));
	lambdaBe	= (double *)calloc((unsigned)AnzBe+1, sizeof(double));
	lambdaWs	= (double *)calloc((unsigned)AnzWs+1, sizeof(double));
	pjBe		= (double *)calloc((unsigned)AnzBe+1, sizeof(double));
	pjWs		= (double *)calloc((unsigned)AnzWs+1, sizeof(double));

	fprintf(FFile, "sim2\n\n");

	fprintf(FFile, "%d %d %d\n\n", netz.typ, netz.knot, netz.klass);

	for (k=0; k<netz.klass; k++)
	{
		for (i=0; i<=AnzBe; i++)
		{
			fscanf(RFile, "%lg %lg %lg ", &rho[i], &lambdaBe[i], &pjBe[i]);
		}
		for (i=0; i<=AnzWs; i++)
		{
			fscanf(RFile, "%lg %lg %lg ", &mwsl[i], &lambdaWs[i], &pjWs[i]);
		}

		for (nk=1; nk<=netz.knot; nk++)
		{
			r = 0.0;	l = 0.0; e = 0.0;
			w = 0.0;	t = 0.0;

			n = (double)GetAnzBe(nk);
			m = (double)GetAnzWs(nk);

			for (i=firstBe[nk]; i<firstBe[nk]+n; i++)
			{
				r += rho[i];
				l += lambdaBe[i];
				e += pjBe[i];
			}
					
			for (i=firstWs[nk]; i<firstWs[nk]+m; i++)
			{
				w += mwsl[i];
				t += lambdaWs[i];
			}

			if (netz.knotentyp[nk-1] == TYP4)
			{
				Rho = 0.0;
			}
			else
			{
				if ((netz.knotentyp[nk-1] == TYP12) || (netz.knotentyp[nk-1] == TYP13))
				{
					Rho = r/GetAnzSE(nk);
				}
				else
				{
					Rho = r/n;
				}
			}

			fprintf(FFile, "%g %g %g %g %g %g %g %g\n",
								l /* /n */,											/* lambda */
								(pjBe[0]!=0.0)?e/pjBe[0]:0.0,					/*    e   */
								(l!=0.0)?      r/l      :0.0,					/* 1/mue	 */
								Rho,													/*  rho   */
								((t != 0.0)&&(l != 0.0))?(r/l)+(w/t):0.0,	/*  mvz   */
								w+r,													/*  maa	 */
								(t != 0.0)?  (w/t):0.0,							/*  mwz	 */
								w														/*  mwsl  */
						);
		}
	}

	(void)fclose(EFile);
	(void)fclose(RFile);
	(void)fclose(FFile);

	(void)exit(0);
}
