/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tput_sim.c\t1.3\t92/08/24 13:00:16";

#include <stdio.h>
#include "d_speicher.h"
#include "d_netz.h"
#define	EXTERN_KNOTENTYPEN
#include "d_konst.h"
#include "sim.h"

extern int dbg;
extern netzstruktur netz;
extern struct prozess **general, *ankunft;
extern int *kapazitaet, sim_dauer, deadlock, block_arrival;

put_sim(fp)
FILE *fp;
{
	register netzstruktur *np = &netz;
	int i,j,k,l;
	double **A, *b, *x;

	if(dbg)
		fprintf(stderr,"put_sim\n");
	
	fprintf(fp,"%d\n",np->knot); /* Knoten  */
	fprintf(fp,"%d\n",np->klass); /* Klass  */
	fprintf(fp,"%c\n","01204"[np->typ]);
	fprintf(fp,"\n");
	if(np->typ == GESCHLOSSEN || np->typ == GEMISCHT)
	{
		for(j=0;j<np->klass;j++) /* # Auftraege pro Klasse */
		{
			fprintf(fp,"%d\n",np->auftr[j]);
		}
	}
	for(i=0;i<np->knot;i++)
	{
		fprintf(fp,"%d\n",np->bede[i]); /* # Bedieneinheiten */
	}
	for(i=0;i<np->knot;i++)
	{
		fprintf(fp,"%d\n",kapazitaet[i]); /* # Warteplaetze */
	}
	if(dbg) fprintf(stderr,"put_sim: Strategien\n");
	for(i=0;i<np->knot;i++)
	{
		/* Strategien */
		switch(np->knotentyp[i])
		{
			case TYP1:  /* FCFS */
			case TYP2:
			case TYP5:
			case TYP6:
			case TYP7:
			case TYP8:
				fprintf(fp,"2\n");
				break;
			case TYP3:   /* PS */
			case TYP12:
			case TYP13:
				fprintf(fp,"8\n");
				break;
			case TYP4:  /* IS */
				fprintf(fp,"9\n");
				break;
			case TYP9:  /* LCFS-PRE */
				fprintf(fp,"6\n");
				break;
			case TYP10: /* FCFS-PRE */
			case TYP14:
			case TYP16:
			case TYP18:
				fprintf(fp,"5\n");
				break;
			case TYP11: /* FCFS-NONPRE */
			case TYP15:
			case TYP17:
			case TYP19:
				fprintf(fp,"1\n");
				break;
			default:
				fprintf(fp,"-1\n");
				break;
		}
	}
	if(dbg) fprintf(stderr,"put_sim: Prozesse\n");
	for(j=0;j<np->klass;j++)
	{
		if((np->typ == OFFEN) || (np->typ == GEMISCHT && np->ankr[j] != 0.0))
		{
			switch(ankunft[j].code)  /* Ankunftsprozesse */
			{
				case 0:	/* automatisch */
					fprintf(fp,"%d\n",0);
					fprintf(fp,"%g\n",ankunft[j].wert1);
					fprintf(fp,"%g\n",ankunft[j].wert2);
					break;
				case 2:	/* automatisch, version 2 */
					fprintf(fp,"%d\n",6);
					fprintf(fp,"%g\n",ankunft[j].wert1);
					fprintf(fp,"%g\n",ankunft[j].wert2);
					break;
				case 1: /* lognormal */
					fprintf(fp,"%d\n",3);
					fprintf(fp,"%g\n",ankunft[j].wert1);
					fprintf(fp,"%g\n",ankunft[j].wert2);
					fprintf(fp,"%g\n",ankunft[j].wert3);
					fprintf(fp,"%g\n",ankunft[j].wert4);
					break;
				default:
					fprintf(fp," --- %d ---\n",ankunft[j].code);
					break;
			}
		}
		for(i=0;i<np->knot;i++)
		{
			switch(general[i][j].code)  /* Bedienprozesse */
			{
				case 0:	/* auotmatisch */
					fprintf(fp,"%d\n",0);
					fprintf(fp,"%g\n",general[i][j].wert1);
					fprintf(fp,"%g\n",general[i][j].wert2);
					break;
				case 2: /* automatisch (version 2) */
					fprintf(fp,"%d\n",6);
					fprintf(fp,"%g\n",general[i][j].wert1);
					fprintf(fp,"%g\n",general[i][j].wert2);
					break;
				case 1: /* lognormal */
					fprintf(fp,"%d\n",3);
					fprintf(fp,"%g\n",general[i][j].wert1);
					fprintf(fp,"%g\n",general[i][j].wert2);
					fprintf(fp,"%g\n",general[i][j].wert3);
					fprintf(fp,"%g\n",general[i][j].wert4);
					break;
				default:
					fprintf(fp,"+++ %d +++\n",general[i][j].code);
					break;
			}
		}
	}
	/* Berechnen der Besuchshaeufigkeiten */

	if((A = d_2dim_alloc(np->knot,np->knot)) == (double **) 0) mall_err("put_sim(1)");
	if((b = d_1dim_alloc(np->knot)) == (double *) 0) mall_err("put_sim(2)");
	if((x = d_1dim_alloc(np->knot)) == (double *) 0) mall_err("put_sim(3)");
	for(i=0;i<np->klass;i++)
	{
		for(j=1;j<=np->knot;j++) /* rechte Seite  und Matrix aufbauen */
		{
			b[j-1] = -np->uew[0][j][i];
			for(k=1;k<=np->knot;k++) /* rechte Seite  und Matrix aufbauen */
			{
				A[j-1][k-1] = np->uew[k][j][i];
			}
			A[j-1][j-1] -= 1.0;
		}
		if(gauss(A,x,b,np->knot,1e-20) < 0)
		{
			fprintf(stderr,"Klasse %d: Matrix singulaer\n",i+1);
			exit(1);
		}
		for(j=0;j<np->knot;j++)
		{
			np->besh[j][i] = x[j]; 
		}
		if (dbg) fprintf(stderr,"\n");
	}

	if(dbg) fprintf(stderr,"put_sim: Uews\n");
	for(k=0;k<np->klass;k++)	/* Uebergangswahrscheinlichkeiten */
	{
		if(np->typ == OFFEN || (np->typ == GEMISCHT && np->ankr[k] > 0.0))
		{
			if(dbg) fprintf(stderr,"put_sim: offene Klasse\n");
			for(j=1;j<np->knot;j++)
			{
				fprintf(fp,"%g\n",np->uew[0][j][k]);
			}
			for(i=1;i<=np->knot;i++) 
			{
				for(j=0;j<np->knot;j++)
				{
					fprintf(fp,"%g\n",np->uew[i][j][k]);
				}
			}
		}
		else
		{
			if(dbg) fprintf(stderr,"put_sim: geschlossene Klasse\n");
			for(i=1;i<=np->knot;i++) 	/* Uebergangswahrscheinlichkeiten */
			{
				if(np->uew[i][0][k] >= 0.0)
				{
					for(l=1;l<=np->knot;l++)
					{
						np->uew[i][l][k] += np->uew[i][0][k] * np->uew[0][l][k];
					}
				}
				for(j=1;j<np->knot;j++)
				{
					fprintf(fp,"%g\n",np->uew[i][j][k]);
				}
			}
		}
		fprintf(fp,"\n");

	}
	if(sim_dauer != 0)
	{
		fprintf(fp,"%d\n%d\n",1,sim_dauer);
	}
	else
	{
		fprintf(fp,"0\n");
	}
	fprintf(fp,"%d\n%d\n",deadlock, block_arrival);
	if(dbg)fprintf(stderr,"put_sim: besh\n");
	for(j=0;j<np->klass;j++)
	{
		for(i=0;i<np->knot;i++)
		{
			fprintf(fp,"%lg ",np->besh[i][j]);
		}
		fprintf(fp,"\n");
	}
	fprintf(fp,"\n");
}


dump_ank(fp)
FILE *fp;
{
	int j;
	for(j=0;j<netz.klass;j++)
	{
		fprintf(fp,"dump_ank: ");
		fprintf(fp,"%d  ",ankunft[j].code);
		fprintf(fp,"%g  ",ankunft[j].wert1);
		fprintf(fp,"%g  ",ankunft[j].wert2);
		fprintf(fp,"%g  ",ankunft[j].wert3);
		fprintf(fp,"%g\n",ankunft[j].wert4);
	}
}

dump_gen(fp)
FILE *fp;
{
	int i,j;

	for(i=0;i<netz.knot;i++)
	{
		for(j=0;j<netz.klass;j++)
		{
			fprintf(fp,"dump_gen : ");
			fprintf(fp,"%d  ",general[i][j].code);
			fprintf(fp,"%g  ",general[i][j].wert1);
			fprintf(fp,"%g  ",general[i][j].wert2);
			fprintf(fp,"%g  ",general[i][j].wert3);
			fprintf(fp,"%g\n",general[i][j].wert4);
		}
		fprintf(fp,"\n");
	}
}
