/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  D-91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tcheck.c\t1.6\t94/05/11 13:28:44";

#include <stdio.h>

#include "d_netz.h"
#include "d_speicher.h"
#include "d_konst.h"
#include "sim.h"

#define max(a,b) ((a)>(b))?(a):(b)	/* um maximalen Fehler zu ermitteln! */
#define OK 			0
#define WARNING	1
#define ERROR		2

#define FATAL		-1			/* sollte nur mit exit(FATAL) benutzt werden	*/

extern void get_standard();
extern void get_mem();
extern void get_sim();

extern double **mem_ank_quadvar;
extern struct prozess **general;

/****		Forwarddeklarationen	*/
int pc_uewtest();
int pc_simtest();
int pc_memtest();

int dbg	=	0;
int verb	=	1;

netzstruktur netz;
netzstruktur *np = &netz;
FILE *fpein;

main(argc,argv)
int argc;
char *argv[];
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

	int i, j;
	int fehler=OK, error=OK;			/* kein Fehler */
	char *c, eingabedatei[255];

	/*     Initialisierungen
	*/
	strcpy(eingabedatei,"e_");

	/*			Uebernahme der Argumente derEingabezeile
	*/
	if (argc==1)
	{
		fprintf(stdout,"Usage:\n%s [Optionen] [Dateiname]\n",argv[0]);
		exit(FATAL);
	}

	i=1;
	while ((*argv[i]=='-') && (argc>=i))
	{
		c = argv[i];
		c++;
		switch(*c) {
			case 'd'	:	c++;
							if (*c!='\0') {
								dbg = atoi(c);
							}
							else {
								dbg = 1;
							}
							break;
			case 'V'	:	fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
							exit(0);
							break;
			case 's'	:	verb=0;
							break;
			default	:	fprintf(stderr,"%s: Unbekannte Option '%s'. (Wird ignoriert!)\n",argv[0],argv[i]);
		}
		i++;
	}

	i=1;
	while ((*argv[i]=='-')&&(i<=argc)) {
		i++;
	}

	if (argc<i)
	{
		fprintf(stderr,"%s: Welche Datei soll untersucht werden?\n",argv[0]);
		exit (FATAL);
	}

	if ( *argv[i] == 'e' && *(argv[i]+1) == '_')
	{
		strcpy(eingabedatei, argv[i]);
	 }
	 else
	 {
		strcat(eingabedatei,argv[i]);
	 }

	if (dbg)
	{
		fprintf(stdout,"%s:\n Dateiname: '%s'\n",argv[0],eingabedatei);
		fprintf(stdout," Debugniveau: %d\n",dbg);
	}

	if ((fpein= fopen(eingabedatei,"r"))==NULL)
	{
		fprintf(stderr,"%s: Kann die Eingabedatei '%s' nicht oeffnen!\n",argv[0],eingabedatei);
		exit(FATAL);
	}

	/* Daten aus der Eingabedatei holen und in globalen Variablen ablegen */

	get_standard(fpein);

	get_mem(fpein);

	get_sim(fpein);

	fclose(fpein);

	/* Vergleich der gefundenen Daten und evtl. Fehlerausgaben */

	fehler = OK;

	error = pc_memtest(); fehler = max(fehler, error);
	error = pc_simtest(); fehler = max(fehler, error);
	error = pc_uewtest(); fehler = max(fehler, error);

	return (fehler);
}

int pc_uewtest()
{
	/* kirschni 19.03.92

		testet ob die Summe der UEW fuer jeden Knoten <1.0 ist

		returnwerte :	OK (0)		kein Fehler aufgetreten
							WARNING (1)	UEW kleiner als 1.0 (Weiterarbeiten kann moeglich sein)
							ERROR	(2)	UEW groesser als 1.0 (Muell)
	*/
	int 		i,
				j,
				k;
	int		summe;
	int		fehler;
	int		status = OK;


	if (verb)
		fprintf(stdout,"\nUeberpruefung der Uebergangswahrscheinlichkeiten:\n");

	if (np->haeuf)
	{
		for (j=0; j<(np->klass); j++)
		{
			if (verb)
				fprintf(stdout,"Klasse %d:\n",j);

			for (i=0; i<=(np->knot); i++)
			{
				if (verb)
					fprintf(stdout,"   Knoten %d: ",i);

				fehler	= OK;
				summe		= 0.0;

				for (k=0; k<=(np->knot); k++)
					summe += (int)(1000.0*np->uew[i][k][j]+0.5);

				if  (summe > 1000)
				{
					if (verb)
						fprintf(stdout, "Summe der UEW > 1.0!\n");
					fehler = ERROR;
				}
				else
				{
					if (summe != 1000)
					{
						if (verb)
							fprintf(stdout, "Summe der UEW < 1.0!\n");
						fehler = WARNING;
					}
				}

				if (fehler==OK) {
					if (verb)
						fprintf(stdout,"OK!");
				}

				if (verb)
					fprintf(stdout,"\n");

				status = max(status,fehler);
			}
		}
	}
	return(status);
}

int pc_simtest()
{	/*************				
		Es werden die Globalen Strukturen general und netz verglichen.

		Bei general.code 0 und 6 :		Mittelwert- und Standardabweichungsvergleich

		returnwerte :	OK (0)		kein Fehler aufgetreten
							WARNING (1)	keine Ueberpruefung moeglich
							ERROR	(2)	Fehler beim Vergleich gefunden

	*/
	int i, j;
	int fehler=OK, status=OK;   /* kein fehler! */

	if (verb) fprintf(stdout,"\nUeberpruefung des Simulationsblockes:\n");

	if (dbg>4) {
		for (j=0; j<(np->klass); j++) {
			for (i=0; i<(np->knot); i++) {
				fprintf(stderr,"Knoten%d : general[%d][%d].wert2=%g\n",i,i,j,general[i][j].wert2);
			}
		}
	}

	for (j=0; j<(np->klass); j++) {
		if (verb) fprintf(stdout,"Klasse %d:\n",j);
		for (i=0; i<(np->knot); i++) {
			if (verb) fprintf(stdout,"   Knoten %d: ",i);
			if ((general[i][j].code != 0) && (general[i][j].code != 6)) {
				if (verb) fprintf(stdout,"keine Ueberpruefung moeglich (Typ: %d)!\n", general[i][j].code);
				fehler = WARNING;
			}
			else {
				fehler = OK;
				if (general[i][j].wert2 != np->quadvar[i][j]) {
					if (verb) fprintf(stdout,"FEHLER bei der Varianz ");
					fehler=ERROR;
				}
				if (general[i][j].wert1 != np->bedr[i][j]) {
					if (fehler!=OK) {
						if (verb) fprintf(stdout,"und ");
					}
					else {
						if (verb) fprintf(stdout,"FEHLER ");
					}
					if (verb) fprintf(stdout,"beim Mittelwert");
					fehler=ERROR;
				}
				if (fehler==OK) {
					if (verb) fprintf(stdout,"OK");
				}
				if (verb) fprintf(stdout,"!\n");
			}
			status = max(status,fehler);
		}
	}
	return(status);
}

int pc_memtest()
{
	/*************		
		Es werden die Globalen Strukturen mem_ank_var und netz verglichen.

		Vergleich der Standardabweichungen

		returnwerte :	OK (0)		kein Fehler aufgetreten
							WARNING (1)	keine Ueberpruefung moeglich
							ERROR	(2)	
	*/
	int i, j;
	int status=OK;   /* kein fehler! */
	int fehler=OK;

	if (verb) fprintf(stdout,"\nUeberpruefung des MEM-Blockes:\n");

	status = OK;

	for (j=0; j<np->klass; j++)
	{
		fehler=OK;
		if (verb) fprintf(stdout,"Klasse %d: ",j);
		for (i=0; i<np->knot; i++)
		{
			/* Testen ob Standardwerte(d.h. alle Werte=1) */
			if (mem_ank_quadvar[i][j]!=1.0)
			{
				status = WARNING;
				fehler = WARNING;
			}
		}
		if (verb)
		{
			if (fehler!=OK) 
				fprintf(stdout," Vergleich mit SIM-Block notwendig!\n");
			else
				fprintf(stdout," OK.\n");
		}
	}

	if ((verb)&&(status!=OK))
	{
		fprintf(stdout,"ACHTUNG! Ankunftsprozess im Mem-Block enthaelt nicht nur Standardwerte!\n");
		fprintf(stdout,"         Vergleich mit Zusatzblock SIM notwendig!\n");
	}

	return (status);
}
