/*

					 P E P S Y
			 
	Performance and Evaluation Prediction System

 	Copyright 1987, 1988

		 Universitaet Erlangen-Nuernberg,
		 Lehrstuhl fuer Betriebssysteme
		 Martensstrasse 1

	     8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
  	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.
*/

#define PHELP_C

#include <stdio.h>
#include <curses.h>

#ifdef SYS_V
#include <term.h> 
#endif

#include <string.h>
#include "libpepsy.h"
#include "libohs.h"

#ifndef SYS_V

#define BOLD_ON standout()
#define REVERSE_ON standout()
#define UNDERLINE_ON standout()
#define NORMAL_ON standend()

#else

#define BOLD_ON attrset(A_BOLD)
#define REVERSE_ON attrset(A_REVERSE)
#define UNDERLINE_ON attrset(A_UNDERLINE)
#define NORMAL_ON attrset(A_NORMAL)

#endif

		int		dbg;
extern	int		SccsRelease;
extern	int		SccsLevel;
static	char	*SccsId="@(#)PEPSY:\tphelp.c\t1.5\t94/01/27 12:46:13";


struct OH_status status; 
struct OH_file helpfile;

void textout();

void clrscr()
{
	clear();
	refresh();
}

void quit(i)
int i;
{
	BOLD_ON;
	addstr("\n\nPress any key to quit... ");
	refresh();
	(void)getchar();
	endwin();
	exit(i);
}


void wait_for_key()
{
	BOLD_ON;
	addstr("\n\nPress any key to continue... ");
	refresh();
	(void)getchar();
	clear();
	refresh();
}

void main(argc, argv)
int argc;
char *argv[];
{
	char text[1024];
	char **files;
	char *helpfilename = "pepsy";
	char *newname;
	char **index;
	int *depth;
	int i,j,k;
	char line[1024];
	int args_ok=1;
	WINDOW *scr;
	int act_lin;
	int mode_kw=0,mode_hl=0,mode_toc=0,mode_ind=0,mode_srt=0,mode_cr=0, mode_nn=0, mode_fn=0, mode_dump=0, mode_version=0;
	int c;
	FILE *fp, *fp_dump;
	char dumpname[1024];
	int mindepth;

	extern char *optarg;
	extern int optind;

	status.eohelp=OH_more_text;
	status.textlen=0;
	status.textmode=OH_normal;
	status.hldepth=0;
	status.indent=0;

	while((c=getopt(argc, argv, "vVd:D:k:h:itc:so:f:")) != -1)
		switch(c) {
		case 'v':	mode_version = 1;
					break;

		case 'V':	printf("%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
					exit(0);
					break;

		case 'd':	strcpy(dumpname,optarg);
					mode_dump=1;
					break;

		case 'D':	dbg = atoi(optarg);
					break;

		case 'k':	strcpy(line,optarg);
					mode_kw=1;
					break;

		case 'h':	strcpy(line,optarg);
					mode_hl=1;
					break;

		case 'i':	mode_ind=1;
					break;

		case 't':	mode_toc=1;
					break;

		case 'c':	mode_cr=1;
					break;

		case 's':	mode_srt=1;
					break;

		case 'f':	newname = strdup(optarg);
					mode_fn=1;
					break;

		case 'o':	newname = strdup(optarg);
					mode_nn=1;
					break;

		case '?':	args_ok=0;

		}

	if(getenv(LANGUAGEENV)==NULL || getenv(HOMEPATH)==NULL)
	{
		printf("Please set the environment for %s and %s\n",LANGUAGEENV,HOMEPATH);
		exit(3);
	}


	/* no arguments -> error */
	if(argc==1) args_ok=0;
	/* no options -> keyword mode */
	if(mode_kw+mode_hl+mode_cr+mode_ind+mode_toc+mode_srt == 0) mode_kw=1;
	/* more than one option */
	if(mode_kw+mode_hl+mode_toc+mode_ind+mode_cr > 1) args_ok=-1;
	/* -s with -k, -h or -c */
	if(mode_srt==1 && mode_kw+mode_hl+mode_cr>0) args_ok=-2;
	/* -s as only option */
	if(mode_srt==1 && mode_kw+mode_hl+mode_ind+mode_toc+mode_cr==0) args_ok=-3;
	/* -d must be alone */
	if(mode_dump==1 && mode_ind+mode_toc+mode_cr+mode_srt+mode_nn>0)
		args_ok=-4;

	if(mode_ind+mode_toc==1)
		if(optind==argc)
			strcpy(text,"*");
		else	strcpy(text,argv[argc-1]);

	if(mode_nn==1)
	{
		switch(mode_cr){
		case 0:
			printf("-o only useful with -c; ignored.\n");
			break;
		case 1:
			helpfilename = strdup(newname);
		}
	}
	if(mode_fn==1)
	{
		int found=0;

		switch(mode_cr){
			case 0:
				sprintf(line,"%s/infos/%s",getenv(HOMEPATH),newname);
				if((fp=fopen(line,"r"))!=NULL)	found=1;
				sprintf(line,"%s/infos/%s.%.2s",getenv(HOMEPATH),newname,getenv(LANGUAGEENV));
				if((fp=fopen(line,"r"))!=NULL)		found=2;
				sprintf(line,"%s/infos/%s.hlp.%.2s",getenv(HOMEPATH),newname,getenv(LANGUAGEENV));
				if((fp=fopen(line,"r"))!=NULL)		found=3;

				switch(found){
					case 0:
							printf("File not found: %s\n",newname);
							args_ok = 0;
							break;
					case 3:
							helpfilename = strdup(newname);
							break;
					case 2:
							newname[strlen(newname)-4] = '\0';
							helpfilename = strdup(newname);
							break;
					case 1:
							newname[strlen(newname)-4-3] = '\0';
							helpfilename = strdup(newname);
							break;
				}
				break;
			case 1:
				printf("-f not useful with -c; ignored.\n");
				break;
		}
	
	}
	
	if(args_ok<=0){
		switch(args_ok) {
		case -1: printf("Too many options !\n"); break;
		case -2: printf("-s only useful with -t or -i\n"); break;
		case -3: printf("-s must be used with -t or -i\n"); break;
		case -4: printf("Can't use -d with other options except -f\n"); break;
		}
		printf("\nUsage:\n\n");
		printf(" %s options\n\n",argv[0]);
		printf("Where options consists of :\n",argv[0]);

		printf(" -k HELPTEXT            : give help for keyword 'HELPTEXT' (default)\n");
		printf(" -h HELPTEXT            : give help for headline 'HELPTEXT'\n");
		printf(" -i [MASK]              : list index of all keywords\n                          containing the string 'MASK'\n                          (all if MASK is empty)\n");
		printf(" -t [MASK]              : list table of contents of all headlines\n                          containing the string 'MASK'\n                          (all if MASK is empty)\n");
		printf(" -s                     : sort index/toc alphabetically\n");
		printf(" -d filename            : dump help into file\n");
		printf(" -c FILE1 [FILE2] [...] : create helpfile consisting of files FILE1, FILE2, ...\n");
		printf(" -f filename            : read help from different file\n                          (default 'pepsy')\n");
		printf(" -o filename            : vary the name of the generated helpfile\n                          (default 'pepsy')\n");
		printf(" -V                     : print version number of %s\n",argv[0]);
		printf(" -v                     : print list of files contained in helpfile\n");
		printf("\nExample:\n\n");
		printf(" %s -h Summary  : Give help regarding headline 'Summary'\n",argv[0]);
		printf(" %s -d testhelp : Write all help to file 'testhelp'\n",argv[0]);
		printf(" %s -is System  : List all keywords containing 'System' (alphabet. sorted).\n\n",argv[0]);
		exit(1);
	}

	/* We have to initialize curses for the program */
	
	if(mode_dump==0 && mode_cr==0 && mode_version==0)
	{
		scr=initscr();
		cbreak();

		clrscr();
	}
	
	/* ok, let's start: */
	inithelp();

	if(mode_cr==1){  	   /* create Helpfile */
		files=(char **)malloc((argc-1)*sizeof(char *));
		if(files==NULL){
			printf("out of mem in main\n");
			exit(1);
		}
		for(i=0;optind<=argc;optind++){
			files[i++]=strdup(argv[optind-1]);
		}
		files[argc-1]=NULL;

		if(convert(files,helpfilename)==OH_error){ /* so lets make it ! */
			printf("%s\n",ohs_error(NULL));
			printf("Couldn't create helpfile\n");
			exit(1);
			} else {
			sprintf(text,"%s.hlp.%.2s",helpfilename,getenv(LANGUAGEENV));
			printf("Helpfile %s created properly !\n",text);
		}
		exit(0);
	}

	helpfile=openhelp(helpfilename);

	if(helpfile.fp==NULL){
		endwin();
		printf("Couldn't open %s\n", helpfilename);
		exit(1);
	}

	if(mode_version==1){	/* print Helpfileversion */
		printf("\nVersion of helpfiles:\n\n");
		rewind(helpfile.fp);
		fscanf(helpfile.fp,"%d",&j);	/* read # of helpfiles */
		for(i=0;i<j;i++)
		{
			fscanf(helpfile.fp,"%s",text);
			text[strlen(text)-1]='\0';
			printf("%2d. %s ",i+1,text);
			do {
				k=fgetc(helpfile.fp);
			}while(k!=':');

			for(k=0;k<(20-strlen(text));k++)
				printf(" ");

			printf(": Date of last modification: ");
			do {
				k=fgetc(helpfile.fp);
				printf("%c",k);
			}while(k!='\n');
		}
		printf("\n");
		exit(0);
	}

	if(mode_ind==1){
		index=(char **)malloc((helpfile.keyw_count+1)*sizeof(char *));

		i=getindex(text,index,helpfile,(mode_srt==1) ? TRUE : FALSE);
	
		REVERSE_ON;
		addstr("\nINDEX:\n\n");
		NORMAL_ON;
		act_lin=3;

		for(j=0;j<i;j++){
			addstr(index[j]);
			addch('\n');
			act_lin++;
			if(act_lin>LINES-4){
				wait_for_key();
				NORMAL_ON;
				act_lin=0;
			}
		}

		refresh();

		for(j=0;j<i;j++)
			free(*index[j]);
		free(index);

		quit(closehelp(helpfile));
	}

	if(mode_toc==1){
		index=(char **)malloc((helpfile.hl_count+1)*sizeof(char *));
		depth=(int *)malloc((helpfile.hl_count+1)*sizeof(int));

		k=gettoc(text,index,depth,helpfile,(mode_srt==1) ? TRUE : FALSE);

		REVERSE_ON;
		addstr("\nTABLE OF CONTENTS:\n\n");
		NORMAL_ON;
		act_lin=3;

		for(j=0;j<k;j++){
			addstr(index[j]);
			addch('\n');
			act_lin++;
			if(act_lin>LINES-4){
				wait_for_key();
				NORMAL_ON;
				act_lin=0;
			}
		}
        
		refresh();

		for(j=0;j<k;j++)
			free(*index[j]);

		free(index);	free(depth);

		quit(closehelp(helpfile));
	}

	if(mode_dump==1){
		if(strcmp(dumpname,"-")==0)
		{
			fp_dump = stdout;
		}
		else
		{
			if((fp_dump = fopen(dumpname,"wb"))==NULL)
			{
				printf("Couldn't open dumpfile !\n");
				exit(1);
			}
		}

		index=(char **)malloc((helpfile.hl_count+1)*sizeof(char *));
		depth=(int *)malloc((helpfile.hl_count+1)*sizeof(int));
		strcpy(text,"*");

		k=gettoc(text,index,depth,helpfile, FALSE);
		mindepth = 1000;

		for(i=0;i<k;i++){
			if(mindepth>=depth[i]){
				mindepth = depth[i];
			}
		}
		for(i=0;i<k;i++){
			if(depth[i]==mindepth){
				textout(index[i], 0, fp_dump); 
			}
		}

		fclose(fp_dump);

		exit(closehelp(helpfile));
	}

	strcpy(line,argv[argc-1]);

	textout(line, mode_kw, NULL);

	quit(closehelp(helpfile));
}

void textout(help, mode, outfile)
char *help;
int mode;
FILE *outfile;

{

	char text[1025], word[512];
	int act_col, act_lin;
	int k,l,m;
	int hlindent;

	status.eohelp = OH_more_text;

	if(outfile != NULL)
	{
		COLS = 80;
	}

	act_col=act_lin=0; /* used to keep track of actual columnpos. and linenumber */
	do {	
		status = readhelp(help,text,1024,(mode==1) ? OH_keyword : OH_headline ,helpfile);

		if(status.eohelp==OH_more_topic){
			if(outfile == NULL)
			{
				BOLD_ON;
				addstr("\n\n\nThere's more helptext... ");
				refresh();
				(void)getchar();
				clrscr();
			}
			act_col=0;
			act_lin=0;
		}

		if(status.textmode==OH_headline){
			if(outfile == NULL)
			{
				NORMAL_ON;
				addstr("\n\n");
				for(k=0;k<status.hldepth*INDENTMULT;k++){
					addch(' ');
					act_col++;
				}
				REVERSE_ON;
				addstr(text);
				addstr("\n\n");
				NORMAL_ON;
				for(k=0;k<status.hldepth*INDENTMULT;k++){
					addch(' ');
				}
				refresh();
				act_col=status.hldepth*INDENTMULT;
				act_lin+=4;
				hlindent=status.hldepth;
			}
			else
			{
				fprintf(outfile,"\n\n");
				for(k=0;k<status.hldepth*INDENTMULT;k++){
					fprintf(outfile," ");
					act_col++;
				}
				fprintf(outfile,"*** %s ***",text);
				fprintf(outfile,"\n\n");
				for(k=0;k<status.hldepth*INDENTMULT;k++){
					fprintf(outfile," ");
				}
				act_col=status.hldepth*INDENTMULT;
				act_lin+=4;
				hlindent=status.hldepth;
			}
		}
	
		if(status.textmode==OH_keyword){
			if(outfile == NULL)
			{
				if(act_col+status.textlen>=COLS-1){
					act_col=0;
					addch('\n');
					act_lin++;

					if(act_lin>LINES-4) {
						wait_for_key();
						act_lin=0;
					}

					for(k=0;k<status.indent+hlindent*INDENTMULT;k++){
						addch(' ');
						act_col++;
					}
				}
				UNDERLINE_ON;
				addstr(text);
				refresh();
				act_col+=strlen(text);
			}
			else
			{
				if(act_col+status.textlen>=COLS-1){
					act_col=0;
					fprintf(outfile,"\n");
					act_lin++;

					for(k=0;k<status.indent+hlindent*INDENTMULT;k++){
						fprintf(outfile," ");
						act_col++;
					}
				}
				fprintf(outfile,">%s<",text);
				act_col+=strlen(text)+2;
			}
		}

		if(status.textmode==OH_bold){
			if(outfile == NULL)
			{
           		if(act_col+status.textlen>=COLS-1){
					act_col=0;
					addch('\n');
					act_lin++;

					if(act_lin>LINES-4) {
						wait_for_key();
						act_lin=0;
					}

					for(k=0;k<status.indent+hlindent*INDENTMULT;k++){
						addch(' ');
						act_col++;
					}
				}
				BOLD_ON;
				addstr(text);
				act_col+=strlen(text);
				refresh();
			}
			else
			{
           		if(act_col+status.textlen>=COLS-1){
					act_col=0;
					fprintf(outfile,"\n");
					act_lin++;

					for(k=0;k<status.indent+hlindent*INDENTMULT;k++){
						fprintf(outfile," ");
						act_col++;
					}
				}
				fprintf(outfile,"%s",text);
				act_col+=strlen(text);
			}
		}

		if(status.textmode==OH_normal){
			if(outfile == NULL)
			{
				NORMAL_ON;
			}

			l=0;
			for(k=0;k<=status.textlen;k++){
				if(text[k]!=' ' && text[k]!='\n' && text[k]!='\0'){
					word[l++]=text[k];
				}
				else
				{
					word[l++]=text[k];
					word[l++]='\0';

					if((act_col+strlen(word)>=COLS-1)||(word[0]=='\n')){
						act_col=0;

						if(outfile == NULL)
							addch('\n');
						else
							fprintf(outfile,"\n");

						act_lin++;
			
						if(outfile == NULL)
						{
							if(act_lin>LINES-4) {
								wait_for_key();
								act_lin=0;
								NORMAL_ON;
							}
						}

						for(m=0;m<status.indent+hlindent*INDENTMULT;m++){
							if(outfile == NULL)
								addch(' ');
							else
								fprintf(outfile," ");
							act_col++;
						}
						if(word[0]=='\n') strcpy(word,"");
					}

					act_col+=strlen(word);

					if(outfile == NULL)
						addstr(word);
					else
						fprintf(outfile,"%s",word);

					if(word[strlen(word)-1]=='\n') act_lin++;
					if(outfile == NULL)
					{
						if(act_lin>LINES-4) {
							wait_for_key();
							act_lin=0;
							NORMAL_ON;
						}
					}	

					l=0;
				} /* else */
			} /* for */
			if(outfile == NULL)
				refresh();
		} /* if  normal_mode*/

		if(status.eohelp==OH_eohelp){
			act_col=0;
			act_lin=0;
			if(outfile!=NULL)
			{
				fprintf(outfile,"\n");
			}
		}

	} while(status.eohelp != OH_eohelp && status.textmode != OH_error);
}

