/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/


#include <stdio.h>
#include <ctype.h>
#include "d_netz.h"
#include "d_konst.h"

extern char *getenv();

static char *SccsId="@(#)PEPSY:\tpinfo.c\t1.15\t93/08/24 14:34:43";

static char* PEPSYHOME;
static char  DOC[2048];
static char  GRENZEN_ALLE[2048];
#define INFOS_GRENZENALL "/infos/grenzenall.txt"

#define USAGE0				spr("usage0")
#define USAGE1				spr("usage1")
#define GENERAL_INFO		spr("geninf")
#define VERF_NAME			spr("v_name")
#define DOMAIN				spr("domain")
#define OFFENE_NETZE		spr("offenN")
#define GESCHLOSSENE_NETZE	spr("geschN")
#define GEMISCHTE_NETZE		spr("gemisN")
#define OFF_O_GESCHL_NETZE	spr("offgeN")
#define MIT_MAX				spr("mitmax")
#define KNOTEN				spr("knoten")
#define UND					spr("und___")
#define KLASSEN				spr("klasse")
#define MENGE_KNOTENTYPEN	spr("menknt")
#define KEINE				spr("keine_")
#define WEITERE_EINSCHRAENK spr("limits")
#define UEW_ANGEBEN			spr("uewang")
#define BESH_ANGEBEN		spr("besang")
#define MMM_BEDRATEN0		spr("mmmbr0")
#define MMM_BEDRATEN1		spr("mmmbr1")
#define MIN_QUAD_VAR		spr("min_qv")
#define SEIN				spr("sein__")
#define SCOV_INTERVALL		spr("sqovin")

#define SEE_ALSO			spr("seeals")
#define LITERATUR        	spr("litera")
#define BESCHREIBUNG		spr("beschr")
#define IMPLEMENTATION		spr("implem")
#define BEMERKUNGEN			spr("bemerk")

#define M_LIT				0x01
#define M_XREF				0x02
#define M_IMPL				0x04
#define M_REM				0x10
#define M_DESC				0x20

#define NO_LIT				!(paragraphs & M_LIT)
#define NO_XREF				!(paragraphs & M_XREF)
#define NO_IMPL				!(paragraphs & M_IMPL)
#define NO_REM				!(paragraphs & M_REM)
#define NO_DESC				!(paragraphs & M_DESC)


#define OK		0
#define FATAL	2

/*
   gibt Information ueber ein Verfahren aus
   							
*/

int 	dbg = 0;       	/* debug - niveau			*/
int 	debug = 0;       	/* debug - niveau			*/

static	char	verfahren[50];
static	char	voller_name[1024];
static	char	datensatz[1024];
static	char	namenspuffer[1024];
static	char	zeile[2048];
static	char	infodatei[2048];
static	int		gefunden;
static	int		netztyp;
static	int		knot;
static	int		klass;
static	int		knottyp;
static	char	char_uew;
static	char	char_besh;
static	char	zusatzblock[50];
static	char	bed_bedr[50];
static	char	bed_quadvar[50];
static	int		bed_bedeinh;


main(argc,argv)
int	argc;
char	*argv[];
{

	/*		Sccs support from SccsLogging.c			*/	
	extern	int		SccsRelease;
	extern	int		SccsLevel;

	extern	int		optind;
	extern	char	*optarg;

			FILE 	*fopen(),
					*fpin,
					*fpout,
					*fpv;
			int		c,
					errflg=0,
					besch,
					i,
					j;
			double	x,
					y;
			int		skipmode;
	unsigned int	mask, 
					paragraphs;



	if (p_lang_init(argv[0]) != 0)  /* PEPSY-NLS Support */
		exit (FATAL);

	if((PEPSYHOME = (char *)getenv("PEPSYHOME")) == NULL)
	{
		fprintf(stderr,"pinfo: Environment Variable PEPSYHOME nicht definiert\n");
		exit(FATAL);
	}

	strcpy(DOC,PEPSYHOME);
	strcat(DOC,"/doc/");
	strcpy(GRENZEN_ALLE,PEPSYHOME);
	strcat(GRENZEN_ALLE,INFOS_GRENZENALL);

	paragraphs |= M_IMPL | M_XREF | M_DESC | M_LIT | M_REM ;

	while(( c= getopt(argc,argv,"Vqbilrxd:")) != EOF)
	{
		switch(c)
		{
			case 'V':	fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
						exit(OK);
						break;
			case 'd':	if((dbg = atoi(optarg)) <= 0)
							dbg = 1;
						break;
			case 'i':	paragraphs &= ~M_IMPL;
						break;
			case 'x':	paragraphs &= ~M_XREF;
						break;
			case 'b':	paragraphs &= ~M_DESC;
						break;
			case 'l':	paragraphs &= ~M_LIT;
						break;
			case 'r':	paragraphs &= ~M_REM;
						break;
			case 'q':	paragraphs &= ~M_IMPL & ~M_XREF & ~M_DESC & ~M_LIT & ~M_REM ;
						break;
			default:	errflg=1;
						break;
		}
	}

	if (errflg)
	{
		fprintf(stderr, "%s\n%s\n", USAGE0, USAGE1);
		exit(FATAL);
	}

	if((fpv = fopen(GRENZEN_ALLE,"r")) == NULL)
	{
		fprintf(stderr,"pinfo: Datei %s nicht lesbar\n",GRENZEN_ALLE);
		exit(FATAL);
	}

	if ((fpout = popen("more", "w")) == NULL)
	{
		fpout = stdout;
	}

	if (dbg)
	{
		fprintf(stderr, "pinfo: paragraphs=0x%x\n", paragraphs);
	}

	if(argc - optind == 0)
	{
		/*	allgemeine Informationen ausgegeben, d.h. alle Verfahren listen
		*/

		fprintf(fpout, "\n%s\n\n", GENERAL_INFO);

		nextline(fpv, datensatz, 1024);
		nextline(fpv, datensatz, 1024);

		i=0;

		while (nextline(fpv, datensatz, 1024) != EOF)
		{
			sscanf(datensatz, "%s ", namenspuffer);
			fprintf(fpout, "%-16s", namenspuffer);
			i++;
			if (i%5 == 0)
				fprintf(fpout, "\n");
		}

		fprintf(fpout, "\n");
	}
	else
	{
		while(argc - optind)
		{
			strcpy(verfahren,argv[optind++]);

			get_verfahren(verfahren,voller_name);

			/* suche Datensatz in GRENZEN_ALLE */

			rewind(fpv);

			gefunden = 0;

			/*
			nextline(fpv,datensatz,1024);
			nextline(fpv,datensatz,1024);
			*/

			while ( gefunden != 1 && nextline(fpv,datensatz,1024) != EOF)
			{
				/* Datensatz einlesen */

				sscanf(datensatz,"%s %o %d %d %o %c %c %s %s %s %d",
									namenspuffer,
									&netztyp,
									&knot,
									&klass,
									&knottyp,
									&char_uew,
									&char_besh,
									zusatzblock,
									bed_bedr,
									bed_quadvar,
									&bed_bedeinh);

				if (dbg)
					fprintf(stderr,"%s '%s' '%o' '%d' '%d' '%o' '%c' '%c' '%s' '%s' '%s' '%d' \n",
						"pinfo:",namenspuffer,netztyp,knot,klass,knottyp,
						char_uew,char_besh,zusatzblock,bed_bedr,bed_quadvar,bed_bedeinh);

				if(strcmp(namenspuffer,voller_name) == 0)
					gefunden = 1;
			}

			if(gefunden == 0)
			{
				fprintf(stderr,"Verfahren %s nicht gefunden\n", voller_name);
				continue;
			}

			/* gib Information ueber Verfahren aus
			*/
			/*
				0. drucke Name des Verfahrens
			*/

			fprintf(fpout, "%s %s\n\n", VERF_NAME, voller_name);

			/*
				1. suche Infodatei
			*/

			strcpy(infodatei,DOC);
			strcat(infodatei,voller_name);

			if(dbg) fprintf(stderr,"Infodatei %s\n",infodatei);

			if((fpin = fopen(infodatei,"r")) != NULL)
			{
				while(nextline(fpin, zeile, 2048) != EOF)
				{
					/*	Zeilen mit ! in der ersten Spalte leiten ein Schluessel-
						wort ein, dass den Beginn eines Abschnittes kennzeichnet
						Die Variable 'skipmode' wird auf FALSE gesetzt, damit
						der vorherige Abschnitt als beendet gilt.
					*/
					if (zeile[0] == '!')
					{
						skipmode = FALSE;

						if (dbg)
							fprintf(stderr, "pinfo: !COMMAND found: '%s'\n", zeile);

						if (!strncmp(zeile, "!LIT", 4))
						{
							if (NO_LIT)
								skipmode = TRUE;
							else
								fprintf(fpout, "\n\n%s", LITERATUR);
						}
						else if (!strncmp(zeile, "!DESC", 5))
						{
							if (NO_DESC)
								skipmode = TRUE;
							else
								fprintf(fpout, "\n\n%s", BESCHREIBUNG);
						}
						else if (!strncmp(zeile, "!XREF", 5))
						{
							if (NO_XREF)
								skipmode = TRUE;
							else
								fprintf(fpout, "\n\n%s", SEE_ALSO);
						}
						else if (!strncmp(zeile, "!IMPL", 5))
						{
							if (NO_IMPL)
								skipmode = TRUE;
							else
								fprintf(fpout, "\n\n%s", IMPLEMENTATION);
						}
						else
						{
							/*	wenn kein Schluesselwort gefunden wird, so
								wird statt dessen eine Leerzeile ausgegeben
							*/
							fprintf(fpout, "\n");
						}
					}
					else
					{
						if (!skipmode)
							fprintf(fpout, "   %s", zeile);
					}
				}
				fclose(fpin);
			}
			fprintf(fpout, "\n");

			/*
				2. Interpretiere GRENZEN_ALLE
			*/

			fprintf(fpout, "%s\n\n   ", DOMAIN);
			switch(netztyp)
			{
				case GESCHLOSSEN:
							fprintf(fpout, "%s ", GESCHLOSSENE_NETZE);
							break;
				case OFFEN:
							fprintf(fpout, "%s ", OFFENE_NETZE);
							break;
				case GEMISCHT:
							fprintf(fpout, "%s ", OFF_O_GESCHL_NETZE);
							break;
				case 07:
							fprintf(fpout, "%s ", GEMISCHTE_NETZE);
							break;
			}
			fprintf(fpout, "%s %d %s %s %s %d %s\n\n",
							MIT_MAX, knot, KNOTEN, UND, MIT_MAX, klass, KLASSEN);

			fprintf(fpout, "%s\n\n", MENGE_KNOTENTYPEN);
			fprintf(fpout, "   ");

			mask	= 1;
			j		= 0;

			for(i=1;i<32;i++)
			{
				if(mask & knottyp )
				{	
					if (j%4 != 0)
						fprintf(fpout, ", ");

					fprintf(fpout, knotentypen[i]);

					if (j%4 == 3)
						fprintf(fpout, ",\n   ");

					j++;
				}
				mask *= 2;
			}

			fprintf(fpout, "\n\n");

			besch = 0;

			fprintf(fpout, "%s\n\n", WEITERE_EINSCHRAENK);

			if(char_besh == '-')
			{
				fprintf(fpout, "   %s\n", UEW_ANGEBEN);
				besch = 1;
			}

			if(char_uew == '-')
			{
				fprintf(fpout, "   %s\n", BESH_ANGEBEN);
				besch = 1;
			}

			if(strcmp(bed_bedr,"MMm-->gleich") == 0 && klass > 1)
			{
				fprintf(fpout, "   %s\n", MMM_BEDRATEN0);
				fprintf(fpout, "   %s\n", MMM_BEDRATEN1);
				besch = 1;
			}

			sscanf(bed_quadvar,"%lf %*c %lf",&x,&y);

			if(y == -1.0 && x != 0.0)
			{
				fprintf(fpout, "   %s %lg %s\n", MIN_QUAD_VAR, x, SEIN);
				besch = 1;
			}
			else if (y != -1.0 && x != 0.0 && ( x != 1.0 || y != 1.0) )
			{
				fprintf(fpout, "   %s [%lg;%lg] %s!\n", SCOV_INTERVALL, x, y, SEIN);
				besch = 1;
			}

			if(besch == 0)
				fprintf(fpout, "   %s\n\n", KEINE);
			else
				fprintf(fpout, "\n");
		}
	}

	pclose(fpout);
	fclose(fpv);

	exit(0);
}
