/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988, 1994
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/
/*
	pls

	Programm zum kommentierten Listen eines Dateikataloges

	Matthias Kirschnick	20.03.92
*/

static char *SccsId = "@(#)PEPSY:\tpls.c\t1.16\t94/04/27 17:31:20";

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "d_debug.h"
#include "libpepsy.h"
#include "database.h"

#define	BUFFERLEN		5120

#define FATAL			-1
#define OK			0

#define	UNKNOWN_OPTION	"unkopt"
#define NO_CODE_FILE	"nocode"
#define OUT_OF_MEMORY	"oofmem"
#define RESULT_FILE	"resfil"
#define MODEL_FILE	"modfil"
#define GRAPH_FILE	"grafil"
#define LOGFILE		"logfil"
#define OPTIMIZE_FILE	"optfil"
#define DB_READ_TWICE	"dbread"
#define NO_DB_PRESENT	"nodbpr"
#define NO_PEPSYHOME	"nophom"
#define USAGE		"usage0"

extern int atoi();
extern void free();
	int		 dbg=0;



char	*GetFileType(filename)
char	*filename;
{
    int		 found=0;
    pdb_entry_t	*entry;
    char	*c;
    char	*save;
    char	*codierung;
    char	*filetype;
    static char	 text[BUFFERLEN];

    if (CountPepsyDB())
    {
	if (filename == (char *)NULL)
	{
	    return((char *)NULL);
	}
	else
	{
	    save = c = strdup(filename);

	    switch(*c)
	    {
	    case 'e' :	
		if (*++c == '_')
		{
		    dbgtext(0, "Model File\n");

		    filetype = (char*)spr(MODEL_FILE);

		    if (*++c == 'o')
		    {
			c++;	/* isdigit ist ein Makro!	*/

			if (isdigit(*c) && (*++c == '_'))
			{
			    dbgtext(0, "Optimization File\n");

			    filetype = (char*)spr(OPTIMIZE_FILE);
			}
		    }
		}
		else
		{
		    dbgtext(0, "unknown file\n");

		    filetype = (char*)NULL;
		}
		break;
	    case 'g' :
		if (*++c == '_')
		{
		    dbgtext(0, "Graph File\n");

		    filetype = (char*)spr(GRAPH_FILE);
		}
		else
		{
		    dbgtext(0, "unknown file\n");

		    filetype = (char*)NULL;
		}
		break;

	    case 'l':
		if (*++c == '_')
		{
		    dbgtext(0, "Logfile\n");
			
		    filetype = spr(LOGFILE);
		}
		else
		{
		    dbgtext(0, "unknown file\n");
		    filetype = (char *)NULL;
		}
		break;
			
	    case 'a' :
		if (*++c == '_')
		{
		    found		= 0;
		    filetype	= (char*)NULL;

		    codierung	= strtok(++c, "_");

		    /* Suchen in der Datenbank */

		    dbginfo(10, "Looking for infix '%s'\n", codierung);

		    ResetDBEntry();

		    while((entry=NextDBEntry()) && !found)
		    {
			dbginfo(25, "entry-code='%s'\n", entry->codierung);

			if (!strcmp(entry->codierung, codierung))
			{
			    dbgtext(0, "Results File\n");

			    found = 1;

			    strcpy(text, (char*)spr(RESULT_FILE));
			    strcat(text, " (");
			    strcat(text, entry->verfahren);
			    strcat(text, ")");

			    filetype = text;
			}
		    }
		}
		else
		{
		    dbgtext(0, "unknown file\n");

		    filetype = (char*)NULL;
		}
		break;
	    default  :	
		dbgtext(0, "unknown file\n");
					
		filetype = (char*)NULL;
		break;
	    }

	    free(save);
	}
    }
    else
    {
	fprintf(stderr, spr(NO_DB_PRESENT));
	exit(FATAL);
    }

    return(filetype);
}


int main(argc, argv)
			int		 argc;
			char	*argv[];
{
    /*		Sccs support from SccsLogging.c			*/	
    extern	int	SccsRelease;
    extern	int	SccsLevel;

    /*		fuer getopt(3C)							*/
    extern	char	*optarg;
    extern	int	optind;

    /*		lokale Variablen 						*/
    int	c;
    int	linelength = 0;
    FILE	*fp;
    char	buffer[BUFFERLEN];
    char	*comment;
    char	*filename;
    char	*spacefilename;
    char	*slashfilename;
			
				

    if (p_lang_init(argv[0]) != 0) /* PEPSY-NLS Support */
	exit (FATAL);

	
    while ((c=(char)getopt(argc, argv, "Vd:")) != EOF)
    {
	switch(c)
	{
	case 'd':
	    dbg = atoi(optarg);
	    fprintf(stderr, "Debug Level %d activated!\n", dbg);
	    break;
	case 'V':
	    fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
	    exit(0);
	    break;
	default :
	    fprintf(stderr, "%s '%c'\n", (char*)spr(UNKNOWN_OPTION), c);
	    fprintf(stderr, "Usage: %s %s\n", argv[0], (char*)spr(USAGE));
	    exit(1);
	    break;
	}
    }


    /*
       Datenbank ueber die Verfahren einlesen, damit GetFileType() auch den entsprechenden
       Kommentar zum Infix finden kann.
       */

    GetPepsyDB();


    /*	
       Commandozeile zusammenbauen.
       Alle Argumente die nicht Optionen sind werden an das ls weitergegeben, um z.B.
       Verzeichnisse o.ae. gezielt auslesen zu koennen.
       */

    strcpy(buffer, "ls -l ");

    for (; optind < argc; optind++)
    {
	strcat(buffer, argv[optind]);
	strcat(buffer, " ");
    }


    /*
       Eigentlicher Listbefehl
       Mit popen werden die Ausgaben des ls Kommandos gelesen. Der letzte String
       jeder Zeile wird als Dateiname interpretiert und ein kleiner Kommentar
       ausgegeben.
       */

    if ((fp = popen(buffer, "r")) != NULL)
    {
	while((linelength = getline(fp, buffer, BUFFERLEN)) != EOF)
	{
	    /*
	       Der eigentliche Dateiname beginnt beim letzten Leerzeichen, oder beim
	       rechtesten Slash '/'.
	       */
	    spacefilename = (char *)(strrchr(buffer, ' ')+1);
	    slashfilename = (char *)(strrchr(buffer, '/')+1);
	    filename 	  = (slashfilename > spacefilename)?slashfilename:spacefilename;

	    comment = GetFileType(filename);

	    /*
	       getline liest abschliessendes Newline mit ein, deshalb muss es entfernt
	       werden, damit alles in einer Zeile steht.
	       Uebrigens gibt fputs() im Gegensatz zu puts() kein abschliessendes NL mit
	       aus.
	       */
	    c=(size_t)strlen(filename);
	    *strchr(buffer, '\n') = ' ';

	    fputs(buffer, stdout);
	    while(c++<20)	fputc(' ', stdout);
	    if (comment)	fputs(comment, stdout);
	    fputc('\n', stdout);
	}

	pclose(fp);
    }
    else
    {
	perror(argv[0]);
	exit(FATAL);
    }

    exit(OK);
}




