/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tmodel1.c\t1.3\t92/08/24 14:13:50";

#include <stdio.h>
#include <d_pepsy.h>
#include <d_netz.h>
#include <d_debug.h>
#define	EXTERN_KNOTENTYPEN
#include <d_konst.h>

#include "transform.h"

extern netzstruktur netz;
extern int dbg;

void transform1();

int model1()		/* Wandelt den Besh'envektor in ein UEW'enfeld um, bei dem die Knoten seriell gekoppelt sind */
{
    int mult_flag;
    int i, j;
    float bypass, help;

    dbgtext(0, "model1\n");

    for(i=0; i<netz.klass; i++)
    {
	netz.uew[0][1][i]=1.0;
		
	for(j=0; j<netz.knot; j++)
	{				/* maximale besh suchen */	
	    if (netz.besh[j][i]>help)
		help=netz.besh[j][i];
	}

	if (help<1.0)
	{
	    dbgtext(0, "Model1:\nACHTUNG: Es muss mindestens eine Besuchshaeufigkeit >=1.0 sein!\n");
	    return(WARNING);
	}

	if (dbg>4)
	{
	    fprintf(stderr,"Besuchshaeufigkeiten nach evtl. Korrektur:\n");
	    for (j=0; j<netz.knot; j++)
		fprintf(stderr," %6.6g",netz.besh[j][i]);
	    fprintf(stderr,"\n");
	}

	for(j=1; j<=netz.knot; j++)	/* Eigentliche Transformation durchfuehren */
	{			
	    dbginfo2(2,  "model1: Klasse%d Knoten%d, rufe transform1 auf!\n",i,j);
	    transform1(netz.besh[j-1][i],j,netz.knot+1,i);
	}

    }
    netz.haeuf = 1;	/* jetzt sind uew gueltig! */

    dbgtext(0, "model1 done\n");

    return(OK);
}

void transform1(besh,akt,max,klasse)	/* Traegt die fuer Knoten akt noetigen Aenderungen in netz.uew ein */
double besh;
int akt;
int max;
int klasse;
{
    float d1, d2, loop, pass;
    float help;
    int i, j;

    if (besh>=1.0)
    {	/**************************************** LOOP erforderlich ************************************
	  wenn besh==1.0 dann loop=0.0! */
	loop = (besh-1.0)/besh;
	netz.uew[akt][akt][klasse]		= loop;				/* uew zu sich selbst	*/
	netz.uew[akt][(akt+1)%max][klasse]	= 1.0-loop;	/* ... Rest geht zum naechsten Knoten	*/
    }

    if ((besh>=0.0) && (besh<1.0)) 
    {	/************************* BYPASS erforderlich *********************************/

	pass = (1.0-besh);
	netz.uew[akt][akt][klasse]=0.0;							/* kein LOOP */

	if (besh!=0.0)
	    netz.uew[akt][(akt+1)%max][klasse]=1.0;	/* was am Knoten ankommt, geht auch durch	*/
	else
	    netz.uew[akt][(akt+1)%max][klasse]=0.0;	/* nichts kommt an, was weitergereicht werden kann */

	/* UEW der Vorgaenger aufteilen */
	for (i=akt-1; i>=0; i--)
	{									/* Vorgaenger suchen ... */
	    if (netz.uew[i][akt][klasse]!=0.0)
	    {							/* ..., der auf den akt- Knoten verweist ...*/
		help = netz.uew[i][akt][klasse];			/* ... uew zum Aufteilen merken ... */

		dbginfo4(5, "Knoten%d wird aufgeteilt; uew[%d][%d][%d]=%g\n",i,i,(i+1)%max,klasse,help);
		dbginfo2(5, "                        ; pass = %g, besh =%g\n",pass,besh);

		netz.uew[i][akt][klasse]	 = help * besh;	/* Anteil der zum akt-Knoten geht, */
		netz.uew[i][(akt+1)%max][klasse] = help * pass;	/* Anteil der zum Nachfolger von akt geht */
	    }
	}
    }

    if (dbg>6)
    {
	for (i=0; i<max-1; i++)
	{
	    for (j=0; j<max-1; j++)
		fprintf(stderr," %4.4g ",netz.uew[i][j][klasse]);
	    fprintf(stderr,"\n");
	}
    }
}
