/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\ttrans.c\t1.2\t92/03/27 12:10:34";

#include <stdio.h>

#include "d_netz.h"
#include "d_speicher.h"
#include "d_konst.h"
#include "sim.h"

#include "transform.h"

/********************************************* Forward's *********/
extern void get_standard();		/* aus libpepsy.a */
extern void sp_ein();				/* aus libein.a	*/
extern void change_knottyp();		/* aus libein.a */	

extern int model1();					/* siehe model1.c */
extern int model2();					/* siehe model2.c */
extern int model3();
extern int model4();

/* FILE *fopen(), *fclose(); */

/********************************************* globale Variablen */
int dbg	=	0;
int verb	=	1;
int model=	1;			/* Model #1 ist Vorbesetzung */
int debug;
int COLS;
netzstruktur netz;
int pepsyversion = 1;


main(argc,argv)
int argc;
char *argv[];
{
	FILE *fpein,
	  *fpout;

	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

	extern int optind;
	extern char *optarg;
	int i, j,ret;
	int fehler=OK;
	char *c, eingabedatei[255], ausgabedatei[255];
	int C;

	if(p_lang_init("eingabe") != 0) /* wegen sp_ein */
		exit(1);
	/*     Initialisierungen
	*/
	strcpy(eingabedatei,"e_");

	/*			Uebernahme der Argumente derEingabezeile
	*/
	if (argc==1) {
		fprintf(stderr,"Usage:\n%s [Optionen] Eingebedateiname [Ausgabedateiname]\n",argv[0]);
		exit(FATAL);
	}

	while((C = getopt(argc,argv,"Vm:d:sct")) != EOF)
	{
		switch(C)
		{
			case 'd':  if((dbg = atoi(optarg)) < 0)
							dbg = 0;
						break;
			case 's':  verb = 0;
						break;
			case 'V' :  fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
							exit(0);
							break;
			case 'm' :	model = atoi(optarg);
						if(model < 1 || model > 4)
						{
							fprintf(stderr,"%s: unbekanntes Modell\n",argv[0]);
							exit(1);
						}
						break;
			case 'c':   /* central server model */
						model = 3;
						break;
			case 't':   /* terminal system or mixed system */
						model = 4;
						break;
			default	:	fprintf(stderr,"%s: Unbekannte Option '%c'. (Wird ignoriert!)\n",argv[0],C);
		}
	}

	if(dbg) fprintf(stderr,"argc %d, optind %d\n",argc,optind);
	if (argc - optind < 1) {
		fprintf(stderr,"%s: Welche Datei soll untersucht werden?\n",argv[0]);
		exit (FATAL);
	}

	strcat(eingabedatei,argv[optind++]);


	if (argc - optind > 0) {
		strcpy(ausgabedatei,"e_");
		strcat(ausgabedatei,argv[optind++]);
		fpout = fopen(ausgabedatei,"w");
		if (fpout==NULL) {
			fprintf(stderr,"%s: Kann die Ausgabedatei '%s' nicht oeffnen!\n",argv[0],ausgabedatei);
			exit(FATAL);
		}
	}
	else {
		fpout = stdout;
	}

	if ((fpein= fopen(eingabedatei,"r"))==NULL) {
		fprintf(stderr,"%s: Kann die Eingabedatei '%s' nicht oeffnen!\n",argv[0],eingabedatei);
		exit(FATAL);
	}

	if (dbg) {
		fprintf(stderr,"%s:\n    Eingabe Dateiname: '%s'\n",argv[0],eingabedatei);
		fprintf(stderr,"    Ausgabe Dateiname: '%s'\n",ausgabedatei);
		fprintf(stderr,"    Debugniveau: %d\n",dbg);
		fprintf(stderr,"    Modell: %d\n",model);
	}

	/* Netzstruktur ausfuellen mit Daten aus Eingabedatei
	*/
	get_standard(fpein);

	if (netz.haeuf==1) {
		fprintf(stderr,"%s: Achtung, es sind schon UEW angegeben!\n",argv[0]);
		exit(ERROR);
	}

	netz.uew = f_3dim_alloc(netz.knot+1,netz.knot+1,netz.klass);
	if (netz.uew == NULL) mall_err("trans(1)");

	switch(model) {
		case 1	: fehler = model1();
					  break;
		case 2	: fehler = model2();
					  break;
		case 3	: fehler = model3();
					  break;
		case 4	: fehler = model4();
					  break;
	}

	/****** Knotentypen wandeln und Datei ausgeben */
	change_knottyp();
	if(dbg) fprintf(stderr,"abspeichern ... \n");
	sp_ein(fpout,1,ausgabedatei);

	fclose(fpein);
	fclose(fpout);

	exit (fehler);
}
