/******************************************************************************/
/*  Diese Datei enthaelt die Algorithmen fuer die Berechnung der Ant-  		*/
/*  wortzeit 									            */
/******************************************************************************/

static char *SccsId = "@(#)PEPSY:\talgo.c\t1.3\t92/06/22 13:43:21";

#include   <stdio.h>
#include   "def.h"


extern int	FindMargProbs();
extern int	ChNeProbs();
extern double	Pm();



/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MM1_FCFS()								*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer die Knotentypen		*/
/*  M/M/1_FCFS, M/G/1_PS und M/G/1_LCFS_PRE.					*/
/*										*/
/********************************************************************************/

tq_MM1_FCFS(knoten)

   int	knoten;

   {

   int		r,s;		/* Laufvariablen fuer die Klassen		*/
   double	tsum;		/* m. Wartezeit aufgrund der Auftraege in der	*/
				/* WS - Schlange				*/

   for ( r=0 ; r<R ; r++ ) 
      {
      for (tsum = 0.0, s = 0 ;  s<R ; s++ )
         tsum += kq_schaetz[r][s][knoten] * sq[s][knoten];
		
      /* Mittlere Antwortzeit */
      /*----------------------*/
      tq[r][knoten] = sq[r][knoten] + tsum;
      }
   }



/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MM1_FCFS_app()								*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer die Knotentypen		*/
/*  M/M/1_FCFS, M/G/1_PS und M/G/1_LCFS_PRE mit einer approximativen Formel	*/
/*										*/
/********************************************************************************/

tq_MM1_FCFS_app(knoten)

   int	knoten;

   {

      if ( R > 1 )
	 tq_MM1_FCFS(knoten);

      else
         /* Mittlere Antwortzeit */
         /*----------------------*/
         tq[0][knoten] = sq[0][knoten] * ( 1 + (( K[0] - 1 ) / K[0]) * kq_schaetz[0][0][knoten] ); 

   }


/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MG0_IS()									*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer den Knotentypen		*/
/*  M/G/0_IS 									*/
/*										*/
/********************************************************************************/

tq_MG0_IS(knoten)

   int	knoten;

   {

   int		r;			/* Laufvariable fuer die Klassen	*/

   for ( r=0 ; r<R ; r++ )

      /* Mittlere Antwortzeit */
      /*----------------------*/
      tq[r][knoten] = sq[r][knoten] ;

   }



/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MMm_FCFS_RW()								*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Ruft auf :				gespeichert in :			*/
/*  - FindMargProbs()			wahrsch.c				*/
/*  - ChNeProbs()			wahrsch.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer die Knotentypen		*/
/*  M/M/m_FCFS und M/G/m_PS(1).							*/
/*  Dieser Algorythmus berechnet die Antwortzeit mit Hilfe der Rand-		*/
/*  wahrscheinlichkeiten entweder nach Bolch/Akyildiz oder nach 		*/
/*  Chandy/Neuse.								*/
/*										*/
/********************************************************************************/

tq_MMm_FCFS_RW(knoten,norm_sq)

   int		knoten;
   double	norm_sq;	/* normierte Bedienzeit 			*/

   {

   int		r,s;		/* Laufvariablen fuer die Klassen		*/
   int		j;		/* Laufvariable fuer die Bedieneinheiten	*/
   double	summe;		/* Hilfsgroesse 				*/
   double	tsum;		/* m. Wartezeit aufgrund der Auftraege in der	*/
				/* WS - Schlange				*/
   double	kq_randw;	/* m. Auftragszahl zur Berechnung der		*/
				/* Randwahrscheinlichkeiten 			*/

   for ( r=0 ; r<R ; r++ ) 
       {

       /* Berechnung der mittleren Wartezeit und der mittleren Auftragszahl fuer die */
       /* Randwahrscheinlichkeiten						     */
       /*----------------------------------------------------------------------------*/
       for (tsum = 0.0, kq_randw = 0.0, s = 0 ;  s<R ; s++ ) 
	  {
	  tsum += kq_schaetz[r][s][knoten] * sq[s][knoten];
	  kq_randw += kq_schaetz[r][s][knoten];
	  }

       /* Randwahrscheinlichkeiten nach Bolch/Akyildiz	*/
       /*------------------------------------------------*/
       if (ba_cn_modus == 1) 
	  FindMargProbs(knoten,kq_randw,r);	

       /* Randwahrscheinlichkeiten nach Chandy/Neuse	*/
       /*------------------------------------------------*/
       else 
	  ChNeProbs(knoten,kq_randw);

       /* Berechnung des von den Randwahrscheinlichkeiten abhaengigen Faktors */
       /*---------------------------------------------------------------------*/
       summe = 0.0;
       for (j = 0 ; j <= mm[knoten] - 2 ; j++) 
	  summe  +=  ((double)(mm[knoten]-1-j)) * P[j] ;

       /* Mittlere Antwortzeit */
       /*----------------------*/
       tq[r][knoten] = ( sq[r][knoten] + tsum + summe * norm_sq) / ((double)mm[knoten]) ;
       }

   } 




/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MMm_FCFS_Pm_norm()							*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Ruft auf :				gespeichert in :			*/
/*  - Pm()				wahrsch.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer die Knotentypen		*/
/*  M/M/m_FCFS und M/G/m_PS(1).							*/
/*  Dieser Algorithmus berechnet die Antwortzeit mit Hilfe der Warte-		*/
/*  wahrscheinlichkeit Pm, abhaengig von der genormten Bedienzeit und		*/
/*  der Gesamtauslastung							*/
/*										*/
/********************************************************************************/


tq_MMm_FCFS_Pm_norm(knoten,norm_sq)

   int		knoten;
   double	norm_sq;	/* normierte Bedienzeit 			*/

   {

   int		r,s;		/* Laufvariablen fuer die Klassen		*/
   double	tsum;		/* m. Wartezeit aufgrund der Auftraege in der	*/
				/* WS - Schlange				*/
   double	rho_hilf;	/* Auslastung 					*/


   /* Berechnung der Auslastung */
   /*---------------------------*/
   for ( rho_hilf = 0.0, r=0 ; r<R ; r++ )
      rho_hilf += besh[r][knoten] * lambda[r] * sq[r][knoten] ;
   rho_hilf = rho_hilf / ((double)mm[knoten]);  /* rho_hilf */

   /* Mittlere Antwortzeit mit normierter Bedienzeit */ 
   /*------------------------------------------------*/
   for ( r=0 ; r<R ; r++ ) 
      {
      for (tsum = 0.0, s = 0 ;  s<R ; s++ )
	  tsum += kq_schaetz[r][s][knoten] * sq[s][knoten];

      /* Wartewahrscheinlichkeit Pm = 0 */
      /*--------------------------------*/
      if (Pm_modus == 4)

          /* Mittlere Antwortzeit */
          /*----------------------*/
          tq[r][knoten] =  sq[r][knoten] + tsum / ((double)mm[knoten]);

      /* Wartewahrscheinlichkeit Pm != 0 */
      /*---------------------------------*/
      else

          /* Mittlere Antwortzeit */
          /*----------------------*/
	  if ( ((double)mm[knoten] - rho_hilf) == 0.0 )
	     {
	     if (dbg)
		fprintf(stderr,"Numerische Probleme bei der Berechnung der Antwortzeit\n");
	     printf("appscat_numerik\n 0 0 0\n");
	     exit(4);
	     }

          tq[r][knoten] =  sq[r][knoten] + tsum / ((double)mm[knoten]) + 
			   ( Pm(rho_hilf,knoten,Pm_modus) / ((double)mm[knoten]) - rho_hilf ) * norm_sq ;
      }

   } 




/******************************************************************************/
/*										                  */
/*  FUNKTION									            */
/*  tq_MMm_FCFS_Pm()								            */
/*										                  */
/*  Wird aufgerufen in :		gespeichert in :			            */
/*  - si_mva()				appscat.c				            */
/*										                  */
/*  Ruft auf :				gespeichert in :			            */
/*  - Pm()				wahrsch.c				                  */
/*										                  */
/*  Diese Funktion berechnet die Antwortzeit fuer die Knotentypen		      */
/*  M/M/m_FCFS und M/G/m_PS(1).							      */
/*  Dieser Algorithmus berechnet die Antwortzeit mit Hilfe der Warte-		*/
/*  wahrscheinlichkeit Pm, abhaengig von der Bedienzeit und			      */
/*  der Auslastung fuer jede Klasse getrennt.					      */
/*										                  */
/******************************************************************************/


tq_MMm_FCFS_Pm(knoten)	

int 	knoten;

{
   int	r,s;		      /* Laufvariablen fuer die Klassen		*/
   double	summe;		/* Hilfsgroesse 				      */
   double	tsum;		      /* m. Wartezeit aufgrund der Auftraege in */
					/* der WS - Schlange				*/
   double	rho_hilf;	      /* Auslastung 					*/
   double   lam_hilf;         /* Hilfsgroesse fuer lambda               */

   extern char *option;
   extern double rho_schnell();
   extern double rho_zufall();


   /* Mittlere Antwortzeit ohne normierter Bedienzeit */
   /*-------------------------------------------------*/
      for ( r=0 ; r<R ; r++ ) 
      {
         for (tsum = 0.0, summe = 0.0, s = 0 ;  s<R ; s++ )
         {
            tsum += kq_schaetz[r][s][knoten] * sq[s][knoten];

            if (typ[knoten]==20)
		{
		   switch(option[0])
		   {
			case 'd' : rho_hilf = besh[s][knoten] * lambda[s] 
						    * sq[s][knoten] / ((double)mm[knoten]);
				     break;
		      case 'f' : lam_hilf = besh[s][knoten]*lambda[s];
			           rho_hilf = rho_zufall(knoten,lam_hilf);
				     break;
		      case 'g' : lam_hilf = besh[s][knoten]*lambda[s];
			           rho_hilf = rho_schnell(knoten,lam_hilf);
				     break;
		   }
		}
		else
               rho_hilf = besh[s][knoten] * lambda[s] * sq[s][knoten] / 
		              ((double)mm[knoten]);

	      if ( ((double)mm[knoten] - rho_hilf) == 0.0 )
	      {
	         if (dbg)
		      fprintf(stderr,
			  "Numerische Probleme bei der Berechnung der Antwortzeit\n");
	         printf("appscat_numerik\n 0 0 0\n");
	         exit(4);
	      }

            summe += (Pm(rho_hilf,knoten,Pm_modus) / ((double)mm[knoten]) 
		         - rho_hilf) * sq[s][knoten];
        }

     /* Wartewahrscheinlichkeit Pm = 0 */
     /*--------------------------------*/
        if (Pm_modus == 4)

	  /* Mittlere Antwortzeit */
        /*----------------------*/
           tq[r][knoten] =  sq[r][knoten] + tsum / ((double)mm[knoten]);

     /* Wartewahrscheinlichkeit Pm != 0 */
     /*---------------------------------*/
        else

        /* Mittlere Antwortzeit */
        /*----------------------*/
           tq[r][knoten] =  sq[r][knoten] + tsum / ((double)mm[knoten]) + summe;
     }

} 




/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MMm_FCFS_app_Pm()							*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Ruft auf :				gespeichert in :			*/
/*  - Pm()				wahrsch.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer die Knotentypen		*/
/*  M/M/m_FCFS und M/G/m_PS(1).							*/
/*  Dieser Algorithmus berechnet die Antwortzeit mit Hilfe der Warte-		*/
/*  wahrscheinlichkeit Pm, abhaengig von der Bedienzeit und			*/
/*  der Auslastung fuer jede Klasse getrennt mit einer approximierten Formel	*/
/*										*/
/********************************************************************************/


tq_MMm_FCFS_app_Pm(knoten)	

   int 	knoten;

   {

   double	hilf;		/* Hilfsgroesse					*/
   double	rho_hilf;	/* Auslastung 					*/


   /* Mittlere Antwortzeit */
   /*----------------------*/
   if ( R > 1 )
      tq_MMm_FCFS_Pm_norm(knoten);

   else
      {
      if ( ( K[0] - (double)mm[knoten] ) == 0 )
	 hilf = 1.0;
      else
         hilf = ( K[0] - (double) mm[knoten] -1.0 ) / ( K[0] - (double) mm[knoten] ) ;
      rho_hilf = besh[0][knoten] * lambda[0] * sq[0][knoten] / ((double)mm[knoten]);

      /* Mittlere Antwortzeit */
      /*----------------------*/
      tq[0][knoten] =  sq[0][knoten] * ( (double) mm[knoten] * ( 1.0 - hilf * rho_hilf )
				       + Pm(rho_hilf,knoten,Pm_modus)
				       + hilf * kq_schaetz[0][0][knoten] );
      }

   } 




/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MGm_PS()									*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer den Knotentypen		*/
/*  M/G/m_PS(2)									*/
/*										*/
/********************************************************************************/


tq_MGm_PS(knoten)	

   int	knoten;

   {

   int		r,s;		/* Laufvariablen fuer die Klassen		*/
   double	tsum;		/* m. Wartezeit aufgrund der Auftraege in der	*/
				/* WS - Schlange				*/

   for ( r=0 ; r<R ; r++ ) 
      {
      for (tsum = 0.0, s = 0 ;  s<R ; s++ )
         tsum += kq_schaetz[r][s][knoten] * sq[s][knoten];

      /* Mittlere Antwortzeit */
      /*----------------------*/
      tq[r][knoten] = ( sq[r][knoten] + tsum ) / ((double)mm[knoten]);
      }

   }





/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MG1_FCFS_norm()								*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer den Knotentypen		*/
/*  M/G/1_FCFS 									*/
/*  Dieser Algorithmus berechnet die Antwortzeit abhaengig von der		*/
/*  genormten Bedienzeit und der Gesamtauslastung				*/
/*										*/
/********************************************************************************/


tq_MG1_FCFS_norm(knoten,norm_cb_q,norm_sq)

   int	knoten;
   double	norm_cb_q;	/* normierter, quadrierter Variationskoeffizient*/
   double	norm_sq;	/* normierte Bedienzeit 			*/

   {

   int		r,s;		/* Laufvariablen fuer die Klassen		*/
   double	tsum;		/* m. Wartezeit aufgrund der Auftraege in der	*/
				/* WS - Schlange				*/
   double	rho_hilf;	/* Auslastung 					*/

   for ( rho_hilf = 0.0, r = 0 ; r<R ; r++ )
       rho_hilf += besh[r][knoten] * lambda[r] * sq[r][knoten] ;
	
   for ( r=0 ; r<R ; r++ ) 
       {
       for (tsum = 0.0, s = 0 ;  s<R ; s++ )
          tsum += kq_schaetz[r][s][knoten] * sq[s][knoten];
	
       /* mittlere Antwortzeit */
       /*----------------------*/
       tq[r][knoten] = sq[r][knoten] + 
		  tsum +
		  rho_hilf * norm_sq * (norm_cb_q - 1.0) / 2.0;
       }

   } 




/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MG1_FCFS()								*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer den Knotentypen		*/
/*  M/G/1_FCFS 									*/
/*  Dieser Algorithmus berechnet die Antwortzeit abhaengig von der		*/
/*  Bedienzeit der Auslastung fuer jede Klasse getrennt.			*/
/*										*/
/********************************************************************************/


tq_MG1_FCFS(knoten)	

   int	knoten;

   {

   int		r,s;		/* Laufvariablen fuer die Klassen		*/
   double	summe;		/* Hilfsgroesse 				*/
   double	tsum;		/* m. Wartezeit aufgrund der Auftraege in der	*/
				/* WS - Schlange				*/
   double	rho_hilf;	/* Auslastung 					*/


   for ( r=0 ; r<R ; r++ ) 
      {
      for (tsum = 0.0, summe=0.0, s = 0 ;  s<R ; s++ ) 
	 {
	 tsum += kq_schaetz[r][s][knoten] * sq[s][knoten];
	 rho_hilf = besh[s][knoten] * lambda[s] * sq[s][knoten];
	 summe += rho_hilf*(vc[s][knoten] - 1.0)/2.0 * sq[s][knoten];
	 }
	
      /* mittlere Antwortzeit */
      /*----------------------*/
      tq[r][knoten] = sq[r][knoten] + tsum + summe;
      } 

   }



/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MG1_FCFS_app()								*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer den Knotentypen		*/
/*  M/G/1_FCFS 									*/
/*  Dieser Algorithmus berechnet die Antwortzeit abhaengig von der		*/
/*  Bedienzeit der Auslastung fuer jede Klasse getrennt	mit einer approximierten*/
/*  Formel									*/
/*										*/
/********************************************************************************/


tq_MG1_FCFS_app(knoten)	

   int	knoten;

   {

   double	summe;		/* Hilfsgroesse 				*/
   double	hilf;		/* Hilfsgroesse 				*/
   double	a_hilf;		/* Hilfsgroesse 				*/
   double	rho_hilf;	/* Hilfsgroesse fuer den Durchsatz im Knoten i 	*/


   if ( R > 1 )
      tq_MG1_FCFS(knoten);

   else
      {
      a_hilf = ( vc[0][knoten] +1.0 ) / 2.0 ;
      if ( ( K[0] - 1.0 ) == 0 )
	 hilf = 1.0;
      else
         hilf = ( K[0] - 1.0 - a_hilf ) / ( K[0] - 1.0 ) ;
      rho_hilf = besh[0][knoten] * lambda[0] * sq[0][knoten];
	
      /* mittlere Antwortzeit */
      /*----------------------*/
      tq[0][knoten] = sq[0][knoten] * ( 1.0 + rho_hilf * ( a_hilf - hilf) + hilf * kq_schaetz[0][0][knoten] ) ;
      } 

   }






/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MGm_FCFS_norm()								*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Ruft auf :				gespeichert in :			*/
/*  - Pm()				wahrsch.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer den Knotentypen		*/
/*  M/G/m_FCFS 									*/
/*  Dieser Algorithmus berechnet die Antwortzeit abhaengig von der		*/
/*  genormten Bedienzeit und der Gesamtauslastung				*/
/*										*/
/********************************************************************************/

tq_MGm_FCFS_norm(knoten,norm_cb_q,norm_sq,mod)

   int		knoten;
   double	norm_cb_q;	/* normierter, quadrierter Variationskoeffizient*/
   double	norm_sq;	/* normierte Bedienzeit 			*/
   int		mod;		/* Berechnungsart bei Pm			*/

  {

   int		r,s;		/* Laufvariablen fuer die Klassen		*/
   double	tsum;		/* m. Wartezeit aufgrund der Auftraege in der	*/
				/* WS - Schlange				*/
   double	rho_hilf;	/* Auslastung 					*/


  for ( rho_hilf = 0.0, r = 0 ; r<R ; r++ )
      rho_hilf += besh[r][knoten] * lambda[r] * sq[r][knoten] / ((double)mm[knoten]);  /* rho_hilf */

  for ( r=0 ; r<R ; r++ ) 
      {
      for (tsum = 0.0, s = 0 ;  s<R ; s++ )
         tsum += kq_schaetz[r][s][knoten] * sq[s][knoten];
	
      /* mittlere Antwortzeit */
      /*----------------------*/
      if ( mod == 4)
         tq[r][knoten] =  sq[r][knoten] + tsum / ((double)mm[knoten]);
      else
	 {

	 if ( ((double)mm[knoten] - rho_hilf) == 0.0 )
	    {
	    if (dbg)
	       fprintf(stderr,"Numerische Probleme bei der Berechnung der Antwortzeit\n");
	    printf("appscat_numerik\n 0 0 0\n");
	    exit(4);
	    }

         tq[r][knoten] =  sq[r][knoten] + 
	            tsum / ((double)mm[knoten]) +
	            ((1.0 + norm_cb_q)/2.0 * Pm(rho_hilf,knoten,mod) / ((double)mm[knoten]) - rho_hilf) * norm_sq;
	 }
      }
   }





/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MGm_FCFS()								*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Ruft auf :				gespeichert in :			*/
/*  - Pm()				wahrsch.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer den Knotentypen		*/
/*  M/G/m_FCFS 									*/
/*  Dieser Algorithmus berechnet die Antwortzeit abhaengig von der		*/
/*  Bedienzeit der Auslastung fuer jede Klasse getrennt.			*/
/*										*/
/********************************************************************************/

tq_MGm_FCFS(knoten,mod)

   int	knoten;
   int	mod;			/* Berechnungsart bei Pm			*/

   {

   int		r,s;		/* Laufvariablen fuer die Klassen		*/
   double	summe;		/* Hilfsgroesse 				*/
   double	tsum;		/* m. Wartezeit aufgrund der Auftraege in der	*/
				/* WS - Schlange				*/
   double	rho_hilf;	/* Auslastung 					*/


   for ( r=0 ; r<R ; r++ ) 
      {
      for (tsum = 0.0, summe=0.0, s = 0 ;  s<R ; s++ ) 
         {
         tsum += kq_schaetz[r][s][knoten] * sq[s][knoten];

         rho_hilf   = besh[s][knoten] * lambda[s] * sq[r][knoten] / ((double)mm[knoten]);

	 if ( ((double)mm[knoten] - rho_hilf) == 0.0 )
	    {
	    if (dbg)
	       fprintf(stderr,"Numerische Probleme bei der Berechnung der Antwortzeit\n");
	    printf("appscat_numerik\n 0 0 0\n");
	    exit(4);
	    }

         summe += ((1.0+vc[s][knoten])/2.0 * Pm(rho_hilf,knoten,mod) / ((double)mm[knoten]) - rho_hilf) * sq[s][knoten];
         }

      /* Wartewahrscheinlichkeit Pm = 0 */
      /*--------------------------------*/
      if (mod == 4) 
	
         /* mittlere Antwortzeit */
         /*----------------------*/
         tq[r][knoten] =  sq[r][knoten] + tsum / ((double)mm[knoten]);

      /* Wartewahrscheinlichkeit Pm != 0 */
      /*---------------------------------*/
      else 
	
         /* mittlere Antwortzeit */
         /*----------------------*/
         tq[r][knoten] =  sq[r][knoten] + tsum / ((double)mm[knoten]) + summe;
      }

   }





/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  tq_MGm_FCFS_app()								*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*										*/
/*  Ruft auf :				gespeichert in :			*/
/*  - Pm()				wahrsch.c				*/
/*										*/
/*  Diese Funktion berechnet die Antwortzeit fuer den Knotentypen		*/
/*  M/G/m_FCFS 									*/
/*  Dieser Algorithmus berechnet die Antwortzeit abhaengig von der		*/
/*  Bedienzeit der Auslastung fuer jede Klasse getrennt	 mit einer approxi-	*/
/*  mierten Formel.								*/
/*										*/
/********************************************************************************/

tq_MGm_FCFS_app(knoten,mod)

   int	knoten;
   int	mod;			/* Berechnungsart bei Pm			*/

   {

   double	hilf;		/* Hilfsgroesse 				*/
   double	a_hilf;		/* Hilfsgroesse 				*/
   double	rho_hilf;	/* Auslastung 					*/


   if ( R > 1 )
      tq_MGm_FCFS_norm(knoten);

   else
      {
	 a_hilf = ( vc[0][knoten] + 1.0 ) / 2.0 ;
	 if ( ( K[0] - (double)mm[knoten] ) == 0 )
	    hilf = 1.0;
	 else
	    hilf = ( K[0] - (double) mm[knoten] - a_hilf ) / ( K[0] - (double) mm[knoten] ) ;
         rho_hilf   = besh[0][knoten] * lambda[0] * sq[0][knoten] / ((double)mm[knoten]);

         /* mittlere Antwortzeit */
         /*----------------------*/
         tq[0][knoten] =  sq[0][knoten] * ( mm[knoten] * ( 1.0 - rho_hilf * hilf ) 
					  + a_hilf * Pm(rho_hilf,knoten,mod)
					  + hilf * kq_schaetz[0][0][knoten] ) ;
      }

   }

