/********************************************************************************/
/*  In dieser Datei werden die Wartewahrscheinlichkeiten, die Wahr-		  */
/*  scheinlichkeitsmasse und die Randwahrscheinlichkeiten berechnet		  */
/********************************************************************************/

static char *SccsId = "@(#)PEPSY:\twahrsch.c\t1.6\t94/02/03 14:38:09";

#include <stdio.h>
#include <math.h>
#include <errno.h>
#include "def.h"

#define	STREUUNG	20	/* maximale Streuungsbreite			      */
				      /* nochmal in speicher_verw.c definiert, um fuer*/
				      /* W und PR dynamisch Speicher zu belegen	      */
#define	ALPHA		45.0	/* Parameter der Skalierungsfunktion PR		*/
#define	BETA		0.7	/* Parameter der Skalierungsfunktion PR		*/
#define     MINRHO      1e-32 /* untere Schranke fuer rho                     */
#define     null        1e-16 /* eigene 'Null'-Definition                     */

extern	int	errno;

/********************************************************************************/
/*										                    */
/*  FUNKTION									              */
/*  Pm()									                    */
/*										                    */
/*  Wird aufgerufen in :		gespeichert in :			              */
/*  - tq_MMm_FCFS_norm()		algo.c					        */
/*  - tq_MMm_FCFS()			algo.c					        */
/*  - tq_MGm_FCFS_norm()		algo.c					        */
/*  - tq_MGm_FCFS()			algo.c					        */
/*  										                    */
/*  ruft auf :				gespeichert in :			              */
/*  - pow()				      math.h					        */
/*  - sp_2_free()			      speicher_verw.c				        */
/*  - sp_1_free()			      speicher_verw.c				        */
/*  - p_wart_zufall()               wahrsch.c                                   */
/*  - p_wart_schnell()              wahrsch.c                                   */
/*  										                    */
/*  Berechnung der Wartewahrscheinlichkeit,					        */
/*  dass bei einem Knoten mit 'mm[knoten]' Bedieneinheiten alle			  */
/*  Bedieneinheiten belegt sind, wenn dieser mit 'rho_h' ausgelastet ist.	  */
/*  'mod' gibt das Berechnungsverfahren an (exakt, naeherungsweise).		  */
/*  Literatur:  Bolch/Bruchner 83						              */
/*										                    */
/********************************************************************************/


double	Pm(rho_h,knoten,mod)
   double	rho_h;       	/* Auslastung 					        */
   int		knoten;  	/* Knotennummer 				              */
   int		mod;       	/* Berechnungsmodus bei Pm 			        */

{
   double p0, sum, hilf, fak, power;    /* Hilfsgroessen 			        */
   int j;                               /* Laufvariable fuer die Bedien-	  */
					/* einheiten				              */

   double	ret_wert;		/* Return - Wert fuer die naeherungs-	        */
					/* weise berechnete Wartewahrscheinlich-          */
					/* keit					              */

   double p_wart_zufall();
   double p_wart_schnell();


   switch (mod)
   {
      case 2: /* exakte Berechnung fuer M/M/m_FCFS - Knoten, Naeherung fuer MGm_FCFS - Knoten */
	    for ( sum=0.0 , hilf=((double)mm[knoten]) * rho_h , fak=1.0 , power=1.0 , j = 0 ;
		  j <= mm[knoten] ; )
	    {
		sum += power / fak;
		j++;
		fak *= j;
		power *= hilf;
	    }

	    if ( (fak * (1.0 - rho_h)) == 0.0 )
	    {
	       if ( dbg )
		  fprintf(stderr,"Numerische Probleme bei der exakten Berechnung der Wartewahrscheinlichkeit\n");
	       printf("appscat_numerik\n 0 0 0\n");
	       exit(2);
	    }

	    hilf	= power / (fak * (1.0 - rho_h));
	    p0		= 1.0 / ( sum + hilf);
	    ret_wert	= p0 * hilf;
	    break;

      case 3: /* gemischte, naeherungsweise Berechnung */
	    if (rho_h <= 0.7) 
		ret_wert =  pow(rho_h,(((double)mm[knoten]) + 1.0)/2.0);
	    else
		ret_wert = (pow(rho_h,((double)mm[knoten])) + rho_h) / 2.0;

	    if ( errno == EDOM )
	    {
	       if ( dbg )
		  fprintf(stderr,"Numerische Probleme bei der exakten Berechnung der Wartewahrscheinlichkeit\n");
	       printf("appscat_numerik\n 0 0 0\n");
	       exit(2);
	    }
	    break;

      case 4: /* Pm = 0 */
	    ret_wert= 0.0;
	    break;


	case 5:
   /* exakte Berechnung der Wartewahrscheinlichkeit nach Baer/Fischer/ */
   /* Hertel fuer M/M/m-FCFS-ASYM-Knoten, zufaellige Auswahl einer     */
   /*	freien Bedieneinheit                                             */
   /* --------------------------------------------------------------   */
	ret_wert = p_wart_zufall(rho_h,knoten);
	break;


	case 6:
   /* exakte Berechnung der Wartewahrscheinlichkeit nach Baer/Fischer/ */
   /* Hertel fuer M/M/m-FCFS-ASYM-Knoten, Auswahl der schnellsten      */
   /*	freien Bedieneinheit                                             */
   /* --------------------------------------------------------------   */
	ret_wert = p_wart_schnell(rho_h,knoten);
	break;


      default:
	    if (dbg) fprintf(stderr,"Undefinierter Modus in Pm()\n");
	    
	    /* Allokierten Speicher freigeben */
	    /*--------------------------------*/
	    if (flag == 1)
	       sp_2_free();
	    sp_1_free();

	    printf("appscat_modus\n 0 0 0\n");
	    exit(1);
   }
   return(ret_wert);
}


/******************************************************************************/
/*										                  */
/*  FUNKTION									            */
/*  p_wart_zufall()            					                  */
/*										                  */
/*  Wird aufgerufen in :		gespeichert in :			            */
/*  - Pm()                 	      wahrsch.c		                        */
/*  										                  */
/*  ruft auf :				gespeichert in :			            */
/*  - all_ws()	                  wahrsch.c                                 */
/*  										                  */
/*  Berechnung der Wartewahrscheinlichkeit nach Hertel.                       */
/*										                  */
/******************************************************************************/

double p_wart_zufall(rho,knoten)
double rho;
int knoten;
{
   struct Zustand * all_ws();

   int r;
   double val=1.0;
   double p_bel=0,c_ex=0,N=0,lamb=0;
   struct Zustand *ptr;

   if (rho >= 1.0)    return 1.0;
   if (rho <= MINRHO) return 0.0;

/* Berechnung der Belegt-Wahrscheinlichkeit */
/* ---------------------------------------- */
   ptr = all_ws(knoten);
   while (ptr->next != NULL) ptr = ptr->next;
   p_bel = ptr->value;

/* Berechnung der Konstanten c_ex */
/* ------------------------------ */
   lamb = lambda[0] * besh[0][knoten];

   for (r = 0; r < mm[knoten]; r++) 
      c_ex = c_ex + (amue[knoten][r]/lamb);
   c_ex = 1 / c_ex;

/* Berechnung der Konstanten N */
/* --------------------------- */
   N = 1 + (p_bel * c_ex / (1 - c_ex));

/* Berechnung der Wartewahrscheinlichkeit */
/* -------------------------------------- */
   val = p_bel / (N * (1 - c_ex));
   return(val);
}



/******************************************************************************/
/*										                  */
/*  FUNKTION									            */
/*  all_ws()                  					                  */
/*										                  */
/*  Wird aufgerufen in :		gespeichert in :			            */
/*  - p_wart_zufall()     	      wahrsch.c		                        */
/*  										                  */
/*  ruft auf :				gespeichert in :			            */
/*  - zust_liste()                  wahrsch.c                                 */
/*  - ini_liste()                   wahrsch.c                                 */
/*  - mul_rho()                     wahrsch.c                                 */
/*  - faku()                        wahrsch.c                                 */
/*  										                  */
/*  Berechnung der Zustandswahrscheinlichkeiten nach Hertel                   */
/*										                  */
/******************************************************************************/

struct Zustand * all_ws(knoten)
int knoten;
{
   struct Zustand * zust_liste();
   struct Zustand * ini_liste();
   double mul_rho();
   double faku();

   int r;
   double lamb, P_0 = 0;
   struct Zustand *first, *ptr, *ini_ptr; 

/* Berechnung der Leerwahrscheinlichkeit */
/* ------------------------------------- */
   for (r = 1; r <= mm[knoten]; r ++)
      P_0 = P_0 + (faku(mm[knoten]-r) * mul_rho(knoten,r));
   P_0 = (P_0 / faku(mm[knoten])) + 1;
   P_0 = 1 / P_0;

/* Berechnung der uebrigen Wahrscheinlichkeiten */
/* -------------------------------------------- */
   first = zust_liste(knoten);  
   ini_ptr = first;
   for (r = 1; r < mm[knoten]; r++) 
   {
       ptr = ini_liste(ini_ptr,r,knoten);
       ini_ptr = ptr;
   }
   ptr = first;
   lamb = lambda[0] * besh[0][knoten];
   while (ptr != NULL)
   {
      ptr->value = (faku(mm[knoten]-(ptr->anz))/faku(mm[knoten])) * P_0;
      for (r = 0; r < mm[knoten]; r++)
         if (ptr->belegt[r] == 1) 
            ptr->value = ptr->value * (lamb / amue[knoten][r]);
      ptr = ptr->next;
   }

   return(first);
}



/******************************************************************************/
/*										                  */
/*  FUNKTION									            */
/*  zust_liste()              					                  */
/*										                  */
/*  Wird aufgerufen in :		gespeichert in :			            */
/*  - all_ws()             	      wahrsch.c		                        */
/*  										                  */
/*  Erstellung einer Liste von Zustaenden des Markov-Graphen                  */
/*										                  */
/******************************************************************************/

struct Zustand * zust_liste(knoten)
int knoten;                         
{
   int r,j;
   int anz;                    
   struct Zustand * first;     
   struct Zustand * ptr;      

   /* Anzahl Listenelemente feststellen */
   /* ----------------------------------*/
      anz = 2;
      for (r = 0; r < (mm[knoten]-1); r++)
         anz = anz*2;

   /* Erstes Element erzeugen und initialisieren */
   /* ------------------------------------------ */
      first = (struct Zustand *) malloc (sizeof(struct Zustand));
      for (r = 0; r < mm[knoten]; r++) first->belegt[r] = 0;
      first->anz   = 0;
      first->value = 1;
      first->next = NULL;
      ptr = first;
   
   /* weitere Elemente erzeugen und initialisieren */
   /* -------------------------------------------- */
      for (r = 0; r < (anz-1); r++)
      {
         ptr->next = (struct Zustand *) malloc (sizeof(struct Zustand));
         ptr = ptr->next;
         for (j = 0; j < mm[knoten]; j++) ptr->belegt[j] = 0;
         ptr->anz   = 0;
         ptr->value = 1;
         ptr->next = NULL;
      }

   /* Ruecksprung */
   /* ----------- */
      return(first);
}



/******************************************************************************/
/*										                  */
/*  FUNKTION									            */
/*  ini_liste()               					                  */
/*										                  */
/*  Wird aufgerufen in :		gespeichert in :			            */
/*  - all_ws()             	      wahrsch.c		                        */
/*  										                  */
/*  Initialisierung der Zustandsliste                                         */
/*										                  */
/******************************************************************************/
        
struct Zustand * ini_liste(ptr,anz,knoten) 
struct Zustand * ptr;         
int anz;             
int knoten;                        
{
   int    j[11];

   switch(anz)
   {
      case 1:
      {
         for (j[1] = 0; j[1] < mm[knoten]; j[1]++)
         {
            ptr->anz = 1;
            ptr->belegt[j[1]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 2:
      {
         for (j[1] = 0;     j[1] < (mm[knoten]-1); j[1]++)
         for (j[2] = j[1]+1;j[2] <  mm[knoten]   ; j[2]++)
         {
            ptr->anz = 2;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 3:
      {
         for (j[1] = 0;      j[1] < (mm[knoten]-2); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[knoten]-1); j[2]++)
         for (j[3] = j[2]+1; j[3] <  mm[knoten]   ; j[3]++)
         {
            ptr->anz = 3;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 4:
      {
         for (j[1] = 0;      j[1] < (mm[knoten]-3); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[knoten]-2); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[knoten]-1); j[3]++)
         for (j[4] = j[3]+1; j[4] <  mm[knoten]   ; j[4]++)
         {
            ptr->anz = 4;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 5:
      {
         for (j[1] = 0;      j[1] < (mm[knoten]-4); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[knoten]-3); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[knoten]-2); j[3]++)
         for (j[4] = j[3]+1; j[4] < (mm[knoten]-1); j[4]++)
         for (j[5] = j[4]+1; j[5] <  mm[knoten]   ; j[5]++)
         {
            ptr->anz = 5;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr->belegt[j[5]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 6:
      {
         for (j[1] = 0;      j[1] < (mm[knoten]-5); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[knoten]-4); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[knoten]-3); j[3]++)
         for (j[4] = j[3]+1; j[4] < (mm[knoten]-2); j[4]++)
         for (j[5] = j[4]+1; j[5] < (mm[knoten]-1); j[5]++)
         for (j[6] = j[5]+1; j[6] <  mm[knoten]   ; j[6]++)
         {
            ptr->anz = 6;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr->belegt[j[5]] = 1; ptr->belegt[j[6]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 7:
      {
         for (j[1] = 0;      j[1] < (mm[knoten]-6); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[knoten]-5); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[knoten]-4); j[3]++)
         for (j[4] = j[3]+1; j[4] < (mm[knoten]-3); j[4]++)
         for (j[5] = j[4]+1; j[5] < (mm[knoten]-2); j[5]++)
         for (j[6] = j[5]+1; j[6] < (mm[knoten]-1); j[6]++)
         for (j[7] = j[6]+1; j[7] <  mm[knoten]   ; j[7]++)
         {
            ptr->anz = 7;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr->belegt[j[5]] = 1; ptr->belegt[j[6]] = 1;
            ptr->belegt[j[7]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 8:
      {
         for (j[1] = 0;      j[1] < (mm[knoten]-7); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[knoten]-6); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[knoten]-5); j[3]++)
         for (j[4] = j[3]+1; j[4] < (mm[knoten]-4); j[4]++)
         for (j[5] = j[4]+1; j[5] < (mm[knoten]-3); j[5]++)
         for (j[6] = j[5]+1; j[6] < (mm[knoten]-2); j[6]++)
         for (j[7] = j[6]+1; j[7] < (mm[knoten]-1); j[7]++)
         for (j[8] = j[7]+1; j[8] <  mm[knoten]   ; j[8]++)
         {
            ptr->anz = 8;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr->belegt[j[5]] = 1; ptr->belegt[j[6]] = 1;
            ptr->belegt[j[7]] = 1; ptr->belegt[j[8]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 9:
      {
         for (j[1] = 0;      j[1] < (mm[knoten]-8); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[knoten]-7); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[knoten]-6); j[3]++)
         for (j[4] = j[3]+1; j[4] < (mm[knoten]-5); j[4]++)
         for (j[5] = j[4]+1; j[5] < (mm[knoten]-4); j[5]++)
         for (j[6] = j[5]+1; j[6] < (mm[knoten]-3); j[6]++)
         for (j[7] = j[6]+1; j[7] < (mm[knoten]-2); j[7]++)
         for (j[8] = j[7]+1; j[8] < (mm[knoten]-1); j[8]++)
         for (j[9] = j[8]+1; j[9] <  mm[knoten]   ; j[9]++)
         {
            ptr->anz = 9;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr->belegt[j[5]] = 1; ptr->belegt[j[6]] = 1;
            ptr->belegt[j[7]] = 1; ptr->belegt[j[8]] = 1;
            ptr->belegt[j[9]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 10:
      {
         for (j[1] = 0;      j[1] < (mm[knoten]-9); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[knoten]-8); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[knoten]-7); j[3]++)
         for (j[4] = j[3]+1; j[4] < (mm[knoten]-6); j[4]++)
         for (j[5] = j[4]+1; j[5] < (mm[knoten]-5); j[5]++)
         for (j[6] = j[5]+1; j[6] < (mm[knoten]-4); j[6]++)
         for (j[7] = j[6]+1; j[7] < (mm[knoten]-3); j[7]++)
         for (j[8] = j[7]+1; j[8] < (mm[knoten]-2); j[8]++)
         for (j[9] = j[8]+1; j[9] < (mm[knoten]-1); j[9]++)
         for (j[10] = j[9]+1; j[10] < mm[knoten]    ; j[10]++)
         {
            ptr->anz = 10;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr->belegt[j[5]] = 1; ptr->belegt[j[6]] = 1;
            ptr->belegt[j[7]] = 1; ptr->belegt[j[8]] = 1;
            ptr->belegt[j[9]] = 1; ptr->belegt[j[10]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
   }
}



/******************************************************************************/
/*										                  */
/*  FUNKTION									            */
/*  mul_rho()                 					                  */
/*										                  */
/*  Wird aufgerufen in :		gespeichert in :			            */
/*  - all_ws()             	      wahrsch.c		                        */
/*  										                  */
/*  Hilfsfunktione fuer all_ws()                                              */
/*										                  */
/******************************************************************************/

double mul_rho(i,anz)
int i,anz;
{
   double rho[10];
   double val = 0;
   double lamb;
   int    r;                       
   int    j[10];

 
   lamb = lambda[0] * besh[0][i];
   for (r = 0; r < mm[i]; r++)
       rho[r] = lamb / amue[i][r];

   switch(anz)
   {
      case 1:
      {
         for (j[1] = 0; j[1] < mm[i]; j[1]++)
         val = val + rho[j[1]];
         return(val);
      }
      case 2:
      {
         for (j[1] = 0;     j[1] < (mm[i]-1); j[1]++)
         for (j[2] = j[1]+1;j[2] <  mm[i]   ; j[2]++)
         val = val + rho[j[1]]*rho[j[2]];
         return(val);
      }
      case 3:
      {
         for (j[1] = 0;      j[1] < (mm[i]-2); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[i]-1); j[2]++)
         for (j[3] = j[2]+1; j[3] <  mm[i]   ; j[3]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]];
         return(val);
      }
      case 4:
      {
         for (j[1] = 0;      j[1] < (mm[i]-3); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[i]-2); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[i]-1); j[3]++)
         for (j[4] = j[3]+1; j[4] <  mm[i]   ; j[4]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]];
         return(val);
      }
      case 5:
      {
         for (j[1] = 0;      j[1] < (mm[i]-4); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[i]-3); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[i]-2); j[3]++)
         for (j[4] = j[3]+1; j[4] < (mm[i]-1); j[4]++)
         for (j[5] = j[4]+1; j[5] <  mm[i]   ; j[5]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]]*rho[j[5]];
         return(val);
      }
      case 6:
      {
         for (j[1] = 0;      j[1] < (mm[i]-5); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[i]-4); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[i]-3); j[3]++)
         for (j[4] = j[3]+1; j[4] < (mm[i]-2); j[4]++)
         for (j[5] = j[4]+1; j[5] < (mm[i]-1); j[5]++)
         for (j[6] = j[5]+1; j[6] <  mm[i]   ; j[6]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]]*rho[j[5]]
                    *rho[j[6]];
         return(val);
      }
      case 7:
      {
         for (j[1] = 0;      j[1] < (mm[i]-6); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[i]-5); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[i]-4); j[3]++)
         for (j[4] = j[3]+1; j[4] < (mm[i]-3); j[4]++)
         for (j[5] = j[4]+1; j[5] < (mm[i]-2); j[5]++)
         for (j[6] = j[5]+1; j[6] < (mm[i]-1); j[6]++)
         for (j[7] = j[6]+1; j[7] <  mm[i]   ; j[7]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]]*rho[j[5]]
                    *rho[j[6]]*rho[j[7]];
         return(val);
      }
      case 8:
      {
         for (j[1] = 0;      j[1] < (mm[i]-7); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[i]-6); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[i]-5); j[3]++)
         for (j[4] = j[3]+1; j[4] < (mm[i]-4); j[4]++)
         for (j[5] = j[4]+1; j[5] < (mm[i]-3); j[5]++)
         for (j[6] = j[5]+1; j[6] < (mm[i]-2); j[6]++)
         for (j[7] = j[6]+1; j[7] < (mm[i]-1); j[7]++)
         for (j[8] = j[7]+1; j[8] <  mm[i]   ; j[8]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]]*rho[j[5]]
                    *rho[j[6]]*rho[j[7]]*rho[j[8]];
         return(val);
      }
      case 9:
      {
         for (j[1] = 0;      j[1] < (mm[i]-8); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[i]-7); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[i]-6); j[3]++)
         for (j[4] = j[3]+1; j[4] < (mm[i]-5); j[4]++)
         for (j[5] = j[4]+1; j[5] < (mm[i]-4); j[5]++)
         for (j[6] = j[5]+1; j[6] < (mm[i]-3); j[6]++)
         for (j[7] = j[6]+1; j[7] < (mm[i]-2); j[7]++)
         for (j[8] = j[7]+1; j[8] < (mm[i]-1); j[8]++)
         for (j[9] = j[8]+1; j[9] <  mm[i]   ; j[9]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]]*rho[j[5]]
                    *rho[j[6]]*rho[j[7]]*rho[j[8]]*rho[j[9]];
         return(val);
      }
      case 10:
      {
         for (j[1] = 0;      j[1] < (mm[i]-9); j[1]++)
         for (j[2] = j[1]+1; j[2] < (mm[i]-8); j[2]++)
         for (j[3] = j[2]+1; j[3] < (mm[i]-7); j[3]++)
         for (j[4] = j[3]+1; j[4] < (mm[i]-6); j[4]++)
         for (j[5] = j[4]+1; j[5] < (mm[i]-5); j[5]++)
         for (j[6] = j[5]+1; j[6] < (mm[i]-4); j[6]++)
         for (j[7] = j[6]+1; j[7] < (mm[i]-3); j[7]++)
         for (j[8] = j[7]+1; j[8] < (mm[i]-2); j[8]++)
         for (j[9] = j[8]+1; j[9] < (mm[i]-1); j[9]++)
         for (j[10] = j[9]+1;j[10] < mm[i]   ; j[10]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]]*rho[j[5]]
                    *rho[j[6]]*rho[j[7]]*rho[j[8]]*rho[j[9]]*rho[j[10]];
         return(val);
      }
   }
}



/******************************************************************************/
/*										                  */
/*  FUNKTION									            */
/*  faku()                    					                  */
/*										                  */
/*  Wird aufgerufen in :		gespeichert in :			            */
/*  - all_ws()             	      wahrsch.c                                 */
/*  										                  */
/*  Berechnung der Fakultaet von (w) (positiv Integer)                        */
/*										                  */
/******************************************************************************/

double faku(w)
int w;
{
   int g;
   double f = 1.0;

   for (g = 2; g <= w; f *= g, g++);
   return f;
}


/******************************************************************************/
/*										                  */
/*  FUNKTION									            */
/*  rho_zufall()                  					                  */
/*										                  */
/*  Wird aufgerufen in :		gespeichert in :			            */
/*  - tq_MMm_FCFS_Pm       	      algo.c                                    */
/*  										                  */
/*  ruft auf :				gespeichert in :			            */
/*  - all_ws()                      wahrsch.c                                 */
/*  										                  */
/*  Berechnung der Auslastung eines MMm-FCFS-ASYM-Knotens nach Hertel         */
/*										                  */
/******************************************************************************/

double rho_zufall(i,lamb)
int i;
double lamb;
{
   int r;
   double eta[10]; 
   double ce = 0, N = 0;
   struct Zustand *ptr1,*ptr2;
   double h =.0;

   if (lamb > null)
   {
   /* erster Schritt: kanalspez. Auslastungen Verlustsystem */
   /* ----------------------------------------------------- */
      ptr1 = ptr2 = all_ws(i);
      for (r = 0; r < 10; r++) eta[r] = 0;
          while (ptr1 != NULL)
          {
              for (r = 0; r < mm[i]; r++)
                 if (ptr1->belegt[r] == 1) 
			  eta[r] = eta[r] + ptr1->value;
              ptr1 = ptr1->next;
           }
           while (ptr2->next != NULL) ptr2 = ptr2->next;
      
   /* zweiter Schritt: kanalspez. Auslastungen Wartesystem */
   /* ---------------------------------------------------- */
      for (r = 0; r < mm[i]; r++) 
         ce = ce + (amue[i][r]/lamb);
      ce = 1 / ce;

      N = 1 + (ptr2->value * ce / (1 - ce));

      for (r = 0; r < mm[i]; r++)
         eta[r] = (eta[r] + (ptr2->value * ce / (1 - ce)))/ N;

   /* dritter Schritt: Gesamtauslastung */
   /* --------------------------------- */
      for (r = 0; r < mm[i]; r++)
         h = h + (1 / eta[r]);
      h = mm[i] * (1 / h);       

	if (h > 1) return(1);
	else return (h);
   }
   else
   return(MINRHO);
}  


/******************************************************************************/
/*										                  */
/*  FUNKTION									            */
/*  Verlust()                 					                  */
/*										                  */
/*  Wird aufgerufen in :		gespeichert in :			            */
/*  - rho_schnell()                 wahrsch.c                                 */
/*  - p_wart_schnell()              wahrsch.c                                 */
/*  										                  */
/*  Berechnung der Verlustwahrscheinlichkeit eines MMm-FCFS-ASYM-Knotens      */
/*  nach Hertel                                                               */
/*										                  */
/******************************************************************************/

double Verlust(anz,bedien,lam)
   int anz;
   double bedien[];
   double lam;
{
   double value = 1;                       /* Rueckgabewert                  */
   double bed_zaehler[10], bed_nenner[10]; /* Argumente fuer rekursiven Auf- */
                                           /* ruf der Funktion               */
   int i,k;                                /* Schleifenindices               */

   if (anz == 0)
	return(1);
   
   else if (anz == 1)
   {
	value = lam/(lam + bedien[0]);
	return(value);
   }

   else
   {
	for (k = 1; k < anz; k++)
	{
	   /* Feld fuer Bedienraten im Zaehler initialisieren */
	   /* =============================================== */
		for (i = 0; i < k-1; i++)
		   bed_zaehler[i] = bedien[i];
		bed_zaehler[k-1] = bedien[anz-1];


	   /* Feld fuer Bedienraten im Nenner initialisieren */
	   /* ============================================== */
		for (i = 0; i < k-1; i++)
		   bed_nenner[i] = bedien[i];
		bed_nenner[k-1] = bedien[k-1] + bedien[anz-1];

	   value = value*(Verlust(k,bed_zaehler,lam)/Verlust(k,bed_nenner,lam));
	}

	value = 1 + (value * bedien[anz-1] / lam);
	value = 1 / value;
	value = value * Verlust(anz-1,bedien,lam);

	return(value);
   }
}




/******************************************************************************/
/*										                  */
/*  FUNKTION									            */
/*  p_wart_schnell()                  					            */
/*										                  */
/*  Wird aufgerufen in :		gespeichert in :			            */
/*  - Pm()                 	      wahrsch.c		                        */
/*  										                  */
/*  ruft auf :				gespeichert in :			            */
/*  - Verlust()                     wahrsch.c                                 */
/*  										                  */
/*  Berechnung der Wartewahrscheinlichkeit nach Hertel.                       */
/*										                  */
/******************************************************************************/

double p_wart_schnell(rho,knoten)
double rho;
int knoten;
{

   int r;
   double val=1.0;
   double p_bel=0,ce=0,N=0,lamb=0;

   if (rho >= 1.0)    return 1.0;
   if (rho <= MINRHO) return 0.0;

/* Berechnung der Belegt-Wahrscheinlichkeit */
/* ---------------------------------------- */
   lamb = lambda[0] * besh[0][knoten];
   p_bel = Verlust(mm[knoten],amue[knoten],lamb);

/* Berechnung der Konstanten ce */
/* ---------------------------- */
   for (r = 0; r < mm[knoten]; r++) 
      ce = ce + (amue[knoten][r]/lamb);
   ce = 1 / ce;

/* Berechnung der Konstanten N */
/* --------------------------- */
   N = 1 + (p_bel * ce / (1 - ce));

/* Berechnung der Wartewahrscheinlichkeit */
/* -------------------------------------- */
   val = p_bel / (N * (1 - ce));
   return(val);
}




/******************************************************************************/
/*										                  */
/*  FUNKTION									            */
/*  rho_schnell()                  					                  */
/*										                  */
/*  Wird aufgerufen in :		gespeichert in :			            */
/*  - tq_MMm_FCFS_Pm       	      algo.c                                    */
/*  										                  */
/*  ruft auf :				gespeichert in :			            */
/*  - all_ws()                      wahrsch.c                                 */
/*  										                  */
/*  Berechnung der Auslastung eines MMm-FCFS-ASYM-Knotens nach Hertel         */
/*										                  */
/******************************************************************************/

double rho_schnell(i,lamb)
int i;
double lamb;
{
   int r,j,k;
   double eta[10]; 
   double P_v;
   double ce = 0, N = 0;
   double h =.0;

   if (lamb > null)
   {
   /* erster Schritt: kanalspez. Auslastungen Verlustsystem */
   /* ----------------------------------------------------- */
	for (r = 0; r < mm[i]; r++)
	   eta[r] = (lamb / amue[i][r]) * (Verlust(r,amue[i],lamb)
		     - Verlust(r+1,amue[i],lamb));
      
   /* zweiter Schritt: kanalspez. Auslastungen Wartesystem */
   /* ---------------------------------------------------- */
      for (r = 0; r < mm[i]; r++) 
         ce = ce + (amue[i][r]/lamb);
      ce = 1 / ce;

	P_v = Verlust(mm[i],amue[i],lamb);

      N = 1 + (P_v * ce / (1 - ce));

      for (r = 0; r < mm[i]; r++)
         eta[r] = (eta[r] + (P_v * ce / (1 - ce)))/ N;

   /* dritter Schritt: Gesamtauslastung */
   /* --------------------------------- */
      for (r = 0; r < mm[i]; r++)
         h = h + (1 / eta[r]);
      h = mm[i] * (1 / h);       

	if (h > 1) return(1);
	else return (h);
   }
   else
   return(MINRHO);
}  


/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  weight()									*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - scat()				appscat.c				*/
/*  										*/
/*  Berechnung der Wahrscheinlichkeitsmasse fuer die				*/
/*  Randwahrscheinlichkeiten							*/
/*										*/
/********************************************************************************/


weight(limit)

   int		limit; 	 /* Wie weit soll die Wahrscheinlichkeitsmasse 		*/
			 /* sich erstrecken 					*/
{

   int 		i, j;
   double 	sum;



/* Skalierungsfunktion */
/*---------------------*/
   PR[0] = 0.0;
   PR[1] = ALPHA; 
   for (i = 2 ; i <= limit ; i++) 
      PR[i] = BETA * PR[i-1];

/* Berechnung der Wahrscheinlichkeitsmasse */
/*-----------------------------------------*/
   W[0][0] = 1.0;


   for (i = 1 ; i <= limit ; i++) 
      {
      for (sum = 0.0 ,j = 0; (j <= i-1) && (j <= STREUUNG) ; j++) 
         {
	 W[i][j] = W[i-1][j] * (100.0 - PR[i]) / 100.0;
	 sum += W[i][j];
	 }
      if (i <= STREUUNG)
         W[i][i] = 1.0 - sum;
      }


}






/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  FindMargProbs()								*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - tq_MMm_FCFS_RW()			algo.c					*/
/*  										*/
/*  Berechnung der Randwahrscheinlichkeiten fuer SCAT.				*/
/*  Literatur:	Feldmann 84,							*/
/*		Akyildiz/Bolch/Cordes 86					*/
/*										*/
/********************************************************************************/


FindMargProbs(knoten,kqd,klasse)

   int		knoten;  		/* Laufvariable fuer Knoten		*/
   double	kqd;      		/* m. Auftagszahl 			*/
   int		klasse;   		/* Laufvariable fuer Klassen		*/

{
/* Hilfsgroessen */
/*---------------*/
   int	floor, ceiling, maxval, j, l_dist, u_dist, upperval, lowerval;
   double	nix;
   int	KR;

   


/************************************************************/
/* Die Funktion weight() wurde bereits in scat() aufgerufen */
/************************************************************/



/* Anzahl der Auftraege im Zustand (K-1r) in allen Klassen */
/*---------------------------------------------------------*/
   for (KR=0, j = 0 ; j<R ; j++) 
       KR += (int)K[j];

   KR--;
   
   floor = (int)kqd ;
   ceiling = floor + 1 ;

   if (2*floor+1 < mm[knoten] ) 
       maxval = 2 * floor +1 ;
   else maxval = mm[knoten];

   if ( floor == KR || N == 1 )
      {
      for (j=0 ; j<= mm[knoten]-2 ; j++) 
         {
         if ( j == KR )
	    P[j] = 1.0;
	 else
	    P[j] = 0.0;
	 }
      }
   
   else
      {
      for (j=0 ; j<= mm[knoten]-2 ; j++) 
         {
         if (j<=maxval) 
            {
	    if (j<= floor) 
	       {
	       l_dist = floor -j;
	       upperval = l_dist +ceiling;
	       lowerval = j;
   	
	       if (upperval > KR) 
	          upperval = KR;
   	
	       if (l_dist > STREUUNG) 
	          P[j] = 0.0;
	       else 
	          {
	          nix = (upperval - kqd) / (upperval -lowerval) ;
	          P[j] = nix * W[floor][l_dist] ;
	          }
               }
 	    else 
               {
	       u_dist = j - ceiling;
	       if (u_dist > STREUUNG) 
	          P[j] = 0.0;
	       else if (j > KR)
	          P[KR] += W[floor][u_dist] - P[floor - u_dist];
	       else
	          P[j] = W[floor][u_dist] - P[floor- u_dist];
	       }
	    }
         else 
            P[j] = 0.0;
         }
      }
   }
   

/********************************************************************************/
/*										*/
/*  FUNKTION									*/
/*  ChNeProbs()									*/
/*										*/
/*  Wird aufgerufen in :		gespeichert in :			*/
/*  - si_mva()				appscat.c				*/
/*  										*/
/*  Berechnung der Randwahrscheinlichkeiten fuer SCAT,				*/
/*  nach Chandy/Neuse								*/
/*  Literatur:	Feldmann 84,							*/
/*		Akyildiz/Bolch/Cordes 86					*/
/*										*/
/********************************************************************************/

ChNeProbs(knoten,kqd)

   int		knoten;    /* Laufvariable fuer Knoten		 */
   double	kqd;      /* m. Auftagszahl 			 */

{
/* Hilfsgroessen */
/*---------------*/
   int	floor, ceiling, j;

   floor = (int)kqd ;
   ceiling = floor + 1 ;

   for (j=0 ; j<= mm[knoten]-2 ; j++) 
      P[j] = 0.0;

   if ((ceiling <= mm[knoten]-2) && ( ceiling > 0)) 
      {
      P[ceiling] = kqd - floor;
      P[floor] = ceiling - kqd ;
      }
}

