/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tbounds.c\t1.3\t94/02/04 11:54:41";

/********************************************************************* 
***                                                                ***
***  Modul:     b o u n d s . c                                    ***
***                                                                ***
***----------------------------------------------------------------***
***                                                                ***
***  Datum:     26.08.87                                           ***
***  Version:   1.0                                                ***
***  Autor:     Gerhard Schmuck                                    ***
***  Inhalt:    Asymptotische Analysen ABA und BJB                 ***
***                                                                ***
**********************************************************************


**********************************************************************
*              Deklarationen und Definitionen                        *
**********************************************************************
*/
#include <stdio.h>
#include <signal.h>
#include "d_netz.h"
#include "stdfkt.h"
#include "memory.h"

#define  ANKRANZ    20
#define  AUFTSTEP   1

extern double atof ();
extern int    UEW_BESH ();

	void ABA();
	void BJB();
	void ELEM_GROESSEN();
	void INP_NETZ();
	int  CHECK_CLN();

netzparameter net;
leistgroessen lgr;

char     netztyp;                /* Netztyp c, o */

int      knottyp [NMAX],         /* Knotentyp ( IS = 0) */
         auft_ug, auft_og, auft, /* Auftragsanzahlen */
         punkte,                 /* Stuetzstellenanzahl */
         uewflag;

float    ankrstep;               /* Schrittweite bei Ankunftsraten */
         
double   ankr_ug, ankr_og, ankr, /* externe Ankunftsraten */
         besh [NMAX],            /* rel. Besuchshfgkt. */
         bedr [NMAX],            /* Bedienraten */
         last_sum,               /* Summe der Einzellasten */
         last_max,               /* groesste Einzellast */
         last_ave,               /* Durchschnittslast */
         denkzeit,               /* "Terminallast" */
         dusa_max, dusa_min,     /* Systemdurchsaetze */
         msvz_max, msvz_min,     /* Mittlere Systemverweilzeiten */
         auft_sat,               /* Saettigungsauftragsanzahl */
         ankr_sat;               /* Saettigungsankunftsrate */
/*
**********************************************************************
*                                                                    *
*                      Hauptprogramm BOUNDS                          *
*                                                                    *
**********************************************************************
*/

void abbruch(sig)
int sig;
{
        printf("bounds_signal_%d\n 0 0 0 \n", sig);
        exit(1);
}

int main ( argc, argv)
      int   argc;
      char  *argv[];
{
    /*		Sccs support from SccsLogging.c			*/	
    extern int SccsRelease;
    extern int SccsLevel;

    int   kn;
    char  *comline;

    {
	int i;
	void abbruch();

	for (i=1; i<12; i++)
	signal (i, abbruch);
    }

    if ((argc > 1) && (!strcmp(argv[1], "-V")))    /* kirschni 06.03.92 */
    {
	fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
	exit(0);
    }

      comline  = "  dusa-min   dusa-max   msvz-min   msvz-max";
      EXITCTRL ();
      INP_NETZ ();

      /*-------------------- Uew - Beshauf - Umrechnung --------------*/
      if ( uewflag == 1) {
            net.klass   = 1;    /* Hilfswerte, die die Funktion */
            net.ankr[0] = 1;    /* UEW_BESH benoetigt.          */
            net.besh    = NR_ALLOC ( net.knot, 1);
            UEW_BESH    ();
            NNR_FREE    ( net.uew, net.knot);
            for ( kn=1; kn <= net.knot; kn++) besh[kn-1] = net.besh[kn][0];
            NNR_FREE    ( net.uew, net.knot);
            NR_FREE     ( net.besh, net.knot);
      }

      punkte   = CHECK_CLN ( argc, argv);
      ELEM_GROESSEN ();

      /*-------------------- Offenes Netz --------------------------*/
      if ( netztyp == 'o') {
            printf ( "# Netztyp, Stuetzstellen, Saett.ank.rate\n");           
            printf ( "%c  %d  %.4f\n", netztyp, punkte, ankr_sat);
            
            /*-------------- ABA - Berechnung ----------------------*/
            printf ( "#\n# ABA - Ergebnisse\n");
            printf ( "# Ext.Ank.%s\n", comline);
            ankr   = ankr_ug;
            while  ( ankr <= ankr_og) {
                  ABA    ( ankr, 'o');
                  printf ( "%9.4f  %9.4f  %9.4f  %9.4f  %9.4f\n",
                           ankr, dusa_min, dusa_max, msvz_min, msvz_max);
                  ankr   += ankrstep;
            }
            /*-------------- BJB - Berechnung ----------------------*/
            printf ( "#\n# BJB - Ergebnisse\n");
            printf ( "# Ext.Ank.%s\n", comline);
            ankr   = ankr_ug;
            while  ( ankr <= ankr_og) {
                  BJB    ( ankr, 'o');
                  printf ( "%9.4f  %9.4f  %9.4f  %9.4f  %9.4f\n",
                           ankr, dusa_min, dusa_max, msvz_min, msvz_max);
                  ankr   += ankrstep;
            }
      }
      /*-------------------- Geschlossenes Netz ---------------------*/
      else  {
            printf ( "# Netztyp, Stuetzstellen, Saett.auft.anz\n");
            printf ( "%c  %d  %.4f\n", netztyp, punkte, auft_sat);

            /*-------------- ABA - Berechnung -----------------------*/
            printf ( "#\n# ABA - Ergebnisse\n");
            printf ( "# Auft.anz%s\n", comline);
            auft   = auft_ug;
            while  ( auft <= auft_og) {
                  ABA    ( (double) auft, 'c');
                  printf ( "%9d  %9.4f  %9.4f  %9.4f  %9.4f\n",
                           auft, dusa_min, dusa_max, msvz_min, msvz_max);
                  auft   += AUFTSTEP;
            }
            /*-------------- BJB - Berechnung -----------------------*/
            printf ( "#\n# BJB - Ergebnisse\n");
            printf ( "# Auft.anz%s\n", comline);
            auft   = auft_ug;
            while  ( auft <= auft_og) {
                  BJB    ( (double) auft, 'c');
                  printf ( "%9d  %9.4f  %9.4f  %9.4f  %9.4f\n",
                           auft, dusa_min, dusa_max, msvz_min, msvz_max);
                  auft   += AUFTSTEP;
            }
      }
	exit(0);
}
/*
**********************************************************************
*                                                                    *
*                        Programm - Module                           *
*                                                                    *
**********************************************************************


**********************************************************************
*                         Funktion ABA                               *
**********************************************************************
Asymptotic Bound Analyse fuer einen Stuetzwert
*/
void ABA ( stuetz, netztyp)
      double stuetz;
      char   netztyp;
{
      if ( netztyp == 'o') {
            dusa_max = 1.0 / last_max;
            dusa_min = 0;
            msvz_max = 0;
            msvz_min = last_sum;
            }
      else  {
            dusa_max = MIN ( stuetz / ( last_sum + denkzeit),
                             1.0 / last_max );
            dusa_min = stuetz / ( stuetz * last_sum + denkzeit);
            msvz_max = stuetz * last_sum;
            msvz_min = MAX ( last_sum, stuetz * last_max - denkzeit);
      }
}
/*
**********************************************************************
*                           Funktion BJB                             *
**********************************************************************
Balanced Job Bound Analyse fuer einen Stuetzwert
*/
void BJB ( stuetz, netztyp)
      double stuetz;
      char   netztyp;
{
      double help1, help2, help;

      if ( netztyp == 'o') {
            dusa_max = 1.0 / last_max;
            dusa_min = 0.0;
            help     = stuetz * last_max;
            if ( help == 1)
                  msvz_max = -1;
            else
                  msvz_max = last_sum / ( 1 - help);
            help     = stuetz * last_ave;
            if ( help == 1)
                  msvz_min = -1;
            else
                  msvz_min = last_sum / ( 1 - help);
            }
      else  {
            help1    = ( stuetz - 1) * last_ave / 
                       ( 1 + denkzeit / last_sum);
            help2    = ( stuetz - 1) * last_max /
                       ( 1 + denkzeit / ( stuetz * last_sum));
            dusa_max = MIN ( 1 / last_max,
                             stuetz / ( last_sum + denkzeit + help1) );
            dusa_min = stuetz / ( last_sum + denkzeit + help2);
            msvz_max = last_sum + help2;
            msvz_min = MAX ( stuetz * last_max - denkzeit,
                             last_sum + help1 );
      }
}
/*
**********************************************************************
*                       Funktion CHECK_CLN                           *
**********************************************************************
Bereiche der Stuetzstellen aus Kommandozeile festlegen
Return:  Anzahl der Stuetzstellen
*/
int CHECK_CLN ( argc, argv)
      int    argc;
      char   *argv[];
{
      int    anz;
      double wert1, wert2;

      ankrstep = 1;
      switch ( argc) {

         case 1: /* kein Parameter: Vorbesetzen mit Eingabewert */
                 if ( netztyp == 'o')
                       ankr_og = ankr_ug;
                 else
                       auft_og = auft_ug;
                 anz = 1;
                 break;

         case 2: /* ein Parameter: Besetzen mit Parameterwert */
                 wert1 = atof ( argv[1]);
                 if ( wert1 <= 0) {
                       fprintf ( stderr,
                                 "Nur gueltig im positiven Bereich.\n");
		       printf("bounds_versagt\n 0 0 0\n");
                       /* printf  ( "\004"); */
                       exit    ( 6);
                 }
                 if ( netztyp == 'o')
                       ankr_og = ankr_ug = wert1;
                 else
                       auft_og = auft_ug = (int) wert1;
                 anz = 1;
                 break;

         default: /* zwei und mehr Parameter: kompletter Bereich */
                 wert1 = atof ( argv[1]);
                 wert2 = atof ( argv[2]);
                 if ( wert1 <= 0 || wert2 <= 0) {
                       fprintf ( stderr,
                                 "Nur gueltig im positiven Bereich.\n");
		       printf("bounds_versagt\n 0 0 0\n");
                       exit    ( 6);
                 }
                 if ( netztyp == 'o') {
                       ankr_ug = MIN ( wert1, wert2);
                       ankr_og = MAX ( wert1, wert2);
                       ankrstep= ( ankr_og - ankr_ug) / ANKRANZ;
                       anz     = ANKRANZ + 1;
                       }
                 else  {
                       auft_ug = ( int) MIN ( wert1, wert2);
                       auft_og = ( int) MAX ( wert1, wert2);
                       anz     = ( int) (( auft_og - auft_ug) / AUFTSTEP + 1);
                 }
      }
      return ( anz);
}
/*
**********************************************************************
*                     Funktion ELEM_GROESSEN                         *
**********************************************************************
Berechnen der elementaren bounds-Groessen
*/
void ELEM_GROESSEN ()
{
      int    n, ger_anz = 0;
      double last_einz = 0.0;

      last_max = 0.0;

      for ( n=0; n < net.knot; n++)
      if  ( knottyp[n] == 0 && netztyp == 'c') /* Terminalknoten */
            denkzeit  = besh[n] / bedr[n];
      else  {                    /* "normaler" Geraeteknoten */
            last_einz = besh[n] / bedr[n];
            if ( last_einz > last_max)
                  last_max = last_einz;
            last_sum  += last_einz;
            ger_anz++;
      }
      last_ave = last_sum / ger_anz;
      /* Saettigungsgroessen */
      ankr_sat = 1.0 / last_max;
      auft_sat = ( last_sum + denkzeit) / last_max;
}
/*
**********************************************************************
*                       Funktion INP_NETZ                            *
**********************************************************************
Einlesen der Netzdaten von Standarteingabe
*/
void INP_NETZ ()
{
      int  m, n;

   /* Uewflag, Knotenzahl */
      JUMPCOM ( stdin);
      scanf   ( "%d %d", &uewflag, &net.knot);
   /* Bedieneinheiten pro Knoten = Knotentyp */
      JUMPCOM ( stdin);
      for ( n=0; n < net.knot; n++) scanf ( "%d", &knottyp[n]);
   /* Auftragszahl */
      JUMPCOM ( stdin);
      scanf   ( "%d", &auft_ug);
   /* Ankunftsrate */
      JUMPCOM ( stdin);
      scanf   ( "%lf", &ankr_ug);
      if ( auft_ug > 0)
            netztyp = 'c';
      else
            netztyp = 'o';
   /* Bedienraten */
      JUMPCOM ( stdin);
      for ( n=0; n < net.knot; n++) scanf ( "%lf", &bedr[n]);
   
      JUMPCOM ( stdin);
   /* Uebergangswahrscheinlichkeiten ... */
      if ( uewflag == 1) {
            net.uew = NNR_ALLOC ( net.knot, 1);
            for ( m=0; m <= net.knot; m++)
                  for ( n=0; n <= net.knot; n++)
                        scanf ( "%f", &net.uew[m][n][0]);
            }
   /* ... oder relative Besuchshfgkt. */
      else
            for ( n=0; n < net.knot; n++) scanf ( "%lf", &besh[n]);
}
/*
********************* Ende Modul bounds.c ******************************/

