static char *SccsId = "@(#)PEPSY:\tmemory.c\t1.3\t94/02/04 11:54:48";
/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/


/*********************************************************************
***                                                                ***
***  Modul:     m e m o r y . c                                    ***
***                                                                ***
***----------------------------------------------------------------***
***                                                                ***
***  Datum:     13.07.87                                           ***
***  Version:   1.0                                                ***
***  Autor:     Gerhard Schmuck                                    ***
***  Inhalt:    Funktionen zur dynamischen Speicherverwaltung der  ***
***             Netzparameter                                      ***
***             ACHTUNG: Knoten- und Klassenanzahlen ( net.knot,   ***
***                      net.klass) muessen bekannt sein           ***
***                                                                ***
**********************************************************************
*/
#include <stdio.h>
#include "d_netz.h"
extern void cfree();
extern char *calloc();

extern leistgroessen lgr;
/*
**********************************************************************
*                     Funktion NNR_ALLOC                             *
**********************************************************************
Erzeugen eines Feldes [NMAX+1][NMAX+1][RMAX]
*/
float ***NNR_ALLOC ( knot, klass)
      int knot, klass;
{
      float        ***ptr;
      int          i, j;
      unsigned int size, kn, kl;

      size = sizeof ( float);
      kn   = knot + 1;
      kl   = klass;

      ptr  = ( float ***) calloc ( kn, size);
      for ( i=0; i <= knot; i++) {
            ptr[i] = ( float **) calloc ( kn, size);
            for ( j=0; j <= knot; j++)
                  ptr[i][j] = ( float *) calloc ( kl, size);
      }
      if ( ptr == 0)
      {
	    printf("bounds_speicher\n 0 0 0\n");
            exit ( 1);
      }
      return ( ptr);
}
/*
**********************************************************************
*                      Funktion NN_ALLOC                             *
**********************************************************************
Erzeugen eines Felde [NMAX+1][NMAX+1]
*/
float **NN_ALLOC ( knot)
      int  knot;
{
      float         **ptr;
      int           i;
      unsigned int  size, kn;
 
      size = sizeof ( float);
      kn   = knot + 1;

      ptr  = ( float **) calloc ( kn, size);
      for ( i=0; i <= knot; i++)
            ptr[i] = ( float *) calloc ( kn, size);
      if ( ptr == 0)
      {
	    printf("bounds_speicher\n 0 0 0\n");
            exit ( 1);
      }
      return ( ptr);
}
/*
**********************************************************************
*                      Funktion NR_ALLOC                             *
**********************************************************************
Erzeugen eines Feldes [NMAX+1][RMAX]
*/
double **NR_ALLOC ( knot, klass)
      int  knot, klass;
{
      double        **ptr;
      int           i;
      unsigned int  size, kn, kl;

      size = sizeof ( double);
      kn   = knot + 1;
      kl   = klass;

      ptr  = ( double **) calloc ( kn, size);
      for ( i=0; i <= knot; i++)
            ptr[i] = ( double *) calloc ( kl, size);
      if ( ptr == 0)
      {
	    printf("bounds_speicher\n 0 0 0\n");
            exit ( 1);
      }
      return ( ptr);
}
/*
**********************************************************************
*                        Funktion NNR_FREE                           *
**********************************************************************
Freigeben eines 3dim. Feldes
*/
void NNR_FREE ( ptr, knot)
      float ***ptr;
      int   knot;
{
      int   i, j, kn;

      kn    = knot + 1;
      for ( i=0; i < kn; i++) {
            for ( j=0; j < kn; j++)
                  if ( ptr[i][j] != 0)
                        cfree ( ( char *) ptr[i][j]);
            if ( ptr[i] != 0)
                  cfree ( ( char*) ptr[i]);
      }
      if ( ptr != 0)
            cfree ( ( char*) ptr);
}
/*
**********************************************************************
*                       Funktion NN_FREE                             *
**********************************************************************
Freigeben eines 2dim. Feldes
*/
void NN_FREE ( ptr, knot)
      float **ptr;
      int   knot;
{
      int kn, i;

      kn  = knot + 1;
      for ( i=0; i < knot; i++)
            if ( ptr[i] != 0)
                  cfree ( ( char *) ptr[i]);
      if ( ptr != 0)
            cfree ( ( char *) ptr);
}
/*
**********************************************************************
*                         Funktion NR_FREE                           *
**********************************************************************
Freigeben eines 2dim. Feldes
*/
void NR_FREE ( ptr, knot)
      double **ptr; 
      int    knot;
{    
      int    kn, i;

      kn     = knot + 1;
      for ( i=0; i < kn; i++)
            if ( ptr[i] != 0)
                  cfree ( ( char *) ptr[i]);
      if ( ptr != 0)
            cfree ( ( char *) ptr);
}
/*
*********************************************************************
*                         Funktion ALLOC_LGR                        *
*********************************************************************
Speicherplatz fuer Leistungsgroessen bereitstellen
*/
void ALLOC_LGR ( knot, klass)
      int  knot, klass;
{
      lgr.lambda = NR_ALLOC ( knot, klass);
      lgr.rho    = NR_ALLOC ( knot, klass);
      lgr.vzeit  = NR_ALLOC ( knot, klass);
      lgr.wzeit  = NR_ALLOC ( knot, klass);
      lgr.mauft  = NR_ALLOC ( knot, klass);
      lgr.mqlen  = NR_ALLOC ( knot, klass);
} 
/*
*********************************************************************
*                       Funktion FREE_LGR                           *
*********************************************************************
Speicherplatz fuer Leistungsgroessen freigeben
*/
void FREE_LGR ( knot)
      int knot;
{
      NR_FREE  ( lgr.lambda, knot);
      NR_FREE  ( lgr.rho, knot);
      NR_FREE  ( lgr.vzeit, knot);
      NR_FREE  ( lgr.wzeit, knot);
      NR_FREE  ( lgr.mauft, knot);
      NR_FREE  ( lgr.mqlen, knot);
}
/******************* Ende des Moduls  memory.c **********************/

