static char *SccsId = "@(#):PEPSY\tchylla.c\t1.5\t94/02/03 14:39:26";
/****************************************************************/
/****************************************************************/
/*****   Programm zur Analyse von offenen Warteschlangen-   *****/
/*****   netzen nach der Dekompositionsmethode von Kuehn    *****/
/****************************************************************/
/****************************************************************/

#include <stdio.h>
#include <math.h>
#include "xxalloc.h"

/****************************************************************/
/******     Konstantendefinition :                         ******/
/******                                                    ******/
/******       epsi     Konstante zur Ueberpruefung der     ******/
/******                Abbruchbedingung                    ******/
/****************************************************************/

#define epsi 0.002


/****************************************************************/
/******     Variablendeklaration :                         ******/
/******                                                    ******/
/******       lam      Ankunftsrate                        ******/
/******       auslam   externe Ankunftsrate                ******/
/******       ca       qu. Varkoef. der Zwischenankunfts-  ******/
/******                zeitverteilung                      ******/
/******       ca1      qu. Varkoef. der Ankunftszeitver-   ******/
/******                teilung fuer Knoten                 ******/
/******       ca2      qu. Varkoef. der Ankunftszeitver-   ******/
/******                teilung fuer Knoten und Klasse      ******/
/******       c0       qu. Varkoef. der externen Ankunfts- ******/
/******                zeitverteilung                      ******/
/******       geslam   gesamte Ankunftsrate                ******/
/******       mue      Bedienrate                          ******/
/******       gesmue   gesamte Bedienrate                  ******/
/******       cb       qu. Varkoef. der Bedienzeitver-     ******/
/******                teilung                             ******/
/******       cd       qu. Varkoef. der gesamten Bedien-   ******/
/******                zeitverteilung                      ******/
/******       cd1      qu. Varkoef. des Abgangsprozesses   ******/
/******       p        Uebergangswahrscheinlichkeiten      ******/
/******       e        Besuchshaeufigkeiten                ******/
/******       ueber    Uebergangsraten                     ******/
/******       wert     Hilfsvariable zur Ueberpruefung der ******/
/******                Abbruchbedingung                    ******/
/******       maa      mittlere Anzahl der Auftraege       ******/
/******       typ      Nummer des Knotentyps               ******/
/******       n        Anzahl Knoten                       ******/
/******       r        Anzahl Klassen                      ******/
/******       m        Anzahl der Bedieneinheiten          ******/
/******       te       Iterationsschrittezaehlvariable     ******/
/******       pruef    Hilfsvariable                       ******/
/****************************************************************/
 
double **mue,**lam,*auslam,**e,
      **c0,***ca,**cb,*geslam,
      *gesmue,*cd,**maa,***wert,
      ***ueber,*ca1,**ca2,*cd1,
      **sp,***p;
int *typ,n,r,pruef,te,*m;

void abbruch(sig)
int sig;
{
        printf("chylla_signal_%d\n 0 0 0 \n", sig);
        exit(1);
}

main(argc, argv)
	int	argc;
	char *argv[];
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

	int i;

	void abbruch();

	for (i=1; i<12; i++)
		signal(i, abbruch);

	if ((argc > 1) && (!strcmp(argv[1], "-V")))    /* kirschni 22.11.1991 */
	{
		fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
		exit(0);
	}

	eingabe();
	besuch();
	berechnung();
	bedienrate();
	init();
	pruef=1;
	te=0;
	while (pruef!=0)
	{
		te=te+1;
		for (i=1;i<=n;i=i+1)
		{
			merge1(i);
			merge(i);
			departure(i);
			split(i);
		}
		abweichung();
	}
	mitauftr();
	ausgabe();
	exit(0);
}


/***************************/
/** Eingabe der Netzdaten **/
/***************************/

eingabe()
{
  int i,j,l,h1;
  double h;

  scanf("%d",&n);
  scanf("%d",&r);

/* Initialisierung aller globalen Felder */

  mue = d2_alloc(0,n, 0,r);
  lam = d2_alloc(0,n, 0,r);
  auslam = d1_alloc(0,r);
  e = d2_alloc(0,n, 0,r);
  c0 = d2_alloc(0,n, 0,r);
  ca = d3_alloc(0,r, 0,n, 0,n);
  cb = d2_alloc(0,n, 0,r);
  geslam = d1_alloc(0,n);
  gesmue = d1_alloc(0,n);
  cd = d1_alloc(0,n);
  maa = d2_alloc(0,n, 0,r);
  wert = d3_alloc(0,r, 0,n, 0,n);
  ueber = d3_alloc(0,r, 0,n, 0,n);
  ca1 = d1_alloc(0,n);
  ca2 = d2_alloc(0,n, 0,r);
  cd1 = d1_alloc(0,n);
  sp = d2_alloc(0,n, 0,r);
  p = d3_alloc(0,r, 0,n, 0,n);

  typ = i1_alloc(0,n);
  m = i1_alloc(0,n);

  for (i=1;i<=n;i=i+1)
   {
     scanf("%d",&h1);
     typ[i]=h1;
     if ((typ[i]==2)||(typ[i]==6)||(typ[i]==8))
      {
        scanf("%d",&h1);
        m[i]=h1;
      }
     else
        m[i]=1;
   }
  for (j=1;j<=r;j=j+1)
    {
       scanf("%lf",&h);
       auslam[j]=h;
    }
  for (j=1;j<=r;j=j+1)
   {
     for (i=1;i<=n;i=i+1)
      {
        scanf("%lf",&h);
        mue[i][j]=h;
        if ((typ[i]==5)||(typ[i]==6)||(typ[i]==7)||(typ[i]==8))
	 {
          scanf("%lf",&h);
          cb[i][j]=h;
	 }
        else
          cb[i][j]=1;
      }
     for (i=0;i<=n;i=i+1)
      {
       for (l=0;l<=n;l=l+1)
        {
          scanf("%lf",&h);
          p[j][i][l]=h;
        }
      }
   }
  for (j=1;j<=r;j=j+1)
   {
     for (i=1;i<=n;i=i+1)
       if (p[j][0][i]!=0)
        {
          scanf("%lf",&h);
          c0[i][j]=h;
        }
   }
}


/*********************************************/
/** Bestimmung der Besuchshaeufigkeiten aus **/
/** den Uebergangswahrscheinlichkeiten      **/
/*********************************************/

besuch()
{
	int i,j,l,zei,nzei,spa,x;
	double **li,*re,*q,hi,hilf;

	li	= d2_alloc(0,n+1, 0,n+1);
	re	= d1_alloc(0,n+1);
	q	= d1_alloc(0,n+1);

	for (j=1;j<=r;j++)
	{
		for (i=0;i<=n;i++)
		{
			for (l=1;l<=n;l++)
			{
				if (i==0)
					re[l]=0-p[j][i][l];
				else
				{
					if (i==l)
						li[i][l]=p[j][i][l]-1;
					else
						li[i][l]=p[j][l][i];
				}
			}
		}

		for (zei=1;zei<=n;zei++)
		{
			if (li[zei][zei]==0)
			{
				hilf=li[zei][zei];
				x=0;
				while (hilf==0)
				{
					x++;
					hilf=li[zei][zei+x];
				}

				for (i=0;i<=n;i=i+1)
				{
					if (i==0)
					{
						hi		=re[i];
						re[i]	=re[i+x];
						re[i+x]	=hi;
					}
					else
					{
						hi			=li[zei][i];
						li[zei][i]	=li[zei][i+x];
						li[zei][i+x]=hi;
					}
				}
			}
			else
			{
				for (nzei=1;nzei<=n;nzei++)
					if (nzei!=zei)
					{
						q[nzei]=0-(li[nzei][zei]/li[zei][zei]);
						for (spa=1;spa<=n;spa++)
						{
							if (spa==zei)
								li[nzei][spa]=0;
							else
								li[nzei][spa]=li[nzei][spa]+q[nzei]*li[zei][spa];
						}
						re[nzei]=re[nzei]+q[nzei]*re[zei];
					}
			}
		}

		for (i=1;i<=n;i++)
		{
			e[i][j]=(re[i] == 0.0)?0.0:re[i]/li[i][i];
		}
	}
}


/*************************************/
/** Bestimmung der Ankunftsraten an **/ 
/**     jedem Knoten                **/
/*************************************/

berechnung()
{
  int o,l,g;
  double h;

  for (o=1;o<=n;o=o+1)
    for (l=1;l<=r;l=l+1)
      lam[o][l]=auslam[l]*e[o][l];
  for (l=1;l<=r;l=l+1)
   for (o=0;o<=n;o=o+1)
    for (g=1;g<=n;g=g+1)
     {
       if (o==0)
         ueber[l][o][g]=auslam[l]*p[l][o][g];
       else
         ueber[l][o][g]=lam[o][l]*p[l][o][g];
     }
  for (l=1;l<=n;l=l+1)
   {
    h=0;
    for (o=1;o<=r;o=o+1)
      h=h+lam[l][o];
    geslam[l]=h;
   }
}


/*******************************************/
/** Berechnung der Bedienrate und des     **/ 
/** quadratischen Variationskoeffizienten **/
/** der gesamten Bedienzeitverteilung     **/
/*******************************************/

bedienrate()
{
  int l1,l2,l3;
  double h,h1,h2,h3;

  for (l1=1;l1<=n;l1=l1+1)
   if (typ[l1]!=4)
   {
    h =0;
    h1=0;
    h2=0;
    h3=m[l1];
    for (l2=1;l2<=r;l2=l2+1)
     {
      h =h+lam[l1][l2]/(h3*mue[l1][l2]*geslam[l1]);
      h1=mue[l1][l2]*mue[l1][l2]*h3*h3;
      h2=h2+lam[l1][l2]/(geslam[l1]*h1)*(1+cb[l1][l2]);
     }
    gesmue[l1]=1/h;
    h=h*h;
    cd[l1]=(h2-h)/h;
   }
}


/*************************************************/
/** Initialisierung der Variationskoeffizienten **/
/**        der Zwischenankunftsprozesse         **/
/*************************************************/

init()
{
  int x,y,z;
 
  for (x=1;x<=r;x=x+1)
    for (y=0;y<=n;y=y+1)
      for (z=1;z<=n;z=z+1)
        {
          if ((y==0)&&(p[x][y][z]!=0))
            ca[x][y][z]=c0[z][x];
          else
            ca[x][y][z]=1;
        }
}


/********************************************/
/** Bestimmung der Variationskoeffizeinten **/
/** der Ankunftsprozesse pro Knoten und    **/ 
/**               Klasse                   **/
/********************************************/

merge1(x)
int x;
{
	int o,l,g;
	double h;

	for (o=1;o<=n;o=o+1)
		for (l=1;l<=r;l=l+1)
		{
			h=0;
			for (g=0;g<=n;g=g+1)
			{
				if (lam[o][l]!=0.0)
					h=h+ueber[l][g][o]/lam[o][l]*(ca[l][g][o]-1);
			}
			ca2[o][l]=1+h;
		}
}


/********************************************/
/** Berechnung der Variationskoeffizienten **/
/** der Ankunftsprozesse an jedem Knoten   **/
/********************************************/

merge(x)
int x;
{
  int y,z;
  double h;

  h=0;
  for (y=1;y<=r;y=y+1)
    h=h+lam[x][y]/geslam[x]*(ca2[x][y]-1);
  ca1[x]=1+h;
}


/********************************************/
/** Bestimmung der Variationskoeffizienten **/
/** der Abgangsprozesse an jedem Knoten    **/
/********************************************/

departure(x)
int x;
{
  double h1,h2,h0,p1(),rho();

  if (typ[x]==4)
    cd1[x]=ca1[x];
  else
   {
	h0=p1(rho(x),m[x]);
    h1=h0*h0*(cd[x]-1);
    h2=(1-h0)*(ca1[x]-1);
    cd1[x]=1+h1+h2;
   }
}


/********************************************/
/** Berechnung der Wartewahrscheinlichkeit **/
/********************************************/

double p1(a1,a2)
double a1;
int a2;
{
  int l;
  double h,h1,h2,rho1(),fak();

  h1=1.0;
  for (l=1;l<=a2-1;l=l+1)
     h1=h1+pow(a1*a2,l)/fak(l);
  h2=pow(a1*a2,a2)/fak(a2);
  h=h2/(h2+(1-a1)*h1);
  return(h);
}


/***********************/
/** Berechnung von z! **/
/***********************/

double fak(z)
int z;
{
  int l1;
  double h;

  h=1;
  l1=1;
  while (l1<=z)
   {
    h=h*l1;
    l1++;
   }
  return(h);
}


/*************************************/
/** Berechnung der Gesamtauslastung **/
/*************************************/

double rho(a)
int a;
{
  double h,h1;

  if (typ[a]==4)
    h=0;
  else
    h=geslam[a]/gesmue[a];
  if (h >= 1.0)
  {
     printf("chylla_instabil\n 0 0 0\n");
     exit(1);
  }
  return(h);
}


/*************************************/
/***** Berechnung der Auslastung *****/
/***** fuer jeden Knoten und     *****/
/*****      Klasse               *****/
/*************************************/

double rho1(x,y)
int x,y;
{
  double h,h1,rho();
  
  h1=m[x];
  if (typ[x]==4)
    h=0;
  else
    h=lam[x][y]/(mue[x][y]*h1);
  if (h >= 1.0)
  {
     printf("chylla_instabil\n 0 0 0\n");
     exit(1);
  }
  return(h);
}


/**********************/
/** Aufsplittung der **/
/** Abgangsstroeme   **/
/**********************/

split(x)
int x;
{
  int y,z;

  for (y=1;y<=r;y=y+1)
    for (z=1;z<=n;z=z+1)
      if (p[y][x][z]!=0)
       {
         wert[y][x][z]=ca[y][x][z];
         ca[y][x][z]=1+p[y][x][z]*(cd1[x]-1);
       }
}


/***********************/
/** Ueberpruefung der **/
/** Abbruchbedingung  **/
/***********************/

abweichung()
{
  int x,y,z;
  double h;

  pruef=0;
  for (y=1;y<=r;y=y+1)
    for (x=1;x<=n;x=x+1)
      for (z=1;z<=n;z=z+1)
        {
          if (p[y][x][z]!=0)
           {
             h=fabs(ca[y][x][z]-wert[y][x][z]);
             if (h>=epsi)
               pruef=1;
           }
        }
}


/**********************************************/
/** Berechnung der mittleren Anzahl der      **/
/** Auftraege in jedem Knoten                **/
/**********************************************/

mitauftr()
{
  int x,y;
  double h2,h3,p1(),rho(),rho1();

  for (x=1;x<=n;x=x+1)
   for (y=1;y<=r;y=y+1)
    {
     if (typ[x]==4)
       maa[x][y]=lam[x][y]/mue[x][y];
     else
      {
       h2=rho1(x,y)/(1-rho(x));
       h3=(cb[x][y]+ca2[x][y])/2;
       maa[x][y]=m[x]*rho1(x,y)+p1(rho(x),m[x])*h2*h3;
      }
    }
}


/***********************************/
/** Ausgabe der Leistungsgroessen **/
/***********************************/

ausgabe()
{
  int i,j,l;
  double rho(),rho1(),mvz,mwz,mwsl;

  printf("'chylla'\n");
  printf("2 %d %d\n",n,r);
  for (j=1;j<=r;j=j+1)
  {
    for (i=1;i<=n;i=i+1)
     {
	  if (lam[i][j] != 0.0)
	  {
		  mvz=maa[i][j]/lam[i][j];
		  mwz=mvz-1/mue[i][j];
		  if (mwz<0.0)
			mwz=0.0;
		  mwsl=mwz*lam[i][j];
	  }
	  else
	  {
		  mvz=0.0;
		  mwz=0.0;
		  mwsl=0.0;
	  }
      printf("%8.6lf %8.6lf %8.6lf ",lam[i][j],e[i][j],1/mue[i][j]);
      printf("%8.6lf %8.6lf %8.6lf %8.6lf %8.6lf ",rho1(i,j),mvz,maa[i][j],mwz,mwsl);
      printf("\n");
     }
    printf("\n");
  }
}
