static char *SccsId = "@(#)PEPSY:\tgrassmann.c\t1.2\t92/03/27 11:43:26";

/*
	Grassmann Algorithmus fuer die Bestimmung der
	Gleichgewichtsverteilung

	Literatur: Krieger, U.: Computational Methods for
		Markovian Queueing Systems - A Survey

	Annahmen:
		
	-	alle benoetigen Dateien liegen in ./
	-	in 'Dimension' steht die Groesse der Matrix
	-	in 'Matrix' stehen die von Null verschiedenen
		Ausserdiagonalelemente in der Form "von nach rate"
	-	in 'Gleichgewicht' wird die Loesung abgelegt in der
		Form "index Wert"
*/


# include <stdio.h>
# include <ctype.h>

static double *A;	/* Matrix, hier eindimensional gespeichert */
static double *x;	/* Loesungsvektor */
static double rate;
static int    dim;	/* Dimension der Matrix */
static int    von;
static int    nach;
static char   line[512];


main(argc,argv)
int argc;
char *argv[];
{
	register i,j,k;
	register double diag, G;
	register char *sp;

	FILE *fopen(), *fpin, *fpout;
	char *calloc(), *fgets();
	double atof();

	if((fpin = fopen("Dimension","r")) == NULL)
	{
		fprintf(stderr,"%s: Datei 'Dimension' laesst sich nicht oeffnen\n",argv[0]);
		exit(1);
	}
	fgets(line,512,fpin);
	sp = line;
	while(isspace(*sp))
		sp++;
	dim = atoi(sp);
	fclose(fpin);
	if((fpin = fopen("Matrix","r")) == NULL)
	{
		fprintf(stderr,"%s: Datei 'Matrix' laesst sich nicht oeffnen\n",argv[0]);
		exit(1);
	}
	if((fpout = fopen("Gleichgewicht","w")) == NULL)
	{
		fprintf(stderr,"%s: Datei 'Gleichgewicht' laesst sich nicht oeffnen\n",argv[0]);
		exit(1);
	}
	if((A = (double *) calloc(dim*dim,sizeof(double))) == (double *)0)
	{
		fprintf(stderr,"%s: nicht genuegend Speicheplatz fuer Matrix A\n",argv[0]);
		exit(1);
	}
	if((x = (double *) calloc(dim,sizeof(double))) == (double *)0)
	{
		fprintf(stderr,"%s: nicht genuegend Speicheplatz fuer Vektor Ax\n",argv[0]);
		exit(1);
	}
	/* transponiertes Einlesen und berechnen der Diagonalen */

	while(fgets(line,512,fpin) != NULL)
	{
		sp = &line[0];
		while(isspace(*sp))
			sp++;
		von = atoi(sp);
		while(!isspace(*sp))
			sp++;
		while(isspace(*sp))
			sp++;
		nach = atoi(sp);
		while(!isspace(*sp))
			sp++;
		while(isspace(*sp))
			sp++;
		rate = atof(sp);
		A[nach*dim+von] += rate;
		A[von*dim+von]  -= rate;
	}

	

	/* Gauss Elimination fuer die ersten dim-1 Zeilen */

	for(k=0;k<dim-1;k++)
	{
		/* Normieren der Zeile */

		diag = 0.0;
		for(i=k+1;i<dim;i++)
		{
			diag += A[i*dim+k];
		}
		/* ueberpruefen von diag */
		if(diag == 0.0)
		{
			fprintf(stderr,"Zeile %d: diag = %lg\n",k,diag);
			exit(1);
		}
		for(i=k+1;i<dim;i++)
		{
			A[k*dim+i] /= diag;
		}

		/* Loeschen der k-ten Spalte */

		for(i=k+1;i<dim;i++)
		{
			if(A[i*dim+k] == 0.0)
				continue;
			for(j=k+1;j<dim;j++)
			{
				A[i*dim+j] += A[i*dim+k] * A[k*dim+j];
			}
		}
	}

	/* Ruecksubstitutuion */

	x[dim-1] = 1.0;
	G        = 1.0;

	for(i=dim-2;i>=0;i--)
	{
		for(j=i+1;j<dim;j++)
		{
			x[i] += A[i*dim+j] * x[j];
		}
		G += x[i];
	}

	/* Normierung und Ausgabe */

	for(i=0;i<dim;i++)
	{
		fprintf(fpout,"%d %.8g\n",i,x[i] / G);
	}

	fclose(fpin);
	fclose(fpout);

	exit(0);
}


dump_A(k)
int k;
{
	register i,j;

	fprintf(stderr,"\n");
	for(i=0;i<dim;i++)
	{
		for(j=0;j<dim;j++)
		{
			fprintf(stderr,"%5.2f ",A[i*dim+j]);
		}
		if(i == k)
			fprintf(stderr," < ");
		fprintf(stderr,"\n");
	}
}

