static char *SccsId = "@(#)PEPSY:\tpower.c\t1.3\t93/06/11 11:41:20";
#include <stdio.h>
#include <signal.h>

#define OUTPUT(data) fprintf(fpout, "%lu %.16lg\n", i, data)

#define DIM 65536
#define PROMPT_DST 10
#define P_UNDERFLW 1.0e-60
#define EPS 1.0e-30

/* Datenstrukturen fuer die Iteration. */
unsigned long dim;
unsigned long no_m_elem;
unsigned long *c_length;
struct m_elem
{
   unsigned long index;
   double data;
} *matrix;
double *diag;
double *iter_v1, *iter_v2;

unsigned short verbose = 0, file_inp = 0, show_flow = 0;
unsigned long itercount = 0, iterlimit = 0;


void output(v_ptr)
double *v_ptr;
{
   unsigned long i;
   double help;
   FILE *fpout;

   fpout = fopen("Gleichgewicht", "w");
   if (fpout == (FILE *)0) error(__FILE__, "fopen-Gleichgewicht", __LINE__);
   
   help = *v_ptr;
   for (i=1; i<dim; i++)
      help += *(v_ptr+i);
   fprintf(stderr, "\nNormierungsfaktor: %le.\n", help);

   for (i=0; i<dim; i++)
      OUTPUT(*(v_ptr+i)/help);
   fclose(fpout);
}

void p_sig_hand(sig)
int sig;
{
   if (verbose && iterlimit) fprintf(stderr, "\nSignal %d!\n", sig);
   if (file_inp < 2) exit(0);

   if (itercount < 1)
   {
      fprintf(stderr, "Signal vor oder waehrend der ersten Iteration.\n");
      output(iter_v1);
   }
   else
   {
      if ((itercount%2) == 0) output(iter_v2);
      else output(iter_v1);
   }

   fprintf(stderr, "Iterationen: %lu;\n", itercount);
   exit(0);

}

/* Speicherbedarf des Programmes (in Bytes):
   Matrix:   (no_m_elem - dim) * sizeof(struct m_elem) Bytes;
   Vektoren: 7 * 4 * dim Bytes;
             Einige Variablen;
*/
   
main(argc, argv)
int argc;
char *argv[];
{
   int c;
   long mass = -1;
   unsigned long i;
   FILE *fpin;

   extern char *optarg;
   extern int optind, opterr;
   
   void read_matrix(), iterate();
   void p_sig_hand(), output();

   void setbuf();
   int getopt(), fseek(), fclose();
   long atol();
   FILE *fopen();

   setbuf(stderr, (char *)0);
   setbuf(stdout, (char *)0);

   signal(SIGHUP, p_sig_hand);
   signal(SIGINT, p_sig_hand);
   signal(SIGQUIT, p_sig_hand);
   signal(SIGFPE, p_sig_hand);

   fpin = fopen("Dimension", "r");
   if (fpin == (FILE *)0) error(__FILE__, "fopen-Dimension", __LINE__);
   
   fscanf(fpin, "%lu", &dim);
   if (dim > DIM)
   {
      fprintf(stderr, "Die Dimension des GLS ist zu gross!\n\n");
      exit(1);
   }
   fclose (fpin);

   while ((c=getopt(argc, argv, "vdi:m:f")) != EOF)
   {
      switch (c) {
      case 'v': verbose = 1;
                break;
      case 'f': show_flow = 1;
                break;
      case 'd': file_inp = 1;
                break;
      case 'i': iterlimit = atol(optarg);
                break;
      case 'm': if((mass = atol(optarg)) >= dim)
				{
					fprintf(stderr,"Ausgewaehlter Zustand groeser als Dimension %d\n",dim);
					fprintf(stderr,"gleichmaessige Verteilung der W'keitsmasse\n");
				}
                break;
      }
   }

   if (iterlimit <= 0)
   {
      printf("Obergrenze fuer die Anzahl der Iterationen: ");
      scanf("%lu", &iterlimit);
   }
   
   iter_v1 = (double *)calloc(dim, sizeof(double));
   if (iter_v1 == (double *)0) error(__FILE__, "calloc()", __LINE__);

   if (file_inp)
   {
      fpin = fopen("Gleichgewicht", "r");
      if (fpin == (FILE *)0)
         error(__FILE__, "fopen-Gleichgewicht", __LINE__);

      for (i=0; i<dim; i++)
         fscanf(fpin, "%*lu %le\n", (iter_v1+i));

      fclose(fpin);
   }
   else if(mass < 0)
   {
      for (i=0; i<dim; i++)
         *(iter_v1+i) = 1.0 / (double)dim;
   }
   else
   {
	 *(iter_v1+mass) = 1.0;
   }
   file_inp = 2; 
   if (iterlimit == 0)
   {
      output(iter_v1);
      exit(0);
   }

   fpin = fopen("MElemente", "r");
   if (fpin == (FILE *)0) error(__FILE__, "fopen-MElemente", __LINE__);
   
   fscanf(fpin, "%lu", &no_m_elem);
   no_m_elem -= dim;
   fclose (fpin);
   
   fpin = fopen("Matrix", "r");
   if (fpin == (FILE *)0) error(__FILE__, "fopen-Matrix", __LINE__);
   
   matrix = (struct m_elem *)calloc(no_m_elem, sizeof(struct m_elem));
   if (matrix == (struct m_elem *)0) error(__FILE__, "calloc()", __LINE__);
   
   c_length = (unsigned long *)calloc(dim, sizeof( long));
   if (c_length == (unsigned long*)0) error(__FILE__, "calloc()", __LINE__);
   
   diag = (double *)calloc(dim, sizeof(double));
   if (diag == (double *)0) error(__FILE__, "calloc()", __LINE__);
   
   
   if (verbose) fprintf(stderr,"Einlesen der Matrix.\n");
   read_matrix(fpin);
   
   iter_v2 = (double *)calloc(dim, sizeof(double));
   if (iter_v2 == (double *)0) error(__FILE__, "calloc()", __LINE__);

   if (verbose) fprintf(stderr, "Beginn der Iteration.\n");
   iterate();

   fclose (fpin);
}

void read_matrix(fp)
FILE *fp;
{
   int retval;
   unsigned long i;
   unsigned long row, column;
   unsigned long *c_index;
   double entry, max_d_elem;
   struct m_elem *m_ptr;

#define INPUT1 fscanf(fp, "%*lu %lu %*le\n", &column)

   while (INPUT1 != EOF)
   {
#ifdef DEBUG
	printf ( " ???  -> %5d   ???\t",column);
#endif
      c_length[column]++;
#ifdef DEBUG
	printf ( "c_length[%5d] = %d\n",column,c_length[column]);
#endif
	}

   c_index = (unsigned long *)calloc(dim, sizeof(long));
   if (c_index == (unsigned long *)0) error(__FILE__, "calloc()", __LINE__);
   c_index[0] = 0;
   for (i=1; i<dim; i++)
      c_index[i] = c_length[i-1] + c_index[i-1];

   retval = fseek(fp, 0L, 0);
   if (retval != 0) error(__FILE__, "fseek()", __LINE__);

#define INPUT2 fscanf(fp, "%lu %lu %le\n", &row, &column, &entry)

   while (INPUT2 != EOF)
   {
#ifdef DEBUG
	printf ( " %5d -> %5d    %8.6f\t", row,column,entry);
#endif
      m_ptr = matrix + c_index[column];
      c_index[column]++;
      m_ptr->index = row;
      m_ptr->data = entry;

      diag[row] += entry;
#ifdef DEBUG
	printf ( " abgelegt im Feld mit index %d\n", c_index[column] - 1);
#endif
   }
#ifdef DEBUG
   printf("no_m_elem = %d \n ",no_m_elem);
   /*
   m_ptr = matrix;
   for (i=0; i<no_m_elem; i++)
   {
      printf("%lu %le + ", m_ptr->index, m_ptr->data);
      m_ptr++;
	  if( i % 5 == 0 )
		printf("\n");
   }
   for (i=1; i<dim; i++)
      printf("Diag[%lu]: %le\n", i, diag[i]);
   for (i=0; i<dim; i++)
      printf("c_index[%lu]: %d c_length[%lu]: %d \n", i, c_index[i],i,c_length[i]);
*/
#endif

   max_d_elem = diag[0];
   for (i=0; i<dim; i++)
      if (max_d_elem < diag[i]) max_d_elem = diag[i];

   m_ptr = matrix;
   for (i=0; i<no_m_elem; i++)
   {
      m_ptr->data = m_ptr->data / max_d_elem;
      m_ptr++;
   }

   max_d_elem = -max_d_elem; 
   for (i=0; i<dim; i++)
   {
      *(diag+i) = (*(diag+i) / max_d_elem) + 1.0;
      /* fprintf(stderr, "diag[%lu]: %le;\n", i,  diag[i]); */
   }

   free((char *)c_index); 

}


void iterate()
{
   register struct m_elem *m_ptr;
   register double *v_ptr1, *v_ptr2;
   register double help;
   register unsigned long i, j;

   while (itercount < iterlimit)
   {
      if (verbose && ((itercount%PROMPT_DST) == PROMPT_DST-1)) putc('.', stderr);

      if ((itercount%2) == 0)
      {
         v_ptr1 = iter_v1;
         v_ptr2 = iter_v2;
      }
      else
      {
         v_ptr1 = iter_v2;
         v_ptr2 = iter_v1;
      }

      m_ptr = matrix; 
      for (i=0; i<dim; i++)
      {
#ifdef DEBUG
	printf("Zugriff auf v_ptr1, index %5d\n", m_ptr->index);
	printf("\t\tc_length[%d] = %d\n",i,c_length[i]);
#endif
		 if(c_length[i] == 0)
			continue;
		/*
		if( *(diag+i) < EPS)
			continue;
			*/
         help = *(v_ptr1+m_ptr->index) * m_ptr->data;
         m_ptr++;
         for (j=c_length[i]-1; j>0; j--)
         {
#ifdef DEBUG
	printf("\tZugriff auf v_ptr1, index %5d\n", m_ptr->index);
#endif
            help += *(v_ptr1+m_ptr->index) * m_ptr->data;
            m_ptr++;
         }
#ifdef DEBUG
	printf("\n");
#endif

         *(v_ptr2+i) = help + *(v_ptr1+i) * *(diag+i);
#ifdef CCC
         /* Concurrent Computer Corp. Maschienen haben Probleme
            mit Unterlauf.
         */
         if (*(v_ptr2+i) < P_UNDERFLW) *(v_ptr2+i) = 0.0;
#endif

		if(show_flow)
			fprintf(stderr,"%8.6lf %8.6lf\n",*(v_ptr1+i),*(v_ptr2+i));
      }  /* END for. */
		if(show_flow)
			fprintf(stderr,"\n");
      itercount++;

   }  /* END while. */

   output(v_ptr2);
   exit(0);
}

