/**********************************************************************************************************************/
/*													              */
/*	MODULNAME:							ERSTELLT:				      */
/*	"erreichbar.c"							Mai 1989	(Peter Menninger)	      */
/*													              */
/*	KURZ-BESCHREIBUNG:											      */
/*	Zustaende ohne einmuendende Kanten (= nicht erreichbare Zustaende) werden (erreichbar[zustand] = - 1)	      */
/*	gekennzeichnet												      */
/*													              */
/*	FUNKTIONEN:											              */
/*  	- Erreichbarkeitsanalyse ()										      */
/*	- Nicht_erreichbarer_Zustand ()									              */
/*  	- Welche_Zustaende_sind_erreichbar ()						     		     	      */
/*													              */
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/*													              */
/*	FUNKTION												      */
/*  	Erreichbarkeitsanlyse ()							     		     	      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:               	                     	GESPEICHERT IN:				      */
/*  	- Berechnungen ()						"main.c"     				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	Zustaende ohne ankommende Kanten werden an 'Nicht_erreichbarer_Zustand ()' uebergeben			      */
/*									     					      */
/*  	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:                               */
/*  	- Head_Meldung_erreichbar ()					"meldungen.c"				      */
/*	- Welche_Zustaende_sind_erreichbar () 				"erreichbar.c"				      */
/*	- Balken ()							"meldungen.c"				      */
/*									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*	- ALLE_ZUSTAENDE	 					"define.h"				      */
/*									     					      */
/**********************************************************************************************************************/


static char *SccsId = "@(#)PEPSY:\terreichbar.c\t1.3\t93/05/17 09:27:16";

#include "define.h"
#include "defs.h"


long		bereits_gedruckt;					/* Anzahl Spalten fuer die 'stdout'-Ausgabe   */

Erreichbarkeitsanalyse ()

{
	long		zustand;					/* durchlaueft alle Zustaende		      */

	bereits_gedruckt = 0;

	Head_Meldung_erreichbar ();					/* Ueberschrift				      */

	Welche_Zustaende_sind_erreichbar ();				/* erreichbare Zustaende ermitteln	      */


	Balken (2);

	ALLE_ZUSTAENDE {						/* alle Zustaende bearbeiten		      */
	    if (erreichbar[zustand] == 0 && erreichbar[zustand] != -1)	/*    falls Zustand nicht erreichbar 	      */
		Nicht_erreichbarer_Zustand (zustand);			/*       bearbeite diesen Zustand	      */
	}

	Balken (2);
}




/**********************************************************************************************************************/
/*													              */
/*	FUNKTION												      */
/*  	Nicht_erreichbarer_Zustand ()							     		     	      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:      	                              	GESPEICHERT IN:				      */
/*  	- Ereichbarkeitsanalyse ()				 	"erreichbar.c" 				      */
/*	- Nicht_erreichbarer_Zustand ()   {rekursiv}			"erreichbar.c"				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*	alle Kanten des nicht erreichbaren Zustandes ungueltig kennzeichnen					      */
/*  	daraus neu entstehende Zustaende ohne ankommende Kanten werden an 'Nicht_erreichbarer_Zustand ()' 	      */
/*	uebergeben   {rekursiv}											      */
/*									     					      */
/*	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:				      */
/*	- Work_Meldung_nicht_erreichbar ()				"meldungen.c"  				      */
/*	- Nicht_erreichbarer_Zustand ()   {rekursiv!}			"erreichbar.c"				      */
/*									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*  	- KEINE_KANTE     						"define.h"				      */
/*	- ALLE_KANTEN	(KEINE_KANTE)     				"define.h"				      */
/*									     					      */
/**********************************************************************************************************************/


Nicht_erreichbarer_Zustand (zustand)
long	zustand;

{
	struct kante		*kante;					/* durchlaeuft alle Kanten		      */

	if (DBG_2)
	    Work_Meldung_nicht_erreichbar (bereits_gedruckt++, zustand);/* Anzeige der Taetigkeit		      */


	ALLE_KANTEN (kante, zustand) {					/* alle Kanten des nicht erreichba. Zustandes */
	    if(kante->uebergangs_rate != 0.0)
	    {
		kanten_reduziert++;						/*      entfernte Kanten hochzaehlen	      */
									    /*      Erreichbarkeit des 		      */
		if (--erreichbar[kante->nach] == 0) 		 	/*      'nach'-Zustandes vermindern 	      */
		    Nicht_erreichbarer_Zustand (kante->nach);		/*      Erreichbarkeit = 0: rekursiver Aufruf */
	    }
	}

	erreichbar[zustand] = -1;					/* nicht erreichbarer Zustand kennzeichnen    */
	dimension_reduziert++;						/* Dimension reduzieren			      */
}									




/**********************************************************************************************************************/
/*													              */
/*	FUNKTION												      */
/*  	Welche_Zustaende_sind_erreichbar ()						     		     	      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:      	                              	GESPEICHERT IN:				      */
/*  	- Ereichbarkeitsanalyse ()				 	"erreichbar.c" 				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*	Zahl der einmuendenden Kanten pro Zustand ermitteln (==> Zustand erreichbar)				      */
/*									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/* 	- CALLOC							"define.h"				      */
/*	- ALLE_ZUSTAENDE	 					"define.h"				      */
/*	- ALLE_KANTEN	(KEINE_KANTE)     				"define.h"				      */
/*									     					      */
/**********************************************************************************************************************/


Welche_Zustaende_sind_erreichbar ()

{
	register long		zustand;				/* durchlaeuft alle Zustaende		      */
	register struct kante	*kante;					/* durchlaeuft alle Kanten		      */
	register int		*erreichbar_neu;

	CALLOC (erreichbar_neu, dimension_aktuell, int);		/* fuer neuen Vektor Speicherplatz reservieren*/

	ALLE_ZUSTAENDE {						/* alle Zustaende bearbeiten		      */
	    if (erreichbar[zustand] == -1) {				/*    falls schon bekannt als nicht erreichbar*/
		erreichbar_neu[zustand] = -1;				/*        uebernehmen			      */
		continue;						/*        naechster Zustand		      */
	    }
	    ALLE_KANTEN (kante, zustand) {				/*    alle Kanten bearbeiten 		      */
			if(kante->uebergangs_rate != 0.0)
				erreichbar_neu[kante->nach]++;				/*       Erreichbarkeit des 'nach'-Zustandes  */
	    }								/*       erhoehen			      */ 
	}

	free (erreichbar);						/* 'erreichbar' alt freigeben		      */

	erreichbar = erreichbar_neu;					/* 'erreichbar' neu benutzen		      */
}
