static char *SccsId = "@(#)PEPSY:\tio_allgemein.c\t1.2\t92/03/27 11:45:31";
/**********************************************************************************************************************/
/*													              */
/*	MODULNAME:							ERSTELLT:				      */
/*	"io_allgemein.c"						Mai 1989	(Peter Menninger)	      */
/*													              */
/*	KURZ-BESCHREIBUNG:											      */
/*	Enthaelt Funktionen mit 'Input/Output' - Verhalten	 						      */
/*													              */
/*	FUNKTIONEN:											              */
/*	- In_Dateioeffnen ()										              */
/*	- Out_Dateioeffnen ()										              */
/*	- In_MElemente ()										              */
/*	- Out_MElemente ()										              */
/*	- In_Dimension ()										              */
/*	- Out_Dimension ()										              */
/*													              */
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	In_Dateioeffnen (filename_in)								     		      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                    		GESPEICHERT IN:				      */
/*  	- Verdichten_Codierung ()					"verdichten.c"				      */
/*  	- Verdichten_Matrix ()						"verdichten.c"				      */
/*  	- In_Dimension ()						"io_allgemein.c" 			      */
/*  	- In_MElemente ()						"io_allgemein.c"    			      */
/*	- Codierung_einlesen ()						"io_datei.c"				      */
/*	- Codierung_schreiben ()					"io_datei.c"				      */
/*	- Matrix_einlesen ()						"io_datei.c"				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	- oeffnet 'filename_in' zum Lesen									      */
/*  	- bei 'fopen-' Fehler: Fehlermeldung und Abbruch							      */
/*									      					      */
/*  	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:			     	      */
/*  	- Error ()                                              	"error.c"				      */
/* 									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:			     	      */
/*  	- HALT     							"define.h"				      */
/*  	- NULL     							<stdio.h>				      */
/*													              */
/**********************************************************************************************************************/


#include "define.h"
#include "defs.h"


In_Dateioeffnen (filename_in)
char	*filename_in;

{
	if ((fp_in = fopen (filename_in, "r")) == NULL)
	    HALT strcat ("fopen ", filename_in));
}



/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	Out_Dateioeffnen (filename_out)								     		      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                    		GESPEICHERT IN:				      */
/*  	- Verdichten_Codierung ()					"verdichten.c"				      */
/*  	- Verdichten_Matrix ()						"verdichten.c"				      */
/*  	- Out_Dimension ()						"io_allgemein.c"		     	      */
/*  	- Out_MElemente ()						"io_allgemein.c"		     	      */
/*	- Codierung_schreiben ()					"io_datei.c"				      */
/*	- Matrix_schreiben ()						"io_datei.c"				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	- oeffnet 'filename_out' zum Schreiben									      */
/*  	- bei 'fopen-' Fehler: Fehlermeldung und Abbruch							      */
/*									      					      */
/*  	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:		     		      */
/*  	- Error ()                                              	"error.c"				      */
/* 									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*  	- HALT 								"define.h"				      */
/*  	- NULL								<stdio.h>			              */
/*													              */
/**********************************************************************************************************************/


Out_Dateioeffnen (filename_out)
char	*filename_out;

{
	if ((fp_out = fopen (filename_out, "w")) == NULL)
	    HALT strcat ("fopen ", filename_out));
}



/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	long    In_MElemente ()											      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                    		GESPEICHERT IN:				      */
/*  	- main ()							"main.c" 			     	      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	zaehlt Zeilen von 'Matrix.tmp' und vergleicht mit eingelesenen Wert aus 'MElemente', falls vorhanden	      */
/*									      					      */
/*  	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:		     		      */
/*  	- In_Dateioeffnen ()                                     	"io_allgemein.c"			      */
/*	- Head_Meldung_In_MElemente ()					"meldungen.c"				      */
/*	- Meldung_falsche_MElemente ()					"meldungen.c"				      */
/* 									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*  	- MATRIX							"define.h"			              */
/*  	- MATRIX_ELEMENTE						"define.h"			              */
/*  	- NULL								<stdio.h>			              */
/* 									     					      */
/**********************************************************************************************************************/


long    In_MElemente ()

{
	long		melemente;
	long		kanten_ein;
	register long 	kanten_gezaehlt = 0;
	register int	zeichen;


	In_Dateioeffnen (MATRIX_TMP);					/* 'MATRIX.TMP' vorhanden?	   	      */
	while ((zeichen = getc (fp_in)) != EOF)				/* lesen bis EOF			      */
	    if (zeichen == '\n')					/*    falls '\n'			      */
                kanten_gezaehlt++;					/*    Zeilen (= Kanten) zaehlen		      */
	fclose (fp_in);

	if ((fp_in = fopen (MATRIX_ELEMENTE, "r")) != NULL) {		/* 'MATRIX_ELEMENTE' vorhanden?		      */
	    fscanf (fp_in, "%ld", &melemente);				/*     Wert aus der Datei lesen		      */
	    fclose (fp_in);						/*     Datei schliessen			      */
	    kanten_ein = melemente - dimension_in;			/*     Kanten = Matrix-Elemente - Diagonale   */
	    if (kanten_ein != kanten_gezaehlt)				/*     verschiedene Werte?		      */
		Meldung_falsche_MElemente (kanten_ein, kanten_gezaehlt);/*         Fehlermeldung und Abbruch	      */
	}

	Head_Meldung_In_MElemente (kanten_gezaehlt);			/* Ueberschrift				      */
	kanten_reduziert = 0;

	return (kanten_gezaehlt);
}



/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	Out_MElemente (anzahl_kanten)										      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                    		GESPEICHERT IN:				      */
/*  	- main ()							"main.c" 				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	- oeffnet MATRIX_ELEMENTE zum Schreiben									      */
/*  	- Anzahl der Matrixelemente auf 'mat_elemente'								      */
/*									      					      */
/*  	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:			     	      */
/*  	- Out_Dateioeffnen ()                                     	"io_allgemein.c"			      */
/*	- Head_Meldung_Out_MElemente ()					"meldungen.c"				      */
/* 									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*  	- MATRIX_ELEMENTE						"define.h"			              */
/* 									     					      */
/**********************************************************************************************************************/


Out_MElemente (anzahl_kanten)
long    anzahl_kanten;

{
	Out_Dateioeffnen (MATRIX_ELEMENTE);

	fprintf (fp_out, "%ld\n", anzahl_kanten + dimension_in - dimension_reduziert);

	fclose (fp_out);

	Head_Meldung_Out_MElemente (anzahl_kanten);

}



/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	long    In_Dimension ()			    								      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                   		GESPEICHERT IN:				      */
/*  	- main ()							"main.c"				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	zaehlt Zeilen von 'Codierung.tmp' und vergleicht mit eingelesenen Wert aus 'Dimension'			      */
/*									      					      */
/*  	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:		     		      */
/*  	- In_Dateioeffnen ()                                     	"io_allgemein.c"			      */
/*	- Head_Meldung_In_Dimension ()					"meldungen.c"				      */
/*	- Meldung_falsche_Dimension ()					"meldungen.c"				      */
/* 									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*  	- NULL								<stdio.h>			              */
/*  	- COD_TMP							"define.h"			              */
/*  	- DIMENSION							"define.h"			              */
/*	- HALT								"define.h"				      */
/* 									     					      */
/**********************************************************************************************************************/


long    In_Dimension ()

{
	long		dimension_ein;
	register long	dimension_gezaehlt = 0;
	register int	zeichen;

	In_Dateioeffnen (COD_TMP);					/* 'COD_TMP' vorhanden?		   	      */
	while ((zeichen = getc (fp_in)) != EOF) 			/* lesen bis EOF			      */
	    if (zeichen == '\n')					/*    falls '\n'			      */
                dimension_gezaehlt++;					/*        Zeilen (= Dimension) zaehlen        */
	fclose (fp_in);							/* Datei schliesen	 		      */

	if ((fp_in = fopen (DIMENSION, "r")) != NULL) {			/* 'DIMENSION' vorhanden?		      */
	    fscanf (fp_in, "%ld", &dimension_ein);			/*     Wert aus der Datei lesen		      */
	    fclose (fp_in);						/*     Datei schliesen	 		      */
	    if (dimension_ein != dimension_gezaehlt)			/*     verschiedene Werte?		      */
		Meldung_falsche_Dimension (dimension_ein, dimension_gezaehlt);/*   Fehlermeldung und Abbruch	      */
	} else { 
	    HALT strcat ("fopen ", DIMENSION));				/*     Fehlermeldung und Abbruch	      */
            Balken (dbg);
        }

	Head_Meldung_In_Dimension (dimension_gezaehlt);			/* Ueberschrift				      */

	dimension_reduziert = 0;
	dimension_aktuell   = dimension_gezaehlt;

	return (dimension_gezaehlt);
}



/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	Out_Dimension (dimension)										      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                   		GESPEICHERT IN:				      */
/*  	- main ()							"main.c"				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	- oeffnet DIMENSION zum Schreiben									      */
/*  	- Wert auf 'dim'										     	      */
/*									      					      */
/*  	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:		     		      */
/*  	- Out_Dateioeffnen ()                                     	"io_allgemein.c"			      */
/*	- Head_Meldung_Out_Dimension ()					"meldungen.c"				      */
/* 									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*  	- DIMENSION							"define.h"			              */
/* 									     					      */
/**********************************************************************************************************************/


Out_Dimension (dimension)
long	dimension;
{
	Out_Dateioeffnen (DIMENSION);

	fprintf (fp_out, "%ld\n", dimension);

	fclose (fp_out);

	Head_Meldung_Out_Dimension (dimension);
}

