/**********************************************************************************************************************/
/*													              */
/*	MODULNAME:							ERSTELLT:				      */
/*	"io_datei.c"							Mai 1989	(Peter Menninger)	      */
/*													              */
/*	KURZ-BESCHREIBUNG:											      */
/*	CODIERUNG_TMP und MATRIX_TMP  werden mit den folgenden Funktionen:					      */
/*	- eingelesen (dabei 1-mal verdichtet) 									      */
/*	- geschrieben                          								              */
/*													              */
/*	FUNKTIONEN:											              */
/*	- Codierung_einlesen (filename_in) 								              */
/*	- Codierung_schreiben (filename_in, filename_out) 						              */
/*	- Matrix_einlesen (filename_in)									              */
/*	- Matrix_schreiben (filename_out, flip)								              */
/*	- Erste_Kante_einlesen ()									              */
/*													              */
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/*													              */
/*	FUNKTION												      */
/*  	Codierung_einlesen ()											      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                         	GESPEICHERT IN:                               */
/*  	- main ()							"main.c"     				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*	- alle Original-Zustands-Nummern einlesen 								      */
/*	  mit Ueberpruefung auf aufsteigende Reihenfolge							      */
/*	  und gleichzeitiger Verdichtung				     					      */
/*									     					      */
/*	VERWENDETE FUNKTIOEN:						GESPEICHERT IN:				      */
/*	- In_Dateioeffnen ()				     		"io_allgemein.c"			      */
/*	- Head_Meldung_Codierung_verdichten ()				"meldungen.c"				      */
/*	- Work_Meldung_Codierung_verdichten ()				"meldungen.c"				      */
/*	- Meldung_Zustand_groesser_vorheriger () 			"meldungen.c"		     		      */
/*	- Dim_Ueberpruefung () 					    	"error.c"				      */
/*	- Balken ()	 					    	"meldungen.c"				      */
/*									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:			     	      */
/*	- CALLOC							"define.h"				      */
/*	- EOF								<stdio.h>				      */
/*	- ZEILENREST_UEBERLESEN						"define.h"				      */
/*													              */
/**********************************************************************************************************************/


static char *SccsId = "@(#)PEPSY:\tio_datei.c\t1.3\t93/05/17 09:27:19";

#include	"define.h"
#include	"defs.h"



Codierung_einlesen (filename_in)
char	*filename_in;

{
	long		zustand  	 = 0;				/* durchlaeuft alle Zustaende		      */
	long		bereits_gedruckt = 0;				/* Anzahl Spalten fuer die 'stdout'-Ausgabe   */

	Head_Meldung_Codierung_verdichten ();				/* Ueberschrift				      */

	In_Dateioeffnen  (filename_in);					/* Datei 'filename_in'  zum Lesen oeffnen     */

	/**************************************************************************************************************/
	/*  Speicherplatz fuer folgende Variablen anfordern:							      */
	/*  codierung_original:     Original-Zustaende (aus CODIERUNG_TMP unbehandelt!)				      */
	/*  codierung:	            nach jedem Verdichtungsschritt alte Zustaende (hier:codierung=codierung_original  */
	/*  erreichbar:   	    nicht erreichbare Zustaende werden spaeter den Wert -1 erhalten 		      */
	/*  zuschreibenderZustand:  welche Zustands-Nummer soll beim Schreiben von COD_TMP verwendet werden	      */
	/**************************************************************************************************************/

	CALLOC (codierung_original,    dimension_in, unsigned long);		
	CALLOC (codierung,             dimension_in, unsigned long);	
	CALLOC (erreichbar,	       dimension_in, int);
	CALLOC (zuschreibenderZustand, dimension_in, long);


	Balken (2);

        while (fscanf (fp_in,"%lu",codierung_original+zustand) != EOF) {/* alle Original-Zustands-Nummern einlesen    */
	    ZEILENREST_UEBERLESEN;					/*     Rest der Zeile ueberlesen	      */

	    zuschreibenderZustand[zustand] = zustand;			/*     neue Nummer waere 'zustand'	      */
	    codierung[zustand] = codierung_original[zustand];		/*     alter Zustand ist Original-Zustand     */

	    if (zustand > 0 && 						/*     mindestens 1 Zustand bereits gelesen &&*/
	        codierung_original[zustand] <= codierung_original[zustand-1])/*    aktueller  Zustand <= vorheriger   */
									
	        Meldung_Zustand_groesser_vorheriger (zustand,		/*         Fehlermeldung und Abbruch	      */
	        codierung_original[zustand], codierung_original[zustand-1]);

	    if (DBG_2 && codierung[zustand] != zustand)			/*     falls dbg == 2 und Aenderung bei       */
	        Work_Meldung_Codierung_verdichten			/*         der Verdichtung		      */
		(bereits_gedruckt++, codierung, zustand);		/*         Taetigkeits-Meldung		      */

	    zustand++;							/* 'zustand' erhoehen			      */
	}

	Balken (2);

	fclose (fp_in);							/* Datei mit 'fp_in' schliessen	     	      */

	Dim_Ueberpruefung (zustand, 0);					/* sind 'dimension_in' Zustaende eingelesen?  */


}




/**********************************************************************************************************************/
/*													              */
/*	FUNKTION												      */
/*  	Codierung_schreiben ()											      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                         	GESPEICHERT IN:                               */
/*  	- main ()							"main.c"     				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*	- alle Original-Zustaende einlesen:				     					      */
/*	  falls Zustand intern noch vorhanden 									      */
/*		falls erreichbar										      */
/*		     Original-Zustands-Zeile kopieren								      */
/*	  ansonsten ueberlesen											      */
/*									     					      */
/*	VERWENDETE FUNKTIOEN:						GESPEICHERT IN:				      */
/*	- In_Dateioeffnen ()				     		"io_allgemein.c"			      */
/*	- Out_Dateioeffnen ()				     		"io_allgemein.c"			      */
/*	- Dim_Ueberpruefung ()				     		"error.c"				      */
/*									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:			     	      */
/*	- ZEILENREST_UEBERLESEN						"define.h"				      */
/*	- ZEILENREST_KOPIEREN						"define.h"				      */
/*													              */
/**********************************************************************************************************************/


Codierung_schreiben (filename_in, filename_out)
char	*filename_in;
char 	*filename_out;

{
	register long		noch_vorhanden = 0;			/* durchlaeuft die Position der vorhandenen Z.*/
	unsigned long		original_zustand;			/* eingelesene Original-Zustaende	      */
	long			dimension_pruef = 0;

	In_Dateioeffnen  (filename_in);					/* Datei 'filename_in'  zum Lesen     oeffnen */
	Out_Dateioeffnen (filename_out);				/* Datei 'filename_out' zum Schreiben oeffnen */


	while (fscanf (fp_in, "%lu", &original_zustand) != EOF) {	/* alle Original-Zustaende-Nummern einlesen   */
	    if (original_zustand < codierung_original[noch_vorhanden]) {/* falls gelesener Zustand < noch vorhandener */
	        ZEILENREST_UEBERLESEN;					/*     Rest der Zeile ueberlesen	      */
		continue;
	    }
	    if (original_zustand == codierung_original[noch_vorhanden]){/*falls gelesener Zustand == noch vorhandener */
		if (erreichbar[noch_vorhanden] != -1) {			/*    und erreichbar			      */
		    fprintf(fp_out, "%5ld", 				/*       schreibe zuschreibenden Zustand      */
		    zuschreibenderZustand[noch_vorhanden]);		/*       Zeilenrest (Originaldatei) kopieren  */
	    	    ZEILENREST_KOPIEREN;		
		    dimension_pruef++; 
		} else							/*    nicht erreichbar			      */
	            ZEILENREST_UEBERLESEN;				/*       Zeilenrest (Originaldatei) ueberlesen*/

		noch_vorhanden++;					/*    naechste Position nehmen		      */
		if (noch_vorhanden == dimension_aktuell) 		/*    falls schon alle Zustaende geschrieben  */
		    break;						/*        fertig			      */
	    }
	}


	fclose (fp_in);							/* Datei mit 'fp_in'  schliessen     	      */
	fclose (fp_out);						/* Datei mit 'fp_out' schliessen     	      */

	free (codierung);						/* Speicherplaetze freigeben		      */
	free (codierung_original);
	free (zuschreibenderZustand);

	Dim_Ueberpruefung (dimension_pruef, dimension_reduziert);	/* wurden alle Zustaende geschrieben	      */
}



/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*   	Matrix_einlesen (filename_in)										      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN: 	                                  	GESPEICHERT IN:				      */
/*  	- main ()							"main.c"				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	- Aufbau einer verketteten Liste: 									      */
/*    ***********************************************************************************************************     */
/*    *	jeweils ein neuer Zustand mit einer	* weitere Kanten von bereits bekannnten Zustaenden 	     	*     */
/*    * Kante.  z.B.				*     	*	*       *       * 	*	*	*	*     */
/*    *	0.1  1.1  2.1  3.1  4.1  5.1  6.1  7.1 	*  1.2  *  1.3  *  2.2  *  2.3  *  2.4  *  4.2  *  4.3  *  6.2 	*     */
/*    ***********************************************************************************************************     */
/*									      					      */
/*	VERWENDETE FUNKTIOEN:						GESPEICHERT IN:				      */
/*	- In_Dateioeffnen ()				     		"io_allgemein.c"			      */
/*	- Out_Dateioeffnen ()				     		"io_allgemein.c"			      */
/*	- Erste_Kante_einlesen ()					"io_datei.c"			              */
/*	- Verdichten ()							"routinen.c" 			              */
/*	- Head_Meldung_Matrix_verdichten () 				"meldungen.c"				      */
/*	- Work_Meldung_Matrix_verdichten ()				"meldungen.c"			              */
/*	- Meldung_Kantenanfang_kleiner_vorheriger ()			"meldungen.c"			              */
/*	- Balken ()							"meldungen.c"			              */
/*	- Mat_Elem_Ueberpruefung ()					"error.c"			              */
/*													              */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:			     	      */
/*  	- CALLOC 							"define.h"			              */
/*	- ZUWEISEN 							"define.h"			              */
/*													              */
/**********************************************************************************************************************/


static unsigned long  		von;					/* 'von'  - Zustand			      */
static unsigned long          	nach;					/* 'nach' - Zustand			      */
static long			von__verdichtet;			/* verdichteter 'von'  - Zustand	      */
static long			nach_verdichtet;			/* verdichteter 'nach' - Zustand	      */
static long			von__vorheriger;			/* vorheriger 'von'-Zustand		      */
static long			nach_vorheriger;			/* vorheriger 'nach'-Zustand		      */
static double 	 	    	uebergangs_rate;			/* Uebergangsrate			      */
static struct kante		*kante;					/* Verbindung zwischen den einzelnen Kanten   */
static struct kante		*erste_kante;				/* erste Kante eines jeden Zustandes	      */
static struct kante		*naechste_freie_stelle;			/* Platz fuer weiteren Kanten des Zustandes   */
static long			kanten_pruef     = 0;			/* Anzahl der Kanten mit 'uebergangsrate' <> 0*/
static long			bereits_gedruckt = 0;			/* Anzahl Spalten fuer die 'stdout'-Ausgabe   */
long				Verdichten ();




Matrix_einlesen (filename_in)
char	*filename_in;

{
	Head_Meldung_Matrix_verdichten ();

	In_Dateioeffnen (filename_in);					/* Datei 'filename_in' zum Lesen oeffnen      */

	CALLOC (liste, kanten_in+dimension_in, struct kante);     	/* Speicherplatz fuer alle Kanten schaffen    */


	Balken (2);

	Erste_Kante_einlesen ();					/* erste Kante gesondert einlesen, 	      */
									/* um ... _vorheriger besetzen zu koennen     */
	while (fscanf (fp_in,"%lu %lu %lg", 				/* alle Kanten einlesen			      */
	       &von, &nach, &uebergangs_rate) != EOF) {
	    if (uebergangs_rate == 0.0) {				/* nur Kanten mit 'uebergangs_rate' <> 0.0    */
		kanten_reduziert++;					/*     weiterbehandeln			      */
		continue;
	    }
	    if (von < von__vorheriger)					/* falls Kantenanfang kleiner als vorheriger  */
		Meldung_Kantenanfang_kleiner_vorheriger 		/*     Meldung und Abbruch		      */
		(von, von__vorheriger, nach, nach_vorheriger);

	    kanten_pruef++;						/* Kanten Anzahl erhoehen		      */

	    if (von > von__vorheriger) {				/* falls neuer Zustand 			      */
	        von__verdichtet = Verdichten (von, von, nach);		/*    Kantenanfang verdichten		      */
		von__vorheriger = von;					/*    neuer vorheriger Zustand merken	      */
		erste_kante  = &liste[von__verdichtet];			/*    in Liste naechste Position nehmen	      */
	        nach_verdichtet = Verdichten (nach, von, nach);		/*    Kantenende verdichten		      */
		ZUWEISEN (erste_kante, nach_verdichtet, uebergangs_rate);/*   und Werte zuweisen		      */
		kante 	= erste_kante;					/*    Zeiger auf Kante, falls noch weitere    */
	    } else {							/*    Kanten von diesem Zustand ausgehen      */
		kante->next = naechste_freie_stelle;			/* kein neuer Zustand			      */
	        nach_verdichtet = Verdichten (nach, von, nach);		/*    Kantenende verdichten		      */
		ZUWEISEN (naechste_freie_stelle, 			/*    2-ten der Liste nehmen		      */
			  nach_verdichtet, uebergangs_rate);		/*    Werte zuweisen			      */
		naechste_freie_stelle++;				/*    naechste freie Position nehmen	      */
		kante 	= kante->next;					/*    Zeiger schalten, falls noch weitere     */
	    }							        /*    Kanten von diesem Zustand ausgehen      */
	    nach_vorheriger = nach;
	   
	    if (DBG_2 && 						/* falls dbg == 2 			      */
	       !(von == von__verdichtet && nach == nach_verdichtet))	/* und Aenderung bei der Verdichtung          */
	        Work_Meldung_Matrix_verdichten (bereits_gedruckt++,	/*    Taetigkeits-Meldung		      */
	        von, von__verdichtet, nach, nach_verdichtet);	
	}

	Balken (2);
	
	fclose (fp_in);							/* Datei mit 'fp_in' schliessen		      */

	Mat_Elem_Ueberpruefung (kanten_pruef, kanten_reduziert);	/* wurden 'kanten_in' Kanten eingelesen?      */
}




/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*   	Erste_Kante_einlesen ()											      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN: 	                                  	GESPEICHERT IN:				      */
/*  	- Matrix_einlesen ()						"verdichten.c"				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*	erste Kante mit 'uebergangs_rate' <> 0.0 einlesen um 'von__vorheriger' und 'nach_vorheriger' besetzen zu      */
/*	koennen								      					      */
/*									      					      */
/*	VERWENDETE FUNKTIOEN:						GESPEICHERT IN:				      */
/*	- Verdichten ()							"routinen.c" 			              */
/*	- Work_Meldung_Matrix_verdichten ()				"meldungen.c"			              */
/*	- Meldung_keine_Kante_vorhanden()				"meldungen.c"			              */
/*													              */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:			     	      */
/*	- ZUWEISEN 							"define.h"			              */
/*	- EOF	 							<stdio.h>			              */
/*													              */
/**********************************************************************************************************************/


Erste_Kante_einlesen ()

{
	int			fscan;					/* Kontrollwert von fscanf		      */

	while ((fscan = fscanf (fp_in, "%lu %lu %lg", &von, &nach,	/* erste Kante mit 'uebergangs_rate' <> 0.0   */
	      &uebergangs_rate)) != EOF && uebergangs_rate == 0.0) 			
	    kanten_reduziert++;

	if (fscan == EOF)						/* wurde ueberhaupt eine Kante gefunden?      */
	    Meldung_keine_Kante_vorhanden ();				/*     nein: Meldung und Abbruch	      */

	von__verdichtet = Verdichten (von, von, nach);			/* Kantenanfang verdichten		      */
	nach_verdichtet = Verdichten (nach, von, nach);			/* Kantenende   verdichten		      */
	
	if (DBG_2 && 							/* falls dbg == 2 			      */
	    !(von == von__verdichtet && nach == nach_verdichtet))	/* und Aenderung bei der Verdichtung          */
	    Work_Meldung_Matrix_verdichten (bereits_gedruckt++,		/*    Anzeige der Taetigkeit		      */
	    von, von__verdichtet, nach, nach_verdichtet);	

	von__vorheriger = von;						
	nach_vorheriger = nach;

	erste_kante     = &liste[von__verdichtet];			
	ZUWEISEN (erste_kante, nach_verdichtet, uebergangs_rate);	/* Werte zuweisen			      */
	kante 	              = erste_kante;				
	naechste_freie_stelle = &liste[dimension_in];
	kanten_pruef++;							/* Anzahl Kanten erhoehen		      */

}




/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	Matrix_schreiben (filename_out)									              */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                    		GESPEICHERT IN:			   	      */
/*  	- main ()							"main.c"				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*    	verkettete Liste in 'filename_out'  schreiben				      				      */
/*									      					      */
/*  	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:			     	      */
/*  	- Out_Dateioeffnen ()                                    	"io_allgemein.c"			      */
/*	- Mat_Elem_Ueberpruefung ()					"error.c"			              */
/*													              */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:			       	      */
/*	- ALLE_ZUSTAENDE						"define.h"			              */
/*	- ALLE_KANTEN  (KEINE_KANTE)					"define.h"			              */
/*													              */
/**********************************************************************************************************************/


Matrix_schreiben (filename_out)
char   	*filename_out;

{
	register long		zustand;				/* durchlaeuft alle Zustaende		      */
	register struct kante	*kante;					/* durchlaeuft alle Kanten		      */
	register long		kanten_pruef =  0;			/* zaehlt alle geschriebenen Kanten	      */

	Out_Dateioeffnen (filename_out);


	ALLE_ZUSTAENDE {						/* alle Zustaende bearbeiten		      */

	    if (erreichbar[zustand] != -1) {				/*    falls zustand erreichbar		      */

	        ALLE_KANTEN (kante, zustand) {				/*       alle Kanten bearbeiten		      */

				if(kante->uebergangs_rate != 0.0)
				{
					fprintf (fp_out,"%5ld %5ld   %.14lg\n",		/*          schreibe Kante auf 'fp_out'	      */
					zustand, kante->nach, kante->uebergangs_rate); 

					kanten_pruef++;					/*          geschriebene Kanten hochzaehlen   */
				}

	        }
	    }

	}

	fclose (fp_out);						/* Datei mit 'fp_out' schliessen	      */
	free (liste);
	free (erreichbar);

	Mat_Elem_Ueberpruefung (kanten_pruef, kanten_reduziert);	/* Kanten-anzahl mit 'kanten_in' ueberpruefen */
}
