static char *SccsId = "@(#)PEPSY:\tnegPfadLng_1.c\t1.2\t92/03/27 11:45:41";
/**********************************************************************************************************************/
/*													              */
/*	MODULNAME:							ERSTELLT:				      */
/*	"negative_Pfade_Laenge_1.c"					Mai 1988	(Peter Mennninger)	      */
/*													              */
/*	KURZ-BESCHREIBUNG:											      */
/*  	'transitiv - negative' Pfade werden zu Pfaden der Laenge 1 verkuerzt	 				      */
/*													              */
/*	FUNKTIONEN:											              */
/*  	- Negative_Pfade_Laenge_1 ()								     		      */
/*  	- Bearbeite_instabilen_Zustand ()									      */
/*  	- Zyklusueberpruefung ()										      */
/*													              */
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	Negative_Pfade_Laenge_1 ()								     		      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                         	GESPEICHERT IN:                               */
/*  	- Berechnungen ()						"main.c"     				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	- alle instabilen Zustaende werden nacheinander an 'Bearbeite_instabilen_Zustand' uebergeben		      */
/*	- um negative Zyklen zuerkennen: zu behandelner Zustand in reihen_folge speichern			      */
/*					 abbehandelter Zustand wieder freigeben					      */
/*									     					      */
/*  	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:                               */
/*  	- Head_Meldung_Pfad_Laenge_1 ()					"meldungen.c"				      */
/*  	- Balken ()							"meldungen.c"				      */
/*									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*	- ALLE_INSTABILE_ZUSTAENDE 					"define.h"				      */
/*									     					      */
/**********************************************************************************************************************/


#include "define.h"
#include "defs.h"

static int 		*durchlaufen;
static long		*reihen_folge;
static long		stelle  = 0;		

Negative_Pfade_Laenge_1 ()

{
	long		zustand;
	double		norm;

	Head_Meldung_Pfad_Laenge_1 ();					/* Ueberschrift				      */

 	CALLOC (durchlaufen,   dimension_aktuell, int);			/* Speicherplatz fuer Zyklenerkennung schaffen*/
        CALLOC (reihen_folge, dimension_aktuell, long);			/* Speicherplatz fuer Zyklenerkennung schaffen*/


	Balken (2);

	ALLE_INSTABILE_ZUSTAENDE 					/* alle instabilen Zustaende bearbeiten       */
	    reihen_folge[stelle++] = zustand;				/*    Zustand auf Platz 0 ('stelle') setzen   */
	    Bearbeite_instabilen_Zustand (zustand);			/*    instabilen Zustand uebergeben	      */
	    durchlaufen [zustand] = 0;					/*    bearbeiteter Zustand freigeben 	      */
	    stelle--;							/*    'stelle' zurueckzaehlen		      */
	}

	Balken (2);


	free (durchlaufen);						/*Speicherplatz fuer Zyklenerkennung freigeben*/
	free (reihen_folge);						/*Speicherplatz fuer Zyklenerkennung freigeben*/
}




/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	Bearbeite_instabilen_Zustand ()										      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                         	GESPEICHERT IN:                               */
/*  	- Negative_Pfade_Laenge_1 ()					"negative_Pfade_Laenge_1.c"		      */
/*  	- Bearbeite_instabilen_Zustand ()   {rekursiv}			"negative_Pfade_Laenge_1.c"		      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*									     					      */
/*  	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:                               */
/*	- Zyklus_Ueberpruefung ()					"negative_Pfade_Laenge_1.c" 		      */
/*	- Bearbeite_instabilen_Zustand ()   {rekursiv}			"negative_Pfade_Laenge_1.c" 		      */
/*	- Zwei_Kanten_kuerzen ()					"routinen.c"		 		      */
/*									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*      - ALLE_KANTEN (KEINE_KANTE)					"define.h"			              */
/*									     					      */
/**********************************************************************************************************************/


Bearbeite_instabilen_Zustand (zustand)
long	zustand;

{
	struct kante		*neg_kante_1;				/* Variablen fuer 2 Kanten in aufeinander-    */
	struct kante		*neg_kante_2;				/* folgender Reihenfolge		      */
	                                        			/* w------------>x------------>y	      */

	Zyklus_Ueberpruefung (zustand);					/* wurde Zustand schon behandelt?	      */


	ALLE_KANTEN (neg_kante_1, zustand) {				/* alle Kanten bearbeiten		      */

	    if (neg_kante_1->uebergangs_rate < 0.0 &&			/*    falls kante negativ		      */
		neg_kante_1->gueltig == 1 &&				/*    und gueltig   			      */
		instabil[neg_kante_1->nach] > 0.0) {			/*    und 'nach'-Zustand instabil	      */

		ALLE_KANTEN (neg_kante_2, neg_kante_1->nach) {		/*    alle Kanten des 'nach'-Zustandes	      */

		    if (neg_kante_2->uebergangs_rate < 0.0 &&		/*       falls Kante negativ		      */
			neg_kante_2->gueltig == 1) { 			/*       und gueltig			      */

		        if (instabil[neg_kante_2->nach] > 0.0) {	/*       und 'nach'-Zustand instabil	      */
			    reihen_folge[stelle++] = neg_kante_1->nach; /*	       Zustand speichern	      */
			    Bearbeite_instabilen_Zustand (neg_kante_1->nach);/*        insta. Zustand uebergeben      */
			    durchlaufen [neg_kante_1->nach] = 0;	/*             bearbeiteter Zustand freigeben */
			    stelle--;					/*             'stelle' zurueckzaehlen	      */
		        } else {					/*        else				      */
			    Zwei_Kanten_kuerzen				/*             'nach'-Zustand nicht instabil  */
			    (zustand, neg_kante_1, neg_kante_2);	/* 		erste + zweite Kante kuerzen  */
		        }
		    }
		}

		neg_kante_1->gueltig = -1;				/*    1. Kante ungueltig setzen		      */
		kanten_reduziert++;					/*    reduzierte Kanten hochzaehlen	      */      
	    } 
	}
}




/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	Zyklus_Ueberpruefung ()											      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                         	GESPEICHERT IN:                               */
/*  	- Bearbeite_instabilen_Zustand ()   				"negative_Pfade_Laenge_1.c"		      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*	- falls Zustand als schon durchlaufen gekennzeichnet		     					      */
/*	      Zyklus-Ausgabe						     					      */
/*	  ansonsten als durchlaufen kennzeichnen			     					      */
/*									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*	- SEMANTIK							"define.h"			              */
/*									     					      */
/**********************************************************************************************************************/


Zyklus_Ueberpruefung (zustand)
long	zustand;

{
	register long	i;

	if (durchlaufen[zustand] == 1) {				/* 'zustand' schon mal erreicht?	      */
	    SEMANTIK"\n\n--> Semantik-Fehler:  Zyklus  (%ld ---> ... ---> %ld)\n\n",zustand, zustand);

	    for (i = 0; i < stelle; i++)				/* Zykluskette ausdrucken		      */
		if (reihen_folge[i] == zustand)
		    fprintf(stderr,"--- *%ld* ",reihen_folge[i]);
		else
		    fprintf(stderr,"--- %ld ",reihen_folge[i]);
	    fprintf(stderr,"\n\n");
	    exit (1);
	} else
	    durchlaufen[zustand] = 1;					/* 'zustand' erreicht			      */
}

