static char *SccsId = "@(#)PEPSY:\tposnegnorm.c\t1.2\t92/03/27 11:45:43";
/**********************************************************************************************************************/
/*													              */
/*	MODULNAME:							ERSTELLT:				      */
/*	"pos_entfernen_neg_normieren.c"					Mai 1989	(Peter Menninger)	      */
/*													              */
/*	KURZ-BESCHREIBUNG:											      */
/*	Bei instabilen Zustaenden werden:									      */
/*	- Kanten mit positiver 'uebergangs_rate' als ungueltig gekennzeichnet			     	     	      */
/*	- negative 'uebergangs_rate'n (Verursacher der Instabilitaet) werden auf Gesamtsumme -1 (pro Zustand) normiert*/
/*													              */
/*	FUNKTIONEN:											              */
/*  	- Positiv_entfernen ()											      */
/*	- Negativ_normieren ()											      */
/*													              */
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/*													              */
/*	FUNKTION												      */
/*  	Positiv_entfernen ()											      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                         	GESPEICHERT IN:                               */
/*  	- Berechnungen ()						"main.c"     				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	-  Kanten mit positiver 'uebergangs_rate' bei instabilen Zustaenden 				 	      */
/*     	   werden als ungueltig  gekennzeichnet				     	    			 	      */
/*									     					      */
/*	VERWENDETE FUNKTIOEN:						GESPEICHERT IN:				      */
/*	- Head_Meldung_positiv_entfernen ()				"meldungen.c"				      */
/*	- Work_Meldung_positiv_entfernen ()				"meldungen.c"				      */
/*	- Balken ()							"meldungen.c"				      */
/*									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:			     	      */
/*	- ALLE_INSTABILE_ZUSTAENDE 					"define.h"				      */
/*      - ALLE_KANTEN 	(KEINE_KANTE)   			       	"define.h"			              */
/*									     					      */
/**********************************************************************************************************************/

#include "define.h"
#include "defs.h"


Positiv_entfernen ()

{
	register long			zustand;			/* durchlaeuft alle Zustaende		      */
	register struct kante		*kante;				/* durchlaeuft alle Kanten		      */
	double				norm;				/* Gesamtsumme der negativen 'uebergangs_rate'*/
	long	        		bereits_gedruckt = 0;		/* Anzahl Spalten fuer die (Bildschirm)Ausgabe*/

	Head_Meldung_positiv_entfernen ();				/* Ueberschrift				      */


	Balken (2);

	ALLE_INSTABILE_ZUSTAENDE 					/* alle instabile Zustaende bearbeiten        */

	    ALLE_KANTEN (kante, zustand) {				/*    alle Kanten bearbeiten                  */
		if (kante->uebergangs_rate > 0.0) {			/*         mit positiver Uebergangsrate	      */
		    kante->gueltig = -1;  				/*	       als ungueltig kennzeichnen     */
		    kanten_reduziert++;					/*             zaehlen			      */
		    if (DBG_2)						/*             falls dbg == 2		      */
		        Work_Meldung_positiv_entfernen 			/*                Anzeige der Taetigkeit      */
			(bereits_gedruckt++, zustand, kante);
		}
	    }
	}

	Balken (2);
}




/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	Negativ_normieren ()											      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:         	                                GESPEICHERT IN:                               */
/*  	- Berechnungen ()						"main.c"     				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	- negative Uebergangsraten (Verursacher der Instabilitaet) auf Gesamtsumme -1 (pro Zustand) normieren 	      */
/*									     					      */
/*	VERWENDETE FUNKTIOEN:						GESPEICHERT IN:				      */
/*	- Head_Meldung_negativ_normieren ()				"meldungen.c"				      */
/*	- Work_Meldung_negativ_normieren ()				"meldungen.c"				      */
/*	- Balken ()							"meldungen.c"				      */
/*													              */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*	- ALLE_INSTABILE_ZUSTAENDE 					"define.h"				      */
/*      - ALLE_KANTEN 	(KEINE_KANTE)   			       	"define.h"			              */
/*													              */
/**********************************************************************************************************************/


Negativ_normieren ()

{
	register long			zustand;			/* durchlaueft alle Zustaende		      */
	register struct kante		*kante;				/* durchlaeuft alle Kanten		      */
	register long			bereits_gedruckt = 0;		/* Anzahl Spalten fuer die (Bildschirm)Ausgabe*/
	register double			norm;				/* Gesamtsumme der negativen Uebergangsraten  */
									/*     pro Zustand			      */

	Head_Meldung_negativ_normieren ();				/* Ueberschrift				      */


	Balken (2);

	ALLE_INSTABILE_ZUSTAENDE 					/* alle instabilen Zustaende bearbeiten	      */

	    ALLE_KANTEN (kante, zustand) {				/*    alle Kanten bearbeiten		      */

	 	if (kante->uebergangs_rate < 0.0) 			/*       falls 'uebergangs_rate < 0.0	      */
		    if (norm !=  1.0) {					/*             falls Norm ungleich 1	      */
		        if (DBG_2) 					/*                falls dbg == 2	      */
		            Work_Meldung_negativ_normieren 		/*                   Anzeige der Taetigkeits  */
			    (bereits_gedruckt++, zustand, kante, norm);
		        kante->uebergangs_rate /= norm;			/*                 Uebergangsrate normieren   */
		    }
	    }

	}

	Balken (2);
}
