static char *SccsId = "@(#)PEPSY:\troutinen.c\t1.3\t93/12/21 11:59:38";
/**********************************************************************************************************************/
/*													              */
/*	MODULNAME:							ERSTELLT:				      */
/*	"routinen.c"							Mai 1989	(Peter Menninger)	      */
/*													              */
/*	FUNKTIONEN:											              */
/*  	- Verdichten ()												      */
/*  	- Zwei_Kanten_kuerzen () 										      */
/*  	- Instabil_berechnen ()											      */
/*  	- Ungueltige_Kanten_entfernen ()									      */
/*													              */
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	Verdichten (wer)						     		     	     	     	      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                              	      	GESPEICHERT IN:				      */
/*  	- Verdichten_Matrix ()						"verdichten.c"	 			      */
/*  	- Matrix_einlesen ()						"io_datei.c"	 			      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	sucht 'wer' innerhalb von 'codierung' und gibt die Position (= Verdichtung) zurueck			      */
/*    	im Fehlerfall Meldung und Abbruch	(bewirkt Verdichten der Zustaende)				      */
/*									     					      */
/*  	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:			     	      */
/*  	- Meldung_ausserhalb ()						"meldungen.c"				      */
/*  	- Meldung_nicht_innerhalb ()					"meldungen.c"				      */
/* 									     					      */
/**********************************************************************************************************************/

#include "define.h"
#include "defs.h"

long Verdichten (wer, von, nach)
unsigned long	wer;
unsigned long	von;
unsigned long	nach;

{
	register long		right;				        /* rechte Position innerhalb von 'codierung'  */
	register long		left;				        /* linke  Position innerhalb von 'codierung'  */
	register long		position;			        /* verdichteter 'nach'-Zustand (=Position     */
								        /*   von 'wer')				      */
	left  = 0;						        /* Vorbesetzen:    linke Grenze		      */
	right = (dimension_in - dimension_reduziert - 1);		/*                 rechte Grenze	      */

	if ((wer < *codierung) || (wer > *(codierung+right)))           /* zuverdichtender Zustand ausserhalb von     */
	    Meldung_ausserhalb (von, nach, wer,				/* 'codierung'				      */
	    *codierung, *(codierung+right));

        while (right >= left) {					        /* solange rechte Grenze groesser als linke   */
	    position = (right + left) / 2;			        /*     Mitte nehmen			      */

	    if (codierung[position] == wer)	 		        /*     falls dort 'wer' gefunden	      */
		return position;				        /*     Position (=fortlaufende Nummer)	      */
									/*     zurueckliefern			      */
	    if (codierung[position] > wer)			        /*     ansonsten zuuntersuchendes Intervall   */
	        right = position - 1;				        /*     durch Anpassen der Grenzen	      */
	    else						        /*     verkleinern			      */
	        left = position + 1;
        }

	Meldung_nicht_innerhalb (von, nach, wer, 		        /* zuverdichtender Zustand nicht innerhalb    */
	*codierung, *(codierung+dimension_in-dimension_reduziert-1));	/* von 'codierung'			      */
}






/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	Zwei_Kanten_kuerzen ()									     		      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                         	GESPEICHERT IN:                               */
/*  	- Bearbeite_instabilen_Zustand ()				"negative_Pfade_Laenge_1.c" 		      */
/*	- Instabile_Zustaende_entfernen ()				"instabilen_Zustand_aushaengen.c"	      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*  	- zwei verbundene Kanten zu einer Kante zusammenfassen		         				      */
/*	 (x ----------> y ----------> z  ==>  x ----------> z)		     					      */
/*									     					      */
/*  	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:                               */
/*	- Meldung_aus ()						"meldungen.c"				      */
/*	- Meldung_und ()						"meldungen.c"				      */
/*	- Neue_Kante ()							"meldungen.c"				      */
/*									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*  	- CALLOC(name,anz,elem)						"define.h"				      */
/*  	- ZUWEISEN							"define.h"				      */
/*  	- KEINE_KANTE							"define.h"				      */
/*									     					      */
/**********************************************************************************************************************/


Zwei_Kanten_kuerzen (zustand, kante_1, kante_2)
long			zustand;					/* 'von'-Zustand			      */
struct kante		*kante_1;					/* 2 Kanten hintereinander		      */
struct kante		*kante_2;					/* x ------------> y ------------> z	      */ 

{	
	register struct kante	*kante;
	register struct kante	*vorher;

	if (DBG_2) {
	    Meldung_aus (zustand, kante_1);				/* Taetigkeits - Meldungen		      */
	    Meldung_und (kante_1->nach, kante_2);
	}

	for (kante  = vorher = &liste[zustand]; 
	     kante != KEINE_KANTE; 
	     vorher  = kante, kante = kante->next) {

	    if (kante->nach == kante_2->nach)  /* Kante schon vorhanden */
		break;
	}

	if (kante == KEINE_KANTE) {
	    CALLOC (kante, 1, struct kante);			
	    ZUWEISEN (kante, kante_2->nach, - kante_1->uebergangs_rate * kante_2->uebergangs_rate);
	    if (DBG_2) Neue_Kante (zustand, kante, "neu");
		vorher->next = kante;
	    kanten_reduziert--;
	} else if (kante->gueltig == 1) {
	    kante->uebergangs_rate+= - kante_1->uebergangs_rate * kante_2->uebergangs_rate; 
		/*
	    vorher->next = kante->next;
		*/
	    if (DBG_2) Neue_Kante (zustand, kante, "bereits gueltig");
	} else {
	    kante->uebergangs_rate = - kante_1->uebergangs_rate * kante_2->uebergangs_rate; 
	    kante->gueltig = 1;
		/*
	    vorher->next = kante->next;
		*/
	    kanten_reduziert--;
	    if (DBG_2) Neue_Kante (zustand, kante, "vorher ungueltig");
	}

	/*
	kante->next = kante_1->next;
	kante_1->next = kante->next;
	*/

}






/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/*  	Instabil_berechnen ()									     		      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                         	GESPEICHERT IN:                               */
/*	- Berechnungen () 						"main.c"				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*	- speichert im Feld instabil den Betrag der negativen Uebergangsraten pro Zustand			      */
/*									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*  	- CALLOC(name,anz,elem)						"define.h"				      */
/*	- ALLE_ZUSTAENDE						"define.h"				      */
/*	- ALLE_KANTEN							"define.h"				      */
/*									     					      */
/**********************************************************************************************************************/


Instabil_berechnen ()

{
	register long		zustand;
	register struct kante	*kante;

	CALLOC (instabil, dimension_aktuell, double);

	ALLE_ZUSTAENDE {

	    ALLE_KANTEN (kante, zustand) {

		if (kante->uebergangs_rate < 0.0) 
		    instabil[zustand] += -kante->uebergangs_rate;
	    }

	}
}






/**********************************************************************************************************************/
/*													              */
/*  	FUNKTION												      */
/* 	Ungueltige_Kanten_entfernen ()								     		      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                         	GESPEICHERT IN:                               */
/*	- Berechnungen () 						"main.c"				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*	- Kanten von erreichbaren Zustaenden mit den Wert 1 auf der Strukturvariable gueltig werden		      */
/*	  von der liste auf liste_neu kopiert									      */
/*	- anschliesend liste auf die Werte von liste_neu zuruecksetzen						      */
/*									     					      */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:		     	       	      */
/*  	- CALLOC(name,anz,elem)						"define.h"				      */
/*	- ZUWEISEN 						     	"define.h"				      */
/*	- ALLE_KANTEN							"define.h"				      */
/*									     					      */
/**********************************************************************************************************************/

#define ERSTE_GUELTIGE_KANTE for (kante = &liste[zustand]; kante != KEINE_KANTE && kante->gueltig != 1; kante = kante->next);

Ungueltige_Kanten_entfernen ()

{
	register long		zustand;
	register struct kante	*kante;
	register struct kante	*kante_neu;
	register struct kante	*naechste_frei_stelle;
	register struct kante	*liste_neu;

	CALLOC (liste_neu, dimension_aktuell + kanten_in - kanten_reduziert, struct kante);

	naechste_frei_stelle = &liste_neu[dimension_aktuell];

	ALLE_ZUSTAENDE {
	    if (erreichbar[zustand] == -1)
	        continue;
	    ERSTE_GUELTIGE_KANTE ;

	    if (kante != KEINE_KANTE) { 
	        kante_neu = &liste_neu[zustand];
		ZUWEISEN (kante_neu, kante->nach, kante->uebergangs_rate);

		for (kante = kante->next;
		     kante != KEINE_KANTE;
		     kante  = kante->next) {
		    if (kante->gueltig == 1) {
			kante_neu->next = naechste_frei_stelle;
			ZUWEISEN (naechste_frei_stelle, kante->nach, kante->uebergangs_rate);
			naechste_frei_stelle++;
			kante_neu = kante_neu->next;
		    }
		}
	    }
	}

	free (liste);
	liste = liste_neu;
}

