static char *SccsId = "@(#)PEPSY:\tverdichten.c\t1.3\t93/12/21 11:59:40";
/**********************************************************************************************************************/
/*													              */
/*	MODULNAME:							ERSTELLT:				      */
/*	"verdichten.c"							Mai 1989	(Peter Menninger)	      */
/*													              */
/*	KURZ-BESCHREIBUNG:											      */
/*	Zustaende  in  fortlaufende  Numerierung  verdichten (Luecken durch nicht erreichbare Zustaende)	      */
/*													              */
/*	FUNKTIONEN:											              */
/*  	- Verdichten_Codierung ()										      */
/*  	- Verdichten_Matrix ()											      */
/*													              */
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/*													              */
/*	FUNKTION												      */
/*  	Verdichten_Codierung ()											      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                         	GESPEICHERT IN:                               */
/*  	- Berechnungen ()						"main.c"     				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*	nicht erreichbare Zustaende werden entfernt ==> hoehere Zustands-Nummern ruecken in diese Luecken auf	      */
/*	unverdichtete Zustaende werden in 'codierung_neu' gespeichert (fuer Verdichten_Matrix)			      */
/*	verdichtete Zustaende in 'zuschreibendeZustaende_neu'		     					      */
/*	Original-Zustaende werden in 'codierung_original_neu' angepasst		     				      */
/*									     					      */
/*	VERWENDETE FUNKTIOEN:						GESPEICHERT IN:				      */
/*	- Head_Meldung_Codierung_verdichten ()				"meldungen.c"				      */
/*	- Work_Meldung_Codierung_verdichten ()				"meldungen.c"				      */
/*	- Balken ()							"meldungen.c"				      */
/*													              */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:			     	      */
/*	- CALLOC							"define.h"			              */
/*	- DB...								"define.h"			              */
/*	- ALLE_ZUSTAENDE 						"define.h"			              */
/*									     					      */
/*									     					      */
/**********************************************************************************************************************/


#include	"define.h"
#include	"defs.h"


Verdichten_Codierung ()

{
	register unsigned long	*codierung_neu;				/* enthaelt unverdichtete Zustaende	      */
	register unsigned long	*codierung_original_neu;		/* rettet die Original-Zustands-Nummern	      */
	register long  		*zuschreibenderZustand_neu;		/* verdichtete Zustaende		      */
	register long		zustand;				/* durchlaueft alle Zustaende 		      */
	register long		zustand_neu = 0;			/* neue Zustandsnummer 			      */
	long			bereits_gedruckt = 0;			/* Anzahl Spalten fuer die 'stdout'-Ausgabe   */


	Head_Meldung_Codierung_verdichten ();				/* Ueberschrift				      */

	/**************************************************************************************************************/
	/*  Speicherplatz fuer folgende Variablen anfordern:							      */
	/*  codierung_original_neu:	Original-Zustaende (aus CODIERUNG_TMP unbehandelt!)			      */
	/*  codierung_neu:	        nach Verdichtungsschritt alte Zustaende 				      */
	/*  zuschreibenderZustand_neu:  neue Zustands-Nummer  anstatt der unverdichteten Zustands-Nummer 	      */
	/**************************************************************************************************************/
	CALLOC (codierung_neu, 		   dimension_in - dimension_reduziert, unsigned long);
	CALLOC (codierung_original_neu,    dimension_in - dimension_reduziert, unsigned long);
	CALLOC (zuschreibenderZustand_neu, dimension_in - dimension_reduziert, long);


	Balken (2);							        

	ALLE_ZUSTAENDE {						/* alle Zustaende bearbeiten		      */
	    if (erreichbar[zustand] == -1) 				/*     nur erreichbare weiterbearbeiten       */
		continue;						
	    
	    codierung_neu[zustand_neu] = zustand;			/* in 'codierung' alten Zustand speichern     */
	    codierung_original_neu[zustand_neu]=codierung_original[zustand]; /* altes erreichbares Original retten    */
	    zuschreibenderZustand_neu[zustand_neu] = zustand_neu;	/* neuer Zustand heisst 'zustand_neu'	      */

	    if (DBG_2 && 						/* falls dbg == 2 			      */
		codierung_neu[zustand_neu] != zustand_neu)		/*    und Aenderung bei der Verdichtung       */

	        Work_Meldung_Codierung_verdichten 			/*    Anzeige der Taetigkeit		      */
		(bereits_gedruckt++, codierung_neu, zustand_neu);

	    zustand_neu++;						/* Position der neuen Zustaende erhoehen      */
	}	    							/* (= neuer Zustand-Nummer)		      */

	Balken (2);							        

	free (codierung);						/* alte Speicherbereiche freigeben	      */
	free (zuschreibenderZustand);
	free (codierung_original);

	codierung 	      = codierung_neu;				/* alte Zeiger auf neue Speicherbereiche      */
	zuschreibenderZustand = zuschreibenderZustand_neu;		/* zuruecksetzen			      */
	codierung_original    = codierung_original_neu;
}




/**********************************************************************************************************************/
/*													              */
/*	FUNKTION												      */
/*  	Verdichten_Matrix ()											      */
/*  									    					      */
/*  	WIRD AUFGERUFEN IN:                                         	GESPEICHERT IN:                               */
/*  	- Berechnungen ()						"main.c"     				      */
/*									     					      */
/*  	BESCHREIBUNG:									                              */
/*	- bei jedem Zustand: nicht erreichbare Zustaende ueberspringen						      */
/*			     Zustand verdichten				     					      */
/*			     alle Kantenendpunkte verdichten		     					      */
/*	- alte 'liste' loeschen, neue 'liste' verwenden			     					      */
/*									     					      */
/*									     					      */
/*	VERWENDETE FUNKTIONEN:						GESPEICHERT IN:				      */
/*	- Head_Meldung_Matrix_verdichten ()				"meldungen.c"				      */
/*	- Work_Meldung_Matrix_verdichten ()				"meldungen.c"				      */
/*	- Verdichten ()							"routinen.c"				      */
/*													              */
/*  	VERWENDETE ABKUERZUNGEN:					DEFINED IN:			     	      */
/*	- CALLOC							"define.h"			              */
/*	- DB...								"define.h"			              */
/*	- ALLE_ZUSTAENDE 						"define.h"			              */
/*	- ZUWEISEN 							"define.h"			              */
/*									     					      */
/**********************************************************************************************************************/

#define TAETIGKEITSMELDUNG \
	if (DBG_2 && !(zustand == von__verdichtet && kante->nach == nach_verdichtet)) \
	    Work_Meldung_Matrix_verdichten \
	    (bereits_gedruckt++, zustand, von__verdichtet, kante->nach, nach_verdichtet);


Verdichten_Matrix ()

{
	register long		zustand;
	register long		zustand_neu = 0;
	register struct kante	*kante;
	register struct kante	*kante_neu;
	register struct kante	*naechste_frei_stelle;
	register struct kante	*liste_neu;
	long			von__verdichtet;
	long			nach_verdichtet;
	long			bereits_gedruckt = 0;
	long			Verdichten ();


	Head_Meldung_Matrix_verdichten ();				/* Ueberschrift  			      */

	CALLOC (liste_neu, dimension_in - dimension_reduziert + 	/* Speicherplatz fuer verdichtete Kanten      */
	                   kanten_in - kanten_reduziert, struct kante);
	naechste_frei_stelle = &liste_neu[dimension_in - dimension_reduziert];	/* Initialisierung: 2. Teil der Liste */


	Balken (2);							        

	ALLE_ZUSTAENDE {						/* alle Zustaende bearbeiten		      */
	    if (erreichbar[zustand] == -1) {				/*    nur erreichbare weiterbearbeiten 	      */
		erreichbar[zustand] = 0;				/*    'erreichbar' zuruecksetzen	      */
		continue;
	    }

	    kante     = &liste[zustand];				/* erste Kante des unverdichteten Zustandes   */
	    kante_neu = &liste_neu[zustand_neu++];			/* erste Kante des   verdichteten Zustandes   */

	    von__verdichtet = Verdichten (zustand, zustand, kante->nach);	 /* erste Kante verdichten	      */
	    nach_verdichtet = Verdichten (kante->nach, zustand, kante->nach);

	    TAETIGKEITSMELDUNG ;
	    
	    ZUWEISEN (kante_neu, nach_verdichtet, kante->uebergangs_rate);	 /* Werte an neue Kante zuweisen      */

	    kante = kante->next;						 /* naechste Kante nehmen	      */

	    while (kante != KEINE_KANTE) {					 /* alle restlichen Kanten bearbeiten */
		nach_verdichtet = Verdichten (kante->nach, zustand, kante->nach);/* 'nach'-Zustand verdichten	      */

	        TAETIGKEITSMELDUNG ;
		
		kante_neu->next = naechste_frei_stelle;			/* Platz fuer neue naechste Kante	      */
		ZUWEISEN (naechste_frei_stelle, 
			  nach_verdichtet, kante->uebergangs_rate);	/* Werte zuweisen			      */
		naechste_frei_stelle++;					/* neuen Platz suchen			      */
		kante_neu = kante_neu->next;				/* neue Kante weiterschalten		      */
		kante     = kante->next;				/* alte Kante weiterschalten		      */
	    }
	}

	Balken (2);							        


	free (liste);							/* alte 'liste' freigeben		      */

	liste = liste_neu;						/* Zeiger auf neuberechnete Liste	      */
	dimension_aktuell = dimension_in - dimension_reduziert;		/* aktuelle Dimension berichtigen	      */
}
