/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

/***********************************************************
************************************************************
**
**	Modulname: allg.c
**
**
**	Beschreibung:
**  In diesen Modul befinden sich allgemeine, verfahrenunab-
**  haengige Funktionen, die von jedem integrierten Dekom-
**  positionsverfahren benutzt werden koennen.
**
**  Liste der Funktionen:
**  Die Deklaration der globalen Funktionen erfolgt in den 
**  Modul allg.h
**    globale Funktionen:
**      void BerAnkRaten()
**      void CheckStab()
**      void GetOptionen()
**      void Ber_ca2_Kn()
**      void BerBedRaten_Kn()
**      int  CheckGenauigkeit()
**      void InitVarKoeff_alt()
**      void InitVarKoeff()
**      void InitVerfahren()
**      int  CheckKnoten()
**      void BerUebergangsRaten()
**
***********************************************************
***********************************************************/

static char *SccsId = "@(#)PEPSY:\tallg.c\t1.3\t95/05/22 10:52:26";

#define ALLGEMEIN_C
#define EXTERN_KNOTENTYPEN

#include "libpepsy.h"
#include "def.h"

extern int GetVerfahren();	/* init.c */
extern int GetFormelLeistung();	/* init.c */

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: BerAnkRaten
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Diese Funktion berechnet die Ankunftsraten fuer jeden 
**  Knoten pro Klasse
**
**
************************************************************
***********************************************************/

void BerAnkRaten()
{
    double **matrix;
    double *lambda0;
    double *lambda;

    int i, j, k;

    dbgtext(DBG_FKT1, "Funktion  berechne_Ankunftsraten()\n");

    /* Speicherplatz fuer die Matrix matrix und die Vektoren 
       lambda0 und lambda allokieren */

    matrix = (double **)calloc(AnzKnoten, sizeof(double *));

    for (i = 0; i < AnzKnoten; i++)
    {
	matrix[i] = (double *)calloc(AnzKnoten, sizeof(double));
    }

    lambda0 = (double *)calloc(AnzKnoten, sizeof(double));
    lambda  = (double *)calloc(AnzKnoten, sizeof(double));

    for (k = 0; k < AnzKlassen; k++)
    {
	for (i = 0; i < AnzKnoten; i++)
	{
	    for (j = 0; j < AnzKnoten; j++)
	    {
		/* Matrix transponieren */

		matrix[i][j] = klasse[k].uew[j+1][i+1];

		if (i == j)
		    matrix[i][j] -= 1.0;
	    }

	    lambda0[i] = -1.0 * klasse[k].lambda * klasse[k].uew[0][i+1];
	}

	/* gauss(): Bibliotheksfunktion aus 'libpepsy' */
	
	gauss(matrix, lambda, lambda0, AnzKnoten, 0.0000001);

	/* Einlesen der Werte fuer lambda in die Struktur */

	for (i = 0; i < AnzKnoten; i++)
	{
	    klasse[k].knoten[i+1].lambda = lambda[i];
	}
    }

    free(matrix);
    free(lambda);
    free(lambda0);

    /* berechne die Ankunftsraten fuer jeden Knoten des gesamten Netzes */
    /* muss nur berechnet werden, falls Anzahl der Klassen groesser als 1 */

    for (i = 1; i <= AnzKnoten; i++) {
	knoten[i].lambda = 0.0;
	for (k = 0; k < AnzKlassen; k++) {
	    knoten[i].lambda += klasse[k].knoten[i].lambda;
	}
    }


    dbgtext(DBG_L1, "Berechne Ankunftsraten pro Klasse und Knoten\n");
    for (k = 0; k < AnzKlassen; k++) {
	dbginfo(DBG_L1, "KLASSE %d\n", k);
	for (i = 1; i <= AnzKnoten; i++) {
	    dbginfo(DBG_L1, "\t%f",klasse[k].knoten[i].lambda);
	}
	dbgtext(DBG_L1, "\n");
    }
    dbgtext(DBG_L1, "\n");
    dbgtext(DBG_L1, "Berechne Ankunftsraten pro Knoten\n");
    for (i = 1; i <= AnzKnoten; i++) {
	dbginfo(DBG_L1, "\t%6.2f",knoten[i].lambda);
    }
    dbgtext(DBG_L1, "\n");
    dbgtext(DBG_L1, "\n");

    dbgtext(DBG_FKT1, "ENDE berechne_Ankunftsraten()\n");

}


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: CheckStab
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Die Funktion ueberprueft, ob sich das gegebene Netzwerk
**  im Gleichgewichtszustand befindet. Ist dies nicht der 
**  Fall, so bricht sie das Programm mit einer Fehlermeldung 
**  ab.
**
************************************************************
***********************************************************/

void CheckStab()
{

	int i, k;

	for (k = 0; k < AnzKlassen; k++) {
		for (i = 1; i <= AnzKnoten; i++) {

			/* Ueberpruefung, ob alle Auslastungen pro Knoten und Klasse
			   kleiner als eins sind */

			if ((klasse[k].knoten[i].rho = BerRho(i, k)) >= 1.0) {

				fprintf(stderr, "Netz ist nicht analysierbar\n");
				fprintf(stderr, "Auslastung im Knoten %d fuer Klasse %d >= 1.0\n", i, k);
				printf("0 0 0\n");
				exit(1);

			} /* end if */
		} /* end for i */
	} /* end for k */

	for (i = 1; i <= AnzKnoten; i++) {

			/* Ueberpruefung, ob alle Auslastungen pro Knoten
			   kleiner als eins sind */

		if ((knoten[i].rho = BerGRho(i)) >= 1.0) {

			fprintf(stderr, "Netz ist nicht analysierbar\n");
			fprintf(stderr, "Auslastung im Knoten %d >= 1.0\n", i);
			printf("0 0 0\n");
			exit(1);

		}

	} /* end for i */

} /* end for CheckStab() */


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: GetOptionen
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int  argc
**                      char **argv
**
**	Beschreibung:
**  Diese Funktion liest die  gesetzten Optionen ein und 
**  initialisiert die globalen Variablen fuer das zu ver-
**  wendende Analyseverfahren
**
************************************************************
***********************************************************/

void GetOptionen(argc, argv)
int argc;
char **argv;
{
	extern int SccsRelease, SccsLevel;
	extern char *optarg;
	int c;

	verfahren_name	= "auto";
	formel_name	= "auto";

	while ((c = getopt(argc, argv, "Vd:v:f:")) != -1) {

		switch(c)
		{
			case 'V':
				fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
				exit(0);
				break;
			/* Mit der Option v kann ein
			   Dekompositionsverfahren ausgewaehlt werden */
			case 'v':
				verfahren_name = (char *)malloc((strlen(optarg)+1) * sizeof(char));
				strcpy(verfahren_name ,optarg);
				break;
			/* Mit der Option d kann das
			   Debugniveau ausgewaehlt werden */
			case 'd':
				if((dbg = atoi(optarg)) <= 0){
					dbg = 1;
				}
				break;
			/* Mit der Option f kann die Formel zur Berechnung der 
			   Leistungsgroessen ausgewaehlt werden */
			case 'f':
				formel_name = (char *)malloc((strlen(optarg)+1) * sizeof(char));
				strcpy(formel_name ,optarg);
				FLAG_FLG = 1;
				break;

		} /*  end switch */

	} /* end while */

	dbginfo(DBG_OPT, "Verfahren ist %s\n", verfahren_name)
	dbginfo(DBG_OPT, "Formel ist %s\n", formel_name)
	dbginfo(DBG_OPT, "Debug-Level ist %d\n", dbg);


} /* end GetOptionen() */


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Ber_ca2_Kn
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int node - Nummer des zu anlysierenden 
**                                 Knotens
**
**	Beschreibung:
**  Berechnet fuer den Knoten node den quad. Variationskoeff. 
**  des Ankunftsprozesses aus den quad. Variationskoeff. des
**  Knotens pro Klasse.
**
**
************************************************************
***********************************************************/

void Ber_ca2_Kn(node)
int node;
{
	int k;
	double var;

	dbgtext(DBG_FKT1, "Funktion Ber_ca2_Kn()\n")

	var = 0.0;

	for (k = 0; k < AnzKlassen; k++) {

		var += klasse[k].knoten[node].lambda / knoten[node].lambda * 
				(klasse[k].knoten[node].c2A - 1.0);
	} /* end for k */
		
	knoten[node].c2A = 1.0 + var;
	dbginfo(DBG_L1, "knoten[%d].c2A ist ", node)
	dbginfo(DBG_L1, "%4.4f", knoten[node].c2A)

} /* end Ber_ca2_Kn() */


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: BerBedRaten_Kn
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Berechnet fuer alle Knoten den quad. Var.koeff. des Be-
**  dienprozesses aus den quad. Var.koeff. der Bedienprozesse 
**  pro Knoten und Klasse
**
************************************************************
***********************************************************/

void BerBedRaten_Kn()
{
	int i, k;
	double var0, var1, var2;

	dbgtext(DBG_FKT1, "Funktion BerBedRaten_Kn()\n")

	for (i = 1; i <= AnzKnoten; i++){
		if(knoten[i].KnotenTyp == TYP4){
			knoten[i].mue = 1.0;
			knoten[i].c2B = 1.0;
		}
		else {
			var0 = 0;
			var1 = 0;
			var2 = 0;
			for (k = 0; k < AnzKlassen; k++){
				var0 += klasse[k].knoten[i].lambda / (knoten[i].AnzServer * 
					klasse[k].knoten[i].mue * knoten[i].lambda);
				var1 = pow(knoten[i].AnzServer * klasse[k].knoten[i].mue, 2.0);
				var2 += klasse[k].knoten[i].lambda / (knoten[i].lambda * var1) * 
					(1.0 + klasse[k].knoten[i].c2B);
			
			} /* end for k */

			knoten[i].mue = 1.0 / var0;
			var0 = pow(var0, 2.0);
			knoten[i].c2B = (var2 - var0) / var0;
		}

	} /* end for i */

	dbgtext(DBG_L1, "Bedienrate pro Knoten\n")
	for (i = 1; i <= AnzKnoten; i++){
		dbginfo(DBG_L1, "\t%6.2f", knoten[i].mue)
	} /* end for i */
	dbgtext(DBG_L1, "\n")

	dbgtext(DBG_L1, "quad. Varkoeff. des Bedienprozesses pro Knoten\n")
	for (i = 1; i <= AnzKnoten; i++){
		dbginfo(DBG_L1, "\t%6.2f", knoten[i].c2B)
	} /* end for i */
	dbgtext(DBG_L1, "\n")

} /* end BerBedRaten_Kn() */

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: CheckGenauigkeit
**
**	Funktionstyp: int
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  
**
**
************************************************************
***********************************************************/


int CheckGenauigkeit()
{
	int i, j, k, epsilon;
	double diff;
				/* epsilon verdeckt globales epsilon !!!!! */
	/* epsilon = 0.0000001; */
	epsilon = 0.002;

	for(k=0; k < AnzKlassen; k++){
		for (i=1; i<=AnzKnoten; i++) {
			for (j=1; j<=AnzKnoten; j++) {

				diff = klasse[k].c[i][j] - klasse[k].c_alt[i][j];
				if((diff > epsilon) || (diff < epsilon)){
					/* Genauigkeit noch nicht erreicht -> weiterer
					   Iterationsschritt erforderlich */
					return(1);
				}

			} /* end for j */

		} /* end for i */

	} /* end for k */


	/* Genauigkeit erreicht -> Abbruchkriterium erfuellt */

	return(0);

} /* end CheckGenauigkeit() */


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: void InitVarKoeff_alt()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Zwischenspeicherung der zuletzt berechneten Var.koeff.
**  zwecks Ueberpruefung, ob Fortschritt in der iterativen
**  Berechnung der Var.koeff. erzielt wurde.
**
************************************************************
***********************************************************/

void InitVarKoeff_alt()
{
	int i, j, k;

	for(k=0; k < AnzKlassen; k++){
		for (i=1; i<=AnzKnoten; i++) {
			for (j=1; j<=AnzKnoten; j++) {

				klasse[k].c_alt[i][j] = klasse[k].c[i][j];

			} /* end for j */

		} /* end for i */

	} /* end for k */

} /* end InitVarKoeff_alt() */


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: InitVarKoeff()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Initialisierung der quadrierten Var.koeff. mit eins als 
**  Startwert fuer die Iterative Berechnung derselbigen
**
************************************************************
***********************************************************/

void InitVarKoeff()
{
	int i, j, k;

	dbgtext(DBG_FKT1, "Initialisierung der Varkoeff\n")

	for(k=0; k < AnzKlassen; k++){
		for (i=0; i<=AnzKnoten; i++) {
			for (j=1; j<=AnzKnoten; j++) {
				if ((klasse[k].uew[i][j] > 0.0) && (i == 0)) {
					klasse[k].c[i][j] = klasse[k].knoten[j].c2A;
				}
				else if(klasse[k].uew[i][j] > 0.0) {
					klasse[k].c[i][j] = 1.0;
				}
				else {
					klasse[k].c[i][j] = -1.0;
				}

				dbginfo(DBG_L1, "\t%6.2f", klasse[k].c[i][j])

			} /* end for j */
	
			dbgtext(DBG_L1, "\n")

		} /* end for i */

		dbgtext(DBG_L1, "\n")

	} /* end for k */

	dbgtext(DBG_L1, "\n")

/*	return(OK);*/

} /* end InitVarKoeff() */


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: InitVerfahren()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int  argc
**                      char **argv
**
**	Beschreibung:
**  Diese Funktion wird im Hauptprogramm im Modul main.c 
**  aufgerufen. Initialisierung des Analysealgorithmuses,
**  der aus Dekompositionsmethode und Formel zur Berechnung
**  der Leistungsgroessen besteht.
**
************************************************************
***********************************************************/

void InitVerfahren(argc, argv)
int argc;
char **argv;
{
	GetOptionen(argc, argv);

	verfahren_nr = GetVerfahren();

	dbginfo(DBG_L5, "Nr des Verfahren ist %d\n", verfahren_nr)

	formel_nr = GetFormelLeistung();

	dbginfo(DBG_L5, "Nr der Formel ist %d\n", formel_nr)


}


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: CheckKnoten()
**
**	Funktionstyp: int
**
**	Uebergabeparameter: int 
**                      int
**
**	Beschreibung:
**  Ueberpruefung, ob ein gegebenes Element 'Typ' in der Menge 
**  'TypMenge' enthalten ist.
**  Typ symbolisiert dabei einen Knotentypen gemaess der  
**  Definition der Knotentypen in der Include-Datei "d_konst.h". 
**  Die TypMenge ist dabei die Summe der Typen, die fuer ein 
**  Verfahren zugelassen sind. (diese werden fuer jede Dekom-
**  positionsmethode als modulglobale Konstante definiert)
**   
**
************************************************************
***********************************************************/

int CheckKnoten(Typ, TypMenge)
int Typ;
int TypMenge;
{
	int ret;

	if ((Typ & TypMenge) != Typ) {
		ret = ERROR;
	}
	else {
		ret = OK;
	}
	return (ret);
}


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: BerUebergangsRaten()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Berechnet die Uebergangsrate fuer Klasse r von einen  
**  Knoten i zum Knoten j fuer alle Klassen r und alle 
**  Knoten i und j
**
************************************************************
***********************************************************/


void BerUebergangsRaten()
{
    int i, j, k;

    for (k = 0; k < AnzKlassen; k++)
    {
	for (i = 1; i <= AnzKnoten; i++) 
	{
	    for (j = 0; j <= AnzKnoten; j++) 
	    {
		if (j == 0)
		{
		    klasse[k].rate[j][i] = klasse[k].lambda * klasse[k].uew[j][i];
		}
		else
		{
		    klasse[k].rate[j][i] = klasse[k].knoten[j].lambda * klasse[k].uew[j][i];
		}
	    }
	}
    }
}


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: SaveInputData()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Abspeichern der Inputdaten
**
************************************************************
***********************************************************/


void SaveInputData()
{
	int i, k;


	for(k = 0; k < AnzKlassen; k++){
		for(i = 1; i <= AnzKnoten; i++){
			klasse[k].knoten[i].c2A_orig = klasse[k].knoten[i].c2A;
		}
	}



}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: GetOrigInputData()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Abspeichern der Inputdaten
**
************************************************************
***********************************************************/


void GetOrigInputData()
{
	int i, k;

	for(k = 0; k < AnzKlassen; k++){
		for(i = 1; i <= AnzKnoten; i++){
			klasse[k].knoten[i].c2A = klasse[k].knoten[i].c2A_orig;
		}
	}


}

