/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/


/***********************************************************
************************************************************
**
**  Modulname: chylla.c
**
**	Beschreibung:
**  in diesen Modul werden alle verfahrenspezifischen Funk-
**  tionen fuer die Methode nach Chylla definiert.
**  Die Funktionen aus diesen Modul duerfen Funktionen aus 
**  den Modulen 'allgemein.c' und 'leistung.c' verwenden.
**  
**
***********************************************************
***********************************************************/

static char *SccsId = "@(#)PEPSY:\tchylla.c\t1.2\t95/05/19 15:36:45";

#define CHYLLA_C
#define EXTERN_KNOTENTYPEN

#include "init.h"

#define CHYLLA_KN_TYPEN 00000777


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: chylla__berechne_c2A_pro_Knoten_u_Klasse()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int node (Knotennummer)
**
**
**	Beschriebung:
**  Berechnet fuer alle Klassen und alle Knoten den quad.
**  Var.koeff. des Ank.prozesses.
**
************************************************************
***********************************************************/

void chylla__berechne_c2A_pro_Knoten_u_Klasse(node)
int node;
{
    int i, j, k;
    double hv;

    dbgtext(DBG_FKT1, "\nFunktion chylla__berechne_c2A_pro_Knoten_u_Klasse()\n");

	
    for (k = 0; k < AnzKlassen; k++) 
    {
	dbginfo(25, "Klasse %d :\n", k);
	for (i = 1; i <= AnzKnoten; i++)
	{
	    hv = 0.0;
	    for (j = 0; j <= AnzKnoten; j++)
	    {
		if (klasse[k].knoten[i].lambda != 0.0)
		    hv += klasse[k].rate[j][i] / klasse[k].knoten[i].lambda * (klasse[k].c[j][i] - 1.0);
	    }
	    klasse[k].knoten[i].c2A = hv + 1.0;

	    dbginfo(DBG_L1, "\t%6.2f", klasse[k].knoten[i].c2A);
	}

	dbgtext(DBG_L1, "\n");
	    
    }

    dbgtext(DBG_L1, "\n");
}


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: chylla__berechne_c2D_pro_Knoten()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int node (Knotennummer)
**
**	Beschriebung:
**  Berechnet fuer Knoten node den quad. Var.koeff. des 
**  Abgangsprozesses.
**
************************************************************
***********************************************************/

void chylla__berechne_c2D_pro_Knoten(node)
int node; 
{
    double rho, hv1, hv2;

    dbgtext(DBG_FKT1, "\nFunktion chylla__berechne_c2D_pro_Knoten()\n");

    /*	if (knoten[node].KnotenTyp == TYP4)
	rho  = 0.0;
	else
	rho = knoten[node].lambda / knoten[node].mue;*/

    rho = knoten[node].rho;

    if (knoten[node].KnotenTyp == TYP4) 
    {
	knoten[node].c2D = knoten[node].c2A;
    }
    else
    {
	hv1 = pow(Pm( rho, knoten[node].AnzServer), 2.0) * (knoten[node].c2B - 1.0);
	hv2 = (1.0 - Pm( rho, knoten[node].AnzServer)) * (knoten[node].c2A - 1.0);
	knoten[node].c2D = 1.0 + hv1 + hv2;
    } 

    dbginfo2(DBG_L1, "c2D_pro_Knoten(): Knoten[%d]=%f\n", node, knoten[node].c2D);
}


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: chylla__berechne_VarKoeffizienten()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int node (Knotennummer)
**
**
**	Beschriebung:
**  Berechnet fuer Knoten node den quad. Var.koeff. fuer die
**  einzelnen Teilprozesse zu den verschiedenen Knoten nach
**  der Aufsplittung des Abgangsprozesses.
**
************************************************************
***********************************************************/

void chylla__berechne_VarKoeffizienten(node)
int node;
{
    int i, k;

    dbgtext(DBG_FKT1, "\nFunktion chylla__berechne_VarKoeffizienten()\n");

    for (k= 0; k < AnzKlassen; k++)
    {
	dbginfo2(DBG_L1, "Knoten %d, Klasse %d:\n", node, k);
	
	for (i=1; i<=AnzKnoten; i++)
	{
	    if(klasse[k].uew[node][i] > 0.0)
	    {
		klasse[k].c[node][i] = 1.0 + klasse[k].uew[node][i] * 
		    (knoten[node].c2D - 1.0);
	    }
	    dbginfo(DBG_L1, "\t%4.4f", klasse[k].c[node][i]);
	}
	dbgtext(DBG_L1, "\n");
    }
}



/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: chylla_CheckVerfahren()
**
**	Funktionstyp: int *
**
**	Uebergabeparameter: keine
**
**	Beschriebung:
**  Einlesen des Netzwerkes und Ueberpruefung, ob Verfahren
**  auf Netz anwendbar ist.
**
************************************************************
***********************************************************/

int *chylla_CheckVerfahren(argc, argv)
int argc;
char **argv;
{
    int i, check, *p;

    Einlesen(stdin);

    if (NetzTyp != 2) {
	fprintf(stderr, "falscher Netztyp\n");
	printf("0 0 0\n");
	exit(-1);
    }

    for (i = 1; i <= AnzKnoten; i++) {
	if (!(check = CheckKnoten(knoten[i].KnotenTyp, CHYLLA_KN_TYPEN)))
	{
	    /* falscher Knotentyp */
	    /* Verfahren nicht anwendbar */
	    fprintf(stderr, "falscher Knotentyp\n");
	    printf("0 0 0\n");
	    exit(-1);
	}
    }
	
    p = (int *)malloc(2*sizeof(int));

    p[0] = verfahren_nr;
    p[1] = -1;

    AnzVer = SINGLE;

    return (p);

}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: chylla_InitNetz()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschriebung:
**  Berechnung aller direkt ableitbarer Netzgroessen.
**
************************************************************
***********************************************************/

void chylla_InitNetz()
{
    BerAnkRaten();
    BerBedRaten_Kn();
    CheckStab();
    BerUebergangsRaten();
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: chylla_InitIteration()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschriebung:
**  Initialisierung der Var.koeff. mit den Startwerten
**
************************************************************
***********************************************************/

void chylla_InitIteration()
{
    InitVarKoeff();
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: chylla_Iterationsschritt()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int it
**
**	Beschriebung:
**  Iterationsschritte, die bei jedem Schleifendurchlauf 
**  zur Berechnung der quad. Var.koeff. ausgefuehrt werden.
**
************************************************************
***********************************************************/

void chylla_Iterationsschritt(it)
int it;
{
    int node;

    dbginfo(25, "\n\n%d. Iterationsschritt\n\n", it);

    InitVarKoeff_alt();

    for (node=1; node<=AnzKnoten; node++)
    {
	chylla__berechne_c2A_pro_Knoten_u_Klasse(node);
	Ber_ca2_Kn(node);
	chylla__berechne_c2D_pro_Knoten(node);
	chylla__berechne_VarKoeffizienten(node);
    }
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: chylla_IterationsAbbruch()
**
**	Funktionstyp: int
**
**	Uebergabeparameter: int it (Anzahl der Iterationen)
**
**	Beschriebung:
**  Ueberpruefung des Abbruchkriteriums der Schleife zur 
**  approx. Berechnung der quad. Var.koeff..
**
************************************************************
***********************************************************/

int chylla_IterationsAbbruch(it)
int it;
{
    int ret;

    if (it > MAX_IT) {
	dbgtext(DBG_L7, "Abbruch der Iteration erfolgte wegen Ueberschreitung der max. Iterationsschritte\n");
	ret = 0;
    }
    else
    {
	if ((ret = CheckGenauigkeit()))
	{
	    dbgtext(DBG_L7, "Abbruch der Iteration erfolgte da Abbruchkriterium erfuellt\n");
	}
    }

    return(ret);
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: chylla_Leistung()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**
**	Beschriebung:
**  Berechnung der Leistungsgroessen in Abhaengigkeit der
**  quad. Var.koeff. und Ausgabe an den Postfilter.
**
************************************************************
***********************************************************/

void chylla_Leistung(nr)
int nr;
{

    (*(ForLei[formel_nr].BerLeist))();
    Ausgabe_Q(stdout, nr);

}

