/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/


static char *SccsId = "@(#)PEPSY:\teingabe.c\t1.2\t95/05/19 15:36:51";

#define ALLGEMEIN_C
#define EXTERN_KNOTENTYPEN

#include "def.h"

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Einlesen
**
**	Funktionstyp: void
**
**	Uebergabeparameter: FILE *fpin
**
**
**	Beschriebung:
**  Einlesen der Parameter des zu analysierenden Netzwerkes
**  Diese Funktion ist die vordefinierte Schnittstelle
**  zwischen den Programm und den vorgeschalteten Prefilter
**  'pref_dekomp'. (Diese Funktion liest die Daten zur Be-
**  schreibung eines Warteschlangennetzwerkes, welche der
**  Prefilter ueber eine Pipe zum Programm schickt in der ab-
**  gesprochenen Reihenfolge und mit den entsprechenden For-
**  maten ein. 
**
**
************************************************************
***********************************************************/

void Einlesen(fpin)
FILE *fpin;
{
    int i, j, k;

    fpin = stdin;


    dbgtext(DBG_EINGABE, "\tEinlesen der EINGABEDATEN:\n\n");

    /* Einlesen der Daten */
    /* Die Daten werden vom Prefilter pref_dekomp geschickt! */
    
    /* Einlesen des Netztypes und pruefen, ob es ein offenes Netz ist */
    
    fscanf(fpin, "%d", &NetzTyp);
    dbginfo(DBG_EINGABE, "Netz-Typ:\t%6d\n", NetzTyp);
    if (NetzTyp != 2){
	printf("kein offenes Netz\n");
    }

    /* Einlesen der Anzahl der Knoten */
    fscanf(fpin, "%d", &AnzKnoten);
    dbginfo(DBG_EINGABE, "Anzahl Knoten:\t%6d\n", AnzKnoten);

    /* Einlesen der Anzahl der Klassen */
    fscanf(fpin, "%d", &AnzKlassen);
    dbginfo(DBG_EINGABE, "Anzahl Klassen:\t%6d\n", AnzKlassen);

    /* Speicherplatz allocieren */

    klasse = (KLASSE *)calloc((AnzKlassen+1), sizeof(KLASSE));
    knoten = (KNOTEN_NETZ *)calloc((AnzKnoten+1), sizeof(KNOTEN_NETZ));
    for (i=0; i<AnzKlassen; i++){
	klasse[i].knoten = (KNOTEN_KLASSE *)calloc((AnzKnoten+1), sizeof(KNOTEN_KLASSE));
    }

    for(k = 0; k < AnzKlassen; k++){
	klasse[k].uew = (double **)calloc((AnzKnoten+1), sizeof(double *));
	klasse[k].rate = (double **)calloc((AnzKnoten+1), sizeof(double *));
	for(i = 0; i <= AnzKnoten; i++){
	    klasse[k].uew[i] = (double *)calloc((AnzKnoten+1), sizeof(double));
	    klasse[k].rate[i] = (double *)calloc((AnzKnoten+1), sizeof(double));
	}
    }


    for(k = 0; k < AnzKlassen; k++){
	klasse[k].uew_orig = (double **)calloc((AnzKnoten+1), sizeof(double *));
	for(i = 0; i <= AnzKnoten; i++){
	    klasse[k].uew_orig[i] = (double *)calloc((AnzKnoten+1), sizeof(double));
	}
    }


    for(k = 0; k < AnzKlassen; k++){
	klasse[k].c = (double **)calloc((AnzKnoten+1), sizeof(double *));
	for(i = 0; i <= AnzKnoten; i++){
	    klasse[k].c[i] = (double *)calloc((AnzKnoten+1), sizeof(double));
	}
    }


    for(k = 0; k < AnzKlassen; k++){
	klasse[k].c2 = (double **)calloc((AnzKnoten+1), sizeof(double *));
	for(i = 0; i <= AnzKnoten; i++){
	    klasse[k].c2[i] = (double *)calloc((AnzKnoten+1), sizeof(double));
	}
    }


    for(k = 0; k < AnzKlassen; k++){
	klasse[k].c_alt = (double **)calloc((AnzKnoten+1), sizeof(double *));
	for(i = 0; i <= AnzKnoten; i++){
	    klasse[k].c_alt[i] = (double *)calloc((AnzKnoten+1), sizeof(double));
	}
    }


    /* Einlesen der Knotentypen */
    for (j=1; j<=AnzKnoten; j++){
	fscanf(fpin, "%d", &(knoten[j].KnotenTyp));
	dbginfo(DBG_EINGABE, "\t%4d", knoten[j].KnotenTyp);
    }
    dbgtext(DBG_EINGABE, "\n");



    /* Einlesen der Knotentypen und der dazugehoerigen Anzahl von Servern */
    dbgtext(DBG_EINGABE, "Anzahl der Server\n");
    for (j=1; j<=AnzKnoten; j++){
	fscanf(fpin, "%d", &(knoten[j].AnzServer));
	dbginfo(DBG_EINGABE, "\t%4d", knoten[j].AnzServer);
    }
    dbgtext(DBG_EINGABE, "\n");

    dbgtext(DBG_EINGABE, "Ankunftsrate von aussen\n");
    for (i = 0; i < AnzKlassen; i++){
	/*Einlesen der Ankunftsrate von aussen pro Klasse*/
	fscanf(fpin, "%lf", &(klasse[i].lambda));
	dbginfo(DBG_EINGABE, "\t%6.2f", klasse[i].lambda);
    }
    dbgtext(DBG_EINGABE, "\n");

    dbgtext(DBG_EINGABE, "quad. Varkoeff. Ankunftsprozess\n");
    for(k=0; k < AnzKlassen; k++){
	dbginfo(DBG_EINGABE, "KLASSE %d\n", k);
	for (j=1; j<=AnzKnoten; j++){
	    /*Einlesen des quad. Var.koeff. des Ankunftsprozesses pro Klasse*/
	    fscanf(fpin, "%lf", &(klasse[k].knoten[j].c2A));
	    dbginfo(DBG_EINGABE, "\t%6.2f", klasse[k].knoten[j].c2A);
	    klasse[k].c2[0][j] = klasse[k].knoten[j].c2A;
	}
	dbgtext(DBG_EINGABE, "\n");
    }
    dbgtext(DBG_EINGABE, "\n");

    dbgtext(DBG_EINGABE, "Bedienrate pro Klasse und Knoten\n");
    for (i=0; i<AnzKlassen; i++){
	dbginfo(DBG_EINGABE, "KLASSE %d\n", i);
	for (j=1; j<=AnzKnoten; j++){
	    /*Einlesen der Bedienrate pro Klasse*/
	    fscanf(fpin, "%lf", &(klasse[i].knoten[j].mue));
	    dbginfo(DBG_EINGABE, "\t%6.2f", klasse[i].knoten[j].mue);
	}
	dbgtext(DBG_EINGABE, "\n");
    }
    dbgtext(DBG_EINGABE, "\n");

    dbgtext(DBG_EINGABE, "quad. Varkoeff. Bedienprozess\n");
    for (i=0; i<AnzKlassen; i++){
	dbginfo(DBG_EINGABE, "KLASSE %d\n", i);
	for (j=1; j<=AnzKnoten; j++){
	    /*Einlesen des quad. Var.koeff. des Bedienprozesses pro Klasse*/
	    fscanf(fpin, "%lf", &(klasse[i].knoten[j].c2B));
	    dbginfo(DBG_EINGABE, "\t%6.2f", klasse[i].knoten[j].c2B);
	}
	dbgtext(DBG_EINGABE, "\n");
    }
    dbgtext(DBG_EINGABE, "\n");


    dbgtext(DBG_EINGABE, "Uebergangswahrscheinlichkeiten pro Klasse\n");
    /* Einlesen der Uebergangswahrscheinlichkeiten pro Klasse */
    for (i=0; i<AnzKlassen; i++){
	dbginfo(DBG_EINGABE, "KLASSE %d\n", i);
	for (j=0; j<=AnzKnoten; j++){
	    for (k=0; k<=AnzKnoten; k++){
		fscanf(fpin, "%lf", &(klasse[i].uew[j][k]));
		dbginfo(DBG_EINGABE, "\t%6.2f", klasse[i].uew[j][k]);
	    }
	    dbgtext(DBG_EINGABE, "\n");
	}
	dbgtext(DBG_EINGABE, "\n");
    }
    dbgtext(DBG_EINGABE, "\n");



    /* Einlesen der Abbruchbedingung */
    fscanf(fpin, "%lf", &epsilon);


    dbgtext(DBG_EINGABE, "Besuchshaeufigkeit pro Klasse\n");
    for (i=0; i<AnzKlassen; i++){
	dbginfo(DBG_EINGABE, "KLASSE %d\n", i);
	for (j=1; j<=AnzKnoten; j++){
	    /*Einlesen der Besuchshaeufigkeit pro Klasse*/
	    fscanf(fpin, "%lf", &(klasse[i].knoten[j].e));
	    dbginfo(DBG_EINGABE, "\t%6.2f", klasse[i].knoten[j].e);
	}
	dbgtext(DBG_EINGABE, "\n");
    }
    dbgtext(DBG_EINGABE, "\n");
    dbgtext(DBG_EINGABE, "\n");

    dbgtext(DBG_EINGABE, "\tEnde der EINGABE\n\n");
}
