/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/


/***********************************************************
************************************************************
**
**  Modulname: help.c
**
**	Beschreibung:
**  Hilfsfunktionen die in allen Modulen aufgerufen werden
**  koennen
**
***********************************************************
***********************************************************/
static char *SccsId = "@(#)PEPSY:\thelp.c\t1.2\t95/05/19 15:36:56";

#define HELP_C

#include "def.h"

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Fak()
**
**	Funktionstyp: int
**
**	Uebergabeparameter: int x
**
**	Beschreibung:
**  berechnet die Fakultaet von x
**
************************************************************
***********************************************************/

int Fak(x)
int x;
{
	return((x)?(x*Fak(x-1)):1);
} 

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: BerGRho()
**
**	Funktionstyp: double
**
**	Uebergabeparameter: int node (Nummer des Knoten)
**
**	Beschreibung:
**  berechnet fuer Knoten node die Gesamtauslastung
**
************************************************************
***********************************************************/

double BerGRho(node)
int node;
{
	KNOTEN_NETZ *akt_n;
	double GRho;

	akt_n = &(knoten[node]);

	if(akt_n->KnotenTyp == TYP4) {
		GRho = 0.0;
	}
	else {
		GRho = akt_n->lambda / akt_n->mue;
	}
	
	return(GRho);	
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: BerRho()
**
**	Funktionstyp: double
**
**	Uebergabeparameter:
**    int node (Nummer des Knoten)
**    int class (Nummer der Klasse)
**
**	Beschreibung:
**  berechnet fuer Knoten node und Klasse class die Auslastung
**
************************************************************
***********************************************************/

double BerRho(node, class)
int node;
int class;
{
	KNOTEN_KLASSE *akt_n_c;
	KNOTEN_NETZ *akt_n;
	double Rho;

	akt_n_c = &(klasse[class].knoten[node]);
	akt_n = &(knoten[node]);

	if(akt_n->KnotenTyp == TYP4) {
		Rho = 0.0;
	}
	else {
		Rho = akt_n_c->lambda /(akt_n_c->mue * akt_n->AnzServer);
	}
	
	return(Rho);	

}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: BerRate()
**
**	Funktionstyp: double
**
**	Uebergabeparameter:
**    int class (Nummer der Klasse)
**    int n1 (Nummer des aktuellen Knoten)
**    int n1 (Nummer des Nachfolge-Knoten)
**
**	Beschreibung:
**  berechnet fuer Klasse class die Uebergangsrate von
**  Knoten n1 zu n2
**
************************************************************
***********************************************************/

double BerRate(class, n1, n2)
int class;
int n1;
int n2;
{
	KLASSE *akt_c;
	KNOTEN_NETZ *akt_n;	
	double rate;

	akt_c = &(klasse[class]);
	akt_n = &(knoten[n1]);

	if (n1 == 0) {
		rate = akt_c->lambda * akt_c->uew[n1][n2];
	}
	else {
		rate = akt_n->lambda * akt_c->uew[n1][n2];
	}

	return(rate);

}


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Pm()
**
**	Funktionstyp: double
**
**	Uebergabeparameter:
**    double rho (Auslastung)
**    int    m   (Anzahl der Server)
**
**	Beschreibung:
**  Wahrscheinlichkeit, dass ein ankommender Auftrag nicht
**  bedient werden kann und in der Warteschlange warten muss
**
************************************************************
***********************************************************/

double Pm(rho, m)  
double rho;
int    m;
/*
   Berechnung von Pm fuer MMm-Systeme
*/
{
   double p0;
   int    k;

   p0 = 0;

   for (k=0; k<m; k++)
   {
      p0 += pow((double)m*rho, (double)k) / Fak(k);
   }
   p0 += pow((double)m*rho, (double)m) / Fak(m) / (1.0-rho);
   p0 = 1.0 / p0;

   return ( pow((double)m*rho, (double)m) / Fak(m) / (1.0-rho) * p0 );
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Pm_iterativ()
**
**	Funktionstyp: double
**
**	Uebergabeparameter:
**    double rho (Auslastung)
**    int    m   (Anzahl der Server)
**
**	Beschreibung:
**  Wahrscheinlichkeit, dass ein ankommender Auftrag nicht
**  bedient werden kann und in der Warteschlange warten muss
**  Berechnung von Pm fuer MMm-Systeme (iterativ)
**
************************************************************
***********************************************************/

double Pm_iterativ(rho, m) 
   double rho;
   int    m;
{
   double p0, pp;
   int    k, j;

   if (m==1)
      return(rho);

   p0 = 1.0 + (double)m*rho;
   for (k=2; k<m; k++)
   {
      pp = m*rho;
      for (j=2; j<=k; j++)
      {
        pp *= m*rho/j;
       }
       p0 += pp;
   }
   p0 += pow((double)m*rho, (double)m) / Fak(m) / (1.0-rho);
   p0 = 1.0 / p0;

   return ( pow((double)m*rho, (double)m) / Fak(m) / (1.0-rho) * p0 );
}


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: vertausche(x, y)
**
**	Funktionstyp: void
**
**	Uebergabeparameter: double *x
**                      double *y
**
**	Beschreibung:
**  vertauscht die beiden Werte der Zahlen x und y
**
************************************************************
***********************************************************/

void vertausche(x, y)
double *x;
double *y;
{
	double h;

	h = *x;
	*x = *y;
	*y = h;

}

