/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/


/***********************************************************
************************************************************
**
**  Modulname: init.h
**
**  Beschreibung:
**  In diesen Modul werden die implementierten Dekomposi-
**  tionsmethoden und Formeln zur Berechnung der Leistungs-
**  groessen in das Programm integriert.
**  Bei der Integration neuer Dekomp.-Verfahren oder Formeln
**  muessen diese in die Struktur 'func_struct_t' bzw.
**  'leistung_s_t' eingetragen und initialisiert werden.
**  Zudem muessen die Konstanten 'ANZ_VERFAHREN' (entspricht
**  der Anzahl der impl. Dekomp.-Methoden) bzw 'ANZ_FORMELN
**  _LEISTUNG' (entspricht der Anz. der impl. Formeln) inkre-
**  mentiert werden.
**
***********************************************************
***********************************************************/
static char *SccsInit_h = "@(#)PEPSY:\tinit.h\t1.2\t95/05/19 15:37:02";

#include "def.h"

#define ANZ_VERFAHREN 6
#define ANZ_FORMELN_LEISTUNG 7

#ifdef MAIN_C

int abbruch = 0;
int iterationen = 0;

func_struct_t verfahren[ANZ_VERFAHREN] =
{	
	{
		"best\0           ",
		"auto\0           ",
		00000000,
		0,
		auto_best_CheckVerfahren,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		"all\0            ",
		"auto\0           ",
		00000000,
		0,
		auto_all_CheckVerfahren,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		/* Dekompositionsmethode nach Chylla */
		"chylla\0         ",
		"a_c\0            ",
		00000777,
		0,
		chylla_CheckVerfahren,
		chylla_InitNetz,
		chylla_InitIteration,
		chylla_Iterationsschritt,
		chylla_IterationsAbbruch,
		chylla_Leistung
	},
	{
		/* Dekompositionsmethode nach Pujolle */
		"pujolle\0        ",
		"a_c\0            ",
		00000777,
		0,
		puj_CheckVerfahren,
		puj_InitNetz,
		puj_InitIteration,
		puj_Iterationsschritt,
		puj_IterationsAbbruch,
		puj_Leistung
	},
	{
		/* Dekompositionsmethode nach Whitt */
		"whitt\0          ",
		"a_c\0            ",
		00000777,
		0,
		whi_CheckVerfahren,
		whi_InitNetz,
		whi_InitIteration,
		whi_Iterationsschritt,
		whi_IterationsAbbruch,
		whi_Leistung
	},
	{
		/* Dekompositionsmethode nach Kuehn */
		"kuehn\0          ",
		"klb\0            ",
		00000121,
		1,
		kuehn_CheckVerfahren,
		kuehn_InitNetz,
		kuehn_InitIteration,
		kuehn_Iterationsschritt,
		kuehn_IterationsAbbruch,
		kuehn_Leistung
	}
};


leistung_s_t ForLei[ANZ_FORMELN_LEISTUNG] = {
	{
		/* Formel zur Leistungsgroessenberechnung nach Allen/Cunneen */
		"a_c",
		AC_BerLei
	},
	{
		/* Formel zur Leistungsgroessenberechnung nach Kraemer/Langenbach-Belz */
		"klb",
		KLB_BerLei
	},
	{
		/* Formel zur Leistungsgroessenberechnung nach Page */
		"pag",
		PAG_BerLei
	},
	{
		/* Formel zur Leistungsgroessenberechnung nach Cosmetatos */
		"cos",
		COS_BerLei
	},
	{
		/* Formel zur Leistungsgroessenberechnung nach Kimura */
		"kim",
		KIM_BerLei
	},
	{
		/* Formel zur Leistungsgroessenberechnung nach Marchal */
		"mar",
		MAR_BerLei
	},
	{
		/* Formel zur Leistungsgroessenberechnung nach der
		   automatischen Auswahl, wie sie in der DA beschrieben wird */
		"auto",
		AUTO_BerLei
	}

};
#else
	extern func_struct_t verfahren[ANZ_VERFAHREN];
	extern leistung_s_t ForLei[ANZ_FORMELN_LEISTUNG];
#endif
