/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/


/***********************************************************
************************************************************
**
**  Modulname: kuehn.c
**
**	Beschreibung:
**  
**
***********************************************************
***********************************************************/

static char *SccsId = "@(#)PEPSY:\tkuehn.c\t1.2\t95/05/19 15:37:05";

#define KUEHN_C
#define EXTERN_KNOTENTYPEN


#include "def.h"

#define KUEHN_KN_TYPEN 00000121

extern leistung_s_t ForLei[];

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: berechne_I()
**
**	Funktionstyp: double
**
**	Uebergabeparameter: int c1, c2, l1, l2
**
**	Beschreibung:
**  
**
************************************************************
***********************************************************/

double berechne_I(c1, c2, l1, l2)
double c1;
double c2;
double l1;
double l2;
{
	double I1, I2, I3, I4;
	double t11, t12, t21, t22, t1, t2, q11, q12, q21, q22;
	double e, e1, e2, t_sum, t1_sum, t2_sum;

	dbgtext(DBG_FKT3, "\t\tFunktion kuehn__berechne_I() \n")

	t1 = 1.0/l1;
	t2 = 1.0/l2;

	dbgtext(DBG_L5, "\t\tUebergabeparameter:\n")
	dbginfo(DBG_L5, "\t\tc1 ist %4.4f", c1)
	 dbginfo(DBG_L5, "\tc2 ist %4.4f", c2)
	 dbginfo(DBG_L5, "\tl1 ist %4.4f", l1)
	 dbginfo(DBG_L5, "\tl2 ist %4.4f\n", l2)


	 if ((c1 < 1.0) && (c2 < 1.0)){
		 t11 = (1.0-c1)/l1;
		 t12 = c1/l1;
		 t21 = (1.0-c2)/l2;
		 t22 = c2/l2;

		 if (t11 > t21){
			 vertausche(&t11, &t21);
			 vertausche(&t12, &t22);
			 vertausche(&t1, &t2);
		 } /* end if */

		 e = exp((-1) * (t21 - t11)/t12);
		 t_sum = t12 * t21 + t12 * t22 + t21 * t22;

		 I1 = SQR(t11) * (t2/2.0 - t11/3.0 - t12) + 
		 t12 * t1 * (t2 - 2.0 * t12) + 
		 t12 * (t12 * (2.0 * t21 + 2.0 * t12 - t2) - t21 * t22) * e; 

		 I2 = (t12 * SQR(t22)/SQR((t12 + t22))) * 
		 t_sum *e;

		 I3 = t1 * SQR(t11)/2.0 - pow(t11, 3.0)/3.0; 

		 I4 = SQR(t12) * (t1 + ((-1) * t12 - t21 + (t22/SQR((t12 + t22)) * 
		 t_sum) ) * e);

	 } /* end if */

	 else if (((c1 >= 1.0) && (c2 < 1.0)) || ((c1 < 1.0) && (c2 >= 1.0))) {
		 if (c1 < 1.0) {
			 /*Werte von c1 und c2 vertauschen */
			 vertausche(&c1, &c2);
			 /*Werte von l1 und l2 vertauschen */
			 vertausche(&l1, &l2);
			 /*Werte von t1 und t2 vertauschen */
			 vertausche(&t1, &t2);
		 }

		 t11 = 1.0/l1 * 1.0/(1.0 + sqrt((SQR(c1) - 1.0)/(SQR(c1) + 1.0)));
		 t12 = 1.0/l1 * 1.0/(1.0 - sqrt((SQR(c1) - 1.0)/(SQR(c1) + 1.0)));

		 t21 = (1.0 -c2)/l2;
		 t22 = c2/l2;

		 q11 = 1.0/(2.0 * l1 * t11);
		 q12 = 1.0/(2.0 * l1 * t12);

		 e1 = exp((-1) * t21/t11);
		 e2 = exp((-1) * t21/t12);

		 t1_sum = t11 * t21 + t11 * t22 + t21 * t22;
		 t2_sum = t12 * t21 + t12 * t22 + t21 * t22;

		 I1 = q11 * SQR(t11) * t2 * (1.0 - (1.0 + t21/t11) * e1) + 
		 q12 * SQR(t12) * t2 * (1.0 - (1.0 + t21/t12) * e2) -
		 q11 * t11 * (2.0 * SQR(t11) - (2.0 * SQR(t11) + 
		 2.0 * t11 * t21 + SQR(t21)) * e1) -
		 q12 * t12 * (2.0 * SQR(t12) - (2.0 * SQR(t12) + 
		 2.0 * t12 * t21 + SQR(t21)) * e2); 

		 I2 = q11 * t11 * SQR(t22)/SQR((t11 + t22)) * t1_sum * e1 +
		 q12 * t12 * SQR(t22)/SQR((t12 + t22)) * t2_sum * e2;

		 I3 = q11 * pow(t11, 3.0) * (1.0 - (1.0 + t21/t11) * e1) + 
		 q12 * pow(t12, 3.0) *(1.0 - (1.0 + t21/t12) * e2);

		 I4 = q11 * SQR(t11) * t22/SQR((t11 + t22)) * t1_sum * e1 + 
		 q12 * SQR(t12) * t22/SQR((t12 + t22)) * t2_sum * e2;

	 } /* end else if */

	 else if ((c1 >= 1.0) && (c2 >= 1.0)) {
		 t11 = 1.0/l1 * 1.0/(1 + sqrt((SQR(c1) - 1.0)/(SQR(c1) + 1.0)));
		 t12 = 1.0/l1 * 1.0/(1 - sqrt((SQR(c1) - 1.0)/(SQR(c1) + 1.0)));
		 t21 = 1.0/l2 * 1.0/(1 + sqrt((SQR(c2) - 1.0)/(SQR(c2) + 1.0)));
		 t22 = 1.0/l2 * 1.0/(1 - sqrt((SQR(c2) - 1.0)/(SQR(c2) + 1.0)));

		 q11 = 1.0/(2.0 * l1 * t11);
		 q12 = 1.0/(2.0 * l1 * t12);
		 q21 = 1.0/(2.0 * l2 * t21);
		 q22 = 1.0/(2.0 * l2 * t22);

		 I1 = q11 * q21 * SQR(t11) * SQR(t21) / (t11 + t21);

		 I2 = q11 * q22 * SQR(t11) * SQR(t22) / (t11 + t22);

		 I3 = q12 * q21 * SQR(t12) * SQR(t21) / (t12 + t21);

		 I4 = q12 * q22 * SQR(t12) * SQR(t22) / (t12 + t22);

	 } /* end else if */

	 dbgtext(DBG_L5, "\t\tZwischenergebnisse:\n")
	 dbginfo(DBG_L5, "\t\tc1 ist %4.4f", c1)
	 dbginfo(DBG_L5, "\tc2 ist %4.4f\n", c2)
	 dbginfo(DBG_L5, "\t\tt1 ist %4.4f", t1)
	 dbginfo(DBG_L5, "\tt2 ist %4.4f\n", t2)
	 dbginfo(DBG_L5, "\t\tt11 ist %4.4f", t11)
	 dbginfo(DBG_L5, "\tt12 ist %4.4f", t12)
	 dbginfo(DBG_L5, "\tt21 ist %4.4f", t21)
	dbginfo(DBG_L5, "\tt22 ist %4.4f\n", t22)
	dbginfo(DBG_L5, "\t\tq11 ist %4.4f", q11)
	dbginfo(DBG_L5, "\tq12 ist %4.4f", q12)
	dbginfo(DBG_L5, "\tq21 ist %4.4f", q21)
	dbginfo(DBG_L5, "\tq22 ist %4.4f\n", q22)
	dbginfo(DBG_L5, "\t\tI1 ist %4.4f", I1)
	dbginfo(DBG_L5, "\tI2 ist %4.4f", I2)
	dbginfo(DBG_L5, "\tI3 ist %4.4f", I3)
	dbginfo(DBG_L5, "\tI4 ist %4.4f\n", I4)
	dbginfo(DBG_L5, "\t\tI ist %4.4f\n", I1 + I2 + I3 + I4)

	return(I1+I2+I3+I4);

} /* end berechne_I() */


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: kuehn__superposition()
**
**	Funktionstyp: double
**
**	Uebergabeparameter: int c1, c2
**                      int l1, l2
**
**	Beschreibung:
**
**
************************************************************
***********************************************************/

double kuehn__superposition(c1, c2, lambda1, lambda2)
double c1;
double c2;
double lambda1;
double lambda2;
{
	double t1, t2, r;

	double berechne_I();

	dbgtext(DBG_FKT2, "\tFunktion kuehn__superposition()\n")

	if(lambda1 == 0.0)
		return(c2);
	if(lambda2 == 0.0)
		return(c1);

	t1 = 1.0/lambda1;
	t2 = 1.0/lambda2;

	r = 2.0 * (t1 + t2)/SQR((t1 * t2)) * 
	berechne_I(c1, c2, lambda1, lambda2) - 1.0;

	dbginfo(DBG_L2, "\tErgebnis der Superposition ist %4.4f\n", sqrt(r))

	return(sqrt(r));


} /* end kuehn__superposition() */


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: kuehn__rekonfiguriere_netz()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine 
**
**	Beschreibung:
**  Rekonfiguration des Netzes: Alle direkten Rueckkopplungen
**  werden eliminiert; Umstrukturierung der Netzwerktopologie.
**
************************************************************
***********************************************************/

void kuehn__rekonfiguriere_netz() 
{
	double *mue_neu;
	double *c2B_neu;
	double **p;
	int i, j, k;

	/* Speicherplatz allokieren */

	dbgtext(DBG_FKT1, "\nFunktion kuehn__rekonfiguriere_netz()\n")

	mue_neu = (double *)calloc(AnzKnoten+1, sizeof(double));

	c2B_neu = (double *)calloc(AnzKnoten+1, sizeof(double));

	p = (double **)calloc(AnzKnoten+1, sizeof(double *));

	for (i=0; i<=AnzKnoten; i++) {

		p[i] = (double *)calloc(AnzKnoten+1, sizeof(double));

	} /* end for i */

	for (k = 0; k < AnzKlassen; k++) {
		for (i = 1; i <= AnzKnoten; i++) {
			if (klasse[k].uew[i][i] > 0.0){
				mue_neu[i] = klasse[k].knoten[i].mue * (1.0 - klasse[k].uew[i][i]);
				c2B_neu[i] = klasse[k].uew[i][i] + klasse[k].knoten[i].c2B * (1.0 - klasse[k].uew[i][i]);
				for (j = 1; j <= AnzKnoten; j++) {
					if (i == j)
						p[i][j] = 0.0;
					else
						p[i][j] = klasse[k].uew[i][j]/(1.0 - klasse[k].uew[i][i]);

				} /* end for j */
			} /* end if */

		} /* end for i */

	} /* end for k */

	for (k = 0; k < AnzKlassen; k++) {
		for (i = 1; i <= AnzKnoten; i++) {

			klasse[k].knoten[i].mue_orig = klasse[k].knoten[i].mue;
			klasse[k].knoten[i].c2B_orig = klasse[k].knoten[i].c2B;
			klasse[k].knoten[i].lambda_orig = klasse[k].knoten[i].lambda;

			for (j = 1; j <= AnzKnoten; j++) {

				klasse[k].uew_orig[i][j] = klasse[k].uew[i][j];

			} /* end for j */

		} /* end for i */

	} /* end for k */

	for (k = 0; k < AnzKlassen; k++) {
		for (i = 1; i <= AnzKnoten; i++) {
			if (klasse[k].uew[i][i] > 0.0){
				klasse[k].knoten[i].mue = mue_neu[i];
				klasse[k].knoten[i].c2B = c2B_neu[i];
				for (j = 1; j <= AnzKnoten; j++) {
					klasse[k].uew[i][j] = p[i][j];

				} /* end for j */
			} /* end if */

		} /* end for i */

	} /* end for k */



	dbgtext(DBG_L5, "Neue Daten fuer rekonfiguriertes Netz\n")
	for (k = 0; k < AnzKlassen; k++) {
		dbginfo(DBG_L5, "Bedienraten Klasse %d:\n", k)
		for (i = 1; i <= AnzKnoten; i++) {
			dbginfo(DBG_L5, "\t%4.4f", klasse[k].knoten[i].mue)
		}
		dbgtext(DBG_L5, "\n")
		dbginfo(DBG_L5, "Quad. Varkoeff. Bedienprozess Klasse %d:\n", k)
		for (i = 1; i <= AnzKnoten; i++) {
			dbginfo(DBG_L5, "\t%4.4f", klasse[k].knoten[i].c2B)
		}
		dbgtext(DBG_L5, "\n")
		dbginfo(DBG_L5, "Uebergangswahr. Klasse %d:\n", k)
		for (i = 0; i <= AnzKnoten; i++) {
			for (j = 0; j <= AnzKnoten; j++) {
				dbginfo(DBG_L5, "\t%4.4f", klasse[k].uew[i][j])
			}
			dbgtext(DBG_L5, "\n")
		}
		dbgtext(DBG_L5, "\n")
	}


} /* end kuehn__rekonfiguriere_netz() */


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: kuehn__berechne_c2A()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int node 
**
**
**	Beschreibung:
**  Berechnet fuer alle Knoten den quad. Var.koeff. des 
**  Ank.prozesses.
**
************************************************************
***********************************************************/

void kuehn__berechne_c2A(node)
int node;
{
	int i;
	double c_1, c_2, l1, l2;

	dbgtext(DBG_FKT1, "\nFunktion kuehn__berechne_c2A()\n")

	c_1 = 0.0;
	c_2 = 0.0;

	i = 0;

	while ((klasse[0].c[i][node] <= 0.0) && (i <= AnzKnoten)){
		i++;
	}
	if (i > AnzKnoten){
		printf("Fatal Error!\n");
		exit(1);
	}

	c_1 = sqrt(klasse[0].c[i][node]);
	if (i > 0)
		l1 = klasse[0].knoten[i].lambda;
	else
		l1 = klasse[0].lambda * klasse[0].uew[0][node];

	dbginfo(DBG_L5, "c1 ist %f", c_1)
	dbginfo(DBG_L5, "\tl1 ist %f\n", l1)



	i++;

	while(i <= AnzKnoten){
		if(klasse[0].c[i][node] > 0.0){
			c_2 = sqrt(klasse[0].c[i][node]);

			l2 = klasse[0].knoten[i].lambda;

			dbginfo(DBG_L5, "c2 ist %f", c_2)
			dbginfo(DBG_L5, "\tl2 ist %f\n", l2)


			c_1 = kuehn__superposition(c_1, c_2, l1, l2);

			l1 = l1 + l2;

		dbginfo(DBG_L5, "c1 ist %f", c_1)
		dbginfo(DBG_L5, "\tl1 ist %f\n", l1)

		}
		i++;
	}

	klasse[0].knoten[node].c2A = pow(c_1, 2.0);	

	dbginfo(DBG_L1, "klasse[0].knoten[%d].c2A ist ", node)
	dbginfo(DBG_L1, "%f\n", klasse[0].knoten[node].c2A)

}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: kuehn__berechne_c2D()
**
**	Funktionstyp: void 
**
**	Uebergabeparameter: int node 
**
**
**	Beschreibung:
**  Berechnet fuer Knoten node den quad. Var.koeff. des 
**  Abgangsprozesses.
**
************************************************************
***********************************************************/

void kuehn__berechne_c2D(node)
int node;
{
	double rho, c2A, c2B;

	double kuehn__berechne_g();

	dbgtext(DBG_FKT1, "\nFunktion kuehn__berechne_c2D()\n")

	rho = klasse[0].knoten[node].rho;
	c2A = klasse[0].knoten[node].c2A;
	c2B = klasse[0].knoten[node].c2B;

	klasse[0].knoten[node].c2D = c2A + (2.0 * SQR(rho) * c2B) - 
		((SQR(rho) * (c2A + c2B)) * kuehn__berechne_g(rho, c2A, c2B));

	dbginfo(DBG_L1, "klasse[0].knoten[%d].c2D ist ", node)
	dbginfo(DBG_L1, "%f\n", klasse[0].knoten[node].c2D)


	/*return(OK);*/
	
} /* end kuehn__berechne_c2D() */


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: kuehn__berechne_VarKoeffizienten()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int node 
**
**
**	Beschreibung:
**  Berechnet fuer Knoten node den quad. Var.koeff. fuer die
**  einzelnen Teilprozesse zu den verschiedenen Knoten nach
**  der Aufsplittung des Abgangsprozesses.
**
************************************************************
***********************************************************/

void kuehn__berechne_VarKoeffizienten( node)
int node;
{
	int i;

	dbgtext(DBG_FKT1, "\nFunktion kuehn__berechne_VarKoeffizienten()\n")

	dbginfo(DBG_L1, "Varkoeffizienten der Abgangsstroeme von Knoten %d:\n", node)
	for (i=1; i<=AnzKnoten; i++){ 
		if(klasse[0].c[node][i] > 0.0){
			klasse[0].c[node][i] = 
				klasse[0].uew[node][i] * klasse[0].knoten[node].c2D + 
				(1.0 - klasse[0].uew[node][i]);
		}

		dbginfo(DBG_L1, "\t%4.4f", klasse[0].c[node][i])
	}
	dbgtext(DBG_L1, "\n")

	/*return(OK);*/
	
} /* end kuehn__berechne_VarKoeffizienten()*/

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: kuehn__berechne_g()
**
**	Funktionstyp: double
**
**	Uebergabeparameter: double rho
**                      double c2A
**                      double c2B
**
**	Beschreibung:
**  Hilfsfunktion fuer die Funktion kuehn__berechne_c2D()
**  zur Berechnung der Abgangsstroeme.
**
************************************************************
***********************************************************/

double kuehn__berechne_g(rho, c2A, c2B)
double rho;
double c2A;
double c2B;
{
	double ret_wert;

	if (c2A < 1.0) {
		ret_wert = exp(-2 * (1.0 - rho) * SQR((1.0 - c2A)) * 
							(1.0/(3.0 * rho * (c2A + c2B))));

	}
	else {
		ret_wert = exp(-(1.0 - rho) * (c2A - 1.0) *
							(1.0/(c2A + (4.0 * c2B))));
	}

	return (ret_wert);

} /* end kuehn__berechne_g() */

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: kuehn_CheckVerfahren()
**
**	Funktionstyp: int *
**
**	Uebergabeparameter: 
**
**	Beschreibung:
**  Einlesen des Netzwerkes und Ueberpruefung, ob Verfahren
**  auf Netz anwendbar ist.
**
************************************************************
***********************************************************/

int *kuehn_CheckVerfahren(argc, argv)
int argc;
char **argv;
{
	int i, check, *p;

	Einlesen(stdin);

	if (NetzTyp != 2) {
		fprintf(stderr, "falscher Netztyp\n");
		printf("0 0 0\n");
		exit(-1);
	}

	if (AnzKlassen > 1) {
		fprintf(stderr, "mehr als eine Klasse\n");
		printf("0 0 0\n");
		exit(-1);
	}
	
	for (i = 1; i <= AnzKnoten; i++) {
		if (!(check = CheckKnoten(knoten[i].KnotenTyp, KUEHN_KN_TYPEN))) {
			fprintf(stderr, "falscher Netztyp\n");
			printf("0 0 0\n");
			/* Verfahren nicht anwendbar */
			exit(-1);
		}
	}

	p = (int *)malloc(2*sizeof(int));

	p[0] = verfahren_nr;

	AnzVer = SINGLE;

	p[1] = -1;
	return (p);

}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: kuehn_InitNetz()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Berechnung aller direkt ableitbarer Netzgroessen
**
************************************************************
***********************************************************/

void kuehn_InitNetz()
{
	BerAnkRaten();
	BerBedRaten_Kn();
	CheckStab();
	kuehn__rekonfiguriere_netz();
	BerAnkRaten();
	BerBedRaten_Kn();
	CheckStab();
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: kuehn_InitIteration()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Initialisierung der Var.koeff. mit den Startwerten
**
************************************************************
***********************************************************/

void kuehn_InitIteration()
{

	InitVarKoeff();

}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: kuehn_Iterationsschritt()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int it
**
**	Beschreibung:
**  Iterationsschritte, die bei jedem Schleifendurchlauf 
**  zur Berechnung der quad. Var.koeff. ausgefuehrt werden.
**
************************************************************
***********************************************************/

void kuehn_Iterationsschritt(it)
int it;
{
	int node;

	dbginfo(25, "\n\n%d. Iterationsschritt\n\n", it)

	InitVarKoeff_alt();

	for (node=1; node<=AnzKnoten; node++) {
		kuehn__berechne_c2A(node);
		kuehn__berechne_c2D(node);
		kuehn__berechne_VarKoeffizienten(node);
	} /* end for node */



}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: kuehn_IterationsAbbruch()
**
**	Funktionstyp: int
**
**	Uebergabeparameter: int it
**
**	Beschreibung:
**  Ueberpruefung des Abbruchkriteriums der Schleife zur 
**  approx. Berechnung der quad. Var.koeff..
**
************************************************************
***********************************************************/

int kuehn_IterationsAbbruch(it)
int it;
{
	int ret;

	if (it > MAX_IT) {
		dbgtext(DBG_L7, "Abbruch der Iteration erfolgte wegen Ueberschreitung der max. Iterationsschritte\n")
		ret = 0;
	}
	else {
		dbgtext(DBG_L7, "Abbruch der Iteration erfolgte da Abbruchkriterium erfuellt\n")
		ret = CheckGenauigkeit();
	}

	return(ret);

}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: kuehn_Leistung()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Macht zuerst die Rekonfiguration des Netzes rueckgaengig.
**  Berechnung der Leistungsgroessen in Abhaengigkeit der
**  quad. Var.koeff. und Ausgabe an den Postfilter.
**
************************************************************
***********************************************************/

void kuehn_Leistung(nr)
int nr;
{
	int i, j, k;
	KNOTEN_KLASSE *akt_node;


	/* Berechnung der Leistungsgroessen */

  /* Ermittlung der Formel zur Ber. d. Leistungsgroessen */

  /* Falls Option -f gesetz, nimm angegebene Formel */

  /* Falls Option -f nicht gesetzt, oder ungueltiger Wert angegeben,
     nimm den Default-Wert 'DefaultFormel' fuer das Verfahren Kuehn
     aus der Struktur 'verfahren' */

  

	(*(ForLei[formel_nr].BerLeist))();

	/* Netzrekonfiguration rueckgaengig machen */

	for (k = 0; k < AnzKlassen; k++) {
		for (i = 1; i <= AnzKnoten; i++) {
			akt_node = &(klasse[k].knoten[i]);
			akt_node->mue = akt_node->mue_orig;
			akt_node->lambda = akt_node->lambda_orig;
			for (j = 1; j <= AnzKnoten; j++) {
				klasse[k].uew[i][j] = klasse[k].uew_orig[i][j];
			}
		}
	}

	for (i = 1; i <= AnzKnoten; i++) {
		akt_node = &(klasse[0].knoten[i]);
		knoten[i].c2A = akt_node->c2A;
		knoten[i].c2B = akt_node->c2B_orig;
	}


	/* Leistungsgroessen an den Postfilter schicken */

	Ausgabe_Q(stdout, nr);

}


