/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/


/***********************************************************
************************************************************
**
**  Modulname: leistung.c
**
**	Beschreibung:
**  In diesen Modul sind alle Funktionen zur Berechnung der
**  Leistungsgroessen in Abhaengigkeit des quad. Var.koeff.
**  des Ankunftsprozesses untergebracht. 
**  
***********************************************************
***********************************************************/

static char *SccsId = "@(#)PEPSY:\tleistung.c\t1.1\t94/04/21 14:24:14";

#define LEISTUNG_C

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "def.h"
#include "lut_k.h"

#define MAX(a,b) ((a)>(b))?(a):(b) 


double Q_MMm(),Q_MDm(), Q_GDm(), Q_GMm();
double gkl(), gpa(), Pm_iterativ(), Pm();
double C(), f_Kulbatzki(), h_Kulbatzki();
void   BerechneLeistung();
LookUpTab_CB *SelectFuncMwsl();


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Q_MMm
**
**	Funktionstyp: double
**
**	Uebergabeparameter: int n (Anzahl der Knoten)
**                      int c (Anzahl der Klassen)
**
**	Beschreibung:
**    berechnet die mittlere Warteschlangenlaenge nach der
**    Formel fuer MMm-FCFS Knoten
**
**
************************************************************
***********************************************************/

double Q_MMm(n, c)
int n, c;
{

    KNOTEN_KLASSE *akt_n_c;
    KNOTEN_NETZ *akt_n;

    akt_n_c = &(klasse[c].knoten[n]);
    akt_n = &(knoten[n]);

    return( Pm_iterativ(akt_n->rho, akt_n->AnzServer) * akt_n_c->rho / (1.0 - akt_n->rho) );

}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Q_Allen_Cunneen
**
**	Funktionstyp: double
**
**	Uebergabeparameter: int n (Anzahl der Knoten)
**                      int c (Anzahl der Klassen)
**
**	Beschreibung:
**    berechnet die mittlere Warteschlangenlaenge fuer 
**    G/G/m-FCFS Knoten nach der Formel von Allen/Cunneen
**
************************************************************
***********************************************************/

double Q_Allen_Cunneen(n, c)
int n, c;
{
    KNOTEN_KLASSE *akt_n_c;
    akt_n_c = &(klasse[c].knoten[n]);

    return( Q_MMm(n, c) * (akt_n_c->c2A + akt_n_c->c2B) / 2.0 );
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Q_KramerLangen
**
**	Funktionstyp: double
**
**	Uebergabeparameter: int n (Anzahl der Knoten)
**                      int c (Anzahl der Klassen)
**
**	Beschreibung:
**    berechnet die mittlere Warteschlangenlaenge fuer 
**    G/G/m-FCFS Knoten nach der Formel von Kramer und
**    Langenbach-Belz
**
************************************************************
***********************************************************/

double Q_KramerLangen(n, c)
int n, c;
{

    KNOTEN_KLASSE *akt_n_c;
    KNOTEN_NETZ *akt_n;


    akt_n_c = &(klasse[c].knoten[n]);
    akt_n = &(knoten[n]);

    return ( Q_MMm(n, c) * (akt_n_c->c2A + akt_n_c->c2B)/2.0 * gkl(akt_n->rho, akt_n->AnzServer, akt_n_c->c2A, akt_n_c->c2B) );


}


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: gkl()
**
**	Funktionstyp: double
**
**	Uebergabeparameter:
**       int  m     (Anzahl der Server)
**       double rho (Auslastung)
**       double ca2 (quad. Var.koeff. d. Ank.prozesses)
**       double cb2 (quad. Var.koeff. d. Bed.prozesses)
**
**	Beschreibung:
**  Korrekturfaktor nach Page aus Karagioszov ...
**
************************************************************
***********************************************************/

double gkl(rho, m, ca2, cb2) 
double rho, ca2, cb2;
int    m;
{
   register double RetValue;

   if (ca2 <= 1.0)
     RetValue = exp(-2.0*(1.-rho)*(1.0-ca2)*(1.0-ca2)
                / 3.0/Pm_iterativ(rho, m)/(ca2+cb2));
   else
     RetValue = exp(-(1.0-rho)*(ca2-1.0) / (ca2+4.0*cb2));

   return (RetValue);
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Q_Kimura
**
**	Funktionstyp: double
**
**	Uebergabeparameter: int n (Anzahl der Knoten)
**                      int c (Anzahl der Klassen)
**
**	Beschreibung:
**  berechnet die mittlere Warteschlangenlaenge fuer 
**  G/G/m-FCFS Knoten nach der Formel von Kimura
**  Kimura, T.: "Heuristic approximations for the mean
**  waiting time in the GI/G/S queue", Report No. B55,
**  Tokyo Institute of Technology, Tokyo, 1985.
**
************************************************************
***********************************************************/


double Q_Kimura(n, c)
int n, c;
{
    KNOTEN_KLASSE *akt_n_c;
    KNOTEN_NETZ *akt_n;

    double GRho, Rho, ca2, cb2, RetValue;
    int m;

    akt_n_c = &(klasse[c].knoten[n]);	
    akt_n = &(knoten[n]);


    Rho = akt_n_c->rho;
    GRho = akt_n->rho;
	
    ca2 = akt_n_c->c2A;
    cb2 = akt_n_c->c2B;
    m = akt_n->AnzServer;

    RetValue = Q_MMm(n, c) * (ca2 + cb2)/2.0 /
	(
	 (1.0-ca2)/((1.0-4.0*C(m, GRho))*exp(-2./3.*(1.0-GRho)/GRho))
	 + (1.0-cb2)/(1.0+C(m, GRho))
	 + ca2
	 + cb2
	 - 1.0
	 );

    return( MAX(RetValue, 0.0) );
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Q_Marchal
**
**	Funktionstyp: double
**
**	Uebergabeparameter: int n (Anzahl der Knoten)
**                      int c (Anzahl der Klassen)
**
**	Beschreibung:
**  berechnet die mittlere Warteschlangenlaenge fuer 
**  G/G/m-FCFS Knoten nach der Formel von Marchal
**  Karagioszov, K.:  "On the approximations of Moments and
**  operational characteristics of GI/G/S Queues", Inter-
**  national Conference Operations Research 1990.
**  Aenderungen: 1. Anstatt Q_MM1*G wurde Q_MMm*G implemen-
**  tiert. 2. Der Faktor rho wurde entfernt (auskommentiert)
**
************************************************************
***********************************************************/


double Q_Marchal(n, c)
int n, c;
{
	KNOTEN_KLASSE *akt_n_c;
	KNOTEN_NETZ *akt_n;

	double GRho, Rho, ca2, cb2;
	int m;

	akt_n_c = &(klasse[c].knoten[n]);	
	akt_n = &(knoten[n]);


	Rho = akt_n_c->rho;
	GRho = akt_n->rho;
	
	ca2 = akt_n_c->c2A;
	cb2 = akt_n_c->c2B;
	m = akt_n->AnzServer;

	return (
            Q_MMm(n, c)
               * /* Rho
               * */ (1.0+cb2)*(ca2+Rho*Rho*cb2)/2.0/(1.0+Rho*Rho*cb2)
               *    gkl(GRho, m, ca2, cb2)
          );
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Q_Cosmetatos
**
**	Funktionstyp: double
**
**	Uebergabeparameter: int n (Anzahl der Knoten)
**                      int c (Anzahl der Klassen)
**
**	Beschreibung:
**    berechnet die mittlere Warteschlangenlaenge fuer 
**    G/G/m-FCFS Knoten nach der Formel von Cosmetatos
**
************************************************************
***********************************************************/


double Q_Cosmetatos(n, c)
int n, c;
{
	KNOTEN_KLASSE *akt_n_c;

	akt_n_c = &(klasse[c].knoten[n]);

	return( akt_n_c->c2B * Q_GMm(n, c) + (1.0 - akt_n_c->c2B) * Q_GDm(n, c) );

}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Q_Page
**
**	Funktionstyp: double
**
**	Uebergabeparameter: int n (Anzahl der Knoten)
**                      int c (Anzahl der Klassen)
**
**	Beschreibung:
**  berechnet die mittlere Warteschlangenlaenge fuer 
**  G/G/m-FCFS Knoten nach der Formel basierend auf Page
**  und entnommen aus: Karagioszov, K.:  "On the 
**  approximations of Moments and operational characteristics 
**  of GI/G/S Queues", International Conference Operations 
**  Research 1990.
**
************************************************************
***********************************************************/

double Q_Page(n, c)
int n, c;
{
    KNOTEN_KLASSE *akt_n_c;
    KNOTEN_NETZ *akt_n;
    double ca2, cb2, GRho, Rho;
    int    m;

    akt_n_c = &(klasse[c].knoten[n]);
    akt_n = &(knoten[n]);

    m    = akt_n->AnzServer;

    ca2  = akt_n_c->c2A;
    cb2  = akt_n_c->c2B;
	
    Rho  = akt_n_c->rho;
    GRho = akt_n->rho;

    return ( Q_MMm(n, c) * gpa(n, c) /* * gkl(GRho, m, ca2, cb2) */ );
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: gpa()
**
**	Funktionstyp: double
**
**	Uebergabeparameter:
**       int n (Anzahl der Knoten)
**       int c (Anzahl der Klassen)
**
**	Beschreibung:
**  Korrekturfaktor nach Page aus Karagioszov ...
**
************************************************************
***********************************************************/


double gpa(n, c) 
int n, c;
{
    KNOTEN_KLASSE *akt_n_c;
    KNOTEN_NETZ *akt_n;
    register double RetValue;
    double Gdms, Gmds;
    double ca2, cb2, GRho, Rho;
    int    m;

    akt_n_c = &(klasse[c].knoten[n]);
    akt_n = &(knoten[n]);

	
    ca2   = akt_n_c->c2A;
    cb2   = akt_n_c->c2B;

    Rho   = akt_n_c->rho;
    GRho  = akt_n->rho;
    m     = akt_n->AnzServer;

    Gmds  = 0.5*(1.0+C(m, GRho));

    Gdms  = 0.5*exp(-2.0/3.0*(1.0-GRho)/GRho)*(1.0-4.0*C(m, GRho));

    RetValue = (1.0-ca2)*cb2*Gdms
	+ca2*(1.0-cb2)*Gmds
	    +ca2*cb2;

    return (RetValue);

}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Q_GMm()
**
**	Funktionstyp: double
**
**	Uebergabeparameter:
**       int n (Anzahl der Knoten)
**       int c (Anzahl der Klassen)
**
**	Beschreibung:
**  Formel aus: Kulbatzki, J.: "Das Programmsystem PRIORI - 
**  Erweiterung und Validierung mit Hilfe von Simulationen",
**  Diplomarbeit an der Friedrich Alexander Universitaet
**  Erlangen Nuernberg, 1989.
**
************************************************************
***********************************************************/


double Q_GMm(n, c)
int n, c;
{
	KNOTEN_KLASSE *akt_n_c; 
	KNOTEN_NETZ *akt_n; 
	double ca2, Rho, GRho;
	int    m;

	akt_n_c = &(klasse[c].knoten[n]);
	akt_n = &(knoten[n]);

	m    = akt_n->AnzServer;

	ca2  = akt_n_c->c2A;

	Rho  = akt_n_c->rho;
	GRho = akt_n->rho;

   if ((ca2>=0.0)&&(ca2<=1.0))
   {
      return(
              0.5
              * (ca2+1.0)
              * exp(-2./3.*(1.0-GRho)/Pm(GRho, m)*(1.0-ca2)*(1.0-ca2)/(1.0+ca2))
              * Q_MMm(n, c)
            );
   }
   else
   {
      if (ca2>1.0)
      {
         return (
                  0.5*(pow(sqrt(ca2), f_Kulbatzki(sqrt(ca2), 1.0, GRho))+1.0)
                  * Q_MMm(n, c)
                );
      }
      else
      {
         printf("FATAL ERROR: Q_GMm - ca2<0.0!\n");
         exit(10);
      }
   }
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Q_GDm()
**
**	Funktionstyp: double
**
**	Uebergabeparameter:
**       int n (Anzahl der Knoten)
**       int c (Anzahl der Klassen)
**
**	Beschreibung:
**  Formel aus: Kulbatzki, J.: "Das Programmsystem PRIORI - 
**  Erweiterung und Validierung mit Hilfe von Simulationen",
**  Diplomarbeit an der Friedrich Alexander Universitaet
**  Erlangen Nuernberg, 1989.
**
************************************************************
***********************************************************/


double Q_GDm(n, c)
int n, c;
{
	KNOTEN_KLASSE *akt_n_c;
	KNOTEN_NETZ *akt_n;

	double ca2, Rho, GRho;
	int    m;

	akt_n_c = &(klasse[c].knoten[n]);
	akt_n = &(knoten[n]);

	m    = akt_n->AnzServer;

	ca2  = akt_n_c->c2A;
	
	Rho  = akt_n_c->rho;
	GRho = akt_n->rho;

	return ( pow(sqrt(ca2), (f_Kulbatzki(sqrt(ca2), 0.0, GRho)*h_Kulbatzki(GRho, m))) * Q_MDm(n, c) );
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: Q_MDm()
**
**	Funktionstyp: double
**
**	Uebergabeparameter:
**       int n (Anzahl der Knoten)
**       int c (Anzahl der Klassen)
**
**	Beschreibung:
**  Formel aus: [Cosmeta75] Cosmetatos, G. P.:
**  "Approximate explicit formulae for the average queueing
**  time in the process (M/D/r) and (D/M/r)",
**  INFOR, 13, 1975, pp. 328-331.   
**
************************************************************
***********************************************************/


double Q_MDm(n, c)
int n, c;
{
	KNOTEN_KLASSE *akt_n_c;
	KNOTEN_NETZ *akt_n;
	double Rho, GRho;
	int    m;

	akt_n_c = &(klasse[c].knoten[n]);
	akt_n = &(knoten[n]);

	m    = akt_n->AnzServer;
	Rho  = akt_n_c->rho;
	GRho  = akt_n->rho;

	return ( 0.5 * Q_MMm(n, c) * (1. + C(m, GRho)) );

}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: C()
**
**	Funktionstyp: double
**
**	Uebergabeparameter:
**       int    m   (Anzahl der Server)
**       double rho (Auslastung)
**
**	Beschreibung:
**  Berechnung des Kooperationskoeffizienten fuer Q_Kimura  
**
************************************************************
***********************************************************/



double C(m, rho)
   int m;
   double rho;
{
   return (
            (1.0-rho)*((double)m-1.0)*(sqrt((double)(4+5*m))-2.0)
                                     / 16.0/rho/(double)m
          );
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: f_Kulbatzki()
**
**	Funktionstyp: double
**
**	Uebergabeparameter:
**       double ca (Var.koeff. d. Ank.prozesses)
**       double cb (Var.koeff. d. Bed.prozesses)
**       double rho (Auslastung)
**
**	Beschreibung:
**  Berechnung des Exponenten fuer GGm-Approximation von  
**  Kulbatzki
**
************************************************************
***********************************************************/


double f_Kulbatzki(ca, cb, rho) 
double ca, cb, rho;
{
   register double RetValue;

   if ((ca == 1.0)||(ca == 0.0))
   {
      RetValue = 1.0;
   }
   else
   {
      if ((ca>0.0) && (ca<1.0))
      {
          RetValue = (cb*cb-(ca+2.6)*cb+1.6*ca-4.0)*(1.2*rho+0.7*ca-1.7)
                  + 1.7*ca + 0.6;
      }
      else /* ca>1 */
      {
          RetValue = 0.22*sqrt(rho+0.12)*cb + 6.83*rho*rho -
                     8.31*rho + 2.55;
      }
   }
   return (RetValue);
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: h_Kulbatzki()
**
**	Funktionstyp: double
**
**	Uebergabeparameter:
**       double rho (Auslastung)
**       int    m   (Anzahl der Server)
**
**	Beschreibung:
**  Hilfsfunktion zur Berechnung von h(.,.) fuer Q_GDm in 
**  Q_Cosmetatos
**
************************************************************
***********************************************************/


double h_Kulbatzki( rho, m)
double rho;
int    m;
{
   register double RetValue;

   RetValue = 4.0 * sqrt((double)(m-1)/(double)(m+47)) * (1.0-rho) + 1.0;

   return (RetValue);
}


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: AC_BerLei()
**                 KLB_BerLei()
**                 KIM_BerLei()
**                 PAG_BerLei()
**                 COS_BerLei()
**                 MAR_BerLei()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**
**	Beschreibung:
**  Funktionen zur Berechnung der Leistungsgroessen, die von
**  den Dekompositionsmethoden aufgerufen werden koennen.
**  siehe auch Option f.
**
************************************************************
***********************************************************/

void AC_BerLei()
{
	BerechneLeistung(Q_Allen_Cunneen, "a_c");
}


void KLB_BerLei()
{
	BerechneLeistung(Q_KramerLangen, "klb");
}

void KIM_BerLei()
{
	BerechneLeistung(Q_Kimura, "kim");
}

void PAG_BerLei()
{
	BerechneLeistung(Q_Page, "pag");
}


void COS_BerLei()
{
	BerechneLeistung(Q_Cosmetatos, "cos");
}


void MAR_BerLei()
{
	BerechneLeistung(Q_Marchal, "mar");
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: BerechneLeistung()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: PFD (Pointer auf eine double-Funktion)
**
**
**	Beschreibung:
**  Fuer alle Knoten und Klassen wird zuerst die mittl. 
**  Warteschlangenlaenge mit der durch den Uebergabeparameter
**  spezifizierten Formel ausgerechnet. Anschliessend werden
**  alle anderen Leistungsgroessen (wiederum fuer alle 
**  Knoten und Klassen) berechnet.
**
**
************************************************************
***********************************************************/


void BerechneLeistung(func_p, flg_string)
PFD func_p;
char *flg_string;
{
	KNOTEN_KLASSE *akt_node;
	int i, k;


	for (k = 0; k < AnzKlassen; k++) {
		for (i = 1; i <= AnzKnoten; i++) {

			akt_node = &(klasse[k].knoten[i]);

			if(knoten[i].KnotenTyp == TYP4){
				akt_node->rho = 0.0;
				knoten[i].rho = 0.0;
			}			
			else {
				akt_node->rho = akt_node->lambda / (akt_node->mue * knoten[i].AnzServer);
				knoten[i].rho = knoten[i].lambda / (knoten[i].mue);
			}

			/* Name der ausgewaehlten Funktion */

			akt_node->flg_name = flg_string;

			if(knoten[i].KnotenTyp == TYP4){
				akt_node->mwsl = akt_node->lambda / akt_node->mue;
				dbgtext(DBG_L7, "Knoten ist vom Typ 4 ")
			}

			else {
				akt_node->mwsl = (*func_p)(i, k);
			}

			akt_node->maa = akt_node->mwsl + knoten[i].AnzServer * akt_node->rho;
			akt_node->mwz = akt_node->mwsl / akt_node->lambda;
			akt_node->mvz = akt_node->mwz + 1.0/akt_node->mue;

		}
	}
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: AUTO_BerLei()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**
**	Beschreibung:
**  Berechnet wie BerechneLeistung() fuer alle Knoten und 
**  Klassen die Leistungsgroessen. Unterschied ist nur die 
**  der Formel zur Berechnung der mittl. Warteschlangenlaenge.
**  In dieser Funktion wird nicht nur eine Formel verwendet,
**  sondern fuer jeden Knoten und Klasse wird entschieden,
**  welche Formel genommen wird. Zur Auswahl stehen Q_Kimura,
**  Q_Page, Q_Cosmetatos, Q_KramerLangen, Q_AllenCunneen und 
**  Q_Marchal.
**  Die Auswahl trifft die Funktion SelectFuncMwsl()
**
************************************************************
***********************************************************/

void AUTO_BerLei()
{
    PFD func_p;
    LookUpTab_CB *ptr;

    KNOTEN_KLASSE *akt_node;
    int i, k;



    for (k = 0; k < AnzKlassen; k++)
    {
	for (i = 1; i <= AnzKnoten; i++)
	{
	    akt_node = &(klasse[k].knoten[i]);

	    if(knoten[i].KnotenTyp == TYP4)
	    {
		akt_node->rho = 0.0;
		knoten[i].rho = 0.0;
	    }			
	    else
	    {
		akt_node->rho = akt_node->lambda / (akt_node->mue * knoten[i].AnzServer);
		knoten[i].rho = knoten[i].lambda / (knoten[i].mue);
	    }

	    /* Auswahl der Funktion zur Berechnung der mwsl */

	    ptr = SelectFuncMwsl(AnzKlassen, knoten[i].AnzServer, knoten[i].rho, knoten[i].c2A, knoten[i].c2B);

	    func_p = ptr->fkt_ptr;
		
	    /* Name der ausgewaehlten Funktion */

	    akt_node->flg_name = ptr->fkt_name;

	    if(knoten[i].KnotenTyp == TYP4)
	    {
		akt_node->mwsl = akt_node->lambda / akt_node->mue;
		dbgtext(DBG_L7, "Knoten ist vom Typ 4 ");
	    }
	    else
	    {
		akt_node->mwsl = (*func_p)(i, k);
	    }

	    akt_node->maa = akt_node->mwsl + knoten[i].AnzServer * akt_node->rho;
	    if (akt_node->lambda != 0.0)
	    {		
		akt_node->mwz = akt_node->mwsl / akt_node->lambda;
		akt_node->mvz = akt_node->mwz + 1.0/akt_node->mue;
	    }
	    else
	    {
		akt_node->mwz = 0.0;
		akt_node->mvz = 0.0;
	    }
	}
    }
}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: 
**
**	Funktionstyp: 
**
**	Uebergabeparameter:
**        int    kl  (Anzahl der Klassen)
**        int    m   (Anzahl der Server)
**        double rho (Auslastung des Knoten)
**        double ca2 (quad. Var.koeff. d. Ank.prozesses)
**        double cb2 (quad. Var.koeff. d. Bed.prozesses)
**
**
**	Beschreibung:
**  Waehlt fuer die Funktion AUTO_BerLei() die Funktion zur
**  Berechnung der mittl. Warteschlangenlaenge aus. Die
**  Auswahl haengt von der Anzahl der Server, der
**  Gesamtauslastung des Knoten, sowie vom quad. Varkoeff.
**  des Ankunfts- u. Bedienproprozesses ab.
**
************************************************************
***********************************************************/

LookUpTab_CB *SelectFuncMwsl(kl, m, rho, ca2, cb2)
int kl, m;
double rho, ca2, cb2;
{
    PFD	func_p;

    int	IK, IM, IR, ICA, ICB;

    LookUpTab_K *ptr_k;
    LookUpTab_M *ptr_m;
    LookUpTab_R *ptr_r;
    LookUpTab_CA *ptr_ca;
    LookUpTab_CB *ptr_cb;
	
	
    IK = IM = IR = ICA = ICB = 0;


    while((LUT[IK+1].k != -1) && (kl>(LUT[IK+1].k + LUT[IK].k)/2)){
	IK++;
    }
    ptr_k = &(LUT[IK]);

    while((ptr_k->lut_m[IM+1].m != -1) && (m>(ptr_k->lut_m[IM].m + ptr_k->lut_m[IM+1].m)/2)){
	IM++;
    }
    ptr_m = &(ptr_k->lut_m[IM]);

    while((ptr_m->lut_r[IR+1].rho != -1.0) && (rho>(ptr_m->lut_r[IR].rho + ptr_m->lut_r[IR+1].rho)/2)){
	IR++;
    }
    ptr_r = &(ptr_m->lut_r[IR]);

    while((ptr_r->lut_ca[ICA+1].ca2 != -1.0) && (ca2 > (ptr_r->lut_ca[ICA].ca2 + ptr_r->lut_ca[ICA+1].ca2)/2)){
	ICA++;
    }
    ptr_ca = &(ptr_r->lut_ca[ICA]);

    while((ptr_ca->lut_cb[ICB+1].cb2 != -1.0) && (cb2>(ptr_ca->lut_cb[ICB].cb2 + ptr_ca->lut_cb[ICB+1].cb2)/2)){
	ICB++;
    }
    ptr_cb = &(ptr_ca->lut_cb[ICB]);

    dbginfo(5, "SelectFuncMwsl(): %s selected.\n", ptr_cb->fkt_name);

    return(ptr_cb);
}

