/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/


/***********************************************************
************************************************************
**
**  Modulname: whitt.c
**
**	Beschreibung:
**  in diesen Modul werden alle verfahrenspezifischen Funk-
**  tionen fuer die Methode nach Whitt definiert.
**  Die Funktionen aus diesen Modul duerfen Funktionen aus 
**  den Modulen 'allg.c' und 'leistung.c' verwenden.
**  
**
***********************************************************
***********************************************************/

static char *SccsId = "@(#)PEPSY:\twhitt.c\t1.2\t95/05/19 15:37:59";

#define WHITT_C
#define EXTERN_KNOTENTYPEN

#include "def.h"

#define WHITT_KN_TYPEN 00000777

extern leistung_s_t ForLei[];

/************************************************************
*************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: whi__berechne_c2A_pro_Knoten_u_Klasse()
**
**	Funktionstyp: void 
**
**	Uebergabeparameter: int node
**
**	Beschreibung:
**  Berechnet fuer alle Klassen und alle Knoten den quad.
**  Var.koeff. des Ank.prozesses.
**
************************************************************
***********************************************************/

void whi__berechne_c2A_pro_Knoten_u_Klasse(node)
int node;
{
	int i, j, k;
	double hv;

	dbgtext(DBG_FKT1, "\nFunktion whi__berechne_c2A_pro_Knoten_u_Klasse()\n") 

	
	for (k = 0; k < AnzKlassen; k++) {
		dbginfo(25, "Klasse %d :\n", k)
		for (i = 1; i <= AnzKnoten; i++) {
			hv = 0.0;
			for (j = 0; j <= AnzKnoten; j++) {

				/* AENDERUNG Feb 94 */
				if (klasse[k].knoten[i].lambda != 0)

					hv += klasse[k].rate[j][i] / klasse[k].knoten[i].lambda * klasse[k].c[j][i];
			} /* end for j */
			klasse[k].knoten[i].c2A = hv;

			dbginfo(DBG_L1, "\t%6.2f", klasse[k].knoten[i].c2A)

		} /* end for i */

		dbgtext(DBG_L1, "\n")

	} /* end for k */

	dbgtext(DBG_L1, "\n")

}


/************************************************************
*************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: whi__berechne_c2D_pro_Knoten()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int node
**
**
**	Beschreibung:
**  Berechnet fuer Knoten node den quad. Var.koeff. des 
**  Abgangsprozesses.
**
************************************************************
***********************************************************/

void whi__berechne_c2D_pro_Knoten(node)
int node;
{
	double rho, hv1, hv2, hvi;

	dbgtext(DBG_FKT1, "\nFunktion whi__berechne_c2D_pro_Knoten()\n") 

	rho = knoten[node].rho;

	if (knoten[node].KnotenTyp == TYP4) {

		knoten[node].c2D = knoten[node].c2A;
	} /* end if */

	else {
		hvi = knoten[node].AnzServer;
		/* FEHLER mail Matthias! hvi = sqrt(hvi);*/
		hv1 = pow(rho, 2.0) / sqrt(hvi) * (knoten[node].c2B - 1.0);
		hv2 = (1.0 - pow(rho, 2.0)) * (knoten[node].c2A - 1.0);
		knoten[node].c2D = 1.0 + hv1 + hv2;
	
	} /* end else */

	/* kirschni */

	dbginfo(DBG_L1, "c2D_pro_Knoten(): Knoten[%d]=", node)
	dbginfo(DBG_L1, "%f\n", knoten[node].c2D)
}


/************************************************************
*************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: whi__berechne_c2A_pro_Knoten()
**
**	Funktionstyp: void 
**
**	Uebergabeparameter: int node
**
**
**	Beschreibung:
**  Berechnet fuer alle Klassen und alle Knoten den quad.
**  Var.koeff. des Ank.prozesses
**
************************************************************
***********************************************************/

void whi__berechne_c2A_pro_Knoten(node)
int node;
{
	int i, k;
	double var;

	dbgtext(DBG_FKT1, "Funktion whi__berechne_c2A_pro_Knoten()\n")

	var = 0.0;

	for (k = 0; k < AnzKlassen; k++) {
		for (i = 0; i < AnzKnoten; i++) {
			if (klasse[k].uew[i][node] > 0.0){
				var += klasse[k].rate[i][node] / knoten[node].lambda * klasse[k].c[i][node];
			}


		} /* end for k */
	} /* end for k */

	knoten[node].c2A = var;
		
	dbginfo(DBG_L1, "knoten[%d].c2A ist ", node)
	dbginfo(DBG_L1, "%4.4f", knoten[node].c2A)
}

/************************************************************
*************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: whi_CheckVerfahren()
**
**	Funktionstyp: int *
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Einlesen des Netzwerkes und Ueberpruefung, ob Verfahren
**  auf Netz anwendbar ist.
**
************************************************************
***********************************************************/

int *whi_CheckVerfahren(argc, argv)
int argc;
char **argv;
{
	int i, check, *p;

	Einlesen(stdin);

	if (NetzTyp != 2) {
		fprintf(stderr, "falscher Netztyp\n");
		printf("0 0 0\n");
		exit(-1);
	}

	for (i = 1; i <= AnzKnoten; i++) {
		if (!(check = CheckKnoten(knoten[i].KnotenTyp, WHITT_KN_TYPEN))) {
			/* falscher Knotentyp */
			/* Verfahren nicht anwendbar */
			fprintf(stderr, "falscher Knotentyp\n");
			printf("0 0 0\n");
			exit(-1);
		}
	}

	p = (int *)malloc(2*sizeof(int));

	p[0] = verfahren_nr;
	p[1] = -1;
	
	AnzVer = SINGLE;

	return (p);

}

/************************************************************
*************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: whi_InitNetz()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Berechnung aller direkt ableitbarer Netzgroessen.
**
************************************************************
***********************************************************/

void whi_InitNetz()
{
	BerAnkRaten();
	BerBedRaten_Kn();
	CheckStab();
	BerUebergangsRaten();
}

/************************************************************
*************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: whi_InitIteration()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschreibung:
**  Initialisierung der Var.koeff. mit den Startwerten.
**
************************************************************
***********************************************************/

void whi_InitIteration()
{

	InitVarKoeff();

}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: whi_Iterationsschritt()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int it (Anzahl d. Iterationen)
**
**	Beschriebung:
**  Iterationsschritte, die bei jedem Schleifendurchlauf 
**  zur Berechnung der quad. Var.koeff. ausgefuehrt werden.
**
************************************************************
***********************************************************/

void whi_Iterationsschritt(it)
int it;
{
	int node;

	dbginfo(25, "\n\n%d. Iterationsschritt\n\n", it)

	InitVarKoeff_alt();

	for (node=1; node<=AnzKnoten; node++) {
		whi__berechne_c2A_pro_Knoten_u_Klasse(node);
		Ber_ca2_Kn(node);
		whi__berechne_c2D_pro_Knoten(node);
		chylla__berechne_VarKoeffizienten(node);
	} /* end for node */

}


/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: whi_IterationsAbbruch()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: int it (Anzahl d. Iterationen)
**
**
**	Beschriebung:
**  Ueberpruefung des Abbruchkriteriums der Schleife zur 
**  approx. Berechnung der quad. Var.koeff..
**
************************************************************
***********************************************************/

int whi_IterationsAbbruch(it)
int it;
{
	int ret;

	if (it > MAX_IT) {
		dbgtext(DBG_L7, "Abbruch der Iteration erfolgte wegen Ueberschreitung der max. Iterationsschritte\n")
		ret = 0;
	}
	else {
		
		if ((ret = CheckGenauigkeit())) {

			dbgtext(DBG_L7, "Abbruch der Iteration erfolgte da Abbruchkriterium erfuellt\n")

		}

	}

	return(ret);

}

/***********************************************************
************************************************************
**
**	Autor: Martin Fischer
**
**	Datum: November 1993
**
**	Funktionsname: whi_Leistung()
**
**	Funktionstyp: void
**
**	Uebergabeparameter: keine
**
**	Beschriebung:
**  Berechnung der Leistungsgroessen in Abhaengigkeit der
**  quad. Var.koeff. und Ausgabe an den Postfilter.
**
************************************************************
***********************************************************/

void whi_Leistung(nr)
int nr;
{

	/*chylla__berechne_leistungsgroessen();*/
	(*(ForLei[formel_nr].BerLeist))();
	Ausgabe_Q(stdout, nr);

}

