/**********************************************************************/
/**********************************************************************/
/****************  Programm zur Analyse geschlossener  ****************/
/****************  Warteschlangennetze nach der        ****************/
/****************  Erweiterten-Produktform-Methode     ****************/
/**********************************************************************/
/**********************************************************************/

static char *SccsId = "@(#)PEPSY:\tepf.c\t1.5\t94/02/03 15:21:16";

#include <stdio.h>
#include <math.h>
#include <signal.h>
#include "xxalloc.h"

/**********************************************************************/
/******  Definition der Konstanten :                             ******/
/**********************************************************************/

/**********************************************************************/
/******  Deklaration der Variablen :                             ******/
/******                                                          ******/
/******  su              Untergrenze des Intervalls              ******/
/******  so              Obergrenze des  Intervalls              ******/
/******  w               Schrittweite                            ******/
/******  lambda          Durchsatz                               ******/
/******  a               Ankunftsrate                            ******/
/******  opterg          optimales lambda                        ******/
/******  mue             Bedienrate                              ******/
/******  cb              qu. Variationskoef. des Bedienprozesses ******/
/******  p               Uebergangswahrscheinlichkeiten          ******/
/******  F1              Zustandswahrscheinlichkeiten            ******/
/******  G               Normalisierungskonstante                ******/
/******  pr              Randzustandswahrscheinlichkeiten        ******/
/******  lam             neu berechneter Durchsatz               ******/
/******  norm            normalisierter Durchsatz                ******/
/******  maa             mitl. anzahl der Auftraege im Knoten    ******/
/******  gesmaa          mittl. Anzal der Auftraege im Netz      ******/
/******  optrel          Nummer des Engpassknotens               ******/
/******  n               Anzahl der Knoten                       ******/
/******  k               Anzahl der Auftraege                    ******/
/******  typ             Nummer des Knotentyps                   ******/
/******  test,hilf,                                              ******/
/******  p1,f,exfuenf    Hilfsvariablen                          ******/
/**********************************************************************/

/*	Funktionen in diesem Modul
*/
void	abbruch();
void	eingabe();
void	engpass();
void	intgrenzen();
double	min();
void	zustandswarsch();
double	summe();
double	summe1();
double	summe2();
double	rho();
double	hoch();
double	fak();
void	normaldurchs();
void	normkonstant();
double	fehler();
void	naechsintervall();
void	mitauftr();
void	ausgabe();
void	randwarsch();



double su,so,w,test,lambda,*a,hilf,opterg,*mue,*e,
      **p,**p1,*f,**F1,**G,
      *lam,*cb,*maa,gesmaa,**pr,*norm;
int n,k,optrel,*typ,exfuenf;

void abbruch(sig)
int sig;
{
        printf("epf_signal_%d\n 0 0 0 \n", sig);
        exit(1);
}

/************************************************/
/******     Beginn des Hauptprogrammes     ******/
/************************************************/

int main(argc, argv)
	int argc;
	char *argv[];
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

	int i,x,anz,anf;




	for (i=1; i<12; i++)
			signal (i, abbruch);

	if ((argc > 1) && (!strcmp(argv[1], "-V")))    /* kirschni 22.11.1991 */
	{
		fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
		exit(0);
	}


  eingabe();
  engpass();
  intgrenzen();
  for (anz=1;anz<=6;anz=anz+1)
    {
      test=(so-su)/w;
      if (su==0)
        x=1;
      else
        x=0;
      anf=1;
      while (x<=test)
        {
          lambda=su+x*w;
          for (i=1;i<=n;i=i+1)
           {
            a[i]=lambda*e[i];
           }
          zustandswarsch();
          normkonstant();
          normaldurchs();
          if (anf==1)
           {
            opterg=lambda;
            hilf=fehler();
            anf=0;
           }
          else
           if (hilf>fehler())
            {
              hilf=fehler();
              opterg=lambda;
            }
          x=x+1;
        }
       naechsintervall();
    }
  lambda=opterg;
  for (i=1;i<=n;i=i+1)
    a[i]=lambda*e[i];
  zustandswarsch();
  normkonstant();
  normaldurchs();
  mitauftr();
  ausgabe();
  exit(0);
}


/*************************/
/* Eingabe der Netzdaten */
/*************************/

void eingabe()
{
int i,h1;
double h;

  scanf("%d",&n);
  scanf("%d",&k);

  /* Initialisierung der globalen Felder */

  a = d1_alloc(0,n);
  mue = d1_alloc(0,n);
  e = d1_alloc(0,n);
  p = d2_alloc(0,n, 0,k);
  p1 = d2_alloc(0,n, 0,k);
  f = d1_alloc(0,n);
  F1 = d2_alloc(0,n, 0,k);
  G = d2_alloc(0,n, 0,k);
  lam = d1_alloc(0,n);
  cb = d1_alloc(0,n);
  maa = d1_alloc(0,n);
  pr = d2_alloc(0,n, 0,k);
  norm = d1_alloc(0,n);

  typ = i1_alloc(0,n);

  exfuenf=0;
  for (i=1;i<=n;i=i+1)
   {
     scanf("%d",&h1);
     typ[i]=h1;
     if (typ[i]==5)
       exfuenf=1;
   }
  for (i=1;i<=n;i=i+1)
   {
     scanf("%lf",&h);
     mue[i]=h;
     if (typ[i]==5)
      {
        scanf("%lf",&h);
        cb[i]=h;
      }
     else
        cb[i]=1;
   }
  for (i=1;i<=n;i=i+1)
   {
     scanf("%lf",&h);
     e[i]=h;
   }
}


/***************************/
/* Bestimmung des Engpass- */
/* Bedienzentrums          */
/***************************/

void engpass()
{
  int i;
  double h;

  h=e[1]/mue[1];
  optrel=1;
  for (i=2;i<=n;i=i+1)
    if ((e[i]/mue[i])>=h)
     {
       h=e[i]/mue[i];
       optrel=i;
     }
}


/***********************************/
/* Festlegen der ersten Intervall- */
/* grenzen und der ersten Schritt- */
/* weite                           */
/***********************************/

void intgrenzen()
{
  su=0;
  so=min();
  w=0.05*so;
  if (w<=0.05)
    w=0.05;
}


/***********************************/
/* Festlegen der ersten Obergrenze */
/* des Intervalls                  */
/***********************************/

double min()
{
  int i;
  double h;

  h=mue[1]/e[1];
  for (i=2;i<=n;i=i+1)
   {
      if ((mue[i]/e[i])<h)
        h=mue[i]/e[i];
   }
  return(h);
}


/****************************/
/* Bestimmung der Zustands- */
/* wahrscheinlichkeiten     */
/****************************/

void zustandswarsch()
{
  int i,j;
  double hilf1,hi1,hi2,hi3,hi4,hi5,hi6,hi7,hi8,fak();

  for (i=1;i<=n;i=i+1)
   {
     p1[i][0]=1;
     for (j=1;j<=k-1;j=j+1)
      {
         hilf1=j;
      /* exp-verteilt */
      /*==============*/
         if (cb[i]==1)
          {
             hi1=rho(i);
             p1[i][j]=pow(hi1,hilf1);
          }
         else
          {
          /* D-verteilt */
          /*============*/
            if (cb[i]==0)
             {
               if (j==1)
                {
                  hi1=rho(i);
                  p1[i][j]=exp(hi1)-1;
                }
               else
                  p1[i][j]=summe2(i,j);
             }
            else
             {
             /* H-verteilt */
             /*============*/
                if (cb[i]>1)
                 {
                   hi1=0.5*(1-sqrt((cb[i]-1)/(cb[i]+1)));
                   hi2=1-hi1;
                   hi1=2*hi1*mue[i];
                   hi2=2*hi2*mue[i];
                   hi1=a[i]/hi1;
                   hi2=a[i]/hi2;
                   hi3=rho(i);
                   hi4=pow(hi2,2.0)*(hi1*(hi1-2)+1);
                   hi4=hi4+hi2*(hi1*(4*hi3-2*hi1-2))+pow(hi1,2.0);
                   hi4=sqrt(hi4); 
                   hi5=0.5*((hi1+hi2+hi4)/(hi1*hi2)+1);
                   hi6=0.5*((hi1+hi2-hi4)/(hi1*hi2)+1);
                   hi5=1/hi5;
                   hi6=1/hi6;
                   hi7=(hi4+hi1*(1-hi2)+hi2-2*hi3)/(2*hi4);
                   hi8=1-hi7;
                 }
             /* E-verteilt */
             /*============*/
                else
                 {
                   hi1=rho(i);
                   hi2=sqrt((8+hi1)*hi1);
                   hi5=(2*hi1)/(hi1+4+hi2);
                   hi6=(2*hi1)/(hi1+4-hi2);
                   hi7=(hi2-(4+hi1))/(2*hi2);
                   hi8=1-hi7;
                  }
                p1[i][j]=hi7*pow(hi5,hilf1)+hi8*pow(hi6,hilf1);
             }
          }
      }
   }
  for (i=1;i<=n;i=i+1)
   {
     f[i]=1/(p1[i][0]+rho(i)*summe1(i));
     for (j=0;j<=k-1;j=j+1)
      {
        F1[i][j]=f[i]*p1[i][j];
      }
     F1[i][k]=1-f[i]*summe1(i);
   }
}

double summe1(u)
int u;
{
  int l;
  double h;

  h=0;
  for (l=0;l<=k-1;l=l+1)
     h=h+p1[u][l];
  return(h);
}


double summe2(u,l)
int u,l;
{
  int g;
  double h,h1,h2,h3,h4,h5;

  h=0;
  for (g=0;g<=l;g=g+1)
   {
     h4=l;
     h5=g;
     h1=hoch(l-g);
     h2=pow(h5*rho(u),h4-h5-1.0);
     h3=(h5*rho(u)+h4-h5)*exp(h5*rho(u));
     h=h+(h1*h2*h3)/fak(l-g);
   }
  return(h);
}


/*******************************/
/* Berechnung der Auslastungen */
/*     in jedem Knoten         */
/*******************************/
 
double rho(z)
int z;
{
  double h;

  h=a[z]/mue[z];
  return(h);
}


/******************/
/* Bestimmung von */
/* -1 hoch an     */
/******************/

double hoch(an)
int an;
{
  int a1;
  double hl;

  hl=1;
  a1=1;
  while (a1<=an)
   {
     hl=hl*(0-1);
     a1=a1+1;
   }
  return(hl);
}


/********************/
/* Berechnug von x! */
/********************/

double fak(x)
int x;
{
  int la;
  double hi;

  hi=1;
  la=1;
  while (la<=x)
   {
     hi=hi*la;
     la=la+1;
   }
  return(hi);
}


/***********************************/
/* Berechnung der Normalisierungs- */
/*          konstanten             */
/***********************************/

void normkonstant()
{
  int i,j;


  for (j=0;j<=k;j=j+1)
     G[1][j]=F1[1][j];
  for (i=2;i<=n;i=i+1)
     G[i][0]=F1[i][0]*G[i-1][0];
  for (i=2;i<=n;i=i+1)
     for (j=1;j<=k;j=j+1)
          G[i][j]=summe(i,j);
}

double summe(o,p)
int o,p;
{
  int l;
  double h;

  h=0;
  for (l=0;l<=p;l=l+1)
    h=h+F1[o][l]*G[o-1][p-l];
  return(h);
}


/*********************************/
/* Bestimmung des normalisierten */
/*        Durchsatzes            */
/*********************************/

void normaldurchs()
{
  int i;

   for (i=1;i<=n;i=i+1)
    {
     if (typ[i]==5)
      {
       randwarsch(i);
       lam[i]=mue[i]*(1-pr[i][0]);
      }
     else
       lam[i]=a[i]*G[n][k-1]/G[n][k];
     norm[i]=lam[i]/e[i];
    }
}


/********************************/
/* Berechnung der Randzustands- */
/*     wahrscheinlichkeiten     */
/********************************/

void randwarsch(i1)
int i1;
{
  int j1,l1;
  static double **gm = (double **)0,h;

  if (!gm)
     gm = d2_alloc(0,n, 0,k);

  if (i1!=n)
   {
     gm[i1][0]=G[n][0]/F1[i1][0];
     for (j1=1;j1<=k;j1=j1+1)
      {
        h=0;
        for (l1=1;l1<=j1;l1=l1+1)
           h=h+F1[i1][l1]*gm[i1][j1-l1];
        gm[i1][j1]=(G[n][j1]-h)/F1[i1][0];
      }
     for (j1=0;j1<=k;j1=j1+1)
        pr[i1][j1]=F1[i1][j1]*gm[i1][k-j1]/G[n][k];
   }
  else
   { 
     for (j1=0;j1<=k;j1=j1+1)
        pr[n][j1]=F1[n][j1]*G[n-1][k-j1]/G[n][k];
   }
}


/**************************/
/* Auswertung der Fehler- */
/*      funktion          */
/**************************/

double fehler()
{
  int i;
  double h1;

  if (exfuenf==0)
    h1=fabs(G[n][k-1]/G[n][k]-1);
  else
   {
     h1=0;
     for (i=1;i<=n;i=i+1)
       h1=h1+(e[i]/mue[i])*pow(norm[i]-norm[optrel],2.0);
   }
  return(h1);
}


/**************************/
/* Bestimmung der Grenzen */
/* und der Schrittweite   */
/* fuer das naechste      */
/* Intervall              */
/**************************/

void naechsintervall()
{

  su=opterg-w;
  if (su<0)
    su=0;
  so=opterg+w;
  w=0.1*w;
}


/***************************/
/* Berechnung der mittlere */
/* Anzahl der Auftraege    */
/***************************/

void mitauftr()
{
  int i,j;
  double h,h1,h2,h3,h4;

  h4=0;
  for (i=1;i<=n;i=i+1)
   {
    h=0;
    for (j=1;j<=k;j=j+1)
     {
       h3=j;
       h1=pow(rho(i),h3);
       h2=G[n][k-j]/G[n][k];
       if (typ[i]==5)
         h=h+h3*pr[i][j];
       else
         h=h+h1*h2;
     }
    maa[i]=h;
    h4=h4+h;
   }
  gesmaa=h4;
}


/**************************/
/* Ausgabe der Leistungs- */
/*      groessen          */
/**************************/

void ausgabe()
{
  int i;
  double mvz,mwz,mwsl,h,h1;

  h=k;
  printf("'EPF'\n");
  printf("1 %d 1 \n",n);
  for (i=1;i<=n;i=i+1)
    {
      mvz=maa[i]/a[i];
      mwz=mvz-1/mue[i];
      if (mwz <= 0)
        mwz=0;
      mwsl=mwz*a[i];
      h1=rho(i);
      printf("%8.6f %8.6f ",a[i],e[i]);
      printf("%8.6f %8.6f ",1/mue[i],h1);
      printf("%8.6f %8.6f %8.6f %8.6f \n",mvz,maa[i],mwz,mwsl);
    }
}
