/******************************************************************/
/******************************************************************/
/****************  Programm zur Analyse von offenen  **************/
/****************  Warteschlangennetzen nach dem     **************/
/****************  Prinzip der Maximalen Entropie    **************/
/******************************************************************/
/******************************************************************/

static char *SccsId = "@(#)PEPSY:\termem.c\t1.5\t94/04/26 12:54:17";

#include <stdio.h>
#include <math.h>
#include "xxalloc.h"

/******************************************************************/
/******      Definition der Konstanten :                     ******/
/******                                                      ******/
/******************************************************************/

#define epsi 0.002


/******************************************************************/
/******     Deklaration der Variablen :                      ******/
/******                                                      ******/
/******       lam       Ankunftsraten                        ******/
/******       auslam    externe Ankunftsraten                ******/
/******       ca        qu. Varkoef. der Ankunftszeitver-    ******/
/******                 teilung                              ******/
/******       ca0       qu. Varkoef. der externen Ankunfts-  ******/
/******                 zeitverteilung                       ******/
/******       mue       Bedienrate                           ******/
/******       cs        qu. Varkoef. der Bedienzeitverteilung******/
/******       cd1       qu. Varkoef. des Abgangsprozesses    ******/
/******       p         Uebergangswahrscheinlichkeiten       ******/
/******       e         Besuchshaeufigkeiten                 ******/
/******       n         Anzahl der Knoten                    ******/
/******       r         Anzahl der Klassen                   ******/
/******       typ       Nummer des Knotentyps                ******/
/******       te        Hilfsvariable zum Zaehlen der        ******/
/******                 Iterationsschritte                   ******/
/******       spei      Hilfsvariable zur Ueberpruefung der  ******/
/******                 Abbruchbedingung                     ******/
/******************************************************************/

double **mue,***p,**lam,
      *auslam,**ca,***cd,**ca0,
      **cs,**cd1,**spei,**sp,
      **e;
int   n,r,*typ,te;

void abbruch(sig)
int sig;
{
        printf("ermem_signal_%d\n 0 0 0 \n", sig);
        exit(1);
}

main(argc, argv)
	int argc;
	char *argv[];
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

	double pruef();

	int i;
	void abbruch();

        for (i=1; i<12; i++)
                signal (i, abbruch);
  
	if ((argc > 1) && (!strcmp(argv[1], "-V")))    /* kirschni 22.11.1991 */
	{
		fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
		exit(0);
	}


  eingabe();
  besuch();
  ankunftsraten();
  init();
  te=0;
  while (pruef()>=0.5)
   {
     te=te+1;
     abgangsstrom();
     komposition();
   }
  ausgabe();
  exit(0);
}


/*************************/
/* Eingabe der Netzdaten */
/*************************/

eingabe()
{
  int i,j,l,h1;
  double h;

  scanf("%d",&n);
  scanf("%d",&r);

/* Initialisierung aller globalen Felder */

  mue = d2_alloc(0,n, 0,r);
  p = d3_alloc(0,r, 0,n, 0,n);
  lam = d2_alloc(0,n, 0,r);
  auslam = d1_alloc(0,r);
  ca = d2_alloc(0,n, 0,r);
  cd = d3_alloc(0,r, 0,n, 0,n);
  ca0 = d2_alloc(0,n, 0,r);
  cs = d2_alloc(0,n, 0,r);
  cd1 = d2_alloc(0,r, 0,n);
  spei = d2_alloc(0,n, 0,r);
  sp = d2_alloc(0,n, 0,r);
  e = d2_alloc(0,n, 0,r);

  typ = i1_alloc(0,n);

  for (i=1;i<=n;i=i+1)
   {
     scanf("%d",&h1);
     typ[i]=h1;
   }
  for (j=1;j<=r;j=j+1)
    {
       scanf("%lf",&h);
       auslam[j]=h;
    }
  for (j=1;j<=r;j=j+1)
   {
     for (i=1;i<=n;i=i+1)
      {
        scanf("%lf",&h);
        mue[i][j]=h;
        if ((typ[i]==5)||(typ[i]==7))
         {
          scanf("%lf",&h);
          cs[i][j]=h;
         }
        else
          cs[i][j]=1;
      }
     for (i=0;i<=n;i=i+1)
      {
       for (l=0;l<=n;l=l+1)
        {
          scanf("%lf",&h);
          p[j][i][l]=h;
        }
      }
   }
  for (j=1;j<=r;j=j+1)
   {
     for (i=1;i<=n;i=i+1)
       if (p[j][0][i]!=0)
        {
          scanf("%lf",&h);
          ca0[i][j]=h;
        }
   }
}


/***************************************/
/* Berechnung der Besuchshaeufigkeiten */
/* aus den Uebergangswahrscheinlich-   */
/*              keiten                 */
/***************************************/

besuch()
{
  int i,j,l,zei,nzei,spa,x;
  double **li,*re,*q,hi,hilf;

  li = d2_alloc(0,n+1, 0,n+1);
  re = d1_alloc(0,n+1);
  q = d1_alloc(0,n+1);

  for (j=1;j<=r;j=j+1)
   {
    for (i=0;i<=n;i=i+1)
     for (l=1;l<=n;l=l+1)
      {
       if (i==0)
        re[l]=0-p[j][i][l];
       else
        {
         if (i==l)
          li[i][l]=p[j][i][l]-1;
         else
          li[i][l]=p[j][l][i];
        }
      }
    for (zei=1;zei<=n;zei=zei+1)
     if (li[zei][zei]==0)
      {
       hilf=li[zei][zei];
       x=0;
       while (hilf==0)
        {
         x=x+1;
         hilf=li[zei][zei+x];
        }
       for (i=0;i<=n;i=i+1)
        {
          if (i==0)
           {
            hi=re[i];
            re[i]=re[i+x];
            re[i+x]=hi;
           }
          else
           {
            hi=li[zei][i];
            li[zei][i]=li[zei][i+x];
            li[zei][i+x]=hi;
           }
        }
      }
     else
      {
       for (nzei=1;nzei<=n;nzei=nzei+1)
        if (nzei!=zei)
          {
           q[nzei]=0-(li[nzei][zei]/li[zei][zei]);
           for (spa=1;spa<=n;spa=spa+1)
            {
             if (spa==zei)
               li[nzei][spa]=0;
             else
               li[nzei][spa]=li[nzei][spa]+q[nzei]*li[zei][spa];
            }
           re[nzei]=re[nzei]+q[nzei]*re[zei];
          }
       }
    for (i=1;i<=n;i=i+1)
      e[i][j]=(re[i] == 0.0)?0.0:re[i]/li[i][i];
   }
}


/****************************/
/* Bestimmung der Ankunfts- */
/*        raten             */
/****************************/

ankunftsraten()
{
  int o,l;

  for (o=1;o<=n;o=o+1)
    for (l=1;l<=r;l=l+1)
      lam[o][l]=auslam[l]*e[o][l];
}


/*************************************/
/* Initialisierung der quadratischen */
/*      Variationskoeffizienten      */
/*************************************/

init()
{
  int n1,n2;
  
  for (n1=1;n1<=n;n1=n1+1)
   for (n2=1;n2<=r;n2=n2+1)
    {
      spei[n1][n2]=0;
      ca[n1][n2]=1;
    }
}


/**********************************/
/* Bestimmung und Komposition der */
/*    Zwischenankunftsstroeme     */
/**********************************/

komposition()
{
  int lauf1,lauf2,x,y;
  double summe2(),help1,help2;

  for (lauf1=1;lauf1<=r;lauf1=lauf1+1)
    for (x=1;x<=n;x=x+1)
      for (y=1;y<=n;y=y+1)
         cd[lauf1][x][y]=1+p[lauf1][x][y]*(cd1[lauf1][x]-1);

  for (lauf1=1;lauf1<=n;lauf1=lauf1+1)
    for (lauf2=1;lauf2<=r;lauf2=lauf2+1)
      {
	   spei[lauf1][lauf2]=ca[lauf1][lauf2];
	   if (lam[lauf1][lauf2] != 0.0)
	   {
		   help1=lam[lauf1][lauf2]*(ca0[lauf1][lauf2]+1);
		   help2=auslam[lauf2]*p[lauf2][0][lauf1];
		   ca[lauf1][lauf2]=1/(summe2(lauf1,lauf2)+help2/help1)-1;
	   }
      }
}

double summe2(a1,b1)
int a1,b1;
{
  int x1;
  double h,ueber();

  h=0;
  for (x1=1;x1<=n;x1=x1+1)
	 if (lam[a1][b1] != 0.0)
	 {
        h=h+ueber(b1,x1,a1)/(lam[a1][b1]*(cd[b1][x1][a1]+1));
	 }
  return(h);
}

double rho1(l1)
int l1;
{
  int la2;
  double help,rho();

  help=0;
  for (la2=1;la2<=r;la2=la2+1)
     help=help+rho(l1,la2);

  if (help >= 1.0)
  {
     printf("ermem_instabil\n 0 0 0\n");
     exit(1);
  }
  return(help);
}

double rho(l,g)
int l,g;
{
  double h;

  if (typ[l]==4)
    h=0;
  else
    h=lam[l][g]/mue[l][g];
  if (h >= 1.0)
  {
     printf("ermem_instabil\n 0 0 0\n");
     exit(1);
  }
  return(h);
}

double ueber(a,b,y)
int a,b,y;
{
  double h;

  h=lam[b][a]*p[a][b][y];
  return(h);
}


/********************************/
/* Bestimmung der quadratischen */
/* Variationskoeffizienten der  */
/*       Abgangsstroeme         */
/********************************/

abgangsstrom()
{
  int x,y;
  double mql(),p0(),h1,h2;

  for (x=1;x<=n;x=x+1)
    for (y=1;y<=r;y=y+1)
     {
      if (typ[x]==4)
        cd1[y][x]=ca[x][y];
      else
      {
       h1=mql(x,y);
       h2=p0(x,y);
       cd1[y][x]=2*h1*h2+ca[x][y]*(2*h2-1);
      }
     }
}


/****************************/
/* Berechnung der mittleren */
/* Anzahl der Auftraege     */
/****************************/

double mql(a1,a2)
int a1,a2;
{
  double h,h1,h2,h3,rho(),rho1(),sum();

  if ((typ[a1]==1)||(typ[a1]==5)||(typ[a1]==7))
   {
     h1=rho(a1,a2)/2*(ca[a1][a2]+1);
     h2=2*(1-rho1(a1));
     h=h1+sum(a1,a2)/h2;
   }
  else
   {
     if ((typ[a1]==3)||(typ[a1]==9))
        h=rho(a1,a2)/(1-rho1(a1));
     else
        h=lam[a1][a2]/mue[a1][a2];
   }
  return(h);
}

double sum(b1,b2)
int b1,b2;
{
  int u;
  double hi,rho();

  hi=0;
  for (u=1;u<=r;u=u+1)
	{
		if (lam[b1][u] != 0.0)
		{
			hi=hi+lam[b1][b2]/lam[b1][u]*rho(b1,u)*rho(b1,u)*(cs[b1][b2]+ca[b1][b2]);
		}
	}
  return(hi);
}


/********************************/
/* Berechnung der Randzusatnds- */
/*    wahrscheinlichkeit        */
/********************************/

double p0(x1,y1)
int x1,y1;
{
  double h,h1,h2,h3,sig(),Bb(),Bb1(),delt(),rho(),rho1();

  if ((typ[x1]==1)||(typ[x1]==5)||(typ[x1]==7))
   {
     h1=lam[x1][y1]*sig(x1,y1);
	 h2=Bb(x1,y1,h1);
	 if (h1+delt(x1)*(h2-1) != 0.0)
		 h=(h1*(1-rho1(x1))*h2)/(h1+delt(x1)*(h2-1));
	 else
		 h=0.0;
   }
  else
     h=(1-rho1(x1))/(1-(rho1(x1)-rho(x1,y1)));
  return(h);
}

double sig(b7,b8)
int b7,b8;
{
  return(2/(1+ca[b7][b8]));
}

double Bb(b9,b10,b11)
int b9,b10;
double b11;
{
  double B(),h,sig();
  
  h=(sig(b9,b10)*B(b9,b10,b11))/(1-(1-sig(b9,b10))*B(b9,b10,b11));
  return(h);
}

double Bb1(b12,b13,b14)
int b12,b13;
double b14;
{
  int v;
  double var,Bb(),sig(),delt();

  var=0;
  for (v=1;v<=r;v=v+1)
    var=var+((lam[b12][v]*sig(b12,v))/delt(b12))*Bb(b12,v,b14);
  return(var);
}

double delt(b15)
int b15;
{
  int v1;
  double var1,sig();

  var1=0;
  for (v1=1;v1<=r;v1=v1+1)
    var1=var1+lam[b15][v1]*sig(b15,v1);
  return(var1);
}

double B(b17,b18,b19)
int b17,b18;
double b19;
{
  double tau,var2;

  tau=2/(1+cs[b17][b18]);
  var2=1-tau+((tau*tau*mue[b17][b18])/(tau*mue[b17][b18]+b19));
  return(var2);
}


/*********************/
/* Ueberpruefung des */
/* Abbruchkriteriums */
/*********************/

double pruef()
{
  int n3,n4;
  static double he,**abw = (double **)0;

  if (!abw)
    abw = d2_alloc(0,n, 0,r);

  he=0;
  for (n3=1;n3<=n;n3=n3+1)
   for (n4=1;n4<=r;n4=n4+1)
    {
     abw[n3][n4]=fabs(ca[n3][n4]-spei[n3][n4]);
     if (abw[n3][n4]>=epsi)
       he=1;
    }
  return(he);
}


/**************************/
/* Ausgabe der Leistungs- */
/*       groessen         */
/**************************/

ausgabe()
{
  int i,j,l;
  double mql(),rho(),mvz1,maa1,mwz;

  printf("ermem\n");
  printf("2 %d %d\n",n,r);
  for (j=1;j<=r;j=j+1)
  {
    for (i=1;i<=n;i=i+1)
     {
	  if (lam[i][j] != 0.0)
	  {
		  mvz1=mql(i,j)/lam[i][j];
		  mwz=mvz1-1/mue[i][j];
		  if (mwz<0.0)
			mwz=0.0;
		  maa1=mwz*lam[i][j];
	  }
	  else
	  {
	      mvz1=0.0;
	      mwz=0.0;
		  maa1=0.0;
	  }

      printf("%8.6lf %8.6lf %8.6lf ",lam[i][j],e[i][j],1/mue[i][j]);
      printf("%8.6lf %8.6lf %8.6lf %8.6lf %8.6lf ",rho(i,j),mvz1,mql(i,j),mwz,maa1);
      printf("\n");
     }
    printf("\n");
  }
}
