/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tschritt5.c\t1.2\t93/05/19 15:10:23";

#include "gdaten.h"
#include <stdio.h>
#include <math.h>


schritt5()
/* ermittlung der variationskoeffizienten cA[i] des ankunftsprozesses
 * durch komposition aus teilprozessen */
{
	int l,j;
	double c1,c2,lambda1,lambda2;
	double superposition();
#	ifdef DEBUG
	printf("\n");
	printf("schritt5\n");
#	endif
	c1=c2=0;
	l = 0;
	if ( c0[knoten] > 0 )
	{	
		c1 = c0[knoten];
		lambda1 = lambda0[knoten];
#		ifdef DEBUG
		printf("c1 = %lg\n",c1);
		printf("lambda1 = %lg\n",lambda1);
#		endif
	}
	else
	{ 
		for (l=1;l<=N;l++)
			if (c[l][knoten] >0 )
			{  
				c1 = c[l][knoten];
				lambda1 = lambda[l];
#				ifdef DEBUG
				printf("c1 = %lg\n",c1);
				printf("lambda1 = %lg\n",lambda1);
#				endif
				break;
			}
	}
	if (c1 ==0)
	{  	
		fprintf(stderr,"bestimmung von c1 misslungen\n");
		printf("kuehn_versagt\n 0 0 0\n");
		exit(1);
	}
	++l;
	j = l;

	while (j<=N )
	{
		for( ;j<=N ;j++)
		{   
			c2 = 0;
			if (c[j][knoten] >0 )
			{	
				c2 =c[j][knoten];
				lambda2 =lambda[j];
#				ifdef DEBUG
				printf("c2 = %lg\n",c2);
				printf("lambda2 = %lg\n",lambda2);
#				endif
				break;
			}
		}

		if (c2 ==0)
		{
			cA_q[knoten] = c1*c1;
#			ifdef DEBUG
			printf("\n");
			printf("ergebnis der superposition:\n");
			printf("cA_q[%d] = %20.18lf\n",knoten,cA_q[knoten]);
#			endif
			return;
		}
		++j;
#		ifdef DEBUG
		printf("superposition:\n");
#		endif
		c1 = superposition(c1,c2,lambda1,lambda2);
#		ifdef DEBUG
		printf("c1 = %20.18lf\n",c1);
#		endif

		lambda1 = lambda1 +lambda2;
#		ifdef DEBUG
		printf("lambda1 =%lg\n",lambda1);
#		endif
	}
	cA_q[knoten] = c1 * c1 ;
#	ifdef DEBUG
	printf("ergebnis der superposition:\n\n");
	printf("cA_q[%d] = %20.18lf\n",knoten,cA_q[knoten]);
#	endif

	return;
}


double superposition(c1,c2,lambda1,lambda2)
double c1,c2,lambda1,lambda2;
/* komposition zweier teilprozesse ,ergebnis steht dann in C */
{
	double I1,I2,I3,I4,C_Q,C,t1,t2,E;

	/*
	if(lambda1 == 0.0)
	{
		
		return(c2);
	}
	if(lambda2 == 0.0)
	{
		
		return(c1);
	}
	*/
	t1 = 1 / lambda1;
	t2 = 1 / lambda2;
#	ifdef DEBUG
	printf("t1=%lg\t",t1);
	printf("t2=%lg\n",t2);
#	endif

	/*if (c1<=1 && c2<=1)*/
	if (c1<1 && c2<1)
	{
		double a,b,c,d;		/* a:t11,b:t12,c:t21,d:t22 */
		double e,f;	        /*hilfsvariablen   */
#		ifdef DEBUG
		printf("1.fall\n\n");
#		endif
		a = (1-c1)/lambda1;
		b = c1/lambda1;
		c = (1-c2)/lambda2;
		d = c2/lambda2;
#		ifdef DEBUG
		printf("a = %lg  ",a);
		printf("b = %lg  ",b);
		printf("c = %lg  ",c);
		printf("d = %lg\n",d);
#		endif
		if(a>c)
		{
			double h;
			h=a;
			a=c;
			c=h;
			h=b;
			b=d;
			d=h;
			h=t1;
			t1=t2;
			t2=h;
#			ifdef DEBUG
			printf("t1=%lg\t",t1);
			printf("t2=%lg\n",t2);
#			endif
		}


		e = exp( -((c-a)/b) );
		f = b*c +b*d + c*d;
#		ifdef DEBUG
		printf("e = %lg  ",e);
		printf("f = %lg  ",f);
#		endif

		I1 = a*a * (t2/2 - a/3 -b )
		    + b* t1*(t2 -2*b)
		    + b* (b*(2*c + 2*b - t2) - c*d) * e;
		I2 = ( (b*d*d) /((b+d)*(b+d))) * f * e ;
		I3 = (t1*a*a)/2 - (a*a*a)/3;
		I4 = b*b* (t1 +(-b -c   + (d/((b+d)*(b+d)) *f)  ) *e);
#		ifdef DEBUG
		printf("I1 = %lg  ",I1);
		printf("I2 = %lg  ",I2);
		printf("I3 = %lg  ",I3);
		printf("I4 = %lg\n",I4);
#		endif
	}
	/*else if  ((c1>1 && c2<=1) || (c1<=1 && c2>1))*/
	else if  ((c1>=1 && c2<1) || (c1<1 && c2>=1))
	{
		double a,b,c,d,		/*a:11,b:12,c:21,d:22 */
		q1,q2,		/*q1:q11,q2:q12       */
		x,e,f; /*hilfsvariable       */
#		ifdef DEBUG
		printf("2.fall:\n\n");
#		endif
		if(c1<1)		/*die werte muessen vertauscht 
							 *werden!! */
		{	
			double h;
#			ifdef DEBUG
			printf("vertauschen\n");
#			endif
			h = c1;
			c1 = c2;
			c2 = h;
			h =lambda1;
			lambda1 = lambda2;
			lambda2 = h;
			h = t1;
			t1 =t2;
			t2 =h;
#			ifdef DEBUG
			printf("t1=%lg\n",t1);
			printf("t2=%lg\n",t2);
#			endif
		}

		x = sqrt( (c1*c1 - 1) / (c1*c1 + 1) );
		a = (1/lambda1) * (1/(1+x));
		b = (1/lambda1) * (1/(1-x));
		c = (1-c2)/lambda2;
		d = c2/lambda2;
#		ifdef DEBUG
		printf("a = %lg  ",a);
		printf("b = %lg  ",b);
		printf("c = %lg  ",c);
		printf("d = %lg\n",d);
#		endif
		q1= 1/(2*lambda1*a);
		q2= 1/(2*lambda1*b);

		e = exp(-(c/a));
#		ifdef DEBUG
		printf("e = %lg\n",e);
#		endif
		f = exp(-(c/b));
#		ifdef DEBUG
		printf("f = %lg\n",f);
#		endif

		I1 = q1*a*a*t2* (1 - (1+ c/a)*e )
		    + q2*b*b*t2* (1 - (1+ c/b)*f)
		    - q1*a*(2*a*a - (2*a*a + 2*a*c + c*c) *e)
		    - q2*b*(2*b*b - (2*b*b + 2*b*c + c*c) *f);
		I2 = q1* 1/((a+d)*(a+d)) *a*d*d* (a*c + a*d + c*d) *e
		    + q2* 1/((b+d)*(b+d)) *b*d*d* (b*c + b*d + c*d) *f;
		I3 = q1*a*a*a* (1 - (1+c/a)*e)
		    + q2*b*b*b* (1 - (1+c/b)*f);
		I4 = q1* (1/((a+d)*(a+d))) *a*a*d *(a*c + a*d + c*d) *e
		    + q2*(1/((b+d)*(b+d))) *b*b*d *(b*c + b*d + c*d) *f;
#		ifdef DEBUG
		printf("I1 = %lg  ",I1);
		printf("I2 = %lg  ",I2);
		printf("I3 = %lg  ",I3);
		printf("I4 = %lg\n",I4);
#		endif
	}
	/*else if (c1>1 && c2>1)*/
	else if (c1>=1 && c2>=1)
	{
		double a,b,c,d,			/*a:11,b:12,c:21,d:22 */
		q11,q12,q21,q22,
		x,y;
#		ifdef DEBUG
		printf("3.fall:\n\n");
#		endif
		x = sqrt( (c1*c1 -1)/(c1*c1 +1));
		y = sqrt( (c2*c2 - 1)/(c2*c2 + 1));

		a = (1/lambda1) * (1/(1+x));
		b = (1/lambda1) * (1/(1-x));
		c = (1/lambda2) * (1/(1+y));
		d = (1/lambda2) * (1/(1-y));
#		ifdef DEBUG
		printf("a = %lg  ",a);
		printf("b = %lg  ",b);
		printf("c = %lg  ",c);
		printf("d = %lg\n",d);
#		endif

		q11 = 1/(2*lambda1*a);
		q12 = 1/(2*lambda1*b);
		q21 = 1/(2*lambda2*c);
		q22 = 1/(2*lambda2*d);
#		ifdef DEBUG
		printf("q11 = %lg  ",q11);
		printf("q12 = %lg  ",q12);
		printf("q21 = %lg  ",q21);
		printf("q22 = %lg\n",q22);
#		endif

		I1 = q11*q21* (1/(a+c)) * (a*a*c*c);
		I2 = q11*q22* (1/(a+d)) * (a*a*d*d);
		I3 = q12*q21* (1/(b+c)) * (b*b*c*c);
		I4 = q12*q22* (1/(b+d)) * (b*b*d*d);
#		ifdef DEBUG
		printf("I1 = %lg  ",I1);
		printf("I2 = %lg  ",I2);
		printf("I3 = %lg  ",I3);
		printf("I4 = %lg\n",I4);
#		endif
	}

	E = 1.0/(t1*t2) *(I1+I2+I3+I4);
	C_Q = 2.0 * (t1+t2)/(t1*t2) * E - 1.0;
	C = sqrt(C_Q);
#	ifdef DEBUG
	printf("E = %20.18lf\n",E);
	printf("C_Q = %20.18lf\n",C_Q);
	printf("C = %20.18lf\n",C);
#	endif
	return(C);
}
