/*

			     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  D-91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tmarie.c\t1.8\t94/12/13 14:52:00";

#include <stdio.h>
#include <math.h>
#include "d_mnetz.h"
#include "d_mleist.h"
#include "d_debug.h"
#include "xxalloc.h"

/********************************************************/
/*                                                      */
/*                                                      */
/*           M A R I E   -  APPROXIMATION               */
/*                                                      */
/* 23.07.90: auf dyn. Speicherverwaltung umgestellt     */
/* 22.01.92: Fehler in dyn. SpeiV. beseitigt (kirschni) */
/*                                                      */
/********************************************************/


/* G L O B A L E   D E F I N I T I O N E N              */

/* Eingabegroessen */
netzstruktur netz;      /* Die Groessen des Netzes,     */
/* eingegeben mit "pepsy",      */
/* siehe "d_mnetz.h"            */

/* Ergebnissgroessen */
leiststruktur leist;    /* Die Leisungsgroessen,        */
/* siehe "d_mleist.h"           */


/* GNORM     : beinhaltet die Normalisierungsmatrix                */
/* Gvekt     : beinhaltet den Normalisierungsvektor,               */
/*               d.h. Gvekt[i] = GNORM[NMAX][i]                    */
/* NUE[i][j] : bezeichnet die Bedienrate des Bedienzentrums i,wenn */
/*             j Auftraege im Bedienzentrum i enthalten sind       */
/* PR[i][j]  : bezeichnet die Randwahrscheinlichkeiten             */
/* F[i][j]   : Gleichgewichtswahrscheinlichkeitsmatrix             */
/* LAMDA     : enthaelt die in der momentanen Iterationsstufe      */
/*             gueltigen Ankunftsraten der einzelnen Knoten        */
/* GM        : Hilfsfkt.zur Berechnung d. Randwahrscheinlichkeiten */
/*                                                                 */

extern void signal();

/* Forward-Deklarationen					   */
void GNORIT();
void GNOBER();
void GMITER();
void LAMITER();
int QUVATEST();
void TESTWERTE();
int EPSTEST();
void BER_PR();
void VERKEHRSWERT();
void HYPOMARIE();

static double **GNORM, **NUE, **NUE_HILF, **PR;
static double **F, *Gvekt;
static double **GM, **LAMDA;
double lamda;
int err=0;
int dbg=0;

void abbruch(sig)
int sig;
{
    printf("marie_signal_%d\n 0 0 0 \n", sig);
    exit(1);
}

int main(argc,argv)
int argc;
char *argv[];
{
    /*              Sccs support from SccsLogging.c                 */      
    extern int SccsRelease;
    extern int SccsLevel;

    extern int      eingabe();
    extern int      ausgabe();
    extern int      gauss();

    /* DEKLARATIONSTEIL des Hauptprogrammes                              */

    int testvar,prue;
    int i,j,i1;
    int Ntesteps = 0;
    double Randzuw();
    static double **PWA;
    static double **PIJBE;

    void abbruch();

    for (i=1; i<12; i++)
	signal (i, abbruch);

    if ((argc > 1) && (!strcmp(argv[1], "-V")))	/* kirschni 22.11.1991 */
    {
	fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
	exit(0);
    }

    if ((argc > 1) && (!strcmp(argv[1], "-d")))    
    {
	dbg = 100;
    }


    /* Lesen der Eingabedaten von stdin  */
    eingabe();

    leist.vzeit		= d1_alloc(0, netz.knot-1);
    leist.lambda        = d1_alloc(0, netz.knot-1);
    leist.rho           = d1_alloc(0, netz.knot-1);
    leist.fuell         = d1_alloc(0, netz.knot-1);
    GNORM               = d2_alloc(0,netz.knot-1, 0, netz.auftr);
    NUE                 = d2_alloc(0,netz.knot-1, 0, netz.auftr);
    NUE_HILF            = d2_alloc(0,netz.knot-1, 0, netz.auftr);
    PR                  = d2_alloc(0,netz.knot-1, 0, netz.auftr);
    F                   = d2_alloc(0,netz.knot-1, 0, netz.auftr);
    Gvekt               = d1_alloc(0, netz.auftr);
    GM                  = d2_alloc(0,netz.knot-1, 0, netz.auftr);
    LAMDA               = d2_alloc(0,netz.knot-1, 0, netz.auftr);
    PWA                 = d2_alloc(0,netz.knot-1, 0, netz.auftr);
    PIJBE               = d2_alloc(0,netz.knot-1, 0, netz.knot-1);

    if (dbg)
    {
	dbgtext(50, "main(): Pruefwerte fuer MARIEAPPROX:\n\nBesuchshaeufigkeiten:\n");

	for (j=1; j<=netz.knot; j++)
	    dbginfo2(50, "main(): netz.besh[%d] = %f\n",j,netz.besh[j-1]);
    }

    /* Vorbesetzen der Testvar. fuer den Abbruch des Epsilon- */
    /* testes und der Anzahl i1 der Iterationsdurchlaeufe     */
    testvar = 0;
    i1      = 0;

    /* Vorbesetzen der lastabhaengigen Bedienraten   NUE[][]  */
	
    for(i=0;i<netz.knot;i++)
    {
	if (netz.bede[i] == 0)
	    netz.bede[i] = netz.auftr;

	for (j=0; j<=netz.auftr;j++)
	    if (j <= netz.bede[i])
	    {        
		NUE[i][j]       = (double)j * netz.bedr[i];
		NUE_HILF[i][j]  = (double)j * netz.bedr[i];
	    }
	    else 
	    {
		NUE[i][j]       = (double)netz.bede[i] * netz.bedr[i];
		NUE_HILF[i][j]  = (double)netz.bede[i] * netz.bedr[i];
	    }
    }


    /* Beginn der Iteration */
    Ntesteps = 0;

    do 
    {
	i1++;

	/* neues F[i][j] durch die h-te Iterationsstufe berechnen*/
	GNORIT();

	if ( err == 2 )
	{
	    printf("marie_numerik\n 0 0 0\n");
	    exit(12);
	}

	/* neues GNORM[i][j] und neuer Gvekt werden berechnet     */
	GNOBER();

	/*  Bestimmen der neuen Hilfsmatrix GM                    */
	GMITER();

	if ( err == 2 )
	{
	    printf("marie_numerik\n 0 0 0\n");
	    exit(12);
	}

	/* Bestimmen der neuen Ankunftsraten                      */
	LAMITER();

	if ( err == 2 )
	{
	    printf("marie_numerik\n 0 0 0\n");
	    exit(12);
	}

	/*             Fallunterscheidungen fuer                  */
	/*        verschiedene Variationskoeffizienten            */

	prue=QUVATEST(PWA);

	/*  Pruefkonstante prue fuer unerlaubte Varkoeffizienten  */
	if (prue != 0)
	{
	    printf("marie_varkoeff\n 0 0 0\n");
	    exit(12);
	}

	/*Ausschreiben von Pruefgroessen auf das File 'mtestwerte'*/
	TESTWERTE(i1,PWA);

	/* Ueberpruefung , ob die Iteration beendet werden kann,  */
	/* d.h. die Durchsatz und Auftragsinvarianz liegen        */
	/* innerhalb einer v.Benutzer vorgegebenen Fehlertoleranz */

	if (testvar ==  0 )
	{
	    testvar = EPSTEST(PWA);
	    if (err == 2)
	    {
		printf("marie_numerik\n 0 0 0\n");
		exit(12);
	    }
	}
	else
	{
	    Ntesteps = 1;
	}
    }
    while (!Ntesteps);
    /* Ende der Iteration */


    /*  Berechnung der Randzustandswahrscheinlichkeiten      */
    BER_PR(PWA);

    if ( err == 2 )
    {
	printf("marie_numerik\n 0 0 0\n");
	exit(12);
    }

    if (dbg)
    {
	dbgtext(50, "main(): Randzustandswahrsch. PR[][]:\n");

	for (i=0; i<netz.knot; i++)
	{
	    for (j=0; j<=netz.auftr; j++)
		fprintf(stderr, " %f \n",PR[i][j]);
	    fprintf(stderr, "\n");
	}
    }

    /*  Bestimmung der approximierten Auslastungen und der  */
    /*  gewuenschten Zustandswahrscheinlichkeit P(n1,n2,...)*/

    if ( Gvekt[netz.auftr] == 0 )
    {
	dbgtext(50, "main(): DIVISION DURCH NULL bei Berechnung der Zustandswahrscheinlichkeit\n");

	printf("marie_numerik\n 0 0 0\n");
	exit(12);
    }


    /*    Berechnung der Verkehrswerte                      */
    VERKEHRSWERT();

    /* Ausgabe der ermittelten Leistungsgroessen */
    ausgabe();
    exit(0);
}


/*******************************************************/
/*                                                     */
/*      Berechnung neuer Funktionswerte F[i][j]        */


void GNORIT()
{
    int i,j,k;
    double help;
    double p;

    for(i=0;i<netz.knot;i++)
	for(j=0;j<=netz.auftr;j++)
	{
	    help =1;

	    for (k=1;k<=j;k++)
		help = help * NUE[i][k];

	    p = netz.besh[i];

	    if ( help == 0 )
	    {
		dbgtext(50, "GNORIT(): DIVISION DURCH NULL !!\n");

		err = 2;
		goto GNOEND;
	    }

	    F[i][j] = pow(p,(double)j)/help;
	}

 GNOEND:  
    ;

} /* Ende der procedure GNORIT */



/*********************************************************/
/*                                                       */
/*    Berechnung der Normalisierungskonstanten G ,       */
/*    der Hilfsmatrix GNORM und des Vektors Gvekt        */


void GNOBER()
{
    int k,j,i;
    double help;
    static double *Summe = NULL;

    if (!Summe)
	Summe = d1_alloc(0, netz.auftr);

    for(i=0;i<=netz.auftr;i++)
	GNORM[0][i] = F[0][i];

    for (i=1;i<netz.knot;i++)
    {
	GNORM[i][0] =F[i][0] * GNORM[i-1][0];

	for (k=1;k<= netz.auftr;k++)
	{
	    Summe[0] =F[i][0] * GNORM[i-1][k];

	    for (j=1;j<=k;j++)
	    {
		help = F[i][j] *GNORM[i-1][k-j];
		Summe[j] = help +Summe[j-1];
	    }

	    GNORM[i][k] = Summe[k];
	}
    }

    for (k=0;k<=netz.auftr;k++)
	Gvekt[k]  = GNORM[netz.knot-1][k];

} /* Ende der procedure GNOBER */



/*********************************************************/
/*                                                       */
/*        Berechnung der Hilfsgroesse GM[i][j]           */
/*      Fuer die Bestimmung neuer LAMDA[][] und  fuer    */
/*      die  Randzustandswahrscheinlichkeiten noetig     */


void GMITER()
{
    int j,i,k;
    static double **HELP = NULL, **SUMME = NULL;

    if (!HELP)
    {
	HELP = d2_alloc(0, netz.knot-1, 0, netz.auftr);
	SUMME = d2_alloc(0, netz.knot-1, 0, netz.auftr);
    }

    for (i=0; i<netz.knot; i++)
    {
	if ( F[i][0] == 0 )
	{
	    dbgtext(50, "GMITER(): DIVISION DURCH NULL\n");

	    err = 2;
	    goto GMIEND;
	}
	GM[i][0] = Gvekt[0] / F[i][0];
    }

    for (i=0; i<netz.knot; i++)
	for(j=1;j<=netz.auftr ;j++)
	{
	    for(k=1; k<=j;k++)
	    {
		if (k == 1)
		    SUMME[i][k-1] =0;

		HELP[i][k] = F[i][k] * GM[i][j-k];
		SUMME[i][k] =HELP[i][k] + SUMME[i][k-1];
	    }


	    GM[i][j] = (Gvekt[j] - SUMME[i][j]) / F[i][0];
	}

 GMIEND: 
    ;
} /* Ende der GMITER-Prozedur */



/**********************************************************/
/*                                                        */
/*     Berechnung der lastabhaengigen Ankunftsraten       */
/*                     LAMDA[i][j]                        */
/**********************************************************/

void LAMITER()
{
    int i,j;
    double p,q;

    for(i=0 ;i<netz.knot;i++)
    {
	for(j=0 ;j<=netz.auftr-1; j++)
	{
	    p = GM[i][netz.auftr-j-1] ;
	    q = GM[i][netz.auftr-j];

	    if ( q == 0 )
	    {
		dbgtext(50, "LAMITER(): DIVISION DURCH NULL !!\n");

		err = 2;
		goto LAMEND;
	    }
	    LAMDA[i][j] = netz.besh[i] * p / q ;
	}

	LAMDA[i][netz.auftr] = 0;
    }

 LAMEND:
    ;

} /* Ende procedure LAMITER */



/**************************************************************/
/*                                                            */
/*  Unterschiedliche Berechnungsverfahren fuer lastabhaengige */
/*  Bedienraten  NUE  und Zustandswahrscheinlichkeiten  PWA   */
/*  aufgrund unterschiedlicher Variationskoeffizienten :      */
/*   --> Exponentielle Vert.,hypoexp.Vert.,hyperexp.Vert.     */

int QUVATEST(PWA)
double **PWA;
{
    int i,j,k,p;
    double help, help1, help2;
    static double *Phi, **MUEDACH1, **MUEDACH2;

    if (!Phi)
    {
	Phi = d1_alloc(0, netz.knot-1);
	MUEDACH1 = d2_alloc(0, netz.knot-1, 0, netz.auftr);
	MUEDACH2 = d2_alloc(0, netz.knot-1, 0, netz.auftr);
    }

    p = 0;
    for (i=0; i<netz.knot; i++)
    {

	/*Ueberprufungskonstante,ob eine der Bedingungen erfuellt wird*/

	if (( (1.0+netz.epsi) >= netz.quvari[i] && (1.0-netz.epsi) <= netz.quvari[i] ) ||
	    (netz.bedstr[i] == 2) || (netz.bedstr[i] == 3) || (netz.bedstr[i] == 4))
	{
	    dbginfo(50, "QUVATEST(): Knoten %d, Fall 1\n",i);

	    /* keine Aenderung der lastabhaengigen Bedienraten NUE[i][j]  */

	    /* Berechnung der Zustandswahrsheinlichkeiten  PWA[i][j]      */

	    /* PWA[i][0] */
	    help =0;
	    for (j=1; j<=netz.auftr; j++)
	    {
		help1 =1;
		for (k=0; k<=j-1; k++ )
		    help1 = help1 * LAMDA[i][k] / NUE[i][k+1];

		help += help1;
	    }
	    PWA[i][0] = 1 / (help + 1);


	    /* PWA[i][n] */
	    for(j=1; j<=netz.auftr; j++)
	    {
		help = 1;
		for (k=0; k<=j-1; k++)
		    help = help * LAMDA[i][k] / NUE[i][k+1];

		PWA[i][j] =  PWA[i][0] * help;
	    }
	    continue;
	}


	if((netz.quvari[i] >= 0.5) && (netz.bedstr[i] == 1))
	{
	    dbginfo(50, "QUVATEST(): Knoten %d, Fall 2\n",i);

	    for (j=1; j<=netz.auftr; j++)
	    {
		MUEDACH1[i][j] = 2 * NUE_HILF[i][j];
		MUEDACH2[i][j] = NUE_HILF[i][j] / netz.quvari[i];
	    }

	    Phi[i]	= 1 / (2 * netz.quvari[i]);


	    /* Berechnen der lastabhaengigen Bedienraten NUE[i][j]   */


	    NUE[i][1] = ( LAMDA[i][1] * MUEDACH1[i][1] * (1 - Phi[i]) +
			 MUEDACH1[i][1] * MUEDACH2[i][1] ) /

			     ( MUEDACH2[i][1] + Phi[i] * MUEDACH1[i][1] +
			      LAMDA[i][1] );


	    for (j=2; j<=netz.auftr; j++)
	    {
		help    = ( LAMDA[i][j] * MUEDACH1[i][j] * (1 - Phi[i]) +
			   MUEDACH1[i][j] * MUEDACH2[i][j] ) *

			       ( MUEDACH2[i][j-1] - (1 - Phi[i]) * MUEDACH1[i][j-1] );

		help1   = ( LAMDA[i][j] + MUEDACH2[i][j] + Phi[i] * MUEDACH1[i][j] ) *
		    ( MUEDACH2[i][j-1] - (1 - Phi[i]) * MUEDACH1[i][j-1] );

		help2   = ( NUE[i][j-1] - (1 - Phi[i]) * MUEDACH1[i][j-1] ) *
		    ( MUEDACH2[i][j] - (1 - Phi[i]) * MUEDACH1[i][j] );

		NUE[i][j] = help / (help1 - help2);
	    }

	    /* Berechnung der Zustandswahrsheinlichkeiten  PWA[i][j]    */

	    /* PWA[i][0] */
	    help = 0;
	    for (j=1; j<=netz.auftr; j++)
	    {
		help1 =1;
		for (k=0; k<=j-1; k++)
		    help1 = help1 * LAMDA[i][k] / NUE[i][k+1];

		help += help1;
	    }

	    PWA[i][0] = 1 / (help + 1);


	    /* PWA[i][n] */
	    for(j=1; j<=netz.auftr; j++)
	    {
		help = 1;
		for (k=0; k<=j-1; k++)
		    help = help * LAMDA[i][k] / NUE[i][k+1];

		PWA[i][j] =  PWA[i][0] * help;
	    }
	    continue;
	}


	if ((netz.quvari[i] >= 0) && (netz.bedstr[i] == 1))
	{
	    dbginfo(50, "QUVATEST(): Knoten %d, Fall 3\n",i);

	    /* Bestimmen  der k-stufigen Coxverteilung, Berechnen    */
	    /* der Zustandsw. PWA  ueber die procedure HYPOMARIE     */

	    HYPOMARIE(i,PWA);


	    /* Berechnen der lastabhaengigen Bedienraten NUE[i][j]   */
	    for (j=1; j<=netz.auftr; j++)
		NUE[i][j] = LAMDA[i][j-1] * PWA[i][j-1] / PWA[i][j];

	    continue;
	}

	dbginfo4(50, 
		 "QUVATEST(): falsche Angaben von Bedienzeiten oder Bedienstrategien bei Knoten %d bede=%d, bedstr=%d, cquad=%f\n",
		 i+1,
		 netz.bede[i],
		 netz.bedstr[i],
		 netz.quvari[i]);

	p  += 1;		/* Fehlerausgang */
	continue;
    }

    return(p);

} /* Ende der procedure QUVATEST */



/**********************************************************/
/*                                                        */
/*   Berechnung der Gleichgewichtswahrscheinlichkeiten    */
/*   mit rekursivem Loesungsverfahren fuer lamda(n)/G/m/N */
/*   Wartesysteme mit hypoexponentiell verteilten         */
/*   Bedienzeiten                                         */


void HYPOMARIE(i,PWA)
int i;
double **PWA;
{

    /* Erklaerungen der Vereinbarungen                            */
    /*                                                            */
    /* Mueda[]   : enthaelt die mittlere Bedienrate der verallge- */
    /*             meinerten Erlang Einheit des Knotens i         */
    /*                                                            */
    /* Phi[]     : Wahrsch.,dass Auftrag nach Bedienung durch die */
    /*             erste Markov- Stufe die Erlang Einheit verlasst*/
    /*                                                            */
    /* Kstuf[]   : Anzahl der Markovstufen im Knoten i            */
    /*                                                            */
    /* PZUWA[n][k]:Zustandswahrscheinlichkeiten des Knotens i,dass*/
    /*             n Auftraege im System und der sich in Bedienung*/
    /*             befindliche Auftrag in der k-ten exponentiellen*/
    /*             S*/

    int y,j,k1,k,l,n;
    double help,help1,hee;
    static double *Mueda, *Phi, **EHELP;
    static double *Kleing, *Kleinb, **KLEINF, **PZUWA;
    static double *Esumme;
    static int *Kstuf;

    if (!Mueda)
    {
	Mueda	= d1_alloc(0, netz.knot-1);
	Phi	= d1_alloc(0, netz.knot-1);
	EHELP	= d2_alloc(0, netz.auftr, 0, netz.auftr);
	Kleing	= d1_alloc(0, netz.auftr);
	Kleinb	= d1_alloc(0, netz.auftr);
	KLEINF	= d2_alloc(0, netz.auftr, 0, netz.auftr);
	PZUWA	= d2_alloc(0, netz.auftr, 0, netz.auftr);
	Esumme	= d1_alloc(0, netz.auftr);
	Kstuf	= i1_alloc(0, netz.knot-1);
    }

    /* Bestimmen von der Anzahl der exp. Stufen bei der Coxvert. */

    help	= 1/netz.quvari[i];
    Kstuf[i]	= help;

    if (Kstuf[i]  <= help)
	Kstuf[i] = Kstuf[i] + 1;

    k1   = Kstuf[i];

    /*   Mueda[i] und Phi[i] berechnen mit bekanntem Kstuf[i]    */
    /*   d.h. Bedienraten der Coxvert. und die Uebergangswahr-   */
    /*   scheinlichkeit von der ersten zur zweiten exp. Stufe    */

    help = 2 * Kstuf[i] * netz.quvari[i] +(Kstuf[i] - 2);
    hee =sqrt(Kstuf[i]*Kstuf[i]+4-4*Kstuf[i]*netz.quvari[i]);
    help1= (2* ( Kstuf[i] -1)*(netz.quvari[i] + 1));

    Phi[i] = (help -hee) / help1;

    help = Kstuf[i] *(1-Phi[i]);

    Mueda[i] = (Phi[i] + help ) * netz.bedr[i];

    dbginfo4(50,
	     "HYPOMARIE(): Knoten %d: Coxverteilung mit %d Stufen, Muedach=%f, b=Phi=%f\n",
	     i,
	     Kstuf[i],
	     Mueda[i],
	     Phi[i]);


    /*  Berechnung der Zustandswahrscheinlichkeiten p(n,k)       */
    /*  d.h. im Knoten i sind n Auftraege in der k-ten exp.Stufe */
    /*    Mueda[] ,  Phi[]  und  Kstuf[i] sind schon  bekannt    */

    for (j=1; j<=Kstuf[i];j++)
	EHELP[0][j] = 0;

    for( y=0; y <= netz.auftr ; y++)
    {
	for( j=0; j <= netz.auftr ; j++)
	{
	    if (j == y)
		EHELP[j][1] = 1;
	    else
		EHELP[j][1] = 0;
	}

	for (j=1; j<=netz.auftr; j++)
	{
	    help = (1 - Phi[i]) *Mueda[i] * EHELP[j][1];
	    hee  = Mueda[i] + LAMDA[i][j];

	    EHELP[j][2]=help /hee +LAMDA[i][j-1] *EHELP[j-1][2] /hee;
	}

	for (k=3; k<=Kstuf[i]; k++)
	    for (j=1; j<= netz.auftr; j++)
	    {
		help = Mueda[i] *EHELP[j][k-1]/(Mueda[i] +LAMDA[i][j]);
		hee =LAMDA[i][j-1]*EHELP[j-1][k]/(Mueda[i]+LAMDA[i][j]);
		EHELP[j][k] = help + hee;
	    }

	help =0;
	for(n=0; n<=netz.auftr; n++)
	    for (k=1; k<=k1; k++)
		help = help +EHELP[n][k];

	Esumme[y] = help;


	/*Berechnung der Hilfsmatrix " f " zum Bestimmen von p(n,k)*/

	if (y == 0)
	{
	    /* Berechnung von f[1][y] */

	    help = EHELP[0][1] * LAMDA[i][0];
	    help1 = EHELP[1][1] *Phi[i] * Mueda[i];
	    hee  = EHELP[1][k1] * Mueda[i];

	    KLEINF[1][0] = help - help1 - hee;
	}

	/* Berechnung von f[n][y] */

	for(j=2; j<=netz.auftr; j++)
	{
	    help = EHELP[j-1][1] *(Mueda[i] + LAMDA[i][j-1]);
	    help1= EHELP[j][1] * Phi[i] * Mueda[i] + 
		EHELP[j][k1]*Mueda[i];
	    hee  = EHELP[j-2][1] * LAMDA[i][j-2];

	    KLEINF[j][y] = help -help1 -hee;
	}

	/*Berechnung des Hilfsvektors " g "zum Bestimmen der p(n,k)*/

	if ( y > 0 )
	    Kleing[y] =EHELP[y][1] *Phi[i] *Mueda[i] +
		EHELP[y][k1] *Mueda[i];

    }

    /*   Berechnung des Vektors " b "zum Bestimmen der p(n,k)  */

    Kleinb[0] = 1.0;
    for(j=1; j<= netz.auftr ; j++)
    {
	help =0;
	for (l=0; l<=j-1 ; l++)
	    help =Kleinb[l] *KLEINF[j][l] *(1/Kleing[j])+help;

	Kleinb[j] = help;
    }


    /*  Bestimmen von PZUWA[0][1]  = p(0,1)                     */

    help =0;
    for( y=0; y<=netz.auftr; y++)
	help = help + Kleinb[y] * Esumme[y];

    PZUWA[0][1] = 1/help;


    /*..........  PZUWA[n][1]                                   */

    for (j=1; j<=netz.auftr; j++)
	PZUWA[j][1] = Kleinb[j] * PZUWA[0][1];


    /*..........  PZUWA[0][n]                                   */

    for(j=2; j<= k1; j++)
	PZUWA[0][j] = 0;


    /*..........  PZUWA[n][2]                                   */

    for(j=1; j<= netz.auftr;j++)
    {
	help = Mueda[i] + LAMDA[i][j];
	help1 = (1 - Phi[i]) * Mueda[i] * PZUWA[j][1];
	hee  = LAMDA[i][j-1] * PZUWA[j-1][2];

	PZUWA[j][2] = help1 / help + hee / help;
    }


    /*..........  PZUWA[n][n]                                   */

    for(j=1; j<=netz.auftr; j++)
	for(k=3; k<=k1; k++)
	{
	    help = Mueda[i] + LAMDA[i][j];
	    help1 = Mueda[i] * PZUWA[j][k-1];
	    hee  = LAMDA[i][j-1] * PZUWA[j-1][k];

	    PZUWA[j][k] = help1 / help + hee / help;
	}


    /* Zustandswahrscheinlichkeiten fur P i (n)  = PWA[i][n]   */

    for(j=0; j<=netz.auftr; j++)
    {
	help=0;
	for ( k=1; k<= k1; k++)
	    help = help + PZUWA[j][k];

	PWA[i][j] = help;
    }

    /* Die Werte von PWA enthalten die P i (n) und werden  */
    /* an das aufrufende Prozedur zurueckgeliefert         */

} /* Ende der procedure HYPOMARIE */



/***********************************************************/
/*                                                         */
/*  Ausschreiben praegnanter Zwischenergebnisse  der       */
/*  einzelnen Iterationslaeufe auf die Datei 'mtestwerte'  */


void TESTWERTE(i1,PWA)
int i1;
double **PWA;
{
    int i,j;

    if (dbg)
    {
	fprintf(stderr, "\n %d.Iterationsdurchlauf: \n",i1);
	fprintf(stderr, "Funktionswerte F[][] :\n");
	for (j=0; j<netz.knot; j++)
	{
	    for (i=1; i<=netz.auftr; i++)
		fprintf(stderr, " %f ",F[j][i]);
	    fprintf(stderr, "\n");
	}

	fprintf(stderr, "\nNormalisierungsvektor :\n");
	for (i=0; i<=netz.auftr; i++)
	    fprintf(stderr, " %f \n",Gvekt[i]);

	fprintf(stderr, "\nLastabh.Ankunftsraten Lamda[][]\n");
	for (j=0; j<netz.knot; j++)
	{
	    for (i=0; i<=netz.auftr-1; i++)
		fprintf(stderr, " %f ",LAMDA[j][i]);
	    fprintf(stderr, "\n");
	}

	fprintf(stderr, "\n Zustandswahrsch. PWA[][] :\n");
	for (j=0; j<netz.knot; j++)
	{
	    for (i=0; i<=netz.auftr; i++)
		fprintf(stderr, " %f ",PWA[j][i]);
	    fprintf(stderr, "\n");
	}

	fprintf(stderr, "\n Lastabh. Bedienraten NUE[][] :\n");
	for (j=0; j<netz.knot; j++)
	{
	    for (i=1; i<=netz.auftr; i++)
		fprintf(stderr, " %f ",NUE[j][i]);
	    fprintf(stderr, "\n");
	}
    }
} /* Ende procedure TESTWERTE */



/**********************************************************/
/*                                                        */
/*           Auftrags- und Invarianztest                  */
/*    Ueberpruefung des Abbruchkriteriums der Iteration   */


int EPSTEST(PWA)
double **PWA;
{
    int i,j,testvar;
    double uebpru1,uebpru2,help,help1;
    static double *Ktest;

    if (!Ktest)
	Ktest = d1_alloc(0, netz.knot-1);

    testvar = 0;
    help =0;

    for(i=0;i<netz.knot;i++)
	for (j=0;j<=netz.auftr;j++)
	    help = help + j * PWA[i][j];

    uebpru1 = ((double)netz.auftr - (double)help)/(double)netz.auftr;

    if(uebpru1<0)
	uebpru1 = (-uebpru1);

    dbginfo(50, "EPSTEST(): Abweichung beim EPSTEST 1 =%f\n",uebpru1);

    if(uebpru1 > netz.epsi)
	goto   Ausgang;		/*  ---> h+1 Iterationslauf   */

    for(i=0;i<netz.knot;i++)
    {
	help =0;
	for (j=1;j<=netz.auftr;j++)
	{
	    if ( netz.besh[i] == 0 )
	    {
		dbginfo(50, "EPSTEST(): DIVISION DURCH NULL in Knoten %d\n", i);
		err = 2;
		goto Ausgang;
	    }
	    help =help + PWA[i][j] * NUE[i][j] / netz.besh[i];
	}
	Ktest[i] = help;
    }

    help1 =0;
    for(j=0;j<netz.knot;j++)
	help1 =help1 + Ktest[j];

    help1 = help1/netz.knot;

    for(j=0;j<netz.knot;j++)
    {
	if (help1 ==0)
	{
	    dbgtext(50, "EPSTEST(): DIVISION DURCH NULL\n");

	    err = 2;
	    goto Ausgang;
	}
	uebpru2 = (Ktest[j] - help1) / help1;
	if (uebpru2 < 0)
	    uebpru2 = (-uebpru2);

	dbginfo2(50, "EPSTEST(): Abweichung fuer Knoten %d = %f\n", j, uebpru2);

	if (uebpru2 > netz.epsi)
	    goto  Ausgang;	/*  ---> h+1 Iterationsstufe  */

    }

    /*   Beenden der Iterationen nach dem naechsten Durchlauf */
    /*      Werte von (h+1) Iteration werden uebernommen      */

    testvar = 1;

 Ausgang: 
    ;

    return(testvar);

} /* Ende procedure EPSTEST */



/*********************************************************/
/*                                                       */
/*    Berechnung der Randzustandswahrscheinlichkeiten    */


void BER_PR()
{
    int j,i;

    for ( i = 0; i < netz.knot; i++)
    {
	for (j = 0; j <= netz.auftr; j++)
	{
	    if  (Gvekt[netz.auftr] == 0.0 )
	    {
		dbgtext(50, "BER_PR(): DIVISION DURCH NULL !!\n");

		err = 2;
		goto BEREND;
	    }
	    PR[i][j] = F[i][j] / Gvekt[netz.auftr] * GM[i][netz.auftr-j];
	}
    }

 BEREND: 
    ;
} /* Ende der procedure BER_PR */



/*******************************************************/
/*                                                     */
/* Berechnung der Verkehrswerte bzw. Leistungsgroessen */


void VERKEHRSWERT()
{
    int i,j;

    for (i = 0; i < netz.knot; i++)
    {

	/* Berechnen des Durchsatzes leist.lambda durch die jew. Knoten   */
	/* und der mittleren Anzahl von Auftraegen  leist.fuell[i]        */

	leist.lambda[i] = 0.0;
	for (j=1; j<=netz.auftr; j++)
	    leist.lambda[i] += PR[i][j] * NUE[i][j];

	leist.fuell[i] = 0.0;
	for (j=1; j<=netz.auftr; j++)
	    leist.fuell[i] += (double)j * PR[i][j];


	/* Berechnen der Auslastungen leist.rho[i] des Knotens i ,  */
	/* der jeweiligen Verweilzeit und der Zykluszeit            */

	if(netz.bedstr[i] != 3) /* no queueing */
	{
	    leist.rho[i] = leist.lambda[i] / ((double)netz.bede[i] * netz.bedr[i] );
	    leist.vzeit[i] = leist.fuell[i] / leist.lambda[i];
	}
	else
	{
	    leist.rho[i] = 0.0;
	    leist.vzeit[i] = 1.0 / netz.bedr[i];
	}
    }

}
/* Ende procedure VERKEHRSWERT */
