/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tmem_ankunfts.c\t1.2\t92/03/27 11:48:57";

/* ******************* Datei mem_ankunfts.c ********************* */
/* Inhalt                                                         */
/*       gauss_g()              gauss_o()                         */
/*       arate_o()              arate_g()                         */
/*       arate()                                                  */
/* ************************************************************** */

/* *****************************************************************
**                                                                **
**                          Include Dateien                       **
**                                                                **
***************************************************************** */

#include <stdio.h>
#include "mem_netz.h"
#include "mem_def.h"

/* *****************************************************************
**                                                                **
**                          Definitionen                          **
**                                                                **
***************************************************************** */

#define ERROR(a,b)	fprintf(stderr,a,b);

/* *****************************************************************
**                                                                **
**                          Externe Variablen                     **
**                                                                **
***************************************************************** */

extern double **wa;            /* Hilfsvariable, damit Uebergangs-
				  matrix _uebwkt nich veraendert wird */
extern double *bvek;           /* Rechte Seite eines inhomogenen
				  Gleichungssystems */


/* *****************************************************************
**                                                                **
**                        gauss_g                                 **
**                  aufgerufen von : arate_g()                    **
**                                                                **
***************************************************************** */
/*  Gauss'scher Algorithmus zum loesen eines homogenen
    linearen Gleichungssystems mit Spaltenpivotsuche
    die uebergebene Matrix wird durch den Algorithmus zerstoert
    ist das System nicht eindeutig loesbar, so werden bestimmte
    Komponenten mit 1 besetzt
*/

int gauss_g(A,x,b,dim,eps)   /*  loesen von A*x = 0    */
int dim;
double eps;
double x[], b[];
double **A;
{
	int index, andex, ondex, pivot;
	double help1, help2, help3, help4, help5;

	/* obere Dreiecksmatrix */

	for(index=0;index<dim;index++)
	{
		/* bestimme Pivot_Element */

		help1 = 0.0;
		pivot = index;

		for(andex=index;andex<dim;andex++)
		{
			help2 = A[andex][index];

			if(help2<0.0)
				help2 *= -1.0;

			if(help2>help1)
			{
				pivot= andex;
				help1=help2;
			}
		}

		if (help1 > eps)
		{
			/* vertauschen der Zeilen 'pivot' und 'index' */

			for(andex=index;andex<dim;andex++)
			{
				help3 = A[pivot][andex];
				A[pivot][andex]=A[index][andex];
				A[index][andex]=help3;
			}

			/* elementarer Berechnungsschritt */

			help4=A[index][index];

			for(andex=index+1;andex<dim;andex++)
			{
				/* Multiplikator */

				help5=A[andex][index]/help4;
				for(ondex=index;ondex<dim;ondex++)
					A[andex][ondex] -= help5*A[index][ondex];
				b[andex] -= help5*b[index];
			}
		}/* ENDE if(help1) */
	} /* ENDE for(index) */

	/* jetzt sollte die Matrix Dreiecksform haben */

	/* Rueckwaertsaufloesung */

	if (abs(A[dim-1][dim-1]) > eps)
		x[dim-1] = b[dim-1]/A[dim-1][dim-1];
	else
		x[dim-1] = 1; /* willkuerliche Festlegung */
	for(andex=dim-2;andex>=0;andex--)
	{
		if (abs(A[andex][andex]) > eps)
		{
			x[andex]=b[andex];
			for(ondex=andex+1;ondex<dim;ondex++)
				x[andex] -= A[andex][ondex]*x[ondex];
			x[andex] /= A[andex][andex];
		}
		else
			x[andex] = 1; /* willkuerlich */
	}

	/* jetzt sollte in x die Loesung stehen */

	return(0);
} /* ENDE gauss_g */

/* ******************************************************************
**                                                                 **
**                          gauss_o                                **
**                  aufgerufen von : arate_o() , iter_newton       **
**                                                                 **
****************************************************************** */

/*  Gauss'scher Algorithmus zum Loesen eines inhomogenen linearen 
    Gleichungssystems mit Spaltenpivotsuche
    die uebergebene Matrix wird durch den Algorithmus zerstoert
*/

int gauss_o(A,x,b,dim,eps)   /*  loesen von A*x = b    */
int dim;
double eps;
double x[], b[];
double **A;
{
	int index, andex, ondex, pivot;
	double help1, help2, help3, help4, help5;


	/* jetzt geht's los
	   herbeischaffen der Dreiecksform
	*/

	for(index=0;index<dim;index++)
	{
		/* bestimme Pivot_Element */

		help1 = 0.0;
		pivot = index;

		for(andex=index;andex<dim;andex++)
		{
			help2 = A[andex][index];

			if(help2<0.0)
				help2 *= -1.0;

			if(help2>help1)
			{
				pivot= andex;
				help1=help2;
			}
		}

		if(help1<eps)
		{
			ERROR("Matrix singulaer\n"," ");
			ERROR("Epsilon = %g\n",eps);
			return(-1);
		}
		/* vertauschen der Zeilen 'pivot' und 'index' */

		for(andex=index;andex<dim;andex++)
		{
			help3 = A[pivot][andex];
			A[pivot][andex]=A[index][andex];
			A[index][andex]=help3;
		}
		help3 = b[pivot];
		b[pivot]=b[index];
		b[index]=help3;

		/* elementarer Berechnungsschritt */

		help4=A[index][index];

		for(andex=index+1;andex<dim;andex++)
		{
			/* Multiplikator */

			help5=A[andex][index]/help4;
			for(ondex=index;ondex<dim;ondex++)
				A[andex][ondex] -= help5*A[index][ondex];
			b[andex] -= help5*b[index];
		}
	}

	/* jetzt sollte die Matrix Dreiecksform haben */

	/* Rueckwaertsaufloesung */

	x[dim-1] = b[dim-1]/A[dim-1][dim-1];
	for(andex=dim-2;andex>=0;andex--)
	{
		x[andex]=b[andex];
		for(ondex=andex+1;ondex<dim;ondex++)
			x[andex] -= A[andex][ondex]*x[ondex];
		x[andex] /= A[andex][andex];
	}

	/* jetzt sollte in x die Loesung stehen */

	return(0);
} /* ENDE gauss_o */

/* ******************************************************************
**                                                                 **
**                          arate_o                                **
**                 aufgerufen von : arate()                        **
**                                                                 **
****************************************************************** */

arate_o()
{                            /* Berechnung Ankunftsraten des offenen
				Netzes */

	int    l;                   /* Zaehlvariable    l: Knoten */
	int    zeile,spalte;        /* Zaehlvariablen */
	int    klasse;              /* Zaehlvariable  klasse : Klasse */
	int    gauss_o();             /* Berechnung eines inhomogenen Systems
					A * X = b */


	for(klasse=0;klasse<netz.klasse;klasse++)
	{
		for(zeile=0;zeile<netz.knoten;zeile++)
			for(spalte=0;spalte<netz.knoten;spalte++)
				wa[zeile][spalte]=netz.uebwkt[klasse][spalte][zeile];

		/* die Matrix wa verhindert, dass die Originalmatrix 
                   veraendert wird,
                   wa enthaelt die Transponierte von netz.uebwkt
                */


		for(zeile=0;zeile<netz.knoten;zeile++)
			--wa[zeile][zeile];

		/* dekrementieren der Diagonale */

		for(l=0;l<netz.knoten;l++)
			bvek[l] = (-1) * netz.e_ankunftsrate[klasse][l];

		/* Belegung der rechten Seite, fuer 'bvek' wird vorher 
		      Speicherplatz belegt */

		if (gauss_o(wa,netz.ankunftsrate[klasse],bvek,netz.knoten,0.001)==-1)
		{
			fprintf(stderr, "Bestimmung des Flussgleichgewichts nicht moeglich\n");
			printf("mem_gleichgew\n 0 0 0\n");
			exit(1);
		}
	}  /* ENDE for(klasse) */

} /* ENDE arate_o */

/* ******************************************************************
**                                                                 **
**                          arate_g                                **
**                 aufgerufen von arate()                          **
**                                                                 **
****************************************************************** */

arate_g()
{                            /* Berechnung der Ankunftsraten fuer
				ein geschlossenes Netz :
				Ergebnisse sind nicht eindeutig */

	int    l;                   /* Zaehlvariable    l: Knoten */
	int    zeile,spalte;        /* Zaehlvariablen */
	int    klasse;              /* Zaehlvariable klasse : Klasse */
	int    gauss_g();           /* Berechnung eines homogenen Systems
					A * X = 0 */


	for(klasse=0;klasse<netz.klasse;klasse++)
	{
		for(zeile=0;zeile<netz.knoten;zeile++)
			for(spalte=0;spalte<netz.knoten;spalte++)
				wa[zeile][spalte]=netz.uebwkt[klasse][spalte][zeile];

		/* die Matrix wa verhindert, dass die Originalmatrix 
                   veraendert wird,
                   wa enthaelt die Transponierte von netz.uebwkt
                */

		for(zeile=0;zeile<netz.knoten;zeile++)
			--wa[zeile][zeile];

		/* dekrementieren der Diagonale */
		for(l=0;l<netz.knoten;l++)
			bvek[l]= 0;
		/* Belegung der rechten Seite, fuer 'bvek' wird vorher 
		      Speicherplatz belegt */

		gauss_g(wa,netz.ankunftsrate[klasse],bvek,netz.knoten,0.001);
	}  /* ENDE for(klasse) */

} /* ENDE arate_g */

/* *********************************************************************
**                                                                    **
**                             arate                                  **
**                  aufgerufen von : main()                           **
**                                                                    **
********************************************************************* */

arate()
{
	int arate_o();         /* Berechnung der Ankunftsrate fuer ein offenes Netz
			          anhand der globalen Gleichgewichtsgleichungen */
	int arate_g();         /* Berechnung der Ankunftsrate fuer ein geschlossenes
			          Netz, Loesung nicht eindeutig */
	extern int sp_arate(); /* Speichervergabefunktion */
	extern int free_arate();  /* Freigabefunktion */

	sp_arate();                        /* Speicherbelegung */

	if (netz.typ == OFFEN)             /* Netz offen */
		arate_o();
	else /* Netz geschlossen */
		arate_g();

	free_arate();                      /* Speicherfreigabe */

}
