/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tmem_fluss.c\t1.3\t92/12/09 15:51:55";


/* ****************************************************************
**                                                               **
**                         Include Dateien                       **
**                                                               **
**************************************************************** */

#include <stdio.h>
#include "mem_netz.h"
#include "mem_def.h"

/* ****************************************************************
**                                                               **
**                         Externe Variablen                     **
**                                                               **
**************************************************************** */

/* *********** Integer Variablen **************************** */

extern int    auftrag;       /* Anzahl der Auftraege aller Klassen im
				Netz */
extern int    bottle;        /* Nummer des Knotens mit der groessten
				Auslastung ==>>
				  Flaschenhals */

/* *********** Double Vektoren  ***************************** */

extern double *ankunftsrate;        /* Ankunftsrate pro Knoten */
extern double *bedienrate;          /* Bedienrate pro Knoten */
extern double *e_a_quadvar;        /* quadr.  Variationskoeffizient
                                      der externen Ankunftsprozesse
				      pro Knoten */
extern double *e_z_ankunft;        /* Zwischenankunftsrate
				      0 --> Knoten */
extern double *auslastung;         /* Auslastung des Knotens i */
extern double *mul;                /* Auslastung des Knotens i in einem
				      Pseudo-offenem Netz, abhaengig von
				      _mulkonst */
extern double *anzauftrag;        /* Anzahl der Auftraege im Knoten i */
extern double *x_lagrange;        /* Lagrange Multiplikator */
extern double *g_lagrange;        /* Lagrange Multiplikator */
extern double *s_quadvar;         /* quadr. Variationskoeffizient des
			      	     Bedienprozessses */
extern double *a_quadvar;         /* quadr. Variationskoeffizient des
			       	     Ankunftsprozesses */
extern double *d_quadvar;         /* quadr. Variationskoeffizient des 
				     Abgangsprozesses */
extern double *fvek;            /* Vektor der Summen der mittleren
				   Auftragszahl im System pro Klasse */
/* rechte Seite des Gleichungssystems
				   J _mvek = _fvek */
/* wobei J Jacobimatrix 
					 _mvek Loesungsvektor */
extern double *mvek;            /* enthaelt Loesung des Gleichungssystems
                                   J _mvek = _fvek */

/* ************** Double Felder *********************************** */

extern double **uebwkt;           /* Uebergangsmatrix
				     Knoten --> Knoten */
extern double **besuch;           /* Wahrscheinlichkeit, dass ein den
				     Knoten s besuchender Auftrag der
				     Klasse angehoert */
extern double **z_ankunft;        /* Zwischenankunftsrate
				     Knoten --> Knoten */
extern double *mulkonst;          /* multiplikative Konstante 
				     pro Klasse*/
extern double **jacobi;         /* Jacobi Matrix
				   Dimension :
				   Anzahl Klassen X Anzahl Klassen */


/* *******************************************************************
**                                                                  **
**                         bcsstern                                 **
**                   aufgerufen von _bcsbstern                      **
**                                                                  **
******************************************************************* */

double bcsstern(r,i,theta)
int r;         /* Klasse r */
int i;         /* Knoten i */
double theta;  /* theta    */
{                   /* Berechnung von B_cs^* */
	double wert;       /* Rueckgabewert */
	extern double *taucs; /* tau_cs */

	wert = taucs[r] * taucs[r] * netz.bedienrate[r][i];
	wert /= (taucs[r] * netz.bedienrate[r][i] + theta);
	wert += (1 - taucs[r]);

	return(wert);

} /* ENDE bcsstern */

/* *******************************************************************
**                                                                  **
**                        bcsbstern                                 **
**                  aufgerufen von _comppocs und _bsbstern          **
**                                                                  **
******************************************************************* */

double bcsbstern(r,i,theta)
int r;          /* Klasse r */
int i;          /* Knoten */
double theta;   /* theta */
{               /* Berechnung von B_cs^b* */
	double wert;   /* Rueckgabewert */
	extern double *sigmacs; /* sigma_cs */

	wert = sigmacs[r] * bcsstern(r,i,theta);
	wert /= (1 - (1-sigmacs[r]) * bcsstern(r,i,theta));

	return(wert);

} /* ENDE bcsbstern */

/* *******************************************************************
**                                                                  **
**                        bsbstern                                  **
**                 aufgerufen von _comppocs                         **
**                                                                  **
******************************************************************* */

double bsbstern(i,theta)
int i;          /* Knoten */
double theta;   /* theta */
{                        /* Berechnung von B_s^* */
	int r ;                 /* Zaehlvariable r : Klasse */
	double summe;           /* Rueckgabewert */
	double hilf;            /* Hilfsvariable */
	extern double lambdarb; /* lambda_R^b */
	extern double *sigmacs; /* sigma_cs */

	summe = 0;
	for(r=0;r<netz.klasse;r++)
	{
		if (netz.auslastung[r][i] > 0)
		{
			hilf = mulkonst[r] * netz.ankunftsrate[r][i] ;
			hilf *= sigmacs[r] * bcsbstern(r,i,theta);
			summe += hilf;
		} /* ENDE if*/
	} /* ENDE for(r) */

	return(summe/lambdarb);

} /* ENDE bsbstern */

/* *******************************************************************
**                                                                  **
**                        comppocs                                  **
**                  aufgerufen von _f_o_multi, _f_g_multi           **
**                                                                  **
******************************************************************* */

comppocs()
{                             /* Berechnung der Wahrscheinlichkeit p_cs(0),
				 dass kein Auftrag der Klasse c sich im Knoten s
				 befindet */
	int i,r;                     /* Zaehlvariablen i : Knoten r : Klasse */
	double hilf,zaehler, nenner; /* Hilfsvariablen */
	extern double *sigmacs;      /* sigma_cs */
	extern double *taucs;        /* tau_cs */
	extern double **pocs;        /* p_cs(0) */
	extern double bcsbstern();          /* Funktion zur Berechnung von B_cs^b* */
	extern double bsbstern();           /* Funktion zur Berechnung von B_s^b* */
	extern double lambdarb; /* lambda_R^b */

	for(i=0;i<netz.knoten;i++)
	{
		lambdarb = 0;
		for(r=0;r<netz.klasse;r++)
		{
			if (netz.auslastung[r][i] > 0)
			{
				sigmacs[r] = 2 / (netz.a_quadvar[r][i] + 1);
				taucs[r] = 2 / ( netz.s_quadvar[r][i] + 1);
				lambdarb += mulkonst[r] * netz.ankunftsrate[r][i] * sigmacs[r];
			} /* ENDE if */
		} /* ENDE for(r) */
		for(r=0;r<netz.klasse;r++)
		{
			if (netz.auslastung[r][i] > 0)
			{
				zaehler = mulkonst[r] * netz.ankunftsrate[r][i];
				zaehler *=  sigmacs[r] * (1 - mul[i]) ;
				zaehler *=bcsbstern(r,i,mulkonst[r]*netz.ankunftsrate[r][i]*sigmacs[r]);

				/* Berechnung des Zaehlers von p_cs(0) */

				nenner = mulkonst[r] * netz.ankunftsrate[r][i] * sigmacs[r] - lambdarb ;
				hilf =  bsbstern(i,mulkonst[r] * netz.ankunftsrate[r][i]*sigmacs[r]);
				nenner += lambdarb * hilf;

				/* Berechnung des Nenners von p_cs(0) */


				pocs[r][i] = zaehler / nenner;

				/* Wahrscheinlichkeit p_cs(0) , dass kein Auftrag
			    der Klasse r im Knoten i ist */
			} /* ENDE if */
		} /* ENDE for(r) */
	} /* ENDE for(i) */
} /* ENDE comppocs */

/* *******************************************************************
**                                                                  **
**                    compc2dcs                                     **
**               aufgerufen von _f_o_multi,f_g_multi                **
**                                                                  **
******************************************************************* */

int compc2dcs()
{                   /* Berechnung des quadrierten Variationskoeffizienten
		       des Abgangsprozesses pro Klasse und Knoten */
	int i,r ;           /* Zaehlvariablen i :Knoten r : Klasse */
	double hilf;        /* Hilfsvariable */
	extern double **pocs;        /* p_cs(0) */

	for(i=0;i<netz.knoten;i++)
		for(r=0;r<netz.klasse;r++)
		{
			if (netz.auslastung[r][i] > 0)
			{
				hilf = netz.a_quadvar[r][i] * (2* pocs[r][i] - 1);
				netz.d_quadvar[r][i] = hilf + 2 * netz.anzauftrag[r][i] * pocs[r][i];
			}
		} /* ENDE for(r) */

} /* ENDE compc2dcs */

/* *******************************************************************
**                                                                  **
**                     compc2acs                                    **
**             aufgerufen von _f_o_multi,f_g_multi                  **
**                                                                  **
******************************************************************* */

int compc2acs()
{              /* Berechnung des quadrierten Variationskoeffizient C^2_a_cs
		  des Ankunftsprozesses pro Klasse und Knoten */
	int i,j,r;         /* Zaehlvariable i ,j : Knoten r: Klasse */
	double hilf,summe; /* Hilfsvariablen */
	double alterwert;  /* alter Wert von C^2_a_cs */
	int ok;            /* zeigt Konvergenz an */

	ok = TRUE;
	for (i=0;i<netz.knoten;i++)
		for(r=0;r<netz.klasse;r++)
		{
			if (netz.auslastung[r][i] > 0)
			{
				if (netz.typ == OFFEN)
					summe = netz.e_ankunftsrate[r][i] / (netz.e_a_quadvar[r][i] + 1);
				else
					summe = 0;
				for (j=0;j<netz.knoten;j++)
				{
					hilf = mulkonst[r]*netz.ankunftsrate[r][j]*netz.uebwkt[r][j][i];
					hilf /= (2 + netz.uebwkt[r][j][i] * (netz.d_quadvar[r][j] - 1));
					summe += hilf;
				} /* ENDE for(j) */
				alterwert = netz.a_quadvar[r][i];
				netz.a_quadvar[r][i] = mulkonst[r]*netz.ankunftsrate[r][i]/summe -1 ;
				if (abs(alterwert - netz.a_quadvar[r][i]) > SUMTOLERANZ)
					ok = FALSE;
			} /* ENDE if */
		} /* ENDE for(r-i) */

	return(ok);

} /* ENDE compc2acs */

/* ******************************************************************
*                                                                   *
*                     ber_auftrag                                   *
*                 aufgerufen von :  newton_raphson()         *
*                                                                   *
****************************************************************** */

ber_auftrag()
{
	/* Berechnung der mittleren Anzahl von Auftraegen 
                     im Knoten i */

	int    i,r;      /* Laufvariable i : Knoten  */
	double hilf;     /* Hilfsgroesse */

	if (netz.klasse == 1)
	{
		for(i=0;i<netz.knoten;i++)
		{
			hilf=((a_quadvar[i] + (s_quadvar[i] * mul[i])) / (1 - mul[i])) + 1;
			anzauftrag[i] = (hilf * mul[i]) / 2;
		}
	}
	else
	{
		if (netz.typ == GESCHLOSSEN)
		{
			for(i=0;i<netz.knoten;i++)
			{
				anzauftrag[i] = 0;
				for( r=0;r<netz.klasse;r++)
					anzauftrag[i] += netz.anzauftrag[r][i];
			} /* Ende for(i) */
		} /* Ende if */
	} /* Ende else */


}   /* ENDE ber_auftrag */


/* *****************************************************************
*                                                                  *
*                    test_anzauftrag                               *
*              aufgerufen von : newton_raphson()                   *
*                                                                  *
* **************************************************************** */

test_anzauftrag()
{                   /* Test auf Grenzen der Formel fuer die
		       mittlere Anzahl von Auftraegen */
	int    i;          /* Zaehlvariable i : Knoten */
	double hilf;       /* Hilfsvariable */

	for (i=0;i<netz.knoten;i++)
	{
		hilf = (1-a_quadvar[i]) / (1+s_quadvar[i]);
		if ((mul[i] < hilf) || (mul[i] >= 1))
		{
			fprintf(stderr,"Fehler: Formel fuer L_s nicht anwendbar\n");
			printf("mem_versagt\n 0 0 0\n");
			exit(1);
		} /* ENDE if */
	}   /* ENDE for(i) */
} /* ENDE test_anzauftrag */

/* ******************************************************************
*                                                                   *
*                     ber_ablanzauftrag                             *
*             aufgerufen von : newton_raphson()                     *
*                                                                   *
****************************************************************** */

double ber_ablanzauftrag(i)
int i;             /* Knotennummer */
{
	/* Berechnung der Ableitung der Gleichung fuer L_i
                      fuer jeden Knoten i, falls nur eine Auftragsklasse
		      im Netz.
                   */

	double abl;       /* zurueckgegebener Wert */
	double hilf;      /* Hilfsvariable zur Bestimmung des Wertes
			      der Ableitung der Gleichung fuer L_i */



	hilf = ((a_quadvar[i]+mul[i]*s_quadvar[i])/(1 - mul[i])) ;
	hilf +=  2*s_quadvar[i];
	hilf = 1 + ((a_quadvar[i] + (mul[i] * hilf)) / (1 - mul[i]));
	abl  = (netz.auslastung[0][i] * hilf) / 2;

	/* Berechnung der Ableitung von L_i partiell nach 
        	      Klasse 0 */
	return(abl);

} /* ENDE ber_ablanzauftrag */


/* ******************************************************************
*                                                                   *
*                     newton_raphson                                *
*              aufgerufen von : _f_g_single                         *
*                                                                   *
****************************************************************** */

newton_raphson()
{                        /*  nur auf geschlossene Netze mit
			     einer Auftragsklasse anwendbar */
	/*  Berechnung der multiplikativen Konstante
                             mit Hilfe der modifizierten  Newton-Raphson
                             Methode  fuer eine Auftragsklasse      */

	int    i;               /* Zaehlvariable i : Knoten    */
	double alter_wert;      /* Speicher fuer alte Werte der 
				    multiplikativen Konstante */
	double sumanzauftrag;   /* Summe der L_i  */
	double sumablanz;       /* Summe der Ableitungen der L_i */


	/* *** Funktionen *** */

	int test_anzauftrag();   /* Test ob die Formel fuer die mittlere
			     Auftragszahl in den einzelnen Knoten
			     anwenbar ist; falls nein
				     --> Abbruch */
	int ber_auftrag();       /* Berechnung der mittleren Anzahl von 
				     Auftraegen in den einzelnen Knoten */

	/* *** BEGINN *** */

	do
	{
		test_anzauftrag();

		/* Test der Formel fuer L_i */

		ber_auftrag();
		/* Berechnung mittlere Anzahl
			    von Auftraegen */

		alter_wert = mulkonst[0];

		/* merke den alten Wert,
			    beachte nur eine Auftragsklasse ! */

		sumanzauftrag = sumablanz = 0.0;
		for(i=0;i<netz.knoten;i++)
		{
			sumanzauftrag = sumanzauftrag + anzauftrag[i];
			sumablanz     = sumablanz + ber_ablanzauftrag(i);
		} /* ENDE for(i) */
		mulkonst[0] -=  ((sumanzauftrag - auftrag) / sumablanz);

		/* Berechne neue multiplikative Konstante */

		for(i=0;i<netz.knoten;i++)
		{
			mul[i] = mulkonst[0] * auslastung[i];
		}
		/* Berechne neue Auslastung */
	}   while ((abs(alter_wert-mulkonst[0])) > SUMTOLERANZ);

	if (mul[bottle] > 1)
	{
		fprintf(stderr,"MAXENTRO: Fehler bei Newton_Raphson \n");
		fprintf(stderr,"        : Bestimmung des Flussgleichgewichts");
		fprintf(stderr," nicht moeglich \n");
		printf("mem_versagt\n 0 0 0\n");
		exit(1);
	} /* Ende if */

} /* ENDE NEWTON_RAPHSON */

/* *******************************************************************
*                                                                    *
*                     ber_dquadvar                                   *
*              aufgerufen von : _f_o_single,f_g_single               *
*                                                                    *
******************************************************************* */

int ber_dquadvar()
{
	/* Berechnung des quadrierten Variationskoeffizienten 
                     des Abgangsprozesses */
	int    i;         /* Zaehlvariable i : Knoten */
	double hilf;      /* Hilfsvariable zur Berechnung des Variations-
			      koeffizienten */
	double d_quadalt; /* speichert alten Wert des Variations-
			      koeffizienten */
	int    genau;     /* zeigt an,ob Konvergenz */

	genau=TRUE;
	for(i=0;i<netz.knoten;i++)
	{
		d_quadalt = d_quadvar[i];

		/* Speichern des alten Wertes */

		hilf =  (1-mul[i]) * (mul[i]+a_quadvar[i]);
		d_quadvar[i] = hilf + (mul[i]*mul[i])*s_quadvar[i];

		/* Berechnen des neuen Wertes */

		if (abs((d_quadalt - d_quadvar[i])) > SUMTOLERANZ)
			genau=FALSE;

		/* Feststellen, ob Berechnung hinreichend genau */

	} /* Ende for(i) */


	return(genau);

}     /* ENDE ber_dquadvar */

/* ******************************************************************
*                                                                   *
*                     ber_aquadvar                                  * 
*                 aufgerufen von f_o_single,f_g_single              *
*                                                                   *
****************************************************************** */

int ber_aquadvar()
{
	/* Berechnung des quadrierten Variationskoeffizienten
                    des Ankunftprozesses in einem geschlossenen Netz */

	int i,j;           /* Zaehlvariable i,j : Knoten */

	int    genau;      /* zeigt an ob Konvergenz */
	double hilf,sum ;  /* Hilfsgroessen */
	double a_quadalt;  /* speichert alten Wert des Variationskoeffizienten 
	                       im Knoten i */

	genau = TRUE;

	for(i=0;i<netz.knoten;i++)
	{
		a_quadalt = a_quadvar[i];

		/* Speichern des alten Wertes */

		if (netz.typ == OFFEN)
			sum = e_z_ankunft[i] / (1 + e_a_quadvar[i]);
		else 
			sum = 0.0;

		for(j=0;j<netz.knoten;j++)
		{
			hilf = 2 + uebwkt[j][i] * (d_quadvar[j] - 1);
			sum = sum + z_ankunft[j][i] / hilf;
		}
		a_quadvar[i] = (ankunftsrate[i] / sum) - 1;

		/* Berechnung aktuellen  Wert */

		if (abs((a_quadalt - a_quadvar[i])) > SUMTOLERANZ)
			genau = FALSE;

		/* Uberpruefung, ob Berechnung hinreichend genau */

	} /* Ende for(i) */

	return(genau);

} /* ENDE ber_aquadvar */


/* *******************************************************************
*                                                                    *
*                     ber_lagrange                                   *
*                 aufgerufen von f_g_single,f_g_multi                *
*                                                                    *
******************************************************************* */

ber_lagrange()
{
	/* Berechnung der Lagrangekoeffizienten */

	int i;       /* Laufvariable i: Knoten */

	for(i=0;i<netz.knoten;i++)
	{
		x_lagrange[i] = (anzauftrag[i] - mul[i]) / anzauftrag[i];
		g_lagrange[i] = (mul[i]*mul[i])/((1-mul[i])*(anzauftrag[i]-mul[i]));
	}
} /* ENDE ber_lagrange */

/* ***************************************************************
*                                                                *
*                       norm_ankunftsrate                        *
*                    aufgerufen von : _f_g_multi                 *
*                                                                *
**************************************************************** */

norm_ankunftsrate()
{
	/* Normiere Ankunftsrate : dient zur Initialisierung
		      der multiplikativen Konstanten m_c */
	int i,r;         /* Zaehlvariable i : Knoten r : Klasse */
	double sum;      /* Hilfsvariable */

	if ((netz.typ == GESCHLOSSEN) && (netz.klasse > 1))
	{
		for(r=0;r<netz.klasse;r++)
		{
			sum =0;
			for(i=0;i<netz.knoten;i++)
				if (netz.bedienrate[r][i] != 0)
					sum += netz.ankunftsrate[r][i];
			for(i=0;i<netz.knoten;i++)
				if (netz.bedienrate[r][i] != 0)
					netz.ankunftsrate[r][i] *= (netz.auftrag[r] / sum);
		}
	}                    /* Falls mehr als eine Auftragsklasse
				im geschlossenen Netz gewichte die
					Ankunftsraten */

} /* ENDE norm_ankunftsrate */

/* *********************************************************************
**                                                                    **
**                          ber_ankunftsrate                          **
**                    aufgeruefn von : o_ergebnis                     **
**                                                                    **
********************************************************************* */

ber_ankunftsrate()
{                           /* Berechnung klassenunabhaengiger Ankunftsraten */
	int i,r ;                   /* Zaehlvariablen i: Knoten r : Klasse */

	for(i=0;i<netz.knoten;i++)
{                    /* Berechnung fuer jeden Knoten,
	                                for(i)-Schleife  */
		ankunftsrate[i] = 0.0;
		for(r=0;r<netz.klasse;r++)
			ankunftsrate[i] += mulkonst[r] * netz.ankunftsrate[r][i];
		if (ankunftsrate[i] == 0.0)
		{
			fprintf(stderr,"Ankunftsrate des Knotens %d gleich Null\n",i+1);
			printf("mem_ankr\n 0 0 0\n");
			exit(1);
		} /* ENDE if(ankunftsrate) */
	} /* Ende for(i) */

} /* ENDE ber_ankunftsrate */

/* ***************************************************************
*                                                                *
*                       ber_auslastung                           *
*                    aufgerufen von : f_o_single,f_o_multi       *
*                                     f_g_single,f_g_multi       *
*************************************************************** */

ber_auslastung()
{
	/* Berechnung klassenabhaengige Auslastung U_ri,
		     Berechnung klassenunabhaengige Auslastung U_i */
	int    i,r;     /* Zaehlvariablen :     i : Knoten   r: Klasse */

	for(r=0;r<netz.klasse;r++)
		for(i=0;i<netz.knoten;i++)
			netz.auslastung[r][i] = netz.ankunftsrate[r][i]/netz.bedienrate[r][i];

	if (netz.klasse > 1)
	{
		for(i=0;i<netz.knoten;i++)
		{
			auslastung[i] = 0;
			for(r=0;r<netz.klasse;r++)
				auslastung[i] += netz.auslastung[r][i];
		} /* ENDE for (i) */
	} /* ENDE if (netz.klasse) */

} /* ENDE  ber_auslastung */

/* *****************************************************************
*                                                                  *
*                        ber_bedienrate                            *
*                    aufgerufen von : g_ergebnis                   *
*                                                                  *
***************************************************************** */

ber_bedienrate()
{                   /* Berechnung klassenunabhaengige Bedienrate
		       pro Knoten */
	int i;            /* Zaehlvariablen :    i : Knoten             */

	for(i=0;i<netz.knoten;i++)
		bedienrate[i] = ankunftsrate[i] / mul[i];

} /* ENDE ber_bedienrate */

/* *****************************************************************
*                                                                  *
*                        ber_besuch                                *
*                    aufgerufen von : g_ergebnis                   *
*                                                                  *
****************************************************************** */

ber_besuch()
{                   /* Berechnung Wahrscheinlichkeit, dass ein 
		       in Knoten i ankommender Auftrag der 
		       Klasse r angehoert */
	int i, r;         /* Zaehlvariablen :    i : Knoten r : Klasse */

	for(i=0;i<netz.knoten;i++)
		for(r=0;r<netz.klasse;r++)
			besuch[r][i] = mulkonst[r] * netz.ankunftsrate[r][i]/ankunftsrate[i];

	/* Wahrscheinlichkeit  dafuer, dass
                             eintreffender Auftrag aus Klasse r */

} /* ENDE ber_besuch */

/* ********************************************************************
*                                                                     *
*                         ber_zwischenankunft                         * 
*                       aufgerufen von : f_o_single,f_g_single        *
*                                                                     *
******************************************************************** */

ber_zwischenankunft()
{                       /* Berechnung der Zwischenankunftsrate
				      von Knoten --> Knoten */
	int i,j;               /* Laufvariable i, j : Knoten */

	for(i=0;i<netz.knoten;i++)
		for(j=0;j<netz.knoten;j++)
			z_ankunft[i][j] = ankunftsrate[i]*uebwkt[i][j];

} /* ENDE ber_zwischenankunft */

/* *******************************************************************
*                                                                    *
*                            init_var                                *
*                       aufgerufen von : f_g_single,f_o_single       *
*                                        f_g_multi,f_g_single        *
******************************************************************* */

init_var()
{
	/* Initialisierung der Variationskoeffizienten */

	int i,r;             /* Zaehlvariable i : Knoten  r: Klasse       */

	if (netz.klasse > 1)
	{
		for(i=0;i<netz.knoten;i++)
			for(r=0;r<netz.klasse;r++)
				netz.a_quadvar[r][i] = 1.0;
	}
	else
		for(i=0;i<netz.knoten;i++)
			a_quadvar[i]=1;


	/* Initialisierung quadrierter Variationskoeffizient
                     der Ankunftsverteilung */

} /* ENDE init_var */

/* *******************************************************************
*                                                                    *
*                            init_mul                                *
*                    aufgerufen von : f_o_single,f_o_multi           *
*                                     f_g_single,f_g_multi           *
******************************************************************* */

init_mul()
{
	/* Initialisierung der multiplikativen Konstanten */
	int i,r;           /* Zaehlvariable i : Knoten r : Klasse */


	if (netz.typ == OFFEN)
	{
		for(r=0;r<netz.klasse;r++)
			mulkonst[r] = 1;
		mul = auslastung;
	} /* ENDE if(netz.typ) */
	else
	{  /* Netz GESCHLOSSEN */

		bottle = 0 ;
		for(i=1;i<netz.knoten;i++)
			if (auslastung[i] > auslastung[bottle])
				bottle = i;

		/* Berechnung des Knotens mit der 
				 grossten Auslastung ==>
				 Flaschenhals */

		for(r=0;r<netz.klasse;r++)
			mulkonst[r] = 0.99 / auslastung[bottle];

		for(i=0;i<netz.knoten;i++)
			mul[i] = (mulkonst[0] * auslastung[i]);

		/* Jetzt sind alle Auslastungen im
				  Pseudooffenem Netz < 1 */

		/* Initialisierung der multiplikativen Konstanten */
	} /* ENDE if (geschlossen) */

} /* ENDE init_mul */

/* ******************************************************************
*                                                                   *
*                            init_extern                            *
*                    aufgerufen von : f_o_single                    *
*                                                                   *
****************************************************************** */

init_extern()
{                   /* Initialisierungsteil fuer offene Netz mit
		       einer Auftragsklasse */

	e_a_quadvar = netz.e_a_quadvar[0];

	/* Berechne quadrierten Variationskoeffizienten
			des externen Ankunftsprozesses 
			0 --> Knoten */

	e_z_ankunft = netz.e_ankunftsrate[0];

	/* Berechne externe Zwischenankunftszeit
			0 --> Knoten */

} /* ENDE init_extern */

/* *****************************************************************
*                                                                  *
*                     almo_auftrag                                 *
*              aufgerufen von : iter_newton                        *
*                                                                  *
***************************************************************** */

almo_auftrag()
{
	/* Berechnung der mittleren Anzahl von Auftraegen 
                     der Klasse r im Knoten i 
		     nach einer von Almond vorgeschlagenen Formel  */
	/* Abkuerzungen :
			   L_i    mittlere Anzahl von Auftraegen im 
				  Knoten i
                           L_ri   mittlere Anzahl von Aufraegen der
				  Klasse r im Knoten i */

	int    i,r,r1;      /* Laufvariable i : Knoten  r,r1 : Knoten */
	double hilf;     /* Hilfsgroesse */
	double sum;      /* Hilfsgroesse */



	for(i=0;i<netz.knoten;i++)
		for(r=0;r<netz.klasse;r++)
		{
			if (netz.auslastung[r][i] == 0.0)
				netz.anzauftrag[r][i] = 0.0;
			else
			{
				sum = 0;
				for(r1=0;r1<netz.klasse;r1++)
				{
					hilf = netz.auslastung[r1][i] * mulkonst[r1];
					hilf /= netz.bedienrate[r1][i];
					hilf *= (netz.s_quadvar[r1][i] + netz.a_quadvar[r1][i]);
					sum += hilf;
				} /* ENDE for(r1) */
				sum *= netz.ankunftsrate[r][i] * mulkonst[r];

				/* Berechnung Zaehler des ersten
				  Summanden */

				hilf = 0.5 * mulkonst[r] * netz.auslastung[r][i];
				hilf *= (netz.a_quadvar[r][i]+1);

				/* Berechnung zweiter Summand */

				netz.anzauftrag[r][i] = 0.5 * sum / (1-mul[i]) + hilf;
			} /* ENDE else */
		} /* ENDE for(r) */
	/* Berechnung der mittleren Anzahl
				 der Auftraege pro Klasse und Knoten */


}   /* ENDE almo__auftrag */

/* *****************************************************************
*                                                                  *
*                     almo_abl                                     *
*            aufgerufen von : iter_newton                          *
*                                                                  *
***************************************************************** */

double almo_abl(r,r1,i)
int r;             /* Klasse       */
int r1;            /* Klasse       */
int i;             /* Knotennummer */
{
	/* Die Formel fuer die mittlere Anzahl der Auftraege
	           in einem Knoten i der Klasse r wird nach der
	           multiplikativen Konstante _mulkonst[r1] abgeleitet */
	/* Abkuerzungen :
                           L_ri   mittlere Anzahl von Aufraegen der
				  Klasse r im Knoten i */

	int    r2;        /* Zaehlvariable  r2 : Klasse */
	double abl;       /* zurueckgegebener Wert */
	double hilf,
	hilfsum,
	sum1,
	sum2;      /* Hilfsvariablen */


	hilfsum=0;
	for(r2=0;r2<netz.klasse;r2++)
	{
		hilf=netz.ankunftsrate[r2][i] * mulkonst[r2];
		hilf /= (netz.bedienrate[r2][i] * netz.bedienrate[r2][i]);
		hilf *= (netz.s_quadvar[r2][i] + netz.a_quadvar[r2][i]);
		hilfsum += hilf;
	} /* ENDE for(r2) */

	/* Berechnung einer oefters benoetigten 
        			  Zwischensumme */

	if (r == r1)
	{
		/* ***************************************************
	     **                                                  **
	     **           Ableitung von L_ri nach nach           ** 
	     **      mulipikative Konstante der Klasse r1        **
	     **                   mit r gleich r1                **
	     *************************************************** */
		sum1 = netz.auslastung[r][i] * (netz.a_quadvar[r][i] + 1);

		/* Berechnung des ersten Summanden der 
				  Auslastung */

		hilf = hilfsum * mulkonst[r] * netz.ankunftsrate[r][i];
		hilf *= netz.auslastung[r][i];
		hilf /= (1-mul[i]);
		sum2 = hilf + hilfsum*netz.auslastung[r][i]*netz.bedienrate[r][i];
		hilf = netz.s_quadvar[r][i] + netz.a_quadvar[r][i];
		hilf *= mulkonst[r]*netz.auslastung[r][i]*netz.auslastung[r][i];
		sum2 = (sum2 + hilf) / (1-mul[i]);

		/* Berechnung des zweiten Summanden
				 der Ableitung */

		abl = 0.5 * (sum1 + sum2);

		/* Berechnung des Rueckgabewertes */

		/* Berechnung der Ableitung von L_ri partiell nach
		    Klasse r */

	} /* ENDE if(r gleich r1) */
	else
	{ /* r ungleich r1 */
		/* ***************************************************
	     **                                                  **
	     **           Ableitung von L_ri nach nach           **
	     **        muliplikative Konstante der  Klasse r1    **
	     **                   mit r ungleich r1              **
	     *************************************************** */

		hilf = netz.s_quadvar[r1][i] + netz.a_quadvar[r1][i];
		sum1 = hilf * netz.auslastung[r1][i] / netz.bedienrate[r1][i];

		/* Berechnung des ersten Summanden
				   der Ableitung */

		sum2 = hilfsum * netz.auslastung[r1][i] / ( 1 - mul[i]);

		/* Berechnung des zweiten Summanden
				   der Auslastung */

		abl = mulkonst[r] * netz.ankunftsrate[r][i] * (sum1 + sum2);
		abl *= 0.5 / (1 - mul[i]);

		/* Berechnung des Rueckgabewertes */

	} /* ENDE else(r1 ungleich r ) */

	return(abl);

} /* ENDE almo_abl          */

/* *******************************************************
*                                                        *
*                     jac_elem                           *
*           aufgerufen von jac_matrix                    *
*                                                        *
******************************************************* */

double jac_elem(zeile,spalte)
int zeile;
int spalte;
{
	int    i;            /* Zaehlvariable i : Knoten */
	double sum;          /* Rueckgabewert */
	double almo_abl();
	/* Die Formel fuer die mittlere Anzahl der 
			 Auftraege in einem Knoten i der Klasse zeile 
			 wird nach der multiplikativen Konstante
			 _mulkonst[spalte] abgeleitet */

	sum = 0;
	for(i=0;i<netz.knoten;i++)
	{
		sum += almo_abl(zeile,spalte,i);
	}

	return(sum);
	/* Berechnung  Element der Jacobi-
			   Matrix J(zeile,spalte) */
} /* ENDE jac_elem */

/* *******************************************************
*                                                        *
*                     jac_matrix                         *
*           aufgerufen von iter_newton                   *
*                                                        *
******************************************************* */

jac_matrix()
{
	int r,r1;        /* Zaehlvariablen r,r1 : Klassen */
	double jac_elem(); /* Berechnung der Komponenten der Jacobi Matrix */

	for(r=0;r<netz.klasse;r++)
		for(r1=0;r1<netz.klasse;r1++)
			jacobi[r][r1] = jac_elem(r,r1);

	/* Berechnung der Jacobi Matrix */

} /* ENDE jac_matrix */

/* *******************************************************
*                                                        *
*                     f_elem                             *
*           aufgerufen von : f_vektor                    *
*                                                        *
******************************************************* */

double f_elem(zeile)
int zeile;
{                     /* Berechnung der durch _zeile bestimmten
			 Komponenten des Vektors _fvek der Gleichung
			 _jacobi  _mvek = _fvek */
	double sum;          /* Rueckgabewert */
	int    i;            /* Zaehlvariable i : Knoten */

	sum = 0;
	for(i=0;i<netz.knoten;i++)
		sum += netz.anzauftrag[zeile][i];
	sum -= netz.auftrag[zeile];

	return(sum);
	/* Rueckgabe */

} /* ENDE f_elem */

/* *******************************************************
*                                                        *
*                     f_vektor                           *
*              aufgerufen von : iter_newton              *
*                                                        *
******************************************************* */

f_vektor()
{           /* Berechnung der rechten Seite der Gleichung
	       _jacobi _mvek = _fvek */

	int r;                   /* Zaehlvariable r : Klasse */
	double f_elem();         /* Berechnung einzelner Komponenten
				     des Vektors _fvek */

	for(r=0;r<netz.klasse;r++)
		fvek[r] = (-1) * f_elem(r);

} /* ENDE f_vektor */


/* *******************************************************
*                                                        *
*                     iter_newton                        *
*           aufgerufen von : f_g_multi                   *
*                                                        *
******************************************************* */

iter_newton()
{
	/* Berechnung der multiplikativen Konstanten
		    mit Hilfe von Gauss und dem iterativen
		    Newton-Verfahren */
	int i,r;      /* Zaehlvariablen i : Knoten r : Klasse */
	int ok;       /* Boolean : gibt an , ob Konvergenz */
	double merke; /* Festhalten des Wertes der Auslastung */

	/* *** Funktionen *** */

	int almo_auftrag();       /* Berechne neue mittlere Auftragszahlen
				        pro Knoten und  Klasse */
	int jac_matrix();         /* Bestimme _jacobi */
	int f_vektor();           /* Bestimme _fvek */
	int gauss_o();            /* Bestimme inhomogenes Gleichungssystem
					_jacobi * _mvek = _fvek */

	/* *** Beginn *** */

	do
	{
		ok = TRUE;
		almo_auftrag();    /* Berechne neue mittlere Auftragszahlen
					    pro Knoten und  Klasse */
		jac_matrix();
		/* Bestimme _jacobi */
		f_vektor();
		/* Bestimme _fvek */

		gauss_o(jacobi,mvek,fvek,netz.klasse,0.0001);

		/* Berechne _jacobi _mvek = _fvek */

		for(r=0;r<netz.klasse;r++)
		{
			merke = mulkonst[r];
			mulkonst[r] += mvek[r];
			if (abs(merke-mulkonst[r]) > 1E-6)
				ok=FALSE;
		}
		/* Berechne neue multiplikative Kondstante */

		for(i=0;i<netz.knoten;i++)
		{
			mul[i] = 0;
			for(r=0;r<netz.klasse;r++)
				mul[i] += mulkonst[r] * netz.auslastung[r][i];
		} /* ENDE for(i) */
		/* Berechne neue Auslastungen und ueberpruefe 
			     ob Konvergenz erreicht */

		if (mul[bottle] > 1)
		{
			fprintf(stderr,"Fehler: NEWTON \n");
			fprintf(stderr,"      Bestimmung des Flussgleichgewichts \n");
			fprintf(stderr,"nicht moeglich \n ");
			printf("mem_versagt\n 0 0 0\n");
			exit(1);
		} /* ENDE if */

		/* Ueberpruefe ob Auslastung des 
				Flachenhalses groesser --> Abbruch */

	}      while (!ok);
	/* Solange bis alle Auslastungen hinreichend 
		      genau sind fuehre Schleife aus */

} /* ENDE iter_newton */
