/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tmem_loop.c\t1.2\t92/03/27 11:49:08";

/* ****************** Datei mem_loop.c *************************** */
/* Inhalt                                                          */
/*       feedback()      self_loop()                               */
/* *************************************************************** */

/* ******************************************************************
**                                                                 **
**                         Include Dateien                         **
**                                                                 **
****************************************************************** */

#include <stdio.h>
#include "mem_netz.h"
#include "mem_def.h"

/* ******************************************************************
**                                                                 **
**                        Externe Variable                         **
**                                                                 **
****************************************************************** */

extern double *ankunftsrate; /* Ankunftsrate pro Knoten */
extern double *bedienrate;   /* Bedienrate pro Knoten */
extern double *s_quadvar;    /* quadr. Variationskoeffizient des
				Bedienprozessses */
extern double **uebwkt;      /* Uebergangsmatrix
				  Knoten --> Knoten */
extern double **z_ankunft;   /* Zwischenankunftsrate
				  Knoten --> Knoten */

/* ******************************************************************
**                                                                 **
**                         feedback                                **
**               aufgerufen von : f_o_single                       **
**                                f_g_single                       **
**                                                                 **
****************************************************************** */

feedback()
{                    /* beseitigt Schleifen, d.h. Uebergaenge eines
			Knotens in sich selbst */
	/* nicht anwenbar bei geschlossenen Netzen und
			mehreren Auftragsklassen */
	int i,j,r;          /* Zaehlvariable i,j : Knoten r : Klasse */
	double rueb;        /* Hilfsvariable */
	extern int feed;    /* zeigt an, ob Ruecklauefe existiert haben */
	extern int sp_feed(); /* Speichervergabefunktion */
	extern int free_feed(); /* Speicherfreigabe */

	sp_feed();
	/* Belege Speicherplatz  */

	if (netz.typ == OFFEN)
	{
		for(r=0;r<netz.klasse;r++)
			for(i=0;i<netz.knoten;i++)
			{
				netz.altankunftsrate[r][i] = netz.ankunftsrate[r][i];
				netz.altbedienrate[r][i] = netz.bedienrate[r][i];
			}
	}
	else
	{  /* Netz geschlossen */
		for(r=0;r<netz.klasse;r++)
			for(i=0;i<netz.knoten;i++)
				netz.altbedienrate[r][i] = netz.bedienrate[r][i];
	}
	/* Speichern der Werte der Bedienzeit 
			       und Ankunftsrate,
			       wird eventuell in der Funktion 
			       _ber_ergebnis benoetigt */

	feed = FALSE;
	for(i=0;i<netz.knoten;i++)
	{
		if (uebwkt[i][i] > 0)   /* Schleife des Knotens i */
		{
			feed = TRUE;
			/* Korrektur */

			rueb = 1 - uebwkt[i][i];

			/* Hilfsberechnung */

			ankunftsrate[i] *= rueb;

			/* neue Anunftsrate */
			bedienrate[i] *= rueb;

			/* neue Bedienrate */

			z_ankunft[i][i]  = 0;

			/* Schleife eliminieren */

			s_quadvar[i]   = uebwkt[i][i] + rueb * s_quadvar[i];

			/* neuer quadr. Variationskoeffizient des
				Bedienprozesses */

			for(j=0;j<netz.knoten;j++)
			{
				if (i != j)
					uebwkt[i][j] = uebwkt[i][j] / rueb;
			}   /* Ende for(j) */
			uebwkt[i][i] = 0;
			/* Uebergangsmartix angleichen */

		} /* Ende if */
	}  /* Ende for(i) */

	if (! feed)
		free_feed();           /* falls keine Korrektur erforderlich war
				loesche Wert von netz.altbedienrate
					netz.altankunftsrate */

}  /* ENDE feedback() */

/* ************************************************************
**                                                           **
**                     self_loop                             **
**               aufgerufen von : f_g_multi                  **
**				  f_g_multi                  **
**                                                           **
************************************************************ */

self_loop()
{
	/* Beseitigt alle Schleifen von
		         einen Knoten in sich selbst,
			 Korrektur erfolgt klasseab-
			 haengig */

	int i,j,r;            /* Zaehlvariable i,j : Knoten 
					         r : Klasse */
	double rueb;          /* Hilfsvariable */
	extern int feed;      /* zeigt an, ob Korrektur */
	extern int sp_feed(); /* Speichervergabefunktion */
	extern int free_feed(); /* Speicherfreigabe */

	sp_feed();
	/* Belege Speicherplatz fuer netz.altbedienrate,
			 bei einem offenem Netz zus\"atzlich fuer
			 netz.altankunftsrate */


	if (netz.typ == OFFEN)
	{
		for(r=0;r<netz.klasse;r++)
			for(i=0;i<netz.knoten;i++)
			{
				netz.altankunftsrate[r][i] = netz.ankunftsrate[r][i];
				netz.altbedienrate[r][i] = netz.bedienrate[r][i];
			}
	}
	else
	{  /* Netz geschlossen */
		for(r=0;r<netz.klasse;r++)
			for(i=0;i<netz.knoten;i++)
				netz.altbedienrate[r][i] = netz.bedienrate[r][i];
	}

	/* Speichern der Werte der Bedienzeit 
			       und Ankunftsrate,
			       wird eventuell in der Funktion 
			       _ber_ergebnis benoetigt */

	feed = FALSE;
	for(r=0;r<netz.klasse;r++)            /* fuer alle Klassen */
	{
		for(i=0;i<netz.knoten;i++)         /* fuer alle Knoten */
		{
			if (  netz.uebwkt[r][i][i] > 0)   /* Schleife des Knotens i und
								  der Klasse r */
			{
				feed = TRUE;
				/* Ruecklaufkorrektur */

				rueb = 1 - netz.uebwkt[r][i][i];

				/* Hilfsberechnung */

				netz.ankunftsrate[r][i] *= rueb;

				/* neue Ankunftsrate */

				netz.bedienrate[r][i] *= rueb;

				/* neue Bedienrate */


				netz.s_quadvar[r][i] *= rueb;
				netz.s_quadvar[r][i] += netz.uebwkt[r][i][i];

				/* neuer quadr. Variationskoeffizient des
				Bedienprozesses */

				for(j=0;j<netz.knoten;j++)
				{
					if (i != j)
						netz.uebwkt[r][i][j] = netz.uebwkt[r][i][j] / rueb;
				}   /* Ende for(j) */
				netz.uebwkt[r][i][i] = 0;

				/* Uebergangsmartix angleichen */

			} /* Ende if */
		}  /* Ende for(i) */
	} /* ENDE for(r) */

	if (! feed)
		free_feed() ;           /* falls keine Korrektur erforderlich war
					loesche Wert von netz.altbedienrate */


}  /* ENDE self_loop  */
