/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tmem_method.c\t1.2\t92/03/27 11:49:13";

/*  **********************   Datei mem_method.c   ******************  */
/*  Inhalt :                                                          */
/*         f_o_single()            f_g_single()                       */
/*         f_o_multi()             f_g_multi()                        */
/*	   fluss_offen()           pseudo_offen()                     */
/*	   offen_netz()            geschlossen()                      */
/*				   geschl_netz()                      */
/*         null_erg()              o_ergebnis()                       */
/*         g_ergebnis()            ber_ergebnis()                     */
/*  ****************************************************************  */
/* ********************************************************************
**                                                                   **
**                         Include Dateien                           **
**                                                                   **
******************************************************************** */

#include "mem_netz.h"      /* Netzstruktur */
#include "mem_def.h"       /* Konstanten und Macros */

/* **********************************************************************
**                                                                     **
**                          f_o_single                                 **
**                     aufgerufen von : fluss_offen                    **
**                                                                     **
********************************************************************** */

int f_o_single()
{                       /* Analysiere offenes Netz mit einer Auftragsklasse */

	extern int ber_auslastung();   /* Berechnung Auslastung pro Knoten    */
	extern int ber_zwischenankunft(); /* Berechnung der Zwischenankunfts-
					     zeiten                           */
	extern int feedback();         /* Beseitigen von Schleifen            */
	extern int init_var();         /* Initialisierung der multiplikativen
                                  Konstanten und der Variations-
					  koeffizienten                       */
	extern int init_mul();         /* Initialisierung der multiplikativen
					  Konstanten                          */
	extern int init_extern();      /* Initialisierung des Ankunftsprozesses */
	extern int ber_dquadvar();     /* Berechnung quadr. Variations-
					  koeffizient des Abgangsprozesses    */
	extern int ber_aquadvar();     /* Bestimmung quadr. Variations-
				  koeffizient des Ankunftsprozesses  
					  in einem offenem Netz               */
	extern int sp_f_o_single();    /* Speichervergabefunktion */
	int konvergenz;                /* zeigt an, ob Berechnung der quadrierten
					  Variationskoeffizienten erfolgreich war */

	char c;


	sp_f_o_single();

	/* Speicherbelegung */

	ber_auslastung();
	/* Berechnung der Auslastung eines Knotens */

	ber_zwischenankunft();

	/* Berechnung der Zwischenankunftsrate 
					 Knoten --> Knoten */
	feedback();
	/* Beseitigen von Schleifen, Ruecklaeufen */

	init_extern();
	/* Berechnung fuer das offene Netz benoetigte 
			  Groessen des externen Ankunftsprozesses
				   - Gesamtankunft von aussen
				   - Zwischenankunftsrate */

	init_mul();
	/* Initialsierung der multiplikativen Konstanten
			  bei offenen Netzen dient dies dazu, die Formeln
			  fuer die quadrierten Variationskoeffizienten
			  benutzen zu koennen */

	init_var();
	/* Initialisierung der Variationskoeffizienten */

	konvergenz = FALSE;

	/* Abbruchkriterium */

	do
	{
		ber_dquadvar();

		/* Berechnung des quadrierten Variationskoeffizienten
			 des Abgangsprozesses */

		konvergenz = ber_aquadvar();

		/* Berechnung des quadrierten Variationskoeffizineten
			 des Ankunftsprozesses */

	} while (!konvergenz);

	/* Abbruch falls die Variationskoeffizienten hinreichend
			 genau sind */

	ber_auftrag();
	/* Berechnung der mittleren Anzahl von Auftraegen in 
			 den einzelnen Knoten */

} /* ENDE f_o_single */

/* **********************************************************************
**                                                                     **
**                          f_o_multi                                  **
**                     aufgerufen von : fluss_offen                    **
**                                                                     **
********************************************************************** */

int f_o_multi()
{                       /* Analysiere offenes Netz mit mehreren
			   Auftragsklasse */

	extern int ber_auslastung();   /* Berechnung Auslastung pro Knoten    */
	extern int self_loop();        /* Beseitigen von Ruecklaeufen */
	extern int init_var();         /* Initialisierung der multiplikativen
                                  Konstanten und der Variations-
					  koeffizienten                       */
	extern int init_mul();         /* Initialisierung der multiplikativen
					  Konstanten                          */
	extern int almo_auftrag();     /* Berechnung mittlere Anzahl von Auftraegen
					  pro Knoten und Klasse */
	extern int compc2acs();        /* Bestimmung quadr. Variations-
				  koeffizient des Ankunftsprozesses pro
					  Klasse und Knoten */

	extern int compc2dcs();        /* Berechnung quadr. Variations-
				  koeffizient des Abgangsprozesses 
					  pro Klasse und Knoten */
	extern int comppocs();         /* Berechnung der Wahrscheinlichkeit p_cs(0)
                                  dafuer, dass kein Auftrag der Klasse c
					  im Knoten s ist */
	extern int sp_f_o_multi();     /* Speichervergabefunktion */
	int konvergenz;                /* zeigt an, ob Berechnung der quadrierten
					  Variationskoeffizienten erfolgreich war */

	/* *** Beginn *** */

	sp_f_o_multi();
	/* Speicherbelegung */

	ber_auslastung();

	/* Berechnung der Auslastung eines Knotens */

	self_loop();
	/* Beseitigen von Ruecklaeufen */

	init_mul();
	/* Initialsierung der multiplikativen Konstanten
			  bei offenen Netzen dient dies dazu, die Formeln
			  fuer die quadrierten Variationskoeffizienten
			  benutzen zu koennen */

	init_var();
	/* Initialisierung der Variationskoeffizienten */

	konvergenz = FALSE;

	/* Abbruchkriterium */

	do
	{
		almo_auftrag();
		/* Berechne die mittlere Anzahl von Auftraegen
			 pro Klasse und Knoten */

		comppocs();     /* Berechnung p_cs(0)
			 Wahrscheinlicheit, dass kein Auftrag der Klasse
					 c im Knoten S ist */

		compc2dcs();
		/* quadrierter Variationskoeffizient
			 des Abreiseprozesses */

		konvergenz = compc2acs();

		/* quadrierter Variationskoeffizient
			 des Ankunftsprozesses */


	} while (!konvergenz);

	/* Abbruch falls die Variationskoeffizienten hinreichend
			 genau sind */

	/* *** Ende *** */
} /* ENDE offen_multi_class */

/* **********************************************************************
**                                                                     **
**                          fluss_offen                                **
**                     aufgerufen von : _offen_netz                    **
**                                                                     **
********************************************************************** */

fluss_offen()
{
	extern int f_o_single();      /* Bestimme Fluesse in einem offenen Netz
					 mit einer Auftragsklasse */
	extern int f_o_multi();       /* Bestimme Fluesse in einem offenen Netz
					 mit mehreren Auftragsklassen */


	if (netz.klasse == 1)
		f_o_single();       /* Netz mit einer Auftragsklasse */
	else
		f_o_multi();        /* Netz mit mehreren Auftragsklassen */

} /* ENDE fluss_offen */

/* **********************************************************************
**                                                                     **
**                          f_g_single                                 **
**                     aufgerufen von : _pseudo_offen                  **
**                                                                     **
********************************************************************** */

int f_g_single()
{                              /* Bestimme Fluesse in einem zu dem
				  geschlossenne Netz "flussaequivalenten"
				  offenen Netz */

	extern int ber_auslastung();   /* Berechnung Auslastung pro Knoten    */
	extern int ber_zwischenankunft(); /* Berechnung der Zwischenankunfts-
					     zeiten                           */
	extern int feedback();         /* Beseitigen von Schleifen            */
	extern int init_var();         /* Initialisierung der multiplikativen
                                  Konstanten und der Variations-
					  koeffizienten                       */
	extern int init_mul();         /* Initialisierung der multiplikativen
					  Konstanten                          */
	extern int ber_dquadvar();     /* Berechnung quadr. Variations-
					  koeffizient des Abgangsprozesses    */
	extern int newton_raphson();   /* Bestimmung der multiplikativen 
				  Konstante mit Hilfe des Newton Ver-
				  fahrens, anwendbar bei geschlossenen 
					  Netzen mit einer Auftragsklasse    */
	extern int ber_aquadvar();     /* Bestimmung quadr. Variations-
				  koeffizient des Ankunftsprozesses  
					  in einem geschlossenem Netz         */
	extern int ber_lagrange();     /* Berechnung der Lagrangekoeffizienten
					  fuer in geschlossenes Netz          */
	extern int sp_f_g_single();    /* Speichervergabefunktion */
	int konvergenz;                /* zeigt an, ob Berechnung der quadrierten
					  Variationskoeffizienten erfolgreich war */


	/* *** Beginn *** */

	sp_f_g_single();

	/* Speicherbelegung */

	ber_auslastung();
	/* Berechnung der Auslastung eines Knotens */

	ber_zwischenankunft();

	/* Berechnung der Zwischenankunftsrate 
					 Knoten --> Knoten */

	feedback();
	/* Beseitigen von Schleifen, Ruecklaeufen */

	init_mul();
	/* Initialsierung der multiplikativen Konstanten
			  bei offenen Netzen dient dies dazu, die Formeln
			  fuer die quadrierten Variationskoeffizienten
			  benutzen zu koennen */

	init_var();
	/* Initialisierung der Variationskoeffizienten */

	konvergenz = FALSE;

	/* Abbruchkriterium */

	do
	{
		newton_raphson();

		/* Bestimme die multiplikativen Konstanten mit Hilfe des
			  Newton_Raphson_Verfahrens fuer eine Auftragsklasse */

		ber_dquadvar();

		/* Berechnung des quadrierten Variationskoeffizienten
			 des Abgangsprozesses */

		konvergenz = ber_aquadvar();

		/* Berechnung des quadrierten Variationskoeffizineten
			 des Ankunftsprozesses */

	} while (!konvergenz);

	/* Abbruch falls die Variationskoeffizienten hinreichend
			 genau sind */

	ber_auftrag();
	/* Berechnung der mittleren Anzahl von Auftraegen in 
			 den einzelnen Knoten */

	ber_lagrange();
	/* Berechnung der Lagrangemultiplikatoren */

	/* *** Ende *** */

} /* ENDE f_g_single */

/* **********************************************************************
**                                                                     **
**                          pseudo_offen                               **
**                     aufgerufen von : _geschl_netz                   **
**                                                                     **
********************************************************************** */

pseudo_offen()
{
	extern int f_g_single();      /* Bestimme Fluesse in einem pseudo_offenen Netz
					 mit einer Auftragsklasse */
	extern int f_g_multi();       /* Bestimme Fluesse in einem pseudo_offenen Netz
					 mit mehreren Auftragsklassen */


	if (netz.klasse == 1)
		f_g_single();       /* Netz mit einer Auftragsklasse */
	else
		f_g_multi();        /* Netz mit mehreren Auftragsklassen */

} /* ENDE pseudo_offen */

/* **********************************************************************
**                                                                     **
**                          geschlossen                                **
**                     aufgerufen von : geschl_netz                    **
**                                                                     **
********************************************************************** */

int geschlossen()
{                       /* Bestimme mittlere Anzahl von Auftraegen und
			   Auslastung der einzelnen Knoten mit Hilfe der
			   Normalisierungskonstanten */

	extern int normalisierungskonstante(); /* Berechnung der Normalisierungs-
					  konstanten */
	extern int ber_leistung();     /* Bestimmung der Leistungsgroessen 
				  Auslastung und mittlere Anzahl von 
				  Auftraegen in einem geschlossenen 
					  Netz                                */
	extern int test_workrate();    /* pruefen ob Lavenbergsches Workrate
					  Theorem erfuellt ist                */
	extern int sp_geschlossen();   /* Speichervergabefunktion */

	int workrate;                  /* zeigt an, ob Work-Rate Theorem von Lavenberg
					  erfuellt ist */


	/* *** Beginn *** */

	sp_geschlossen();
	/* Speicherbelegung */

	workrate = TRUE;
	do
	{
		normalisierungskonstante();

		/* Berechnung der Normalisierungskonstanten */

		ber_leistung();

		/* Berechne Auslastung und mittlere Anzahl der
			 Auftraege in den einzelnen Knoten mit Hilfe
			 der Normalisierungskonstante */

		workrate = test_workrate();

		/* Test ob Work_Rate Theorem von Lavenberg erfuellt ist */

	} while (!workrate);

	/* Wiederhole bis Work_Rate Theorem erfuellt ist */

	/* *** Ende *** */

} /* ENDE geschlossen */

/* **********************************************************************
**                                                                     **
**                          f_g_multi                                  **
**                     aufgerufen von : _pseudo_offen                  **
**                                                                     **
********************************************************************** */

int f_g_multi()
{                       /* Analysiere Fluesse in einem geschlossenen Netz
			   mit mehreren Auftragsklassen */
	/* Es wird ein "flussaequivalentes offenes Netz
			   betrachtet */

	extern int ber_auslastung();   /* Berechnung Auslastung pro Knoten    */
	extern int init_var();         /* Initialisierung der quadrierten
                                  Variationskoeffizienten des Ankunfts-
					  prozesses */
	extern int init_mul();         /* Initialisierung der multiplikativen
					  Konstanten                          */
	extern int compc2acs();        /* Bestimmung quadr. Variations-
				  koeffizient des Ankunftsprozesses pro
					  Klasse und Knoten */

	extern int compc2dcs();        /* Berechnung quadr. Variations-
				  koeffizient des Abgangsprozesses 
					  pro Klasse und Knoten */
	extern int comppocs();         /* Berechnung der Wahrscheinlichkeit p_cs(0)
                                  dafuer, dass kein Auftrag der Klasse c
					  im Knoten s ist */
	extern int iter_newton();      /* Bestimmung der multiplikativen 
				  Konstante mit Hilfe des Newton Ver-
				  fahrens, anwendbar bei geschlossenen 
					  Netzen mit mehreren Auftragsklassen    */
	extern int ber_auftrag();      /* Berechne mittlere Anzahl von Aufraegen
					  pro Knoten */
	extern int ber_lagrange();     /* Berechnung der Lagrangekoeffizienten */
	extern int norm_ankunftsrate(); /* Gewichte Ankunftsraten mit der 
					   Auftragszahl */
	extern int sp_f_g_multi();     /* Speichervergabefunktion */
	int konvergenz;                /* zeigt an, ob Berechnung der quadrierten
					  Variationskoeffizienten erfolgreich war */

	/* *** Beginn *** */

	sp_f_g_multi();
	/* Speicherbelegung */

	norm_ankunftsrate(); /* Zur Besseren Berechnung der multiplikativen
				    Konstanten */

	ber_auslastung();
	/* Berechnung der Auslastung eines Knotens */


	init_mul();
	/* Initialsierung der multiplikativen Konstanten
			  bei offenen Netzen dient dies dazu, die Formeln
			  fuer die quadrierten Variationskoeffizienten
			  benutzen zu koennen */

	init_var();
	/* Initialisierung der Variationskoeffizienten */

	konvergenz = FALSE;

	/* Abbruchkriterium */

	do
	{
		iter_newton();
		/* Bestimme die multiplikativen Konstanten mit
			 Hilfe des iterativen Newtonverfahrens und
			 der Jacobimatrix */

		comppocs();     /* Berechnung p_cs(0)
			 Wahrscheinlicheit, dass kein Auftrag der Klasse
					 c im Knoten S ist */

		compc2dcs();
		/* quadrierter Variationskoeffizient
			 des Abreiseprozesses */

		konvergenz = compc2acs();

		/* quadrierter Variationskoeffizient
			 des Ankunftsprozesses */


	} while (!konvergenz);

	/* Abbruch falls die Variationskoeffizienten hinreichend
			 genau sind */

	/* Nun sind die quadrierten variationskoeffizienten des 
			 Ankunfs- und des Abgangasprozesse bestimmt und damit
			 die Fluesse im Netz */

	ber_auftrag();
	/* Berechne mittlere Auftragszahl pro Knoten
			 und Klasse */

	ber_lagrange();  /* Berechnung der Lagrangekoeffizienten */

	/* *** Ende *** */

} /* ENDE f_g_multi */

/* **********************************************************************
**                                                                     **
**                           offen_netz                                **
**                     aufgerufen von : _main()                        **
**                                                                     **
********************************************************************** */

int offen_netz()
{                                /* Berechnung der Leistungsgroessenm eines
				    offenen Netzes mit der Maximum Entropie 
				    Methode */

	extern int arate();             /* Bestimme Ankunftsraten durch Loesen der
				            Flussgleichgewichtsgleichungen */
	extern int fluss_offen();    /* Bestimme die Fluesse in dem offenen Netz */
	extern int ber_ergebnis();   /* Berechne saemtliche Leidtungsgroesen des offenen
					Netzes */


	arate();                   /* Loese Flussgleichgewichtsgleichungen
				        und Berechne die Ankunftsraten pro Knoten */
	fluss_offen();             /* Bestimme die quadrierten Variations-
			        koeffizienten der Ankunftsprozesse und der
		         	Abreiseprozesse und berechne die mittlere Anzahl
					von Auftraegen in den einzelne Stationen */
	ber_ergebnis();            /* Berechne Leistungsgroessen des offenen Netzes */

} /* ENDE offen_netz */

/* ******************************************************************
**                                                                 **
**                     o_ergebnis                                  **
**               aufgerufen von : ber_ergebnis                     **
**                                                                 **
****************************************************************** */

o_ergebnis()
{                               /* Berechne Leistungsgroessen eines
				   offenen Netzes */
	int i,r;                        /* Zaehlvariablen i : Knoten r : Klassen */
	double gesamtankunft;           /* Hilfsvariable */
	extern int feed;                /* zeigt an ob Ruecklaufkorektur
					   stattgefunden hat */
	extern int null_erg();          /* Setzt mittlere Verweilzeit
					 mittlere Warteschlangenlaenge
						 mittlere Wartezeit gleich Null */


	if (feed == TRUE )
		for(i=0;i<netz.knoten;i++)
			for(r=0;r<netz.klasse;r++)
			{
				netz.ankunftsrate[r][i] = netz.altankunftsrate[r][i];
				netz.bedienrate[r][i] = netz.altbedienrate[r][i];
			}
	/* Stelle alten Wert von der Bedienrate wieder
				    her, und von der Ankunftsrate */

	for(r=0;r<netz.klasse;r++)
	{
		gesamtankunft = 0.0;
		for(i=0;i<netz.knoten;i++)
			gesamtankunft += netz.e_ankunftsrate[r][i];

		/* Berechne Gesamtankunftsrate einer Klasse */

		for(i=0;i<netz.knoten;i++)
		{
			/*
			netz.bsh[r][i] = netz.ankunftsrate[r][i] / gesamtankunft;
			*/

			/* Berechne Besuchshaeufigkeit */

			if (netz.auslastung[r][i] == 0.0 )
				null_erg(r,i);         /* Setze Werte auf Null */
			else
			{
				netz.verweilzeit[r][i]=netz.anzauftrag[r][i]/netz.ankunftsrate[r][i];

				/* Verweilzeit pro Klasse und Knoten */

				netz.wartezeit[r][i]=netz.verweilzeit[r][i]-1/netz.bedienrate[r][i];

				/* Wartezeit in der Warteschlange */

				netz.warteschlange[r][i]=netz.ankunftsrate[r][i]*netz.wartezeit[r][i];

				/* mittlere Warteschlangenlaenge */
			} /* ENDE else */
		} /* ENDE for(i) */
	} /* ENDE for(r) */


} /* ENDE o_ergebnis */

/* ******************************************************************
**                                                                 **
**                     g_ergebnis                                  **
**               aufgerufen von : _ber_ergebnis                    **
**                                                                 **
****************************************************************** */

g_ergebnis()
{                              /* Berechne Leistungsgroessen in einem
			          geschlossenen Netz */
	int i,r;                       /* Zaehlvariablen i : Knoten r: Klassen */
	double hilf;                   /* Hilfsvariable */

	extern int feed;               /* zeigt an, ob Ruecklaufkorektur statt-
					  gefunden hat */
	extern double **besuch;        /* Wahrscheinlichkeit, dass ein Station s 
					  besuchender Auftrag der Klasse c angehoert */
	extern double *ankunftsrate;   /* Ankunftsrate pro Knoten */
	extern double *auslastung;     /* Auslastung pro Knoten */
	extern double *bedienrate;     /* Bedienrate pro Knoten */
	extern double *anzauftrag;     /* mittlere Anzahl von Auftraegen pro Knoten */
	extern double *mulkonst;       /* multiplikative Konstante */

	extern int ber_ankunftsrate(); /* Ankunftsrate *.
	extern int ber_bedienrate();   /* Bedienrate */
	extern int ber_besuch();       /* Berechnung der Wahrscheinlichkeit,dass ein
				  in Station s ankommender Auftrag der Klasse
					  c angehoert */
	extern int null_erg();         /* Setzt mittlere Verweilzeit
	                                mittlere Warteschlangenlaenge
				                mittlere Wartezeit gleich Null */


	if (netz.klasse > 1)
	{
		ber_ankunftsrate();

		/* Berechne Ankunftsrate */

		ber_bedienrate();
		/* Berechne Bedienraten */

		ber_besuch();
		/* Berechne Wahrscheinlichkeit, dass ein den Knoten i
			   besuchender Auftrag der Klasse r angehoert */

	} /* ENDE if */

	for(i=0;i<netz.knoten;i++)
		ankunftsrate[i] = auslastung[i] * bedienrate[i];

	/* Berechne Ankunftsrate */


	for(i=0;i<netz.knoten;i++)
	{
		for(r=0;r<netz.klasse;r++)
		{
			if (netz.klasse > 1)
			{
				netz.ankunftsrate[r][i] = ankunftsrate[i] * besuch[r][i];

				/* Ankunftsrate pro Klasse und Knoten */

				netz.auslastung[r][i]=netz.ankunftsrate[r][i]/netz.bedienrate[r][i];

				/* Auslastung pro Klasse und Knoten */

				hilf = besuch[r][i]*(anzauftrag[i] - auslastung[i]);
				netz.anzauftrag[r][i] = netz.auslastung[r][i] + hilf;
			} /* ENDE if (klasse) */

			/* mittlere Anzahl von Auftraegen der Klasse r
			im Knoten i */

			/*
			netz.bsh[r][i] = 0.0;
			*/

			/* Besuchshauefigkeit */

			if (feed == TRUE)
			{
				netz.bedienrate[r][i] = netz.altbedienrate[r][i];

				/* Stelle alten Wert der Bedienrate wieder her */

				netz.ankunftsrate[r][i]=netz.auslastung[r][i] * netz.bedienrate[r][i];

				/* Berechne neue Ankunftsrate */
			}

			if (netz.auslastung[r][i] == 0.0)
				null_erg(r,i);            /* Setze Leistungsgroessen der Klasse r
							               und des Knotens i aus 0 */
			else
			{
				netz.verweilzeit[r][i]=netz.anzauftrag[r][i]/netz.ankunftsrate[r][i];

				/* Verweilzeit pro Klasse und Knoten */

				netz.wartezeit[r][i]=netz.verweilzeit[r][i]-1/netz.bedienrate[r][i];

				/* Wartezeit in der Warteschlange */

				netz.warteschlange[r][i]=netz.ankunftsrate[r][i]*netz.wartezeit[r][i];

				/* mittlere Warteschlangenlaenge */
			} /* ENDE else(bedienrate) */
		} /* Ende for(r) */
	} /* Ende for(i) */


} /* ENDE g_ergebnis */

/* *************************************************************************
**                                                                        **
**                              null_erg                                  **
**                    aufgerufen von : o_ergebnis, g_ergebnis             **
**                                                                        **
************************************************************************* */

null_erg(r,i)
int r;                /* Klasse */
int i;                /* Knoten */
{                     /* Setze Leistungsgroessen auf Null */

	netz.ankunftsrate[r][i]  = 0.0;    /* Ankunftrate */
	netz.auslastung[r][i]    = 0.0;    /* Auslastung  */
	netz.anzauftrag[r][i]    = 0.0;    /* mittlere Anzahl von Auftraegen
					           der Klasse r im Knoten i */
	netz.verweilzeit[r][i]   = 0.0;    /* Verweilzeit */
	netz.wartezeit[r][i]     = 0.0;    /* Wartezeit   */
	netz.warteschlange[r][i] = 0.0;    /* Warteschlangenlaenge */

} /* ENDE null_erg */

/* ******************************************************************
**                                                                 **
**                     ber_ergebnis                                **
**               aufgerufen von : _offen_netz                      **
**                                _geschl_netz                     **
**                                                                 **
****************************************************************** */

ber_ergebnis()
{                        /* Berechnung der restlichen Leistungsgroessen */
	extern int o_ergebnis(); /* ... fuer ein offenes Netz */
	extern int g_ergebnis(); /* ... fuer ein geschlossenes Netz */
	extern int sp_ergebnis(); /* Speichervergabefunktion */

	sp_ergebnis();
	/* Speichervergabe */

	if (netz.typ == OFFEN)
		o_ergebnis();        /* Berechne restliche Leistungsgroessen fuer das offene
					    Netz */
	else
		g_ergebnis();        /* Berechne restliche Groessen fuer das geschlosssene
				    Netz */

} /* ENDE ber_ergebnis */

/* **********************************************************************
**                                                                     **
**                           geschl_netz                               **
**                     aufgerufen von : _main()                        **
**                                                                     **
********************************************************************** */

int geschl_netz()
{                                /* Berechnung der Leistungsgroessen eines 
				    geschlossenen Netzes mit der 
				    Maximum Entropie Methode */

	extern int arate();              /* Bestimme Ankunftsraten durch Loesen der
				            Flussgleichgewichtsgleichungen */
	extern int pseudo_offen();       /* Analyse des psedooffenen Netzes */
	extern int geschlossen();        /* Angleichung der Ergebnisse von _pseudo_offen
					    auf eun geschlossenes Netz */
	extern int ber_ergebnis();       /* Berechne die Leistungsgroessen */

	arate();                  /* Loese Flussgleichgewichtsgleichungen
				       und Berechne die Ankunftsraten pro Knoten */
	pseudo_offen();            /* Bestimme die Fluesse in dem "flussaequivalenten"
				       Pseudo_offenen Netz */
	geschlossen();            /* Berechne die Leistungsgroessen (Auslastung und
			       mittlere Anzahl von Auftraegen in den Knoten) 
				       des geschlossenen Netzes */
	ber_ergebnis();           /* Berechne weitere Leistungsgroessen */

} /* ENDE geschl_netz */
