/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tmem_norm.c\t1.2\t92/03/27 11:49:16";

/* ****************** Datei mem_norm.c ******************** */
/* Inhalt                                                   */
/*       init_norm()           ber_norm()                   */
/*       normalisierungskonstante() ber_hilf()              */
/*       init_f()              ber_anzauftrag()             */
/*       ber_ausnutzung()      lgroessen()                  */
/*       ber_leistung()        test_workrate()              */
/* ******************************************************** */

/* ***********************************************************
**                                                          **
**                       Include Dateien                    **
**                                                          **
*********************************************************** */

#include "stdio.h"
#include "mem_netz.h"
#include "mem_def.h"

/* ***********************************************************
**                                                          **
**                      externe Variablen                   **
**                                                          **
*********************************************************** */

extern double *auslastung;    /* Auslastung des Knotens i */
extern double *mul;          /* Auslastung des Knotens i in einem
				Pseudo-offenem Netz, abhaengig von
				_mulkonst */
extern double *anzauftrag;   /* Anzahl der Auftraege im Knoten i */
extern double *x_lagrange;   /* Lagrange Multiplikator */
extern double *g_lagrange;   /* Lagrange Multiplikator */
extern double *normkonst;    /* Normalisierungskonstante */
extern double *hilfvek;      /* Hilfsvektor zur Bestimmung der 
				Normalisierungskonstanten */
extern double *f;            /* Hilfsvektor zur Bestimmung der
				Normalisierungskonstanten */
extern int    auftrag;       /* Anzahl der Auftraege im Netz */

/* ******************************************************************
**                                                                 **
**                       init_norm                                 **
**           aufgerufen von : _normalisierungskonstante            **
**                                                                 **
****************************************************************** */

init_norm()
{             /* Initialisierung zur Bestimmung der Normalisierungs-
                 konstanten mit Hilfe des Faltungsalgorithmusses */

	int k;     /* Laufvariable k : Auftrag  */


	normkonst[0] = 1;
	normkonst[1] = x_lagrange[0] * g_lagrange[0];
	for(k=2;k<=auftrag;k++)
		normkonst[k] = normkonst[k-1] * x_lagrange[0];

} /* ENDE init_norm */

/* *****************************************************************
**                                                                **
**                     ber_norm                                   **
**      aufgerufen von : _normalisierungskonstante                **
**                                                                **
***************************************************************** */

ber_norm()
{                     /* Berechnung der Normalisierungskonstanten */

	int    k,i;          /* Laufvariable i : Knoten  k : Auftrag */
	double hilf;         /* Hilfsvariable */

	hilfvek[0] = 1;
	for(i=1;i<netz.knoten;i++)
		for(k=1;k<=auftrag;k++)
		{
			hilfvek[k] = normkonst[k];
			hilf = ((g_lagrange[i] -1) * hilfvek[k-1]) + normkonst[k-1];
			normkonst[k] = hilfvek[k] + x_lagrange[i]*hilf;
		} /* Ende for(k), Ende for(i) */

} /* ENDE ber_norm() */

/* ******************************************************************
**                                                                 **
**               normalisierungskonstante                          **
**                  aufgerufen von : _geschlossen                  **
**                                                                 **
****************************************************************** */

normalisierungskonstante()
{                         /* Berechne die Normalisierungskonstante */
	int init_norm();
	int ber_norm();


	init_norm();

	/* Initialisierung der Normalisierungskonstante */

	ber_norm();
	/* Berechne die Normalisierungskonstante */

} /* ENDE normalisierungkonstante */

/* ******************************************************************
**                                                                 **
**                     ber_hilf                                    **
**               aufgerufen von : ber_leistung()                   **
**                                                                 **
****************************************************************** */

ber_hilf(i)
int i;
{                   /* Berechnung des Hilfsvektors in Abhaengigkeit 
                       vom Knoten i */
	int    k;          /* Laufvariable k : Auftrag */
	double hilf;       /* Hilfsvariable */
	double glagrange;  /* Hilfsgroesse  */

	glagrange = g_lagrange[i] - 1;

	/* weniger Rechenaufwand */

	/* hilfsvek[0] == 1 */
	for(k=1;k<=auftrag;k++)
	{
		hilf = ((glagrange*hilfvek[k-1])+normkonst[k-1]) * x_lagrange[i];
		hilfvek[k] = normkonst[k] - hilf;
	} /* Ende for(k) */

} /* ENDE ber_hilf */

/* ******************************************************************
**                                                                 **
**                     init_f                                      **
**               aufgerufen von : ber_anzauftrag()                 **
**                                                                 **
****************************************************************** */

init_f(i)
int i;
{                         /* Hilfsvektor zur Bestimmung der mittleren
                             Anzahl von Auftraegen im Knoten i */

	int k;                   /* Laufvariable k: Auftrag */

	f[0] = 1;
	f[1] = x_lagrange[i] * g_lagrange[i];
	for(k=2;k<=auftrag;k++)
		f[k] = x_lagrange[i] * f[k-1];

} /* ENDE init_f */

/* *****************************************************************
**                                                                **
**                     ber_anzauftrag                             **
**               aufgerufen von : lgroessen                       **
**                                                                **
***************************************************************** */

ber_anzauftrag(i)
int i;                        /* i : Knoten */
{                             /* Berechnung mittlere Zahl von 
				 Auftraegen im Knoten i in
				 einem geschlossenem Netz mit Hilfe
				 der Normalisierungskonstanten */
	int k;                        /* Laufvariable k : Auftrag */
	int init_f();                 /* Initialisierung des Vektors _f */

	init_f(i);
	/* Initialisierung des Hilfvektors f,
				 Werte von f abhaengig vom Knoten */

	anzauftrag[i] = 0;
	/* alter,fuer das 'Pseudo-offene' Netz
                                 gespeicherter, Wert wird geloescht */

	for(k=1;k<=auftrag;k++)
		anzauftrag[i] += (hilfvek[auftrag - k ] * k * f[k] );
	anzauftrag[i] = anzauftrag[i] / normkonst[auftrag];

} /* ENDE ber_anzauftrag */

/* ******************************************************************
**                                                                 **
**                       ber_ausnutzung                            **
**               aufgerufen von : lgroessen                        **
**                                                                 **
****************************************************************** */

ber_ausnutzung(i)
int i;             /* i : Knoten */
{                  /* Berechnung der Auslastung des Knotens i, 
                      fuer das geschlossene Netz wird der alte Wert
                      der Auslastung ueberschrieben und ein Neuer wird
                      berechnet */

	auslastung[i] = 1 - hilfvek[auftrag]/ normkonst[auftrag];

	/* Beachte : hilfvek wird in Abhaengigkeit vom
                               Knoten i berechnet */

} /* ENDE ber_ausnutzung */

/* *****************************************************************
**                                                                **
**                     lgroessen                                  **
**         aufgerufen von : _ber_leistung                         **
**                                                                **
***************************************************************** */

lgroessen(i)
int i;                    /* i : Knoten */
{                         /* Berechnung von  Auslastung,mittlerer Anzahl
                             von Auftraegen im Knoten i */
	int ber_anzauftrag();    /* Funktion zur Berechnung der mittleren
			     Anzahl der Auftraege in einem bestimmten
				     Knoten */
	int ber_ausnutzung();    /* Funktion zur Berechnung der Auslastung
			     eines bestimmten Knoten mit Hilfe der
				     Normalisierungskonstante */

	/* *** BEGINN *** */

	ber_anzauftrag(i);
	/* Berechnung : 
			     Anzahl der Auftraege im Knoten i */
	ber_ausnutzung(i);
	/* Berechnung der Auslastung des Knotens i */

} /* ENDE lgroessen */

/* ******************************************************************
**                                                                 **
**                     ber_leistung                                **
**               aufgerufen von : _geschlossen                     **
**                                                                 **
****************************************************************** */

ber_leistung()
{                   /* Berechnung folgender Leistungsgroessen
                                  - Ausnutzung
                                  - mittlere Anzahl von Auftraegen,
                       fuer alle Knoten  */

	int i;             /* Laufvariable i : Knoten */
	int knoten ;       /* Hilfsgroesse */
	int lgroessen();   /* Berechnung der Auslastung und der mittleren
			       Auftragszahle in einem bestimmten Knoten */
	int ber_hilf();    /* Bestimmung des Vektors _hilfvek mit Hilfe
		       der Normalisierungskonstanten und den Lagrange-
			       koeffizienten eines bestimmten Knotens */

	/* *** BEGINN *** */

	knoten = netz.knoten - 1;

	/* weniger Rechenaufwand */

	lgroessen(knoten);

	/* Berechnung der Auslastung und der mittleren
		       Anzahl von Aufraegen des Knotens mit der
		       hoechsten Nummer */

	for(i=0;i<knoten;i++)
	{
		ber_hilf(i);

		/* Berechnung des Hilfsvektors _hilfvek */

		lgroessen(i);

		/* Berechnung Auslastung und mittlere Anzahl von
		       Auftraegen des Knoten i */

	} /* Ende for(i) */

} /* ENDE ber_leistung */

/* *******************************************************************
**                                                                  **
**                     test_workrate                                **
**               aufgerufen von : _geschlossen                      **
**                                                                  **
******************************************************************* */

int test_workrate()
{                 /* Test ,ob 'workrate' Theorem erfuellt ist */

	int    i;        /* Zaehlvariable i : Knoten */
	int    workrate; /* Anzeige ob Lavenberg Theorem erfuellt */
	double sum,galt; /* Hilfsvariablen, 'galt'nimmt den alten Wert 
	                     von g_lagrange auf */

	workrate = TRUE;     /* Annahme 'workrate' Theorem gelte */

	sum = 0;
	for(i=0;i<netz.knoten;i++)
	{
		sum += (anzauftrag[i] * mul[i] / auslastung[i]);
	}
	/* Hilfsgroesse, 
			   konstant fuer alle Knoten */

	for(i=0;i<netz.knoten;i++)
	{
		galt          = g_lagrange[i];

		/* Festhalten des alten Wertes */

		g_lagrange[i] =(galt*mul[i]*auftrag)/(sum*auslastung[i]);

		/* Berechnung eines neuen Lagrangekoeffizienten
			   _g_lagrange ,sodass Workrate Theorem gilt */

		if ((abs(g_lagrange[i] - galt)) > SUMTOLERANZ)
			workrate = FALSE;

		/* Test ob Work-Rate Theorem erfuellt */

	} /* Ende for(i) */


	return(workrate);

} /* ENDE test_workrate */
