/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tmem_speich.c\t1.2\t92/03/27 11:49:18";

/* ***************************************************************
**                                                              **
**                     DATEI mem_speich.c                       **
**                                                              **
**          enthaelt alle Speicherverwaltungsfunktionen         **
**                                                              **
*************************************************************** */
/* **************************************************************
**                                                              **
**                         Include Dateien                      **
**                                                              **
*************************************************************** */

#include "stdio.h"
#include "mem_def.h"                /* Konstanten und Macros */
#include "mem_netz.h"               /* Netzstruktur */

/* *************************************************************
**                                                            **
**                externe Variablen                           **
**                                                            **
************************************************************* */


/* **************    DOUBLE Vektoren         **************** */

extern double *ankunftsrate;        /* Ankunftsrate pro Knoten */
extern double *bedienrate;          /* Bedienrate pro Knoten */
extern double *auslastung;          /* Auslastung des Knotens i */
extern double *mul;                 /* Auslastung des Knotens i in einem
				       Pseudo-offenem Netz, abhaengig von
				       _mulkonst */
extern double *mulkonst;            /* multiplikative Konstanten pro Klasse */
extern double *anzauftrag;          /* Anzahl der Auftraege im Knoten i */
extern double *x_lagrange;          /* Lagrange Multiplikator */
extern double *g_lagrange;          /* Lagrange Multiplikator */
extern double *s_quadvar;           /* quadr. Variationskoeffizient des
				       Bedienprozessses */
extern double *a_quadvar;           /* quadr. Variationskoeffizient des
				       Ankunftsprozesses */
extern double *d_quadvar;           /* quadr. Variationskoeffizient des 
				       Abgangsprozesses */
extern double *normkonst;           /* Normalisierungskonstante */
extern double *hilfvek;             /* Hilfsvektor zur Bestimmung der 
				       Normalisierungskonstanten */
extern double *f;                   /* Hilfsvektor zur Bestimmung der
				       Normalisierungskonstanten */
extern double *bvek;                /* Rechte Seite eines inhomogenen
			               Gleichungssystems */
extern double *fvek;                /* Vektor der Summen der mittleren
				       Auftragszahl im System pro Klasse */
/* rechte Seite des Gleichungssystems
			               J _mvek = _fvek */
/* wobei J Jacobimatrix 
			               _mvek Loesungsvektor */
extern double *mvek;                /* enthaelt Loesung des Gleichungssystems
                                       J _mvek = _fvek */
extern double *sigmacs;             /* sigma_cs */
extern double *taucs;               /* tau_cs */

/* **************    DOUBLE FELDER        **************** */

extern double **besuch;             /* Wahrscheinlickeit, dass ein in Knoten i
				       eintreffender Auftrag der Klasse r 
				       angehoert */
extern double **z_ankunft;          /* Zwischenankunftsrate
				         Knoten --> Knoten */
extern double **uebwkt;             /* Uebergangsmatrix
				         Knoten --> Knoten */
extern double **wa;                 /* Hilfsmatrix, verhindert, dass
				       Uebergangsmatrix _uebwkt veraendert
				       wird */
extern double **jacobi;             /* Jacobi Matrix
			               Dimension :
				          Anzahl Klassen X Anzahl Klassen */
extern double **pocs;               /* Wahrscheinlichkeit p_cs(0) dafuer,
				       dass kein Aufrag der Klasse c sich im,
			       	       Knoten s befindet */

/* ****************************************************************
**                                                               **
**                              free0                            **
**              aufgerufen von : free_????                       **
**                                                               **
**************************************************************** */

free0(na)
double **na;             /* Adresse eines eindim. Feldes */
{                        /* Freigabe des durch speich0 belegten
			    Speicherplatzes*/
	if (*na != 0)
		free((char *) *na);
} /* ENDE free0 */

/* ****************************************************************
**                                                               **
**                              free1                            **
**              aufgerufen von : free_????                       **
**                                                               **
**************************************************************** */

free1(nam,groesse)
double ***nam;               /* Adresse zweidim. Feld */
int    groesse;              /* Feldgroesse : 1.Komponente   */
{                            /* Freigabe des von speich1 belegten
				Speicherplatz */
	int i;

	for(i=0;i<groesse;i++)
	{
		if ((*nam)[i] != 0)
			free((char *)(*nam)[i]);
	}

	if (*nam != 0)
		free((char *)*nam);
}     /* ENDE free1 */

/* ****************************************************************
**                                                               **
**                              free2                            **
**              aufgerufen von : free_????                       **
**                                                               **
**************************************************************** */

free2(na,groesse1,groesse2)
double **** na;            /* Adresse dreidim. Feld */
int    groesse1;           /* Feldgroesse    1. Dimension */
int    groesse2;           /* Feldgroesse    2. Dimension */
{
	int i,j;

	for(i=0;i<groesse1;i++)
	{
		for(j=0;j<groesse2;j++)
			if ((*na)[i][j] != 0)
				free((char *) (*na)[i][j]);
		if ((*na)[i] != 0)
			free((char *) (*na)[i]);
	} /* ENDE for(i) */
	if ( *na != 0 )
		free((char *) *na);
} /* ENDE free(2)
 
/* ****************************************************************
**                                                               **
**                             speich0                           **
**              aufgerufen von : sp_????                         **
**                                                               **
**************************************************************** */

int speich0(name,groesse,typ)
double **name;                      /* Adresse eindim. Feld */
int    groesse;                     /* Groesse des Feldes */
int    typ;                         /* Groesse der Feldkomponenten */
{                                   /* Belegt Speicherplatz fuer
				       ein eindim. Feld , falls
				       zu wenig Speicherplatz vorhanden
					  Rueckgabewert -1
				       sonst 
					  Rueckgabewert 1 */

	*name = (double *)calloc(groesse,typ);
	if (*name == 0)
		return(-1);
	else
		return(1);
} /* ENDE speich0 */

/* ****************************************************************
**                                                               **
**                           speich1                             **
**              aufgerufen von : sp_????                         **
**                                                               **
**************************************************************** */

int speich1(bez,groesse1,groesse2,typ)
double ***bez;            /* Adresse zweidim. Feld */
int    groesse1;          /* Groesse der ersten Dimension */
int    groesse2;          /* Groesse der zweiten Dimension */
int    typ;               /* Groesse der einzelnen Komponenten */
{                         /* Belegt Speicherplatz fuer ein
			     zweidim. Feld */
	int i;                   /* Zaehlvariable */
	int ok=1;                /* Hilfswert */
	int speich0();           /* belegt Speicherplatz fuer ein 
				     eindimensionales Feld */

	/* *** Beginn *** */

	*bez = (double **)calloc(groesse1,typ);
	if (*bez == 0)
		ok = -1;
	else
	{
		for(i=0;i<groesse1;i++)
		{
			ok =  speich0(&(*bez)[i],groesse2,typ);
			if (ok == -1)
				break;
		}
	}
	return(ok);
} /*  ENDE speich1 */

/* ****************************************************************
**                                                               **
**                             speich2                           **
**              aufgerufen von : sp_????                         **
**                                                               **
**************************************************************** */

int speich2(bez,groesse1,groesse2,groesse3,typ)
double ****bez;         /* Adresse des dreidim. Feldes */
int    groesse1;        /* Groesse der ersten Dimension */
int    groesse2;        /* Groesse der zweiten Dimension */
int    groesse3;        /* Groesse der dritten Dimension */
int    typ;             /* Groesse der einzelnen Komponenten */
{                       /* Belegt Speicherplatz fuer ein
   			   dreidimensionales Feld */
	int i;                 /* Zaehlvariable */
	int ok=1;              /* Hilfsvariable */
	int speich1();         /* Belegt Speicherplatz fuer ein
				   dreidimensonales Feld */

	/* *** Beginn *** */

	*bez = (double ***) calloc(groesse1,typ);
	if ( *bez == 0)
		ok = -1;
	else
	{
		for(i=0;i<groesse1;i++)
		{
			ok = speich1( &(*bez)[i],groesse2,groesse3,typ);
			if ( ok == -1)
				break;
		}
	}
	return(ok);
} /* ENDE speich2 */
/* *******************************************************************
**                                                                  **
**                             pruef                                **
**                 aufgerufen von : sp_???????                      **
**                                                                  **
******************************************************************* */

pruef(k)
int k;
{                              /* Fehlerausgabe und Programmabbruch, falls nicht
				  genuegend Speicherplatz frei ist */

	if ( k == -1 )
	{
		fprintf(stderr,"%s \n",FEHLER_SPEICHER);
		printf("mem_speicher\n 0 0 0\n");
		exit(1);
	}

} /* ENDE pruef */

/* ******************************************************************
**                                                                 **
**                      sp_eingabe                                 **
**             aufgerufen von : eingabe                            **
**                                                                 **
****************************************************************** */

sp_eingabe()
{                                   /* Belegt Speicherplatz fuer die
				       zur Eingabe benoetigten Variablen */
	int knoten,klasse;                 /* Hilfsvariablen */

	knoten = netz.knoten;
	klasse = netz.klasse;

	/* Abkuerzungen */

	if (netz.typ == GESCHLOSSEN)
	{
		pruef(speich0(&netz.auftrag,klasse,int_size));

		/* fuer Anzahl der Auftraege im System */
	}
	else /* Netz OFFEN */
	{
		pruef(speich1(&netz.e_ankunftsrate,klasse,knoten,double_size));

		/* fuer externe Ankunftsrate pro Klasse und Knoten */

		pruef(speich1(&netz.e_a_quadvar,klasse,knoten,double_size));

		/* fuer quadrierter Variationskoeffizient pro Klasse
			      und Knoten des aeusseren Ankunfsprozesses */
	} /* ENDE else */

	pruef(speich1(&netz.bedienrate,klasse,knoten,double_size));

	/* fuer die Bedienrate pro Klasse und Knoten */

	pruef(speich1(&netz.s_quadvar,klasse,knoten,double_size));

	/* fuer den quadrierten Variationskoeffizienten der
			      Bedienzeitverteilung pro Knoten */

	pruef(speich2(&netz.uebwkt,klasse,knoten,knoten,double_size));

	/* fuer die Uebergangswahrscheinlichkeiten von
			      Knoten --> Knoten pro Klasse */

	pruef(speich1(&netz.bsh,klasse,knoten,double_size));

	/* fuer die Besuchshaeufigkeiten */
} /* ENDE sp_eingabe() */

/* ******************************************************************
**                                                                 **
**                         sp_arate                                **
**                aufgerufen von :_arate                           **
**                                                                 **
****************************************************************** */

sp_arate()
{              /* Belegt Speicherplatz fuer Ankunftsrate ,
		  Matrix  _wa und Vektor _bvek */
	int knoten,klasse;                 /* Hilfsvariablen */

	knoten = netz.knoten;
	klasse = netz.klasse;

	/* Abkuerzungen */

	pruef(speich1(&netz.ankunftsrate,klasse,knoten,double_size));

	/* Speicherplatz fuer Ankunftrate pro Klasse und
			       Knoten */

	pruef(speich1(&wa,knoten,knoten,double_size));

	/* Speicherplatz fuer Martix wa : Hilfsmatrix,
			       damit Originalmartix nicht zerstoert wird */

	pruef(speich0(&bvek,knoten,double_size));

	/* Speicherplatz fuer Vektor _bvek : rechte
			       Seite eines Gleichumgssystems */
} /* ENDE sp_arate */

/* ********************************************************************
**                                                                   **
**                      sp_f_o_single                                **
**             aufgerufen von : f_o_single                           **
**                                                                   **
******************************************************************** */

sp_f_o_single()
{                 /* Speicherplatz fuer ... */
	int knoten,klasse;                 /* Hilfsvariablen */

	knoten = netz.knoten;
	klasse = netz.klasse;

	/* Abkuerzungen */


	pruef(speich1(&netz.auslastung,klasse,knoten,double_size));

	/* Auslastung pro Klasse und Knoten */

	pruef(speich1(&z_ankunft,knoten,knoten,double_size));

	/* Zwischenankunftsrate Knoten --> Knoten  */

	pruef(speich0(&mulkonst,klasse,double_size));

	/* Multiplikative Konstanten  */

	pruef(speich0(&a_quadvar,knoten,double_size));

	/* quadr. Variationskoeffizient des Ankunftsprozesses  */

	pruef(speich0(&d_quadvar,knoten,double_size));

	/* quadr. Variationskoeffizient des Abgangsprozesses  */

	pruef(speich1(&netz.anzauftrag,klasse,knoten,double_size));

	/* mittlere Anzahl von Auftraegen im Knoten s */

	auslastung = netz.auslastung[0];

	/* Auslastung pro Knoten */

	mul = auslastung;

	/* Auslastung pro Knoten */

	anzauftrag = netz.anzauftrag[0];

	/* mittlere Anzahl von Auftraegen */

	s_quadvar = netz.s_quadvar[0];

	/* quadr. Variationskoeffizient des Bedienprozesses */

	bedienrate = netz.bedienrate[0];

	/* Bedienrate pro Knoten */

	ankunftsrate = netz.ankunftsrate[0];

	/* Ankunftsrate pro Knoten */

	uebwkt = netz.uebwkt[0];

	/* Uebergangsmatrix  Knoten --> Knoten */

} /* ENDE sp_f_o_single */

/* ********************************************************************
**                                                                   **
**                      sp_f_g_single                                **
**                aufgerufen von : f_g_single                        **
**                                                                   **
******************************************************************** */

sp_f_g_single()
{                 /* Speicherplatz fuer ... */
	int knoten,klasse;                 /* Hilfsvariablen */

	knoten = netz.knoten;
	klasse = netz.klasse;

	/* Abkuerzungen */


	pruef(speich1(&netz.auslastung,klasse,knoten,double_size));

	/* Auslastung pro Klasse und Knoten */

	pruef(speich0(&mul,knoten,double_size));

	/* Auslastung in einem pseudo-offenem Netz */

	pruef(speich1(&z_ankunft,knoten,knoten,double_size));

	/* Zwischenankunftsrate Knoten --> Knoten  */

	pruef(speich0(&mulkonst,klasse,double_size));

	/* Multiplikative Konstanten  */

	pruef(speich0(&a_quadvar,knoten,double_size));

	/* quadr. Variationskoeffizient des Ankunftsprozesses  */

	pruef(speich0(&d_quadvar,knoten,double_size));

	/* quadr. Variationskoeffizient des Abgangsprozesses  */

	pruef(speich1(&netz.anzauftrag,klasse,knoten,double_size));

	/* mittlere Anzahl von Auftraegen im Knoten s */

	pruef(speich0(&x_lagrange,knoten,double_size));

	/* Lagrangekoeffizient */

	pruef(speich0(&g_lagrange,knoten,double_size));

	/* Lagrangekoeffizient */

	auslastung = netz.auslastung[0];

	/* Auslastung pro Knoten */

	s_quadvar = netz.s_quadvar[0];

	/* quadr. Variationskoeffizient des Bedienprozesses */

	anzauftrag = netz.anzauftrag[0];

	/* mittlere Anzahl von Auftraegen */

	bedienrate = netz.bedienrate[0];

	/* Bedienrate pro Knoten */

	ankunftsrate = netz.ankunftsrate[0];

	/* Ankunftsrate pro Knoten */

	uebwkt = netz.uebwkt[0];

	/* Uebergangsmatrix  Knoten --> Knoten */

} /* ENDE sp_f_g_single */



/* ********************************************************************
**                                                                   **
**                         sp_f_o_multi                              **
**                    aufgerufen von f_o_multi                       **
**                                                                   **
******************************************************************** */

sp_f_o_multi()
{                                   /* Speicherplatz fuer ... */
	int knoten,klasse;                 /* Hilfsvariablen */

	knoten = netz.knoten;
	klasse = netz.klasse;

	/* Abkuerzungen */


	pruef(speich1(&netz.auslastung,klasse,knoten,double_size));

	/* Auslastung pro Klasse und Knoten */

	pruef(speich1(&netz.a_quadvar,klasse,knoten,double_size));

	/* quadr. Variationskoeffizient des
				       klassenabhaenigen Ankunftsprozesses */

	pruef(speich1(&netz.d_quadvar,klasse,knoten,double_size));

	/* quadr. Variationskoeffizient des
                                      klassenabhaenigen Abgangsprozess */

	pruef(speich1(&netz.anzauftrag,klasse,knoten,double_size));

	/* mittlere Anzahl von Auftraegen pro Klasse
				       und Knoten */

	pruef(speich0(&sigmacs,klasse,double_size));

	/* sigma_cs */

	pruef(speich0(&taucs,klasse,double_size));

	/* tau_cs */

	pruef(speich1(&pocs,klasse,knoten,double_size));

	/* p_cs(0) : Wahrscheinichkeit, dass sich
					 kein Auftrag der Klasse c im Knoten s 
					 befindet */

	pruef(speich0(&mulkonst,klasse,double_size));

	/* multiplikative Konstante */

	pruef(speich0(&auslastung,knoten,double_size));

	/* Auslastung pro Knoten */

	mul = auslastung;

	/* Auslastung */
} /* ENDE sp_f_o_multi */

/* ********************************************************************
**                                                                   **
**                         sp_f_g_multi                              **
**            aufgerufen von : f_g_multi                             **
**                                                                   **
******************************************************************** */

sp_f_g_multi()
{                                   /* Speicherplatz fuer ... */
	int knoten,klasse;                 /* Hilfsvariablen */

	knoten = netz.knoten;
	klasse = netz.klasse;

	/* Abkuerzungen */


	pruef(speich1(&netz.auslastung,klasse,knoten,double_size));

	/* Auslastung pro Klasse und Knoten */

	pruef(speich1(&netz.a_quadvar,klasse,knoten,double_size));

	/* quadr. Variationskoeffizient des
				       klassenabhaenigen Ankunftsprozesses */

	pruef(speich1(&netz.d_quadvar,klasse,knoten,double_size));

	/* quadr. Variationskoeffizient des
                                      klassenabhaenigen Abgangsprozess */

	pruef(speich1(&netz.anzauftrag,klasse,knoten,double_size));

	/* mittlere Anzahl von Auftraegen pro Klasse
				       Knoten */

	pruef(speich0(&anzauftrag,knoten,double_size));

	/* mittlere Anzahl von Aufr\"agen pro
				       Knoten */

	pruef(speich0(&sigmacs,klasse,double_size));

	/* sigma_cs */

	pruef(speich0(&taucs,klasse,double_size));

	/* tau_cs */

	pruef(speich1(&pocs,klasse,knoten,double_size));

	/* p_cs(0) : Wahrscheinichkeit, dass sich
					 kein Auftrag der Klasse c im Knoten s 
					 befindet */

	pruef(speich1(&jacobi,klasse,klasse,double_size));

	/* Jacobi Matrix */

	pruef(speich0(&fvek,klasse,double_size));

	/* Hilfsvektor */

	pruef(speich0(&mvek,klasse,double_size));

	/* Hilfsvektor */

	pruef(speich0(&x_lagrange,knoten,double_size));

	/* Lagrangekoeffizient */

	pruef(speich0(&g_lagrange,knoten,double_size));

	/* Lagrangekoeffizient */


	pruef(speich0(&mulkonst,klasse,double_size));

	/* multiplikative Konstante */

	pruef(speich0(&auslastung,knoten,double_size));

	/* Auslastung pro Knoten */

	pruef(speich0(&mul,knoten,double_size));

	/* Auslastung pro Knoten */
} /* ENDE sp_g_multi */

/* *****************************************************************
**                                                                **
**                            sp_geschlossen                      **
**                aufgerufen von : geschlossen                    **
**                                                                **
***************************************************************** */

sp_geschlossen()
{                                      /* Speicherplatz fuer .. */
	extern int auftrag;                   /* Gesamtzahl der Auftraege im Netz */


	pruef(speich0(&normkonst,(auftrag+1),double_size));

	/* Normalisierungskonstante */

	pruef(speich0(&hilfvek,(auftrag+1),double_size));

	/* Hilfsvektor */

	pruef(speich0(&f,(auftrag+1),double_size));

	/* Hilfsvektor */
} /* sp_geschlossen */

/* *****************************************************************
**                                                                **
**                          sp_feed                               **
**          aufgerufen von : self_loop,feed_back                  **
**                                                                **
***************************************************************** */

sp_feed()
{                            /* Speicherplatz fuer .. */

	pruef(speich1(&netz.altbedienrate,netz.klasse,netz.knoten,double_size));

	/* alten Wert der Bedienrate */
	if (netz.typ == OFFEN)
		pruef(speich1(&netz.altankunftsrate,netz.klasse,netz.knoten,double_size));
	/* alten Wert der Ankunftsrate */
} /* ENDE sp_feed */
/* ****************************************************************
**                                                               **
**                       sp_ergebnis                             **
**           aufgerufen von : ber_ergebnis                       **
**                                                               **
**************************************************************** */

sp_ergebnis()
{                             /* Speicherplatz fuer ... */
	int klasse,knoten;           /* Hilfsvariablen */

	klasse = netz.klasse;
	knoten = netz.knoten;
	/* Abkuerzungen */

	if ((netz.typ == GESCHLOSSEN) && (netz.klasse > 1))
	{
		pruef(speich1(&besuch,klasse,knoten,double_size));

		/* Wahrscheinlichkeit, dass ein den Knoten s
				 besuchender Auftrag der Klasse c angehoert */
		pruef(speich0(&ankunftsrate,knoten,double_size));

		/* Ankunftsrate */

		pruef(speich0(&bedienrate,knoten,double_size));

		/* Bedienrate */
	} /* Ende GESCHLOSSEN */

	/*
	pruef(speich1(&netz.bsh,klasse,knoten,double_size));

	/* Besuchshaeufigkeit */

	pruef(speich1(&netz.warteschlange,klasse,knoten,double_size));

	/* mittlere Warteschlangenlaenge */

	pruef(speich1(&netz.verweilzeit,klasse,knoten,double_size));

	/* mittlere Verweilzeit */

	pruef(speich1(&netz.wartezeit,klasse,knoten,double_size));

	/* mittlere Wartezeit */
} /* ENDE sp_ergebnis() */

/* ************************************************************ 
**                                                           **
**                  free_arate                               **
**            aufgerufen von : arate                         **
**                                                           **
************************************************************ */

free_arate()
{                          /* Speicherfreigabe fuer .. */

	free1(&wa,netz.knoten,netz.knoten,double_size);

	/* Hilfsmartix wa */

	free0(&bvek,netz.knoten,double_size);

	/* Hilfsvektor bvek */

} /* Ende free_arate */

/* ************************************************************ 
**                                                           **
**                  free_feed                                **
**            aufgerufen von : feedback(), self_loop()       **
**                                                           **
************************************************************ */

free_feed()
{                          /* Speicherfreigabe fuer .. */

	if (netz.typ == OFFEN)
		free1(&netz.altankunftsrate,netz.klasse,netz.knoten,double_size);

	/* Ankunftsrate  */

	free1(&netz.altbedienrate,netz.klasse,netz.knoten,double_size);

	/* Bedienrate  */

} /* Ende free_feed */
