/* ausgabe.c

   Ausgabe der Leistungsgroessen gemaess PEPSY-Standard.
*/

static char *SccsId = "@(#)PEPSY:\tausgabe.c\t1.3\t92/12/09 15:58:06";

#include "summa.h"

extern int    *m,
kn,
kl,
it,
*typ,
auftraege;
extern unsigned int   opt;
extern double *lambda,
**e,
**mue;

extern unsigned int   topt();
extern double k(),
rho_kn(),
rho_kn_kl();

/* fehler_ex(n)

   Programmabbruch mit Fehlercode (n).

   Th.Hahn  15.02.1990
*/

void fehler_ex(n)
int n;
{
	fprintf(stdout,"monosum-Abbruch\n");
	fprintf(stdout,"1 0 0\n");
	exit(n);
}

/* mvz(i,r)

   Berechnung der mittleren Verweilzeit Knoten (i), Klasse (r).

   Th.Hahn  11.12.1989
*/

double mvz(i,r)
int i,r;
{
	if (!e[i][r]) return .0;
	if (!m[i]) return (1.0/mue[i][r]);
	return (k(i,r)/lambda[r]/e[i][r]);
}

/* mwz(i,r)

   Berechnung der mittleren Wartezeit Knoten (i), Klasse (r).

   Th.Hahn  11.12.1989
*/

double mwz(i,r)
int i,r;
{
	double h;

	if ((h=mvz(i,r)-1.0/mue[i][r])<.0 || !m[i] || !e[i][r]) return.0;
	return h;
}

/* fout(w)

   Ausgabe eines double-Wertes (w) ueber die Standardausgabe.

   Th.Hahn  11.12.1989
*/

void fout(w)
double w;
{
	if (opt&topt("v")) fprintf(stdout,"%lf ",w);
	else fprintf(stdout,"%.3lf ",w);
}

/* fout_nl(w)

   Ausgabe eines double-Wertes (w) ueber die Standardausgabe.
   Abschluss der Ausgabe mit '\n'.

   Th.Hahn  11.12.1989
*/

void fout_nl(w)
double w;
{
	fout(w);
	fprintf(stdout,"\n");
}

/* ausgabe()

   Ausgabe der Leistungsgroessen.

   Th.Hahn  11.12.1989
*/

void ausgabe()
{
	int i,r;
	double h;

	fprintf(stdout,"monosum\n");
	fprintf(stdout,"1 %d %d\n",kn,kl);
	for(r=0;r<kl;r++)
		for(i=0;i<kn;i++)
		{
			fout(lambda[r]*e[i][r]);
			fout(e[i][r]);
			fout(1.0/mue[i][r]);
			fout(rho_kn_kl(i,r));
			fout(mvz(i,r));
			fout(k(i,r));
			fout((h=mwz(i,r)));
			fout_nl(h*lambda[r]*e[i][r]);
		}
	for(i=0;i<kn;i++)
		if (rho_kn(i)>1.0)
		{
			fprintf(stderr,"** System instabil **\n");
			break;
		}
	fprintf(stdout,"Anzahl Iterationen: %d\n",it);
}
