/* funktionen.c

   Hilfsfunktionen fuer sum.
*/

static char *SccsId = "@(#)PEPSY:\tfunktionen.c\t1.3\t92/12/09 15:58:09";

#include "summa.h"

extern unsigned int   opt;
extern int    *typ,
*m,
kl,
auftraege;
extern double *lambda,
**e,
**mue,
**c,
dyn_a,
dyn_n;

extern unsigned int   topt();
extern double pow(),
exp(),
fabs();

bool pwarn=false;

/* fak(w)

   Berechnet die Fakultaet von (w) (positiv Integer).

   Th.Hahn  24.01.1990
*/

double fak(w)
int w;
{
	int g;
	double f=1.0;

	for(g=2;g<=w;f*=g,g++);
	return f;
}

/* kor_fak(i,r)

   Berechnung des Korrekturfaktors Knoten (i), Klasse (r) in
   Abhaengigkeit des Knotentyps.

   W.Brag 1991 geaendert
*/

double kor_fak(i,r)
int i,r;
{
	int t=typ[i];
	double a=1.0;

	if (t==5 || t==6 || t==14 || t==15 || t==18 || t==19)
		a=(c[i][r]+1.0)/2.0;
	if (t==1 || t==3 || t==9 || t==10 || t==14)
		return ((auftraege-a)/(double)(auftraege));
	else
		if (t==11 || t==15)
		{
			if (auftraege>1) return ((auftraege-1.0-a)/(auftraege-1.0));
			else return MAXFLOAT;
		}
		else
			if (t==2 || t==5 || t==6 || t>15 && t<20)
			{
				if (auftraege>m[i]) return ((auftraege-m[i]-a)/(auftraege-m[i]));
				else return MAXFLOAT;
			}
			else return 1.0;
}

/* rho_kn_kl(i,r)

   Berechnung der Auslastung Knoten (i), Klasse (r).

   Th.Hahn  22.11.1989
*/

double rho_kn_kl(i,r)
int i,r;
{
	double h;

	if (typ[i]==4) return .0;
	else 
		if (fabs(h=lambda[r]*e[i][r]/m[i]/mue[i][r])<MINRHO)
			if (h<.0) return (-MINRHO);
			else return MINRHO;
		else return h;
}

/* rho_kn(i)

   Berechnung der Auslastung Knoten (i).

   Th.Hahn  22.11.1989
*/

double rho_kn(i)
int i;
{
	int r;
	double h=.0;

	for(r=0;r<kl;r++) h+=rho_kn_kl(i,r);
	if (fabs(h)<MINRHO)
		if (h<.0) return (-MINRHO);
		else return MINRHO;
	return h;
}

/* dyn_fak(i,r)

   Berechnet den dynamischen Korrekturfaktor fuer Knoten (i),
   Klasse (r) (BCMP-Knoten).

   Th.Hahn  22.11.1989
*/

double dyn_fak(i,r)
int i,r;
{
	double h=rho_kn(i);

	if (!(opt&topt("f")) || h<=null || h>=1.0) return 1.0;
	if (typ[i]<4 || typ[i]==9)
		return (1.0-dyn_a*pow(h,dyn_n)*(1.0-h));
	return 1.0;
}

/* sig(i,r)

   Berechnung von rho((i),0)+rho((i),1)+...+rho((i),(r)-1) (Knoten (i)).

   Th.Hahn  12.11.1989
*/

double sig(i,r)
int i,r;
{
	int k=0;
	double h=.0;

	while(k<=r-1)
	{
		h+=rho_kn_kl(i,k);
		k++;
	}
	return h;
}

/* p_wart(rho,i)

   Wartewahrscheinlichkeit fuer Auslastung (rho) und Knoten (i).

   Th.Hahn  15.11.89
*/

double p_wart(rho,i)
double rho;
int i;
{
	int f;
	double fak=1.0,sm1=.0,h0=1.0,h1=m[i]*rho;

	if (rho>=1.0) return 1.0;
	if (rho<=MINRHO) return .0;
	if (opt&topt("a"))
	{
		if (!pwarn)
			if (m[i]>1 && fabs(.7-rho)<.001)
			{
				fprintf(stderr,"** p_wart()-Warnung **\n");
				pwarn=true;
			}
		if (rho>=.7) return ((pow(rho,(double)(m[i]))+rho)/2.0);
		else
			if (!(opt&topt("w")) || opt&topt("w") && rho>=.3)
				return (pow(rho,(m[i]+1.0)/2.0));
			else
			{
				for(f=0;f<m[i];f++)
				{
					sm1+=h0/fak;
					fak*=f+1.0;
					h0*=h1;
				}
				return (1.0-sm1/exp(m[i]*rho));
			}
	}
	else
	{
		for(f=0;f<m[i];f++)
		{
			sm1+=h0/fak;
			fak*=f+1.0;
			h0*=h1;
		}
		h0/=fak*(1.0-rho);
		return (h0/(sm1+h0));
	}
}

/* addx()

   Hilfsfunktionen fuer die Berechnung der mittleren
   Auftragszahlen.

   W.Brag 1991 geaendert
*/

double add1(i,j)
int i,j;
{
	int r;
	double h=.0;

	for(r=0;r<=j;r++) h+=rho_kn_kl(i,r)/mue[i][r];
	return h;
}

double add2(i,j)
int i,j;
{
	int r;
	double h=.0;

	for(r=0;r<=j;r++)
		h+=lambda[r]*e[i][r]*(c[i][r]+1.0)/mue[i][r]/mue[i][r];
	return h;
}

double add3(i,j)
int i,j;
{
	int r;
	double h=.0;

	for(r=0;r<=j;r++)
		h+=rho_kn_kl(i,r)*(c[i][r]+1.0)/mue[i][r];
	return h;
}
