static char *SccsId = "@(#)PEPSY:\tauftraege.c\t1.3\t92/12/09 16:43:47";

/* auftraege.c

   Formeln zur Berechnung der Auftragszahlen der jeweiligen Knoten
   sowie die Ableitungen der verwendeten Formeln (notwendig zur
   Durchfuehrung des Newtonschen Einzelschrittverfahrens).

   (** Grundversion von Thomas Huetter (SA 1989) **)
*/

#include "multidef.h"

extern int    *typ,
*m,
*auftrag,
kn,
kl;
extern double *lambda,
**e,
**mue,
**c;

extern double exp(),
exp_nhr(),
abl_exp_nhr(),
kor_fak(),
dyn_fak(),
abl_dyn_fak(),
p_wart(),
abl_p_wart(),
add1(),
add2(),
add3(),
rho_kn(),
rho_kn_kl(),
abl_rho_kn_kl(),
sig();

double sum_k(),
k(),
sum_k_HNB(),
k_HNB(),
abl_sum_k_HNB(),
abl_k_HNB(),
HNenner(),
Nenner_k(),
abl_HNenner(),
abl_Nenner_k();

/* sum_k(r)

   Summiert die mittleren Auftragszahlen der Klasse (r) ueber
   alle Knoten.

   Th.Hahn  22.11.1989
*/

double sum_k(r)
int r;
{
	int i;
	double h=.0;

	for(i=0;i<kn;i++)
		h+=k(i,r);       /* dyn_fak() geht bei k() ein */
	return h;
}

/* k(i,r)

   Mittlere Zahl der Auftraege Knoten (i), Klasse (r).

   Th.Hahn  23.01.1990
*/

double k(i,r)
int i,r;
{
	int t=typ[i];
	double h=.0,f,h1,h2,h3,h4,h5,h6,h7,h8,z1;

	if (!e[i][r]) return .0;
	if (t==4)
		h=lambda[r]*e[i][r]/mue[i][r];
	else
		if (t==1 || t==3 || t==9)
			h=rho_kn_kl(i,r)/(1.0-kor_fak(i,r)*rho_kn(i));
		else
			if (t==2 || t==5 || t==6)
			{
				if (t==2) f=1.0;
				else f=(c[i][r]+1.0)/2.0; 
				h1=rho_kn_kl(i,r);
				h2=rho_kn(i);
				h=m[i]*h1+(h1/(1.0-kor_fak(i,r)*h2))*f*p_wart(h2,i);
			}
			else
				if (t==11)
				{
					h3=kor_fak(i,r);
					h1=(1.0-h3*sig(i,r))*(1.0-h3*sig(i,r+1));
					h=rho_kn_kl(i,r)+lambda[r]*e[i][r]*add1(i,kl-1)/h1;
				}
				else
					if (t==10)
					{
						h2=kor_fak(i,r);
						h3=sig(i,r+1);
						h1=(1.0-h2*sig(i,r))*(1.0-h2*h3);
						h=(rho_kn_kl(i,r)*(1.0-h3)+lambda[r]*e[i][r]*add1(i,r))/h1;
					}
					else
						if (t==15)
						{
							h3=kor_fak(i,r);
							h1=(1.0-h3*sig(i,r))*(1.0-h3*sig(i,r+1));
							h=rho_kn_kl(i,r)+.5*lambda[r]*e[i][r]*add2(i,kl-1)/h1;
						}
						else
							if (t==14)
							{
								h1=kor_fak(i,r);
								h2=sig(i,r+1);
								h3=(1.0-h1*h2)*(1.0-h1*sig(i,r));
								h=(rho_kn_kl(i,r)*(1.0-h2)+.5*lambda[r]*e[i][r]*add2(i,r))/h3;
							}
							else
								if (t==16 || t==18)
								{
/* code folded from here */
	if (t==16)
	{
		h6=add1(i,r);
		h7=add1(i,r-1);
	}
	else
	{
		h6=.5*add3(i,r);
		h7=.5*add3(i,r-1);
	}
	h3=kor_fak(i,r);
	h1=sig(i,r);
	h2=sig(i,r+1);
	h4=1.0-h3*h1;
	h5=1.0-h3*h2;
	h8=rho_kn_kl(i,r);
	z1=(h4*h6*p_wart(h2,i)-h5*h7*p_wart(h1,i))/m[i];
	h=m[i]*h8+lambda[r]*e[i][r]/h8*z1/h4/h5;
/* unfolding */
								}
								else
/* code folded from here */
	if (t==17 || t==19)
	{
		if (t==17) h4=add1(i,kl-1);
		else h4=.5*add3(i,kl-1);
		h3=kor_fak(i,r);
		h1=(1.0-h3*sig(i,r))*(1.0-h3*sig(i,r+1));
		h2=rho_kn(i);
		h=m[i]*rho_kn_kl(i,r)+lambda[r]*e[i][r]*h4*p_wart(h2,i)/m[i]/h2/h1;
	}
	else
		if (t==-1)
		{
			h1=rho_kn_kl(i,r);
			if (h1<=MINRHO) h=.0;
			else h=auftrag[r]*exp(1.0-1.0/h1);
		}
		else
			if (-5<t && t<-1) h=exp_nhr(i,r);
/* unfolding */
	return (dyn_fak(i,r)*h);
}

/* sum_k_HNB(r)

   Summiert die mittleren Auftragszahlen der Klasse (r) ueber
   alle Knoten.

   (Hauptnennerbildung und Multiplikation mit den Hauptnennern
    aller Knotentypen).

   Th.Hahn  03.01.1990
*/

double sum_k_HNB(r)
int r;
{
	int i;
	double h=.0;

	for(i=0;i<kn;i++)
		h+=dyn_fak(i,r)*k_HNB(i,r);
	return h;
}

/* k_HNB(i,r)

   Mittlere Zahl der Auftraege Knoten (i), Klasse (r).

   (Hauptnennerbildung und Multiplikation mit den Hauptnennern
    aller Knotentypen).

   Th.Hahn  23.01.1990
*/

double k_HNB(i,r)
int i,r;
{
	int t=typ[i];
	double h=.0,f,f1,h1,h2,h3,h4,h5,h6,h7,h8,z1;

	if (!e[i][r]) return .0;
	if (t==1 || t==3 || t==4 || t==9)
		h=lambda[r]*e[i][r]/mue[i][r]*HNenner(i,-1,r);
	else
		if (t==2 || t==5 || t==6)
		{
			if (t==2) f=1.0;
			else f=(c[i][r]+1.0)/2.0; 
		/*	else f=(c[i][r]*c[i][r]+1.0)/2.0; */
			h1=rho_kn_kl(i,r);
			h=h1*(m[i]*HNenner(-1,-1,r)+HNenner(i,-1,r)*f*p_wart(rho_kn(i),i));
		}
		else
			if (t==11)
				h=rho_kn_kl(i,r)*HNenner(-1,-1,r)+lambda[r]*e[i][r]*add1(i,kl-1)*HNenner(i,-1,r);
			else
				if (t==10)
					h=(rho_kn_kl(i,r)*(1.0-sig(i,r+1))+lambda[r]*e[i][r]*add1(i,r))*HNenner(i,-1,r);
				else
					if (t==15)
						h=rho_kn_kl(i,r)*HNenner(-1,-1,r)+.5*lambda[r]*e[i][r]*add2(i,kl-1)*HNenner(i,
						    -1,r);
					else
						if (t==14)
							h=(rho_kn_kl(i,r)*(1.0-sig(i,r+1))+.5*lambda[r]*e[i][r]*add2(i,r))*HNenner(i,
							    -1,r);
						else
							if (t==16 || t==18)
							{
								if (t==16)
								{
/* code folded from here */
	f1=1.0;
	h6=add1(i,r);
	h7=add1(i,r-1);
/* unfolding */
								}
								else
								{
/* code folded from here */
	h6=add3(i,r);
	h7=add3(i,r-1);
	f1=.5;
/* unfolding */
								}
								h3=kor_fak(i,r);
								h1=sig(i,r);
								h2=sig(i,r+1);
								h4=1.0-h3*h1;
								h5=1.0-h3*h2;
								h8=rho_kn_kl(i,r);
								z1=(h4*h6*p_wart(h2,i)-h5*h7*p_wart(h1,i))*f1/m[i];
								h=m[i]*h8*HNenner(-1,-1,r)+lambda[r]*e[i][r]/h8*z1*HNenner(i,
								    -1,r);
							}
							else
								if (t==17 || t==19)
								{
/* code folded from here */
	h1=rho_kn(i);
	if (t==17) h2=add1(i,kl-1);
	else h2=.5*add3(i,kl-1);
	h=m[i]*rho_kn_kl(i,r)*HNenner(-1,-1,r)+lambda[r]*e[i][r]*h2*p_wart(h1,i)*HNenner(i,-1,r)/m[i]/h1;
/* unfolding */
								}
								else
/* code folded from here */
	if (t==-1)
	{
		h1=rho_kn_kl(i,r);
		if (h1<=MINRHO) h=.0;
		else h=auftrag[r]*exp(1.0-1.0/h1)*HNenner(i,-1,r);
	}
	else
		if (-5<t && t<-1) h=exp_nhr(i,r)*HNenner(i,-1,r);
/* unfolding */
	return h;
}

/* abl_sum_k_HNB(r)

   Ableitung von sum_HNB() nach lambda[r].

   Th.Hahn  22.11.1989
*/

double abl_sum_k_HNB(r)
int r;
{
	int i;
	double h=.0;

	for(i=0;i<kn;i++)
		h+=dyn_fak(i,r)*abl_k_HNB(i,r)+abl_dyn_fak(i,r)*k_HNB(i,r);
	return h;
}

/* abl_k_HNB(i,r)

   Ableitung von k_HNB() nach Lambda[r].

   Th.Hahn  23.01.1990
*/

double abl_k_HNB(i,r)
int i,r;
{
	int t=typ[i];
	double h=.0,f,f1,h1,h2,h3,h4,h5,h6,h7,h8,h9,h10,z1,z2;

	if (!e[i][r]) return .0;
	if (t==1 || t==3 || t==4 || t==9)
	{
		h1=e[i][r]/mue[i][r];
		h=h1*HNenner(i,-1,r)+lambda[r]*h1*abl_HNenner(i,r);
	}
	else
		if (t==2 || t==5 || t==6)
		{
			if (t==2) f=1.0;
		/*	else f=(c[i][r]+1.0)/2.0; */
			else f=(c[i][r]*c[i][r]+1.0)/2.0;
			h3=HNenner(i,-1,r);
			h4=rho_kn_kl(i,r);
			h5=abl_rho_kn_kl(i,r);
			h6=rho_kn(i);
			h7=p_wart(h6,i);
			h1=m[i]*(h5*HNenner(-1,-1,r)+h4*abl_HNenner(-1,r));
			h2=f*(h3*(h5*h7+h4*abl_p_wart(i,r,kl))+h4*abl_HNenner(i,r)*h7);
			h=h1+h2;
		}
		else
			if (t==11)
			{
				h3=rho_kn_kl(i,r);
				h4=abl_rho_kn_kl(i,r);
				h5=e[i][r]*add1(i,kl-1);
				h1=h4*HNenner(-1,-1,r)+h3*abl_HNenner(-1,r);
				h2=(h5+h3*h4)*HNenner(i,-1,r);
				h=h1+h2+lambda[r]*h5*abl_HNenner(i,r);
			}
			else
				if (t==10)
				{
					h1=rho_kn_kl(i,r);
					h4=add1(i,r);
					h5=abl_rho_kn_kl(i,r);
					h6=sig(i,r+1);
					h2=h5*(1.0-h6)+e[i][r]*h4;
					h3=h1*(1.0-h6)+lambda[r]*e[i][r]*h4;
					h=h2*HNenner(i,-1,r)+h3*abl_HNenner(i,r);
				}
				else
					if (t==15)
					{
						h2=(c[i][r]+1.0)/mue[i][r];
						h5=.5*add2(i,kl-1)*e[i][r];
						h6=rho_kn_kl(i,r);
						h7=HNenner(i,-1,r);
						h3=h5*lambda[r]*abl_HNenner(i,r)+.5*e[i][r]*h6*h2*h7;
						h4=h5*h7+abl_rho_kn_kl(i,r)*HNenner(-1,-1,r)+h6*abl_HNenner(-1,r);
						h=h3+h4;
					}
					else
						if (t==14)
						{
							h1=rho_kn_kl(i,r);
							h5=abl_rho_kn_kl(i,r);
							h4=.5*add2(i,r)*e[i][r];
							h6=1.0-sig(i,r+1);
							h2=h5*(h6-h1)+.5*e[i][r]*h1*(c[i][r]+1.0)/mue[i][r]; 
							h3=h1*h6+h4*lambda[r];
							h=(h2+h4)*HNenner(i,-1,r)+h3*abl_HNenner(i,r);
						}
						else if (t==17 || t==19)
						{
							h6=rho_kn_kl(i,r);
							h7=abl_rho_kn_kl(i,r);
							h1=m[i]*(h7*HNenner(-1,-1,r)+h6*abl_HNenner(-1,r));
							if (t==17)
							{
								h2=add1(i,kl-1);
								h5=1.0/mue[i][r];
								f1=1.0;
							}
							else
							{
								h2=add3(i,kl-1);
								h5=(c[i][r]+1.0)/mue[i][r]; 
								f1=.5;
							}
							h3=HNenner(i,-1,r);
							f=rho_kn(i);
							h4=p_wart(f,i);
							h=lambda[r]*e[i][r];
							h=h1+((e[i][r]*h3*h4*(h2+h6*h5)+h*h2*(abl_HNenner(i,r)*h4+h3*abl_p_wart(i,
							    r,kl)))*f-h7*h*h2*h3*h4)*f1/f/f/m[i];
						}
						else
							if (t==16 || t==18)
							{
								if (t==18)
								{
/* code folded from here */
	h1=add3(i,r);
	f1=.5;
/* unfolding */
								}
								else
								{
/* code folded from here */
	f1=1.0;
	h1=add1(i,r);
/* unfolding */
								}
								h2=HNenner(i,-1,r);
								h3=abl_HNenner(i,r);
								h4=sig(i,r);
								h5=sig(i,r+1);
								h6=1.0-kor_fak(i,r)*h4;
								h7=lambda[r]*e[i][r];
								h8=rho_kn_kl(i,r);
								h9=abl_rho_kn_kl(i,r);
								h10=p_wart(h5,i);
								z1=h8/mue[i][r];
								if (t==18) z1*=c[i][r]+1.0;
								z1=(z1+h1)*e[i][r]*h10*h2*h6;
								z2=h7*h1*h6*(abl_p_wart(i,r,r+1)*h2+h10*h3);
								h=(h8*(z1+z2)-h7*h10*h1*h2*h9*h6)*f1/h8/h8/m[i];
								if (t==18) h1=add3(i,r-1);
								else h1=add1(i,r-1);
								h6=1.0-kor_fak(i,r)*h5;
								h10=p_wart(h4,i);
								z1=h1*h10*(h2*h6*e[i][r]+h7*(h6*h3-h9*h2));
								h-=(h8*z1-h7*h9*h10*h1*h2*h6)*f1/h8/h8/m[i];
								h+=m[i]*(h9*HNenner(-1,-1,r)+h8*abl_HNenner(-1,r));
							}
							else
								if (t==-1)
								{
/* code folded from here */
	h4=rho_kn_kl(i,r);
	if (h4<=MINRHO) h=.0;
	else
	{
		h1=1.0/h4;
		h2=exp(1.0-h1);
		h3=abl_rho_kn_kl(i,r);
		h=auftrag[r]*h2*(h3*h1*h1*HNenner(i,-1,r)+abl_HNenner(i,r));
	}
/* unfolding */
								}
								else
/* code folded from here */
	if (-5<t && t<-1)
		h=abl_exp_nhr(i,r)*HNenner(i,-1,r)+exp_nhr(i,r)*abl_HNenner(i,r);
/* unfolding */
	return h;
}

/* HNenner(i,l,r)

   Produkt der Hauptnenner der Funktionen k[][r] ohne die Hauptnenner
   der Komponenten k[i][r] und k[l][r].

   Th.Hahn  22.11.1989
*/

double HNenner(i,l,r)
int i,l,r;
{
	int j;
	double h1=1.0;

	for(j=0;j<kn;j++)
		if (j!=i && j!=l) h1*=Nenner_k(j,r);
	return h1;
}

/* Nenner_k(i,r)

   Hauptnenner der Funktion k().

   Th.Hahn  08.12.1989
*/

double Nenner_k(i,r)
int i,r;
{
	int t=typ[i];
	double h1;

	if (-5<t && t<0 || t==4) return 1.0;
	else
		if (t>0 && t<4 || t==5 || t==6 || t==9)
			return (1.0-kor_fak(i,r)*rho_kn(i));
		else
			if (t==10 || t==11 || t>13 && t<20)
			{
				h1=kor_fak(i,r);
				return ((1.0-h1*sig(i,r))*(1.0-h1*sig(i,r+1)));
			}
			else return 1.0;
}

/* abl_HNenner(i,r)

   Ableitung von HNenner() nach Lambda[r] ohne den Hauptnenner der
   Komponente k[i][r].

   Th.Hahn  22.11.1989
*/

double abl_HNenner(i,r)
int i,r;
{
	int j;
	double h=.0;

	for(j=0;j<kn;j++)
		if (j!=i) h+=abl_Nenner_k(j,r)*HNenner(i,j,r);
	return h;
}

/* abl_Nenner_k(i,r)

   Ableitung von Nenner_k() nach Lambda[r].

   Th.Hahn  08.12.1989
*/

double abl_Nenner_k(i,r)
int i,r;
{
	int t=typ[i];
	double h1;

	if (-5<t && t<0 || t==4) return .0;
	else
		if (t>0 && t<4 || t==5 || t==6 || t==9)
			return (-kor_fak(i,r)*abl_rho_kn_kl(i,r));
		else if (t==10 || t==11 || t>13 && t<20)
		{
			h1=kor_fak(i,r);
			return (-h1*abl_rho_kn_kl(i,r)*(1.0-h1*sig(i,r)));
		}
		else return .0;
}
