static char *SccsId = "@(#)PEPSY:\tausgabe.c\t1.3\t92/12/09 16:43:49";

/* ausgabe.c

   Ausgabe der Leistungsgroessen gemaess PEPSY-Standard.
*/

#include "multidef.h"

extern bool   ownt,
mm1;
extern int    *m,
kn,
kl,
it,
*typ,
auftraege;
extern unsigned int   opt;
extern double *lambda,
**e,
**mue,
**prand,
*auftrm1,
*auftrzm2,
*maazm2,
*mvzzm2,
*mwzzm2;

extern void   momaus(),
momaus1(),
auftrmom();
extern unsigned int   topt();
extern double k(),
rho_kn(),
rho_kn_kl(),
var_maa(),
var_mvz(),
var_mwz();

/* fehler_ex(n)

   Programmabbruch mit Fehlercode (n).

   Th.Hahn  15.02.1990
*/

void fehler_ex(n)
int n;
{
	fprintf(stdout,"multisum-Abbruch\n");
	fprintf(stdout,"1 0 0\n");
	exit(n);
}

/* mvz(i,r)

   Berechnung der mittleren Verweilzeit fuer Knoten (i), Klasse (r).

   Th.Hahn  11.12.1989
*/

double mvz(i,r)
int i,r;
{
	if (!e[i][r]) return .0;
	if (!m[i]) return (1.0/mue[i][r]);
	return (k(i,r)/lambda[r]/e[i][r]);
}

/* mwz(i,r)

   Berechnung der mittleren Wartezeit fuer Knoten (i), Klasse (r).

   Th.Hahn  11.12.1989
*/

double mwz(i,r)
int i,r;
{
	double h=mvz(i,r)-1.0/mue[i][r];

	if (h<=.0 || !m[i] || !e[i][r]) return.0;
	return h;
}

/* fout(w)

   Ausgabe eines double-Wertes (w) ueber die Standardausgabe.

   Th.Hahn  11.12.1989
*/

void fout(w)
double w;
{
	if (opt&topt("v")) fprintf(stdout,"%lf ",w);
	else fprintf(stdout,"%.3lf ",w);
}

/* fout_nl(w)

   Ausgabe eines double-Wertes (w) ueber die Standardausgabe.
   Abschluss der Ausgabe mit '\n'.

   Th.Hahn  11.12.1989
*/

void fout_nl(w)
double w;
{
	fout(w);
	fprintf(stdout,"\n");
}

/* ausgabe()

   Ausgabe der Leistungsgroessen.

   Th.Hahn  11.12.1989
*/

void ausgabe()
{
	int i,r;
	double h;

	fprintf(stdout,"multisum\n");
	fprintf(stdout,"1 %d %d\n",kn,kl);
	for(r=0;r<kl;r++)
		for(i=0;i<kn;i++)
		{
			fout(lambda[r]*e[i][r]);
			fout(e[i][r]);
			fout(1.0/mue[i][r]);
			fout(rho_kn_kl(i,r));
			fout(mvz(i,r));
			fout(k(i,r));
			fout((h=mwz(i,r)));
			fout_nl(h*lambda[r]*e[i][r]);
		}
	for(i=0;i<kn;i++)
		if (rho_kn(i)>1.0)
		{
			fprintf(stderr,"** System instabil **\n");
			if (ownt)
				fprintf(stderr,"** Diagnose : unguenstige Parameter fuer Exponential-Naeherung **\n");
			break;
		}
	if (opt&topt("m"))
	{
		if (mm1 && kl==1)
		{
			fprintf(stdout,"Randwahrscheinlichkeiten (M/M/1-FCFS) :\n");
			for(i=0;i<kn;i++)
				if (typ[i]==1)
					for(r=0;r<=auftraege;r++)
					{
						fprintf(stdout,"p(%d)(%d)=",i,r);
						fout_nl(prand[i][r]);
					}
			fprintf(stdout,"Auftragsmomente (M/M/1-FCFS) :\n");
			momaus(auftrmom,auftrm1,auftrzm2);
			fprintf(stdout,"Varianzen maa (M/M/1-FCFS) :\n");
			momaus1(k,var_maa,maazm2);
			fprintf(stdout,"Varianzen mvz (M/M/1-FCFS) :\n");
			momaus1(mvz,var_mvz,mvzzm2);
			fprintf(stdout,"Varianzen mwz (M/M/1-FCFS) :\n");
			momaus1(mwz,var_mwz,mwzzm2);
		}
	}
	fprintf(stdout,"Anzahl Iterationen: %d\n",it);
}
