static char *SccsId = "@(#)PEPSY:\texpnhr.c\t1.3\t92/12/09 16:43:52";

/* expnhr.c

   Verwaltung der bei der Naeherung mit Exponentialfunktionen nach
   /INZEL_89/ benoetigten Parameter.

   Berechnung der entsprechenden Exponentialfunktionen.
*/

#include "multidef.h"

extern unsigned int   opt;
extern int    *typ,
*auftrag,
kn,
kl;

extern char   *block();
extern unsigned int   topt();
extern double log(),
exp(),
rho_kn_kl(),
abl_rho_kn_kl();

double ***parm=0; /* Parametervektoren fuer Exponentialfkt. */

/* read_parms(i,r)

   Einlesen des Parametervektors (parm[i][r][]) fuer Knoten (i) und
   Klasse (r) aus dem Eingabefile (default) oder deren Berechnung.

   ACHTUNG : Bei Einlesen aus dem Eingabefile werden immer 4 Parameter
   =======   gelesen, auch wenn nur 3 benoetigt werden.

   Th.Hahn  27.01.1990
*/

void read_parms(i,r)
int i,r;
{
	int f;
	double *p=parm[i][r];

	if (opt&topt("b"))  /* Parameter berechnen */
	{
		for(f=0;f<4;f++) p[f]=1.0;
	}
	else /* Parameter einlesen */
		for(f=0;f<4;f++) fscanf(stdin,"%lf",&p[f]);           /** ! **/
	if (typ[i]==-4)     /* Parameter an Ansatz anpassen */
		p[3]=log((double)(auftrag[r]))-p[0]-p[1]-p[2];
	else
		if (typ[i]==-2) p[3]=.0;
}

/* get_parms()

   Speicherplatz fuer Parameter reservieren und Parameter einlesen
   (aus dem Eingabefile (default)) oder berechnen.

   Th.Hahn  23.01.1990
*/

void get_parms()
{
	int f,g,h;

	for(f=0;f<kn;f++)
		if (-5<typ[f] && typ[f]<-1) break;
	if (f<kn)
	{
		if (opt&topt("b"))
			fprintf(stderr,"** -b : noch kein Berechnungsalgorithmus implementiert ! **\n");
		else
			fprintf(stderr,"** Achtung : fuer -o sind Zusatzangaben im Eingabefile erforderlich ! **\n");
		parm=(double ***)(block(kn*sizeof(double **)));
		for(f=0;f<kn;f++) parm[f]=0;
		for(f=0;f<kn;f++)
		{
			g=typ[f];
			if (g==-2 || g==-3 || g==-4)
			{
				parm[f]=(double **)(block(kl*sizeof(double *)));
				for(g=0;g<kl;g++) parm[f][g]=0;
				for(g=0;g<kl;g++)
				{
					parm[f][g]=(double *)(block(4*sizeof(double)));
					for(h=0;h<4;h++) parm[f][g][h]=.0;
					read_parms(f,g);
					fprintf(stderr,"Parameter Knoten %d Klasse %d :\n\t",f,g);
					for(h=0;h<4;h++) fprintf(stderr,"%lf ",parm[f][g][h]);
					fprintf(stderr,"\n");
				}
			}
		}
	}
}

/* exp_nhr(i,r)

   Berechnet je nach Typ des Knotens (i) mit einer Exponentialfunktion
   fuer die Auftragsklasse (r) eine Naeherung fuer die mittlere Zahl
   der Auftraege.

   Th.Hahn  16.12.1989
*/

double exp_nhr(i,r)
int i,r;
{
	double h=rho_kn_kl(i,r);
	double *p=parm[i][r];

	return (h*exp(((p[3]*h+p[2])*h+p[1])*h+p[0]));
}

/* abl_exp_nhr(i,r)

   Ableitung von exp_nhr() nach Lambda[r].

   Th.Hahn  16.12.1989
*/

double abl_exp_nhr(i,r)
int i,r;
{
	double h=rho_kn_kl(i,r),h1=exp_nhr(i,r);
	double *p=parm[i][r];

	return (abl_rho_kn_kl(i,r)*h1*(1.0+h*(p[1]+h*(2*p[2]+3*p[3]*h))));
}
