static char *SccsId = "@(#)PEPSY:\tfaktoren.c\t1.3\t92/12/09 16:43:53";
/* faktoren.c

   Korrektur(faktor)-Funktionen fuer multisum.
*/

#include "multidef.h"

extern int    *typ,
*m,
*auftrag,
auftraege,
kl,
fn;
extern unsigned int   opt;
extern FILE   *fd;
extern double *lambda,
**c,
dyn_a,
dyn_n;

extern unsigned int   topt();
extern double pow(),
pot(),
fabs(),
sum_k(),
rho_kn(),
rho_kn_kl(),
abl_rho_kn_kl(),
min_lam();

/* kor_lam(r)

   Korrigiert Lambda[r] bei Bedarf durch Intervallschachtelung.

   Th.Hahn  13.02.1990
*/

void kor_lam(r)
int r;
{
	 unsigned long kc=0,kit=KMAXIT;
	double lo=lambda[r],t=.0,lu=.0;

	if (opt&topt("d")) fprintf(fd,"\t* kor_lam(%d) : %lf",r,lo);
	if (lo<null)
	{
		lambda[r]=lo=min_lam(r);
		t=sum_k(r);
		if (opt&topt("p"))
		{
			fprintf(stderr,"** kor_lam() : <null **\n");
			fprintf(stderr,"** lu %lf lo %lf t %lf **\n",lu,lo,t);
		}
	}
	if (fabs(t-auftrag[r])>KEPS)
	{
		do
		{
			lambda[r]=(lu+lo)/2.0;
			if ((t=sum_k(r)-auftrag[r])>.0) lo=lambda[r];
			else lu=lambda[r];
			kc++;
			if (--kit==0)
			{
				fprintf(stderr,"** kor_lam() : KMAXIT erreicht **\n");
				lambda[r]=null;
				break;
			}
		}while(fabs(t)>KEPS);
	}
	if (opt&topt("d")) fprintf(fd," %lf\n",lambda[r]);
	if (opt&topt("p")) fprintf(stderr,"** kor_lam(%d) (%ld) **\n",r,kc);
}

/* kor_fak(i,r)

   Berechnung des Korrekturfaktors Knoten (i), Klasse (r) in
   Abhaengigkeit des Knotentyps.

   Th.Hahn  09.11.1989
*/

double kor_fak(i,r)
int i,r;
{
	int t=typ[i];
	double a=1.0;

	if (t==5 || t==6 || t==14 || t==15 || t==18 || t==19)
		a=(c[i][r]+1.0)/2.0;
	if (t==1 || t==3 || t==9 || t==10 || t==14)
		return ((auftraege-a)/(double)(auftraege));
	else
		if (t==11 || t==15)
		{
			if (auftraege>1) return ((auftraege-1.0-a)/(auftraege-1.0));
			else return MAXFLOAT;
		}
		else
			if (t==2 || t==5 || t==6 || t>15 && t<20)
			{
				if (auftraege>m[i]) return ((auftraege-m[i]-a)/(auftraege-m[i]));
				else return MAXFLOAT;
			}
			else return 1.0;
}

/* dyn_fak(i,r)

   Berechnet den dynamischen Korrekturfaktor fuer Knoten (i),
   Klasse (r) (BCMP-Knoten und negative Knotentypen).

   Th.Hahn  22.11.1989
*/

double dyn_fak(i,r)
int i,r;
{
	double h=rho_kn(i);

	if (!(opt&topt("f"))) return 1.0;
	if ((fn!=3 || kl==1) && (h<=null || h>=1.0)) return 1.0;
	if (typ[i]<4 || typ[i]==9)
	{
		switch(fn)
		{
		case 1 : 
			return (1.0-dyn_a*rho_kn_kl(i,r)*pow(h,dyn_n-1.0)*(1.0-h));
		case 2 : 
			return (1.0-dyn_a*pot(rho_kn_kl(i,r),dyn_n)*(1.0-h));
		case 3 : 
			h=rho_kn_kl(i,r);
			return (1.0-dyn_a*pot(h,dyn_n)*(1.0-h));
		case 4 : 
			return (1.0-dyn_a*pow(h,dyn_n)*(1.0-h));
		}
	}
	return 1.0;
}

/* abl_dyn_fak(i,r)

   Ableitung von dyn_fak() nach Lambda[r].

   Th.Hahn  22.11.1989
*/

double abl_dyn_fak(i,r)
int i,r;
{
	double h=rho_kn(i),h1,h2=rho_kn_kl(i,r);

	if (!(opt&topt("f"))) return .0;
	if ((fn!=3 || kl==1) && (h<=null || h>1.0)) return .0;
	if (typ[i]<4 || typ[i]==9)
	{
		switch (fn)
		{
		case 1 : 
			h1=pow(h,dyn_n-2.0);
			return (dyn_a*abl_rho_kn_kl(i,r)*h1*(h*h2-(1.0-h)*(h+(dyn_n-1.0)*h2)));
		case 2 : 
			h1=pot(h2,dyn_n-1.0);
			return (dyn_a*abl_rho_kn_kl(i,r)*h1*(h2-dyn_n*(1.0-h)));
		case 3 : 
			h1=pot(h2,dyn_n-1.0);
			return (dyn_a*abl_rho_kn_kl(i,r)*h1*(h2-dyn_n*(1.0-h2)));
		case 4 : 
			h1=pow(h,dyn_n-1.0);
			return (dyn_a*abl_rho_kn_kl(i,r)*h1*(h-dyn_n*(1.0-h)));
		}
	}
	return .0;
}
