static char *SccsId = "@(#)PEPSY:\tfunktionen.c\t1.3\t92/12/09 16:43:55";

/* funktionen.c

   Hilfsfunktionen fuer multisum.
*/

#include "multidef.h"

extern unsigned int   opt;
extern int    *typ,
*m,
*auftrag,
kn,
kl;
extern double *lambda,
**e,
**mue,
**c;

extern unsigned int   topt();
extern double pow(),
exp(),
fabs();

bool pwarn=false;

/* pot(x,y)

   Berechnet pow(x,y) fuer x>null.

   Th.Hahn  17.12.1989
*/

double pot(x,y)
double x,y;
{
	if (x<=null) return .0;
	return (pow(x,y));
}

/* min_lam(r)

   Liefert als Rueckgabewert die Obergrenze des Definitionsbereiches
   von lambda[(r)].

   Th.Hahn  29.01.1990
*/

double min_lam(r)
int r;
{
	int i;
	double lo=MAXFLOAT;

	for(i=0;i<kn;i++)
		if (e[i][r])
			if (typ[i]==4) lo=min(lo,mue[i][r]*auftrag[r]/e[i][r]);
			else lo=min(lo,mue[i][r]*m[i]/e[i][r]);
	return lo;
}

/* rho_kn_kl(i,r)

   Berechnung der Auslastung Knoten (i), Klasse (r).

   Th.Hahn  22.11.1989
*/

double rho_kn_kl(i,r)
int i,r;
{
	double h;

	if (typ[i]==4) return .0;
	else 
		if (fabs(h=lambda[r]*e[i][r]/m[i]/mue[i][r])<MINRHO)
			if (h<.0) return (-MINRHO);
			else return MINRHO;
		else return h;
}

/* rho_kn(i)

   Berechnung der Auslastung Knoten (i).

   Th.Hahn  22.11.1989
*/

double rho_kn(i)
int i;
{
	int r;
	double h=.0;

	for(r=0;r<kl;r++) h+=rho_kn_kl(i,r);
	if (fabs(h)<MINRHO)
		if (h<.0) return (-MINRHO);
		else return MINRHO;
	return h;
}

/* abl_rho_kn_kl(i,r)

   Ableitung von rho_kn_kl() nach Lambda[r].

   Th.Hahn  22.11.1989
*/

double abl_rho_kn_kl(i,r)
int i,r;
{
	return (e[i][r]/m[i]/mue[i][r]);
}

/* sig(i,r)

   Berechnung von rho((i),0)+rho((i),1)+...+rho((i),(r)-1) (Knoten (i)).

   Th.Hahn  12.11.1989
*/

double sig(i,r)
int i,r;
{
	int k=0;
	double h=.0;

	while(k<=r-1)
	{
		h+=rho_kn_kl(i,k);
		k++;
	}
	return h;
}

/* p_wart(rho,i)

   Wartewahrscheinlichkeit fuer Auslastung (rho) und Knoten (i).

   Th.Hahn  15.11.89
*/

double p_wart(rho,i)
double rho;
int i;
{
	int f;
	double fak=1.0,sm1=.0,h0=1.0,h1=m[i]*rho;

	if (rho>=1.0) return 1.0;
	if (rho<=MINRHO) return .0;
	if (opt&topt("a"))
	{
		if (!pwarn)
			if (m[i]>1 && fabs(.7-rho)<.001)
			{
				fprintf(stderr,"** p_wart()-Warnung **\n");
				pwarn=true;
			}
		if (rho>=.7) return ((pow(rho,(double)(m[i]))+rho)/2.0);
		else
			if (!(opt&topt("w")) || opt&topt("w") && rho>=.3)
				return (pow(rho,(m[i]+1.0)/2.0));
			else
			{
				for(f=0;f<m[i];f++)
				{
					sm1+=h0/fak;
					fak*=f+1.0;
					h0*=h1;
				}
				return (1.0-sm1/exp(m[i]*rho));
			}
	}
	else
	{
		for(f=0;f<m[i];f++)
		{
			sm1+=h0/fak;
			fak*=f+1.0;
			h0*=h1;
		}
		h0/=fak*(1.0-rho);
		return (h0/(sm1+h0));
	}
}

/* abl_p_wart(i,r,s)

   Ableitung der Wartewahrscheinlichkeit nach Lambda[r] fuer
   Knoten (i) und Auslastung sig(i,s).

   Th.Hahn  15.11.1989
*/

double abl_p_wart(i,r,s)
int i,r,s;
{
	int f;
	double rho=sig(i,s),g=abl_rho_kn_kl(i,r),fak=1.0,sm1=.0,sm2=.0,
	h0=1.0,h1=m[i]*rho,h2;

	if (r>=s || rho<=MINRHO || rho>=1.0) return .0;
	if (opt&topt("a"))
	{
		if (!pwarn)
			if (m[i]>1 && fabs(.7-rho)<.001)
			{
				fprintf(stderr,"** abl_p_wart()-Warnung **\n");
				pwarn=true;
			}
		if (rho>=.7) return (.5*(m[i]*pow(rho,m[i]-1.0)*g+g));
		else
			if (!(opt&topt("w")) || opt&topt("w") && rho>=.3)
				return (.5*(m[i]+1.0)*pow(rho,(m[i]-1.0)*.5)*g);
			else
			{
				for(f=0;f<m[i];f++)
				{
					sm1+=h0/fak;
					if (f) sm2+=h0/rho*f/fak;
					fak*=f+1.0;
					h0*=h1;
				}
				return (g/exp(m[i]*rho)*(m[i]*sm1-sm2));
			}
	}
	else
	{
		for(f=0;f<m[i];f++)
		{
			sm1+=h0/fak;
			if (f) sm2+=h0/h1*f/fak;
			fak*=f+1.0;
			h0*=h1;
		}
		h1=1.0-rho;
		h0/=fak*h1;
		sm1+=h0;
		h2=pow(rho,m[i]-1.0);
		h2=(m[i]*h1+rho)*h2*pow((double)(m[i]),m[i]-1.0)/h1/h1/fak;
		return (e[i][r]/mue[i][r]*(h2*sm1-h0*(sm2+h2))/sm1/sm1);
	}
}

/* add1() .. add3()

   Hilfsfunktionen fuer die Berechnung der mittleren
   Auftragszahlen.

   Th.Hahn  22.11.1989
*/

double add1(i,j)
int i,j;
{
	int r;
	double h=.0;

	for(r=0;r<=j;r++) h+=rho_kn_kl(i,r)/mue[i][r];
	return h;
}

double add2(i,j)
int i,j;
{
	int r;
	double h=.0;

	for(r=0;r<=j;r++)
		h+=lambda[r]*e[i][r]*(c[i][r]+1.0)/mue[i][r]/mue[i][r]; 
	return h;
}

double add3(i,j)
int i,j;
{
	int r;
	double h=.0;

	for(r=0;r<=j;r++)
		h+=rho_kn_kl(i,r)*(c[i][r]+1.0)/mue[i][r];
	return h;
}
