static char *SccsId = "@(#)PEPSY:\tstuetz.c\t1.3\t92/12/09 16:44:04";

/* stuetz.c

   -u :

   Berechnung einer sortierten Stuetzpunktdatei rho-maa 'wstuetz'
   fuer einen ausgewaehlten Knoten (ukn) und eine ausgewaehlte
   Klasse (ukl) durch Variation von mue[ukn][ukl].

   -l :

   Berechnung einer sortierten Stuetzpunktdatei lambda-Wert(Syst.Glg.)
   'wlstuetz' fuer eine ausgewaehlte Klasse (lkl) durch Variation
   von Lambda[lkl]. Intervallvorgabe mit -i.
*/

#include "multidef.h"

extern unsigned int   opt;
extern int    ukn,
ukl,
lkl,
*typ,
*auftrag,
kl,
it;
extern double *lambda,
**e,
**mue,
iu,
io;

extern void   Analyse(),
Blockfree(),
fehler_ex();
extern long   ftell();
extern FILE   *fopen();
extern double fabs(),
k(),
sum_k_HNB(),
HNenner(),
rho_kn_kl(),
min_lam();

bool   mit;
long   wsfp;
 unsigned long  count=0;
FILE   *st=0;
double maxab,
hl1,hl2,hl3,
hr1,hr2,hr3;

struct Wert {
	double rho,
	maa,
	mue;
	struct Wert *follow[3];    /* [2] ist equal-Pointer */
	int cit;   /* Zahl benoetigter Multisum-Iterationen */
};

struct Wert *first=0,
*hpw1,*hpw2;

void set_mue(),
set_wert(),
insort(),
travers(),
check(),
vgl(),
fuelle(),
stuetz(),
lstuetz();

/* set_mue(mw)

   setzt neuen mue-Wert (mw) fuer Knoten ukn und Klasse ukl
   unter Beruecksichtigung des Knotentyps.

   Th.Hahn  10.08.1989
*/

void set_mue(mw)
double mw;
{
	int f;

	if (typ[ukn]==1 || typ[ukn]==2)
		for(f=0;f<kl;mue[ukn][f]=mw,f++);
	else
		mue[ukn][ukl]=mw;
}

/* set_wert(p)

   Traegt Analysedaten in Struct Wert (*p) ein.

   Th.Hahn  29.11.1989
*/

void set_wert(p)
struct Wert *p;
{
	(*p).follow[0]=(*p).follow[1]=(*p).follow[2]=0;
	(*p).rho=rho_kn_kl(ukn,ukl);
	(*p).mue=mue[ukn][ukl];
	(*p).maa=k(ukn,ukl);
	(*p).cit=it;
}

/* travers(p)

   gibt Wertetabelle geordnet ab (*p) in File (*st) aus und hebt
   dabei die Speicherreservierung auf (rekursiv).

   Th.Hahn  07.07.1989
*/

void travers(p)
struct Wert *p;
{
	if ((*p).follow[0]) travers((*p).follow[0]);
	hl1=(*p).rho;
	hl2=(*p).maa;
	hl3=(*p).mue;
	if ((hpw1=(*p).follow[2]))
		while(hpw1)
		{
			hpw2=hpw1;
			if ((*hpw1).maa > hl2)
			{
				hl2=(*hpw1).maa;
				hl3=(*hpw1).mue;
			}
			hpw1=(*hpw1).follow[2];
			free(hpw2);
			count--;
		}
	fprintf(st,"%lf %lf (%lf) (%d)\n",hl1,hl2,hl3,(*p).cit);
	if ((*p).follow[1]) travers((*p).follow[1]);
	if (p) free(p);
	else fprintf(stderr,"** travers()-free-error **\n");
}

/* insort(p)

   sortiert neues Wertepaar (*p) in Liste ein.

   Th.Hahn  07.07.1989
*/

void insort(p)
struct Wert *p;
{
	struct Wert *h,*g;
	bool eq=false;

	count++;
	h=g=first;
	while(h)
	{
		g=h;
		if (fabs((*h).rho - (*p).rho)<1e-06)  /* equal */
		{
			while(h)
			{
				g=h;
				h=(*h).follow[2];
			}
			(*g).follow[2]=p;
			eq=true;
			break;
		}
		if ((*h).rho > (*p).rho) h=(*h).follow[0];
		else h=(*h).follow[1];
	}
	if (!eq)
		if ((*g).rho > (*p).rho) (*g).follow[0]=p;
		else (*g).follow[1]=p;
	if (count==MAXPKT)
	{
		fprintf(stderr,"** MAXPKT erreicht ! **\n");
		travers(first);
		first=0;
		fseek(st,wsfp,0);
		fprintf(st,"%d",count);
		Blockfree();
		fehler_ex(0);
	}
}

/* check(p,fmin)

   sucht unter allen equal-Eintraegen von (*p) den kleinsten
   (wenn (fmin) true), sonst den groessten maa-Wert.

   Th.Hahn  10.08.1989
*/

void check(p,fmin)
struct Wert *p;
bool fmin;
{
	hr1=(*p).maa;
	hr2=(*p).mue;
	hr3=(*p).rho;
	while((*p).follow[2])
	{
		p=(*p).follow[2];
		if (!fmin && (*p).maa > hr1 || fmin && (*p).maa < hr1)
		{
			hr1=(*p).maa;
			hr2=(*p).mue;
			hr3=(*p).rho;
		}
	}
}

/* vgl(l,r)

   sucht unter allen equal-Eintraegen von (*l),(*r) diejenigen aus,
   die am dichtesten zusammenliegen (bzgl. ihrer maa-Werte).

   Th.Hahn  10.08.1989
*/

void vgl(l,r)
struct Wert *l,*r;
{
	check(l,true);
	hl1=hr1;
	hl2=hr2;
	hl3=hr3;
	check(r,false);
}

/* fuelle(p,right)

   verdichtet Anfangswertetabelle ab (*p) (rekursiv).
   Wird Flag (right) gesetzt, wird bei der ersten Verzweigung
   nach rechts verzweigt.

   Th.Hahn  07.07.1989
*/

void fuelle(p,right)
struct Wert *p;
bool right;
{
	struct Wert *g=0;
	double rhoi;
	bool f=right;

	for(;f<2;f++)
		if ((g=(*p).follow[f])!=(struct Wert *)0)
		{
			if (f)
			{
				while((*g).follow[0]) g=(*g).follow[0];
				vgl(g,p);
			}
			else
			{
				while((*g).follow[1]) g=(*g).follow[1];
				vgl(p,g);
			}
			if (hl1-hr1 > maxab)
			{
				set_mue(.5*(hl2+hr2));
				Analyse();
				if (mit) rhoi=.5*(hl3+hr3);
				else rhoi=rho_kn_kl(ukn,ukl);
				if (hl3>rhoi && rhoi>hr3)
				{
					g=(struct Wert *)(block(sizeof(struct Wert)));
					set_wert(g);
					(*g).rho=rhoi;
					if (mit)
					{
						(*g).maa=.5*(hl1+hr1);
						(*g).mue=.5*(hl2+hr2);
						fprintf(stdout,"** count %lu rho %lf maa %lf mue %lf (%d) -lin.- **\n",count,
						    rhoi,(*g).maa,(*g).mue,it);
					}
					else
					{
						(*g).maa=k(ukn,ukl);
						(*g).mue=mue[ukn][ukl];
					}
					insort(g);
					if (f) fuelle(p,true);
					else fuelle(p,false);
				}
				else fuelle((*p).follow[f],false);
			}
			else fuelle((*p).follow[f],false);
		}
}

/* stuetz()

   Berechnet fuer einen Knoten die Stuetzstellen zum Zeichnen einer
   rho-maa-Kurve. Eine entsprechende Wertetabelle wird in File wstuetz
   ausgegeben und kann weiterverarbeitet werden.

   Th.Hahn  07.07.1989
*/

void stuetz()
{
	struct Wert *p;
	bool up=false;

	if (e[ukn][ukl]==.0)
	{
		fprintf(stderr,"** stuetz() : Besuchshaeufigkeit Knoten %d Klasse %d gleich null ! **\n",ukn,ukl);
		return;
	}
	st=fopen("wstuetz","w");
	if (!st)
	{
		fprintf(stderr,"File 'wstuetz' kann nicht geoeffnet werden !\n");
		return;
	}
	if (typ[ukn]==4)
		fprintf(stderr,"** stuetz() : ausgewaehlter Knoten ist IS ! **\n");
	fprintf(st,"\n");
	wsfp=ftell(st);
	fprintf(st,"0      stuetz() : (Knoten : %d   Klasse : %d)\n",ukn,ukl);
	fprintf(st,".0 .0\n");
	count=1;
	maxab=auftrag[ukl]/(double)(STPKT);
	if (maxab<MINMAB) maxab=MINMAB;
	set_mue(1.0);
	Analyse();
	p=first=(struct Wert *)(block(sizeof(struct Wert)));
	set_wert(p);
	count++;
	while(true)
	{
		if (up)
			if ((*p).maa < auftrag[ukl]-maxab && (*p).mue > MINMUE)
			{
				mue[ukn][ukl]*=XF;
				p=0;
			}
			else break;
		else
			if ((*p).maa > maxab && (*p).mue < MAXMUE)
			{
				mue[ukn][ukl]*=YF;
				p=0;
			}
			else
			{
				up=true;
				p=first;
				set_mue(1.0);
			}
		if (!p)
		{
			set_mue(mue[ukn][ukl]);
			Analyse();
			p=(struct Wert *)(block(sizeof(struct Wert)));
			set_wert(p);
			insort(p);
		}
	}
	fuelle(first,false);
	travers(first);
	first=0;
	fseek(st,wsfp,0);
	fprintf(st,"%d ",count);
	fclose(st);
	st=0;
}

/* lstuetz()

   Berechnet fuer Klasse (lkl) die Stuetzstellen zum Zeichnen einer
   lambda-Wert(Syst.Glg.)-Kurve. Eine entsprechende Wertetabelle wird 
   in File wlstuetz ausgegeben und kann weiterverarbeitet werden.

   Achtung : Bei Aufruf von -l muss das Infile zusaetzliche Angaben
   -------   enthalten : alle Lambda[] (Lambda[(lkl)] wird variiert)

   Th.Hahn  14.11.1989
*/

void lstuetz()
{
	unsigned int f;
	double xe,y,xmax;

	st=fopen("wlstuetz","w");
	if (!st)
	{
		fprintf(stderr,"File 'wlstuetz' kann nicht geoeffnet werden !\n");
		return;
	}
	fprintf(stderr,"** Achtung : fuer -l sind Zusatzangaben im Eingabefile erforderlich ! **\n");
	for(f=0;f<kl;f++)
	{
		fscanf(stdin,"%lf",&lambda[f]);         /* ! */
		if (lambda[f]<null || lambda[f]>MAXFLOAT) lambda[f]=null;
		else 
			if (f!=lkl)
				if (lambda[f]>(y=min_lam(f)))
					fprintf(stderr,"** lstuetz() : lambda(%d) ausserhalb Definitionsbereich (<=%g) **\n",
					    f,y);
	}
	lambda[lkl]=xmax=min_lam(lkl);
	if (!(opt&topt("i"))) io=xmax;
	else
	{
		if (iu<null) iu=null;
		if (iu>=io)
		{
			iu=null;
			io=xmax;
		}
	}
	xe=(io-iu)/(MAXPKT-1.0);
	fprintf(st,"\n%d  lstuetz() : (lambda(x) :",MAXPKT);
	for(f=0;f<kl;f++)
		if (f!=lkl) fprintf(st," %lf",lambda[f]);
		else fprintf(st," x");
	fprintf(st,")\n");
	lambda[lkl]=iu;
	for(f=0;f<MAXPKT;f++)
	{
		y=auftrag[lkl]*HNenner(-1,-1,lkl)-sum_k_HNB(lkl);
		fprintf(st,"%lf %g\n",lambda[lkl],y);
		lambda[lkl]+=xe;
	}
	fclose(st);
	st=0;
}
