static char *SccsStuetz_h = "@(#)PEPSY:\tstuetz.h\t1.3\t92/12/09 16:44:06";

#ifndef STUETZ_H
#define STUETZ_H 

extern long ftell();

void set_mue(),
     set_wert(),
     insort(),
     travers(),
     check(),
     vgl(),
     fuelle(),
     stuetz(),
     lstuetz();

#define MINMUE 1e-05
#define MAXMUE 1e+05
#define MAXPKT 622  /* max. Zahl Stuetzpkt. (graph : 622) */
#define STPKT  50   /* ungef.Zahl gew. Stuetzpkt. */
#define MINAB  .1   /* minim. Wert von maxab */
#define XF .3
#define YF 2.2

FILE *st=0;
bool mit;
ulong count=0;
ulong Mcount=0;        
long wsfp;
double maxab;
double hl1,hl2,hl3;
double hr1,hr2,hr3;
struct Wert {
             double rho,
                    maa,
                    mue;
             struct Wert *follow[3]; /* [2] ist equ.-Pointer */
             int it; /* Zahl benoet. Multisum-Iterationen */
            };
struct Wert *first=0;
struct Wert *hpw1,*hpw2;

/* set_mue(mw)

   setzt neuen mue-Wert (mw) fuer Knoten knot und Klasse klas
   unter Beruecksichtigung des Knotentyps.

   Th.Hahn  10.08.1989
*/

void set_mue(mw)
double mw;
{
 int f;

 if (typ[knot]<3)
  for(f=0;f<kl;mue[knot][f]=mw,f++);
 else
  mue[knot][klas]=mw;
}

/* set_wert(p)

   Traegt Analysedaten in Struct Wert (*p) ein.

   Th.Hahn  29.11.1989
*/

void set_wert(p)
struct Wert *p;
{
 (*p).follow[0]=(*p).follow[1]=(*p).follow[2]=0;
 (*p).rho=rho_kn_kl(knot,klas);
 (*p).mue=mue[knot][klas];
 (*p).maa=k(knot,klas);
 (*p).it=it;
}

/* insort(p)

   sortiert neues Wertepaar (*p) in Liste ein.

   Th.Hahn  07.07.1989
*/

void insort(p)
struct Wert *p;
{
 register struct Wert *h,*g;
 bool eq=false;

 count++;
 h=g=first;
 while(h)
 {
  g=h;
  if (fabs((*h).rho - (*p).rho)<1e-06)  /* equal */
  {
   while(h)
   {
    g=h;
    h=(*h).follow[2];
   }
   (*g).follow[2]=p;
   eq=true;
   break;
  }
  if ((*h).rho > (*p).rho) h=(*h).follow[0];
  else h=(*h).follow[1];
 }
 if (!eq)
  if ((*g).rho > (*p).rho) (*g).follow[0]=p;
  else (*g).follow[1]=p;
 if (count==MAXPKT)
 {
  fprintf(stderr,"** MAXPKT erreicht ! **\n");
  travers(first);
  first=0;
  fseek(st,wsfp,0);
  fprintf(st,"%d",count);
  Blockfree();
  exit(0);
 }
}

/* travers(p)

   gibt Wertetabelle geordnet ab (*p) in File (*st) aus und hebt
   dabei die Speicherreservierung auf (rekursiv).

   Th.Hahn  07.07.1989
*/

void travers(p)
struct Wert *p;
{
 if ((*p).follow[0]) travers((*p).follow[0]);
 hl1=(*p).rho;
 hl2=(*p).maa;
 hl3=(*p).mue;
 if ((hpw1=(*p).follow[2]))
  while(hpw1)
  {
   hpw2=hpw1;
   if ((*hpw1).maa > hl2)
   {
    fprintf(stderr," >\n");
    hl2=(*hpw1).maa;
    hl3=(*hpw1).mue;
   }
   hpw1=(*hpw1).follow[2];
   free(hpw2);
   count--;
  }
 fprintf(st,"%lf %lf (%lf) (%d)\n",hl1,hl2,hl3,(*p).it);
 if ((*p).follow[1]) travers((*p).follow[1]);
 if (p) free(p);
 else fprintf(stderr,"** free-p-error **\n");
}

/* check(p,fmin)

   sucht unter allen equal-Eintraegen von (*p) den kleinsten
   (wenn (fmin) true), sonst den groessten maa-Wert.

   Th.Hahn  10.08.1989
*/

void check(p,fmin)
struct Wert *p;
bool fmin;
{
 hr1=(*p).maa;
 hr2=(*p).mue;
 hr3=(*p).rho;
 while((*p).follow[2])
 {
  p=(*p).follow[2];
  if (!fmin && (*p).maa > hr1 || fmin && (*p).maa < hr1)
  {
   hr1=(*p).maa;
   hr2=(*p).mue;
   hr3=(*p).rho;
  }
 }
}

/* vgl(l,r)

   sucht unter allen equal-Eintraegen von (*l),(*r) diejenigen aus,
   die am dichtesten zusammenliegen (bzgl. ihrer maa-Werte).

   Th.Hahn  10.08.1989
*/

void vgl(l,r)
struct Wert *l,*r;
{
 check(l,true);
 hl1=hr1;
 hl2=hr2;
 hl3=hr3;
 check(r,false);
}

/* fuelle(p,right)

   verdichtet Anfangswertetabelle ab (*p) (rekursiv).
   Wird Flag (right) gesetzt, wird bei der ersten Verzweigung
   nach rechts verzweigt.

   Th.Hahn  07.07.1989
*/

void fuelle(p,right)
struct Wert *p;
bool right;
{
 struct Wert *g=0;
 double rhoi;
 bool f=right;

 for(;f<2;f++)
  if ((g=(*p).follow[f])>0)
  {
   if (!f)
   {
    while((*g).follow[1]) g=(*g).follow[1];
    vgl(p,g);
   }
   else
   {
    while((*g).follow[0]) g=(*g).follow[0];
    vgl(g,p);
   }
   if (hl1-hr1 > maxab)
   {
    set_mue(.5*(hl2+hr2));
    Analyse();
    if (!mit) rhoi=rho_kn_kl(knot,klas);
    else rhoi=.5*(hl3+hr3);
    if (hl3>rhoi && rhoi>hr3)
    {
     g=(struct Wert *)(block(sizeof(struct Wert)));
     set_wert(g);
     (*g).rho=rhoi;
     if (mit)
     {
      (*g).maa=.5*(hl1+hr1);
      (*g).mue=.5*(hl2+hr2);
      fprintf(stderr,"  count %lu rho %lf maa %lf mue %lf (%d)\n",count,rhoi,(*g).maa,(*g).mue,it);
     }
     else
     {
      (*g).maa=k(knot,klas);
      (*g).mue=mue[knot][klas];
     }
     insort(g);
     if (f) fuelle(p,true);
     else fuelle(p,false);
    }
    else fuelle((*p).follow[f],false);
   }
   else fuelle((*p).follow[f],false);
  }
}

/* stuetz()

   Berechnet fuer einen Knoten die Stuetzstellen zum Zeichnen einer
   Rho-K-Kurve. Eine entsprechende Wertetabelle wird in File (*st)
   ausgegeben und kann weiterverarbeitet werden.

   Th.Hahn  07.07.1989
*/

void stuetz()
{
 struct Wert *p;
 bool up=false;

 st=fopen("wstuetz","w");
 if (!st)
 {
  fprintf(stderr,"File 'wstuetz' kann nicht geoeffnet werden !\n");
  return;
 }
 fprintf(st,"1\n");
 fprintf(st,".0 1.0\n");
 fprintf(st,".0  .0\n");
 wsfp=ftell(st);
 fprintf(st,"0     \n");
 fprintf(st,".0 .0\n");
 count=1;
 maxab=auftrag[klas]/(double)(STPKT);
 if (maxab<MINAB) maxab=MINAB;
 set_mue(1.0);
 Analyse();
 p=first=(struct Wert *)(block(sizeof(struct Wert)));
 set_wert(p);
 count++;
 while(true)
 {
  if (up)
   if ((*p).maa < auftrag[klas]-maxab && (*p).mue>MINMUE)
   {
    mue[knot][klas]*=XF;
    p=0;
   }
   else break;
  else
   if ((*p).maa > maxab && (*p).mue<MAXMUE)
   {
    mue[knot][klas]*=YF;
    p=0;
   }
   else
   {
    up=true;
    p=first;
    set_mue(1.0);
   }
  if (!p)
  {
   set_mue(mue[knot][klas]);
   Analyse();
/*
   if (!mit)
*/
   {
    p=(struct Wert *)(block(sizeof(struct Wert)));
    set_wert(p);
    insort(p);
   }
  }
 }
 fuelle(first,false);
 travers(first);
 first=0;
 fseek(st,wsfp,0);
 fprintf(st,"%d",count);
}

/* lstuetz()

   Berechnet fuer eine Klasse die Stuetzstellen zum Zeichnen einer
   Lambda-F-Kurve. Eine entsprechende Wertetabelle wird in File (*st)
   ausgegeben und kann weiterverarbeitet werden.

   Achtung : Bei Aufruf von -l (bzw. -i) muss das Infile zusaetzliche
   -------   Angaben enthalten :

               -l : alle Lambda[r] bis auf Lambda[klas]
                    Zahl der Graphikpunkte (max.622)
               -i : zusaetzlich Lambda[klas]-min, Lambda[klas]-max

   Th.Hahn  14.11.1989
*/

void lstuetz()
{
 int f,g,xpkt;
 double xe,y,xmax,xv1=1e-16,xv2=.0;

 st=fopen("wlstuetz","w");
 if (!st)
 {
  fprintf(stderr,"File 'wlstuetz' kann nicht geoeffnet werden !\n");
  return;
 }
 for(f=0;f<kl;f++)
  if (f!=klas)
  {
   fscanf(stdin,"%lf",&lambda[f]);        /* ! */
   if (lambda[f]<1e-16) lambda[f]=1e-16;
   for(g=0;g<kn;g++)
   {
    xe=mue[g][f]/e[g][f];
    if (typ[g]!=4 && lambda[f]>m[g]*xe || typ[g]==4 && lambda[f]>auftrag[f]*xe)
    {
     fprintf(stderr,"** lambda out %d **\n",f);
     lambda[f]=1e-16;
    }
   }
  }
 fscanf(stdin,"%d",&xpkt);                /* ! */
 if (xpkt<0 || xpkt>MAXPKT) xpkt=MAXPKT;
 if (typ[0]==4) xmax=auftrag[klas]*mue[0][klas]/e[0][klas];
 else xmax=m[klas]*mue[0][klas]/e[0][klas];
 if (opt&256)
 {
  fscanf(stdin,"%lf %lf",&xv1,&xv2);      /* ! */
  if (xv1<1e-16) xv1=1e-16;
/*
  if (xv2>xmax || xv1>=xv2)
*/
  if (xv1>=xv2)
  {
   xv1=1e-16;
   xv2=xmax;
  }
 }
 else xv2=xmax;
 xe=(xv2-xv1)/(xpkt-1);
 fprintf(st,"1\n");
 fprintf(st,".0 .0\n");
 fprintf(st,".0 .0\n");
 fprintf(st,"%d (lambda(x) :",xpkt);
 for(f=0;f<kl;f++)
  if (f!=klas) fprintf(st," %lf",lambda[f]);
  else fprintf(st," x");
 fprintf(st,")\n");
 lambda[klas]=xv1;
 for(g=0;g<xpkt;g++)
 {
  y=auftrag[klas]*HNenner(-1,-1,klas)-sum_k_HNB(klas);
  fprintf(st,"%lf %lf\n",lambda[klas],y);
  lambda[klas]+=xe;
 }
}

#endif
