static char *SccsId = "@(#)PEPSY:\tvermom.c\t1.3\t92/12/09 16:44:07";

/* vermom.c

   Routinen zur appr. Berechnung von Verteilungen und Momenten
   fuer M/M/1-FCFS-Knoten.
*/

#include "multidef.h"

extern bool   mm1;
extern int    *typ,
kn,
kl,
auftraege;
extern unsigned int   opt;
extern double **mue;

extern void   fout_nl();
extern char   *block();
extern unsigned int   topt();
extern double pow(),
sqrt(),
fabs(),
rho_kn(),
k();

double **prand=0,   /* appr. Randwahrscheinlichkeiten */
*auftrm1=0,  /* appr. 1. Auftragsmomente */
*auftrzm2=0, /* appr. 2. zentr. Auftragsmomente */
*maazm2=0,   /* appr. Varianz maa */
*mvzzm2=0,   /* appr. Varianz mvz */
*mwzzm2=0,   /* appr. Varianz mwz */
*mhilf=0;    /* Hilfsfeld fuer Momente */

/* varkoef(ew,vr)

   Ausgabe der Variationskoeffizienten, wobei :
   (ew) Pointer auf Erwartungswerte, (vr) Pointer auf Varianzen.
   (M/M/1-FCFS-Knoten, Einklassenfall !)

   Th.Hahn  28.12.1989
*/

void varkoef(ew,vr)
double *ew,*vr;
{
	unsigned int g;

	for(g=0;g<kn;g++)
		if (typ[g]==1)
		{
			fprintf(stdout,"Knoten %d : ",g);
			if (fabs(ew[g])>null) fout_nl(sqrt(vr[g])/ew[g]);
			else fout_nl(MAXFLOAT);
		}
}

/* momaus(fmom,ew,vr)

   Ausgabe von Erwartungswert, Varianz und Variationskoeffizienten fuer
   folgenden Fall :
   mit (*fmom)() koennen hoehere (auch zentrale) Momente fuer M/M/1-FCFS-
   Knoten berechnet werden, die Momente werden dabei im Erwartungswerte-
   Feld (ew[]) und im Varianzen-Feld (vr[]) abgespeichert.
   (Einklassenfall !)

   Th.Hahn  28.12.1989
*/

void momaus(fmom,ew,vr)
void (*fmom)();
double *ew,*vr;
{
	unsigned int f;

	fprintf(stdout,"Erwartungswerte :\n");
	(*fmom)(false,1,ew);
	for(f=0;f<kn;f++)
		if (typ[f]==1)
		{
			fprintf(stdout,"Knoten %d : ",f);
			fout_nl(ew[f]);
		}
	fprintf(stdout,"Varianzen :\n");
	(*fmom)(true,2,vr);
	for(f=0;f<kn;f++)
		if (typ[f]==1)
		{
			fprintf(stdout,"Knoten %d : ",f);
			fout_nl(vr[f]);
		}
	fprintf(stdout,"Variationskoeffizienten :\n");
	varkoef(ew,vr);
}

/* momaus1(few,fvar,vr)

   Ausgabe von Varianz und Variationskoeffizienten fuer folgenden Fall :
   mit (*few)() kann der Erwartungswert, mit (*fvar)() die Varianz fuer
   M/M/1-FCFS-Knoten berechnet werden.
   Die Varianzen werden im Varianzen-Feld (vr[]) abgespeichert.
   (Einklassenfall !)

   Th.Hahn  28.12.1989
*/

void momaus1(few,fvar,vr)
double (*few)(),(*fvar)();
double *vr;
{
	unsigned int f;

	for(f=0;f<kn;f++)
		if (typ[f]==1) mhilf[f]=(*few)(f,0);
		else mhilf[f]=.0;
	for(f=0;f<kn;f++)
		if (typ[f]==1)
		{
			vr[f]=(*fvar)(f);
			fprintf(stdout,"Knoten %d : ",f);
			fout_nl(vr[f]);
		}
		else vr[f]=.0;
	fprintf(stdout,"Variationskoeffizienten :\n");
	varkoef(mhilf,vr);
}

/* var_maa(i)

   Berechnet die appr. Varianz der mittleren Auftragszahl 
   fuer M/M/1-FCFS-Knoten (i).
   (Einklassenfall !)

   Th.Hahn  30.12.1989
*/

double var_maa(i)
int i;
{
	double h=rho_kn(i),h1=(auftraege-1.0)/auftraege;

	return ((1.0-pow(h,vmaa_n))*h/pow(1.0-h1*h,2.0));
}

/* var_mvz(i)

   Berechnet die appr. Varianz der mittleren Verweilzeit
   fuer M/M/1-FCFS-Knoten (i).
   (Einklassenfall !)

   Th.Hahn  30.12.1989
*/

double var_mvz(i)
int i;
{
	double h=rho_kn(i),h1=(auftraege-1.0)/auftraege;

	return ((1.0-pow(h,vmvz_n))/pow(mue[i][0]*(1.0-h1*h),2.0));
}

/* var_mwz(i)

   Berechnet die appr. Varianz der mittleren Wartezeit
   fuer M/M/1-FCFS-Knoten (i).
   (Einklassenfall !)

   Th.Hahn  30.12.1989
*/

double var_mwz(i)
int i;
{
	double h=rho_kn(i),h1=(auftraege-1.0)/auftraege;

	return ((1.0-pow(h,vmwz_n))*h*(2.0-h)/pow(mue[i][0]*(1.0-h1*h),2.0));
}

/* auftrmom(zentral,k,z)

   berechnet (k)-tes Moment fuer die Anzahl der Kunden in den
   M/M/1-FCFS-Knoten des Netzes aus den Randwahrscheinlichkeiten,
   wobei :   (zentral) Flag fuer zentrale Momente,
             (z) Zeiger auf double-Ergebnisvektor.
   (Einklassenfall !)

   Th.Hahn  28.12.1989
*/

void auftrmom(zentral,k,z)
bool zentral;
unsigned int k;
double *z;
{
	unsigned int f,g;

	for(f=0;f<kn;f++)
	{
		z[f]=.0;
		if (typ[f]==1)
		{
			for(g=0;g<=auftraege;g++)
				if (zentral)
					z[f]+=pow(g-auftrm1[f],(double)(k))*prand[f][g];
				else
					if (g) z[f]+=pow((double)(g),(double)(k))*prand[f][g];
		}
	}
}

/* umvert()

   Umverteilung der appr. Randwahrscheinlichkeiten.
   (M/M/1-FCFS-Knoten, Einklassenfall !)

   Th.Hahn  27.12.1989
*/

void umvert()
{
	int f,g;
	double h,h1;

	fprintf(stderr,"** umvert() **\n");
	if (auftraege==1)
		fprintf(stderr,"** umvert() nicht moeglich **\n");
	else
		for(g=0;g<kn;g++)
			if (typ[g]==1)
				if (prand[g][auftraege]==.0)
					fprintf(stderr,"** umvert() Knoten %d nicht moeglich **\n",g);
				else
				{
					h=k(g,0);
					for(f=1;f<=auftraege;f++) h-=f*prand[g][f];
					h*=3.0/(-auftraege-1.0);
					if (h<=.0 || prand[g][auftraege]<h)
						fprintf(stderr,"** umvert() Knoten %d nicht moeglich **\n",g);
					else
					{
						h1=(auftraege-1.0)*auftraege/2.0;
						for(f=1;f<auftraege;f++) prand[g][f]+=f*h/h1;
						prand[g][auftraege]-=h;
					}
				}
}

/* ber_prand()

   Berechnung der appr. Randwahrscheinlichkeiten fuer alle
   M/M/1-FCFS-Knoten im Netz (Einklassenfall).

   Alle fuer die Berechnung entsprechender Verteilungen und Momente
   notwendigen Speicherplaetze werden zuvor reserviert.

   Th.Hahn  23.01.1990
*/

void ber_prand()
{
	unsigned int f,g;
	double norm,prh,h;

	if (mm1 && kl==1)
	{
		f=kn*sizeof(double);
		prand=(double **)(block(kn*sizeof(double *)));
		auftrm1=(double *)(block(f));
		auftrzm2=(double *)(block(f));
		maazm2=(double *)(block(f));
		mvzzm2=(double *)(block(f));
		mwzzm2=(double *)(block(f));
		mhilf=(double *)(block(f));
		for(f=0;f<kn;f++) prand[f]=0;
		for(f=0;f<kn;f++)
			if (typ[f]==1)
			{
				prand[f]=(double *)(block((auftraege+1)*sizeof(double)));
				h=rho_kn(f);
				if (h<=.0)
				{
					prand[f][0]=1.0;
					for(g=1;g<=auftraege;g++) prand[f][g]=.0;
				}
				else
					if (h>=1.0)
					{
						for(g=0;g<auftraege;g++) prand[f][g]=.0;
						prand[f][auftraege]=1.0;
					}
					else
					{
						prh=1.0;
						norm=.0;
						for(g=0;g<=auftraege;g++)
						{
							prand[f][g]=(1.0-h)*prh/(1.0-pow((double)(g)/auftraege,prand_n)*h);
							prh*=h;
							norm+=prand[f][g];
						}
						if (norm!=1.0)
							for(g=0;g<=auftraege;g++) prand[f][g]/=norm;
					}
			}
		if (opt&topt("t")) umvert();
	}
}
