/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsFunktionen_h = "@(#)PEPSY:\tfunktionen.h\t1.14\t95/05/04 12:24:45";

/* Hilfs- und Hauptfunktionen fuer Moments and Mean Value Analysis (MMVA) */

#ifndef FUNKTIONEN_H
#define FUNKTIONEN_H

#define MAX(a,b) ((a)>(b)?(a):(b))
#define MIN(a,b) ((a)<=(b)?(a):(b))

void		AKTdxpe();
void		AKTpe();
double		ANTWZEIT();
double		ANTWZEIT2();
void		Blockalloc1();
void		Blockalloc2();
void		Blockalloc3();
void		Blockfree();
Ulong		Index();
void		LEIST();
void		MOMAUS1();
void		MOMAUS2();
void		MWA();
void		MWA2();
int		main();
void		NORM();
void		RHO();
void		Reset();
void		SMOMAUS();
double		Verlust();
void		WAHRSCH();
void		abbruch();
void		aktprand();
void		aktpwschl();
struct Zustand	*all_ws();
double		anzvekt();
void		auftrmom();
bool		bednextzv();
double		binkoef();
char		*block();
void		code();
Ulong		decode();
Ulong		dekr();
Ulong		dekrdec();
double		dxANTWZEIT();
void		dxMWA();
Ulong		dxindex();
Ulong		dxpindex();
double		fak();
void		foutnl();
void		foutsp();
struct Zustand*	ini_liste();
double		mul_rho();
bool		nextnzv();
bool		nextzv();
Ulong		pindex();
double		pot();
double		pzustand();
double		rho_schnell();
double		rho_zufall();
Ulong		sum();
void		varkoef();
void		vout();
void		vzeitmom();
void		wschlmom();
void		wzeitmom();
struct Zustand*	zust_liste();


/* foutnl(z)

   dient zur einheitlichen Wertausgabe ueber den
   Ausgabekanal stdout.
   Jede Ausgabe wird mit 'newline' abgeschlossen.

*/

void foutnl(z)
double z;
{
	fprintf(stdout,"%f\n",(float)(z));
}

/* foutsp(z)

   dient zur einheitlichen Wertausgabe ueber den
   Ausgabekanal stdout.
   Jede Ausgabe wird mit 'space' abgeschlossen.

*/

void foutsp(z)
double z;
{
	fprintf(stdout,"%f ",(float)(z));
}

/* vout(y,z)

   gibt double-Vektor (y[]) mit (z) Komponeten mit foutsp() ueber 
   den Ausgabekanal stdout aus. 
   Die Vektorausgabe wird mit 'newline' abgeschlossen.

*/

void vout(y,z)
double y[];
int z;
{
	register f=0;

	while (f<z)
		foutsp(y[f++]);
	fprintf(stdout,"\n");
}

/* block(n)

   liefert Zeiger auf Speicherblock der Groesse (n) * sizeof(char);
   gibt Fehlermeldung aus,wenn nicht genuegend Speicher vorhanden ist.
   (Programm wird dann abgebrochen !)

*/

char *block(n)
Ulong n;
{
	register char *zg;

	if ((zg=(char *)malloc(n)) == (char *)NULL)
	{
		if (ctlmsg) fprintf(stderr, "block(): ERROR: Did NOT get %lu Byte(s)\n", n);
		oom=true;
		fprintf(stderr,"\n%s\n",spr("chk022"));
		fprintf(stderr,"%s %lu Byte(s)\n",spr("chk023"),MEM);
		Blockfree();
		printf("%s\n0 0 0 \n",spr("chk024"));
		exit(9);
	}
	if (ctlmsg) fprintf(stderr, "block(): got %lu Byte(s)\n", n);
	MEM+=n;
	return zg;
}

/* Index(n,k,v)

   liefert Feldindex fuer MWA()-Felder in Abhaengigkeit von:
   Knoten (n),Klasse (k),dekodiertem Populationsvektor (v).
   (sind (n) und/oder (k) negativ,so wird die entsprechende
   Dimension bei der Indexberechnung nicht beruecksichtigt!)

*/

Ulong Index(n,k,v)
int n,k;
Ulong v;
{
	register g=(k<0)?1:kl,h=(n<0)?1:kn;
	register Ulong i=0;

	if (k>0) i+=k;
	if (n>0) i+=g*n;
	if (v--) i+=g*h*v;
	return i;
}

/* pindex(n,l,v)

   liefert Feldindex fuer MWA()-bed.Wahrscheinlichkeiten-Feld
   in Abhaengigkeit von:
   Knoten (n),Auftraege (l),dekodiertem Populationsvektor (v).

*/

Ulong pindex(n,l,v)
int n,l;
Ulong v;
{
	return (m[n]*v+l);
}

/* dxindex(j,l,n,k,v)

   liefert Feldindex fuer dxMWA()-Felder in Abhaengigkeit von:
   Ableitung der MWA() nach mue-(Knoten (j),Klasse (l)),
   Knoten (n),Klasse (k),dekodiertem Populationsvektor (v).
   (sind (n) und/oder (k) negativ,so wird die entsprechende
   Dimension bei der Indexberechnung nicht beruecksichtigt!)

*/

Ulong dxindex(j,l,n,k,v)
int j,l,n,k;
Ulong v;
{
	register g=(k<0)?1:kl,h=(n<0)?1:kn;
	register Ulong i=0;

	if (k>0) i+=k;
	if (n>0) i+=g*n;
	if (l)   i+=h*g*l;
	if (j)   i+=kl*g*h*j;
	if (v--) i+=kl*kn*g*h*v;
	return i;
}

/* dxpindex(j,k,n,l,v)

   liefert Feldindex fuer dxMWA()-bed.Wahrscheinlichkeiten-Feld
   in Abhaengigkeit von:
   Ableitung der MWA() nach mue-(Knoten (j),Klasse (k)),
   Knoten (n),Auftraege (l),dekodiertem Populationsvektor (v).

*/

Ulong dxpindex(j,k,n,l,v)
int j,k,n,l;
Ulong v;
{
	register Ulong i= --v * m[n]+l,h=(MAXDECODE-1)*m[n];

	if (k) i+=k*h;
	if (j) i+=j*kl*h;
	return i;
}

/* pot(x,y)

   berechnet (x)^(y),
   (x) double,(y) positiv integer.

*/

double pot(x,y)
double x;
int y;
{
	double z=1.0;

	if (x==.0 || y==1) return x;
	if (x==1.0 || y==0) return 1.0;
	while (y>0)
	{
		if (y%2) z*=x;
		y/=2;
		x*=x;
	}
	return z;
}

/* fak(x)

   berechnet (x)-Fakultaet,
   (x) positiv integer.

*/

double fak(x)
int x;
{
	register f;
	double g=1.0;

	for(f=2;f<=x;g*=f,f++);
	return g;
}

/* binkoef(n,k)

   berechnet Binomialkoeffizienten (n) ueber (k),
   (n) und (k) positiv integer, 0 <= (k) <= (n).

*/

double binkoef(n,k)
int n,k;
{
	register g,f=(k<=n/2)?n-k+1:k+1;
	double w=(double)(f);

	if (k==0 || k==n) return 1.0;
	if (k==1 || k==n-1) return (double)(n);
	for(g=f+1;g<=n;w*=g,g++);
	if (k<=n/2) return (w/fak(k));
	return (w/fak(n-k));
}

/* anzvekt()

   berechnet die Anzahl der Vektoren,die zur Berechnung der
   Zustandswahrscheinlichkeiten erzeugt werden muessen.

*/

double anzvekt()
{
	register f,g=kn-1;
	double z=binkoef(g+anzkl[0],anzkl[0]);

	for(f=1;f<kl;f++)
		z*=binkoef(g+anzkl[f],anzkl[f]);
	return z;
}

/* decode(x)

   wandelt Populationsvektor (x[]) in eindeutige Ulong-Zahl,
   (x[]) positiv-integer-Populationsvektor.

*/

Ulong decode(x)
int x[];
{
	register c=0;
	register Ulong y=0;

	while (c<kl)
	{
		if (x[c]) y+=x[c]*codefakt[c];
		c++;
	}
	return y;
}

/* dekrdec(x,y)

   wandelt Populationsvektor (x[]-y[]) in eindeutige Ulong-Zahl,
   (x[]) und (y[]) positiv-integer-Populationsvektoren,
   0 <= (y[]) <= (x[]).

*/

Ulong dekrdec(x,y)
int x[],y[];
{
	register c=0,h;
	register Ulong w=0;

	while (c<kl)
	{
		if ((h=x[c]-y[c])) w+=h*codefakt[c];
		c++;
	}
	return w;
}

/* code(z,x)

   wandelt Ulong-Zahl (z) in entsprechenden Populationsvektor (x[]),
   (x[]) integer-Zielvektor.

*/

void code(z,x)
Ulong z;
int x[];
{
	register c=kl-1;

	while (c>0)
		if (codefakt[c]<=z)
		{
			x[c]=(int)(z/codefakt[c]);
			z%=codefakt[c--];
		}
		else x[c--]=0;
	x[0]=(int)(z);
}

/* nextzv(x)

   erzeugt naechste Population in (x[]) in Abhaengigkeit von der
   momentanen in (x[]) und liefert true (=erzeugt) oder false
   (=geht nicht) zurueck.
   (x[]) positiv-integer-Populationsvektor.

*/

bool nextzv(x)
int x[];
{
	register f=0;

	while (f<kl && x[f]==anzkl[f]) x[f++]=0;
	if (f==kl) return false;
	x[f]++;
	return true;
}

/* bednextzv(x)

   erzeugt naechste Population eines Knotens in (x[]) in Abhaengigkeit
   von der momentanen in (x[]) und in den anderen Knoten und liefert
   true (=erzeugt) oder false (=geht nicht) zurueck.
   (x[]) positiv-integer-Populationsvektor.

*/

bool bednextzv(x)
int x[];
{
	register f=0;

	while (f<kl && !hv[f])
	{
		hv[f]+=x[f];
		x[f++]=0;
	}
	if (f==kl) return false;
	hv[f]--;
	x[f]++;
	return true;
}

/* nextnzv()

   erzeugt naechste Population fuer alle Knoten in (nzv[]) in
   Abhaengigkeit von der momentanen in (nzv[]) und liefert true
   (=erzeugt) oder false (=geht nicht) zurueck.
   (Vor dem ersten Aufruf aktvekt und nzv[] auf Null, hv[]=anzkl[] setzen !)
 
*/

bool nextnzv()
{
	register g;
	register Ulong i,f;

	while (aktvekt<kn)
	{
		if (aktvekt>1)
			for(f=1;f<aktvekt;f++)
				for(g=0,i=f*kl;g<kl;hv[g]+=nzv[i+g],nzv[i+g]=0,g++);
		if (aktvekt)
			if (bednextzv(&nzv[aktvekt*kl])) aktvekt=0;
			else aktvekt++;
		if (!aktvekt)
		{
			for(g=0;g<kl;nzv[g]=hv[g],g++);
			aktvekt++;
			return true;
		}
	}
	return false;
}

/* sum(x)

   summiert Populationsvektor (x[]),
   (x[]) positiv-integer-Populationsvektor.

*/

Ulong sum(x)
int x[];
{
	register g=kl;
	register Ulong f=0;

	while (g>0) f+=x[--g];
	return f;
}

/* pzustand()

   berechnet die Zustandswahrscheinlichkeit fuer die in (nzv[])
   gegebene Population der Knoten.

*/

double pzustand()
{
	register f,g;
	register Ulong h,s;
	double p=1.0;

	for(f=0;f<kn;f++)
	{
		for(g=0;g<kl;g++)
		{
			h=Index(f,g,0);
			p*=pot(aus[h],nzv[h])/fak(nzv[h]);
		}
		s=sum(&nzv[f*kl]);
		if (m[f] && s>m[f])
			p*=fak(s)/pot((double)(m[f]),s-m[f])/fak(m[f]);
	}
	return (p/gg[MAXDECODE-1]);
}

/* aktprand(p)

   aktualisiert Randwahrscheinlichkeiten aus der Wahrscheinlichkeit (p)
   fuer die in (nzv[]) gegebene Population der Knoten.

*/

void aktprand(p)
double p;
{
	register f;

	for(f=0;f<kn;f++)
		prandbase[f*MAXDECODE+decode(&nzv[f*kl])]+=p;
}

/* WAHRSCH()

   steuert die Berechnung von Zustands- und Randwahrscheinlichkeiten.

*/

void WAHRSCH()
{
	register Ulong f;

	aktvekt=0;
	for(f=0;f<kn*kl;nzv[f]=0,f++);
	for(f=0;f<kl;hv[f]=anzkl[f],f++);
	while (nextnzv())
		aktprand(pzustand());
}

/* varkoef(ew,vr,z)

   Ausgabe der Variationskoeffizienten,wobei:
   (ew) Pointer auf Erwartungswerte,(vr) Pointer auf Varianzen,
   (z) Zeilengroesse der Felder (ew[]),(vr[]).

*/

void varkoef(ew,vr,z)
double ew[],vr[];
int z;
{
	register f,g;
	register Ulong i;

	for(f=0;f<z;f++)
	{
		for(g=0;g<kl;g++)
		{
			i=Index(f,g,0);
			if (ew[i])
				foutsp(sqrt(vr[i])/ew[i]);
			else
				if (vr[i]) foutsp(MAXFLOAT);
				else foutsp(.0);
		}
		fprintf(stdout,"\n");
	}
}

/* auftrmom(zentral,n,k,z)

   berechnet (k)tes Moment fuer Anzahl Kunden im Knoten (n),wobei:
   (zentral) <=> Flag fuer zentrale Momente,
   (z) Zeiger auf double-Ergebnisvektor.

*/

void auftrmom(zentral,n,k,z)
bool zentral;
int n,k;
double z[];
{
	register f;
	register double *g=prandbase+n*MAXDECODE;

	aktvekt=0;
	code(aktvekt,zv);
	for(f=0;f<kl;z[f]=.0,f++);
	do
	{
		for(f=0;f<kl;f++)
			if (zentral)
				z[f]+=pot(zv[f]-auftrm1[Index(n,f,0)],k)*g[aktvekt];
			else
				if (zv[f]) z[f]+=pot((double)(zv[f]),k)*g[aktvekt];
		aktvekt++;
	}while (nextzv(zv));
}

/* SMOMAUS()

   steuert Berechnung und Ausgabe der Strelen-Auftrags-Momente (dxMWA()).
   (1.Moment entspricht der mittl.Anz.Auftraege bei MWA()).

*/

void SMOMAUS()
{
	register g,h;
	register Ulong i,f=MAXDECODE-1;

	fprintf(stdout,"\n%s\n",spr("res007"));
	for(h=0;h<kn;h++)
	{
		fprintf(stdout,"\n %s %4d  |  %s  %s  %s\n",spr("std001"),h+1,spr("res008"),spr("res009"),spr("res010"));
		fprintf(stdout,"%s\n",spr("res011"));
		for(g=0;g<kl;g++)
		{
			fprintf(stdout," %s %4d  |  ",spr("std002"),g+1);

			i=Index(h,g,f);
			fprintf(stdout,"%f  ",ewka[i]*ewka[i]-(dxewka[dxindex(h,g,h,g,f)]*mue[Index(h,g,0)]));

			i=Index(h,g,0);
			sauftrzm2[i]= -dxewka[dxindex(h,g,h,g,f)]*mue[i];
			fprintf(stdout,"%f  ",sauftrzm2[i]);

			i=Index(0,0,f);
			if (ewka[h*kl+g+i])
				fprintf(stdout,"%f\n",sqrt(sauftrzm2[h*kl+g])/ewka[h*kl+g+i]);
			else
				if (sauftrzm2[h*kl+g]) fprintf(stdout,"%f\n",MAXFLOAT);
				else fprintf(stdout,".0\n");
		}
		fprintf(stdout,"\n");
	}
}

/* aktpwschl()

   berechnet mit Hilfe der Randwahrscheinlichkeiten die
   Wahrscheinlichkeiten fuer die Warteschlangenlaengen.

*/

void aktpwschl()
{
	register f,g,i,j=MAXAUFTR+1,k;
	register Ulong s;
	double p,h;

	aktvekt=0;
	code(aktvekt,zv);
	do
	{
		s=sum(zv);
		for(f=0;f<kn;f++)
			if (pwschlbase[f])
			{
				p=prandbase[f*MAXDECODE+aktvekt];
				for(g=0;g<kl;g++)
					if (s<=m[f])
						pwschlbase[f][j*g]+=p;
					else
						if (s==zv[g] && zv[g]>m[f])
							pwschlbase[f][j*g+zv[g]-m[f]]+=p;
						else
						{
							h=p/binkoef(s,zv[g]);
							for(i=MAX(0,zv[g]-m[f]);i<=(k=MIN(zv[g],s-m[f]));i++)
								pwschlbase[f][j*g+i]+=binkoef(m[f],zv[g]-i)*binkoef(s-m[f],
								    i)*h;
						}
			}
		aktvekt++;
	}while(nextzv(zv));
}

/* wschlmom(zentral,n,k,z)

   berechnet (k)tes Moment fuer Laenge der Warteschlange im Knoten (n),
   (zentral) <=> Flag fuer zentrale Momente,
   (z) Zeiger auf double-Ergebnisvektor.

*/

void wschlmom(zentral,n,k,z)
bool zentral;
int n,k;
double z[];
{
	register f;
	register Ulong g,i,j;

	for(f=0;f<kl;z[f]=.0,f++);
	if (pwschlbase[n])
		for(f=0;f<kl;f++)
		{
			g=(MAXAUFTR+1)*f;
			for(i=0;i<=(j=MIN(anzkl[f],GESAUFTR-m[n]));i++)
				if (zentral)
					z[f]+=pot(i-wschlm1[Index(n,f,0)],k)*pwschlbase[n][g+i];
				else
					if (i) z[f]+=pot((double)(i),k)*pwschlbase[n][g+i];
		}
}

/* MOMAUS1(fkt,ew,vr)

   Ausgabe von Momenten und Variationskoeffizienten fuer folgenden Fall:
   mit (*fkt)() koennen hoehere (auch zentrale) Momente berechnet werden,
   die Momente werden dabei im Erwartungswert-Feld (ew[]) und
   im Varianzen-Feld (vr[]) abgespeichert.

*/

void MOMAUS1(fkt,ew,vr)
void (*fkt)();
double *ew, *vr;
{
	register f,g;
	register double *h;

	for(f=0;f<kn;f++)
	{
		for(g=1;g<3;g++)
		{
			if (g==1) h= &ew[f*kl];
			else h=hdv;
			(*fkt)(false,f,g,h);
		}
		(*fkt)(true,f,2,&vr[f*kl]);

		fprintf(stdout,"\n %s %4d  |  %s  %s  %s  %s\n",spr("std001"),f+1,spr("res012"),spr("res008"),spr("res009"),spr("res010"));
		fprintf(stdout,"%s\n",spr("res013"));
		for(g=0;g<kl;g++)
		{
			fprintf(stdout," %s %4d  |  ",spr("std002"),g+1);
			fprintf(stdout,"%f  %f  ",ew[f*kl+g],hdv[g]);
			fprintf(stdout,"%f  ",vr[f*kl+g]);

			if (ew[f*kl+g])
				fprintf(stdout,"%f\n",sqrt(vr[f*kl+g])/ew[f*kl+g]);
			else
				if (vr[f*kl+g]) fprintf(stdout,"%f\n",MAXFLOAT);
				else fprintf(stdout,".0\n");
		}
		fprintf(stdout,"\n");
	}
}

/* wzeitmom(n,k,z)

   berechnet (k)tes Moment der Wartezeit fuer M/M/1-Knoten (n)
   approximativ.
   (z[]) Ergebnisvektor.

*/

void wzeitmom(n,k,z)
int n,k;
double z[];
{
	register f;
	register Ulong i;
	double h;

	if (m[n]==1 && typ[n]==1)
		for(f=0;f<kl;f++)
		{
			i=Index(n,f,0);
			if (!e[i]) z[f]=.0;
			else
			{
				h=(anzkl[f]-1.0)/anzkl[f];
				z[f]=fak(k)*rho[i]/pot((1.0-h*rhoges[n])*mue[i],k);
			}
		}
}

/* vzeitmom(n,k,z)

   berechnet (k)tes Moment der Verweilzeit fuer M/M/1-Knoten (n)
   approximativ.
   (z[]) Ergebnisvektor.

*/

void vzeitmom(n,k,z)
int n,k;
double z[];
{
	register f;
	register Ulong i;
	double h;

	if (m[n]==1 && typ[n]==1)
		for(f=0;f<kl;f++)
		{
			i=Index(n,f,0);
			if (!e[i]) z[f]=.0;
			else
			{
				h=(anzkl[f]-1.0)/anzkl[f];
				z[f]=fak(k)*pot(rho[i],k)/pot((1.0-h*rhoges[n])*lambda[i],k);
			}
		}
}

/* MOMAUS2(fkt,ew,vr)

   Ausgabe von Momenten und Variationskoeffizienten fuer folgenden Fall:
   mit (*fkt)() koennen hoehere (keine zentralen) Momente berechnet
   werden,die Varianz wird mit (2.Moment)-(1.Moment)^2 berechnet,
   die Momente werden dabei im Erwartungswert-Feld (ew[][]) und
   im Varianzen-Feld (vr[][]) abgespeichert.

   (diese Funktion bearbeitet nur M/M/1-Knoten!)

*/

void MOMAUS2(fkt,ew,vr)
void (*fkt)();
pointer ew[],vr[];
{
	register f,g;
	register double *h;

	for(f=0;f<kn;f++)
	{
		if (m[f]==1 && typ[f]==1)
		{
			for(g=1;g<3;g++)
			{
				if (g==1) h=ew[f];
				else h=hdv;
				(*fkt)(f,g,h);
			}
			for(g=0;g<kl;g++)
				vr[f][g]=h[g]-ew[f][g]*ew[f][g];

			fprintf(stdout,"\n %s %4d  |  %s  %s  %s  %s\n",spr("std001"),f+1,spr("res012"),spr("res008"),spr("res009"),spr("res010"));
			fprintf(stdout,"%s\n",spr("res013"));
			for(g=0;g<kl;g++)
			{
				fprintf(stdout," %s %4d  |  ",spr("std002"),g+1);
				fprintf(stdout,"%f  %f  ",ew[f][g],hdv[g]);
				fprintf(stdout,"%f  ",vr[f][g]);

				if (ew[f][g])
					fprintf(stdout,"%f\n",sqrt(vr[f][g])/ew[f][g]);
				else
					if (vr[f][g]) fprintf(stdout,"%f\n",MAXFLOAT);
					else fprintf(stdout,".0\n");
			}
			fprintf(stdout,"\n");
		}
	}
}

/* dekr(z,k)

   liefert Decodierung des in Klasse (k) dekrementierten
   Populationsvektors (z[]), (z[k]) > 0 ;
   ((z[k]) ist hinterher unveraendert).

*/

Ulong dekr(z,k)
int z[],k;
{
	register Ulong f;

	z[k]--;
	f=decode(z);
	z[k]++;
	return f;
}

/* ANTWZEIT(n,k)

   berechnet die MWA()-Antwortzeit fuer Knoten (n),Klasse (k)
   und die aktuelle Population (geg.in aktvekt).

*/

double ANTWZEIT(n,k)
int n,k;
{
	register l;
	register Ulong f;
	double h=1.0;

	if (m[n])
	{
		if ((f=dekr(zv,k))) h+=ka1[Index(n,-1,f)];
		if (m[n]>1)
		{
			for(l=0;l<m[n]-1;l++)
				h+=(m[n]-l-1)*pe[n][pindex(n,l,f)];
			h/=m[n]; /* ASYM bereits bei Berechnung von (aus[]) beruecksichtigt */
		}
	}
	return (aus[Index(n,k,0)]*h);
}

/* ANTWZEIT2(n,k)

   berechnet die MWA2()-Antwortzeit fuer Knoten (n),Klasse (k)
   und die aktuelle Population (geg.in aktvekt).

*/

double ANTWZEIT2(n,k)
int n,k;
{
	register l;
	register Ulong i,j=Index(n,k,0);
	double h=.0,s=.0;

	if (!m[n]) return aus[j];
	if ((i=dekr(zv,k)))
		if (typ[n]==2) return (aus[j]*(1.0+ka1[Index(n,-1,i)]));
		else
			for(l=0;l<kl;l++)
				h+=ewka[Index(n,l,i)]/mue[Index(n,l,0)];
	h=aus[j]+e[j]*h;
	if (m[n]>1)
	{
		for(l=0;l<m[n]-1;l++)
			s+=(m[n]-l-1)*pe[n][pindex(n,l,i)];
		h=(e[j]*norm[n][aktvekt-1]*s+h)/m[n];
	}
	return h;
}

/* AKTpe()

   berechnet die fuer MWA() noetigen bedingten Wahrscheinlichkeiten
   fuer aktuelle Population (geg.in aktvekt).

*/

void AKTpe()
{
   register n,r,l,x;
   register Ulong i;
   double s,f,h,hilf;

	for (n=0;n<kn;n++)
	if (pe[n])
	{
		h=f=.0;
		for (l=m[n]-1;l>0;l--)
		{
			s=.0;
			for (r=0;r<kl;r++)
			if (zv[r])
				if (typ[n]==20)
				{
					i=Index(n,r,0);
					switch(opt&112)
					{
						case 16 : /* Scheuerer */
		        		         s+=e[i]*lam[Index(-1,r,aktvekt)] 
					  					*pe[n][pindex(n,l-1,dekr(zv,r))];
		                 		if (l==1) f+=aus[i]*lam[Index(-1,r,aktvekt)];
				     				break;

						case 32 : /* Baer/Fischer/Hertel, zufaellige Bedieneinheit */
			        			   s+=rho_zufall(n,lam[Index(-1,r,aktvekt)])
			              			*pe[n][pindex(n,l-1,dekr(zv,r))];
		                 		if (l==1) f+=aus[i]*lam[Index(-1,r,aktvekt)];
				     				break;

						case 64 : /* Baer/Fischer/Hertel, schnellste Bedieneinheit */
			        			   s+=rho_schnell(n,lam[Index(-1,r,aktvekt)])
			              			*pe[n][pindex(n,l-1,dekr(zv,r))];
		                 			if (l==1) f+=aus[i]*lam[Index(-1,r,aktvekt)];
				     				break;

						default  : /* falsche Option fuer asymmetrische Knoten */     
					   		   fprintf(stdout,"%s\n",spr("chk050"));
					}
					/*s+=e[i]*lam[Index(-1,r,aktvekt)]*pe[n][pindex(n,l-1,dekr(zv,r))];*/
				}
				else
				{
			   	i=Index(n,r,0);
			   	s+=aus[i]*lam[Index(-1,r,aktvekt)]*pe[n][pindex(n,l-1,dekr(zv,r))];
			   	if (l==1) f+=aus[i]*lam[Index(-1,r,aktvekt)];
				}

				/* fprintf(stdout,"l ist = %d  ",l);
				  fprintf(stdout,"s ist = %lf  ",s);    */

 			if ((typ[n]==20) && (opt&16))
			{
				hilf=0;
				for (x=0;x<l;x++) hilf+=amue[n][m[n]-1-x];
				s/=hilf;
			}
			else
				if (typ[n] != 20) s/=l;

			/* fprintf(stdout,"s/l(hilf) ist = %lf\n",s); */

			h+=(m[n]-l)*s;
			pe[n][pindex(n,l,aktvekt)]=s;

			/* fprintf(stdout,"x1x pe[%d][%d] ist %lf\n",n,pindex(n,l,aktvekt),pe[n][pindex(n,l,aktvekt)]);*/

		}  
		pe[n][pindex(n,0,aktvekt)]=1.0-(f+h)/m[n];

		/* fprintf(stdout,"x2x pe[%d][%d] ist %lf\n",n,pindex(n,0,aktvekt),pe[n][pindex(n,0,aktvekt)]);*/

	}  
}


/* all_ws(i,lamb)

   berechnet die Zustandswahrscheinlichkeiten fuer Knoten (i) in
   Abhaengigkeit des aktuellen lambdas

*/

struct Zustand * all_ws(i,lamb)
int i;
double lamb;
{
        int r;
        double  P_0 = 0;
        struct Zustand *first, *ptr, *ini_ptr; 

     /* Berechnung der Leerwahrscheinlichkeit */
     /* ------------------------------------- */
        for (r = 1; r <= m[i]; r ++)
           P_0 = P_0 + (fak(m[i]-r) * mul_rho(i,r,lamb));
        P_0 = (P_0 / fak(m[i])) + 1;
        P_0 = 1 / P_0;

     /* Berechnung der uebrigen Wahrscheinlichkeiten */
     /* -------------------------------------------- */
        first = zust_liste(i);  
        ini_ptr = first;
        for (r = 1; r < m[i]; r++) 
        {
           ptr = ini_liste(ini_ptr,r,i);
           ini_ptr = ptr;
        }
        ptr = first;
        while (ptr != NULL)
        {
           ptr->value = (fak(m[i]-(ptr->anz))/fak(m[i])) * P_0;
           for (r = 0; r < m[i]; r++)
              if (ptr->belegt[r] == 1) 
                 ptr->value = ptr->value * (lamb / amue[i][r]);
           ptr = ptr->next;
        }
	  return(first);
}



/* zust_liste(i) 

   Erstellung einer Liste von Zustaenden des MARKOV-Graphen

*/

struct Zustand* zust_liste(i)
int i;                         
{
   int r,j;
   int anz;                    
   struct Zustand* first;     
   struct Zustand* ptr;      

   /* Anzahl Listenelemente feststellen */
   /* ----------------------------------*/
      anz = 2;
      for (r = 0; r < (m[i]-1); r++)
         anz = anz*2;

   /* Erstes Element erzeugen und initialisieren */
   /* ------------------------------------------ */
      first = (struct Zustand *) malloc (sizeof(struct Zustand));
      for (r = 0; r < m[i]; r++) first->belegt[r] = 0;
      first->anz   = 0;
      first->value = 1;
      first->next = NULL;
      ptr = first;
   
   /* weitere Elemente erzeugen und initialisieren */
   /* -------------------------------------------- */
      for (r = 0; r < (anz-1); r++)
      {
         ptr->next = (struct Zustand *) malloc (sizeof(struct Zustand));
         ptr = ptr->next;
         for (j = 0; j < m[i]; j++) ptr->belegt[j] = 0;
         ptr->anz   = 0;
         ptr->value = 1;
         ptr->next = NULL;
      }

   /* Ruecksprung */
   /* ----------- */
      return(first);
}



/* ini_liste(ini_ptr,anz,i)

   Initialisierung der Zustandsliste

*/
        
struct Zustand* ini_liste(ptr,anz,i) 
struct Zustand* ptr;         
int anz;             
int i;                        
{
   int    j[11];

   switch(anz)
   {
      case 1:
      {
         for (j[1] = 0; j[1] < m[i]; j[1]++)
         {
            ptr->anz = 1;
            ptr->belegt[j[1]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 2:
      {
         for (j[1] = 0;     j[1] < (m[i]-1); j[1]++)
         for (j[2] = j[1]+1;j[2] <  m[i]   ; j[2]++)
         {
            ptr->anz = 2;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 3:
      {
         for (j[1] = 0;      j[1] < (m[i]-2); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-1); j[2]++)
         for (j[3] = j[2]+1; j[3] <  m[i]   ; j[3]++)
         {
            ptr->anz = 3;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 4:
      {
         for (j[1] = 0;      j[1] < (m[i]-3); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-2); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-1); j[3]++)
         for (j[4] = j[3]+1; j[4] <  m[i]   ; j[4]++)
         {
            ptr->anz = 4;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 5:
      {
         for (j[1] = 0;      j[1] < (m[i]-4); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-3); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-2); j[3]++)
         for (j[4] = j[3]+1; j[4] < (m[i]-1); j[4]++)
         for (j[5] = j[4]+1; j[5] <  m[i]   ; j[5]++)
         {
            ptr->anz = 5;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr->belegt[j[5]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 6:
      {
         for (j[1] = 0;      j[1] < (m[i]-5); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-4); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-3); j[3]++)
         for (j[4] = j[3]+1; j[4] < (m[i]-2); j[4]++)
         for (j[5] = j[4]+1; j[5] < (m[i]-1); j[5]++)
         for (j[6] = j[5]+1; j[6] <  m[i]   ; j[6]++)
         {
            ptr->anz = 6;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr->belegt[j[5]] = 1; ptr->belegt[j[6]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 7:
      {
         for (j[1] = 0;      j[1] < (m[i]-6); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-5); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-4); j[3]++)
         for (j[4] = j[3]+1; j[4] < (m[i]-3); j[4]++)
         for (j[5] = j[4]+1; j[5] < (m[i]-2); j[5]++)
         for (j[6] = j[5]+1; j[6] < (m[i]-1); j[6]++)
         for (j[7] = j[6]+1; j[7] <  m[i]   ; j[7]++)
         {
            ptr->anz = 7;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr->belegt[j[5]] = 1; ptr->belegt[j[6]] = 1;
            ptr->belegt[j[7]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 8:
      {
         for (j[1] = 0;      j[1] < (m[i]-7); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-6); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-5); j[3]++)
         for (j[4] = j[3]+1; j[4] < (m[i]-4); j[4]++)
         for (j[5] = j[4]+1; j[5] < (m[i]-3); j[5]++)
         for (j[6] = j[5]+1; j[6] < (m[i]-2); j[6]++)
         for (j[7] = j[6]+1; j[7] < (m[i]-1); j[7]++)
         for (j[8] = j[7]+1; j[8] <  m[i]   ; j[8]++)
         {
            ptr->anz = 8;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr->belegt[j[5]] = 1; ptr->belegt[j[6]] = 1;
            ptr->belegt[j[7]] = 1; ptr->belegt[j[8]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 9:
      {
         for (j[1] = 0;      j[1] < (m[i]-8); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-7); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-6); j[3]++)
         for (j[4] = j[3]+1; j[4] < (m[i]-5); j[4]++)
         for (j[5] = j[4]+1; j[5] < (m[i]-4); j[5]++)
         for (j[6] = j[5]+1; j[6] < (m[i]-3); j[6]++)
         for (j[7] = j[6]+1; j[7] < (m[i]-2); j[7]++)
         for (j[8] = j[7]+1; j[8] < (m[i]-1); j[8]++)
         for (j[9] = j[8]+1; j[9] <  m[i]   ; j[9]++)
         {
            ptr->anz = 9;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr->belegt[j[5]] = 1; ptr->belegt[j[6]] = 1;
            ptr->belegt[j[7]] = 1; ptr->belegt[j[8]] = 1;
            ptr->belegt[j[9]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
      case 10:
      {
         for (j[1] = 0;      j[1] < (m[i]-9); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-8); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-7); j[3]++)
         for (j[4] = j[3]+1; j[4] < (m[i]-6); j[4]++)
         for (j[5] = j[4]+1; j[5] < (m[i]-5); j[5]++)
         for (j[6] = j[5]+1; j[6] < (m[i]-4); j[6]++)
         for (j[7] = j[6]+1; j[7] < (m[i]-3); j[7]++)
         for (j[8] = j[7]+1; j[8] < (m[i]-2); j[8]++)
         for (j[9] = j[8]+1; j[9] < (m[i]-1); j[9]++)
         for (j[10] = j[9]+1; j[10] < m[i]    ; j[10]++)
         {
            ptr->anz = 10;
            ptr->belegt[j[1]] = 1; ptr->belegt[j[2]] = 1;
            ptr->belegt[j[3]] = 1; ptr->belegt[j[4]] = 1;
            ptr->belegt[j[5]] = 1; ptr->belegt[j[6]] = 1;
            ptr->belegt[j[7]] = 1; ptr->belegt[j[8]] = 1;
            ptr->belegt[j[9]] = 1; ptr->belegt[j[10]] = 1;
            ptr = ptr->next;
         } 
         return(ptr);
      }
	default: fprintf(stderr,"ini_liste(): (anz)-Error\n");
   }
   return(NULL);
}



/* mul_rho(i,anz,lamb)

   Hilfsfunktion fuer all_ws

*/

double mul_rho(i,anz,lamb)
int i,anz;
double lamb;
{
   double rho[10];
   double val = 0;
   int r;                       
   int j[10];

 
   for (r = 0; r < m[i]; r++)
       rho[r] = lamb / amue[i][r];

   switch(anz)
   {
      case 1:
      {
         for (j[1] = 0; j[1] < m[i]; j[1]++)
         val = val + rho[j[1]];
         return(val);
      }
      case 2:
      {
         for (j[1] = 0;     j[1] < (m[i]-1); j[1]++)
         for (j[2] = j[1]+1;j[2] <  m[i]   ; j[2]++)
         val = val + rho[j[1]]*rho[j[2]];
         return(val);
      }
      case 3:
      {
         for (j[1] = 0;      j[1] < (m[i]-2); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-1); j[2]++)
         for (j[3] = j[2]+1; j[3] <  m[i]   ; j[3]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]];
         return(val);
      }
      case 4:
      {
         for (j[1] = 0;      j[1] < (m[i]-3); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-2); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-1); j[3]++)
         for (j[4] = j[3]+1; j[4] <  m[i]   ; j[4]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]];
         return(val);
      }
      case 5:
      {
         for (j[1] = 0;      j[1] < (m[i]-4); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-3); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-2); j[3]++)
         for (j[4] = j[3]+1; j[4] < (m[i]-1); j[4]++)
         for (j[5] = j[4]+1; j[5] <  m[i]   ; j[5]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]]*rho[j[5]];
         return(val);
      }
      case 6:
      {
         for (j[1] = 0;      j[1] < (m[i]-5); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-4); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-3); j[3]++)
         for (j[4] = j[3]+1; j[4] < (m[i]-2); j[4]++)
         for (j[5] = j[4]+1; j[5] < (m[i]-1); j[5]++)
         for (j[6] = j[5]+1; j[6] <  m[i]   ; j[6]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]]*rho[j[5]]
                    *rho[j[6]];
         return(val);
      }
      case 7:
      {
         for (j[1] = 0;      j[1] < (m[i]-6); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-5); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-4); j[3]++)
         for (j[4] = j[3]+1; j[4] < (m[i]-3); j[4]++)
         for (j[5] = j[4]+1; j[5] < (m[i]-2); j[5]++)
         for (j[6] = j[5]+1; j[6] < (m[i]-1); j[6]++)
         for (j[7] = j[6]+1; j[7] <  m[i]   ; j[7]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]]*rho[j[5]]
                    *rho[j[6]]*rho[j[7]];
         return(val);
      }
      case 8:
      {
         for (j[1] = 0;      j[1] < (m[i]-7); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-6); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-5); j[3]++)
         for (j[4] = j[3]+1; j[4] < (m[i]-4); j[4]++)
         for (j[5] = j[4]+1; j[5] < (m[i]-3); j[5]++)
         for (j[6] = j[5]+1; j[6] < (m[i]-2); j[6]++)
         for (j[7] = j[6]+1; j[7] < (m[i]-1); j[7]++)
         for (j[8] = j[7]+1; j[8] <  m[i]   ; j[8]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]]*rho[j[5]]
                    *rho[j[6]]*rho[j[7]]*rho[j[8]];
         return(val);
      }
      case 9:
      {
         for (j[1] = 0;      j[1] < (m[i]-8); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-7); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-6); j[3]++)
         for (j[4] = j[3]+1; j[4] < (m[i]-5); j[4]++)
         for (j[5] = j[4]+1; j[5] < (m[i]-4); j[5]++)
         for (j[6] = j[5]+1; j[6] < (m[i]-3); j[6]++)
         for (j[7] = j[6]+1; j[7] < (m[i]-2); j[7]++)
         for (j[8] = j[7]+1; j[8] < (m[i]-1); j[8]++)
         for (j[9] = j[8]+1; j[9] <  m[i]   ; j[9]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]]*rho[j[5]]
                    *rho[j[6]]*rho[j[7]]*rho[j[8]]*rho[j[9]];
         return(val);
      }
      case 10:
      {
         for (j[1] = 0;      j[1] < (m[i]-9); j[1]++)
         for (j[2] = j[1]+1; j[2] < (m[i]-8); j[2]++)
         for (j[3] = j[2]+1; j[3] < (m[i]-7); j[3]++)
         for (j[4] = j[3]+1; j[4] < (m[i]-6); j[4]++)
         for (j[5] = j[4]+1; j[5] < (m[i]-5); j[5]++)
         for (j[6] = j[5]+1; j[6] < (m[i]-4); j[6]++)
         for (j[7] = j[6]+1; j[7] < (m[i]-3); j[7]++)
         for (j[8] = j[7]+1; j[8] < (m[i]-2); j[8]++)
         for (j[9] = j[8]+1; j[9] < (m[i]-1); j[9]++)
         for (j[10] = j[9]+1;j[10] < m[i]   ; j[10]++)
         val = val + rho[j[1]]*rho[j[2]]*rho[j[3]]*rho[j[4]]*rho[j[5]]
                    *rho[j[6]]*rho[j[7]]*rho[j[8]]*rho[j[9]]*rho[j[10]];
         return(val);
      }
      default: fprintf(stderr,"mul_rho(): (anz)-Error\n");
   }
   return(0.0);
}



/* rho_zufall(n,lamb)

   berechnet die Auslastung des M/M/m-Knotens (n) in Abhaengigkeit
   der aktuellen Ankunftsrate lamb nach Baer/Fischer/Hertel bei
   zufaelliger Auswahl einer freien Bedieneinheit

*/

double rho_zufall (n,lamb)
int n;
double lamb;
{
   double ce,N,h;
   double eta[10];
   int r;
   struct Zustand *ptr1, *ptr2;

   if (lamb > null)
   {
     /* erster Schritt: kanalspez. Auslastungen Verlustsystem */
     /* ----------------------------------------------------- */
        ptr1 = ptr2 = all_ws(n,lamb);
        for (r = 0; r < 10; r++) eta[r] = 0;
        while (ptr1 != NULL)
        {
           for (r = 0; r < m[n]; r++)
              if (ptr1->belegt[r] == 1) 
	            eta[r] = eta[r] + ptr1->value;
           ptr1 = ptr1->next;
        }
        while (ptr2->next != NULL) ptr2 = ptr2->next;
      
     /* zweiter Schritt: kanalspez. Auslastungen Wartesystem */
     /* ---------------------------------------------------- */
        for (r = 0; r < m[n]; r++) 
           ce = ce + (amue[n][r]/lamb);
        ce = 1 / ce;

        N = 1 + (ptr2->value * ce / (1 - ce));

        for (r = 0; r < m[n]; r++)
           eta[r] = (eta[r] + (ptr2->value * ce / (1 - ce)))/ N;

     /* dritter Schritt: Gesamtauslastung */
     /* --------------------------------- */
        for (r = 0; r < m[n]; r++)
           h = h + (1 / eta[r]);
        h = m[n] * (1 / h);       

     return h;
    }
    else return(MINRHO);
}  


/* Verlust(anz,bedien,lam)

   Berechnung der Verlustwahrscheinlichkeit fuer bestimmte Anzahl von
   Bedieneinheiten mit zugehoerigen Bedienraten und Durchsatz lam
   nach Baer/Fischer/Hertel bei Auswahl der schnellsten Bedieneinheit

*/
double Verlust(anz,bedien,lam)
   int anz;
   double bedien[];
   double lam;
{
   double value = 1;                       /* Rueckgabewert                  */
   double bed_zaehler[10], bed_nenner[10]; /* Argumente fuer rekursiven Auf- */
                                           /* ruf der Funktion               */
   int i,k;                                /* Schleifenindices               */

   if (anz == 0)
	return(1);
   
   else if (anz == 1)
   {
	value = lam/(lam + bedien[0]);
	return(value);
   }

   else
   {
	for (k = 1; k < anz; k++)
	{
	   /* Feld fuer Bedienraten im Zaehler initialisieren */
	   /* =============================================== */
		for (i = 0; i < k-1; i++)
		   bed_zaehler[i] = bedien[i];
		bed_zaehler[k-1] = bedien[anz-1];


	   /* Feld fuer Bedienraten im Nenner initialisieren */
	   /* ============================================== */
		for (i = 0; i < k-1; i++)
		   bed_nenner[i] = bedien[i];
		bed_nenner[k-1] = bedien[k-1] + bedien[anz-1];

	   value = value*(Verlust(k,bed_zaehler,lam)/Verlust(k,bed_nenner,lam));
	}

	value = 1 + (value * bedien[anz-1] / lam);
	value = 1 / value;
	value = value * Verlust(anz-1,bedien,lam);

	return(value);
   }
}



/* rho_schnell(n,lamb)

   berechnet die Auslastung des M/M/m-Knotens (n) in Abhaengigkeit
   der aktuellen Ankunftsrate lamb nach Baer/Fischer/Hertel bei 
   Auswahl der schnellsten Bedieneinheit

*/

double rho_schnell (n,lamb)
int n;
double lamb;
{
   double P_v;
   double ce = 0,N = 0,h = 0;
   double eta[10];
   int r;

   if (lamb > null)
   {
     /* erster Schritt: kanalspez. Auslastungen Verlustsystem */
     /* ----------------------------------------------------- */
	  for (r = 0; r < m[n]; r++)
	     eta[r] = (lamb / amue_geord[n][r]) * (Verlust(r,amue_geord[n],lamb)
			- Verlust(r+1,amue_geord[n],lamb));
      
     /* zweiter Schritt: kanalspez. Auslastungen Wartesystem */
     /* ---------------------------------------------------- */
	  ce = 0;
	  for (r = 0; r < m[n]; r++)
	  {
	     ce = ce + (1 / (lamb/amue_geord[n][r]));
        }	
	  ce = 1 / ce;
	  P_v = Verlust(m[n],amue_geord[n],lamb);
        N = 1 + (P_v * ce / (1 - ce));
	  for (r = 0; r < m[n]; r++)
	  {
	      eta[r] = (eta[r] + (P_v * ce / (1 - ce))) / N;
	  }


     /* dritter Schritt: Gesamtauslastung */
     /* --------------------------------- */
        for (r = 0; r < m[n]; r++)
           h = h + (1 / eta[r]);
        h = m[n] * (1 / h);       

     return h;
    }
    else return(MINRHO);
}  


/* MWA()

   Mittelwertanalyse.

*/

void MWA()
{
	register f,g;
	register Ulong i;
	double h,s;

	aktvekt=1;
	code(aktvekt,zv);
	do
	{
		for(f=0;f<kl;f++)
			if (zv[f])
			{
				s=.0;
				for(g=0;g<kn;g++)
				{
					antwzeit[Index(g,f,aktvekt)]=h=ANTWZEIT(g,f);
					s+=h;
				}
				lam[Index(-1,f,aktvekt)]=s=zv[f]/s;
				for(g=0;g<kn;g++)
				{
					i=Index(g,f,aktvekt);
					ewka[i]=h=s*antwzeit[i];
					ka1[Index(g,-1,aktvekt)]+=h;
				}
			}
		AKTpe();
		aktvekt++;
	}while(nextzv(zv));
}

/* MWA2()

   Mittelwertanalyse fuer Netze,in denen M/M/m-Knoten mit
   unterschiedlichen Klassen-Bedienraten vorkommen.

*/

void MWA2()
{
	register f,g;
	register Ulong i;
	double h,s,t;

	aktvekt=1;
	code(aktvekt,zv);
	do
	{
		if (mts)
		{
			for(f=0;f<kl;f++)
				if (zv[f])
				{
					s=.0;
					for(g=0;g<kn;g++)
					{
						antwzeit[Index(g,f,aktvekt)]=h=ANTWZEIT(g,f);
						s+=h;
					}
					lam[Index(-1,f,aktvekt)]=zv[f]/s;
				}
			for(f=0;f<kn;f++)
				if (m[f]>1)
				{
					h=t=.0;
					for(g=0;g<kl;g++)
					{
						i=Index(f,g,0);
						s=lam[Index(-1,g,aktvekt)]*e[i];
						h+=s/mue[i];
						t+=s;
					}
					if (t) norm[f][aktvekt-1]=h/t;
					else 
						if (h) norm[f][aktvekt-1]=MAXDOUBLE;
						else norm[f][aktvekt-1]=.0;
				}
		}
		for(f=0;f<kl;f++)
			if (zv[f])
			{
				s=.0;
				for(g=0;g<kn;g++)
				{
					antwzeit[Index(g,f,aktvekt)]=h=ANTWZEIT2(g,f);
					s+=h;
				}
				lam[Index(-1,f,aktvekt)]=s=zv[f]/s;
				for(g=0;g<kn;g++)
				{
					i=Index(g,f,aktvekt);
					ewka[i]=h=s*antwzeit[i];
					ka1[Index(g,-1,aktvekt)]+=h;
				}
			}
		AKTpe();
		aktvekt++;
	}while(nextzv(zv));
}

/* NORM()

   Berechnung der Normalisierungskonstanten. 

*/

void NORM()
{
	register f;

	gg[0]=1.0;
	aktvekt=1;
	code(aktvekt,zv);
	do
	{
		f=0;
		while (!zv[f]) f++;
		gg[aktvekt]=gg[dekr(zv,f)]/lam[Index(-1,f,aktvekt)];
		aktvekt++;
	}while(nextzv(zv));
}

/* RHO()

   berechnet die Auslastung fuer Multiple-Server-Knoten bei mehreren 
   Auftragsklassen.

*/

void RHO()
{
	register i;
	register Ulong g,s,k,c;
	register int *v=anzkl;
	double h,r,t,u;

	if (ctlmsg) fprintf(stderr,"RHO()\n");
	for(i=0;i<kn;i++)
		if (m[i]>1)
		{
			aktvekt=1;
			code(aktvekt,zv);
			ff[0]=1.0;
			c=i*kl;
			do
			{
				s=sum(zv);
				h=r=1.0;
				for(g=0;g<kl;h*=pot(e[c+g],zv[g])/fak(zv[g]),g++);
				for(g=1;g<=s;g++)
					if (g<=m[i]) r*=g*muequer[i];
					else r*=m[i]*muequer[i];
				h*=fak(s)/r;
				ff[aktvekt]=h;
				aktvekt++;
			}while(nextzv(zv));
			aktvekt=1;
			code(aktvekt,zv);
			gN[0]=1.0;
			do
			{
				t=.0;
				for(g=0;g<kl;g++)
					if (zv[g])
					{
						u=.0;
						c=dekr(zv,g);
						code(c,hv);
						code(0,nzv);
						anzkl=hv;
						do
						{
							k=sum(nzv);
							if (k<=m[i]-2)
								u+=(m[i]-k-1.0)/(k+1.0)*ff[decode(nzv)]*gN[dekrdec(hv,nzv)];
						}while(nextzv(nzv));
						anzkl=v;
						t+=aus[Index(i,g,0)]*(gg[c]+u);
					}
				u=gg[aktvekt]-t/m[i];
				gN[aktvekt]=u;
				aktvekt++;
			}while(nextzv(zv));
			aktvekt=0;
			code(aktvekt,zv);
			do
			{
				u=ff[aktvekt]/gg[MAXDECODE-1]*gN[dekrdec(anzkl,zv)];
				if (!(opt & 8))
					if (u<.0 || u>1.0) u=.0;
				pI[aktvekt]=u;
				aktvekt++;
			}while(nextzv(zv));
			for(g=0;g<kl;g++)
			{
				t=.0;
				aktvekt=codefakt[g];
				code(aktvekt,zv);
				do
				{
					if (zv[g]) t+=pI[aktvekt]*zv[g]/sum(zv)*(m[i] < sum(zv) ? m[i] : sum(zv));
					aktvekt++;
				}while(nextzv(zv));
				rho[Index(i,g,0)]=t / m[i];
			}
		}
}

/* LEIST(vn)

   Berechnung und Ausgabe der MWA-Leistungsgroessen,
   (vn) Pointer auf Verfahrensnamen (String).

*/

void LEIST(vn)
char vn[];
{
	register f=1,g;
	register Ulong i;
	register char *t=vn;
	double j,l;

	do                          /* Pfadnamen aus (vn[]) ausfiltern */
	{
		vn=t++;
		while(*t != '/' && *t != '\0') t++;
	}while(*t);
	if (*vn == '/') vn++;
	fprintf(stdout,"%s\n",vn);  /* Name des Verfahrens */
	fprintf(stdout,"%d ",f);    /* Netztyp: geschlossen */
	fprintf(stdout,"%d ",kn);
	fprintf(stdout,"%d\n",kl);
	/*
	if (kl>1 && mts) RHO();
	*/
	for(f=0;f<kl;f++)
	{
		j=gg[dekr(anzkl,f)]/gg[MAXDECODE-1];
		for(g=0;g<kn;g++)
		{
			i=Index(g,f,0);
			lambda[i]=e[i]*j;
			foutsp(lambda[i]);
			foutsp(e[i]);
			foutsp(1.0/mue[i]);
			if (!m[g] || !e[i]) rho[i]=.0;
			/*
			else if (m[g]==1 || kl==1) rho[i]=lambda[i]/mue[i]/m[g];
			*/
			else rho[i]=lambda[i]/mue[i]/m[g];
			if (rho[i] > 1.0) rho[i]=1.0;
			foutsp(rho[i]);
			rhoges[g]+=rho[i];
			if (!e[i]) l=.0;
			else if (!m[g]) l=1.0/mue[i];
			else l=ewka[Index(g,f,MAXDECODE-1)]/lambda[i];
			foutsp(l);
			foutsp(ewka[Index(g,f,MAXDECODE-1)]);
			if (!m[g] || !e[i]) l=.0;
			else l-=1.0/mue[i];
			if(l < 0.0) l = 0.0;
			foutsp(l);
			foutnl(l*lambda[i]);
		}
	}
}

/* dxANTWZEIT(j,l,n,k)

   berechnet die dxMWA()-Antwortzeit fuer Knoten (n) und Klasse (k)
   fuer aktuelle Population (geg.in aktvekt) und Ableitung nach
   mue-(Knoten (j),Klasse (l)).

*/

double dxANTWZEIT(j,l,n,k)
int j,l,n,k;
{
	bool f=(j==n && l==k)?true:false;
	register r;
	register Ulong i=Index(n,k,0),g=dekr(zv,k);
	double h,s=.0;

	if (!m[n] && f) s= -1.0;
	if (m[n])
	{
		if (f)
		{
			if (g)
				s-=ka1[Index(n,-1,g)];
			if (m[n]>1)
				for(r=0;r<m[n]-1;r++)
					s-=(m[n]-r-1)*pe[n][pindex(n,r,g)];
			s-=1.0;
		}
		if (g)
		{
			h=dxka1[dxindex(j,l,n,-1,g)];
			if (m[n]>1)
				for(r=0;r<m[n]-1;r++)
					h+=(m[n]-r-1)*dxpe[n][dxpindex(j,l,n,r,g)];
			s+=h*mue[i];
		}
		if (m[n]>1) s/=m[n];
	}
	return (aus[i]*s/mue[i]);
}

/* AKTdxpe()

   berechnet die fuer die dxMWA() noetigen bedingten Wahrscheinlichkeiten
   fuer die aktuelle Population (geg.in aktvekt).

*/

void AKTdxpe()
{
	register j,z,n,r,l;
	register Ulong i,g;
	double s,h,f,k;

	for(j=0;j<kn;j++)
		for(z=0;z<kl;z++)
			for(n=0;n<kn;n++)
				if (dxpe[n])
				{
					h=.0;
					i=Index(j,z,0);
					if (n==j && zv[z])
						k= -aus[i]/mue[i]*lam[Index(-1,z,aktvekt)];
					else
						k=.0;
					for(l=m[n]-1;l>0;l--)
					{
						if (n==j && zv[z])
							s=k*pe[n][pindex(j,l-1,dekr(zv,z))];
						else
							s=.0;
						for(r=0;r<kl;r++)
							if (zv[r])
							{
								i=Index(n,r,0);
								g=dekr(zv,r);
								f=dxlam[dxindex(j,z,-1,r,aktvekt)]*pe[n][pindex(n,l-1,g)];
								if (g)
									f+=lam[Index(-1,r,aktvekt)]*dxpe[n][dxpindex(j,z,n,l-1,g)];
								s+=f*aus[i];
								if (l==1)
									k+=aus[i]*dxlam[dxindex(j,z,-1,r,aktvekt)];
							}
						s/=l;
						h+=(m[n]-l)*s;
						dxpe[n][dxpindex(j,z,n,l,aktvekt)]=s;
					}
					s= -(k+h)/m[n];
					dxpe[n][dxpindex(j,z,n,0,aktvekt)]=s;
				}
}

/* dxMWA()

   nach mue abgeleitete Mittelwertanalyse zur Berechnung einiger
   Auftragsmomente nach Strelen.

*/

void dxMWA()
{
	register f,g,j,l;
	register Ulong i;
	double h,k,s;

	aktvekt=1;
	code(aktvekt,zv);
	do
	{
		for(j=0;j<kn;j++)
			for(l=0;l<kl;l++)
				for(f=0;f<kl;f++)
					if (zv[f])
					{
						s=k=.0;
						for(g=0;g<kn;g++)
						{
							dxantwzeit[dxindex(j,l,g,f,aktvekt)]=h=dxANTWZEIT(j,l,g,f);
							s+=h;
							k+=antwzeit[Index(g,f,aktvekt)];
						}
						dxlam[dxindex(j,l,-1,f,aktvekt)]=h= -zv[f]*s/k/k;
						for(g=0;g<kn;g++)
						{
							i=dxindex(j,l,g,f,aktvekt);
							k=h*antwzeit[Index(g,f,aktvekt)];
							k+=lam[Index(-1,f,aktvekt)]*dxantwzeit[i];
							dxewka[i]=k;
							dxka1[dxindex(j,l,g,-1,aktvekt)]+=k;
						}
					}
		AKTdxpe();
		aktvekt++;
	}while(nextzv(zv));
}

/* Blockalloc1()

   Anmeldung der fuer die Netzwerkdaten noetigen Felder.

*/

void Blockalloc1()
{
	m=(int *)(block(kn*sizeof(int)));
	typ=(int *)(block(kn*sizeof(int)));
	anzkl=(int *)(block(kl*sizeof(int)));
	e=(double *)(block(kn*kl*sizeof(double)));
	mue=(double *)(block(kn*kl*sizeof(double)));
	sumamue=(double *)(block(kn*kl*sizeof(double)));
	amue=(double **)(block(kn*sizeof(double *)));
	amue_geord=(double **)(block(kn*sizeof(double *)));
}

/* Blockalloc2()

   Anmeldung aller benoetigten Datenfelder in Abhaengigkeit der
   gewaehlten Optionen.

*/

void Blockalloc2()
{
	register Ulong h,g=kl*sizeof(int),f=kn*kl*(MAXDECODE-1)*sizeof(double);

	zv=(int *)(block(g));
	hv=(int *)(block(g));
	hdv=(double *)(block(kl*sizeof(double)));
	nzv=(int *)(block(kn*g));
	codefakt=(Ulong *)(block(kl*sizeof(Ulong)));
	if (opt & 1)
	{
		g=kn*kl*sizeof(double);
		auftrm1=(double *)(block(g));
		auftrzm2=(double *)(block(g));
		prandbase=(double *)(block(kn*MAXDECODE*sizeof(double)));
		wschlm1=(double *)(block(g));
		wschlzm2=(double *)(block(g));
		pwschlbase=(pointer *)(block(kn*sizeof(pointer)));
		h=(MAXAUFTR+1)*kl*sizeof(double);
		for(g=0;g<kn;g++)
			if (!m[g] || m[g]>=GESAUFTR) pwschlbase[g]=0;
			else pwschlbase[g]=(double *)(block(h));
	}
	if (opt & 3)
	{
		g=kn*sizeof(pointer);
		wzeitm1=(pointer *)(block(g));
		wzeitzm2=(pointer *)(block(g));
		vzeitm1=(pointer *)(block(g));
		vzeitzm2=(pointer *)(block(g));
		h=kl*sizeof(double);
		for(g=0;g<kn;g++)
			if (m[g]==1 && typ[g]==1)
			{
				wzeitm1[g]=(double *)(block(h));
				wzeitzm2[g]=(double *)(block(h));
				vzeitm1[g]=(double *)(block(h));
				vzeitzm2[g]=(double *)(block(h));
			}
			else wzeitm1[g]=wzeitzm2[g]=vzeitm1[g]=vzeitzm2[g]=0;
	}
	antwzeit=(double *)(block(f));
	ka1=(double *)(block((MAXDECODE-1)*kn*sizeof(double)));
	lam=(double *)(block((MAXDECODE-1)*kl*sizeof(double)));
	ewka=(double *)(block(f));
	gg=(double *)(block(MAXDECODE*sizeof(double)));
	muequer=(double *)(block(kn*sizeof(double)));
	rho=(double *)(block(kn*kl*sizeof(double)));
	rhoges=(double *)(block(kn*sizeof(double)));
	aus=(double *)(block(kn*kl*sizeof(double)));
	lambda=(double *)(block(kn*kl*sizeof(double)));
	if (opt & 2)
	{
		dxantwzeit=(double *)(block(kn*kl*f));
		dxka1=(double *)(block(kn*f));
		dxlam=(double *)(block(kl*f));
		dxewka=(double *)(block(kn*kl*f));
		sauftrzm2=(double *)(block(kn*kl*sizeof(double)));
	}
	f=kn*sizeof(pointer);
	pe=(pointer *)(block(f));
	if (opt & 4)
		norm=(pointer *)(block(f));
	if (opt & 2)
		dxpe=(pointer *)(block(f));
	h=(MAXDECODE-1)*sizeof(double);
	g=h+sizeof(double);
	for(f=0;f<kn;f++)
		if (m[f]>1)
		{
			if (!ff && kl>1)
			{
				ff=(double *)(block(g));
				gN=(double *)(block(g));
				pI=(double *)(block(g));
			}
			pe[f]=(double *)(block(g*m[f]));
			if (opt & 4)
				norm[f]=(double *)(block(h));
			if (opt & 2)
				dxpe[f]=(double *)(block(kn*kl*m[f]*h));
		}
		else
		{
			pe[f]=0;
			if (opt & 4) norm[f]=0;
			if (opt & 2) dxpe[f]=0;
		}
}

/******************************************************************************/ 
/* Blockalloc3()

   Speicherfelder fuer Bedienraten asymmetrischer Knoten
*/
/******************************************************************************/ 

void Blockalloc3()
{
     int g,f;

     for (f=0;f<kn;f++)
     {
              g= m[f]*sizeof(double);
	      if (g)
	      {
		  amue[f]= (double *)(block(g));
		  amue_geord[f]= (double *)(block(g));
	      }
     }
}

/* Reset()

   Initialisiert Datenfelder (soweit erforderlich).

*/

void Reset()
{
	register Ulong h,f,g=MAXDECODE-1,i;

	f=1;
	for(h=0;h<kl;codefakt[h]=f,f*=anzkl[h]+1,h++);
	for(f=0;f<kn*kl*g;f++)
		antwzeit[f]=ewka[f]=.0;
	for(f=0;f<kl*g;f++)
		lam[f]=.0;
	for(f=0;f<kn*g;f++)
		ka1[f]=.0;
	for(f=0;f<kn*kl;f++)
	{
		aus[f]=e[f]/mue[f];
		if (!bsn)
			if (!e[f]) bsn=true;
	}
	for(f=0;f<kn;f++)
	{
		i=f*kl;
		rhoges[f]=.0;
		muequer[f]=mue[i];
		if (kl>1)
		{
			for(h=1;h<kl;muequer[f]+=mue[i+h],h++);
			muequer[f]/=kl;
		}
	}
	if (opt & 2)
	{
		for(f=0;f<kn;f++)
			if (dxpe[f])
				for(h=0;h<m[f]*kn*kl*g;dxpe[f][h]=.0,h++);
		g*=kn*kl;
		for(f=0;f<kn*kl*g;f++)
			dxantwzeit[f]=dxewka[f]=.0;
		for(f=0;f<kl*g;f++)
			dxlam[f]=.0;
		for(f=0;f<kn*g;f++)
			dxka1[f]=.0;
	}
	if (opt & 1)
	{
		for(f=0;f<kn*MAXDECODE;prandbase[f]=.0,f++);
		for(f=0;f<kn;f++)
			if (pwschlbase[f])
				for(h=0;h<kl;h++)
					for(g=0,i=(MAXAUFTR+1)*h;g<=MAXAUFTR;pwschlbase[f][i+g]=.0,g++);
	}
	for(f=0;f<kn;f++)
		if (pe[f])
		{
			pe[f][0]=1.0;
			for(g=1;g<m[f]*MAXDECODE;pe[f][g]=.0,g++);
		}
	if (kl>1)
	{
		f=0;
		while(!bdv && f<kn)
		{
			if (typ[f]==1)
				for(g=1,h=f*kl;g<kl;g++)
					if (mue[h+g]!=mue[h])
					{
						bdv=true;
						break;
					}
			f++;
		}
	}
	f=0;
	while(f<kn && m[f]<2) f++;
	if (f<kn) mts=true;
	f=0;
	while(f<kn)
		if (m[f]!=1) f++;
		else
			if (typ[f]!=1) f++;
			else
			{
				mm1=true;
				break;
			}
}

/* Blockfree()

   Rueckgabe aller reservierten Speicherfelder.

*/

void Blockfree()
{
	register f;

	if (!oom && ctlmsg) fprintf(stderr,"%s %lu Byte(s)\n",spr("chk025"),MEM);
	if (zv) free(zv);
	if (nzv) free(nzv);
	if (codefakt) free(codefakt);
	if (pe)
	{
		for(f=0;f<kn;f++)
			if (pe[f]) free(pe[f]);
		free(pe);
	}
	if (norm)
	{
		for(f=0;f<kn;f++)
			if (norm[f]) free(norm[f]);
		free(norm);
	}
	if (dxpe)
	{
		for(f=0;f<kn;f++)
			if (dxpe[f]) free(dxpe[f]);
		free(dxpe);
	}
	if (pwschlbase)
	{
		for(f=0;f<kn;f++)
			if (pwschlbase[f]) free(pwschlbase[f]);
		free(pwschlbase);
	}
	if (wzeitm1)
	{
		for(f=0;f<kn;f++)
			if (wzeitm1[f]) free(wzeitm1[f]);
		free(wzeitm1);
	}
	if (wzeitzm2)
	{
		for(f=0;f<kn;f++)
			if (wzeitzm2[f]) free(wzeitzm2[f]);
		free(wzeitzm2);
	}
	if (vzeitm1)
	{
		for(f=0;f<kn;f++)
			if (vzeitm1[f]) free(vzeitm1[f]);
		free(vzeitm1);
	}
	if (vzeitzm2)
	{
		for(f=0;f<kn;f++)
			if (vzeitzm2[f]) free(vzeitzm2[f]);
		free(vzeitzm2);
	}
	if (amue)
	{
		for(f=0;f<kn;f++)
			if (amue[f]) free(amue[f]);
		free(amue);
	}
	if (amue_geord)
	{
		for(f=0;f<kn;f++)
			if (amue_geord[f]) free(amue_geord[f]);
		free(amue_geord);
	}
	if (hv) free(hv);
	if (hdv) free(hdv);
	if (auftrm1) free(auftrm1);
	if (auftrzm2) free(auftrzm2);
	if (prandbase) free(prandbase);
	if (wschlm1) free(wschlm1);
	if (wschlzm2) free(wschlzm2);
	if (m) free(m);
	if (typ) free(typ);
	if (anzkl) free(anzkl);
	if (e) free(e);
	if (mue) free(mue);
	if (sumamue) free(sumamue);
	if (muequer) free(muequer);
	if (antwzeit) free(antwzeit);
	if (ka1) free(ka1);
	if (lam) free(lam);
	if (ewka) free(ewka);
	if (gg) free(gg);
	if (dxantwzeit) free(dxantwzeit);
	if (dxka1) free(dxka1);
	if (dxlam) free(dxlam);
	if (dxewka) free(dxewka);
	if (sauftrzm2) free(sauftrzm2);
	if (ff) free(ff);
	if (gN) free(gN);
	if (pI) free(pI);
	if (rho) free(rho);
	if (rhoges) free(rhoges);
	if (aus) free(aus);
	if (lambda) free(lambda);
}
#endif
