/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tbeshauf.c\t1.3\t92/03/27 11:51:14";

/*********************************************************************
***                                                                ***
***  Modul:      b e s h a u f . c                                 ***
***                                                                ***
***----------------------------------------------------------------***
***                                                                ***
***  Version:    2.0                                               ***
***  Datum:      24.09.87                                          ***
***  Autor:      Gerhard Schmuck                                   ***
***  Inhalt:     Modul zur Umrechnung der Uebergangswahrschein-    ***
***              lichkeiten in rel.Besuchshaeufigkeiten            ***
***                                                                ***
**********************************************************************
*/
#include <stdio.h>
#include "d_netz.h"
#include "memory.h"

extern netzparameter net;
float  **drmat;           /* Arbeitsmatrix */
double drvek [NMAX+1];    /* Arbeitsvektor "Rechte Seite" */
/*
**********************************************************************
*                       Funktion UEW_BESH                            *
**********************************************************************
Uebergangswahrsch. net.uew und rel.Besuchshfgkt. net.besh muessen
bereitgestellten Speicherplatz haben.
*/
UEW_BESH ()
{
      int    dim, fz, zl, sp, kl,
             tausch = 0,
             erg    = 0;
      double             sum, korr, hilf;
      
      dim   = net.knot;
      drmat = NN_ALLOC ( net.knot); /* Speicherplatz fuer Arbeitsmatrix */
                                    /* bereitstellen.                   */

      for ( kl=0; kl < net.klass; kl++) {

          /*----- Koeff.matrix bereitstellen, kopieren, invertieren --*/
            for ( zl=1; zl <= dim; zl++)
                  for (sp=1; sp <= dim; sp++)
                  if  ( zl == sp)
                        drmat[zl][sp] = 1.0 - net.uew[zl][sp][kl];
                  else
                        drmat[zl][sp] = -net.uew[sp][zl][kl];

			  for ( sp=1; sp <= dim; sp++)
					drvek[sp]     = net.uew[0][sp][kl];
     
          /*----- Umgruppierung der Koeffizientenmatrix --------------*/
          /* Zeilentausch mit tieferer Zeile wenn moeglich            */
            for ( zl=1; zl < dim; zl++)
            if  ( drmat[zl][zl] == 0.0) {
                  fz     = zl + 1;
                  tausch = 1;
                  while ( fz++ <= dim && tausch == 1)
                  if ( drmat[fz][zl] != 0.0) {
                        /*--- Zeilen tauschen ---*/
                        for ( sp=1; sp <= dim; sp++) {
                              hilf          = drmat[zl][sp];
                              drmat[zl][sp] = drmat[fz][sp];
                              drmat[fz][sp] = hilf;
                        }
                        hilf      = drvek[zl];
                        drvek[zl] = drvek[fz];
                        drvek[fz] = hilf;
                        tausch    = 0;
                  } 
            }

          /*----- Umrechnen in Dreiecksmatrix ------------------------*/
            for ( zl=1; zl < dim; zl++)
                  for ( fz = zl+1; fz <= dim; fz++) {
                        if ( drmat[zl][zl] != 0.0)
                              korr = drmat[fz][zl] / drmat[zl][zl];
                        else
                              korr = 0.0;
                        for ( sp = zl+1; sp <= dim; sp++)
                              drmat[fz][sp] = 
                                    drmat[fz][sp] - korr * drmat[zl][sp];
                        drvek[fz] = drvek[fz] - korr * drvek[zl];
                  }

		
          /*----- Loesen des dreieckigen Gleichungssystems -----------*/
            for ( zl=dim; zl >= 1; zl--) {
                  sum  = drvek[zl];
                  for ( sp = zl+1; sp <= dim; sp++)
                        sum = sum - ( drmat[zl][sp] * net.besh[sp][kl]);
                  if  ( drmat[zl][zl] != 0.0)
                        net.besh[zl][kl] = sum / drmat[zl][zl];
                  else 
                        net.besh[zl][kl] = 0.0;  /* frei waehlbar */
            }   
      }
      NN_FREE ( drmat, net.knot);  /* Speicherplatz der Arbeitsmatrix */
                                   /* wieder freigeben.               */
}
/*
******************** Ende des Moduls beshauf.c **********************/

beshdump()
{
	int i, j;

	fprintf(stderr,"Besh\n");
	for(j=0;j<net.klass;j++)
	for(i=0;i<=net.knot;i++)
	{
			fprintf(stderr,"%f ",net.besh[i][j]);
	}
	fprintf(stderr,"\n");
}
vekdump()
{
	int i, j;

	fprintf(stderr,"rechte Seite\n");
	for(i=0;i<=net.knot;i++)
	{
			fprintf(stderr,"%f ",drvek[i]);
	}
	fprintf(stderr,"\n");
}
matdump()
{
	int i, j;

	fprintf(stderr,"Matrix\n");
	for(i=0;i<=net.knot;i++)
	{
		for(j=0;j<=net.knot;j++)
		{
			fprintf(stderr,"%f ",drmat[i][j]);
		}
		fprintf(stderr,"\n");
	}
}


netmatdump()
{
	int i, j, k;

	for(k=0;k<net.klass;k++)
	{
		for(i=0;i<=net.knot;i++)
		{
			for(j=0;j<=net.knot;j++)
			{
				fprintf(stderr,"%f ",net.uew[i][j][k]);
			}
		}
		fprintf(stderr,"\n");
	}
}

