/*


				   P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tmvamix.c\t1.4\t93/09/28 10:36:45";

/*********************************************************************
***                                                                ***
***  Modulname:  m v a m i x . c                           main    ***
***                                                                ***
***----------------------------------------------------------------***
***                                                                ***
***  Version:    1.0                                               ***
***  Datum:      14.09.87                                          ***
***  Autor:      Gerhard Schmuck                                   ***
***  Inhalt:     Mittelwertanalyse fuer gemischte Netze mit        ***
***              mehreren Auftragsklassen                          ***
***                                                                ***
**********************************************************************


**********************************************************************
*                  Deklarationen und Definitionen                    *
**********************************************************************
*/ 
#include <stdio.h>
#include "stdfkt.h"
#include "memory.h"
#include "d_netz.h"        /* Netzparameter                  */

struct auftrag {                  /* Struktur fuer Zustandsvektor   */
      int           zustand[RMAX];
      float          *zahl;
      struct auftrag *next;
};

extern int     OUT_STDKOPF (),    /* Funktionen zur Ausgabe der     */
               OUT_EINZLGR (),    /* Leistungsgroessen              */
               OUT_KLASLGR (),
               UEW_BESH    ();    /* Umrechnung Uew ---> Beshauf    */

struct auftrag *LOAD_AUFT  ();

netzparameter  net;
leistgroessen  lgr;
klassenleistgr kls;

struct auftrag *altauft,          /* Variablen zur Listenverwaltung */
               *neuauft, *last;
int           endzust  [RMAX],   /* Netzzustaende                  */
               zustand     [RMAX+1];
int		*zust;
int            endniv = 0, niv,
               stelle, rest, ls,  /* Variablen zur Zustandserzeugung*/
               uewflag;
double         cld_mauft[NMAX+1],
               opn_rho  [NMAX+1];
/*
**********************************************************************
*                                                                    *
*                     Hauptprogramm MVAMIX                           *
*                                                                    *
**********************************************************************
*/

void abbruch(sig)
int sig;
{
        printf("mvamix_signal_%d\n 0 0 0 \n", sig);
        exit(1);
}

main(argc, argv)
	int argc;
	char *argv[];
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

    int  i, r, s;
    int  maxkomb, anzkomb,  /* Zahlen erzeugter Zustandskombinationen */
         maxsave, anzsave;  /* Zahlen gespeicherter Zustaende */

    void abbruch();

    for (i=1; i<12; i++)
        signal (i, abbruch);

	if ((argc > 1) && (!strcmp(argv[1], "-V")))    /* kirschni 22.11.1991 */
	{
		fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
		exit(0);
	}


	/*---- Daten einlesen, event. Uew's berechnen, off.Auslast ber. --*/
	EXITCTRL   ();
	INP_NETZ   ();   
	if ( uewflag == 1) {
		UEW_BESH ();
		NNR_FREE ( net.uew, net.knot);
	}
	ALLOC_LGR  ( net.knot, net.klass);  
	OPENRHO    ();     /* Berechnen der offenen Auslastungen */

	/*---------- Initialisierungen -------------------------------*/
	zust = &zustand[1];
	for ( r=0; r < net.klass; r++) {
		zust [r] = 0;
		if ( net.ankr[r] > 0) { /* offene Auftragsklasse */
			endzust[r] = 0;
			kls.typ[r] = 'o';
		}
        else  {                 /* geschlossene Auftragsklasse */
            endzust[r] = net.auft[r];
            endniv     += net.auft[r];
            kls.typ[r] = 'c';
        }
	}
	INIT_ALTAUFT ();

	/*---------- Vorbesetzung fuer Zustandserzeugung -------------*/
	if ( net.klass > 1) 
		ls = 1; 
	else
		ls = 0;
	maxkomb = 0;
	maxsave = 0;

	/*---- Iteration der geschlossenen Klassen fuer die ein- -----*/
	/*---- zelnen Zustaende bis Endzustand erreicht ist      -----*/
	while ( niv < endniv) {

		niv++;
		last   = NULL;

		/*---- Vorbesetzung fuer Zustandserzeugung -------------*/
		anzkomb = 0;
		anzsave = 0;
		stelle  = net.klass;
		rest    = niv;
		for ( s=0; s < net.klass; s++) zust[s] = 0;
		zust[net.klass-2] = -1;

		/*---- Berechnung fuer alle Zustaende des Niveaus niv -*/
		do    {
			KOMLORD ();
			anzkomb ++;
			if ( ZUST_KLEIN_ENDZ ( zust, endzust) == 0 ) {
				anzsave ++;
				MWA     ();
				SAVE_NEUAUFT ( zust);
			}
		}
		while ( zust[0] < niv);

		if ( anzkomb > maxkomb)
			maxkomb = anzkomb;
		if ( anzsave > maxsave)
			maxsave = anzsave;

		FREE_AUFT ( altauft); /* Loeschen der alten Werte */
		altauft   = neuauft;  /* Zuweisen der jetzt alten Werte */
		neuauft   = NULL;     /* Bereitstellen fuer neue Werte */
	}
	/*---------- Leistungsgroessen berechnen und ausgeben ---------*/
	BERECH_LGR  ();
	OUT_STDKOPF ( "mvamix", net.typ, net.knot, net.klass);
	OUT_EINZLGR ( net.knot, net.klass); 

	exit(0);
}
/*
**********************************************************************
*                                                                    *
*                      Programm - Module                             *
*                                                                    *
**********************************************************************


**********************************************************************
*                     Funktion BERECH_LGR                            *
**********************************************************************
Berechnung der Einzel- und Klassenleistungsgroessen
*/
BERECH_LGR ()
{
	int    i, r;
	double rho;

	/*------------ Einzelleistungsgroessen -----------------------*/
	for ( r=0; r < net.klass; r++) 
	{
	if  ( net.ankr[r] > 0)
		/*------ offene Auftragsklassen ------------------------*/
		for ( i=1; i <= net.knot; i++) {
			lgr.lambda[i][r] = net.besh[i][r] * net.ankr[r];
			/* lgr.rho bekannt aus OPENRHO */
			if ( net.bede[i] == 0) /* IS-Knoten */
				  {
						lgr.rho[i][r] = 0.0;
						lgr.mauft[i][r] = lgr.lambda[i][r] / net.bedr[i][r];
						lgr.vzeit[i][r] = 1.0 / net.bedr[i][r];
						lgr.mqlen[i][r] = lgr.wzeit[i][r] = 0.0;
				  }
			else                /* Nicht-IS-Knoten */
				  {
						lgr.mauft[i][r] = ( lgr.rho[i][r] * ( 1 + cld_mauft[i])) / ( 1 - opn_rho[i]);
					  if (lgr.lambda[i][r] != 0.0)
					  {
						  lgr.vzeit[i][r] = lgr.mauft[i][r] / lgr.lambda[i][r];
						  lgr.wzeit[i][r] = lgr.vzeit[i][r] - 1.0 / net.bedr[i][r];
					  }
					  else
					  {
						  lgr.vzeit[i][r] = 0.0;
						  lgr.wzeit[i][r] = 0.0;
					  }
					  lgr.mqlen[i][r] = lgr.lambda[i][r] * lgr.wzeit[i][r];
				  }
		}
	else
		/*------ geschlossene Auftragsklassen ------------------*/
		for ( i=1; i <= net.knot; i++) {
			lgr.lambda[i][r] = net.besh[i][r] * kls.lambda[r];
			if ( net.bede[i] == 0) /* IS-Knoten */
				  {
						lgr.rho[i][r] = 0.0;
						lgr.mauft[i][r] = lgr.lambda[i][r] / net.bedr[i][r];
						lgr.vzeit[i][r] = 1.0 / net.bedr[i][r];
						lgr.mqlen[i][r] = lgr.wzeit[i][r] = 0.0;
				  }
				  else
				  {
					  lgr.rho   [i][r] = lgr.lambda[i][r] / net.bedr[i][r];
					  /* lgr.vzeit bekannt aus MWA */
					  /* lgr.mauft bekannt aus MWA */
					  lgr.wzeit [i][r] = lgr.vzeit[i][r] - 1.0 /
						                   net.bedr[i][r];
					  lgr.mqlen [i][r] = lgr.lambda[i][r] * lgr.wzeit[i][r];
				  }
		}
	}

	/*--------------- Klassenleistungsgroessen -------------------*/
	  /*
	for ( r=0; r < net.klass; r++) {
		if ( net.ankr[r] > 0)
			kls.lambda[r] = net.ankr[r];
				  */
		/* kls.lambda fuer geschl.Klassen bekannt aus MWA */
			/*
		kls.mauft[r] = 0;
		kls.mqlen[r] = 0;
		for ( i=1; i <= net.knot; i++) {
			kls.mauft[r] += lgr.mauft[i][r];
			kls.mqlen[r] += lgr.mqlen[i][r];
		}
		kls.vzeit[r] = kls.mauft[r] / kls.lambda[r];
		kls.wzeit[r] = kls.mqlen[r] / kls.lambda[r];
	}
	  */
}
/*
**********************************************************************
*                      Funktion FREE_AUFT                            *
**********************************************************************
Gibt den gesamten von "auft" belegten Speicherplatz wieder freiRD/
*/
FREE_AUFT ( auft)
	struct auftrag *auft;
{
	struct auftrag *satz;
	int            i;

	while ( auft != NULL) {
		satz = auft->next;
		cfree ( (char *) auft->zahl);
		cfree ( (char *) auft);
		auft  = satz;
	}
}
/*
**********************************************************************
*                       Funktion INIT_ALTAUFT                        *
**********************************************************************
Initialisieren des Startwertes fuer Speichervektor
*/
INIT_ALTAUFT ()
{
	int          i;
	unsigned int kn;

	kn = net.knot + 1;

	altauft = ( struct auftrag *) calloc ( 1, sizeof( struct auftrag));
	if ( altauft == NULL) {
		fprintf ( stderr, "Nicht genuegend Speicherplatz.\n");
	    printf("mvamix_speicher\n0 0 0 \n");
		exit    ( 3);
	}
	ZUSTCPY       ( altauft->zustand, zust);
	altauft->next = NULL;
	altauft->zahl = ( float *) calloc ( kn, sizeof( float));
	if ( altauft->zahl == NULL) {
		fprintf ( stderr, "Nicht genuegend Speicherplatz.\n");
	    printf("mvamix_speicher\n0 0 0 \n");
		exit    ( 3);
	}
	for ( i=1; i <= net.knot; i++) {
		cld_mauft[i]     = 0;
		altauft->zahl[i] = 0;
	}
}
/*
**********************************************************************
*                       Funktion INP_NETZ                            *
**********************************************************************
Einlesen der Netzparameter von Standarteingabe
*/
INP_NETZ ()
{
	int  m, n, r;

   /* Uewflag, Netztyp, Knotenanzahl, Klassenanzahl */
	JUMPCOM ( stdin);
	scanf   ( "%d %1s %d %d", &uewflag, &net.typ, &net.knot, &net.klass);
   /* Bedieneinheiten pro Knoten */
	JUMPCOM ( stdin);
	for ( n=1; n <= net.knot; n++) scanf ( "%d", &net.bede[n]);
   /* Auftragszahlen pro Klasse */
	JUMPCOM ( stdin);
	for ( r=0; r < net.klass; r++) scanf ( "%d", &net.auft[r]);
   /* Externe Klassen-Ankunftsraten */
	JUMPCOM ( stdin);
	for ( r=0; r < net.klass; r++) scanf ( "%lf", &net.ankr[r]);   

   /* Klassenparameter */
	net.bedr = NR_ALLOC ( net.knot, net.klass);
	net.besh = NR_ALLOC ( net.knot, net.klass);
	if ( uewflag == 1)
		net.uew = NNR_ALLOC ( net.knot, net.klass);
	for ( r=0; r < net.klass; r++) {
	   /* Bedienraten */
		JUMPCOM  ( stdin);
		for ( n=1; n <= net.knot; n++)
			scanf ( "%lf", &net.bedr[n][r]);
		JUMPCOM  ( stdin); 
	   /* Uebergangswahrscheinlichkeiten ... */
		if ( uewflag == 1)
			for ( m=0; m <= net.knot; m++)
				for ( n=0; n <= net.knot; n++)
					scanf ( "%f", &net.uew[m][n][r]);
	   /* ... oder rel.Besuchshaeufigkeiten */
		else
			for ( n=1; n <= net.knot; n++)
				scanf ( "%lf", &net.besh[n][r]);
	}
#ifdef DEBUG 
	dump_net();
#endif
}

dump_net ()
{
	int  m, n, r;

   /* Uewflag, Netztyp, Knotenanzahl, Klassenanzahl */
	fprintf(stderr,    "uewflag %d  net.typ %c net.knot %d  net.klass %d ", uewflag, net.typ, net.knot, net.klass);
   /* Bedieneinheiten pro Knoten */
	  fprintf(stderr,"\nnet.bede ");
	for ( n=1; n <= net.knot; n++) fprintf(stderr,  "%d ", net.bede[n]);
   /* Auftragszahlen pro Klasse */
	  fprintf(stderr,"\nnet.auftr ");
	for ( r=0; r < net.klass; r++) fprintf(stderr,  "%d ", net.auft[r]);
   /* Externe Klassen-Ankunftsraten */
	  fprintf(stderr,"\nnet.ankr ");
	for ( r=0; r < net.klass; r++) fprintf(stderr,  "%lf ", net.ankr[r]);   

   /* Klassenparameter */
	for ( r=0; r < net.klass; r++) {
	   /* Bedienraten */
		  fprintf(stderr,"\nnet.bedr ");
		for ( n=1; n <= net.knot; n++)
			fprintf(stderr,  "%lf ", net.bedr[n][r]);
	   /* Uebergangswahrscheinlichkeiten ... */
		  fprintf(stderr,"\nnet.besh ");
		if ( uewflag == 1)
			for ( m=0; m <= net.knot; m++)
				  {
					  fprintf(stderr,"\n ");
				for ( n=0; n <= net.knot; n++)
					fprintf(stderr,  "%f ", net.uew[m][n][r]);
				  }
	   /* ... oder rel.Besuchshaeufigkeiten */
		else
			{
				  fprintf(stderr,"\n ");
			for ( n=1; n <= net.knot; n++)
				fprintf(stderr,  "%lf ", net.besh[n][r]);
			}
	}
	  fprintf(stderr,"\n ");
}

/*
**********************************************************************
*                       Funktion KOMLORD                             *
**********************************************************************
Algorithmus zur Erzeugung aller voneinander verschiedener Kombina-
tionen mit Wiederholung von k Elementen zur Ordnung r in lexiko-
grafischer Anordnung
*/
KOMLORD ()
{
	if ( zust[net.klass-1] == 0) {
		stelle--;
		zust [stelle]    = 0;
		zust [stelle-ls] ++;
		zust [net.klass-1] = rest;
		rest--;
		if ( rest < 0)
			rest   = zust [stelle-1] - 1;
		else
			stelle = net.klass - 1;
		}
	else  {
		zust [stelle] --;
		zust [stelle-1] ++;
	}
}
/*
**********************************************************************
*                       Funktion LOAD_AUFT                           *
**********************************************************************
Gibt den Auftragssatz des angegebenen Zustandes zurueck
*/
struct auftrag *LOAD_AUFT ( auft, zustand)
	struct auftrag *auft;
	int           *zustand;
{
	struct auftrag *suchauft;
    
	suchauft = auft;
	while ( ZUSTCMP( zustand, suchauft->zustand) != 0 && suchauft != NULL)
		suchauft = suchauft->next;
	return( suchauft);
}
/*
***********************************************************************
*                          Funktion MWA                               *
***********************************************************************
Berechnen der mittleren Auftragszahlen fuer einen gegebenen Netzzustand
*/
MWA ()
{
	struct auftrag *auft;
	int            i, r, s;
	int           altzust [RMAX];
 
	for ( r=0; r < net.klass; r++) {

		if ( zust[r] > 0 ) {
			kls.vzeit[r] = 0;
			ZUSTCPY ( altzust, zust);
			altzust[r]--;
			auft    = LOAD_AUFT ( altauft, altzust);
			if ( auft == NULL) {
				fprintf ( stderr, "Zustand nicht gefunden.\n");
			  printf("mvamix_zustand\n0 0 0 \n");
				/*printf  ( "\004");*/
				exit    ( 1);
			}

			for ( i=1; i <= net.knot; i++) {
				if ( net.bede[i] == 0) /* IS - Knoten */
					lgr.vzeit[i][r] = 1.0 / ( net.bedr[i][r] *
						            ( 1 - opn_rho[i]) );
				else                   /* Knotentyp 1,2,4 */
					lgr.vzeit[i][r] = ( 1 + auft->zahl[i]) /
						            ( net.bedr[i][r] *
						            ( 1 - opn_rho[i]) );
				kls.vzeit[r] += net.besh[i][r] * lgr.vzeit[i][r];
			}
			kls.lambda[r] = zust[r] / kls.vzeit[r];
			for ( i=1; i <= net.knot; i++)
				lgr.mauft[i][r] = net.besh[i][r] * kls.lambda[r] *
						                 lgr.vzeit[i][r];
			}
		else  {
			kls.lambda[r] = 0;
			for ( i=1; i <= net.knot; i++) {
				lgr.vzeit[i][r] = 0;
				lgr.mauft[i][r] = 0;
			}
		}
	}

	for ( i=1; i <= net.knot; i++) {
		cld_mauft[i] = 0;
		for ( r=0; r < net.klass; r++)
			cld_mauft[i] += lgr.mauft[i][r];
	}
}
/*
***********************************************************************
*                       Funktion OPENRHO                              *
***********************************************************************
Berechnen der Knoten- und Einzelauslastungen der offenen Klassen
*/
OPENRHO ()
{
	int  i, r;

	for ( i=1; i <= net.knot; i++) {
		opn_rho[i] = 0;
		for ( r=0; r < net.klass; r++)
		if  ( net.ankr[r] > 0) {
				  if(net.bede[i] == 0)
					lgr.rho[i][r] = 0.0;
				  else
					  lgr.rho[i][r] = net.besh[i][r] * net.ankr[r] /
					    net.bedr[i][r];
			opn_rho[i]   += lgr.rho[i][r];
		}
		if  ( opn_rho[i] >= 1) {
			fprintf ( stderr, "Stabilitaetsbedingung nicht erfuellt.\n");
		  printf("mvamix_instabil\n0 0 0 \n");
			/*printf  ( "\004");*/
			exit    ( 4);
		}
	}
}
/*
**********************************************************************
*                     Funktion SAVE_NEUAUFT                          *
**********************************************************************
Erzeugen und speichern eines neuen Auftragssatzes
*/
SAVE_NEUAUFT ( zustand)
	int  zustand [RMAX]; 
{ 
	int            i;
	unsigned int   kn;

	kn = net.knot + 1;

	neuauft = ( struct auftrag *) calloc ( 1, sizeof( struct auftrag));
	if ( neuauft == NULL) {
		fprintf ( stderr, "Nicht genuegend Speicherplatz.\n");
	    printf("mvamix_speicher\n0 0 0 \n");
		exit    ( 3);
	}
	ZUSTCPY ( neuauft->zustand, zustand);
	neuauft->zahl = ( float *) calloc ( kn, sizeof( float));
	if ( neuauft == NULL) {
		fprintf ( stderr, "Nicht genuegend Speicherplatz.\n");
	    printf("mvamix_speicher\n0 0 0 \n");
		exit    ( 3);
	}
	for ( i=1; i <= net.knot; i++) neuauft->zahl[i] = cld_mauft[i];
	neuauft->next = last;
	last          = neuauft;
}
/*
**********************************************************************
*                  Funktion ZUST_KLEIN_ENDZ                          *
**********************************************************************
liefert 0, wenn jedes Zustandselement von zust <= entsprechendes 
	     Element von endzust
	  1  sonst
*/
ZUST_KLEIN_ENDZ ( zust, endzust)
	int  *zust, *endzust;
{
	int  r, erg = 0;

	for ( r=0; r < net.klass; r++)
	if  ( zust[r] > endzust[r])
		erg = 1;
	return ( erg);
}
/*
*********************************************************************
*                       Funktion ZUSTCMP                            *
*********************************************************************
Vergleichen von zust1 und zust2, Ergebnis: 0, wenn gleich
						       1, wenn nicht gleich
*/
ZUSTCMP ( z1, z2)
	int *z1, *z2;
{
	int  r, erg = 0;
	for ( r=0; r < net.klass; r++)
	if ( z1[r] != z2[r]) {
		erg = 1;
		break;
	}
	return ( erg);
}
/*
*********************************************************************
*                       Funktion ZUSTCPY                            *
*********************************************************************
Kopieren von Zustand2 nach Zustand1
*/
ZUSTCPY ( z1, z2)
	int *z1, z2[];
{
	int  r;
	for ( r=0; r < net.klass; r++) *( z1+r) = z2[r];
}
/*    
**********************************************************************
*                    Funktion ZUSTPRI                                *
**********************************************************************
Ausgeben eines Zustandes mit Kommentaren auf Standarterror
*/
ZUSTPRI ( fp, kom1, zustand, kom2)
	FILE *fp;
	  int *zustand;
	char *kom1, *kom2;
{
	int  r;
	fprintf ( fp, "%s (", kom1);
	for ( r=0; r < net.klass; r++) fprintf ( fp, " %d", zustand[r]);
	fprintf ( fp, ") %s", kom2);
	  fflush( fp);
}
/*
******************** Ende des Moduls mvamix.c ***********************/

dump_lgr()
{
	int i,j;

	for(i=0;i<net.klass;i++)
	{
		for(j=0;j<=net.knot;j++)
		{
			fprintf(stderr,"%lf ",lgr.lambda[j][i]);
			fprintf(stderr,"%lf ",lgr.rho[j][i]);
			fprintf(stderr,"%lf ",lgr.vzeit[j][i]);
			fprintf(stderr,"%lf ",lgr.wzeit[j][i]);
			fprintf(stderr,"%lf ",lgr.mauft[j][i]);
			fprintf(stderr,"%lf ",lgr.mqlen[j][i]);
			fprintf(stderr,"\n");
		}
		fprintf(stderr,"\n");
	}
}

dump_uew()
{
	int i,j,k;

	for(i=0;i<net.klass;i++)
	{
		for(j=0;j<=net.knot;j++)
		{
			for(k=0;k<=net.knot;k++)
			{
				printf("%f ",net.uew[j][k][i]);
			}
			printf("\n");
		}
		printf("\n");
	}
}
