/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\topenpfn.c\t1.4\t94/02/03 16:54:52";

/********************************************************************* 
***                                                                ***
***  Modul:     o p e n p f n . c                          main    ***
***                                                                ***
***----------------------------------------------------------------***
***                                                                ***
***  Datum:     31.07.87                                           ***
***  Version:   1.0                                                ***
***  Autor:     Gerhard Schmuck                                    ***
***  Inhalt:    Berechnungsverfahren fuer mehrklassige, offene     ***
***             Produktformnetze mit einer Bedieneinheit           ***
***                                                                ***
**********************************************************************

Hinweis: Das Verfahren arbeitet mit rel.Besuchshaeufigkeiten. Wurden
         Uebergangswahrsch. an das Verfahren geliefert, was am Flag
         'uewflag' zu erkennen ist, werden vor Analysebeginn diese
         in rel. Besuchshaefigkeiten umgerechnet.

**********************************************************************
*              Deklarationen und Definitionen                        *
**********************************************************************
*/
#include <stdio.h>
#include <signal.h>
#include "stdfkt.h"
#include "d_netz.h"
#include "memory.h"

netzparameter  net;
leistgroessen  lgr;
klassenleistgr kls;
gesamtleistgr  ges;

extern int     OUT_EINZLGR (), 
               OUT_NETZLGR (), 
               OUT_STDKOPF (),
               UEW_BESH    ();

void	INP_NETZ();
void	BER_EINZ_LGR();

int            kn, kl, uewflag;
/*
**********************************************************************
*                                                                    *
*                      Hauptprogramm OPENPFN                         *
*                                                                    *
**********************************************************************
*/

void abbruch(sig)
int sig;
{
        printf("openpfn_signal_%d\n 0 0 0 \n", sig);
        exit(1);
}

int main (argc, argv)
	int argc;
	char *argv[];
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

        int i;
        void abbruch();

        for (i=1; i<12; i++)
                signal (i, abbruch);

		if ((argc > 1) && (!strcmp(argv[1], "-V")))    /* kirschni 22.11.1991 */
		{
			fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
			exit(0);
		}


    /*---------- Initialisierungen ---------------------------------*/
      EXITCTRL ();
      INP_NETZ ();
      if ( uewflag == 1) {
            UEW_BESH ();
            NNR_FREE ( net.uew, net.knot);
      }
      ALLOC_LGR ( net.knot, net.klass);

    /*---------- Vorbesetzen von Gesamtleistungsgroessen -----------*/
	/*
      for ( kn=1; kn <= net.knot; kn++) {
            ges.rho   [kn]     = 0.0;
            ges.lambda[kn]     = 0.0;
      }
	  */
    /*---------- Berechnen der Durchsaetze und Auslastungen --------*/
      for ( kl=0; kl < net.klass; kl++)
      for ( kn=1; kn <= net.knot; kn++) {
            lgr.lambda[kn][kl] = net.ankr[kl] * net.besh[kn][kl];
			if(net.bede[kn] == 0)  /* IS Knoten */
				lgr.rho[kn][kl] = 0.0;
			else
				lgr.rho[kn][kl] = lgr.lambda[kn][kl] / net.bedr[kn][kl];
            ges.lambda[kn]    += lgr.lambda[kn][kl];
            ges.rho   [kn]    += lgr.rho[kn][kl];
      }
    /*---------- Pruefen der Stabilitaetsbedingungen --------------*/
      for ( kn=1; kn <= net.knot; kn++)
      if  ( ges.rho[kn] >= 1) {
            fprintf ( stderr, "Stabilitaetsbedingung nicht erfuellt.\n");
	    printf("openpfn_instabil\n0 0 0 \n");
            /*printf  ( "\004");*/
            exit    ( 4);
      }
    /*--------- Berechnen und Ausgeben der Leistungsgroessen ------*/
      BER_EINZ_LGR ();
	  /*
      BER_NETZ_LGR ();
	  */
      OUT_STDKOPF  ( "openpfn", 'o', net.knot, net.klass);
      OUT_EINZLGR  ( net.knot, net.klass);
      FREE_LGR     ( net.knot);
	exit(0);
}
/*
**********************************************************************
*                                                                    *
*                        Programm - Module                           *
*                                                                    *
**********************************************************************


**********************************************************************
*                       Funktion BER_EINZ_LGR                        *
**********************************************************************
Berechnung der Einzel-Leistungsgroessen
*/
void BER_EINZ_LGR ()
{
      for ( kn=1; kn <= net.knot; kn++)
      for ( kl=0; kl < net.klass; kl++)
      if  ( net.bede[kn] == 0) {
          /*---------- M/G/0-IS ( Infinite-Server-Knoten) ----------*/
			lgr.rho[kn][kl]   = 0.0;
            lgr.vzeit[kn][kl] = 1 / net.bedr[kn][kl];
            lgr.mauft[kn][kl] = lgr.lambda[kn][kl] * lgr.vzeit[kn][kl];
            lgr.wzeit[kn][kl] = 0.0;
            lgr.mqlen[kn][kl] = 0.0;
            }
      else  {
          /*---------- M/M/1-FCFS, M/G/1-PS, M/G/1-LCFS ------------*/
            lgr.mauft[kn][kl] = lgr.rho[kn][kl] / ( 1 - ges.rho[kn]);

			if (lgr.lambda[kn][kl] != 0.0)
			{
				lgr.vzeit[kn][kl] = lgr.mauft[kn][kl] / lgr.lambda[kn][kl];
				lgr.wzeit[kn][kl] = lgr.vzeit[kn][kl] - 1 / net.bedr[kn][kl];
				lgr.mqlen[kn][kl] = lgr.lambda[kn][kl] * lgr.wzeit[kn][kl];
			}
			else
			{
				lgr.vzeit[kn][kl] = 0.0;
				lgr.wzeit[kn][kl] = 0.0;
				lgr.mqlen[kn][kl] = 0.0;
			}
      }
}
/*
**********************************************************************
*                     Funktion BER_NETZ_LGR                          *
**********************************************************************
Berechnung der Netz - Leistungsgroessen ( Klassengroessen)
*/
void BER_NETZ_LGR ()
{
      for ( kl=0; kl < net.klass; kl++) {
            kls.typ   [kl] = 'o';
            kls.lambda[kl] = net.ankr[kl];
            kls.mauft [kl] = 0;
            kls.mqlen [kl] = 0;
            for ( kn=1; kn <= net.knot; kn++) {
                  kls.mauft[kl] += lgr.mauft[kn][kl];
                  kls.mqlen[kl] += lgr.mqlen[kn][kl];
            }

			if (kls.lambda[kl] != 0.0)
			{
				kls.vzeit [kl] = kls.mauft[kl] / kls.lambda[kl];
				kls.wzeit [kl] = kls.mqlen[kl] / kls.lambda[kl];
			}
			else
			{
				kls.vzeit [kl] = 0.0;
				kls.wzeit [kl] = 0.0;
			}
      }
}
/*
**********************************************************************
*                        Funktion INP_NETZ                           *
**********************************************************************
Netzgroessen von stdin einlesen und Speicherplatz bereitstellen
*/
void INP_NETZ ()
{
      int  m, n, r;

   /* Uewflag, Knotenzahl, Klassenzahl */
      JUMPCOM ( stdin);
      scanf   ( "%d %d %d", &uewflag, &net.knot, &net.klass);
   /* Klassen - Ankunftsraten */
      JUMPCOM ( stdin);
      for ( r=0; r < net.klass; r++) 
            scanf ( "%lf", &net.ankr[r]);
   /* Bedieneinheiten */
      JUMPCOM ( stdin);
      for ( n=1; n <= net.knot; n++)
            scanf ( "%d", &net.bede[n]);
   /* Einzelgroessen */
      net.bedr = NR_ALLOC ( net.knot, net.klass);
      net.besh = NR_ALLOC ( net.knot, net.klass);
      if  ( uewflag == 1)
            net.uew = NNR_ALLOC ( net.knot, net.klass);

      for ( r=0; r < net.klass; r++) {
         /* Bedienraten */
            JUMPCOM ( stdin);
            for ( n=1; n <= net.knot; n++)
                  scanf ( "%lf", &net.bedr[n][r]);
            JUMPCOM ( stdin);
         /* Uebergangswahrscheinlichkeiten ... */
            if  ( uewflag == 1)
                  for ( m=0; m <= net.knot; m++)
                        for ( n=0; n <= net.knot; n++)
                              scanf ( "%f", &net.uew[m][n][r]);
         /* ... oder Besuchshaeufigkeiten*/
            else
                  for ( n=1; n <= net.knot; n++)
                        scanf ( "%lf", &net.besh[n][r]);
      }
}
/*    
********************* Ende des Moduls openpfn.c ************************/
