/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tsopenpfn.c\t1.5\t94/02/03 16:54:54";

/********************************************************************* 
***                                                                ***
***  Modul:     s o p e n p f n . c                        main    ***
***                                                                ***
***----------------------------------------------------------------***
***                                                                ***
***  Datum:     31.07.87                                           ***
***  Version:   1.0                                                ***
***  Autor:     Gerhard Schmuck                                    ***
***  Inhalt:    Berechnungsverfahren fuer einklassige, offene      ***
***             Produktformnetze mit mehreren Bedieneinheit        ***
***                                                                ***
**********************************************************************

Hinweis: Das Verfahren arbeitet mit rel.Besuchshaeufigkeiten. Wurden
         Uebergangswahrsch. an das Verfahren geliefert, was am Flag
         'uewflag' zu erkennen ist, werden vor Analysebeginn diese
         in rel. Besuchshaefigkeiten umgerechnet.

**********************************************************************
*              Deklarationen und Definitionen                        *
**********************************************************************/
#include <stdio.h>
#include <string.h>                 /* K.R. 14.04.92                 */
#include <math.h>
#include <signal.h>
#include "stdfkt.h"
#include "d_netz.h"
#include "memory.h"

netzparameter  net;
leistgroessen  lgr;
klassenleistgr kls;
gesamtleistgr  ges;
char        *option;    /* Variable fuer die Funktion getopt() */

extern	void	qsort();

extern	int	OUT_EINZLGR(), 
		OUT_NETZLGR(), 
		OUT_STDKOPF(),
		UEW_BESH();
extern	double	rho_zufall(),
		rho_schnell(),
		p_wart_sch(),
		p_wart_bfh(),
		q_quer();

void	INP_NETZ();
void	BER_EINZ_LGR();
void	BER_NETZ_LGR();
int            kn, kl, uewflag;
double         P_w;
/*
**********************************************************************
*                                                                    *
*                      Hauptprogramm OPENPFN                         *
*                                                                    *
**********************************************************************/

void abbruch(sig)
int sig;
{
        printf("sopenpfn_signal_%d\n 0 0 0 \n", sig);
        exit(1);
}

int main (argc, argv)
	int argc;
	char *argv[];
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;
	extern char *optarg;    /* Variable fuer die Funktion getopt() */


	int         reval;      /* Return-Wert der Funktion getopt()   */

        int i;
        void abbruch();

        for (i=1; i<12; i++)
                signal (i, abbruch);

/*	  if ((argc > 1) && (!strcmp(argv[1], "-V")))*/  /* kirschni 22.11.1991 */
/*		{
			fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
			exit(0);
		}*/

    /* Interpretation der Kommandozeile, Kirsten Roggenkamp 07.04.92 */
    /* ------------------------------------------------------------- */
       while (( reval = getopt(argc,argv,"v:V")) != EOF)
	 {
	    switch (reval)
	    {
	       case 'V':  /* kirschni 22.11.1991 */
			      fprintf(stdout, "%s Version %d.%d\n", argv[0], 
					  SccsRelease, SccsLevel);
			      exit(0);
				break;
		 case 'v':
		            option = optarg;
			      break;
		 default :
	                  option[0] = '0';
				break;
	    }
	 }

    /*---------- Initialisierungen ---------------------------------*/
      EXITCTRL ();
      INP_NETZ ();
      if ( uewflag == 1) {
            UEW_BESH ();
            NNR_FREE ( net.uew, net.knot);
      }
      ALLOC_LGR ( net.knot, net.klass);

    /*---------- Vorbesetzen von Gesamtleistungsgroessen -----------*/
    /*
      for ( kn=1; kn <= net.knot; kn++) {
            ges.rho   [kn]     = 0.0;
            ges.lambda[kn]     = 0.0;
      }
	  */
    /*---------- Berechnen der Durchsaetze und Auslastungen --------*/
      for ( kl=0; kl < net.klass; kl++)
      for ( kn=1; kn <= net.knot; kn++) {
          lgr.lambda[kn][kl] = net.ankr[kl] * net.besh[kn][kl];

	    switch (net.bede[kn]) 
	    { 
		case 0:  /*--------- IS Knoten ---------*/
			     lgr.rho[kn][kl] = 0.0;
			     break;
		case 1:  /*---M/M/1-FCFS,M/G/1-PS,M/G/1-LCFS---*/
			     lgr.rho[kn][kl] = lgr.lambda[kn][kl] / net.bedr[kn][kl];
			     break;
		default: /*----------M/M/m-FCFS------------*/
			     if (net.knottyp[kn] != 20)/* symmetrischer Knoten      */
			        lgr.rho[kn][kl] = lgr.lambda[kn][kl] / 
				   		        (net.bede[kn] * net.bedr[kn][kl]);
			     else
			     switch(option[0])        /* asymmetrischer Knoten      */
			     {
			        case 'a' : /*            Scheuerer           */
						 /* ------------------------------ */
						   {
							double mueges = 0;
				                  for (i = 0; i < net.bede[kn]; i++) 
							{
						   	   mueges += net.amue[kn][i];
							}
						      lgr.rho[kn][kl] = lgr.lambda[kn][kl] /
									      mueges;
						   }
						   break;
				  case 'b' : /*       Baer/Fischer/Hertel      */
						 /*        zufaellige Auswahl      */
						 /* ------------------------------ */
						   lgr.rho[kn][kl] = rho_zufall(kn);    
						   break;
				  case 'c' : /*       Baer/Fischer/Hertel      */
						 /*    schnellste Bedieneinheit    */
						 /* ------------------------------ */
						   lgr.rho[kn][kl] = rho_schnell(kn);
						   break;
				  default  : /*  Falsches Verfahren angegeben  */
						 /*      Speicher freigeben        */
						 /* ------------------------------ */
						   { 
							FREE_LGR(net.knot);
						      printf("sopenpf_option\n 0 0 0\n");
						      exit(1);
						   }
			     }
	    }
            ges.lambda[kn]    += lgr.lambda[kn][kl];
            ges.rho   [kn]    += lgr.rho[kn][kl];
      }
    /*---------- Pruefen der Stabilitaetsbedingungen --------------*/
      for ( kn=1; kn <= net.knot; kn++)
      if  ( ges.rho[kn] >= 1) {
            fprintf ( stderr, "Stabilitaetsbedingung nicht erfuellt.\n");
	    printf("sopenpfn_instabil\n0 0 0 \n");
            /*printf  ( "\004");*/
            exit    ( 4);
      }
    /*--------- Berechnen und Ausgeben der Leistungsgroessen ------*/
      BER_EINZ_LGR ();
	  /*
      BER_NETZ_LGR ();
	  */
      OUT_STDKOPF  ( "sopenpfn", 'o', net.knot, net.klass);
      OUT_EINZLGR  ( net.knot, net.klass);
      FREE_LGR     ( net.knot);
	exit(0);
}
/*
**********************************************************************
*			Funktion fac				     *
**********************************************************************
Berechnung der Fakultaet des uebergebenen Wertes
*/
int fac(i)
	int i;
{
	int wert = 1;

	for ( ; i > 0 ; i--)
		wert *= i;
	return(wert);
}

/*
**********************************************************************
*                                                                    *
*                        Programm - Module                           *
*                                                                    *
**********************************************************************


**********************************************************************
*                       Funktion BER_EINZ_LGR                        *
**********************************************************************
Berechnung der Einzel-Leistungsgroessen
*/
void BER_EINZ_LGR ()
{
      for ( kn=1; kn <= net.knot; kn++)
      for ( kl=0; kl < net.klass; kl++)
      switch  ( net.bede[kn]) {
	  case 0:
          /*---------- M/G/0-IS ( Infinite-Server-Knoten) ----------*/
			lgr.rho[kn][kl]   = 0.0;
            lgr.vzeit[kn][kl] = 1 / net.bedr[kn][kl];
            lgr.mauft[kn][kl] = lgr.lambda[kn][kl] * lgr.vzeit[kn][kl];
            lgr.wzeit[kn][kl] = 0.0;
            lgr.mqlen[kn][kl] = 0.0;
	    break;
	  case 1:
          /*---------- M/M/1-FCFS, M/G/1-PS, M/G/1-LCFS ------------*/
            lgr.mauft[kn][kl] = lgr.rho[kn][kl] / ( 1 - ges.rho[kn]);
			if (lgr.lambda[kn][kl] != 0.0)
			{
				lgr.vzeit[kn][kl] = lgr.mauft[kn][kl] / lgr.lambda[kn][kl];
				lgr.wzeit[kn][kl] = lgr.vzeit[kn][kl] - 1 / net.bedr[kn][kl];
			}
			else
			{
				lgr.vzeit[kn][kl] = 0.0;
				lgr.wzeit[kn][kl] = 0.0;
			}
            lgr.mqlen[kn][kl] = lgr.lambda[kn][kl] * lgr.wzeit[kn][kl];
	    break;
	  default:{
	    /*---------- M/M/m-FCFS ----------------------------------*/
	    if (net.knottyp[kn] != 20)            /* symmetrischer Knoten */
	    {
		 /*-------Hilfsvariablen vereinbaren---------------------*/
	       register int i;    /*--------Laufvariable---------------*/
	       double hilf = net.bede[kn] * lgr.rho[kn][kl];
	       double factor = pow(hilf,(double)net.bede[kn])/(fac(net.bede[kn]) *
				     (1 - lgr.rho[kn][kl]));
	       double zustw0 = 0; /*----Wahrscheinlichkeit,dass 0 Auftraege
				          im i-ten Knoten der r-ten Klasse sind--*/
	    
	       /*--------Berechnug von 'zustw0'-------------------------*/
	       for (i=0 ; i < net.bede[kn] ; i++)
		   zustw0 += (pow(hilf,(double)i) / fac(i));
	       zustw0 += factor;
	       zustw0 = 1/zustw0;
             /*--------Berechnung der Leistungsgroessen---------------*/
	       lgr.mqlen[kn][kl] = zustw0 * factor * lgr.rho[kn][kl] / 
					   (1 - lgr.rho[kn][kl]);
			if (lgr.lambda[kn][kl] != 0.0)
			{
			   lgr.wzeit[kn][kl] = lgr.mqlen[kn][kl] / lgr.lambda[kn][kl];
			   lgr.vzeit[kn][kl] = lgr.wzeit[kn][kl] + 1 / net.bedr[kn][kl];
			}
			else
			{
				lgr.vzeit[kn][kl] = 0.0;
				lgr.wzeit[kn][kl] = 0.0;
			}
	       lgr.mauft[kn][kl] = hilf + lgr.mqlen[kn][kl];
	    }
	    else switch(option[0])           /* asymmetrischer Knoten */
	    {
		 case 'a': /* Scheuerer */
			     {
				  register int i;
				  double sumamue = 0;

				  for (i = 0; i < net.bede[kn]; i++)
				     sumamue += net.amue[kn][i];

				  lgr.wzeit[kn][kl] = p_wart_sch(lgr.rho[kn][kl],kn)/
							    ((1-lgr.rho[kn][kl])*sumamue);    
				  lgr.mqlen[kn][kl] = lgr.lambda[kn][kl] * lgr.wzeit[kn][kl];
				  lgr.vzeit[kn][kl] = lgr.wzeit[kn][kl] + (net.bede[kn]/sumamue);
				  lgr.mauft[kn][kl] = net.bede[kn] * lgr.rho[kn][kl] + lgr.mqlen[kn][kl];
                       }
			     break;
		 case 'b': /* Baer/Fischer/Hertel, zufaellige Auswahl */
			     {
				  register int i;
				  double sumamue = 0;

				  for (i = 0; i < net.bede[kn]; i++)
				     sumamue += net.amue[kn][i];
				  lgr.wzeit[kn][kl] = P_w/
							    ((1-lgr.rho[kn][kl])* sumamue);
				  lgr.mqlen[kn][kl] = q_quer(lgr.rho[kn][kl],kn,0);
				  lgr.vzeit[kn][kl] = lgr.wzeit[kn][kl] + (net.bede[kn]/sumamue);
				  lgr.mauft[kn][kl] = net.bede[kn] * lgr.rho[kn][kl] + lgr.mqlen[kn][kl];
			     }
			     break;
		 case 'c': /* Baer/Fischer/Hertel, Auswahl der schnellsten BE */
			     {
				  register int i;
				  double sumamue = 0;

				  for (i = 0; i < net.bede[kn]; i++)
				     sumamue += net.amue[kn][i];

				  lgr.wzeit[kn][kl] = P_w/
							   ((1-lgr.rho[kn][kl])* sumamue);
				  lgr.mqlen[kn][kl] = q_quer(lgr.rho[kn][kl],kn,1);
				  lgr.vzeit[kn][kl] = lgr.wzeit[kn][kl] + 
							    (net.bede[kn]/sumamue);
				  lgr.mauft[kn][kl] = net.bede[kn] * lgr.rho[kn][kl] +
							    lgr.mqlen[kn][kl];
                       }
			     break;
	       default : /*  Falsches Verfahren angegeben  */
			     /*      Speicher freigeben        */
			     /* ------------------------------ */
			     {
				  FREE_LGR(net.knot);
				  printf("sopenpf_option\n 0 0 0\n");
				  exit(1);
			     }
	    }
	  }
      }
}
/*
**********************************************************************
*                     Funktion BER_NETZ_LGR                          *
**********************************************************************
Berechnung der Netz - Leistungsgroessen ( Klassengroessen)
*/
void BER_NETZ_LGR ()
{
      for ( kl=0; kl < net.klass; kl++) {
            kls.typ   [kl] = 'o';
            kls.lambda[kl] = net.ankr[kl];
            kls.mauft [kl] = 0;
            kls.mqlen [kl] = 0;
            for ( kn=1; kn <= net.knot; kn++) {
                  kls.mauft[kl] += lgr.mauft[kn][kl];
                  kls.mqlen[kl] += lgr.mqlen[kn][kl];
            }
			if (kls.lambda[kl] != 0.0)
			{
				kls.vzeit [kl] = kls.mauft[kl] / kls.lambda[kl];
				kls.wzeit [kl] = kls.mqlen[kl] / kls.lambda[kl];
			}
			else
			{
				kls.vzeit [kl] = 0.0;
				kls.wzeit [kl] = 0.0;
			}
      }
}
/*
**********************************************************************
*                        Funktion INP_NETZ                           *
**********************************************************************
Netzgroessen von stdin einlesen und Speicherplatz bereitstellen
*/
void INP_NETZ ()
{
      int  j, m, n, r;
	double help;
	int  max_bed = 0;       /* maximale Anzahl der Bedieneinheiten */

   /* Uewflag, Knotenzahl, Klassenzahl */
      JUMPCOM ( stdin);
      scanf   ( "%d %d %d", &uewflag, &net.knot, &net.klass);
      if (net.klass > 1)
      {
	fprintf(stderr,"'sopenpfn' nur auf Netze mit einer Klasse anwendbar\n");
	printf("sopenpfn_klassen\n0 0 0 \n");
	exit(1);
      }

   /* Knotentypen */                /* K.R. 07.04.92 */
	JUMPCOM (stdin);
	for (n = 1; n <= net.knot; n++)
	   scanf("%d", &net.knottyp[n]);

   /* Klassen - Ankunftsraten */
      JUMPCOM ( stdin);
      for ( r=0; r < net.klass; r++) 
            scanf ( "%lf", &net.ankr[r]);
   /* Bedieneinheiten */
      JUMPCOM ( stdin);
      for ( n=1; n <= net.knot; n++)
            scanf ( "%d", &net.bede[n]);
   /* Einzelgroessen */
      net.bedr = NR_ALLOC ( net.knot, net.klass);
      net.besh = NR_ALLOC ( net.knot, net.klass);
      if  ( uewflag == 1)
            net.uew = NNR_ALLOC ( net.knot, net.klass);

      for ( r=0; r < net.klass; r++) {
         /* Bedienraten */
            JUMPCOM ( stdin);
            for ( n=1; n <= net.knot; n++)
                  scanf ( "%lf", &net.bedr[n][r]);
            JUMPCOM ( stdin);
         /* Uebergangswahrscheinlichkeiten ... */
            if  ( uewflag == 1)
                  for ( m=0; m <= net.knot; m++)
                        for ( n=0; n <= net.knot; n++)
                              scanf ( "%f", &net.uew[m][n][r]);
         /* ... oder Besuchshaeufigkeiten*/
            else
                  for ( n=1; n <= net.knot; n++)
                        scanf ( "%lf", &net.besh[n][r]);
      }

    /* bei asymmetrischen Knoten: Einlesen der Bedienraten amue */
    /* pro Knoten und Bedieneinheit und                         */
    /* Bedienraten absteigend sortieren (wichtig fuer Auswahl   */
    /* der schnellsten Bedieneinheit)                           */

	 for (n = 0; n < net.knot; n++)
	    if (max_bed < net.bede[n]) max_bed = net.bede[n];
	 net.amue = NR_ALLOC(net.knot, max_bed+2);


	 for (n = 1; n <= net.knot; n++)
	 {
	    JUMPCOM(stdin);
	    if (net.knottyp[n] == 20)
	    {
		 for (r = 0; r < net.bede[n]; r++)
		    scanf("%lf", &net.amue[n][r]);
		 qsort(net.amue[n],net.bede[n],sizeof(double),strcmp);
		 j = net.bede[n]-1;
		 while (j > ((net.bede[n]-1)/2))
		 {
		    help = net.amue[n][net.bede[n]-j-1];
		    net.amue[n][net.bede[n]-j-1] = net.amue[n][j];
		    net.amue[n][j] = help;
		    j--;
		 }
	    }
       }
}
/*    
********************* Ende des Moduls sopenpfn.c ************************/

