/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

/*
 #####   #####      #     ####   #    #  #    #    ##      ####
 #    #  #    #     #    #    #  ##  ##  #    #   #  #    #    #
 #    #  #    #     #    #    #  # ## #  #    #  #    #   #
 #####   #####      #    #    #  #    #  #    #  ######   #
 #       #   #      #    #    #  #    #   #  #   #    #   #    #
 #       #    #     #     ####   #    #    ##    #    #    ####

 fuer geschlossene Netze

 programmiert von Otto Niesser
*/

static char *SccsId = "@(#)PEPSY:\tpriomva2c.c\t1.2\t92/03/27 11:52:01";

#include <stdio.h>

#define BOOLEAN int
#define TRUE 1
#define FALSE 0

#define AND &&
#define OR ||
#define NOT ! 

/*        Festlegung des Gleitkommaformats                              */
/*        bei float : REAL_FORM = "%9.5f "                              */
/*                    REAL_READ = "%f"                                  */
/*                                                                      */
/*        bei double: REAL_FORM = "%9.5lf "                             */
/*                    REAL_READ = "%lf"                                 */

#define REAL double
#define REAL_FORM "%9.5lf "
#define REAL_READ "%lf"

/*        Makro zum Einlesen aus stdio mit Fehlerbehandlung             */

#define LIES(A,B)       scanf(A,&B);                                    \
                      		if((REAL)B < 0.0)                               \
                      		{                                               \
                        		f_eingabe = TRUE;                             \
                        		fehler = TRUE;                                \
                        		return;                                       \
                      		}                

/*        Makro zur Speicherplatzbelegung mit Erfolgsueberpruefg        */

#define ALLOCATE(P,S,T) P=(T*)calloc((unsigned)(S),sizeof(T));          \
                      		if(P == NULL)                                   \
                      		{                                               \
                        		f_speicher = TRUE;                            \
                        		fehler = TRUE;                                \
                        		return;                                       \
                      		}                                               \
                      		else                                            \
                        		mem += (S) * sizeof(T);

/*        Nachkommaanteil                                               */

#define FRAC(X)         ( X - (REAL)((int)X) )

/*        Abbildung 2-dim Index -> 1-dim Index                          */

#define IND2(I,J) ((J) * M + I)

/*        Globale Variablen                                             */

struct  pp 
      		{ REAL *ptr; };    /* fuer p                                    */

int     *N,                /* Populationsvektor                         */
      		*n,                /* laeuft ueber alle Populationen            */
      		M,                 /* Anzahl der Knoten                         */
      		*kn_typ,           /* Knotentyp                                 */
      		*anz_bs,           /* Anzahl Bedienstationen bei M/M/k          */
      		R,                 /* Anzahl der Klassen                        */
      		typ,               /* Netztyp                                   */
      		*prod_v,           /* Produkt(N[i]+1)                           */
      		ue_w;              /* Flag fuer Uebergangsw'keiten              */
BOOLEAN prio;              /* Prioritaetsknoten im Netz ?               */
struct pp *p;              /* Zustandsw'keit bei Ber. von M/M/k         */
REAL    *mue,              /* Bedienrate                                */
      		*theta,            /* Besuchshaeufigkeiten                      */
      		*W,                /* Mittlere Verweilzeit                      */
      		*T,                /* Durchsatz                                 */
      		*L,                /* Mittlere Auftragszahl                     */
      		*rho,              /* Auslastung                                */
      		*rho_str;          /* Rho Strich                                */
BOOLEAN f_eingabe=FALSE,   /* Eingabefehler                             */
      		f_speicher=FALSE,  /* kein Speicher mehr frei                   */
      		f_uew=FALSE,       /* Uebergangsw'keiten nicht mehr unterstuetzt*/
      		fehler=FALSE;      /* allgemeiner Fehler                        */

char    *calloc();

int     mem = 0;           /* benoetigter Speicherplatz                 */

/*        Schleife ueber alle Populationen                              */

pop_loop()
{
	int i,uebertrag;

	uebertrag = 0;
	for (i = 0; i < R; i++)
		n[i] = 0;
	do
	{
		if (NOT fehler)
    		verfahren();

		n[0]++;
		for (i = 0; i < R; i++)
		{
    		n[i] += uebertrag;
    		if (n[i] > N[i])
    		{
      		n[i] = 0;
      		uebertrag = 1;
    		}
    		else
      		uebertrag = 0;
		}
	}while (uebertrag == 0);
}

/*        Initialisierung des Verfahrens                                */
/*        Vorbesetzung von Lij(0), Tij, rho ij und p                    */

verf_init()
{
	int i,j;

	for (j = 0; j < R; j++)
		n[j] = 0; 

	for (i = 0; i < M; i++)
	{
		for (j = 0; j < R; j++)
		{
    		L[ind(i,j,n)] = 0.0;
    		rho_str[IND2(i,j)] = 0.0;
		}
		if (kn_typ[i] == 4)
		{ 
    		for (j = 1; j < anz_bs[i]; j++)
      		p[i].ptr[ind_p(i,j,n)] = 0.0;
    		p[i].ptr[ind_p(i,0,n)] = 1.0;
		}
	}
}

/*        Berechnungsschritt des Mittelwertalgorithmus                  */
/*        Ermittlung der rho' nach Chandy                               */

verfahren()
{
	int i,j,k;
	REAL s1,s2,s3,s4;

	/*        a) Berechnung der Wij(n) in den geschl. Klassen             */

	rho_str_ber(TRUE);

	for (i = 0; i < M; i++)
	{
		for (j = 0; j < R; j++)
		{
      		switch(kn_typ[i])
      		{
        		/******/
        		/* IS */
        		/******/
        		case 0: if (n[j] == 0)
                		{ W[ind(i,j,n)] = 0; }
                		else
                		{ W[ind(i,j,n)] = 1/mue[IND2(i,j)]; }
                		break;

        		/********************/
        		/* FCFS / PS / LCFS */
        		/********************/
        		case 1: if(n[j] == 0)
                		{ W[ind(i,j,n)] = 0.0;}
                		else
                		{
                  		n[j]--;
                  		s1 = 0.0;
                  		s2 = 0.0;
                  		for (k = 0; k < R; k++)
                    		s1 += L[ind(i,k,n)];
                  		n[j]++;
                  		W[ind(i,j,n)] = 
                    		(1+s1)/(mue[IND2(i,j)]*(1-s2));
                		}
                		break;

        		/*******/
        		/* PRE */
        		/*******/
        		case 2: if (n[j] == 0)
                		{ W[ind(i,j,n)] = 0; }
                		else
                		{
                  		n[j]--;
                  		s1 = 0.0;
                  		s2 = 0.0;
                  		for (k = 0; k <= j; k++)
                  		{ s1 += L[ind(i,k,n)] / mue[IND2(i,k)]; }
                  		for (k = 0; k <= j-1; k++)
                  		{ s2 += rho_str[IND2(i,k)]; }
                  		n[j]++;
                  		W[ind(i,j,n)] = (1/mue[IND2(i,j)]+s1)/(1-s2);
                		}
                		break;

        		/**********/
        		/* NONPRE */
        		/**********/
        		case 3 :if (n[j] == 0)
                		{ W[ind(i,j,n)] = 0;}
                		else
                		{
                  		n[j]--;
                  		s1 = 0.0;
                  		s2 = 0.0;
                  		s3 = 0.0;
                  		for (k = 0; k <= j; k++)
                  		{ s1 += L[ind(i,k,n)] / mue[IND2(i,k)]; }
                  		for(k = 0; k <= j-1; k++)
                  		{ s2 += rho_str[IND2(i,k)]; }
                  		for (k = j+1; k < R; k++)
                  		{ s3 += rho[ind(i,k,n)] / mue[IND2(i,k)]; }
                  		n[j]++;
                  		W[ind(i,j,n)] = 
                    		1/mue[IND2(i,j)]+(s1+s3)/(1-s2);
                		}
                		break;
          
        		/**************/
        		/* M/M/k-FCFS */
        		/**************/
        		case 4 :if (n[j] == 0)
                		{ W[ind(i,j,n)] = 0; }
                		else
                		{
                  		n[j]--;
                  		s1 = 0.0;
                  		s2 = 0.0;
                  		for (k = 0; k < R; k++)
                  		{ s1 += L[ind(i,k,n)]; }
                  		for (k = 0; k < anz_bs[i] - 1; k++)
                  		{ s2 += (anz_bs[i]-k-1) * p[i].ptr[ind_p(i,k,n)]; }
                  		n[j]++;
                  		W[ind(i,j,n)] = 1/(mue[IND2(i,j)] * anz_bs[i]) *
                                  		(1 + s1 + s2);
                		}
                		break;
      		}
		}
	}
  
	/*        b) Berechnung der Tij(n), Lij(n) und rho in C               */
	/*           Danach Berechnung der p fuer M/M/k-Knoten                */

	for (i = 0; i< M; i++)
	{
		for (j = 0; j < R; j++)
		{
      		if (n[j] == 0)
      		{ T[ind(i,j,n)] = 0.0; }
      		else
      		{
        		s1 = 0.0;
        		for (k = 0; k < M; k++)
        		{ s1 += theta[IND2(k,j)] * W[ind(k,j,n)]; }
        		T[ind(i,j,n)] = theta[IND2(i,j)] * n[j] / s1; 
      		}
      		L[ind(i,j,n)] = T[ind(i,j,n)] * W[ind(i,j,n)];
      		if (kn_typ[i] == 4)
        		rho[ind(i,j,n)] = T[ind(i,j,n)] / (mue[IND2(i,j)]*anz_bs[i]);
      		else
        		rho[ind(i,j,n)] = T[ind(i,j,n)] / mue[IND2(i,j)];
		}
	}

	for (i = 0; i < M; i++)
	{
		if (kn_typ[i] == 4)
		{
    		for (j = 1; j < anz_bs[i]; j++)
    		{
      		s1 = 0;
      		for (k = 0; k < R; k++)
      		{
        		if (n[k] > 0)
        		{
          		s2 = T[ind(i,k,n)]/mue[IND2(i,k)];
          		n[k]--;
          		s1 += s2 * p[i].ptr[ind_p(i,j-1,n)];
          		n[k]++;
        		}
      		}
      		p[i].ptr[ind_p(i,j,n)] = s1 / j;
    		}

    		s1 = 0;
    		for (k = 0; k < R; k++)
    		{ s1 += T[ind(i,k,n)]/mue[IND2(i,k)]; }

    		s2 = 0;
    		for (j = 1; j < anz_bs[i]; j++)
    		{ s2 += (anz_bs[i] - j) * p[i].ptr[ind_p(i,j,n)]; }

    		p[i].ptr[ind_p(i,0,n)] = 1 - (s1+s2)/anz_bs[i];
		}
	}

}

/*        Berechnung von rho'                                          */

rho_str_ber(minus)
BOOLEAN minus;
{
	REAL hilf;
	REAL x1,x2,r1,r2;
	int i,j;

	if (NOT prio) return;

	for (i = 0; i < M; i++)
		for (j = 0; j < R; j++)
      		{
        		if (minus)
        		{
          		n[j]--;
          		hilf = L[ind(i,j,n)];
          		n[j]++;
        		}
        		else
          		hilf = L[ind(i,j,n)];

        		n[j] -= (int)hilf;
        		r1 = rho[ind(i,j,n)];
        		if ( ( ((int)hilf > 0) AND (NOT minus) ) OR ((int)hilf > 1) )
         		{
          		n[j]++;
          		r2 = rho[ind(i,j,n)];
          		n[j]--;
        		}
        		else
          		r2 = r1;
        		n[j] += (int)hilf;

        		rho_str[IND2(i,j)] = (1-FRAC(hilf))*r1 + FRAC(hilf)*r2;
      		}
}

/*        Ausgabe                                                      */

aus()
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

	int i,j,kk;
	REAL rho_sum;
	BOOLEAN w_rho;

	w_rho = FALSE;
  
	printf("# Verfahrensname \n");
	printf("  priomva2c \n");
	printf("# Version %d.%d \n\n",SccsRelease, SccsLevel);
	printf("# Speicherplatzbedarf: %d Bytes \n\n",mem);

	printf("# Netztyp Knoten Klassen \n");
	printf("  %d       %d      %d \n\n",typ,M,R);

	printf("# fuer alle Klassen \n");
	printf("#    fuer alle Knoten \n");
	printf("# lambda    e         1/mue     rho       mvz       ");
	printf("maa       mwz       mwsl \n");
	for (j = 0; j < R; j++)
	{
		for (i = 0; i < M; i++)
		{
    		printf(REAL_FORM,T[ind(i,j,N)]);
    		printf(REAL_FORM,theta[IND2(i,j)]); 
    		printf(REAL_FORM,1/mue[IND2(i,j)]);
      
    		if (theta[IND2(i,j)] > 0.0)
    		{
      		if (kn_typ[i] == 0)
      		{ printf("%9.5f ",0.0); }
      		else
        		printf(REAL_FORM,rho[ind(i,j,N)]); 
 
      		printf(REAL_FORM,W[ind(i,j,N)]);
      		printf(REAL_FORM,L[ind(i,j,N)]);

      		if (kn_typ[i] == 0)
      		{ printf("%9.5f %9.5f ",0.0,0.0); }
      		else
      		{
        		printf(REAL_FORM,W[ind(i,j,N)] - 1/mue[IND2(i,j)]);
        		printf(REAL_FORM,T[ind(i,j,N)] * (W[ind(i,j,N)] - 
          		1/mue[IND2(i,j)])); 
      		}
    		}
    		else
      		for (kk = 0; kk < 5; kk++)
        		printf("%9.5f ",0.0);
    		printf("\n");
		}
		printf("\n");
	}
	for (i = 0; i < M; i++)
	{
		if ((kn_typ[i] == 2) OR (kn_typ[i] == 3))
		{
    		rho_sum = 0.0;
    		for (j = 0; j < R; j++)
      		rho_sum += rho[ind(i,j,N)];
    		if (rho_sum > 0.7)
      		w_rho = TRUE;
		}
	}
	if (w_rho)
	{
		fprintf(stderr,
    		"Die Auslastung in einem/mehreren Prio'knoten ist zu hoch.\n");
		fprintf(stderr,"Die Ergebnisse koennten falsch sein.\n");
	}
}

/*        Ausgabe im Fehlerfall                                        */

fehler_aus()
{
	if (f_eingabe)
	{ 
		fprintf(stderr,"fehlerhafte Eingabedatei \n"); 
		printf("priomva2c_eingabe\n0 0 0 \n");
	}

	if (f_speicher)
	{ 
		fprintf(stderr,"nicht genug Speicher vorhanden \n");
		printf("priomva2c_speicher\n0 0 0 \n");
	}

	if (f_uew)
	{ 
		fprintf(stderr,
          		"Uebergangsw'keiten ab v0.24 nicht mehr unterstuetzt \n");
		printf("priomva2c_uew\n0 0 0 \n");
	} 
}

/*        Initialisierung der Variablen                                */

init()
{
	int i,j,k;
	int prod;
	int MR;

	LIES("%d",M);
	LIES("%d",R);

	if ((M == 0) OR (R == 0))
	{
		fehler = TRUE;
		f_eingabe = TRUE;
		return;
	}

	MR = M * R;
	ALLOCATE(n,R,int);
	ALLOCATE(N,R,int);
	ALLOCATE(kn_typ,M,int);
	ALLOCATE(anz_bs,M,int);
	ALLOCATE(p,M,struct pp);
	ALLOCATE(mue,MR,REAL);
	ALLOCATE(theta,MR,REAL);
	ALLOCATE(prod_v,R+1,int);

	prio = FALSE;

	for (i = 0; i < M; i++)
	{ 
		LIES("%d",kn_typ[i]);
		if (kn_typ[i] == 4) /* M/M/k */
		{ LIES("%d",anz_bs[i]); }

		if ((kn_typ[i] == 2) OR (kn_typ[i] == 3))
    		prio = TRUE;
	}

	for (j = 0; j < R; j++)
	{
    		LIES(" %*d %d",N[j]); 
	}

	typ = 1; 
  
	LIES("%d",ue_w);
  
	if (NOT ue_w)
		for (j = 0; j < R; j++)
		{
    		for (i = 0; i < M; i++)
    		{
      		LIES(REAL_READ,mue[IND2(i,j)]);
      		LIES(REAL_READ,theta[IND2(i,j)]);
    		}
		}
	else
	{
		fehler = TRUE;
		f_uew = TRUE;
		return;
	}

	prod_v[0] = 1;
	for (j = 0; j < R; j++)
	{ prod_v[j+1] = prod_v[j] * (N[j] + 1); }

	prod = MR * prod_v[R];

	ALLOCATE(W,prod,REAL);
	ALLOCATE(T,prod,REAL);
	ALLOCATE(L,prod,REAL);
	ALLOCATE(rho,prod,REAL);
	ALLOCATE(rho_str,MR,REAL);

	for (i = 0; i < M; i++)
	{ 
		if (kn_typ[i] == 4) /* M/M/k */
		{ ALLOCATE(p[i].ptr,prod_v[R] * anz_bs[i],REAL); }
		else
    		p[i].ptr = NULL; 
	}

	verf_init();
}

/*        Abbildung R+2-dim Index -> 1-dim Index                       */

ind(i,j,n)
int i,j,*n;
{
	int k,sum;

	sum = 0;
	for (k = 0; k < R; k++)
	{ sum += (n[k] * prod_v[k]); } 
	sum = (sum * R + j) * M + i;
	return(sum);
}

ind_p(i,j,n)
int i,j,*n;
{
	int k,sum;

	sum = 0;
	for (k = 0; k < R; k++)
	{ sum += (n[k] * prod_v[k]); }
	sum = sum * anz_bs[i] + j;
	return(sum);
}

/*         Hauptprogramm                                               */

void abbruch(sig)
int sig;
{
      		printf("priomva2c_signal_%d\n 0 0 0 \n", sig);
      		exit(1);
}

main(argc, argv)
	int argc;
	char *argv[];
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

	int i;
	void abbruch();

	for (i=1; i<12; i++)
			signal (i, abbruch);

	if ((argc > 1) && (!strcmp(argv[1], "-V")))    /* kirschni 22.11.1991 */
	{
		fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
		exit(0);
	}


	init();

	if (NOT fehler)
	{
		pop_loop();
	}

	if (NOT fehler)
	{
		aus();
		i=0;
	}
	else
	{
		fehler_aus();
		i=1;
	}

	exit(i);
}
