/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

/*
 #####   #####      #     ####   #    #  #    #    ##      #    #
 #    #  #    #     #    #    #  ##  ##  #    #   #  #     ##  ##
 #    #  #    #     #    #    #  # ## #  #    #  #    #    # ## #
 #####   #####      #    #    #  #    #  #    #  ######    #    #
 #       #   #      #    #    #  #    #   #  #   #    #    #    #
 #       #    #     #     ####   #    #    ##    #    #    #    #

 fuer gemischte Netze

 programmiert von Otto Niesser
*/

static char *SccsId = "@(#)PEPSY:\tpriomva2m.c\t1.2\t92/03/27 11:52:11";

#include <stdio.h>

#define BOOLEAN int
#define TRUE 1
#define FALSE 0

#define AND &&
#define OR ||
#define NOT ! 

/*        Festlegung des Gleitkommaformats                              */
/*        bei float : REAL_FORM = "%9.5f "                              */
/*                    REAL_READ = "%f"                                  */
/*                                                                      */
/*        bei double: REAL_FORM = "%9.5lf "                             */
/*                    REAL_READ = "%lf"                                 */

#define REAL double
#define REAL_FORM "%9.5lf "
#define REAL_READ "%lf"

/*        Makro zum Einlesen aus stdio mit Fehlerbehandlung             */

#define LIES(A,B)       scanf(A,&B);                                    \
                        if((REAL)B < 0.0)                               \
                        {                                               \
                          f_eingabe = TRUE;                             \
                          fehler = TRUE;                                \
                          return;                                       \
                        }                

/*        Makro zur Speicherplatzbelegung mit Erfolgsueberpruefg        */

#define ALLOCATE(P,S,T) P=(T*)calloc((unsigned)(S),sizeof(T));          \
                        if(P == NULL)                                   \
                        {                                               \
                          f_speicher = TRUE;                            \
                          fehler = TRUE;                                \
                          return;                                       \
                        }                                               \
                        else                                            \
                          mem += (S) * sizeof(T);

/*        Nachkommaanteil                                               */

#define FRAC(X)         ( X - (REAL)((int)X) )

/*        Abbildung 2-dim Index -> 1-dim Index                         */

#define IND2(I,J) ((J) * M + I)

/*        Globale Variablen                                             */

int     *N,                /* Populationsvektor                         */
        *n,                /* laeuft ueber alle Populationen            */
        M,                 /* Anzahl der Knoten                         */
        *kn_typ,           /* Knotentyp                                 */
        R,                 /* Anzahl der Klassen                        */
        typ,               /* Netztyp                                   */
        *prod_v,           /* Produkt(N[i]+1)                           */
        ue_w;              /* Flag fuer Uebergangsw'keiten              */
BOOLEAN *offen,            /* k-te Klasse offen ?                       */
        op,                /* Offene Klasse im Netz ?                   */
        prio;              /* Prioritaetsknoten im Netz ?               */
REAL    *lambda,           /* Ankunftsrate offene Klassen               */
        *mue,              /* Bedienrate                                */
        *theta,            /* Besuchshaeufigkeiten                      */
        *W,                /* Mittlere Verweilzeit                      */
        *T,                /* Durchsatz                                 */
        *L,                /* Mittlere Auftragszahl                     */
        *rho,              /* Auslastung                                */
        *rho_o,            /* Auslastung offene                         */
        *rho_str,          /* Rho Strich                                */
        *T_o;              /* Durchsatz offene                          */
BOOLEAN f_stabil=FALSE,    /* Stabilitaetsbedingung verletzt ?          */
        f_eingabe=FALSE,   /* Eingabefehler                             */
        f_speicher=FALSE,  /* kein Speicher mehr frei                   */
        f_uew=FALSE,       /* Uebergangsw'keiten nicht mehr unterstuetzt*/
        fehler=FALSE;      /* allgemeiner Fehler                        */

char    *calloc();

int     mem = 0;           /* benoetigter Speicherplatz                 */

/*        Schleife ueber alle Populationen                              */

pop_loop()
{
  int i,uebertrag;

  uebertrag = 0;
  for (i = 0; i < R; i++)
    n[i] = 0;
  do
  {
    if (NOT fehler)
      verfahren();

    n[0]++;
    for (i = 0; i < R; i++)
    {
      n[i] += uebertrag;
      if (n[i] > N[i])
      {
        n[i] = 0;
        uebertrag = 1;
      }
      else
        uebertrag = 0;
    }
  }while (uebertrag == 0);
}

/*        Initialisierung des Verfahrens                                */
/*        Vorbesetzung von Lij(0), Tij und rho ij                       */

verf_init()
{
  int i,j;

  for (j = 0; j < R; j++)
    n[j] = 0; 

  for (i = 0; i < M; i++)
  {
    for (j = 0; j < R; j++)
    {
      L[ind(i,j,n)] = 0.0;
      if(offen[j])
      {
        T_o[IND2(i,j)] = theta[IND2(i,j)] * lambda[j];
        rho_o[IND2(i,j)] = T_o[IND2(i,j)] / mue[IND2(i,j)];
        rho_str[IND2(i,j)] = rho_o[IND2(i,j)];
        if (rho_o[IND2(i,j)] >= 1 && kn_typ[i] != 0)
        { 
          fehler = TRUE;
          f_stabil = TRUE;
        }
      }
      else
      {
        T_o[IND2(i,j)] = 0.0;
        rho_o[IND2(i,j)] = 0.0;
        rho_str[IND2(i,j)] = 0.0;
      }
    }
  }
}

/*        Berechnungsschritt des Mittelwertalgorithmus                  */
/*        Ermittlung der rho' nach Chandy                               */

verfahren()
{
  int i,j,k;
  REAL s1,s2,s3,s4;

  /*        a) Berechnung der Wij(n) in den geschl. Klassen             */

  rho_str_ber(TRUE);

  for (i = 0; i < M; i++)
  {
    for (j = 0; j < R; j++)
    {
      if(NOT offen[j])
      {
        switch(kn_typ[i])
        {
          /******/
          /* IS */
          /******/
          case 0: if (n[j] == 0)
                  { W[ind(i,j,n)] = 0; }
                  else
                  { W[ind(i,j,n)] = 1/mue[IND2(i,j)]; }
                  break;

          /********************/
          /* FCFS / PS / LCFS */
          /********************/
          case 1: if(n[j] == 0)
                  { W[ind(i,j,n)] = 0.0;}
                  else
                  {
                    n[j]--;
                    s1 = 0.0;
                    s2 = 0.0;
                    for (k = 0; k < R; k++)
                    {
                      if (NOT offen[k])
                      { s1 += L[ind(i,k,n)]; }
                      else
                      { s2 += rho_o[IND2(i,k)]; }
                    }
                    n[j]++;
                    W[ind(i,j,n)] = 
                      (1+s1)/(mue[IND2(i,j)]*(1-s2));
                  }
                  break;

          /*******/
          /* PRE */
          /*******/
          case 2: if (n[j] == 0)
                  { W[ind(i,j,n)] = 0; }
                  else
                  {
                    n[j]--;
                    s1 = 0.0;
                    s2 = 0.0;
                    for (k = 0; k <= j; k++)
                    { s1 += L[ind(i,k,n)] / mue[IND2(i,k)]; }
                    for (k = 0; k <= j-1; k++)
                    { s2 += rho_str[IND2(i,k)]; }
                    n[j]++;
                    W[ind(i,j,n)] = (1/mue[IND2(i,j)]+s1)/(1-s2);
                  }
                  break;

          /**********/
          /* NONPRE */
          /**********/
          case 3 :if (n[j] == 0)
                  { W[ind(i,j,n)] = 0;}
                  else
                  {
                    n[j]--;
                    s1 = 0.0;
                    s2 = 0.0;
                    s3 = 0.0;
                    for (k = 0; k <= j; k++)
                    { s1 += L[ind(i,k,n)] / mue[IND2(i,k)]; }
                    for(k = 0; k <= j-1; k++)
                    { s2 += rho_str[IND2(i,k)]; }
                    for (k = j+1; k < R; k++)
                    { 
                      if (offen[k])
                        s3 += rho_o[IND2(i,k)] / mue[IND2(i,k)];
                      else
                        s3 += rho[ind(i,k,n)] / mue[IND2(i,k)];
                    }
                    n[j]++;
                    W[ind(i,j,n)] = 
                      1/mue[IND2(i,j)]+(s1+s3)/(1-s2);
                  }
                  break;
        }
      }
    }
  }
  
  /*        b) Berechnung der Tij(n), Lij(n) und rho in C               */

  for (i = 0; i< M; i++)
  {
    for (j = 0; j < R; j++)
    {
      if (NOT offen[j])
      {
        if (n[j] == 0)
        { T[ind(i,j,n)] = 0.0; }
        else
        {
          s1 = 0.0;
          for (k = 0; k < M; k++)
          { s1 += theta[IND2(k,j)] * W[ind(k,j,n)]; }
          T[ind(i,j,n)] = theta[IND2(i,j)] * n[j] / s1; 
        }
        L[ind(i,j,n)] = T[ind(i,j,n)] * W[ind(i,j,n)];
        rho[ind(i,j,n)] = T[ind(i,j,n)] / mue[IND2(i,j)];
      }
    }
  }

  /*        c) Berechnung der Wij(n) in den offenen Klassen             */

  rho_str_ber(FALSE);

  if (NOT op) return;

  for (i = 0; i < M; i++)
  {
    for (j = 0; j < R; j++)
    {
      if(offen[j])
      {
        switch(kn_typ[i])
        {
          /******/
          /* IS */
          /******/
          case 0: W[ind(i,j,n)] = 1/mue[IND2(i,j)]; 
                  break;

          /********************/
          /* FCFS / PS / LCFS */
          /********************/
          case 1: s1 = 0.0;
                  s2 = 0.0;
                  for (k = 0; k < R; k++)
                  {
                    if (NOT offen[k])
                    { s1 += L[ind(i,k,n)]; }
                    else
                    { s2 += rho_o[IND2(i,k)]; }
                  }
                  W[ind(i,j,n)] = (1+s1)/(mue[IND2(i,j)]*(1-s2));
                  break;

          /*******/
          /* PRE */
          /*******/
          case 2: s1 = 0.0;
                  s2 = 0.0;
                  for (k = 0; k <= j-1; k++)
                  { s1 += L[ind(i,k,n)] / mue[IND2(i,k)]; }
                  for (k = 0; k <= j; k++)
                  { s2 += rho_str[IND2(i,k)]; }
                  W[ind(i,j,n)] = (1/mue[IND2(i,j)]+s1)/(1-s2);
                  break;

          /**********/
          /* NONPRE */
          /**********/
          case 3: s1 = 0.0;
                  s2 = 0.0;
                  s3 = 0.0;
                  s4 = 0.0;
                  for (k = 0; k <= j-1; k++)
                  { s1 += L[ind(i,k,n)] / mue[IND2(i,k)]; }
                  for(k = 0; k <= j; k++)
                  { s2 += rho_str[IND2(i,k)]; }
                  for (k = j+1; k < R; k++)
                  { 
                    if (offen[k])
                      s3 += rho_o[IND2(i,k)] / mue[IND2(i,k)];
                    else
                      s3 += rho[ind(i,k,n)] / mue[IND2(i,k)];
                  }
                  for (k = 0; k <= j-1; k++)
                  { s4 += rho_str[IND2(i,k)]; }
                  W[ind(i,j,n)] = 
                    ((1-s4)/mue[IND2(i,j)]+s1+s3)/(1-s2);
                  break;
        }

        L[ind(i,j,n)] = T_o[IND2(i,j)] * W[ind(i,j,n)]; 
      }
    }
  }
}

/*        Berechnung von rho'                                          */

rho_str_ber(minus)
BOOLEAN minus;
{
  REAL hilf;
  REAL x1,x2,r1,r2;
  int i,j;

  if (NOT prio) return;

  for (i = 0; i < M; i++)
    for (j = 0; j < R; j++)
      if (NOT offen[j])
        if (n[j] == 0)
        { rho_str[IND2(i,j)] = 0.0; }
        else
        {
          if (minus)
          {
            n[j]--;
            hilf = L[ind(i,j,n)];
            n[j]++;
          }
          else
            hilf = L[ind(i,j,n)];

          n[j] -= (int)hilf;
          r1 = rho[ind(i,j,n)];
          if ( ( ((int)hilf > 0) AND (NOT minus) ) OR ((int)hilf > 1) )
           {
            n[j]++;
            r2 = rho[ind(i,j,n)];
            n[j]--;
          }
          else
            r2 = r1;
          n[j] += (int)hilf;

          rho_str[IND2(i,j)] = (1-FRAC(hilf))*r1 + FRAC(hilf)*r2;
        }
}

/*        Ausgabe                                                      */

aus(N)
int *N;
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

  int i,j,kk;
  REAL rho_sum;
  BOOLEAN w_rho;

  w_rho = FALSE;
 
  printf("# Verfahrensname \n");
  printf("  priomva2m \n");
  printf("# Version %d.%d \n\n", SccsRelease, SccsLevel);
  printf("# Speicherplatzbedarf: %d Bytes \n\n",mem);

  printf("# Netztyp Knoten Klassen \n");
  printf("  %d       %d      %d \n\n",typ,M,R);

  printf("# fuer alle Klassen \n");
  printf("#    fuer alle Knoten \n");
  printf("# lambda    e         1/mue     rho       mvz       ");
  printf("maa       mwz       mwsl \n");
  for (j = 0; j < R; j++)
  {
    for (i = 0; i < M; i++)
    {
      if (NOT offen[j])
      { printf(REAL_FORM,T[ind(i,j,N)]); }
      else
      { printf(REAL_FORM,T_o[IND2(i,j)]); }
      printf(REAL_FORM,theta[IND2(i,j)]); 
      printf(REAL_FORM,1/mue[IND2(i,j)]);
      
      if (theta[IND2(i,j)] > 0.0)
      {
        if (kn_typ[i] == 0)
        { printf("%9.5f ",0.0); }
        else
        { 
          if (NOT offen[j])
          { printf(REAL_FORM,rho[ind(i,j,N)]); }
          else
          { printf(REAL_FORM,rho_o[IND2(i,j)]); }
        }
 
        printf(REAL_FORM,W[ind(i,j,N)]);
        printf(REAL_FORM,L[ind(i,j,N)]);

        if (kn_typ[i] == 0)
        { printf("%9.5f %9.5f ",0.0,0.0); }
        else
        {
          printf(REAL_FORM,W[ind(i,j,N)] - 1/mue[IND2(i,j)]);
          if(NOT offen[j])
            printf(REAL_FORM,T[ind(i,j,N)] * (W[ind(i,j,N)] - 
              1/mue[IND2(i,j)])); 
          else
            printf(REAL_FORM,T_o[IND2(i,j)] * (W[ind(i,j,N)] - 
              1/mue[IND2(i,j)]));
        }
      }
      else
        for (kk = 0; kk < 5; kk++)
          printf("%9.5f ",0.0);
      printf("\n");
    }
    printf("\n");
  }
 for (i = 0; i < M; i++)
  {
    if((kn_typ[i] == 2) OR (kn_typ[i] == 3))
    {
      rho_sum = 0.0;
      for (j = 0; j < R; j++)
        if (NOT offen[j])
          rho_sum += rho[ind(i,j,N)];
        else
          rho_sum += rho_o[IND2(i,j)];
      if (rho_sum > 0.7) 
        w_rho = TRUE;
    }
  }
  if (w_rho)
  {
    fprintf(stderr,
      "Die Auslastung in einem/mehreren Prio'knoten ist zu hoch.\n");
    fprintf(stderr,"Die Ergebnisse koennten falsch sein.\n");
  }  
}

/*        Ausgabe im Fehlerfall                                        */

fehler_aus()
{
  if (f_eingabe)
  { 
    fprintf(stderr,"fehlerhafte Eingabedatei \n"); 
    printf("priomva2m_eingabe\n0 0 0 \n");
  }

  if (f_speicher)
  { 
    fprintf(stderr,"nicht genug Speicher vorhanden \n"); 
    printf("priomva2m_speicher\n0 0 0 \n");
  }

  if (f_stabil)
  {  
    fprintf(stderr,"Stabilitaetsbedingung verletzt \n"); 
    printf("priomva2m_instabil\n0 0 0 \n");
  }

  if (f_uew)
  { 
    fprintf(stderr,
            "Uebergangsw'keiten ab v0.24 nicht mehr unterstuetzt \n"); 
    printf("priomva2m_uew\n0 0 0 \n");
  }
}

/*        Initialisierung der Variablen                                */

init()
{
  int i,j,k;
  int prod;
  int MR;

  LIES("%d",M);
  LIES("%d",R);

  if ((M == 0) OR (R == 0))
  {
    fehler = TRUE;
    f_eingabe = TRUE;
    return;
  }

  MR = M * R;
  ALLOCATE(n,R,int);
  ALLOCATE(N,R,int);
  ALLOCATE(offen,R,BOOLEAN);
  ALLOCATE(kn_typ,M,int);
  ALLOCATE(lambda,R,REAL);
  ALLOCATE(mue,MR,REAL);
  ALLOCATE(theta,MR,REAL);
  ALLOCATE(prod_v,R+1,int);

  prio = FALSE;

  for (i = 0; i < M; i++)
  { 
    LIES("%d",kn_typ[i]);

    if ((kn_typ[i] == 2) OR (kn_typ[i] == 3))
      prio = TRUE;
  }

  op = FALSE;

  for (j = 0; j < R; j++)
  {
    LIES("%d", offen[j]); 
    if (NOT offen[j])
    {
      LIES("%d",N[j]); 
      lambda[j] = 0.0;
    }
    else
    {
      LIES(REAL_READ,lambda[j]); 
      N[j] = 0;
      op = TRUE; 
    }
  }

  typ = 0;
  for (j = 0; j < R; j++)
  {
    if (offen[j])
    {
      if ((typ == 0) OR (typ == 2))
      { typ = 2; }
      else
      { typ = 4; }
    }
    else
    {
      if ((typ == 0) OR (typ == 1))
      { typ = 1; }
      else
      { typ = 4; }
    }
  }
  
  LIES("%d",ue_w);
  
  if (NOT ue_w)
    for (j = 0; j < R; j++)
    {
      for (i = 0; i < M; i++)
      {
        LIES(REAL_READ,mue[IND2(i,j)]);
        LIES(REAL_READ,theta[IND2(i,j)]);
      }
    }
  else
  {
    fehler = TRUE;
    f_uew = TRUE;
    return;
  }

  prod_v[0] = 1;
  for (j = 0; j < R; j++)
  { prod_v[j+1] = prod_v[j] * (N[j] + 1); }

  prod = MR * prod_v[R];

  ALLOCATE(W,prod,REAL);
  ALLOCATE(T,prod,REAL);
  ALLOCATE(L,prod,REAL);
  ALLOCATE(rho,prod,REAL);
  ALLOCATE(rho_o,MR,REAL);
  ALLOCATE(rho_str,MR,REAL);
  ALLOCATE(T_o,MR,REAL);

  verf_init();
}

/*        Abbildung R+2-dim Index -> 1-dim Index                       */

ind(i,j,n)
int i,j,*n;
{
  int k,sum;

  sum = 0;
  for (k = 0; k < R; k++)
  { sum += (n[k] * prod_v[k]); } 
  sum = (sum * R + j) * M + i;
  return(sum);
}

/*         Hauptprogramm                                               */

void abbruch(sig)
int sig;
{
        printf("priomva2m_signal_%d\n 0 0 0 \n", sig);
        exit(1);
}

main(argc, argv)
	int argc;
	char *argv[];
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

  {
        int i;
        void abbruch();

        for (i=1; i<12; i++)
                signal (i, abbruch);
  }
	if ((argc > 1) && (!strcmp(argv[1], "-V")))    /* kirschni 22.11.1991 */
	{
		fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
		exit(0);
	}


  init();
  if(NOT fehler)
  { pop_loop(); }
  if(NOT fehler)
  { aus(N); exit(0); }
  else
  { fehler_aus(); exit(1); }
}

