/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tcore_alg.c\t1.2\t92/03/27 11:52:46";

#include "d_scat.h"
#include "xxalloc.h"

/***********************************************************

 Definitionen fuer den Core-Algorithmus

***********************************************************/

double ***pp = (double ***)0,
                 /* Randwahrscheinlichkeiten */
      **kgesch,
                 /* geschaetzte Zahl von Auftraegen */
      ***khilf,
                 /* Zwischenspeicherung von k-Werten */
      **wm,
                 /* Wahrscheinlichkeitenmasse */
      altlim,neulim;
                 /* Abbruchkriterien fuer Core */

int   stop,      /* Entscheidung fuer Abbruch */
                 /* von Core */
      breite,    /* Variable bei Berechnung von wm */
      *Auftraege;
                 /* Gesamtzahl der Auftraege */
                 /* je Population */

extern int BEmax;

/***********************************************************

  FUNKTION
  Initialisierung()

  Wird aufgerufen in
  - HAUPTPROGRAMM
    SCAT-Alg.     gespeichert auf File  "SCAT.c"
  - HAUPTPROGRAMM
    LINEARIZER        "        "    "   "LINEARIZER..c"

Berechnet die Gesamtzahl der Auftraege im System und besetzt
die k- und D-Werte vor.

***********************************************************/

Initialisierung()
{
 extern	int 	R;           	/* Anzahl der Auftragsklassen */
 extern	int 	N;           	/* Anzahl der Knoten */
 extern	int     **Auftr;
                    	/* Anzahl der Auftraege pro Klasse */
                    	/* ist der 2. Index 0,wird die ganze */
                    	/* Population betrachtet,sonst die */
                    	/* Population bei Entfernen eines */
                    	/* Auftrags der jeweiligen Klasse */
 extern	double	***k;
                    	/* mittlere Auftragszahl pro */
                    	/* Knoten und Klasse; ist der */
                    	/* 3. Index 0 fuer die gesamte */
                    	/* Population,sonst bei Entfernen */
                    	/* eines Auftrags der jeweiligen */
                    	/* Klasse */
 extern	double   ***D;
                    	/* D-Werte bei Schaetzungen */
 extern	int	kl;	    	/* Hilfsvariable */
 
 int	i,r,j;	/* Hilfsvariablen */

 if (!pp)
 {
    pp = d3_alloc(0,N-1, 0,R-1, 0,BEmax);
    kgesch = d2_alloc(0,N-1, 0,R-1);
    khilf = d3_alloc(0,N-1, 0,R-1, 0,R);
    wm = d2_alloc(0,STREUUNG, 0,STREUUNG);
    Auftraege = i1_alloc(0,R);
 }

 Auftraege[kl]=0;
 for ( r=0; r<R; r++ )
    {
     for ( i=0; i<N; i++ )
        {
         k[i][r][kl]=(1.0*Auftr[r][kl])/N;
         for ( j=0; j<R; j++ )
             D[i][r][j]=0.0;
        }
     Auftraege[kl]=Auftraege[kl]+Auftr[r][kl];
    }
} /* Ende Initialisierung */


/***********************************************************

  FUNKTION
  Wahrscheinlichkeit()

  Wird aufgerufen in
  - HAUPTPROGRAMM
    SCAT-Alg.     gespeichert auf File  "scat.c"
  - HAUPTPROGRAMM
    LINEARIZER         "       "    "   "LINEARIZER.c"

Berechnet die Verteilung der Wahrscheinlichkeitenmasse,
bei lastabhaengigen Knoten im System.

***********************************************************/

Wahrscheinlichkeit()
{
 extern	int	kl;

 int smax;
 int	i;
 double sum;
 static double *pr = (double *)0;

 if (!pr)
    pr = d1_alloc(0,STREUUNG-1);

 if ( (Auftraege[kl]-1)>STREUUNG )
    smax=STREUUNG;
 else
    smax=Auftraege[kl]-1;

 pr[0]=45;
 for ( breite=1; breite<smax; breite++ )
     pr[breite]=pr[breite-1]*0.7;

 wm[0][0]=1.0;
 for ( breite=1; breite<=smax; breite++ )
    {
     sum=0.0;
     for ( i=0; i<=breite-1; i++ )
        {
         wm[breite][i]=wm[breite-1][i]*(1-pr[breite-1]/100);
         sum=sum+wm[breite][i];
        }
     wm[breite][breite]=1.0-sum;
    }
} /* Ende Wahrscheinlichkeit */


/***********************************************************

  FUNKTION
  Core()

  Ruft auf:
  - Schaetzungen()    gespeichert auf File  "Core-Alg.c"
  - tundk()                "       "    "        "
  - Termination()          "       "    "        "

  Wird aufgerufen in:
  - HAUPTPROGRAMM
    SCAT-Alg.              "       "    "   "scat.c"
  - HAUPTPROGRAMM
    LINEARIZER             "       "    "   "LINEARIZER.c"

 Diese Funktion entspricht dem Core-Algorithmus

***********************************************************/

Core()
{
 extern	int	kl;

 stop=0;
 altlim=Auftraege[kl]+1;

 /* solange Abbruchbedingung nicht erfuellt */
 while ( stop==0 )
      {
       Schaetzungen();
       tundk();
       Termination();
      }

} /* Ende Core */


/***********************************************************

  FUNKTION
  Schaetzungen()

  Wird aufgerufen in:
  - Core()     gespeichert auf File  "Core-Alg.c"

Berechnet die F-Werte und schaetzt die k-Werte bei
Entfernung eines Auftrags aus einer Klasse.Fuer lastab-
haengige Knoten werden die Randwahrscheinlichkeiten
berechnet.

***********************************************************/

Schaetzungen()
{
 extern	int 	R;           	/* Anzahl der Auftragsklassen */
 extern	int 	N;           	/* Anzahl der Knoten */
 extern	int     **Auftr;
                    	/* Anzahl der Auftraege pro Klasse */
                    	/* ist der 2. Index 0,wird die ganze */
                    	/* Population betrachtet,sonst die */
                    	/* Population bei Entfernen eines */
                    	/* Auftrags der jeweiligen Klasse */
 extern	int	*BE;
			/* Anzahl der identischen */
                    	/* Bedieneinheiten pro Knoten */

 extern	double	***k;
                    	/* mittlere Auftragszahl pro */
                    	/* Knoten und Klasse; ist der */
                    	/* 3. Index 0 fuer die gesamte */
                    	/* Population,sonst bei Entfernen */
                    	/* eines Auftrags der jeweiligen */
                    	/* Klasse */
                    	/* Knoten und Klasse */
 extern	double	***F,
                    	/* F-Werte bei Schaetzungen */
       		***D;
                    	/* D-Werte bei Schaetzungen */
 extern int	kl;	    	/* Hilfsvariable */

 double zahl,sum;
 int floor,hilf1,hilf2;
 int	i,r,j;	/* Hilfsvariablen */

 /* fuer alle Knoten und Klassen */
 for ( i=0; i<N; i++ )
     for ( r=0; r<R; r++ )
	if ( Auftr[r][kl] != 0 ) 
        {
         sum=0.0;
         for ( j=0; j<R; j++ ) /* alle Klassen */
            {
             zahl=0.0;

             /* berechne F-Werte */
             if ( Auftr[j][kl]==0 )
                F[i][j][kl]=0.0;
             else
                F[i][j][kl]=k[i][j][kl]/Auftr[j][kl];

             /* schaetze k */
             if ( j==r )
                if ( Auftr[j][kl]==0 )
                   zahl=0.0;
                else
                   zahl=(Auftr[j][kl]-1)*
                        (F[i][j][kl]+D[i][j][r]);
             else
                zahl=Auftr[j][kl]*(F[i][j][kl]+D[i][j][r]);
             sum=sum+zahl;
            }
         kgesch[i][r]=sum;

         /* Randwahrscheinlichkeiten */
         if ( BE[i]>1 )
           {
            floor=kgesch[i][r]; /* ganzzahliger Anteil */
            hilf1=Auftraege[kl]-1;

            if ( hilf1<=BEmax )
               pp[i][r][hilf1]=0.0;
 
            if ( floor>STREUUNG )
              {
               breite=STREUUNG;
               hilf2=floor-STREUUNG;
              }
            else
              {
               breite=floor;
               hilf2=0;
              }

           if ( N==1 || hilf1==floor )
	      {
	      for ( j=0; j<=BE[i]-2; j++ )
		  if ( j==hilf1 )
		     pp[i][r][hilf1]=1.0;
                  else
		     pp[i][r][j]=0.0;
	      }
           else
            for ( j=0; j<=BE[i]-2; j++ )
              {
               if ( hilf1-floor<=floor && j>=hilf2 &&
                    j<=(2*floor-hilf1) )
                 {
                 pp[i][r][j]=(hilf1-kgesch[i][r])*
                             wm[breite][floor-j]/(hilf1-j);
                 if ( hilf1<=BEmax )
                    pp[i][r][hilf1]=pp[i][r][hilf1]+
                                    1-pp[i][r][j];
                 }
               else
                 {
                 if ( j>=(floor-breite) && j<=floor )
                    pp[i][r][j]=(2*floor-j+1-kgesch[i][r])*
                                wm[breite][floor-j]/
                                (2*floor-2*j+1);
                 else
                   {
                   if ( j>floor && j<=(floor+breite+1) )
                      pp[i][r][j]=wm[breite][2*floor-j+1]-
                                  pp[i][r][2*floor-j+1];
                   else
                      pp[i][r][j]=0.0;
                   }
                 }
              }
           } /* if */

        } /* Knoten und Klassen */

} /* Ende Schaetzungen */


/***********************************************************

  FUNKTION
  tundk()

  Wird aufgerufen in:
  - Core()      gespeichert auf File   "Core-Alg.c"

Berechnet die mittleren Verweilzeiten und die mittlere Zahl
von Auftraegen pro Knoten und Klasse.

***********************************************************/

tundk()
{
 extern	int 	R;           	/* Anzahl der Auftragsklassen */
 extern	int 	N;           	/* Anzahl der Knoten */
 extern	int     **Auftr;
                    	/* Anzahl der Auftraege pro Klasse */
                    	/* ist der 2. Index 0,wird die ganze */
                    	/* Population betrachtet,sonst die */
                    	/* Population bei Entfernen eines */
                    	/* Auftrags der jeweiligen Klasse */
 extern	int	*BE;
			/* Anzahl der identischen */
                    	/* Bedieneinheiten pro Knoten */

 extern	double 	**my;
                    	/* Bedienrate pro Knoten und Klasse */
 extern	double	**ee;
                    	/* Besuchshaeufigkeiten pro */
                    	/* Knoten und Klasse */
 extern	double  **t;
                    	/* mittlere Verweilzeit pro */
                    	/* Knoten und Klasse */
 extern int	kl;	    	/* Hilfsvariable */

 double sum,summe;
 int	i,r,j;	/* Hilfsvariablen */

 /* fuer alle Klassen */
 for ( r=0; r<R; r++ )
    if ( Auftr[r][kl] != 0 ) 
    {
     summe=0.0;
     /* fuer alle Knoten */
     for ( i=0; i<N; i++ )
        {

         /* berechne Verweilzeiten */
         if ( my[i][r]==0.0 )
            t[i][r]=0.0;
         else
           {
            if ( BE[i]==0 )  /* Infinite Server */
               t[i][r]=1/my[i][r];
            else
              {
               if ( BE[i]==1 )  /* Single Server */
                  t[i][r]=(1+kgesch[i][r])/my[i][r];
               else
                 {
                  sum=0.0;
                  for ( j=0; j<=BE[i]-2; j++ )
                      sum=sum+(BE[i]-1-j)*pp[i][r][j];
                  t[i][r]=(1+kgesch[i][r]+sum)/
                          (my[i][r]*BE[i]);
                 }
              }
           }
         summe=summe+(t[i][r]*ee[i][r]);
        } /* Knoten */

     /* berechne k,Zwischenspeicherung der Werte */
     for ( i=0; i<N; i++ )
       khilf[i][r][kl]=Auftr[r][kl]*ee[i][r]*t[i][r]/summe;

    } /* Klassen */
} /* Ende tundk */


/***********************************************************

  FUNKTION
  Termination()

  Wird aufgerufen in:
  - Core()     gespeichert auf File   "core-alg.c"

 Ueberprueft ob die Iteration des Core-Algorithmus 
 abgebrochen werden kann.

***********************************************************/

Termination()
{
 extern	int 	R;           	/* Anzahl der Auftragsklassen */
 extern	int 	N;           	/* Anzahl der Knoten */
 extern	int     **Auftr;
                    	/* Anzahl der Auftraege pro Klasse */
                    	/* ist der 2. Index 0,wird die ganze */
                    	/* Population betrachtet,sonst die */
                    	/* Population bei Entfernen eines */
                    	/* Auftrags der jeweiligen Klasse */
 extern	double	***k;
                    	/* mittlere Auftragszahl pro */
                    	/* Knoten und Klasse; ist der */
                    	/* 3. Index 0 fuer die gesamte */
                    	/* Population,sonst bei Entfernen */
                    	/* eines Auftrags der jeweiligen */
                    	/* Klasse */

 extern int	kl;	/* Hilfsvariable */

 double max,wert,abbruch;
 int	i,r;	/* Hilfsvariablen */

 abbruch=1.0/(4000.0+16.0*Auftraege[kl]);
 max=0.0;
 for ( r=0; r<R; r++ )
     for ( i=0; i<N; i++ )
        {
         if ( Auftr[r][kl]==0 )
	      wert=0.0;  
         else
	    {
              wert=khilf[i][r][kl]-k[i][r][kl];
              /* Betragsbildung */
             if ( wert<0.0 )
                  wert= -wert;
             wert=wert/Auftr[r][kl];
	    }

         /* bestimme maximalen Wert */
         if ( wert>max )
            max=wert;
        }

 neulim=max;

 if ( neulim>abbruch && neulim<altlim )
    stop=0;
 else
    stop=1;

 for ( i=0; i<N; i++ )
     for ( r=0; r<R; r++ )
         k[i][r][kl]=khilf[i][r][kl];

 altlim=neulim;

} /* Ende Termination */
