/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  8520 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/

static char *SccsId = "@(#)PEPSY:\tscat.c\t1.2\t92/03/27 11:52:42";

# include <stdio.h>
# include "d_scat.h"
# include "xxalloc.h"

/***********************************************************
  HAUPTPROGRAMM
  SCAT

  Ruft auf:		Gespeichert in
  - eingabe()           "eingabe.c"
  - Gauss()           	"gauss.c" 
  - Initialisierung()   "core-alg.c"
  - Wahrscheinlichkeit()    " 
  - Core()                  "
  - Leistungsgroessen()  "ergebnis.c"
  - ausgabe()            "ausgabe.c"

  Externe Definitionen stehen auf File "def-SL.c".
  Kann vom Bedienprogramm oder direkt gestartet werden.
  Berechnet Leistungsgroessen mit dem SCAT-Algorithmus.

***********************************************************/

/* GLOBALE DEKLARATIONEN */

int   	netz;    	/* Netztyp	*/
int 	R;           	/* Anzahl der Auftragsklassen */
int 	N;           	/* Anzahl der Knoten */
int     **Auftr;
                    	/* Anzahl der Auftraege pro Klasse */
                    	/* ist der 2. Index 0,wird die ganze */
                    	/* Population betrachtet,sonst die */
                    	/* Population bei Entfernen eines */
                    	/* Auftrags der jeweiligen Klasse */
int	*BE;
			/* Anzahl der identischen */
                    	/* Bedieneinheiten pro Knoten */
int	BEmax;

double 	**my;
                    	/* Bedienrate pro Knoten und Klasse */
int   	haeuf; 	/* Besuchshaeufigkeiten oder Uebergangsw. ? */
double	**ee;
                    	/* Besuchshaeufigkeiten pro */
                    	/* Knoten und Klasse */
double	***k;
                    	/* mittlere Auftragszahl pro */
                    	/* Knoten und Klasse; ist der */
                    	/* 3. Index 0 fuer die gesamte */
                    	/* Population,sonst bei Entfernen */
                    	/* eines Auftrags der jeweiligen */
                    	/* Klasse */
double  **t;
                    	/* mittlere Verweilzeit pro */
                    	/* Knoten und Klasse */
double	**lamda;
                    	/* Durchsatz pro Knoten und Klasse */
double	**rho;
                    	/* Auslastung pro Knoten und Klasse */
double	**w;
                    	/* mittlere Wartezeit pro */
                    	/* Knoten und Klasse */
double	**q;
                    	/* mittlere Warteschlangenlaenge pro */
                    	/* Knoten und Klasse */
double	***F;
                    	/* F-Werte bei Schaetzungen */
double	***D;
                    	/* D-Werte bei Schaetzungen */
int	kl;	    	/* Hilfsvariable */

void abbruch(sig)
int sig;
{
        printf("scat_signal_%d\n 0 0 0 \n", sig);
        exit(1);
}

main(argc, argv)
	int argc;
	char *argv[];
{
	/*		Sccs support from SccsLogging.c			*/	
	extern int SccsRelease;
	extern int SccsLevel;

	extern		eingabe();
	extern		Initialisierung();
	extern		Wahrscheinlichkeit();
	extern		Core();
	extern		Leistungsgroessen();
	extern		ausgabe();

	int  wa=0;        /* Entscheidung fuer Aufruf von */
                   /* Wahrscheinlichkeit() */
	int  r,i,j;

    void abbruch();

    for (i=1; i<12; i++)
        signal (i, abbruch);

	if ((argc > 1) && (!strcmp(argv[1], "-V")))    /* kirschni 22.11.1991 */
	{
		fprintf(stdout, "%s Version %d.%d\n", argv[0], SccsRelease, SccsLevel);
		exit(0);
	}


	/* Lesen der Eingabedaten von "stdin" */
	eingabe();

	k		= d3_alloc(0,N-1, 0,R-1, 0,R);
	t		= d2_alloc(0,N-1, 0,R-1);
	lamda	= d2_alloc(0,N-1, 0,R-1);
	rho		= d2_alloc(0,N-1, 0,R-1);
	w		= d2_alloc(0,N-1, 0,R-1);
	q		= d2_alloc(0,N-1, 0,R-1);
	F		= d3_alloc(0,N-1, 0,R-1, 0,R);
	D		= d3_alloc(0,N-1, 0,R-1, 0,R-1);

	kl=0;

	/* Initialisierung bei gesamter Population */
	Initialisierung();

	/* wenn sich ein lastabhaengiger Knoten im */
	/* System befindet,berechne Verteilung der */
	/* Wahrscheinlichkeitenmasse */
	for ( i=0; i<N; i++ )
	    if ( BE[i]>1 )
	       wa=1;
	if ( wa==1 )
	   Wahrscheinlichkeit();

	/* Aufruf von Core bei gesamter Population */
	Core();

	/* Aufruf von Core bei jeder Population mit */
	/* einem Auftrag weniger in einer Klasse */
	for ( kl=1; kl<=R; kl++ )
	   {
	    /* vorbesetzen der Auftraege pro Klasse */
	    for ( r=1; r<=R; r++ )
	        if ( kl==r )
	           Auftr[r-1][kl]=Auftr[r-1][0]-1;
	        else
	           Auftr[r-1][kl]=Auftr[r-1][0];

	    Initialisierung();
	    Core();
	   }

	/* Berechnen der D-Werte fuer anschliessenden */
	/* Aufruf von Core */
	for ( i=0; i<N; i++ )
	    for ( r=0; r<R; r++ )
	       {
	        F[i][r][0]=k[i][r][0]/Auftr[r][0];
	        for ( kl=1; kl<=R; kl++ )
	            if ( Auftr[r][kl]==0 )
	               F[i][r][kl]=0.0;
	            else
	              {
	               F[i][r][kl]=k[i][r][kl]/
	                           Auftr[r][kl];
	               D[i][r][kl-1]=F[i][r][kl]-
	                             F[i][r][0];
	              }
	        }

	kl=0;
	Core();

	/* Berechnen weiterer Leistungsgroessen */
	Leistungsgroessen();

	/* Ausgabe der Ergebnisse */
	ausgabe("scat");

	exit(0);
} /* Ende SCAT-Hauptprogramm */
