static char *SccsId = "@(#)PEPSY:\tinitsim.c\t1.6\t94/02/04 11:22:59";
#include "sys/types.h"	/*fuer size_t*/
#include "stdio.h"
#include "data.h"

/*************************** Funktionen und Prozeduren des BS ***************************/
extern void		exit(),
					free(),
					srand48(),
				  *calloc();
extern int	strncmp();
extern size_t	strlen();
extern char	  *strcpy();
extern long		time();
extern double	sqrt();

/************************* Prozeduren und Funktionen aus anderen Modulen ***************/
extern struct JOB		*CreateJob();
extern struct EVENT	*CreateEvent();
extern char				*fscanName();													/* lowlevel.c */
extern void				 StartJob(),
							 Dump(),
							 AllocGlobal();

/***************************** globale Variablen (aus main.c!) **************************/
extern struct LKopf	 EventListe,
						 	 EmptyJobs,
						 	 EmptyEvents;
extern tunit          Time,            /* Simulationsuhr */
						 	 Warmup;
extern struct BE   	*be;              /* globale Netzelemente */
extern struct WS   	*ws;
extern struct AUSSEN *AussenWelt;
extern char			 	*ModellName;
extern boolean		    dump, zufallinit, mkpepout;
extern long			    zufall;
extern int			    SimulationsPhase;
extern int            AnzWs, AnzBe, AnzKl, AnzBa, aktBa;
extern FILE			 	*ErrOut;
extern double		    SFaktor;

void Liste_MakeEmpty(Liste)
	struct LKopf *Liste;
/*
**	Eine Liste wird explizit geleert.
**
**	ACHTUNG! Wenn die Liste Elemente enthaelt, werden diese nicht geloescht, sondern ver-
**	schwinden mangels Referenzierung im Speichernirwana!
**
*/
{
   Liste->Anfang = (struct LNode *)NULL;
   Liste->Ende = (struct LNode *)NULL;
   Liste->Laenge = 0L;
}

void InitPrevWs()
/*
**	Aus der in den WSen gespeicherten Nachfolgerrelation wird eine Vorgaengerrelation fuer
**	die BE ermittelt.
**
*/
{
	int *buffer;
	int  aktBe, aktWs, i;
	int  AnzVorg;

	buffer = (int *)calloc((unsigned)AnzWs, sizeof(int));

	for (aktBe=0; aktBe<AnzBe+1; aktBe++)
   {
      AnzVorg = 0;

      for (aktWs=0; aktWs<AnzWs+1; aktWs++)
      {
         for (i=0; i<ws[aktWs].AnzNachfolgBE; i++)
         {
				if (ws[aktWs].Next_BE[i] == aktBe)
            {
               buffer[AnzVorg] = aktWs;
               AnzVorg ++;
               i = ws[aktWs].AnzNachfolgBE;
            }
         }
      }
		be[aktBe].AnzVorgWS = AnzVorg;
      if (AnzVorg == 0)
         be[aktBe].Prev_WS = (int *)NULL;
      else
         be[aktBe].Prev_WS = (int *)calloc((unsigned)AnzVorg, sizeof(int));

#		ifdef DBG
      fprintf(ErrOut,"BE%d hat %d Vorgaenger: ", aktBe, AnzVorg);
#		endif

		for (i=0; i<AnzVorg; i++)
      {
#			ifdef DBG
         fprintf(ErrOut,"[%d]:%d ",i,buffer[i]);
#			endif

         be[aktBe].Prev_WS[i] = buffer[i];
      }

#		ifdef DBG
      fprintf(ErrOut,"\n");
#		endif
   }

   free((void *)buffer);
}

void InitWkeiten()
/*
**	Aus den Uebergangswahrscheinlichkeiten wird eine kummulierte UEW berechnet, damit die
**	Nachfolger in ErmittleNext() schneller errechnet werden koennen.
**
*/
{
	struct BE_DATA *bd;
	int aktBe, aktK, i;

	for (aktBe=0; aktBe<=AnzBe; aktBe++)
	{
		for (aktK=0; aktK<AnzKl; aktK++)
		{
			bd = &be[aktBe].BE_Data[aktK];

			for (i=1; i<=AnzWs; i++)
				bd->UEW[i] += bd->UEW[i-1];

			if ((bd->UEW[AnzWs] != 1000) && (bd->UEW[AnzWs] != 0))
				fprintf(ErrOut, "FEHLER: In BE%d KL%d ist die Summe der UEW!=1000!\n",aktBe,aktK);

			if (bd->KlWechselWkeit != (int *)NULL)
			{
				for (i=1; i<AnzKl; i++)
					bd->KlWechselWkeit[i] += bd->KlWechselWkeit[i-1];

				if ((bd->KlWechselWkeit[AnzKl-1] != 1000) && bd->KlWechselWkeit[AnzKl-1] != 0)
					fprintf(ErrOut, "FEHLER: In BE%d KL%d ist die Summe der KWW!=1000!\n",aktBe,aktK);

				if (bd->KlWechselWkeit[AnzKl-1] == 0)
				{	
					free((void *)bd->KlWechselWkeit);
					bd->KlWechselWkeit = (int *)NULL;
				}
			}
		}
	}
}


void InitBZPar()
/*
**	In der Modellbeschreibung werden nur der MW und der QuadVarKoeff uebergeben. Hier werden
**	die fuer die Vtlgen noetigen Parameter berechnet und der SFaktor ermittelt, damit die durch
**	das Runden der Zufallszahlen entstandene Ungenauigkeit moeglichst klein wird.
**
**	In bd->BZPar1 wird fuer die Vtlg der Mittelwert (M) angegeben und in
**		bd->BZPar2 der quadrierte Variationskoeffizient (C2!)
**
*/
{
#	define MINMW	 100.0							/* minimale MW einer Verteilung (um Rundungsfehler
																zur verringern */
	struct BE_DATA	*bd;
	int				 i, j, k, p;
	double			 h, m, c2;

	SFaktor = 1.0;

	for (i=0; i<=AnzBe; i++)
	{
		for (j=0; j<AnzKl; j++)
		{
			bd = &be[i].BE_Data[j];

			c2	= bd->BZPar2;
			m  = bd->BZPar1;

			switch(bd->BedZeitVtlg)
			{
				case CONST_VTLG	:
					break;
				case NORMAL_VTLG	:
				case LOGNORM_VTLG	:
					bd->BZPar2 = 1.0/m/sqrt(c2);			/* Berechnen von sigma aus C2 */

					while (m*SFaktor < MINMW)
					{
						SFaktor *= 10.0;
					}
					break;
				case EXPON_VTLG	:
					if (m == 0.0)											/* Berechnen von mue aus mw */
					{
						bd->BZPar1 = 0.0;
					}
					else
					{
						bd->BZPar1 = 1.0/m;

						while (SFaktor/bd->BZPar1 < MINMW)
						{
							SFaktor *= 10.0;
						}
					}
					break;
				case COX_VTLG		: 
					if ((c2<=0.1) || (c2>=1))
					{
						fprintf(ErrOut,"FEHLER: BE%d KL%d CoxVtlg : c2 nicht in (0.1;1.0)!\n", i, j);
						exit(-1);
					}
					bd->BZPar3 = k = (1.0/c2+0.999999);
					bd->BZPar2 = p = (int)(1000.0 *
															((double)(2*k)*c2+(double)k-2.0-
																	sqrt((double)(k*k+4)-4.0*c2*(double)k)) /
										 					(2.0*(c2+1.0)*(double)(k-1))
												 +0.5);
					bd->BZPar1 	  = ((double)k-(double)p/1000.0*(double)(k-1))/m;

					while (SFaktor/bd->BZPar1 < MINMW)
					{
						SFaktor *= 10.0;
					}
					break;
				case COX2_VTLG		:
					if (c2<=1.0)
					{
						fprintf(ErrOut,"FEHLER: BE%d KL%d Cox2Vtlg : c2 nicht > 1.0!\n", i, j);
						exit(-1);
					}
									 h = sqrt(1.0-2.0/(1.0+c2));				/* Hilfsgroesse */
					bd->BZPar3 = p = (int)(1000.0 *
																c2*(1.0-h)
													+0.5);
					bd->BZPar1 =     (1.0+h)/m;
					bd->BZPar2 =     (1.0-h)/m;

					while (SFaktor/bd->BZPar1 < MINMW)
					{
						SFaktor *= 10.0;
					}

					while (SFaktor/bd->BZPar2 < MINMW)
					{
						SFaktor *= 10.0;
					}
					break;
				case DETEXP_VTLG	:
					if (c2>=1.0)
					{
						fprintf(ErrOut, "FEHLER: BE%d KL%d DetExpVtlg mit c2>1.0!\n", i, j);
						exit(-1);
					}
					if (c2==0.0)
					{
						bd->BedZeitVtlg = CONST_VTLG;
						bd->BZPar1		 = m;
					}
					else
					{
						bd->BZPar2 = 1.0/m/sqrt(c2);
						bd->BZPar1 = m-(1.0/bd->BZPar2);

						while (SFaktor/bd->BZPar2 < MINMW)
						{
							SFaktor *= 10.0;
						}
					}
					break;
				case GE_VTLG		:
					if (c2<1.0)
					{
						fprintf(ErrOut, "FEHLER: BE%d KL%d GEVtlg mit c2<1.0!\n", i, j);
						exit(-1);
					}
					
					bd->BZPar1 = 2.0 / (c2+1.0) / m;
					bd->BZPar2 = p = (int)(1000.0*2.0/(c2+1.0)+0.5);

					while (SFaktor/bd->BZPar1 < MINMW)
					{
						SFaktor *= 10.0;
					}
					break;
				case HYPEREXP_VTLG	:
					if (c2<1.0)
					{
						fprintf(ErrOut, "FEHLER: BE%d KL%d HyperexponentialVtlg mit c2<1.0!\n", i,j);
						exit(-1);
					}
					/*	Berechnen von BZPar1 (mue1), BZPar2 (mue2) nach Formel in
						Stephen S. Lavenberg "", S.35

						Berechnung von BZPar3 (p)  geaendert!
					*/
					bd->BZPar2 = 2.0/m;
					bd->BZPar1 = 1.0/(m*(1.0+(c2-1.0)/(2.0*(1-1/bd->BZPar2*m))));
					bd->BZPar3 = (int)(1000.0*
														(m*bd->BZPar2-1.0)*bd->BZPar1/(bd->BZPar2-bd->BZPar1)
											 + 0.5);

					while (SFaktor/bd->BZPar1 < MINMW)
					{
						SFaktor *= 10.0;
					}

					while (SFaktor/bd->BZPar2 < MINMW)
					{
						SFaktor *= 10.0;
					}
					break;
				case HYPOEXP_VTLG	:
					if ((c2<0.5) || (c2>1.0))
					{
						fprintf(ErrOut,"FEHLER: BE%d KL%d HypoexpVtlg : c2 nicht in [0.5;1.0]!\n", i, j);
						exit(-1);
					}
								h = sqrt(1.0+2.0*(c2-1.0));
					bd->BZPar1 = 2.0/m/(1.0+h);
					bd->BZPar2 = 2.0/m/(1.0-h);

					while (SFaktor/bd->BZPar1 < MINMW)
					{
						SFaktor *= 10.0;
					}

					while (SFaktor/bd->BZPar2 < MINMW)
					{
						SFaktor *= 10.0;
					}
					break;
				case PRIVATE_VTLG	:
					while (m*SFaktor < MINMW)
					{
						SFaktor *= 10.0;
					}
					break;
				default			:
					fprintf(ErrOut, "FEHLER: InitBZPar() BE%d KL%d unbekannter Verteilungstyp %d!\n",
								i, j, bd->BedZeitVtlg);
					exit(-1);
			}
		}
	}

	for (i=0; i<=AnzBe; i++)
	{
		for (j=0; j<AnzKl; j++)
		{
			bd = &be[i].BE_Data[j];

			switch(bd->BedZeitVtlg)
			{
				case NORMAL_VTLG	:
				case LOGNORM_VTLG	:
					bd->BZPar1 *= SFaktor;
					bd->BZPar2 *= SFaktor;
					break;
				case CONST_VTLG	:
				case PRIVATE_VTLG	:
					bd->BZPar1 *= SFaktor;
					break;
				case EXPON_VTLG	:
				case COX_VTLG		:
				case GE_VTLG		:
					bd->BZPar1 /= SFaktor;
					break;
				case DETEXP_VTLG	:
					bd->BZPar1 *= SFaktor;
					bd->BZPar2 /= SFaktor;
					break;
				case HYPEREXP_VTLG	:
				case HYPOEXP_VTLG		:
				case COX2_VTLG			:
					bd->BZPar1 /= SFaktor;
					bd->BZPar2 /= SFaktor;
					break;
				default			:
					fprintf(ErrOut, "FEHLER: InitBZPar() BE%d KL%d unbekannter Verteilungstyp %d!\n",
								i, j, bd->BedZeitVtlg);
					exit(-1);
			}
		}
	}
}

void InitIRQFlag()
/*
**	Wenn eine NachfolgerBE einer WS eine PRE-Strategie befolgt, wird dies in der WS vermerkt.
**
*/
{
	int i, j, k, n, flag, aktBe;

	for (i=0; i<=AnzWs; i++)
	{
		flag = 0;

		for (n=0; n<ws[i].AnzNachfolgBE; n++)
		{
			aktBe = ws[i].Next_BE[n];

			for (k=0; k<AnzKl; k++)
			{
				for (j=0; j<AnzKl; j++)
					flag |= (be[aktBe].BE_Data[k].IRQ[j]);
			}
		}
		ws[i].PR_IRQ = (flag != 0)?TRUE:FALSE;
	}
}

void TestModell()
/*
**	Am im Rechner erzeugten Modell werden Plausibilitaetsueberpruefungen vorgenommen und ggf
**	mit einer Fehlermeldung abgebrochen.
**	Es sind noch nicht alle moeglichen Ueberpruefungen vorgenommen.
**
*/
{
	boolean	test, error;
	int		i, j, l;

	error = FALSE;

	if (AnzBa%2 != 0)
	{
		fprintf(ErrOut, "FEHLER: Die Anzahl der Batches muss gerade sein!\n");
		error = TRUE;
	}
	if (AnzBa < 8)
	{
		fprintf(ErrOut, "FEHLER: Die Anzahl der Batches muss groesser als 8 sein, sonst \
							  funktioniert die statistische Auswertung nicht!\n");
		error = TRUE;
	}

	for (i=0; i<=AnzBe; i++)
	{
		/*
		**	fuer alle BE testen, ob die requeue auch in den VorgaengerWs dabei ist
		*/

		for (j=0; j<AnzKl; j++)
		{
			test = ~OK;
	
			for (l=0; l<be[i].AnzVorgWS; l++)
			{
				if (be[i].BE_Data[j].requeue == be[i].Prev_WS[l])
					test = OK;
			}
			if (test != OK)
				fprintf(ErrOut, "ACHTUNG: BE%d.KL%d.Requeue ist nicht VorgaengerWs!\n", i, j);

			/*
			**	Bei PS-BE darf kein stationaerer KW vorgenommen werden
			*/

			test = FALSE;

			for (l=0; l<AnzKl; l++)
				test |= (be[i].BE_Data[j].StatKW[l] != 0);
		
			if (test && (be[i].BE_Typ == PS_BT))
			{
				fprintf(ErrOut, "ACHTUNG: Stationaerer KlWechsel auf BE%d in KL%d trotz BE_Typ=%d!\n",
								 i, j, be[i].BE_Typ);
			}
		}

		/*
		**	testen, ob Unterbrechungsstrategien nicht bei MULSER_BT/IS_BT/PS_BT vorkommen
		*/

		test = FALSE;

		for (j=0; j<AnzKl; j++)
			for (l=0; l<AnzKl; l++)
				test |= (be[i].BE_Data[j].IRQ[l] != 0);
		
		if (test && ((be[i].BE_Typ == MULSER_BT)||(be[i].BE_Typ == PS_BT)||(be[i].BE_Typ == IS_BT)))
		{
			fprintf(ErrOut, "ACHTUNG: Unterbrechungsstrategie auf BE%d trotz BE_Typ=%d!\n",
								 i, be[i].BE_Typ);
		}
	}

	if (error)
	{
		fprintf(ErrOut, 
				"\nSimulator aufgrund schwerwiegender Fehler in der Initialisierung abgebrochen!\n"
				 );
		exit(-1);
	}
}

void InitSim(InFile)
	FILE *InFile;
/*
**	Hier werden die Daten fuer das zu simulierende Netz aus der Eingabedatei ausgelesen
**
*/
{
	struct JOB *Job;
   int i, j, k;

	if (zufallinit)       				 /* Initialisieren des Zufallszahlengenerators drand48() */
		srand48((long)zufall);
	else
		srand48((long)(time(0L)));

   Time					= 0L;                      /* Anfang der Simulation, d.h Zeitpunkt null */
  
	aktBa					= NOTFOUND;
	SimulationsPhase	= WARMUP_SP;

   Liste_MakeEmpty(&EmptyJobs);                       /* Speicherverwaltungslisten entleeren */
   Liste_MakeEmpty(&EmptyEvents);
   Liste_MakeEmpty(&EventListe);            /* Keine Events, d.h. EventListe ist leere Liste */

   (void)CreateEvent(BATCHEND_ET, 0, (struct JOB *)NULL, Warmup);

	ModellName = fscanName(InFile);

	if (strncmp(ModellName,"e_",(size_t)2)==0)
	{
		mkpepout = TRUE;
	}

	(void)fscanf(InFile, "%d %d %d", &AnzWs, &AnzBe, &AnzKl);

	AllocGlobal(AnzBe, AnzWs, AnzKl, AnzBa);

	for(i=1; i<=AnzWs; i++)
   {
   	ws[i].ListTyp	= FCFS_LT;

      Liste_MakeEmpty(&(ws[i].JobListe));

		for (k=0; k<AnzKl; k++)
		{
			ws[i].WS_Data[k].WSLaenge		= 0L;
			ws[i].WS_Data[k].StatUpdate	= 0L;
			ws[i].WS_Data[k].AnzAuftraege	= 0;
			ws[i].WS_Data[k].PassedJobs	= 0;
		}
   }
	for (i=0; i<=AnzBe; i++)
   {
      Liste_MakeEmpty(&be[i].JobListe);
      be[i].Event			= (struct EVENT *)NULL;
		be[i].IRQStatus	= (int *)NULL;
		be[i].Status 		= IDLE_ST;
		be[i].StatUpdate  = 0L;

		for (k=0; k<AnzKl; k++)
		{
			be[i].BE_Data[k].PassedJobs	= 0;
			be[i].BE_Data[k].BedZeitVtlg	= CONST_VTLG;
			be[i].BE_Data[k].BZPar1			= 0.0;
			be[i].BE_Data[k].BZPar2			= 0.0;
		}
   }

	for (i=0; i<=AnzWs; i++)
	{
		ws[i].Name = fscanName(InFile);

		(void)fscanf(InFile, "%d %d", &(ws[i].ListTyp), &(ws[i].AnzNachfolgBE));

   	ws[i].Next_BE = (int *)calloc((unsigned)ws[i].AnzNachfolgBE, sizeof(int));
		
		for (k=0; k<ws[i].AnzNachfolgBE; k++)
		{
			(void)fscanf(InFile, "%d", &(ws[i].Next_BE[k]));
		}
	}

	for (i=0; i<=AnzBe; i++)
	{
		be[i].Name = fscanName(InFile);

		(void)fscanf(InFile, "%d ", &be[i].BE_Typ);

		if (be[i].BE_Typ == PS_BT)
		{
			(void)fscanf(InFile, "%d ", &be[i].AnzBE);
		}

		for (k=0; k<AnzKl; k++)
		{
			(void)fscanf(InFile, "%d %lf %lf ",	&(be[i].BE_Data[k].BedZeitVtlg),
															&(be[i].BE_Data[k].BZPar1),
															&(be[i].BE_Data[k].BZPar2));

			(void)fscanf(InFile, "%d", &(be[i].BE_Data[k].requeue));

			for (j=0; j<AnzKl; j++)
			{
				(void)fscanf(InFile, "%d", &(be[i].BE_Data[k].KlWechselWkeit[j]));

				if ((j==0)&&(be[i].BE_Data[k].KlWechselWkeit[j]==-1))
				{
					free((void *)be[i].BE_Data[k].KlWechselWkeit);
					be[i].BE_Data[k].KlWechselWkeit = (int *)NULL;
					break;
				}
			}

			for (j=0; j<AnzKl; j++)
				(void)fscanf(InFile, "%d", &(be[i].BE_Data[k].StatKW[j]));

			for (j=0; j<AnzKl; j++)
				(void)fscanf(InFile, "%d", &(be[i].BE_Data[k].SuspendKWW[j]));

			for (j=0; j<AnzKl; j++)
				(void)fscanf(InFile, "%d", &(be[i].BE_Data[k].IRQ[j]));

			for (j=0; j<=AnzWs; j++)
				(void)fscanf(InFile, "%d", &(be[i].BE_Data[k].UEW[j]));
		}
	}

	InitIRQFlag();
	InitPrevWs();
	InitBZPar();
	InitWkeiten();

	for(k=0; k<AnzKl; k++)
	{
		AussenWelt[k].Name = fscanName(InFile);

		fscanf(InFile, "%d", &i);
		
		AussenWelt[k].KlassenTyp = i;

		if (i==OFFEN_KL)
		{
			Job = CreateJob(k);
			StartJob(Job, 0);			/* Fuer jede offene Klasse einen Kontrollauftrag starten */
		}

		fscanf(InFile, "%d %d", &i, &j);
		while (i>0)
		{
			Job = CreateJob(k);
   		(void)CreateEvent(ARRIVAL_ET, j, Job, Time);
			i--;
		}
	}

	if (dump)
		Dump(16);

	TestModell();
}
