static char *SccsId = "@(#)PEPSY:\tlists.c\t1.4\t94/02/04 11:23:02";
/**********************************************************************************************
**
**	Modul mit Listenverwaltungsfunktionen
** -------------------------------------
**
** int Insert(Liste, AktElem, Element)		Element in der Liste nach AktElem einfuegen
**	int EnqueueHead(Liste, Element)			Element am Anfang der Liste einfuegen
**	int Insort(Liste, Element)					Element in die Liste einfuegen (increasing order)
**	int Enqueue(Liste, Element)				Listenelemnet anfuegen
**	struct LNode *GetFirst (Liste)			erstes Listenelement ausketten und zurueckgeben
**	int RemoveFirst (Liste)						erstes Listenelement ausketten
** int Remove (Liste, Element)				Listenelement aus der Liste ausketten
**
**********************************************************************************************/

#include "lists.h"
int Enqueue();
int RemoveFirst();

int Insert(Liste, AktElem, Element)
	struct LKopf *Liste;
	struct LNode *AktElem, *Element;
/*
**	Das Element wird in der Liste nach dem AktElem eingefuegt.
**	ACHTUNG! Es wird nicht ueberprueft, ob AktElem wirklich in der Liste
**		eingekettet ist, die auch angegeben wurde!
**	Wenn AktElem oder Element nicht vorhanden sind wird abgebrochen!
**	Ansonsten wird Element eingefuegt und die Listenlaenge um eins erhoeht.
**
**	Returns:
**	0	alles ok (hoffentlich!)
**	1	Fehler (z.B. AktElemPtr oder Element == NULL)
**
*/
{
	if ((Element == (struct LNode *)NULL) ||
	    (AktElem == (struct LNode *)NULL))
		return(1);

	Element->Next = AktElem->Next;					/* neues Element nach dem AktElem einfuegen */
	AktElem->Next = Element;
	Liste->Laenge += 1;

	return(0);
}

int EnqueueHead(Liste, Element)
	struct LKopf *Liste;
	struct LNode *Element;
/*
**	Das Element wird am Anfang der Liste angehaengt und die Listenlaenge
**	erhoeht.
**	Element darf nicht NULL sein, d.h. es muss ein Element zum Anhaengen
**	vorhanden sein.
**
**	Returns:
**	0	alles ok (hoffentlich!)
**	1	Fehler beim Einhaengen des Elementes (z.B. Element==NULL)
**
*/
{
	if (Element == (struct LNode *)NULL)							/* kein Element zum Anhaengen */
		return(1);

	/* Das Element am Anfang der Liste einketten, die Liste an das Element
	   anhaengen und Laenge korrigieren!*/

	Element->Next = Liste->Anfang;
	Liste->Anfang = Element;
	Liste->Laenge += 1;

	if (Liste->Laenge == 1)							/* d.h. das eben eingefuegte Element ist das */
		Liste->Ende = Element;						/* einzige Element, und damit auch das Letzte! */

	return(0);
}

int Insort(Liste, Element)
	struct LKopf *Liste;
	struct LNode *Element;
{
/*
**	Das Element wird in die Liste Eingefuegt und die Listenlaenge erhoeht.
**	Es muss ein Element uebergeben werden (d.h. Element!=NULL), sonst wird
**	ein Returncode != 0 zurueckgegeben.
**
**	Returns:
**	0	alles ok (hoffentlich!)
**	1	Fehler beim Einhaengen des Elementes (z.B. Element==NULL)
**
*/
	struct LNode *AktElemPtr;

	if (Element == (struct LNode *)NULL)
		return(1);

	AktElemPtr = Liste->Anfang;									/* Zeiger auf erstes Listenelement */

	/* Sonderbehandlung, falls am Anfang eingefuegt werden muss: */
	if ((AktElemPtr == (struct LNode *)NULL) || /* d.h Liste ist leer     */
		 (AktElemPtr->Prio >= Element->Prio))	  /* d.h. Einfuegen vor dem
															     ersten Element         */
	{
		return(EnqueueHead(Liste, Element));					 /* mit Standardfunktion einfuegen */
	}

	while (AktElemPtr->Next != (struct LNode *)NULL)
	/* d.h. es folgt auf das aktuelle Element noch mindestens eins! */
	{
		if (AktElemPtr->Next->Prio < Element->Prio)
		{
			AktElemPtr = AktElemPtr->Next;
		}
		else
		{
			/* hier neues Element nach dem AktElem einfuegen */
			return(Insert(Liste, AktElemPtr, Element));
		}
	}

	/* nun ist AktElemPtr durch die ganze Liste gewandert und man muss am Ende einfuegen */

	return(Enqueue(Liste, Element));
}


int Enqueue(Liste, Element)
	struct LKopf *Liste;
	struct LNode *Element;
/*
**	Das Element wird ans Ende der Liste angehaengt und die Listenlaenge
**	erhoeht.
**	Element darf nicht NULL sein, d.h. es muss ein Element zum Anhaengen
**	vorhanden sein.
**
**	Returns:
**	0	alles ok (hoffentlich!)
**	1	Fehler beim Einhaengen des Elementes (z.B. Element==NULL)
**
*/
{
	if (Element == (struct LNode *)NULL)							/* kein Element zum Anhaengen */
		return (1);

	if (Liste->Ende == (struct LNode *)NULL) 
	{																						 	 /* Liste ist leer! */
		Liste->Ende	  = Element;
		Liste->Anfang = Element;
		Liste->Laenge += 1;
	}
	else
	{												 /* Liste ist nicht leer, => es gibt letztes Element */
		Liste->Ende->Next = Element;
		Liste->Ende			= Element;
		Liste->Laenge		+= 1;
	}
	Element->Next = (struct LNode *)NULL;				 /* sicherstellen, dass Verkettungsende! */

	return (0);																		 /* Alles glatt gelaufen. */
}

struct LNode *GetFirst (Liste)
	struct LKopf *Liste;
/*
**	Das erste Element der Liste wird ausgekettet und als Ergebnis ein Zeiger
**	auf dieses Zuruckgegeben. Zum Ausketten wird RemoveFirst() aufgerufen und
**	die Liste korrekt zurueckgelassen.
**
**	Returns:
**	NULL	Wenn kein Element in der Liste vorhanden war oder ein Fehler auf-
**		getreten ist (kann durch abpruefen von Liste.Laenge, erkannt werden)
**		sonst
**	Element	ein Zeiger auf ein ausgekettetes Listenelement.
**
*/
{
	struct LNode *Element;

	Element = Liste->Anfang;					/* Element wird Zeiger auf erstes Listenelement */

	if (Element != (struct LNode *)NULL)								/* d.h. Liste ist nicht leer */
	{
		if (RemoveFirst(Liste) != 0)											/* Fehler beim Ausketten */	
			Element = (struct LNode *)NULL;
	}
	
	return(Element);
}

int RemoveFirst (Liste)
	struct LKopf *Liste;
/*
**	Das erste Element der Liste wird ausgekettet und die Listenkopfstruktur
**	korrigiert. Bei einer leeren Liste ist nichts auszuketten, deshalb KEIN
**	Fehler!
**
**	Returns:
**	0		alles ok
**	!=0	Fehler
**
*/
{
	if (Liste->Anfang == (struct LNode *)NULL)								/* d.h. Liste ist leer */
		return(0);
	
	Liste->Anfang = Liste->Anfang->Next;

	if (Liste->Anfang == (struct LNode *)NULL)					/* es war einelementige Liste */
		Liste->Ende = (struct LNode *)NULL;
	
	Liste->Laenge -= 1;
	return(0);
}

int Remove(Liste, Element)
	struct LKopf *Liste;
	struct LNode *Element;
/*
**	Ein beliebiges Element soll aus einer Liste entfernt werden. Wenn das Element nicht
**	In der Liste enthalten ist, wird die Liste unveraendert zurueckgegeben.
**
**	Returns:
**	0		alles ok
** 1		Das Element wurde nicht in der Liste gefunden (, oder die Liste ist leer)!
**
*/
{
	struct LNode *AktElemPtr;

	if ((Element == (struct LNode *)NULL) ||	  /* Wenn es kein Element gibt, oder die Liste */
		 (Liste->Anfang == (struct LNode *)NULL))	 /* leer ist, gibt es nichts zu entfernen ! */
		return(1);

	if (Liste->Anfang == Element)						 	/* evtl. ist Element am Anfang der Liste */
		return(RemoveFirst(Liste));

	AktElemPtr = Liste->Anfang;							  /* Zeiger auf erstes Element der Liste */

	while (AktElemPtr->Next != (struct LNode *)NULL)
	{
		if (AktElemPtr->Next == Element)
		{
						  /* Element ist gefunden! Es ist das auf AktElem folgende Listenelement. */
			AktElemPtr->Next = AktElemPtr->Next->Next;
			Liste->Laenge	  -= 1;
			if (AktElemPtr->Next == (struct LNode *)NULL)   /* Letztes Element wurde entfernt! */
			{
				Liste->Ende = AktElemPtr;
			}
			return(0);
		}
		else
		{
			AktElemPtr = AktElemPtr->Next;
		}
	}
	return(1); 			/* Element ist in der Liste nicht vorhanden. Liste unveraendert lassen */
}		
