static char *SccsId = "@(#)PEPSY:\tmisc.c\t1.10\t94/02/04 11:23:12";
/**********************************************************************************************
**	ts=3
**
**
**	MODUL misc.c
**
**	Modul mit Funktionen, die den Simulationsablauf unmittelbar beeinflussen
**	------------------------------------------------------------------------
**
**	void 				 Job_in_WS(Job, aktWs)			Ein Auftrag in eine WS einhaengen
**	struct JOB		*Job_aus_WS(aktWs)				Einen Auftrag aus einer Warteschlange holen
**	void				 StartJob(Job, aktBe)			Einen Auftrag in einer BE (re-)starten
**	void				 SuspendJob(aktBe)				Einen Auftrag in einer BE unterbrechen
**	int				 GetPrevWS_NEMPTY(aktBe)		nichtleere VorgaengerWs ermittlen
**	int				 GetBe_IDLE(aktWs)				unbeschaeftigte NachfolgerBE ermitteln
**	int				 GetBe_IRQ(aktWs, aktK)			zu unterbrechende BE aussuchen
**	int				 Ermittle_Next(UEW, dimension) Mit UEW (KWW)-Vektor nachfolger ermitteln
**
**********************************************************************************************/

#include "stdio.h"
#include "data.h"

/*************************** BS-Funktionen ***************************************************/
extern void exit();
extern double drand48();

/*************************** Funktionen aus anderen Modulen **********************************/
extern struct EVENT	*CreateEvent(),									/* lowlevel.c */
							*AllocEvent();										/* memory.c */
extern void				 FreeEvent();										/* memory.c */
extern struct LNode	*GetFirst();										/* lists.c */
extern int				 Remove(),											/* lists.c */
			 				 Insort(),											/* lists.c */
							 RemoveFirst(),									/* lists.c */
							 EnqueueHead(),									/* lists.c */
							 Enqueue();											/* lists.c */
extern tunit			 GetBedZeit();										/* lowlevel.c */
extern struct JOB		*CreateJob(),										/* lowlevel.c */
							*ShortestJob();									/*	lowlevel.c */
extern void				 Liste_MakeEmpty(),								/* initsim.c */
							 DBG_EventList(),
							 DBG_JobList();

/*************************** (globale !) Externe Variablen ***********************************/ 
extern struct BE		*be;													/* alle aus main.c */
extern struct WS		*ws;
extern struct LKopf	 EventListe;
extern int	AnzBe,
				AnzWs,
				AnzKl;
extern tunit Time;
extern FILE	*ErrOut;

/******************************** Prozeduren und Funktionen **********************************/
void UpdatePS();		/* Forward-Deklaration */
void UpdateIS();		/* Forward-Deklaration */
void BE_belegen();	/* Forward-Deklaration */
int  Ermittle_Next();/* Forward-Deklaration */

void Job_in_WS(Job, aktWs)
	struct JOB *Job;
	int			aktWs;
/*
**	Der uebergebene Job wird in die ws[aktWs] nach dort vermerkter Strategie eingereiht.
**	Moegliche Strategien: (siehe data.h)
**	Ausserdem wird in der WS die Summe aller Laengen (WSLaenge) neu berechnet.
*/
{
	register int aktK;
	register int validWSLaenge;
	
	aktK = Job->Klasse;

#	ifdef DBG
		fprintf(ErrOut,"JOB_IN_WS  WS%03d KL%03d        JB%08.8lx\n",aktWs, aktK, Job);
#	endif

	validWSLaenge = ws[aktWs].WS_Data[aktK].AnzAuftraege;

	ws[aktWs].WS_Data[aktK].WSLaenge		+= (Time-ws[aktWs].WS_Data[aktK].StatUpdate)*validWSLaenge;
   ws[aktWs].WS_Data[aktK].StatUpdate	 = Time;
	ws[aktWs].WS_Data[aktK].AnzAuftraege	+= 1;

	if (Job->suspendTime != Time)
		ws[aktWs].WS_Data[aktK].PassedJobs	+= 1;

	switch (ws[aktWs].ListTyp)
   {
		case FCFS_LT:
		case SIRO_LT:
		case FCFS_RS_LT:
		case SIRO_RS_LT:
		case PS_LT: 		(void)Enqueue(&(ws[aktWs].JobListe), (struct LNode *)Job);
								break;
      case LCFS_RS_LT: 
      case LCFS_LT: 		(void)EnqueueHead(&(ws[aktWs].JobListe), (struct LNode *)Job);
  								break;
      default:      		fprintf(ErrOut, "Unbekannte WS-Strategie: %d!\n",
																				 (int)ws[aktWs].ListTyp);
	                		exit(-1);
   }
}

struct JOB *Job_aus_WS(aktWs)
	int aktWs;
/*
**	Aus der aktWs wird ein Job ausgekettet und ein Zeiger darauf zurueckgegeben. 
** ACHTUNG: Die Liste darf nicht leer sein!
**	Ausserdem wird in der WS die Summe aller Laengen (WSLaenge) neu berechnet.
*/
{
	struct JOB		*Job;
	register int	 validWSLaenge;
	register int	 aktK;
	
	switch(ws[aktWs].ListTyp)
	{
		case FCFS_RS_LT:
      case LCFS_RS_LT: 
		case FCFS_LT:
		case LCFS_LT:	Job = (struct JOB *)GetFirst(&(ws[aktWs].JobListe));
							break;
		case PS_LT	:	Job = be[ws[aktWs].Next_BE[0]].Job;
							(void)Remove(&ws[aktWs].JobListe, (struct LNode *)Job);

							break;
		default 		:	fprintf(ErrOut,
                             "FATAL: Unbekannte WS-Strategie %d in WS%03d!\n",
                             (int)ws[aktWs].ListTyp, aktWs
                            );
                     exit(-1);
	}

	aktK = Job->Klasse;
 
#	ifdef DBG
		fprintf(ErrOut,"JOB_AUS_WS WS%03d KL%03d        JB%08.8lx\n",aktWs, aktK, Job);
#	endif

	validWSLaenge = ws[aktWs].WS_Data[aktK].AnzAuftraege;
	
	ws[aktWs].WS_Data[aktK].WSLaenge		+= (Time-ws[aktWs].WS_Data[aktK].StatUpdate)*validWSLaenge;
   ws[aktWs].WS_Data[aktK].StatUpdate	 = Time;
	ws[aktWs].WS_Data[aktK].AnzAuftraege-= 1;

	return (Job);
}

void StartJob(Job, aktBe)
	struct JOB *Job;
	int aktBe;
/*
**	Der uebergebene Job wird auf der BE gestartet. Wenn die RestBearbZeit des Jobs nicht 0 ist,
** wird der Job fortgesetzt. Sonst wird nach in der BE vermerkter Vtlg eine RestBearbZeit er-
**	zeugt und der Job damit gestartet. Ein Ereignis, das die Fertigstellung betrifft, wird in 
**	die EventListe eingereiht.
**	Die Daten fuer die BelegtZeit der BE werden neu berechnet.
**
*/
{
	struct JOB		*job;
	int				 psWs, aktK;
	tunit				 ComplTime;
	
	if ((be[aktBe].Status == IDLE_ST) || (be[aktBe].BE_Typ == IS_BT) || (aktBe==0))
	{
		aktK = Job->Klasse;

		if (Job->RestBearbZeit == 0L)
			Job->RestBearbZeit = GetBedZeit(aktBe, aktK)
		;

		switch(be[aktBe].BE_Typ)
		{
			case NORMAL_BT	:
					ComplTime		 = Time + Job->RestBearbZeit;
					be[aktBe].Event = CreateEvent(COMPLETE_ET, aktBe, Job, ComplTime);
					(void)Enqueue(&be[aktBe].JobListe, (struct LNode *)Job);

					be[aktBe].StatUpdate = Time;
					be[aktBe].Status		= WORK_ST;
					be[aktBe].IRQStatus	= be[aktBe].BE_Data[aktK].IRQ;

					break;
			case MULSER_BT	:
			case IS_BT		:
					UpdateIS(aktBe);

					if (be[aktBe].Event != (struct EVENT *)NULL)	/* momentan zugeordnetes */
					{															/* Ereignis entfernen	*/
						(void)Remove(&EventListe, (struct LNode *)be[aktBe].Event);
						FreeEvent(be[aktBe].Event);
					}

					Job->Node.Prio = Time + Job->RestBearbZeit;	/* neuen Job nach aufsteigender 
																						Complet.Time einsortieren	*/
					(void)Insort(&be[aktBe].JobListe, (struct LNode *)Job);	

					job				 = (struct JOB*)be[aktBe].JobListe.Anfang;
					be[aktBe].Event = CreateEvent(COMPLETE_ET, aktBe, job, job->Node.Prio);

					be[aktBe].StatUpdate = Time;
					be[aktBe].Status		= WORK_ST;
					be[aktBe].IRQStatus	= be[aktBe].BE_Data[aktK].IRQ;

					break;
			case PS_BT		:
					psWs = be[aktBe].Prev_WS[0];

					UpdatePS(aktBe);
					Job_in_WS(Job, psWs);
					BE_belegen(aktBe);

#					ifdef DBG
						fprintf(ErrOut, "StartJob nachher:\n");
						DBG_JobList(&ws[psWs].JobListe);
#					endif

					break;
			default			:
					fprintf(ErrOut, "ERROR: Unbekannter BE-Typ in BE%d!\n", aktBe);
		}

#		ifdef DBG
			fprintf(ErrOut,"JOB_START  BE%03d KL%03d        JB%08.8lx\n",aktBe, aktK, Job);
#		endif
	}
	else
	{
		fprintf(ErrOut,"ERROR: Jobstart : BE%d not in IDLE-State!\n",aktBe);
	}
}

void EndJob(aktBe, Job)
	int aktBe;
	struct JOB *Job;
/* 
**	Der Job soll auf der aktuellen BE beendet werden. in EndJob werden alle statistischen Daten
**	der BE angepasst und der Job aus der BE hersausgeloest.
**
*/
{
	int			 aktK, psWs;

	aktK	= Job->Klasse;

#	ifdef DBG
		fprintf(ErrOut, "ENDJOB Time %lu\nJob{RBZ%lu}\nBeJob{RBZ%lu}\n", Time,
													Job->RestBearbZeit, be[aktBe].Job->RestBearbZeit);
#	endif

	switch (be[aktBe].BE_Typ)
	{
		case NORMAL_BT	:
			Job->RestBearbZeit                   = 0L;

			be[aktBe].BE_Data[aktK].BelegtZeit  += Time-be[aktBe].StatUpdate;
			be[aktBe].StatUpdate                 = Time;
	
			be[aktBe].Status     = IDLE_ST;
			be[aktBe].Event      = (struct   EVENT *)NULL;
			Liste_MakeEmpty(&be[aktBe].JobListe);

			break;
		case MULSER_BT	:
		case IS_BT	:
			Job->RestBearbZeit                   = 0L;

			UpdateIS(aktBe);
			be[aktBe].StatUpdate                 = Time;

			if (Job != (struct JOB *)be[aktBe].JobListe.Anfang)
			{
				fprintf(ErrOut, "FATAL: BE%d 1st Job ist nicht CompleteJob!\n", aktBe);
				exit(-1);
			}
			
			be[aktBe].Status     = IDLE_ST;
			be[aktBe].Event      = (struct   EVENT *)NULL;
			(void)RemoveFirst(&be[aktBe].JobListe);						/* fertigen Job entfernen */

			break;
		case PS_BT	:
			/* Das Eintreten eine COMPLETION-Events bei einer PS-BE macht eine Neuberechnung der
				Daten notwendig. Ein Auftrag muss nicht unbedingt fertig geworden sein. Dies dient
				der Rechenzeitersparnis
			*/
			psWs= be[aktBe].Prev_WS[0];

			UpdatePS(aktBe); 										/* Die angefallenen BZQuanten verteilen */

			Liste_MakeEmpty(&be[aktBe].JobListe);									  /* MerkListe leeren */

			Job = ShortestJob(&ws[psWs].JobListe);
/*			fprintf(ErrOut, "ENDJOB: ShortestJob: %lu\n", Job->RestBearbZeit); */

			while (	(Job != (struct JOB *)NULL)
					&& (Job->RestBearbZeit == (tunit)0))  /* es koennen auch mehrere Jobs fertig sein! */
			{
				/*	die BelegtZeit wird schon in UpdatePS aktualisiert!	*/
	
				be[aktBe].Job = Job;
				Job = Job_aus_WS(psWs);
/*				fprintf(ErrOut, "ENDJOB: Fertig: %lu\n", Job->RestBearbZeit); */

				(void)Enqueue(&be[aktBe].JobListe, Job); 		  /* merken fuer KW, statKW, UEW, ... */
			
				Job = ShortestJob(&ws[psWs].JobListe);
/*				fprintf(ErrOut, "ENDJOB: ShortestJob: %lu\n", Job->RestBearbZeit); */
			}

			be[aktBe].Status     = IDLE_ST;
			be[aktBe].Event      = (struct   EVENT *)NULL;

#			ifdef DBG
				fprintf(ErrOut, "ENDJOB: Fertige Jobs:\n");
				DBG_JobList(&be[aktBe].JobListe);
				fprintf(ErrOut, "EndJob nachher:\n");
				DBG_JobList(&ws[psWs].JobListe);
#			endif

			break;
		default		:
			fprintf(ErrOut, "FATAL: EndJob, unbekannter BE-Typ %d in BE%03d!\n", be[aktBe].BE_Typ,
																									  aktBe);
			exit(-1);
	}
}

void SuspendJob(aktBe, newKl)
	int aktBe;
	int newKl;
/*
**	Der Job der aktBe wird gestoppt, die RestBearbZeit neu berechnet und das zugehoerige Event
**	aus der EventListe ausgekettet und mit einem Event vermerkt, dass der Auftrag in eine WS
** zurueckgegeben wird. Die statistischen Daten werden angepasst.
**
*/
{
	struct JOB *Job;
	struct EVENT *Event;
	int aktK;

	if (be[aktBe].Status != IDLE_ST) 
	{
		if (be[aktBe].BE_Typ == NORMAL_BT)
		{
			Event = be[aktBe].Event;
			Job   = Event->Job;
			aktK	= Job->Klasse;

#			ifdef DBG
  	   		fprintf(ErrOut,"JOBSUSPEND BE%03d KL%03d        JB%08.8lx\n",aktBe, aktK, Job);
#			endif

			Job->RestBearbZeit = Event->Node.Prio-Time;
	
			be[aktBe].BE_Data[aktK].BelegtZeit += Time-be[aktBe].StatUpdate;
			be[aktBe].StatUpdate                = Time;
	
			be[aktBe].Status		= IDLE_ST;
			
			Liste_MakeEmpty(&be[aktBe].JobListe);
			be[aktBe].Event		= (struct EVENT *)NULL;
			be[aktBe].IRQStatus	= be[aktBe].BE_Data[aktK].IRQ;
	
			(void)Remove(&EventListe, (struct LNode *)Event);
			FreeEvent(Event);

			if (be[aktBe].BE_Data[aktK].IRQ[newKl] == PREEMPTRESTART)
			{
				Job->RestBearbZeit = 0L;
			}

			Job->Klasse = Ermittle_Next(be[aktBe].BE_Data[aktK].SuspendKWW, AnzKl);
			Job->suspendTime = Time;
			Job->suspendBE	  = aktBe;

			(void)CreateEvent(ARRIVAL_ET, be[aktBe].BE_Data[aktK].requeue, Job, Time);
		}
		else
		{
			fprintf(ErrOut, "ERROR: Kein JobSuspend in nicht NORMAL_BT-BE! aktBe:%d\n", aktBe);
		}
	}
	else
	{
		fprintf(ErrOut,"ERROR: JobSuspend in BE mit IDLE_ST! aktBE:%d\n", aktBe);
	}
}

boolean TestPrevWS_EMPTY(aktBe, maxWs)
	int aktBe;
	int maxWs;
/*
**	Sucht fuer die aktBe eine nichtleere Ws, die vor der maxWs in der Vorgaengerliste der BE
**	vorkommt. Wenn es eine solche gibt, liefere FALSE zurueck, sonst TRUE.
**	Zweck: Fuer stationaeren KW auf einer BE, der nur erlaubt ist, wenn alle Ws niedriger Prio-
**	ritaet leer sind.
**
*/
{
	int i=0;

	while ((be[aktBe].Prev_WS[i] != maxWs) && (i < be[aktBe].AnzVorgWS))
		if (ws[be[aktBe].Prev_WS[i++]].JobListe.Laenge > 0)
			return (FALSE)
	;
	return (TRUE);
}

int GetPrevWS_NEMPTY(aktBe)
	int aktBe;
/*
**	Sucht fuer die aktBe eine WS die mindestens einen Auftrag enthaelt.
**
*/
{
	int i, examineWs;

	i=0;

	while (i<be[aktBe].AnzVorgWS)
	{
		examineWs = be[aktBe].Prev_WS[i];

		if (ws[examineWs].JobListe.Laenge != 0)
			return(examineWs);
		i++;
	}
	return(NOTFOUND);
}

int GetBe_IDLE(aktWs)
	int aktWs;
/*
**	Es wird entweder NOTFOUND oder die erste BE, die von der aktWs versorgt wird und im IDLE-Zu-
**	stand ist zurueckgegeben. Bei _RS_ Warteschlangen wird erst die Zahl der moeglichen BEen
**	gezaehlt (count), dann eine Zufallszahl aus (1;count) ermittelt un diese BE zurueckgeliefert.
**
*/
{
	int	i,
			count,
			examineBe;

	switch (ws[aktWs].ListTyp)
	{
		case	FCFS_LT:
		case	LCFS_LT:
		case	SIRO_LT:
		case	PS_LT:	i = 0;
						while (i<ws[aktWs].AnzNachfolgBE)
							{
								examineBe = ws[aktWs].Next_BE[i];

								if (	(be[examineBe].Status == IDLE_ST)
									|| (be[examineBe].BE_Typ == IS_BT)
									||	(	(be[examineBe].BE_Typ == MULSER_BT)
										&& (be[examineBe].AnzBE > be[examineBe].JobListe.Laenge)))
									return(examineBe);
								i++;
							}
							break;
		case FCFS_RS_LT:
		case LCFS_RS_LT:
		case SIRO_RS_LT:	i = 0;
							count=0;
							while (i<ws[aktWs].AnzNachfolgBE)
							{
								examineBe = ws[aktWs].Next_BE[i];

								if (	(be[examineBe].Status == IDLE_ST)
									|| (be[examineBe].BE_Typ == IS_BT)
									||	(	(be[examineBe].BE_Typ == MULSER_BT)
										&& (be[examineBe].AnzBE > be[examineBe].JobListe.Laenge)))
									count++;
								i++;
							}

							if (count==0)
								return(NOTFOUND);

							/*	Eine Zufallszahl zwischen 1 und count ermitteln:
							*/
							count=1+(int)((double)count*drand48());


							/*	Die Bedieneinheiten erneut durchsuchen und die zufaellig ermittelte
								zuruckliefern
							*/
							i = 0;
							while (i<ws[aktWs].AnzNachfolgBE)
							{
								examineBe = ws[aktWs].Next_BE[i];

								if (	(be[examineBe].Status == IDLE_ST)
									|| (be[examineBe].BE_Typ == IS_BT)
									||	(	(be[examineBe].BE_Typ == MULSER_BT)
										&& (be[examineBe].AnzBE > be[examineBe].JobListe.Laenge)))
								{
									if (!--count)
									{
										return (examineBe);
									}
								}
								i++;
							}

#							ifdef DBG
								fprintf(ErrOut,"misc.c: Get_BeIDLE() count:%d i:%d\n", count, i);
#							endif

							break;
		default:			fprintf(ErrOut, "FATAL: Unbekannte WS-Strategie %d in WS%03d!\n",
																			  (int)ws[aktWs].ListTyp, aktWs);
                     exit(-1);
							break;
	}
	return(NOTFOUND);
}
 
int GetBe_IRQ(aktWs, aktK, job)
	int aktWs;
	int aktK;
	struct JOB *job;
/*
**	Es wird eine BE gesucht, die von der aktWs mit Auftraegen versorgt wird und deren Klasse
**	von der Klasse des neuen Auftrages unterbrochen werden darf. Dabei werden die Nachfolger
**	der WS mit fallender Prioritaet nacheinander getestet. 
**
**	Zusaetzlich wird ueberprueft, ob der Job schon zum aktuellen Zeitpkt von der BE ver-
**	draengt wurde. Dies darf nicht geschehen, deshalb wird evtl. weitergesucht.
**
*/
{
	int i, examineBe;

	i = 0;

   while (i<ws[aktWs].AnzNachfolgBE)
   {
      examineBe = ws[aktWs].Next_BE[i];

      if (be[examineBe].IRQStatus[aktK] != NOIRQ)
			if ((job->suspendBE != examineBe) || (job->suspendTime != Time))
         	return(examineBe);
      i++;
   }
	return(NOTFOUND);
}

int Ermittle_Next(UEW, dimension)
	int *UEW;
	int dimension;
/*
**	Ermitteln eines Nachfolgers mit Hilfe von UEW. Dazu wird eine Zufallszahl [0;1) erzeugt und
**	die UEW des Vektors solange aufaddiert, bis die Summe groesser ist als die Zufallszahl. 
**	Die UEW muessen als int vorliegen. Die Summe aller Werte im Vektor muss 1000 betragen.
**
*/
{
	register int i, x;

   i = 0; x = (int)(1000.0*drand48());

   while (i<dimension)
	{
		if (*UEW>x)
			return(i);
		i++; UEW++;
   } 

	fprintf(ErrOut, "FEHLER: Ermittle_Next() Kein Nachfolger gefunden!\nUEW:");

	for(i=0; i<dimension; i++)
		fprintf(ErrOut," %d",UEW[i]);
	fprintf(ErrOut," X: %d\n",x);

	return(NOTFOUND);
}

void UpdateIS(isBe)
	int isBe;
/*
**	In dieser Prozedur soll die Belegtzeit fuer jede Klasse neu berechnet werden. Dazu muss
**	die JobListe durchlaufen werden und fuer die jeweilige Klasse des aktJob die Belegtzeit
**	erhoeht werden (um Time-Update).
**
** ACHTUNG: be[isBe].StatUpdate wird nicht gesetzt. Um SeitenEffekt zu vermeiden!
**
*/
{
	struct JOB *aktJob;

	for (aktJob = (struct JOB *)be[isBe].JobListe.Anfang; aktJob != (struct JOB *)NULL;
														  aktJob = (struct JOB *)aktJob->Node.Next)
	{
		be[isBe].BE_Data[aktJob->Klasse].BelegtZeit += Time - be[isBe].StatUpdate;
	}
}

void UpdatePS(psBe)
	int psBe;
/*
**	Diese Prozedur soll einen PS-Knoten auf einen Zeitpunkt (Time) aktualisieren. Dazu sind
** die bisher angefallenen Bedienzeitquanten auf die Auftraege der PS-WS zu verteilen, die
**	WS neu zu ordnen und die statistischen Daten zu aktualisieren.
**
*/
{
	int psWs, l, n;
	tunit	g, worktime;
	struct JOB *Job;

	psWs	= be[psBe].Prev_WS[0];
	l		= ws[psWs].JobListe.Laenge;     /* Anzahl der gleichzeitig bearbeiteten Jobs */
																              /* Zahl der BedienZeitquanten */
	g		= (Time - be[psBe].StatUpdate) * (tunit)((l<be[psBe].AnzBE)?l:be[psBe].AnzBE);

#	ifdef DBG
		fprintf(ErrOut, "UpdatePS vorher:\ng:%lu l:%lu Time%lu\n",g ,l, (Time - be[psBe].StatUpdate));
		DBG_JobList(&ws[psWs].JobListe);
		DBG_EventList();
#	endif

	if ((l!=(tunit)0) && (g!=(tunit)0))	 /* Wenn ueberhaupt schon Zeit auf die Jobs der WS ver-
																	 wendet wurde und die WS nicht leer ist */
	{
		n=0;
		for (Job=(struct JOB *)ws[psWs].JobListe.Anfang; Job != (struct JOB *)NULL;
																			Job = (struct JOB *)Job->Node.Next)
		{
			worktime =  g/(tunit)l+(tunit)(n<(g%(tunit)l));

			if (worktime > Job->RestBearbZeit)
				fprintf(stderr, "Negative BZ! wt:%lu jrbz:%lu\n", worktime, Job->RestBearbZeit);

			Job->RestBearbZeit								-= worktime;
			be[psBe].BE_Data[Job->Klasse].BelegtZeit  += worktime;
			n++;
		}

		/* Auftraege, die ein Bedienzeitquant mehr bekommen haben hinten an die Liste anreihen */
		for (n=0; n<(g%l); n++)
			(void)Enqueue(&ws[psWs].JobListe, GetFirst(&ws[psWs].JobListe));
	}

	be[psBe].StatUpdate = Time;

#	ifdef DBG
		if (be[psWs].Job != (struct JOB *)NULL)
			fprintf(ErrOut, "Job {KL%d RBZ%lu}\n", be[psBe].Job->Klasse, be[psBe].Job->RestBearbZeit);
		fprintf(ErrOut, "UpdatePS nachher:\n");
		DBG_JobList(&ws[psWs].JobListe);
#	endif

}

void BE_belegen(aktBe)
	int aktBe;
/*
**	Nach einer Auftragfertigstellung muss geprueft werden, ob ein neuer Auftrag auf die BE
**	wartet und gegebenenfalls muss dieser gestartet werden.
**	Bei einem IS-Knoten werden nie Jobs warten, ebenso bei PS-Knoten. Bei diesen Knotentypen
**	muss daher nur das neue Ereignis in der EventList eingetragen werden.
**
*/
{
	int			sourceWs, psWs;
	tunit			ComplTime;
	struct JOB *Job;

	switch (be[aktBe].BE_Typ)
	{
		case NORMAL_BT	:
		case MULSER_BT	:
					if	((sourceWs=GetPrevWS_NEMPTY(aktBe)) != NOTFOUND)	/* Nachsehen, ob es Job in	*/
					{																		/*			ZulieferWS gibt	*/
						Job =	(struct JOB	*)Job_aus_WS(sourceWs);
						StartJob(Job, aktBe);
					}

					break;
		case PS_BT	:
					psWs = be[aktBe].Prev_WS[0];

					if (be[aktBe].Event != (struct EVENT *)NULL)
					{
						(void)Remove(&EventListe, be[aktBe].Event);
						FreeEvent(be[aktBe].Event);
					}

					if (ws[psWs].JobListe.Laenge == 0)
					{
			        	be[aktBe].Event	      = (struct   EVENT *)NULL;
						be[aktBe].Job				= (struct JOB *)NULL;
					}
					else
					{
						Job = be[aktBe].Job = ShortestJob(&ws[psWs].JobListe);

						/* minimale Zahl von notwendigen ZE berechnen */
						ComplTime = Time + (tunit)
							((float)(ws[psWs].JobListe.Laenge)*(float)(Job->RestBearbZeit)/
							(float)((ws[psWs].JobListe.Laenge<be[aktBe].AnzBE)?ws[psWs].JobListe.Laenge:be[aktBe].AnzBE));
						be[aktBe].Event = CreateEvent(COMPLETE_ET, aktBe, be[aktBe].Job, ComplTime);
					}

					break;
		case IS_BT	:
					if (be[aktBe].JobListe.Laenge == 0)
					{
						be[aktBe].Status     = IDLE_ST;           /* ...die BE ist jetzt unbeschaeftigt */
        			 	be[aktBe].Event      = (struct   EVENT *)NULL;
					}
					else
					{
						Job						= (struct JOB *)be[aktBe].JobListe.Anfang;
						be[aktBe].Status		= WORK_ST;
						be[aktBe].Event		= CreateEvent(COMPLETE_ET, aktBe,
																							Job, (tunit)Job->Node.Prio);
					}

					break;
		default	:
					fprintf(ErrOut, "FATAL: BE_belegen() Unbekannter BE-Typ in BE%03d: %d!\n",
																							aktBe, be[aktBe].BE_Typ);
					exit(-1);
	}
}
