static char *SccsId = "@(#)PEPSY:\trnd.c\t1.5\t94/02/04 11:23:22";
/**********************************************************************************************
**
**	Modul mit Funktionen zur zufaelligen Ermittelung von Bedienzeiten
**	-----------------------------------------------------------------
**
**	Alle Funktionen sollen eine Anzahl von ZE ermitteln - gemaess der Zufallsverteilung die im-
**	plementiert wurde - und diese als 'unsigned long' (= tunit) zurueckliefern.
**	Fuer jede Vtlg ist in modell.h eine TypVariable (EXPON_VTLG, CONST_VTLG, ...) vorzusehen
**	und in den Modulen initsim.c und lowlevel.c sind Fallunterscheidungen daraufhin anzupassen.
**	Wenn eine Vtlg nicht ueberall bekannt gemacht wird werden Fehlermeldungen ausgegeben.
**
**	Implementiert sind schon:
**
**	EXPON_VTLG	:	tunit exp_rnd(mue)					exponentiell verteile Bedienzeiten
**	LOGNORM_VTLG:	tunit lognorm_rnd(mue, sigma2)		lognormal-verteilte BZ
**	COX_VTLG	:	tunit cox_rnd(mue, p, k)			
**	COX2_VTLG	:	tunit cox2_rnd(mue1, mue2, p)		coxverteilte BZ mit zwei Phasen
**	DETEXP_VTLG	:	tunit detexp_rnd(mue1, mue2)		determin. gefolgt von exponentieller Phase
**	HYPEREXP_VTLG:	tunit hyperexp_rnd(mue1, mue2, p)	hyperexponentiell verteilt BZ
**	HYPOEXP_VTLG:	tunit hypoexp_rnd(mue1, mue2)		Hypoexponentialverteilung
**	GE_VTLG		:	tunit ge_rnd(mue, p)				General-Erlang-Verteilung
**	NORMAL_VTLG	:	tunit normal_rnd(mue, sigma2)		normalverteile Bedienzeiten
**	PRIVATE_VTLG:	tunit private_rnd(mue, sigma2)		anwenderspezifizierte BZVtlg
**
**********************************************************************************************/

#include "data.h"
/* #include "limits.h" */

extern double drand48();
extern double log();
extern double exp();
extern double sqrt();

double drandnorm()
/*
**	Algorithmus N2 aus Fishman: Principles of discrete event simulation
**	S. 412
**
*/
{
	static int A=0;
	static double W1, W2;
	double U1, U2, V, B;
	int S, T;

	if (A==1)
	{
		A = 0;
		return(W2);
	}
	else
	{
		A = 1;
		do
		{
			U1 = 2.0*drand48();
			if (U1>1.0)
			{
				S = 0; U1 = U1 - 1.0;
			}
			else
				S = 1;

			U2 = 2.0*drand48();
			if (U2>1.0)
			{
				T = 0; U2 = U2 - 1.0;
			}
			else
				T = 1;

			B = U1*U1 + U2*U2;
		} while (B>1.0);

		V = -log(1.0-drand48());
		W1 = U1*sqrt(2*V/B);
		W2 = U2*sqrt(2*V/B);
		if (S==0) W1 = - W1;
		if (T==0) W2 = - W2;
		return (W1);
	}
}

tunit exp_rnd(mue)
	double mue;
{
	/*	Fuer Rundung Summand 0.5			22.05.92 kirschni
	*/
	return((tunit)(0.5+(-1.0/mue)*log(1.0-drand48())));
}

tunit cox_rnd(mue, p, k)
	double mue;
	int	 p;
	int 	 k;
{
	tunit r;

	r = exp_rnd(mue); k--;

	while (k>0)
	{
		if ((int)(1000.0*drand48())>=p)
 			r += exp_rnd(mue);
		
		k--;
	}

	  return (r);
}

tunit cox2_rnd(mue1, mue2, p)
  double mue1,
			mue2;
		  int p;
{
	tunit r;

	r = exp_rnd(mue1);

	if ((int)(1000.0*drand48())>=p)
		r += exp_rnd(mue2)
	;

	return(r);
}


tunit detexp_rnd(mw1, mue2)
	tunit	mw1;
	double mue2;
{
	return(mw1+exp_rnd(mue2));
}

tunit hyperexp_rnd(mue1, mue2, p)
	double mue1,
			 mue2;
	int	 p;
{
	if (p>(int)(1000.0*drand48()))
		return(exp_rnd(mue1));
	else
		return(exp_rnd(mue2));
}

tunit hypoexp_rnd(mue1, mue2)
	double mue1,
			 mue2;
{
	return(exp_rnd(mue1)+exp_rnd(mue2));
}

tunit ge_rnd(mue, p)
	double mue;
	int p;
{
	if (p>(int)(1000.0*drand48()))
		return(exp_rnd(mue));
	else
		return((tunit)0);
}

tunit normal_rnd(mw, sigma)
	double mw;
	double sigma;
{
	tunit h;

	h = (drandnorm()*sigma+mw)+0.5;

	return ((tunit)((h<0.0)?0.0:h));
}

tunit lognorm_rnd(mw, sigma)
	double mw,
			 sigma;
/*
** Algorithmus aus Fishman: Principles of discrete event simulation
**	S. 417
**
*/
{
	double h;

	h = log(drandnorm()*sigma+mw);

	return ((tunit)h);
}

tunit private_rnd(mw, quadvarkoeff)
	double mw;
	double quadvarkoeff;
{
	quadvarkoeff += (double)0.0;
	mw += (double)0.0;
	return(1L);
}
