/*

	                     P E P S Y 

		  Performance and Evaluation Prediction System

	Copyright 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995
	
			  Universitaet Erlangen-Nuernberg,
			  Lehrstuhl fuer Betriebssysteme
			  Martensstrasse 1

			  D-91058 Erlangen

	Alle Rechte, einschliesslich Verfilmung und Vertonung, vorbehalten.
	PEPSY darf ohne Erlaubnis der Verfasser nicht in Lesezirkeln
	gefuehrt werden.

*/
static char *RcsId="@(#)PEPSY:\t$Id: statistik.c,v 1.3 1995/12/14 10:19:36 i4pepsy Exp i4pepsy $";

#include "stdio.h"
#include "data.h"

/**************************** BS-Funktionen ***********************************************/
extern double sqrt();

/*************************** externe Funktionen *******************************************/
extern struct LNode *GetFirst();
extern void BE_belegen(),
				UpdatePS();

/************************* globale Variablen (aus main.c!) ********************************/
extern int AnzWs, AnzKl, AnzBe, AnzBa, aktBa;
extern int KILimit;
extern boolean	trace;
extern struct BE *be;
extern struct WS *ws;
extern struct AUSSEN *AussenWelt;
extern struct LKopf EventListe;
extern tunit BatchLaenge,
				Time;
extern FILE	*tracefp,
				*ErrOut;

void GetKI(ki, X, n)
	struct KI  *ki;
	double	  *X;
	int			n;
/*
**	Zu gegebener Stichprobensumme, Stichprobenquadratsumme, Stichprobengroesse und einer
**	Konstanten c, die aus der KI-Konfidenzzahl berechnet wird, wird ein KI berechnet.
**	Die Struktur KI enthaelt den Mittelwertund die Intervallbreite absolut und in Prozenten
**
*/
{
    int i;
    double sum, sum2;
    double	c;	

    /*
      Breite des Intervalls einer t-verteilten Zufallsvariable bei 95% Trefferwahrschein-
      lichkeit variiert ueber die Zahl der Freiheitsgrade (1-25). Fuer sehr grosse Stich-
      proben (sehr grosses n) naehert sich die Verteilungsfunktion der Gaussschen Einheits-
      verteilung, deshalb ist fuer n>120 c=1.96.
      Die Tabelle ist entnommen aus:
      Kreyszig: [...]
      */
    static double Student_verteilt[26] = {
	0.0, 12.71,	4.30,	3.18,	2.78,	2.57,	2.45,	2.37,	2.31,	2.26,	2.23,	2.20,	2.18,	2.16,
	2.15,	2.13,	2.12,	2.11,	2.10,	2.09,	2.09,	2.08,	2.07,	2.07,	2.06,	2.06
    };

    if (n <= 0) 
    {
	fprintf(ErrOut, "GetKI Fehler! Stichprobenlaenge <= 0 !!!\n");
	return;
    }
    if ((n>0) && (n<26))
    {
	c = Student_verteilt[n];
    }
    else if (n<30)
    {
	c = 2.04;
    }
    else if (n<40)
    {
	c = 2.02;
    }
    else if (n<60)
    {
	c = 2.00;
    }
    else if (n<120)
    {
	c = 1.98;
    }
    else
    {
	c = 1.96;
    }

    sum = sum2 = 0.0;
    if (n>1)
    {
	for (i=0; i<n; i++)
	{
	    sum	+= X[i];
	    sum2	+= X[i]*X[i];
	}
	ki->mw = sum/(double)n;

	ki->breite = c * sqrt ((sum2-sum*sum/(double)n)/(double)((n-1)*n));

	ki->quality = (ki->mw==0.0)?(0.0):(ki->breite*100.0/ki->mw);
    }
    else
    {
	ki->mw = ki->breite = ki->quality = 0.0;
    }
}

void FalteBatches(batch)
	double *batch;
/*
**	Durch Zusammenlegen von zwei benachbarten Batches zu einem wird die Zahl der Batches
**	halbiert.
**
*/
{
    int b, first, second;

    b	 = 0;
    first	 = b*2;
    second = b*2+1;

    while ((first<AnzBa)&&(second<AnzBa))
    {
	first		= b*2;
	second	= b*2+1;
	batch[b] = 0.5*(batch[first]+batch[second]);
	b++;
    }
}

void Batches_zuslegen()
/*
**	Bei allen BE, WS und der AussenWelt werden die Batches zusammengefasst und somit Platz
**	fuer weitere geschaffen.
**
*/
{
    int i, k;

    for(k=0; k<AnzKl; k++)
    {
	FalteBatches(AussenWelt[k].BatchMean);
    }

    for(i=0; i<=AnzBe; i++)
    {
	for(k=0; k<AnzKl; k++)
	{
	    FalteBatches(be[i].BE_Data[k].BatchMean);
	}
    }
    for(i=0; i<=AnzWs; i++)
    {
	for(k=0; k<AnzKl; k++)
	{
	    FalteBatches(ws[i].WS_Data[k].BatchMean);
	}
    }
}

void GetBatch()
/*
**	Ein Simulationszeitintervall ist abgelaufen. Jetzt werden die Daten fuer die Statistik
**	auf den aktuellen Zeitpunkt angeglichen und neue Batchmittelwerte erfasst und in die 
**	dafuer vorgesehenen Datenstrukturen eingetragen.
**
*/
{
    int				 i, k;
    boolean			 BatLen_verdoppeln;
    struct BE_DATA *be_data;
    struct WS_DATA *ws_data;
    struct EVENT   *Event;
    struct JOB		*Job;

    aktBa ++;
    BatLen_verdoppeln = FALSE;

    if (aktBa>=AnzBa)
    {
	Batches_zuslegen();
	BatLen_verdoppeln	= TRUE;
	aktBa = (int)(AnzBa/2);
    }

    if (trace)
	fprintf(tracefp, "Batch %d. Time %ld\n", aktBa, Time);

    /* erst die statistischen Daten auf Batchende aktualisieren */
    for (Event=(struct EVENT*)(EventListe.Anfang); Event!=(struct EVENT *)NULL;
	 Event = (struct EVENT *)Event->Node.Next)
    {
	if (Event->Typ == COMPLETE_ET)
	{
	    i = Event->Nr.Verursacher;
	    if (i!=0)
	    {
		switch(be[i].BE_Typ)
		{
		case MULSER_BT	:
		case NORMAL_BT	:
		case IS_BT		:
		    for (Job=(struct JOB*)be[i].JobListe.Anfang; Job != (struct JOB *)NULL;
			 Job = (struct JOB *)(Job->Node.Next))
		    {
			be[i].BE_Data[Job->Klasse].BelegtZeit	+= Time-be[i].StatUpdate;
			Job->RestBearbZeit							-= Time-be[i].StatUpdate;
		    }

		    break;
		case PS_BT		:
		    UpdatePS(i);
		    BE_belegen(i);

		    break;
		default			:
		    fprintf(ErrOut, "ERROR: Unbekannter BE-Typ in BE%d!\n", i);
		}
		be[i].StatUpdate = Time;
	    }
	}
    }
    for(k=0; k<AnzKl; k++)
    {
	if (AussenWelt[k].BatchJobs!=0)
	{
	    AussenWelt[k].BatchMean[aktBa] = (double)AussenWelt[k].Verweilzeit /
		(double)AussenWelt[k].BatchJobs;
	}
	else
	{
	    AussenWelt[k].BatchMean[aktBa] = (double) 0.0;
	}
	if (trace)
	{
	    fprintf(tracefp,"AUSSEN KL%03d %f\n",k ,AussenWelt[k].BatchMean[aktBa]);
	    fprintf(tracefp,"AUSJOB KL%03d %d\n",k ,AussenWelt[k].BatchJobs);
	}

	AussenWelt[k].PassedJobs += AussenWelt[k].BatchJobs;
	AussenWelt[k].BatchJobs   = 0;
	AussenWelt[k].Verweilzeit = 0L;
    }
    for (i=0; i<=AnzWs; i++)
    {
	ws_data = ws[i].WS_Data;
	for (k=0; k<AnzKl; k++)
	{
	    /* erst die statistischen Daten auf Batchende aktualisieren */
	    ws_data[k].WSLaenge     += (Time-ws_data[k].StatUpdate) * ws_data[k].AnzAuftraege;
	    ws_data[k].StatUpdate    = Time;

	    if (ws[i].ListTyp == PS_LT)
	    {
		ws_data[k].BatchMean[aktBa] =
		    (double)(ws_data[k].WSLaenge - be[ws[i].Next_BE[0]].BE_Data[k].BelegtZeit)
		    / (double)BatchLaenge;
	    }
	    else
	    {
		ws_data[k].BatchMean[aktBa] = (double)ws_data[k].WSLaenge/(double)BatchLaenge;
	    }
	    ws_data[k].WSLaenge         = 0L;
	    if (trace)
		fprintf(tracefp,"WS%03d KL%03d %f\n",i ,k ,ws_data[k].BatchMean[aktBa]);
	}
    }

    for (i=0; i<=AnzBe; i++)
    {
	be_data = be[i].BE_Data;
	for (k=0; k<AnzKl; k++)
	{
	    be_data[k].BatchMean[aktBa] =  (double)be_data[k].BelegtZeit/(double)BatchLaenge;
	    be_data[k].BelegtZeit       =  0L;
	    if (trace)
		fprintf(tracefp,"BE%03d KL%03d %f\n",i ,k ,be_data[k].BatchMean[aktBa]);
	}
    }

    if (BatLen_verdoppeln)
    {
	BatchLaenge *= 2L;
    }
}

void InitStats()
{
    int i, k;

    for (k=0; k<AnzKl; k++)
    {
	AussenWelt[k].Verweilzeit	= 0L;
	AussenWelt[k].PassedJobs	= 0;
	AussenWelt[k].BatchJobs	= 0;
    }
    for (i=0; i<=AnzBe; i++)
    {
	for (k=0; k<AnzKl; k++)
	{
	    be[i].BE_Data[k].BelegtZeit	= 0L;
	    be[i].BE_Data[k].PassedJobs	= 0;
	}
	be[i].StatUpdate = Time;
    }
    for (i=0; i<=AnzWs; i++)
    {
	for (k=0; k<AnzKl; k++)
	{
	    ws[i].WS_Data[k].WSLaenge	 	= 0;
	    ws[i].WS_Data[k].StatUpdate 	= Time;
	    ws[i].WS_Data[k].PassedJobs	= 0;
	}
    }
    aktBa = -1;
}

boolean TestBatches()
{
/****************************************************************************
  return(UNABHAENGIG);
  *****************************************************************************
  Thanks to Babette for the bug fix. Now the check can be performed and
  the bloody imediate return is obsolete! Matthias (Damned typo's!)
  ****************************************************************************/
    double *Y;
    double nenner, zaehler, mean, grenze, summand, Ck;
    int    aktBe, i, j, k; boolean r;

  return(UNABHAENGIG);
    r = UNABHAENGIG;

    if (trace)
	fprintf(ErrOut, "Batch %d\n", aktBa);

    if (aktBa<8)
	return (!UNABHAENGIG);

    k		= aktBa+1;
    grenze	= 1.645*sqrt((double)(k-2)/(double)(k*k-1));

    for (aktBe=0; aktBe<=AnzBe; aktBe++)
    {
	for (j=0; j<AnzKl; j++)
	{
	    Y = be[aktBe].BE_Data[j].BatchMean;
	    zaehler = nenner = mean	= 0.0;

	    for (i=0; i<k-1; i++)
	    {
		summand = Y[i]-Y[i+1];
		zaehler += summand*summand; 
	    }

	    for (i=0; i<k; i++)
		mean += Y[i];
	    mean = mean/(double)(aktBa+1);

	    for (i=0; i<k; i++)
	    {
		summand = (Y[i] - mean);
		nenner += summand*summand;
	    }

	    if (nenner != 0.0)
	    {
		Ck = 1.0 - (zaehler/(2.0*nenner));
		Ck = (Ck<0.0)?-Ck:Ck;
	
		if (Ck>grenze)
		    r = !UNABHAENGIG;
	
		if (trace)
		    fprintf(ErrOut, "BE%d KL%d VAR%f ZAE%f CK%f GRENZE%f :%s\n", aktBe, j,
			    (double)nenner/k, zaehler, Ck, grenze,
			    (Ck<=grenze)?"UNABHAENGIG":"NICHT UNABHAENGIG");
	    }
	}
    }
    return (r);
}

boolean TestKI(worstKIQuality)
double *worstKIQuality;
{
    struct KI *ki;
    int aktBe, aktWs, aktK;
    boolean r;

    *worstKIQuality = 0.0;

    r = OK;	/* "Beweis durch Widerspruch" */

    for (aktBe=0; aktBe<=AnzBe; aktBe++)
    {
	for (aktK=0; aktK<AnzKl; aktK++)
	{
	    ki = &be[aktBe].BE_Data[aktK].ki;

	    GetKI(ki, be[aktBe].BE_Data[aktK].BatchMean, aktBa+1);

	    if ((ki->mw < 0.01) || (be[aktBe].BE_Data[aktK].PassedJobs==0))
	    {
		/* wenn der MW zu klein ist, soll die Guete des KI keinen Einfluss auf den
		   Abbruch der Simulation haben. Deshalb wird hier r nicht geaendert!	*/
	    }
	    else
	    {
		if (ki->quality>*worstKIQuality)
		    *worstKIQuality = ki->quality;

		if (ki->quality>(((double)KILimit)/10.0))
		    r = NOTOK;
	    }
	}
    }

    for (aktWs=1; aktWs<=AnzWs; aktWs++)
    {
	for (aktK=0; aktK<AnzKl; aktK++)
	{
	    ki = &ws[aktWs].WS_Data[aktK].ki;
	    GetKI(ki, ws[aktWs].WS_Data[aktK].BatchMean, aktBa+1);

	    if ((ki->mw < 0.01) || (ws[aktWs].WS_Data[aktK].PassedJobs == 0))
	    {
		/* wenn der MW zu klein ist, soll die Guete des KI keinen Einfluss auf den
		   Abbruch der Simulation haben. Deshalb wird hier r nicht geaendert!	*/
	    }
	    else
	    {
		if (ki->quality>*worstKIQuality)
		    *worstKIQuality = ki->quality;

		if (ki->quality>(((double)KILimit)/10.0))
		    r = NOTOK;
	    }
	}
    }

    return (r);
}
