static char *SccsId = "@(#)PEPSY:\tggen.c\t1.2\t92/03/27 11:56:07";
#include <stdio.h>
#include <fcntl.h>
#include "gpss_prepare.h"
#include "gpss_decl.h"


int	ggen(et, pr, kap, vel, nterm, nstrat)
float	et;
float	pr;
int	kap;
float	vel;
int	nterm;
int	nstrat;
/* Hier ist mindestens ein alternativer Ruecksprung zu beruecksichtigen */
{
	/*     ***  */
	/*     *** CALL GGEN(ET,PR,KAP,VEL,NTERM,NSTRAT,*9999) */
	/*     ***  */
	/*     *** PURPOSE : GENERATE A TRANSACTION REPRESENTING A TRANSPORTER  */
	/*     *** PARAMETERS: ET     = TIME BETWEEN SUCCESSIVE GENERATIONS */
	/*     ***             PR     = PRIORITY OF THE TRANSPORTER GENERATED */
	/*     ***             KAP    = CAPACITY OF THE TRANSPORTER */
	/*     ***             VEL    = SPEED OF THE TRANSPORTER */
	/*     ***             NTERM  = TERMINAL AT WHICH THE TRANSPORTER */
	/*     ***                      IS GENERATED */
	/*     ***             NSTRAT = STRATEGY, ACCORDING TO WHICH THE */
	/*     ***                      TRANSPORTER SELECTS TRANSACTIONS */
	/*     ***                      = 0:       NO SELECTION (AS FOR BUS) */
	/*     ***                      = 1: OLD:  OLDEST REQUEST */
	/*     ***                      = 2: PRI:  TRANSACTION WITH HIGHEST */
	/*     ***                                 PRIORITY */
	/*     ***                      = 3: SDS:  SHORTEST DISTANCE  */
	/*     ***                      = 4: LDS:  LONGEST DISTANCE */
	/*     ***                      = 5: TOP:  PRESENT TERMINALS  */
	/*     ***                      = 6: BEN:  STRATEGY PROVIDED BY USER */
	/*     ***             *9999  = ERROR EXIT */
	/*     ***  */
	/* */

	int	blocnr, ctx, nact, slist;
	short	found, trace;
	extern int	tchain(), next_int();


	/*     TEST FOR ERRORS */
	/*     =============== */
	if (et < -eps)                  
		goto l_4000;
	if (kap < 0)                    
		goto l_4100 ;
	if (vel <= eps)                 
		goto l_4200 ;
	if (nterm < 1 || nterm > ansta) 
		goto l_4300 ;
	if (nstrat < 0 || nstrat > 6)   
		goto l_4400 ;
	if (row_source == NULL)         
		goto l_4500;
	if (tx_ctr >= txmax || 
	    row_source->remain == 0)    
		goto l_4600;
	if (row_source->sourci != 3)    
		goto l_4700;
	/* */
	/*     UPDATE COUNTER */
	/*     ============== */
	tx_ctr = tx_ctr + 1;
	row_source->remain = row_source->remain - 1;
	/* */
	/*     GENERATE */
	/*     ======== */
	for (row_txact = ztx; row_txact >= 1; row_txact--)  {
		if (tx_system[row_txact].nr == 0) 
			goto l_200;
	}
	fprintf (nunit2, "\n\n+++++ ggen  : t=%12.4f ERROR  :  overflow of transaction matrix\n", TiMe);
	goto l_99;
	/* */
l_200: 
	tx_system[row_txact].nr = tx_ctr;
	tx_system[row_txact].priority = pr;
	txi[row_txact]  = 3;
	for (ctx = 1; ctx <= zcar; ctx++)  {
		if (next_int(carma[ctx][1]) == 0) 
			goto l_400;
	}
	fprintf (nunit2, "\n\n+++++ ggen  : t=%12.4f ERROR: overflow of carma array\n", TiMe);
	goto l_99;
	/* */
	/*     ASSIGN TXADD */
	/*     ============ */
l_400: 
	txadd[row_txact][1] = (double)ctx;
	/* */
	/*     INITIALIZE CARMA ARRAY */
	/*     ====================== */
	carma[ctx][1]  = (double)row_txact;
	carma[ctx][2]  = -1.0;
	carma[ctx][3]  = (double)kap;
	carma[ctx][4]  = 0.0;
	carma[ctx][5]  = vel;
	carma[ctx][6]  = (double)nterm + 0.1;
	carma[ctx][7]  = 3.0;
	carma[ctx][8]  = 0.0;
	carma[ctx][9]  = (double)nstrat;
	carma[ctx][10] = -1.0;
	/* */
	/*     INITIALIZE TARMA ARRAY */
	/*     ====================== */
	tarma[ctx][1] = 0.0;
	tarma[ctx][2] = 0.0;
	tarma[ctx][3] = 0.0;
	tarma[ctx][4] = 0.0;
	tarma[ctx][5] = 0.0;
	tarma[ctx][6] = 0.0;
	tarma[ctx][7] = 0.0;
	/* */
	trace = ((iprint == 1 || jprint[23] > 0) && jprint[23] != -1);
	slist = row_source - sourcelist;
	if (trace) 
		fprintf (nunit2, "ggen  :  t=%12.4f  transporter%4d generated by source%3d at terminal%4d\n",
		    TiMe, next_int(txadd[row_txact][1]), slist, nterm);
	if (kprint == 1) {
		nact = 30;
		fprintf (nunit23, "%12.4f%4d%4d%4d%4d\n", TiMe, txi[row_txact], ctx, nact, nterm);
	}
	/* */
	/*     SCHEDULE THE NEXT GENERATION */
	/*     ============================ */
	if (tx_ctr != txmax)  {
		if (row_source->remain != 0) {
			row_source->next = TiMe + et;
			if (row_source->link != 0) 
				goto l_4800;
			tchain(2, slist);
l_500:      
			if (trace) 
				fprintf (nunit2, "                         next generation at t=%12.4f\n", row_source->next);
		} else {
			/* */
			/*     GENERATION OF TRANSPORTERS BY THE SOURCE COMPLETE */
			/*     ================================================= */
			if (trace) 
				fprintf (nunit2, "                         source%3d is closed down\n", row_source - sourcelist);
		}
	} else {
		head[2].next = 0.0;
		head[2].link = -1;
		if (trace) 
			fprintf (nunit2, "                         all sources closed down\n");
	}
	/* */
	return(NORMAL);
	/* */
	/*     ERROR MESSAGES */
	/*     ============== */
l_4000:  
	fprintf (nunit2, "\n\n+++++ ggen  :  t=%12.4f ERROR : negative TiMe between successive generations (%8.4f)\n", TiMe,
	     et);
	goto l_99;
	/* */
l_4100:  
	fprintf (nunit2, "\n\n+++++ ggen  :  t=%12.4f ERROR : negative capacity (%3d) of the transporter to be generated \n",
	     TiMe, kap);
	goto l_99;
	/* */
l_4200:  
	fprintf (nunit2, "\n\n+++++ ggen  :  t=%12.4f ERROR : negative speed (%8.4f) of the transporter to be generated\n",
	     TiMe, vel);
	goto l_99;
	/* */
l_4300:  
	fprintf (nunit2, "\n\n+++++ ggen  :  t=%12.4f ERROR : invalid terminal(%4d) of the transporter to be generated\n",
	     TiMe, nterm);
	goto l_99;
	/* */
l_4400:  
	fprintf (nunit2, "\n\n+++++ ggen  :  t=%12.4f ERROR : invalid strategy (%3d) of the transporter to be generated\n",
	     TiMe, nstrat);
	goto l_99;
	/* */
l_4500:  
	fprintf (nunit2, "\n\n+++++ ggen  :  t=%12.4f ERROR : in index of source = 0\n                                        check label selector\n",
	     TiMe);
	goto l_99;
	/* */
l_4600:  
	fprintf (nunit2, "\n\n+++++ ggen  :  t=%12.4f ERROR : source with index%3d exceeds transaction limit\n", TiMe, row_source);
	goto l_99;
	/* */
l_4700:  
	fprintf (nunit2, "\n\n+++++ ggen  :  t=%12.4f ERROR : source%3d attempts to generate a transporter\n", TiMe, row_source);
	goto l_99;
	/* */
l_4800:  
	fprintf (nunit2, "\n\n+++++ ggen  :  t=%12.4f ERROR : source with index%5d already scheduled\n", TiMe, row_source);
l_99:    
	return(FEHLER);
	/* */
}


